#include <optional>
#include <string>
#include <vector>

namespace SOH {
enum class MigrationAction {
    Rename,
    Remove,
};

struct Migration {
    MigrationAction action;
    std::string from;
    std::optional<std::string> to = std::nullopt;
};

std::vector<Migration> version3Migrations = {
    { MigrationAction::Rename, "gSwitchAge", "gGeneral.SwitchAge" },
    { MigrationAction::Rename, "gFrameAdvance", "gDeveloperTools.FrameAdvanceTick" },
    { MigrationAction::Rename, "gRandoGenerating", "gGeneral.RandoGenerating" },
    { MigrationAction::Rename, "gNewSeedGenerated", "gGeneral.NewSeedGenerated" },
    { MigrationAction::Rename, "gOnFileSelectNameEntry", "gGeneral.OnFileSelectNameEntry" },
    { MigrationAction::Rename, "gBetterDebugWarpScreenMQMode", "gGeneral.BetterDebugWarpScreenMQMode" },
    { MigrationAction::Rename, "gBetterDebugWarpScreenMQModeScene", "gGeneral.BetterDebugWarpScreenMQModeScene" },
    { MigrationAction::Rename, "gCheatEasyPauseBufferLastInputs", "gGeneral.CheatEasyPauseBufferLastInputs" },
    { MigrationAction::Rename, "gCheatEasyPauseBufferTimer", "gGeneral.CheatEasyPauseBufferTimer" },
    { MigrationAction::Rename, "gRandomizerNewFileDropped", "gGeneral.RandomizerNewFileDropped" },
    { MigrationAction::Rename, "gRandomizerDroppedFile", "gGeneral.RandomizerDroppedFile" },
    { MigrationAction::Rename, "gLetItSnow", "gGeneral.LetItSnow" },
    { MigrationAction::Rename, "gPrevTime", "gGeneral.PrevTime" },
    { MigrationAction::Rename, "gSwitchTimeline", "gGeneral.SwitchTimeline" },
    { MigrationAction::Rename, "gSpoilerLog", "gGeneral.SpoilerLog" },
    { MigrationAction::Rename, "gRestoreQPA", "gGeneral.RestoreQPA" },
    { MigrationAction::Rename, "gFixIceTrapWithBunnyHood", "gGeneral.FixIceTrapWithBunnyHood" },
    { MigrationAction::Rename, "gBetterDebugWarpScreenCurrentScene", "gGeneral.BetterDebugWarpScreenCurrentScene" },
    { MigrationAction::Rename, "gBetterDebugWarpScreenTopDisplayedScene",
      "gGeneral.BetterDebugWarpScreenTopDisplayedScene" },
    { MigrationAction::Rename, "gBetterDebugWarpScreenPageDownIndex", "gGeneral.BetterDebugWarpScreenPageDownIndex" },
    { MigrationAction::Rename, "gActorViewerEnabled", "gOpenWindows.ActorViewer" },
    { MigrationAction::Rename, "gAdvancedResolutionEditorEnabled", "gOpenWindows.AdvancedResolutionEditor" },
    { MigrationAction::Rename, "gAudioEditor.WindowOpen", "gOpenWindows.AudioEditor" },
    { MigrationAction::Rename, "gCheckTrackerEnabled", "gOpenWindows.CheckTracker" },
    { MigrationAction::Rename, "gCheckTrackerSettingsEnabled", "gOpenWindows.CheckTrackerSettings" },
    { MigrationAction::Rename, "gCollisionViewerEnabled", "gOpenWindows.CollisionViewer" },
    { MigrationAction::Rename, "gCosmeticsEditorEnabled", "gOpenWindows.CosmeticsEditor" },
    { MigrationAction::Rename, "gDLViewerEnabled", "gOpenWindows.DisplayListViewer" },
    { MigrationAction::Rename, "gEntranceTrackerEnabled", "gOpenWindows.EntranceTracker" },
    { MigrationAction::Rename, "gGameplayStatsEnabled", "gOpenWindows.GameplayStats" },
    { MigrationAction::Rename, "gItemTrackerEnabled", "gOpenWindows.ItemTracker" },
    { MigrationAction::Rename, "gItemTrackerSettingsEnabled", "gOpenWindows.ItemTrackerSettings" },
    { MigrationAction::Rename, "gMessageViewerEnabled", "gOpenWindows.MessageViewer" },
    { MigrationAction::Rename, "gRandomizerSettingsEnabled", "gOpenWindows.RandomizerSettings" },
    { MigrationAction::Rename, "gSaveEditorEnabled", "gOpenWindows.SaveEditor" },
    { MigrationAction::Rename, "gValueViewer.WindowOpen", "gOpenWindows.ValueViewer" },
    { MigrationAction::Rename, "gConsoleEnabled", "gOpenWindows.Console" },
    { MigrationAction::Rename, "gControllerConfigurationEnabled", "gOpenWindows.ControllerConfiguration" },
    { MigrationAction::Rename, "gControllerDisconnectedWindowEnabled", "gOpenWindows.ControllerDisconnectedWindow" },
    { MigrationAction::Rename, "gControllerReorderingWindowEnabled", "gOpenWindows.ControllerReorderingWindow" },
    { MigrationAction::Rename, "gGfxDebuggerEnabled", "gOpenWindows.GfxDebugger" },
    { MigrationAction::Rename, "gStatsEnabled", "gOpenWindows.Stats" },
    { MigrationAction::Rename, "gDisableChangingSettings", "gSettings.DisableChanges" },
    { MigrationAction::Rename, "gExtraLatencyThreshold", "gSettings.ExtraLatencyThreshold" },
    { MigrationAction::Rename, "gImGuiScale", "gSettings.ImGuiScale" },
    { MigrationAction::Rename, "gTitleScreenTranslation", "gSettings.TitleScreenTranslation" },
    { MigrationAction::Rename, "gA11yTTS", "gSettings.A11yTTS" },
    { MigrationAction::Rename, "gLanguages", "gSettings.Languages" },
    { MigrationAction::Rename, "gMatchRefreshRate", "gSettings.MatchRefreshRate" },
    { MigrationAction::Rename, "gSimulatedInputLag", "gSettings.SimulatedInputLag" },
    { MigrationAction::Rename, "gInterpolationFPS", "gSettings.InterpolationFPS" },
    { MigrationAction::Rename, "gLowResMode", "gSettings.LowResMode" },
    { MigrationAction::Rename, "gZFightingMode", "gSettings.ZFightingMode" },
    { MigrationAction::Rename, "gDisableChangingSettings", "gSettings.DisableChanges" },
    { MigrationAction::Rename, "gExtraLatencyThreshold", "gSettings.ExtraLatencyThreshold" },
    { MigrationAction::Rename, "gImGuiScale", "gSettings.ImGuiScale" },
    { MigrationAction::Rename, "gTitleScreenTranslation", "gSettings.TitleScreenTranslation" },
    { MigrationAction::Rename, "gA11yTTS", "gSettings.A11yTTS" },
    { MigrationAction::Rename, "gLanguages", "gSettings.Languages" },
    { MigrationAction::Rename, "gMatchRefreshRate", "gSettings.MatchRefreshRate" },
    { MigrationAction::Rename, "gInterpolationFPS", "gSettings.InterpolationFPS" },
    { MigrationAction::Rename, "gLedColorSource", "gSettings.LEDColorSource" },
    { MigrationAction::Rename, "gLedPort1Color", "gSettings.LEDPort1Color" },
    { MigrationAction::Rename, "gLedBrightness", "gSettings.LEDBrightness" },
    { MigrationAction::Rename, "gLedCriticalOverride", "gSettings.LEDCriticalOverride" },
    { MigrationAction::Rename, "gOcarinaD5BtnMap", "gSettings.CustomOcarina.D5Button" },
    { MigrationAction::Rename, "gOcarinaB4BtnMap", "gSettings.CustomOcarina.B4Button" },
    { MigrationAction::Rename, "gOcarinaA4BtnMap", "gSettings.CustomOcarina.A4Button" },
    { MigrationAction::Rename, "gOcarinaF4BtnMap", "gSettings.CustomOcarina.F4Button" },
    { MigrationAction::Rename, "gOcarinaD4BtnMap", "gSettings.CustomOcarina.D4Button" },
    { MigrationAction::Rename, "gOcarinaDisableBtnMap", "gSettings.CustomOcarina.DisableButton" },
    { MigrationAction::Rename, "gOcarinaSharpBtnMap", "gSettings.CustomOcarina.SharpButton" },
    { MigrationAction::Rename, "gOcarinaFlatBtnMap", "gSettings.CustomOcarina.FlatButton" },
    { MigrationAction::Rename, "gCustomOcarinaControls", "gSettings.CustomOcarina.Enabled" },
    { MigrationAction::Rename, "gDpadOcarina", "gSettings.OcarinaControl.Dpad" },
    { MigrationAction::Rename, "gRStickOcarina", "gSettings.OcarinaControl.Rstick" },
    { MigrationAction::Rename, "gRightStickAiming", "gSettings.Controls.RightStickAim" },
    { MigrationAction::Rename, "gInvertAimingXAxis", "gSettings.Controls.InvertAimingXAxis" },
    { MigrationAction::Rename, "gInvertAimingYAxis", "gSettings.Controls.InvertAimingYAxis" },
    { MigrationAction::Rename, "gInvertShieldAimingYAxis", "gSettings.Controls.InvertShieldAimingYAxis" },
    { MigrationAction::Rename, "gInvertShieldAimingXAxis", "gSettings.Controls.InvertShieldAimingYAxis" },
    { MigrationAction::Rename, "gInvertZAimingYAxis", "gSettings.Controls.InvertZAimingYAxis" },
    { MigrationAction::Rename, "gDisableAutoCenterViewFirstPerson", "gSettings.DisableFirstPersonAutoCenterView" },
    { MigrationAction::Rename, "gEnableFirstPersonSensitivity", "gSettings.FirstPersonCameraSensitivity.Enabled" },
    { MigrationAction::Rename, "gFirstPersonCameraSensitivityX", "gSettings.FirstPersonCameraSensitivity.X" },
    { MigrationAction::Rename, "gFirstPersonCameraSensitivityY", "gSettings.FirstPersonCameraSensitivity.Y" },
    { MigrationAction::Rename, "gFreeCamera", "gSettings.FreeLook.Enabled" },
    { MigrationAction::Rename, "gInvertXAxis", "gSettings.FreeLook.InvertXAxis" },
    { MigrationAction::Rename, "gInvertYAxis", "gSettings.FreeLook.InvertYAxis" },
    { MigrationAction::Rename, "gThirdPersonCameraSensitivityX", "gSettings.FreeLook.CameraSensitivity.X" },
    { MigrationAction::Rename, "gThirdPersonCameraSensitivityY", "gSettings.FreeLook.CameraSensitivity.Y" },
    { MigrationAction::Rename, "gFreeCameraDistMax", "gSettings.FreeLook.MaxCameraDistance" },
    { MigrationAction::Rename, "gFreeCameraTransitionSpeed", "gSettings.FreeLook.TransitionSpeed" },
    { MigrationAction::Rename, "gDpadPause", "gSettings.DpadOnPause" },
    { MigrationAction::Rename, "gDpadText", "gSettings.DpadInText" },
    { MigrationAction::Rename, "gPauseAnyCursor", "gEnhancements.PauseAnyCursor" },
    { MigrationAction::Rename, "gEnableWalkModify", "gSettings.WalkModifier.Enabled" },
    { MigrationAction::Rename, "gWalkSpeedToggle", "gSettings.WalkModifier.SpeedToggle" },
    { MigrationAction::Rename, "gWalkModifierDoesntChangeJump", "gSettings.WalkModifier.DoesntChangeJump" },
    { MigrationAction::Rename, "gWalkModifierOne", "gSettings.WalkModifier.Mapping1" },
    { MigrationAction::Rename, "gWalkModifierTwo", "gSettings.WalkModifier.Mapping2" },
    { MigrationAction::Rename, "gSwimModifierOne", "gSettings.WalkModifier.SwimMapping1" },
    { MigrationAction::Rename, "gSwimModifierTwo", "gSettings.WalkModifier.SwimMapping2" },
    { MigrationAction::Rename, "gNaviOnL", "gEnhancements.NaviOnL" },
    { MigrationAction::Rename, "gDpadEquips", "gEnhancements.DpadEquips" },
    { MigrationAction::Rename, "gMoveWhileFirstPerson", "gSettings.MoveInFirstPerson" },
    { MigrationAction::Rename, "gA11yDisableIdleCam", "gSettings.A11yDisableIdleCam" },
    { MigrationAction::Rename, "gDpadHoldChange", "gSettings.DpadHoldChange" },
    { MigrationAction::Rename, "gAddTraps.Ammo", "gEnhancements.ExtraTraps.Ammo" },
    { MigrationAction::Rename, "gAddTraps.Bomb", "gEnhancements.ExtraTraps.Bomb" },
    { MigrationAction::Rename, "gAddTraps.Burn", "gEnhancements.ExtraTraps.Burn" },
    { MigrationAction::Rename, "gAddTraps.Ice", "gEnhancements.ExtraTraps.Ice" },
    { MigrationAction::Rename, "gAddTraps.Kill", "gEnhancements.ExtraTraps.Kill" },
    { MigrationAction::Rename, "gAddTraps.Knock", "gEnhancements.ExtraTraps.Knockback" },
    { MigrationAction::Rename, "gAddTraps.Shock", "gEnhancements.ExtraTraps.Shock" },
    { MigrationAction::Rename, "gAddTraps.Speed", "gEnhancements.ExtraTraps.Speed" },
    { MigrationAction::Rename, "gAddTraps.Tele", "gEnhancements.ExtraTraps.Teleport" },
    { MigrationAction::Rename, "gAddTraps.Void", "gEnhancements.ExtraTraps.Void" },
    { MigrationAction::Rename, "gAddTraps.enabled", "gEnhancements.ExtraTraps.Enabled" },
    { MigrationAction::Rename, "gAdultBunnyHood", "gEnhancements.AdultBunnyHood" },
    { MigrationAction::Rename, "gAdultMinimumWeightFish", "gEnhancements.MinimumFishWeightAdult" },
    { MigrationAction::Rename, "gAdultShootingGalleryAmmunition", "gEnhancements.ShootingGalleryAmmoAdult" },
    { MigrationAction::Rename, "gAllDogsRichard", "gEnhancements.AllDogsRichard" },
    { MigrationAction::Rename, "gAlwaysShowDungeonMinimapIcon", "gEnhancements.AlwaysShowDungeonMinimapIcon" },
    { MigrationAction::Rename, "gAnubisFix", "gEnhancements.AnubisFix" },
    { MigrationAction::Rename, "gAskToEquip", "gEnhancements.AskToEquip" },
    { MigrationAction::Rename, "gAssignableTunicsAndBoots", "gEnhancements.AssignableTunicsAndBoots" },
    { MigrationAction::Rename, "gAuthenticLogo", "gEnhancements.AuthenticLogo" },
    { MigrationAction::Rename, "gBetterFW", "gEnhancements.BetterFarore" },
    { MigrationAction::Rename, "gBetterOwl", "gEnhancements.BetterOwl" },
    { MigrationAction::Rename, "gBlueFireArrows", "gEnhancements.BlueFireArrows" },
    { MigrationAction::Rename, "gBombchuBowlingAmmunition", "gEnhancements.BombchuBowlingAmmo" },
    { MigrationAction::Rename, "gBombchuBowlingNoBigCucco", "gEnhancements.BombchuBowlingNoBigCucco" },
    { MigrationAction::Rename, "gBombchuBowlingNoSmallCucco", "gEnhancements.BombchuBowlingNoSmallCucco" },
    { MigrationAction::Rename, "gBombchuDrops", "gEnhancements.BombchuDrops" },
    { MigrationAction::Rename, "gBombchusOOB", "gEnhancements.BombchusOOB" },
    { MigrationAction::Rename, "gBonkDamageMul", "gEnhancements.BonkDamageMult" },
    { MigrationAction::Rename, "gBowReticle", "gEnhancements.BowReticle" },
    { MigrationAction::Rename, "gBowSlingShotAmmoFix", "gEnhancements.BowSlingshotAmmoFix" },
    { MigrationAction::Rename, "gBushDropFix", "gEnhancements.BushDropFix" },
    { MigrationAction::Rename, "gChestSizeAndTextureMatchesContents",
      "gEnhancements.ChestSizeAndTextureMatchContents" },
    { MigrationAction::Rename, "gChestSizeDependsStoneOfAgony", "gEnhancements.ChestSizeDependsStoneOfAgony" },
    { MigrationAction::Rename, "gChildMinimumWeightFish", "gEnhancements.MinimumFishWeightChild" },
    { MigrationAction::Rename, "gChildShootingGalleryAmmunition", "gEnhancements.ShootingGalleryAmmoChild" },
    { MigrationAction::Rename, "gClimbSpeed", "gEnhancements.ClimbSpeed" },
    { MigrationAction::Rename, "gConstantAdultGallery", "gEnhancements.ConstantAdultGallery" },
    { MigrationAction::Rename, "gCowOfTime", "gEnhancements.CowOfTime" },
    { MigrationAction::Rename, "gCrawlSpeed", "gEnhancements.CrawlSpeed" },
    { MigrationAction::Rename, "gCreditsFix", "gEnhancements.CreditsFix" },
    { MigrationAction::Rename, "gCrouchStabFix", "gEnhancements.CrouchStabFix" },
    { MigrationAction::Rename, "gCrouchStabHammerFix", "gEnhancements.CrouchStabHammerFix" },
    { MigrationAction::Rename, "gCuccoStayDurationMultiplier", "gEnhancements.CuccoStayDurationMult" },
    { MigrationAction::Rename, "gCustomizeBombchuBowling", "gEnhancements.CustomizeBombchuBowling" },
    { MigrationAction::Rename, "gCustomizeFishing", "gEnhancements.CustomizeFishing" },
    { MigrationAction::Rename, "gCustomizeOcarinaGame", "gEnhancements.CustomizeOcarinaGame" },
    { MigrationAction::Rename, "gCustomizeShootingGallery", "gEnhancements.CustomizeShootingGallery" },
    { MigrationAction::Rename, "gDamageMul", "gEnhancements.DamageMult" },
    { MigrationAction::Rename, "gDampeAllNight", "gEnhancements.DampeAllNight" },
    { MigrationAction::Rename, "gDampeWin", "gEnhancements.DampeWin" },
    { MigrationAction::Rename, "gDashInterval", "gEnhancements.RupeeDashInterval" },
    { MigrationAction::Rename, "gDayGravePull", "gEnhancements.DayGravePull" },
    { MigrationAction::Rename, "gDekuNutUpgradeFix", "gEnhancements.DekuNutUpgradeFix" },
    { MigrationAction::Rename, "gDeleteFileOnDeath", "gEnhancements.DeleteFileOnDeath" },
    { MigrationAction::Rename, "gDisableBlackBars", "gEnhancements.DisableBlackBars" },
    { MigrationAction::Rename, "gDisableBombBillboarding", "gEnhancements.DisableBombBillboarding" },
    { MigrationAction::Rename, "gDisableCritWiggle", "gEnhancements.DisableCritWiggle" },
    { MigrationAction::Rename, "gDisableDrawDistance", "gEnhancements.DisableDrawDistance" },
    { MigrationAction::Rename, "gDisableFirstPersonChus", "gEnhancements.DisableFirstPersonChus" },
    { MigrationAction::Rename, "gDisableGrottoRotation", "gEnhancements.DisableGrottoRotation" },
    { MigrationAction::Rename, "gDisableKokiriDrawDistance", "gEnhancements.DisableKokiriDrawDistance" },
    { MigrationAction::Rename, "gDisableLOD", "gEnhancements.DisableLOD" },
    { MigrationAction::Rename, "gDisableNaviCallAudio", "gAudioEditor.DisableNaviCallAudio" },
    { MigrationAction::Rename, "gDisableTunicWarningText", "gEnhancements.DisableTunicWarningText" },
    { MigrationAction::Rename, "gDogFollowsEverywhere", "gEnhancements.DogFollowsEverywhere" },
    { MigrationAction::Rename, "gDpadNoDropOcarinaInput", "gEnhancements.DpadNoDropOcarinaInput" },
    { MigrationAction::Rename, "gDrawLineupTick", "gEnhancements.DrawLineupTick" },
    { MigrationAction::Rename, "gDynamicWalletIcon", "gEnhancements.DynamicWalletIcon" },
    { MigrationAction::Rename, "gEnemyHealthBar", "gEnhancements.EnemyHealthBar" },
    { MigrationAction::Rename, "gEnemySizeScalesHealth", "gEnhancements.EnemySizeScalesHealth" },
    { MigrationAction::Rename, "gEnemySpawnsOverWaterboxes", "gEnhancements.EnemySpawnsOverWaterboxes" },
    { MigrationAction::Rename, "gEquipmentCanBeRemoved", "gEnhancements.EquipmentCanBeRemoved" },
    { MigrationAction::Rename, "gFPSGauntlets", "gEnhancements.FirstPersonGauntlets" },
    { MigrationAction::Rename, "gFallDamageMul", "gEnhancements.FallDamageMult" },
    { MigrationAction::Rename, "gFastBoomerang", "gEnhancements.FastBoomerang" },
    { MigrationAction::Rename, "gFastChests", "gEnhancements.FastChests" },
    { MigrationAction::Rename, "gFastDrops", "gEnhancements.FastDrops" },
    { MigrationAction::Rename, "gFastFarores", "gEnhancements.FastFarores" },
    { MigrationAction::Rename, "gFastOcarinaPlayback", "gEnhancements.FastOcarinaPlayback" },
    { MigrationAction::Rename, "gFasterBlockPush", "gEnhancements.FasterBlockPush" },
    { MigrationAction::Rename, "gFasterHeavyBlockLift", "gEnhancements.FasterHeavyBlockLift" },
    { MigrationAction::Rename, "gFileSelectMoreInfo", "gEnhancements.FileSelectMoreInfo" },
    { MigrationAction::Rename, "gFishNeverEscape", "gEnhancements.FishNeverEscape" },
    { MigrationAction::Rename, "gFixCameraDrift", "gEnhancements.FixCameraDrift" },
    { MigrationAction::Rename, "gFixCameraSwing", "gEnhancements.FixCameraSwing" },
    { MigrationAction::Rename, "gFixDungeonMinimapIcon", "gEnhancements.FixDungeonMinimapIcon" },
    { MigrationAction::Rename, "gFixEyesOpenWhileSleeping", "gEnhancements.FixEyesOpenWhileSleeping" },
    { MigrationAction::Rename, "gFixHangingLedgeSwingRate", "gEnhancements.FixHangingLedgeSwingRate" },
    { MigrationAction::Rename, "gFixSawSoftlock", "gEnhancements.FixSawSoftlock" },
    { MigrationAction::Rename, "gFixTexturesOOB", "gEnhancements.FixTexturesOOB" },
    { MigrationAction::Rename, "gFixVineFall", "gEnhancements.FixVineFall" },
    { MigrationAction::Rename, "gFixZoraHintDialogue", "gEnhancements.FixZoraHintDialogue" },
    { MigrationAction::Rename, "gForgeTime", "gEnhancements.ForgeTime" },
    { MigrationAction::Rename, "gFullHealthSpawn", "gEnhancements.FullHealthSpawn" },
    { MigrationAction::Rename, "gGerudoWarriorClothingFix", "gEnhancements.GerudoWarriorClothingFix" },
    { MigrationAction::Rename, "gGoronPot", "gEnhancements.GoronPot" },
    { MigrationAction::Rename, "gGravediggingTourFix", "gEnhancements.GravediggingTourFix" },
    { MigrationAction::Rename, "gGsCutscene", "gEnhancements.GSCutscene" },
    { MigrationAction::Rename, "gGuaranteeFishingBite", "gEnhancements.GuaranteeFishingBite" },
    { MigrationAction::Rename, "gGuardVision", "gEnhancements.GuardVision" },
    { MigrationAction::Rename, "gHideBunnyHood", "gEnhancements.HideBunnyHood" },
    { MigrationAction::Rename, "gHookshotableReticle", "gEnhancements.HookshotableReticle" },
    { MigrationAction::Rename, "gHoverFishing", "gEnhancements.HoverFishing" },
    { MigrationAction::Rename, "gHurtContainer", "gEnhancements.HurtContainer" },
    { MigrationAction::Rename, "gHyperBosses", "gEnhancements.HyperBosses" },
    { MigrationAction::Rename, "gHyperEnemies", "gEnhancements.HyperEnemies" },
    { MigrationAction::Rename, "gInstantFishing", "gEnhancements.InstantFishing" },
    { MigrationAction::Rename, "gInstantOcarinaGameWin", "gEnhancements.InstantOcarinaGameWin" },
    { MigrationAction::Rename, "gInstantPutaway", "gEnhancements.InstantPutaway" },
    { MigrationAction::Rename, "gInstantShootingGalleryWin", "gEnhancements.InstantShootingGalleryWin" },
    { MigrationAction::Rename, "gIvanCoopModeEnabled", "gEnhancements.IvanCoopModeEnabled" },
    { MigrationAction::Rename, "gLinkDefaultName", "gEnhancements.LinkDefaultName" },
    { MigrationAction::Rename, "gLowHpAlarm", "gAudioEditor.LowHpAlarm" },
    { MigrationAction::Rename, "gMMBunnyHood", "gEnhancements.MMBunnyHood" },
    { MigrationAction::Rename, "gMarketSneak", "gEnhancements.MarketSneak" },
    { MigrationAction::Rename, "gMaskSelect", "gEnhancements.MaskSelect" },
    { MigrationAction::Rename, "gMinFrameCount", "gEnhancements.MinFrameCount" },
    { MigrationAction::Rename, "gMinimalUI", "gEnhancements.MinimalUI" },
    { MigrationAction::Rename, "gMirroredWorldMode", "gEnhancements.MirroredWorldMode" },
    { MigrationAction::Rename, "gMweepSpeed", "gEnhancements.MweepSpeed" },
    { MigrationAction::Rename, "gN64WeirdFrames", "gEnhancements.N64WeirdFrames" },
    { MigrationAction::Rename, "gNGCKaleidoSwitcher", "gEnhancements.NGCKaleidoSwitcher" },
    { MigrationAction::Rename, "gNaviTextFix", "gEnhancements.NaviTextFix" },
    { MigrationAction::Rename, "gNewDrops", "gEnhancements.NewDrops" },
    { MigrationAction::Rename, "gNightGSAlwaysSpawn", "gEnhancements.NightGSAlwaysSpawn" },
    { MigrationAction::Rename, "gNoForcedNavi", "gEnhancements.NoForcedNavi" },
    { MigrationAction::Rename, "gNoHUDHeartAnimation", "gEnhancements.NoHUDHeartAnimation" },
    { MigrationAction::Rename, "gNoHeartDrops", "gEnhancements.NoHeartDrops" },
    { MigrationAction::Rename, "gNoInputForCredits", "gEnhancements.NoInputForCredits" },
    { MigrationAction::Rename, "gNoRandomDrops", "gEnhancements.NoRandomDrops" },
    { MigrationAction::Rename, "gNutsExplodeBombs", "gEnhancements.NutsExplodeBombs" },
    { MigrationAction::Rename, "gOcarinaGameNoteSpeed", "gEnhancements.OcarinaGameNoteSpeed" },
    { MigrationAction::Rename, "gOcarinaGameRoundOneNotes", "gEnhancements.OcarinaGameRoundOneNotes" },
    { MigrationAction::Rename, "gOcarinaGameRoundThreeNotes", "gEnhancements.OcarinaGameRoundThreeNotes" },
    { MigrationAction::Rename, "gOcarinaGameRoundTwoNotes", "gEnhancements.OcarinaGameRoundTwoNotes" },
    { MigrationAction::Rename, "gOcarinaGameStartingNotes", "gEnhancements.OcarinaGameStartingNotes" },
    { MigrationAction::Rename, "gOcarinaUnlimitedFailTime", "gEnhancements.OcarinaUnlimitedFailTime" },
    { MigrationAction::Rename, "gPauseWarp", "gEnhancements.PauseWarp" },
    { MigrationAction::Rename, "gPermanentHeartLoss", "gEnhancements.PermanentHeartLoss" },
    { MigrationAction::Rename, "gQuickBongoKill", "gEnhancements.QuickBongoKill" },
    { MigrationAction::Rename, "gQuickPutaway", "gEnhancements.QuickPutaway" },
    { MigrationAction::Rename, "gQuitFishingAtDoor", "gEnhancements.QuitFishingAtDoor" },
    { MigrationAction::Rename, "gRandomizedEnemies", "gEnhancements.RandomizedEnemies" },
    { MigrationAction::Rename, "gRandomizedEnemySizes", "gEnhancements.RandomizedEnemySizes" },
    { MigrationAction::Rename, "gRedGanonBlood", "gEnhancements.RedGanonBlood" },
    { MigrationAction::Rename, "gRememberSaveLocation", "gEnhancements.RememberSaveLocation" },
    { MigrationAction::Rename, "gRemoveExplosiveLimit", "gEnhancements.RemoveExplosiveLimit" },
    { MigrationAction::Rename, "gRestoreRBAValues", "gEnhancements.RestoreRBAValues" },
    { MigrationAction::Rename, "gRupeeDash", "gEnhancements.RupeeDash" },
    { MigrationAction::Rename, "gSceneSpecificDirtPathFix", "gEnhancements.SceneSpecificDirtPathFix" },
    { MigrationAction::Rename, "gSeparateArrows", "gEnhancements.SeparateArrows" },
    { MigrationAction::Rename, "gShadowTag", "gEnhancements.ShadowTag" },
    { MigrationAction::Rename, "gShowDoorLocksOnBothSides", "gEnhancements.ShowDoorLocksOnBothSides" },
    { MigrationAction::Rename, "gSilverRupeeJingleExtend", "gEnhancements.SilverRupeeJingleExtend" },
    { MigrationAction::Rename, "gSkipArrowAnimation", "gEnhancements.SkipArrowAnimation" },
    { MigrationAction::Rename, "gSkipSaveConfirmation", "gEnhancements.SkipSaveConfirmation" },
    { MigrationAction::Rename, "gSkipScarecrow", "gEnhancements.InstantScarecrow" },
    { MigrationAction::Rename, "gSkipSwimDeepEndAnim", "gEnhancements.SkipSwimDeepEndAnim" },
    { MigrationAction::Rename, "gSkipText", "gEnhancements.SkipText" },
    { MigrationAction::Rename, "gSkulltulaFreeze", "gEnhancements.SkulltulaFreeze" },
    { MigrationAction::Rename, "gStaticExplosionRadius", "gEnhancements.StaticExplosionRadius" },
    { MigrationAction::Rename, "gStrengthDisabled", "gEnhancements.StrengthDisabled" },
    { MigrationAction::Rename, "gSunlightArrows", "gEnhancements.SunlightArrows" },
    { MigrationAction::Rename, "gSwitchPerfMode", "gEnhancements.SwitchPerfMode" },
    { MigrationAction::Rename, "gTextSpacing", "gEnhancements.TextSpacing" },
    { MigrationAction::Rename, "gTextSpeed", "gEnhancements.TextSpeed" },
    { MigrationAction::Rename, "gTimeFlowFileSelect", "gEnhancements.TimeFlowFileSelect" },
    { MigrationAction::Rename, "gTimeTravel", "gEnhancements.TimeTravel" },
    { MigrationAction::Rename, "gToTMedallionsColors", "gEnhancements.ToTMedallionsColors" },
    { MigrationAction::Rename, "gToggleStrength", "gEnhancements.ToggleStrength" },
    { MigrationAction::Rename, "gTreeStickDrops", "gEnhancements.TreesDropSticks" },
    { MigrationAction::Rename, "gTwoHandedIdle", "gEnhancements.TwoHandedIdle" },
    { MigrationAction::Rename, "gUniformLR", "gEnhancements.FixMenuLR" },
    { MigrationAction::Rename, "gVisualAgony", "gEnhancements.VisualAgony" },
    { MigrationAction::Rename, "gVoidDamageMul", "gEnhancements.VoidDamageMult" },
    { MigrationAction::Rename, "gGameplayStats.ShowIngameTimer", "gGameplayStats.ShowInGameTimer" },
    { MigrationAction::Rename, "gGameplayStats.TimestampsReverse", "gGameplayStats.ReverseTimestamps" },
    { MigrationAction::Rename, "gMirroredWorld", "gEnhancements.MirroredWorld" },
    { MigrationAction::Rename, "gBetaQuestWorld", "gCheats.BetaQuestWorld" },
    { MigrationAction::Rename, "gBombTimerMultiplier", "gCheats.BombTimerMultiplier" },
    { MigrationAction::Rename, "gCheatEasyInputBufferingEnabled", "gCheats.EasyInputBuffer" },
    { MigrationAction::Rename, "gCheatEasyPauseBufferEnabled", "gCheats.EasyPauseBuffer" },
    { MigrationAction::Rename, "gCheatHookshotReachMultiplier", "gCheats.HookshotReachMultiplier" },
    { MigrationAction::Rename, "gClimbEverything", "gCheats.ClimbEverything" },
    { MigrationAction::Rename, "gDekuStickCheat", "gCheats.DekuStick" },
    { MigrationAction::Rename, "gDropsDontDie", "gCheats.DropsDontDie" },
    { MigrationAction::Rename, "gEnableBetaQuest", "gCheats.EnableBetaQuest" },
    { MigrationAction::Rename, "gEzISG", "gCheats.EasyISG" },
    { MigrationAction::Rename, "gEzQPA", "gCheats.EasyQPA" },
    { MigrationAction::Rename, "gFireproofDekuShield", "gCheats.FireproofDekuShield" },
    { MigrationAction::Rename, "gFreezeTime", "gCheats.FreezeTime" },
    { MigrationAction::Rename, "gHookshotEverything", "gCheats.HookshotEverything" },
    { MigrationAction::Rename, "gInfiniteAmmo", "gCheats.InfiniteAmmo" },
    { MigrationAction::Rename, "gInfiniteEpona", "gCheats.InfiniteEponaBoost" },
    { MigrationAction::Rename, "gInfiniteHealth", "gCheats.InfiniteHealth" },
    { MigrationAction::Rename, "gInfiniteMagic", "gCheats.InfiniteMagic" },
    { MigrationAction::Rename, "gInfiniteMoney", "gCheats.InfiniteMoney" },
    { MigrationAction::Rename, "gInfiniteNayru", "gCheats.InfiniteNayru" },
    { MigrationAction::Rename, "gMoonJumpOnL", "gCheats.MoonJumpOnL" },
    { MigrationAction::Rename, "gNoBugsDespawn", "gCheats.NoBugsDespawn" },
    { MigrationAction::Rename, "gNoClip", "gCheats.NoClip" },
    { MigrationAction::Rename, "gNoFishDespawn", "gCheats.NoFishDespawn" },
    { MigrationAction::Rename, "gNoKeeseGuayTarget", "gCheats.NoKeeseGuayTarget" },
    { MigrationAction::Rename, "gNoRedeadFreeze", "gCheats.NoRedeadFreeze" },
    { MigrationAction::Rename, "gNoRestrictItems", "gCheats.NoRestrictItems" },
    { MigrationAction::Rename, "gSaveStatePromise", "gCheats.SaveStatePromise" },
    { MigrationAction::Rename, "gSaveStatesEnabled", "gCheats.SaveStatesEnabled" },
    { MigrationAction::Rename, "gShieldTwoHanded", "gCheats.ShieldTwoHanded" },
    { MigrationAction::Rename, "gSuperTunic", "gCheats.SuperTunic" },
    { MigrationAction::Rename, "gTimeSync", "gCheats.TimeSync" },
    { MigrationAction::Rename, "gTimelessEquipment", "gCheats.TimelessEquipment" },
    { MigrationAction::Rename, "gBetterDebugWarpScreen", "gDeveloperTools.BetterDebugWarpScreen" },
    { MigrationAction::Rename, "gDebugEnabled", "gDeveloperTools.DebugEnabled" },
    { MigrationAction::Rename, "gDebugSaveFileMode", "gDeveloperTools.DebugSaveFileMode" },
    { MigrationAction::Rename, "gDebugWarpScreenTranslation", "gDeveloperTools.DebugWarpScreenTranslation" },
    { MigrationAction::Rename, "gRegEditEnabled", "gDeveloperTools.RegEditEnabled" },
    { MigrationAction::Rename, "gSaveFileID", "gDeveloperTools.SaveFileID" },
    { MigrationAction::Rename, "gSkipLogoTitle", "gDeveloperTools.SkipLogoTitle" },
    { MigrationAction::Rename, "gSkulltulaDebugEnabled", "gDeveloperTools.SkulltulaDebugEnabled" },
    { MigrationAction::Rename, "gDebugActorViewerNameTags", "gDeveloperTools.ActorViewer.NameTags" },
    { MigrationAction::Rename, "gActorViewerAdvancedParams", "gDeveloperTools.ActorViewer.AdvancedParams" },
    { MigrationAction::Rename, "gColViewerEnabled", "gDeveloperTools.ColViewer.Enabled" },
    { MigrationAction::Rename, "gColViewerScene", "gDeveloperTools.ColViewer.Scene" },
    { MigrationAction::Rename, "gColViewerBgActors", "gDeveloperTools.ColViewer.BGActors" },
    { MigrationAction::Rename, "gColViewerColCheck", "gDeveloperTools.ColViewer.ColCheck" },
    { MigrationAction::Rename, "gColViewerWaterbox", "gDeveloperTools.ColViewer.Waterbox" },
    { MigrationAction::Rename, "gColViewerDecal", "gDeveloperTools.ColViewer.Decal" },
    { MigrationAction::Rename, "gColViewerShaded", "gDeveloperTools.ColViewer.Shaded" },
    { MigrationAction::Rename, "gColViewerColorNormal", "gDeveloperTools.ColViewer.ColorNormal" },
    { MigrationAction::Rename, "gColViewerColorHookshot", "gDeveloperTools.ColViewer.ColorHookshot" },
    { MigrationAction::Rename, "gColViewerColorEntrance", "gDeveloperTools.ColViewer.ColorEntrance" },
    { MigrationAction::Rename, "gColViewerColorSpecialSurface", "gDeveloperTools.ColViewer.ColorSpecialSurface" },
    { MigrationAction::Rename, "gColViewerColorInteractable", "gDeveloperTools.ColViewer.ColorInteractable" },
    { MigrationAction::Rename, "gColViewerColorSlope", "gDeveloperTools.ColViewer.ColorSlope" },
    { MigrationAction::Rename, "gColViewerColorVoid", "gDeveloperTools.ColViewer.ColorVoid" },
    { MigrationAction::Rename, "gColViewerColorOC", "gDeveloperTools.ColViewer.ColorOC" },
    { MigrationAction::Rename, "gColViewerColorAC", "gDeveloperTools.ColViewer.ColorAC" },
    { MigrationAction::Rename, "gColViewerColorAT", "gDeveloperTools.ColViewer.ColorAT" },
    { MigrationAction::Rename, "gColViewerColorWaterbox", "gDeveloperTools.ColViewer.ColorWaterbox" },
    { MigrationAction::Rename, "gValueViewer.EnablePrinting", "gDeveloperTools.ValueViewerEnablePrinting" },
    { MigrationAction::Rename, "gEnableMapToggle", "gDeveloperTools.EnableMapToggle" },
    { MigrationAction::Rename, "gGfxPrintCharStartOffset", "gDeveloperTools.GfxPrintChar.StartOffset" },
    { MigrationAction::Rename, "gGfxPrintCharSpacing", "gDeveloperTools.GfxPrintChar.Spacing" },
    { MigrationAction::Rename, "gEnemyBGMDisable", "gAudioEditor.EnemyBGMDisable" },
    { MigrationAction::Rename, "gLostWoodsConsistentVolume", "gAudioEditor.LostWoodsConsistentVolume" },
    { MigrationAction::Rename, "gSeqNameOverlay", "gAudioEditor.SeqNameOverlay" },
    { MigrationAction::Rename, "gSeqNameOverlayDuration", "gAudioEditor.SeqNameOverlayDuration" },
    { MigrationAction::Rename, "gLinkVoiceFreqMultiplier", "gAudioEditor.LinkVoiceFreqMultiplier" },
    { MigrationAction::Rename, "gExperimentalOctaveDrop", "gAudioEditor.ExperimentalOctaveDrop" },
    { MigrationAction::Rename, "gCosmetics.Hud_AButton", "gCosmetics.HUD.AButton" },
    { MigrationAction::Rename, "gCosmetics.Hud_BButton", "gCosmetics.HUD.BButton" },
    { MigrationAction::Rename, "gCosmetics.Hud_CButtons", "gCosmetics.HUD.CButtons" },
    { MigrationAction::Rename, "gCosmetics.Hud_CUpButton", "gCosmetics.HUD.CUpButton" },
    { MigrationAction::Rename, "gCosmetics.Hud_CDownButton", "gCosmetics.HUD.CDownButton" },
    { MigrationAction::Rename, "gCosmetics.Hud_CLeftButton", "gCosmetics.HUD.CLeftButton" },
    { MigrationAction::Rename, "gCosmetics.Hud_CRightButton", "gCosmetics.HUD.CRightButton" },
    { MigrationAction::Rename, "gCosmetics.Hud_StartButton", "gCosmetics.HUD.StartButton" },
    { MigrationAction::Rename, "gCosmetics.Hud_Dpad", "gCosmetics.HUD.Dpad" },
    { MigrationAction::Rename, "gCosmetics.Hud_KeyCount", "gCosmetics.HUD.KeyCount" },
    { MigrationAction::Rename, "gCosmetics.Hud_StoneOfAgony", "gCosmetics.HUD.StoneOfAgony" },
    { MigrationAction::Rename, "gCosmetics.Hud_Minimap", "gCosmetics.HUD.Minimap" },
    { MigrationAction::Rename, "gCosmetics.Hud_MinimapPosition", "gCosmetics.HUD.MinimapPosition" },
    { MigrationAction::Rename, "gCosmetics.Hud_MinimapEntrance", "gCosmetics.HUD.MinimapEntrance" },
    { MigrationAction::Rename, "gCosmetics.Hud_EnemyHealthBar", "gCosmetics.HUD.EnemyHealthBar" },
    { MigrationAction::Rename, "gCosmetics.Hud_EnemyHealthBorder", "gCosmetics.HUD.EnemyHealthBorder" },
    { MigrationAction::Rename, "gCosmetics.Hud_NameTagActorText", "gCosmetics.HUD.NameTagActorText" },
    { MigrationAction::Rename, "gCosmetics.Hud_NameTagActorBackground", "gCosmetics.HUD.NameTagActorBackground" },
    { MigrationAction::Rename, "gCosmetics.Kal_ItemSelA", "gCosmetics.Kaleido.ItemSelA" },
    { MigrationAction::Rename, "gCosmetics.Kal_ItemSelB", "gCosmetics.Kaleido.ItemSelB" },
    { MigrationAction::Rename, "gCosmetics.Kal_ItemSelC", "gCosmetics.Kaleido.ItemSelC" },
    { MigrationAction::Rename, "gCosmetics.Kal_ItemSelD", "gCosmetics.Kaleido.ItemSelD" },
    { MigrationAction::Rename, "gCosmetics.Kal_EquipSelA", "gCosmetics.Kaleido.EquipSelA" },
    { MigrationAction::Rename, "gCosmetics.Kal_EquipSelB", "gCosmetics.Kaleido.EquipSelB" },
    { MigrationAction::Rename, "gCosmetics.Kal_EquipSelC", "gCosmetics.Kaleido.EquipSelC" },
    { MigrationAction::Rename, "gCosmetics.Kal_EquipSelD", "gCosmetics.Kaleido.EquipSelD" },
    { MigrationAction::Rename, "gCosmetics.Kal_MapSelDunA", "gCosmetics.Kaleido.MapSelDunA" },
    { MigrationAction::Rename, "gCosmetics.Kal_MapSelDunB", "gCosmetics.Kaleido.MapSelDunB" },
    { MigrationAction::Rename, "gCosmetics.Kal_MapSelDunC", "gCosmetics.Kaleido.MapSelDunC" },
    { MigrationAction::Rename, "gCosmetics.Kal_MapSelDunD", "gCosmetics.Kaleido.MapSelDunD" },
    { MigrationAction::Rename, "gCosmetics.Kal_QuestStatusA", "gCosmetics.Kaleido.QuestStatusA" },
    { MigrationAction::Rename, "gCosmetics.Kal_QuestStatusB", "gCosmetics.Kaleido.QuestStatusB" },
    { MigrationAction::Rename, "gCosmetics.Kal_QuestStatusC", "gCosmetics.Kaleido.QuestStatusC" },
    { MigrationAction::Rename, "gCosmetics.Kal_QuestStatusD", "gCosmetics.Kaleido.QuestStatusD" },
    { MigrationAction::Rename, "gCosmetics.Kal_MapSelectA", "gCosmetics.Kaleido.MapSelectA" },
    { MigrationAction::Rename, "gCosmetics.Kal_MapSelectB", "gCosmetics.Kaleido.MapSelectB" },
    { MigrationAction::Rename, "gCosmetics.Kal_MapSelectC", "gCosmetics.Kaleido.MapSelectC" },
    { MigrationAction::Rename, "gCosmetics.Kal_MapSelectD", "gCosmetics.Kaleido.MapSelectD" },
    { MigrationAction::Rename, "gCosmetics.Kal_SaveA", "gCosmetics.Kaleido.SaveA" },
    { MigrationAction::Rename, "gCosmetics.Kal_SaveB", "gCosmetics.Kaleido.SaveB" },
    { MigrationAction::Rename, "gCosmetics.Kal_SaveC", "gCosmetics.Kaleido.SaveC" },
    { MigrationAction::Rename, "gCosmetics.Kal_SaveD", "gCosmetics.Kaleido.SaveD" },
    { MigrationAction::Rename, "gCosmetics.Kal_NamePanel", "gCosmetics.Kaleido.NamePanel" },
    { MigrationAction::Rename, "gCosmetics.Title_FileChoose", "gCosmetics.Title.FileChoose" },
    { MigrationAction::Rename, "gCosmetics.Title_NintendoLogo", "gCosmetics.Title.NintendoLogo" },
    { MigrationAction::Rename, "gCosmetics.Title_N64LogoRed", "gCosmetics.Title.N64LogoRed" },
    { MigrationAction::Rename, "gCosmetics.Title_N64LogoBlue", "gCosmetics.Title.N64LogoBlue" },
    { MigrationAction::Rename, "gCosmetics.Title_N64LogoGreen", "gCosmetics.Title.N64LogoGreen" },
    { MigrationAction::Rename, "gCosmetics.Title_N64LogoYellow", "gCosmetics.Title.N64LogoYellow" },
    { MigrationAction::Rename, "gCosmetics.Title_FirePrimary", "gCosmetics.Title.FirePrimary" },
    { MigrationAction::Rename, "gCosmetics.Title_FireSecondary", "gCosmetics.Title.FireSecondary" },
    { MigrationAction::Rename, "gCosmetics.Arrows_NormalPrimary", "gCosmetics.Arrows.NormalPrimary" },
    { MigrationAction::Rename, "gCosmetics.Arrows_NormalSecondary", "gCosmetics.Arrows.NormalSecondary" },
    { MigrationAction::Rename, "gCosmetics.Arrows_FirePrimary", "gCosmetics.Arrows.FirePrimary" },
    { MigrationAction::Rename, "gCosmetics.Arrows_FireSecondary", "gCosmetics.Arrows.FireSecondary" },
    { MigrationAction::Rename, "gCosmetics.Arrows_IcePrimary", "gCosmetics.Arrows.IcePrimary" },
    { MigrationAction::Rename, "gCosmetics.Arrows_IceSecondary", "gCosmetics.Arrows.IceSecondary" },
    { MigrationAction::Rename, "gCosmetics.Arrows_LightPrimary", "gCosmetics.Arrows.LightPrimary" },
    { MigrationAction::Rename, "gCosmetics.Arrows_LightSecondary", "gCosmetics.Arrows.LightSecondary" },
    { MigrationAction::Rename, "gCosmetics.Magic_DinsPrimary", "gCosmetics.Magic.DinsPrimary" },
    { MigrationAction::Rename, "gCosmetics.Magic_DinsSecondary", "gCosmetics.Magic.DinsSecondary" },
    { MigrationAction::Rename, "gCosmetics.Magic_FaroresPrimary", "gCosmetics.Magic.FaroresPrimary" },
    { MigrationAction::Rename, "gCosmetics.Magic_FaroresSecondary", "gCosmetics.Magic.FaroresSecondary" },
    { MigrationAction::Rename, "gCosmetics.Magic_NayrusPrimary", "gCosmetics.Magic.NayrusPrimary" },
    { MigrationAction::Rename, "gCosmetics.Magic_NayrusSecondary", "gCosmetics.Magic.NayrusSecondary" },
    { MigrationAction::Rename, "gCosmetics.SpinAttack_Level1Primary", "gCosmetics.SpinAttack.Level1Primary" },
    { MigrationAction::Rename, "gCosmetics.SpinAttack_Level1Secondary", "gCosmetics.SpinAttack.Level1Secondary" },
    { MigrationAction::Rename, "gCosmetics.SpinAttack_Level2Primary", "gCosmetics.SpinAttack.Level2Primary" },
    { MigrationAction::Rename, "gCosmetics.SpinAttack_Level2Secondary", "gCosmetics.SpinAttack.Level2Secondary" },
    { MigrationAction::Rename, "gCosmetics.Trails_Bombchu", "gCosmetics.Trails.Bombchu" },
    { MigrationAction::Rename, "gCosmetics.Trails_Boomerang", "gCosmetics.Trails.Boomerang" },
    { MigrationAction::Rename, "gCosmetics.Trails_KokiriSword", "gCosmetics.Trails.KokiriSword" },
    { MigrationAction::Rename, "gCosmetics.Trails_MasterSword", "gCosmetics.Trails.MasterSword" },
    { MigrationAction::Rename, "gCosmetics.Trails_BiggoronSword", "gCosmetics.Trails.BiggoronSword" },
    { MigrationAction::Rename, "gCosmetics.Trails_Stick", "gCosmetics.Trails.Stick" },
    { MigrationAction::Rename, "gCosmetics.Trails_Hammer", "gCosmetics.Trails.Hammer" },
    { MigrationAction::Rename, "gCosmetics.World_BlockOfTime", "gCosmetics.World.BlockOfTime" },
    { MigrationAction::Rename, "gCosmetics.World_Moon", "gCosmetics.World.Moon" },
    { MigrationAction::Rename, "gCosmetics.World_GossipStone", "gCosmetics.World.GossipStone" },
    { MigrationAction::Rename, "gCosmetics.World_RedIce", "gCosmetics.World.RedIce" },
    { MigrationAction::Rename, "gCosmetics.Navi_IdlePrimary", "gCosmetics.Navi.IdlePrimary" },
    { MigrationAction::Rename, "gCosmetics.Navi_IdleSecondary", "gCosmetics.Navi.IdleSecondary" },
    { MigrationAction::Rename, "gCosmetics.Navi_NPCPrimary", "gCosmetics.Navi.NPCPrimary" },
    { MigrationAction::Rename, "gCosmetics.Navi_NPCSecondary", "gCosmetics.Navi.NPCSecondary" },
    { MigrationAction::Rename, "gCosmetics.Navi_EnemyPrimary", "gCosmetics.Navi.EnemyPrimary" },
    { MigrationAction::Rename, "gCosmetics.Navi_EnemySecondary", "gCosmetics.Navi.EnemySecondary" },
    { MigrationAction::Rename, "gCosmetics.Navi_PropsPrimary", "gCosmetics.Navi.PropsPrimary" },
    { MigrationAction::Rename, "gCosmetics.Navi_PropsSecondary", "gCosmetics.Navi.PropsSecondary" },
    { MigrationAction::Rename, "gCosmetics.Ivan_IdlePrimary", "gCosmetics.Ivan.IdlePrimary" },
    { MigrationAction::Rename, "gCosmetics.Ivan_IdleSecondary", "gCosmetics.Ivan.IdleSecondary" },
    { MigrationAction::Rename, "gCosmetics.NPC_FireKeesePrimary", "gCosmetics.NPC.FireKeesePrimary" },
    { MigrationAction::Rename, "gCosmetics.NPC_FireKeeseSecondary", "gCosmetics.NPC.FireKeeseSecondary" },
    { MigrationAction::Rename, "gCosmetics.NPC_IceKeesePrimary", "gCosmetics.NPC.IceKeesePrimary" },
    { MigrationAction::Rename, "gCosmetics.NPC_IceKeeseSecondary", "gCosmetics.NPC.IceKeeseSecondary" },
    { MigrationAction::Rename, "gCosmetics.NPC_Dog1", "gCosmetics.NPC.Dog1" },
    { MigrationAction::Rename, "gCosmetics.NPC_Dog2", "gCosmetics.NPC.Dog2" },
    { MigrationAction::Rename, "gCosmetics.NPC_GoldenSkulltula", "gCosmetics.NPC.GoldenSkulltula" },
    { MigrationAction::Rename, "gCosmetics.NPC_Kokiri", "gCosmetics.NPC.Kokiri" },
    { MigrationAction::Rename, "gCosmetics.NPC_Gerudo", "gCosmetics.NPC.Gerudo" },
    { MigrationAction::Rename, "gCosmetics.NPC_MetalTrap", "gCosmetics.NPC.MetalTrap" },
    { MigrationAction::Rename, "gCosmetics.NPC_IronKnuckles", "gCosmetics.NPC.IronKnuckles" },
    { MigrationAction::Rename, "gCosmetics.Link_KokiriTunic.Value", "gCosmetics.Link.KokiriTunic.Value" },
    { MigrationAction::Rename, "gCosmetics.Link_GoronTunic.Value", "gCosmetics.Link.GoronTunic.Value" },
    { MigrationAction::Rename, "gCosmetics.Link_ZoraTunic.Value", "gCosmetics.Link.ZoraTunic.Value" },
    { MigrationAction::Rename, "gCosmetics.Link_Hair.Value", "gCosmetics.Link.Hair.Value" },
    { MigrationAction::Rename, "gCosmetics.Link_Linen.Value", "gCosmetics.Link.Linen.Value" },
    { MigrationAction::Rename, "gCosmetics.Link_Boots.Value", "gCosmetics.Link.Boots.Value" },
    { MigrationAction::Rename, "gCosmetics.MirrorShield_Body.Value", "gCosmetics.MirrorShield.Body.Value" },
    { MigrationAction::Rename, "gCosmetics.MirrorShield_Mirror.Value", "gCosmetics.MirrorShield.Mirror.Value" },
    { MigrationAction::Rename, "gCosmetics.MirrorShield_Emblem.Value", "gCosmetics.MirrorShield.Emblem.Value" },
    { MigrationAction::Rename, "gCosmetics.Swords_KokiriBlade.Value", "gCosmetics.Swords.KokiriBlade.Value" },
    { MigrationAction::Rename, "gCosmetics.Swords_KokiriHilt.Value", "gCosmetics.Swords.KokiriHilt.Value" },
    { MigrationAction::Rename, "gCosmetics.Swords_MasterBlade.Value", "gCosmetics.Swords.MasterBlade.Value" },
    { MigrationAction::Rename, "gCosmetics.Swords_MasterHilt.Value", "gCosmetics.Swords.MasterHilt.Value" },
    { MigrationAction::Rename, "gCosmetics.Swords_BiggoronBlade.Value", "gCosmetics.Swords.BiggoronBlade.Value" },
    { MigrationAction::Rename, "gCosmetics.Swords_BiggoronHilt.Value", "gCosmetics.Swords.BiggoronHilt.Value" },
    { MigrationAction::Rename, "gCosmetics.Gloves_GoronBracelet.Value", "gCosmetics.Gloves.GoronBracelet.Value" },
    { MigrationAction::Rename, "gCosmetics.Gloves_SilverGauntlets.Value", "gCosmetics.Gloves.SilverGauntlets.Value" },
    { MigrationAction::Rename, "gCosmetics.Gloves_GoldenGauntlets.Value", "gCosmetics.Gloves.GoldenGauntlets.Value" },
    { MigrationAction::Rename, "gCosmetics.Gloves_GauntletsGem.Value", "gCosmetics.Gloves.GauntletsGem.Value" },
    { MigrationAction::Rename, "gCosmetics.Equipment_BoomerangBody.Value", "gCosmetics.Equipment.BoomerangBody.Value" },
    { MigrationAction::Rename, "gCosmetics.Equipment_BoomerangGem.Value", "gCosmetics.Equipment.BoomerangGem.Value" },
    { MigrationAction::Rename, "gCosmetics.Equipment_SlingshotBody.Value", "gCosmetics.Equipment.SlingshotBody.Value" },
    { MigrationAction::Rename, "gCosmetics.Equipment_SlingshotString.Value",
      "gCosmetics.Equipment.SlingshotString.Value" },
    { MigrationAction::Rename, "gCosmetics.Equipment_HammerHead.Value", "gCosmetics.Equipment.HammerHead.Value" },
    { MigrationAction::Rename, "gCosmetics.Equipment_HammerHandle.Value", "gCosmetics.Equipment.HammerHandle.Value" },
    { MigrationAction::Rename, "gCosmetics.Equipment_HookshotChain.Value", "gCosmetics.Equipment.HookshotChain.Value" },
    { MigrationAction::Rename, "gCosmetics.Equipment_HookshotTip.Value", "gCosmetics.Equipment.HookshotTip.Value" },
    { MigrationAction::Rename, "gCosmetics.HookshotReticle_Target.Value", "gCosmetics.HookshotReticle.Target.Value" },
    { MigrationAction::Rename, "gCosmetics.HookshotReticle_NonTarget.Value",
      "gCosmetics.HookshotReticle.NonTarget.Value" },
    { MigrationAction::Rename, "gCosmetics.Equipment_BowTips.Value", "gCosmetics.Equipment.BowTips.Value" },
    { MigrationAction::Rename, "gCosmetics.Equipment_BowString.Value", "gCosmetics.Equipment.BowString.Value" },
    { MigrationAction::Rename, "gCosmetics.Equipment_BowBody.Value", "gCosmetics.Equipment.BowBody.Value" },
    { MigrationAction::Rename, "gCosmetics.Equipment_BowHandle.Value", "gCosmetics.Equipment.BowHandle.Value" },
    { MigrationAction::Rename, "gCosmetics.Equipment_ChuFace.Value", "gCosmetics.Equipment.ChuFace.Value" },
    { MigrationAction::Rename, "gCosmetics.Equipment_ChuBody.Value", "gCosmetics.Equipment.ChuBody.Value" },
    { MigrationAction::Rename, "gCosmetics.Equipment_BunnyHood.Value", "gCosmetics.Equipment.BunnyHood.Value" },
    { MigrationAction::Rename, "gCosmetics.Consumable_Hearts.Value", "gCosmetics.Consumable.Hearts.Value" },
    { MigrationAction::Rename, "gCosmetics.Consumable_HeartBorder.Value", "gCosmetics.Consumable.HeartBorder.Value" },
    { MigrationAction::Rename, "gCosmetics.Consumable_DDHearts.Value", "gCosmetics.Consumable.DDHearts.Value" },
    { MigrationAction::Rename, "gCosmetics.Consumable_DDHeartBorder.Value",
      "gCosmetics.Consumable.DDHeartBorder.Value" },
    { MigrationAction::Rename, "gCosmetics.Consumable_Magic.Value", "gCosmetics.Consumable.Magic.Value" },
    { MigrationAction::Rename, "gCosmetics.Consumable_MagicActive.Value", "gCosmetics.Consumable.MagicActive.Value" },
    { MigrationAction::Rename, "gCosmetics.Consumable_MagicBorder.Value", "gCosmetics.Consumable.MagicBorder.Value" },
    { MigrationAction::Rename, "gCosmetics.Consumable_MagicBorderActive.Value",
      "gCosmetics.Consumable.MagicBorderActive.Value" },
    { MigrationAction::Rename, "gCosmetics.Consumable_GreenRupee.Value", "gCosmetics.Consumable.GreenRupee.Value" },
    { MigrationAction::Rename, "gCosmetics.Consumable_BlueRupee.Value", "gCosmetics.Consumable.BlueRupee.Value" },
    { MigrationAction::Rename, "gCosmetics.Consumable_RedRupee.Value", "gCosmetics.Consumable.RedRupee.Value" },
    { MigrationAction::Rename, "gCosmetics.Consumable_PurpleRupee.Value", "gCosmetics.Consumable.PurpleRupee.Value" },
    { MigrationAction::Rename, "gCosmetics.Consumable_GoldRupee.Value", "gCosmetics.Consumable.GoldRupee.Value" },
    { MigrationAction::Rename, "gCosmetics.Consumable_SilverRupee.Value", "gCosmetics.Consumable.SilverRupee.Value" },
    { MigrationAction::Rename, "gCosmetics.Hud_AButton.Value", "gCosmetics.HUD.AButton.Value" },
    { MigrationAction::Rename, "gCosmetics.Hud_BButton.Value", "gCosmetics.HUD.BButton.Value" },
    { MigrationAction::Rename, "gCosmetics.Hud_CButtons.Value", "gCosmetics.HUD.CButtons.Value" },
    { MigrationAction::Rename, "gCosmetics.Hud_CUpButton.Value", "gCosmetics.HUD.CUpButton.Value" },
    { MigrationAction::Rename, "gCosmetics.Hud_CDownButton.Value", "gCosmetics.HUD.CDownButton.Value" },
    { MigrationAction::Rename, "gCosmetics.Hud_CLeftButton.Value", "gCosmetics.HUD.CLeftButton.Value" },
    { MigrationAction::Rename, "gCosmetics.Hud_CRightButton.Value", "gCosmetics.HUD.CRightButton.Value" },
    { MigrationAction::Rename, "gCosmetics.Hud_StartButton.Value", "gCosmetics.HUD.StartButton.Value" },
    { MigrationAction::Rename, "gCosmetics.Hud_Dpad.Value", "gCosmetics.HUD.Dpad.Value" },
    { MigrationAction::Rename, "gCosmetics.Hud_KeyCount.Value", "gCosmetics.HUD.KeyCount.Value" },
    { MigrationAction::Rename, "gCosmetics.Hud_StoneOfAgony.Value", "gCosmetics.HUD.StoneOfAgony.Value" },
    { MigrationAction::Rename, "gCosmetics.Hud_Minimap.Value", "gCosmetics.HUD.Minimap.Value" },
    { MigrationAction::Rename, "gCosmetics.Hud_MinimapPosition.Value", "gCosmetics.HUD.MinimapPosition.Value" },
    { MigrationAction::Rename, "gCosmetics.Hud_MinimapEntrance.Value", "gCosmetics.HUD.MinimapEntrance.Value" },
    { MigrationAction::Rename, "gCosmetics.Hud_EnemyHealthBar.Value", "gCosmetics.HUD.EnemyHealthBar.Value" },
    { MigrationAction::Rename, "gCosmetics.Hud_EnemyHealthBorder.Value", "gCosmetics.HUD.EnemyHealthBorder.Value" },
    { MigrationAction::Rename, "gCosmetics.Hud_NameTagActorText.Value", "gCosmetics.HUD.NameTagActorText.Value" },
    { MigrationAction::Rename, "gCosmetics.Hud_NameTagActorBackground.Value",
      "gCosmetics.HUD.NameTagActorBackground.Value" },
    { MigrationAction::Rename, "gCosmetics.Kal_ItemSelA.Value", "gCosmetics.Kaleido.ItemSelA.Value" },
    { MigrationAction::Rename, "gCosmetics.Kal_ItemSelB.Value", "gCosmetics.Kaleido.ItemSelB.Value" },
    { MigrationAction::Rename, "gCosmetics.Kal_ItemSelC.Value", "gCosmetics.Kaleido.ItemSelC.Value" },
    { MigrationAction::Rename, "gCosmetics.Kal_ItemSelD.Value", "gCosmetics.Kaleido.ItemSelD.Value" },
    { MigrationAction::Rename, "gCosmetics.Kal_EquipSelA.Value", "gCosmetics.Kaleido.EquipSelA.Value" },
    { MigrationAction::Rename, "gCosmetics.Kal_EquipSelB.Value", "gCosmetics.Kaleido.EquipSelB.Value" },
    { MigrationAction::Rename, "gCosmetics.Kal_EquipSelC.Value", "gCosmetics.Kaleido.EquipSelC.Value" },
    { MigrationAction::Rename, "gCosmetics.Kal_EquipSelD.Value", "gCosmetics.Kaleido.EquipSelD.Value" },
    { MigrationAction::Rename, "gCosmetics.Kal_MapSelDunA.Value", "gCosmetics.Kaleido.MapSelDunA.Value" },
    { MigrationAction::Rename, "gCosmetics.Kal_MapSelDunB.Value", "gCosmetics.Kaleido.MapSelDunB.Value" },
    { MigrationAction::Rename, "gCosmetics.Kal_MapSelDunC.Value", "gCosmetics.Kaleido.MapSelDunC.Value" },
    { MigrationAction::Rename, "gCosmetics.Kal_MapSelDunD.Value", "gCosmetics.Kaleido.MapSelDunD.Value" },
    { MigrationAction::Rename, "gCosmetics.Kal_QuestStatusA.Value", "gCosmetics.Kaleido.QuestStatusA.Value" },
    { MigrationAction::Rename, "gCosmetics.Kal_QuestStatusB.Value", "gCosmetics.Kaleido.QuestStatusB.Value" },
    { MigrationAction::Rename, "gCosmetics.Kal_QuestStatusC.Value", "gCosmetics.Kaleido.QuestStatusC.Value" },
    { MigrationAction::Rename, "gCosmetics.Kal_QuestStatusD.Value", "gCosmetics.Kaleido.QuestStatusD.Value" },
    { MigrationAction::Rename, "gCosmetics.Kal_MapSelectA.Value", "gCosmetics.Kaleido.MapSelectA.Value" },
    { MigrationAction::Rename, "gCosmetics.Kal_MapSelectB.Value", "gCosmetics.Kaleido.MapSelectB.Value" },
    { MigrationAction::Rename, "gCosmetics.Kal_MapSelectC.Value", "gCosmetics.Kaleido.MapSelectC.Value" },
    { MigrationAction::Rename, "gCosmetics.Kal_MapSelectD.Value", "gCosmetics.Kaleido.MapSelectD.Value" },
    { MigrationAction::Rename, "gCosmetics.Kal_SaveA.Value", "gCosmetics.Kaleido.SaveA.Value" },
    { MigrationAction::Rename, "gCosmetics.Kal_SaveB.Value", "gCosmetics.Kaleido.SaveB.Value" },
    { MigrationAction::Rename, "gCosmetics.Kal_SaveC.Value", "gCosmetics.Kaleido.SaveC.Value" },
    { MigrationAction::Rename, "gCosmetics.Kal_SaveD.Value", "gCosmetics.Kaleido.SaveD.Value" },
    { MigrationAction::Rename, "gCosmetics.Kal_NamePanel.Value", "gCosmetics.Kaleido.NamePanel.Value" },
    { MigrationAction::Rename, "gCosmetics.Title_FileChoose.Value", "gCosmetics.Title.FileChoose.Value" },
    { MigrationAction::Rename, "gCosmetics.Title_NintendoLogo.Value", "gCosmetics.Title.NintendoLogo.Value" },
    { MigrationAction::Rename, "gCosmetics.Title_N64LogoRed.Value", "gCosmetics.Title.N64LogoRed.Value" },
    { MigrationAction::Rename, "gCosmetics.Title_N64LogoBlue.Value", "gCosmetics.Title.N64LogoBlue.Value" },
    { MigrationAction::Rename, "gCosmetics.Title_N64LogoGreen.Value", "gCosmetics.Title.N64LogoGreen.Value" },
    { MigrationAction::Rename, "gCosmetics.Title_N64LogoYellow.Value", "gCosmetics.Title.N64LogoYellow.Value" },
    { MigrationAction::Rename, "gCosmetics.Title_FirePrimary.Value", "gCosmetics.Title.FirePrimary.Value" },
    { MigrationAction::Rename, "gCosmetics.Title_FireSecondary.Value", "gCosmetics.Title.FireSecondary.Value" },
    { MigrationAction::Rename, "gCosmetics.Arrows_NormalPrimary.Value", "gCosmetics.Arrows.NormalPrimary.Value" },
    { MigrationAction::Rename, "gCosmetics.Arrows_NormalSecondary.Value", "gCosmetics.Arrows.NormalSecondary.Value" },
    { MigrationAction::Rename, "gCosmetics.Arrows_FirePrimary.Value", "gCosmetics.Arrows.FirePrimary.Value" },
    { MigrationAction::Rename, "gCosmetics.Arrows_FireSecondary.Value", "gCosmetics.Arrows.FireSecondary.Value" },
    { MigrationAction::Rename, "gCosmetics.Arrows_IcePrimary.Value", "gCosmetics.Arrows.IcePrimary.Value" },
    { MigrationAction::Rename, "gCosmetics.Arrows_IceSecondary.Value", "gCosmetics.Arrows.IceSecondary.Value" },
    { MigrationAction::Rename, "gCosmetics.Arrows_LightPrimary.Value", "gCosmetics.Arrows.LightPrimary.Value" },
    { MigrationAction::Rename, "gCosmetics.Arrows_LightSecondary.Value", "gCosmetics.Arrows.LightSecondary.Value" },
    { MigrationAction::Rename, "gCosmetics.Magic_DinsPrimary.Value", "gCosmetics.Magic.DinsPrimary.Value" },
    { MigrationAction::Rename, "gCosmetics.Magic_DinsSecondary.Value", "gCosmetics.Magic.DinsSecondary.Value" },
    { MigrationAction::Rename, "gCosmetics.Magic_FaroresPrimary.Value", "gCosmetics.Magic.FaroresPrimary.Value" },
    { MigrationAction::Rename, "gCosmetics.Magic_FaroresSecondary.Value", "gCosmetics.Magic.FaroresSecondary.Value" },
    { MigrationAction::Rename, "gCosmetics.Magic_NayrusPrimary.Value", "gCosmetics.Magic.NayrusPrimary.Value" },
    { MigrationAction::Rename, "gCosmetics.Magic_NayrusSecondary.Value", "gCosmetics.Magic.NayrusSecondary.Value" },
    { MigrationAction::Rename, "gCosmetics.SpinAttack_Level1Primary.Value",
      "gCosmetics.SpinAttack.Level1Primary.Value" },
    { MigrationAction::Rename, "gCosmetics.SpinAttack_Level1Secondary.Value",
      "gCosmetics.SpinAttack.Level1Secondary.Value" },
    { MigrationAction::Rename, "gCosmetics.SpinAttack_Level2Primary.Value",
      "gCosmetics.SpinAttack.Level2Primary.Value" },
    { MigrationAction::Rename, "gCosmetics.SpinAttack_Level2Secondary.Value",
      "gCosmetics.SpinAttack.Level2Secondary.Value" },
    { MigrationAction::Rename, "gCosmetics.Trails_Bombchu.Value", "gCosmetics.Trails.Bombchu.Value" },
    { MigrationAction::Rename, "gCosmetics.Trails_Boomerang.Value", "gCosmetics.Trails.Boomerang.Value" },
    { MigrationAction::Rename, "gCosmetics.Trails_KokiriSword.Value", "gCosmetics.Trails.KokiriSword.Value" },
    { MigrationAction::Rename, "gCosmetics.Trails_MasterSword.Value", "gCosmetics.Trails.MasterSword.Value" },
    { MigrationAction::Rename, "gCosmetics.Trails_BiggoronSword.Value", "gCosmetics.Trails.BiggoronSword.Value" },
    { MigrationAction::Rename, "gCosmetics.Trails_Stick.Value", "gCosmetics.Trails.Stick.Value" },
    { MigrationAction::Rename, "gCosmetics.Trails_Hammer.Value", "gCosmetics.Trails.Hammer.Value" },
    { MigrationAction::Rename, "gCosmetics.World_BlockOfTime.Value", "gCosmetics.World.BlockOfTime.Value" },
    { MigrationAction::Rename, "gCosmetics.World_Moon.Value", "gCosmetics.World.Moon.Value" },
    { MigrationAction::Rename, "gCosmetics.World_GossipStone.Value", "gCosmetics.World.GossipStone.Value" },
    { MigrationAction::Rename, "gCosmetics.World_RedIce.Value", "gCosmetics.World.RedIce.Value" },
    { MigrationAction::Rename, "gCosmetics.Navi_IdlePrimary.Value", "gCosmetics.Navi.IdlePrimary.Value" },
    { MigrationAction::Rename, "gCosmetics.Navi_IdleSecondary.Value", "gCosmetics.Navi.IdleSecondary.Value" },
    { MigrationAction::Rename, "gCosmetics.Navi_NPCPrimary.Value", "gCosmetics.Navi.NPCPrimary.Value" },
    { MigrationAction::Rename, "gCosmetics.Navi_NPCSecondary.Value", "gCosmetics.Navi.NPCSecondary.Value" },
    { MigrationAction::Rename, "gCosmetics.Navi_EnemyPrimary.Value", "gCosmetics.Navi.EnemyPrimary.Value" },
    { MigrationAction::Rename, "gCosmetics.Navi_EnemySecondary.Value", "gCosmetics.Navi.EnemySecondary.Value" },
    { MigrationAction::Rename, "gCosmetics.Navi_PropsPrimary.Value", "gCosmetics.Navi.PropsPrimary.Value" },
    { MigrationAction::Rename, "gCosmetics.Navi_PropsSecondary.Value", "gCosmetics.Navi.PropsSecondary.Value" },
    { MigrationAction::Rename, "gCosmetics.Ivan_IdlePrimary.Value", "gCosmetics.Ivan.IdlePrimary.Value" },
    { MigrationAction::Rename, "gCosmetics.Ivan_IdleSecondary.Value", "gCosmetics.Ivan.IdleSecondary.Value" },
    { MigrationAction::Rename, "gCosmetics.NPC_FireKeesePrimary.Value", "gCosmetics.NPC.FireKeesePrimary.Value" },
    { MigrationAction::Rename, "gCosmetics.NPC_FireKeeseSecondary.Value", "gCosmetics.NPC.FireKeeseSecondary.Value" },
    { MigrationAction::Rename, "gCosmetics.NPC_IceKeesePrimary.Value", "gCosmetics.NPC.IceKeesePrimary.Value" },
    { MigrationAction::Rename, "gCosmetics.NPC_IceKeeseSecondary.Value", "gCosmetics.NPC.IceKeeseSecondary.Value" },
    { MigrationAction::Rename, "gCosmetics.NPC_Dog1.Value", "gCosmetics.NPC.Dog1.Value" },
    { MigrationAction::Rename, "gCosmetics.NPC_Dog2.Value", "gCosmetics.NPC.Dog2.Value" },
    { MigrationAction::Rename, "gCosmetics.NPC_GoldenSkulltula.Value", "gCosmetics.NPC.GoldenSkulltula.Value" },
    { MigrationAction::Rename, "gCosmetics.NPC_Kokiri.Value", "gCosmetics.NPC.Kokiri.Value" },
    { MigrationAction::Rename, "gCosmetics.NPC_Gerudo.Value", "gCosmetics.NPC.Gerudo.Value" },
    { MigrationAction::Rename, "gCosmetics.NPC_MetalTrap.Value", "gCosmetics.NPC.MetalTrap.Value" },
    { MigrationAction::Rename, "gCosmetics.NPC_IronKnuckles.Value", "gCosmetics.NPC.IronKnuckles.Value" },
    { MigrationAction::Rename, "gCosmetics.Link_KokiriTunic.Rainbow", "gCosmetics.Link.KokiriTunic.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Link_GoronTunic.Rainbow", "gCosmetics.Link.GoronTunic.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Link_ZoraTunic.Rainbow", "gCosmetics.Link.ZoraTunic.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Link_Hair.Rainbow", "gCosmetics.Link.Hair.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Link_Linen.Rainbow", "gCosmetics.Link.Linen.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Link_Boots.Rainbow", "gCosmetics.Link.Boots.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.MirrorShield_Body.Rainbow", "gCosmetics.MirrorShield.Body.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.MirrorShield_Mirror.Rainbow", "gCosmetics.MirrorShield.Mirror.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.MirrorShield_Emblem.Rainbow", "gCosmetics.MirrorShield.Emblem.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Swords_KokiriBlade.Rainbow", "gCosmetics.Swords.KokiriBlade.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Swords_KokiriHilt.Rainbow", "gCosmetics.Swords.KokiriHilt.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Swords_MasterBlade.Rainbow", "gCosmetics.Swords.MasterBlade.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Swords_MasterHilt.Rainbow", "gCosmetics.Swords.MasterHilt.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Swords_BiggoronBlade.Rainbow", "gCosmetics.Swords.BiggoronBlade.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Swords_BiggoronHilt.Rainbow", "gCosmetics.Swords.BiggoronHilt.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Gloves_GoronBracelet.Rainbow", "gCosmetics.Gloves.GoronBracelet.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Gloves_SilverGauntlets.Rainbow",
      "gCosmetics.Gloves.SilverGauntlets.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Gloves_GoldenGauntlets.Rainbow",
      "gCosmetics.Gloves.GoldenGauntlets.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Gloves_GauntletsGem.Rainbow", "gCosmetics.Gloves.GauntletsGem.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Equipment_BoomerangBody.Rainbow",
      "gCosmetics.Equipment.BoomerangBody.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Equipment_BoomerangGem.Rainbow",
      "gCosmetics.Equipment.BoomerangGem.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Equipment_SlingshotBody.Rainbow",
      "gCosmetics.Equipment.SlingshotBody.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Equipment_SlingshotString.Rainbow",
      "gCosmetics.Equipment.SlingshotString.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Equipment_HammerHead.Rainbow", "gCosmetics.Equipment.HammerHead.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Equipment_HammerHandle.Rainbow",
      "gCosmetics.Equipment.HammerHandle.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Equipment_HookshotChain.Rainbow",
      "gCosmetics.Equipment.HookshotChain.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Equipment_HookshotTip.Rainbow", "gCosmetics.Equipment.HookshotTip.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.HookshotReticle_Target.Rainbow",
      "gCosmetics.HookshotReticle.Target.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.HookshotReticle_NonTarget.Rainbow",
      "gCosmetics.HookshotReticle.NonTarget.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Equipment_BowTips.Rainbow", "gCosmetics.Equipment.BowTips.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Equipment_BowString.Rainbow", "gCosmetics.Equipment.BowString.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Equipment_BowBody.Rainbow", "gCosmetics.Equipment.BowBody.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Equipment_BowHandle.Rainbow", "gCosmetics.Equipment.BowHandle.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Equipment_ChuFace.Rainbow", "gCosmetics.Equipment.ChuFace.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Equipment_ChuBody.Rainbow", "gCosmetics.Equipment.ChuBody.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Equipment_BunnyHood.Rainbow", "gCosmetics.Equipment.BunnyHood.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Consumable_Hearts.Rainbow", "gCosmetics.Consumable.Hearts.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Consumable_HeartBorder.Rainbow",
      "gCosmetics.Consumable.HeartBorder.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Consumable_DDHearts.Rainbow", "gCosmetics.Consumable.DDHearts.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Consumable_DDHeartBorder.Rainbow",
      "gCosmetics.Consumable.DDHeartBorder.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Consumable_Magic.Rainbow", "gCosmetics.Consumable.Magic.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Consumable_MagicActive.Rainbow",
      "gCosmetics.Consumable.MagicActive.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Consumable_MagicBorder.Rainbow",
      "gCosmetics.Consumable.MagicBorder.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Consumable_MagicBorderActive.Rainbow",
      "gCosmetics.Consumable.MagicBorderActive.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Consumable_GreenRupee.Rainbow", "gCosmetics.Consumable.GreenRupee.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Consumable_BlueRupee.Rainbow", "gCosmetics.Consumable.BlueRupee.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Consumable_RedRupee.Rainbow", "gCosmetics.Consumable.RedRupee.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Consumable_PurpleRupee.Rainbow",
      "gCosmetics.Consumable.PurpleRupee.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Consumable_GoldRupee.Rainbow", "gCosmetics.Consumable.GoldRupee.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Consumable_SilverRupee.Rainbow",
      "gCosmetics.Consumable.SilverRupee.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Hud_AButton.Rainbow", "gCosmetics.HUD.AButton.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Hud_BButton.Rainbow", "gCosmetics.HUD.BButton.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Hud_CButtons.Rainbow", "gCosmetics.HUD.CButtons.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Hud_CUpButton.Rainbow", "gCosmetics.HUD.CUpButton.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Hud_CDownButton.Rainbow", "gCosmetics.HUD.CDownButton.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Hud_CLeftButton.Rainbow", "gCosmetics.HUD.CLeftButton.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Hud_CRightButton.Rainbow", "gCosmetics.HUD.CRightButton.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Hud_StartButton.Rainbow", "gCosmetics.HUD.StartButton.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Hud_Dpad.Rainbow", "gCosmetics.HUD.Dpad.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Hud_KeyCount.Rainbow", "gCosmetics.HUD.KeyCount.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Hud_StoneOfAgony.Rainbow", "gCosmetics.HUD.StoneOfAgony.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Hud_Minimap.Rainbow", "gCosmetics.HUD.Minimap.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Hud_MinimapPosition.Rainbow", "gCosmetics.HUD.MinimapPosition.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Hud_MinimapEntrance.Rainbow", "gCosmetics.HUD.MinimapEntrance.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Hud_EnemyHealthBar.Rainbow", "gCosmetics.HUD.EnemyHealthBar.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Hud_EnemyHealthBorder.Rainbow", "gCosmetics.HUD.EnemyHealthBorder.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Hud_NameTagActorText.Rainbow", "gCosmetics.HUD.NameTagActorText.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Hud_NameTagActorBackground.Rainbow",
      "gCosmetics.HUD.NameTagActorBackground.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Kal_ItemSelA.Rainbow", "gCosmetics.Kaleido.ItemSelA.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Kal_ItemSelB.Rainbow", "gCosmetics.Kaleido.ItemSelB.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Kal_ItemSelC.Rainbow", "gCosmetics.Kaleido.ItemSelC.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Kal_ItemSelD.Rainbow", "gCosmetics.Kaleido.ItemSelD.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Kal_EquipSelA.Rainbow", "gCosmetics.Kaleido.EquipSelA.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Kal_EquipSelB.Rainbow", "gCosmetics.Kaleido.EquipSelB.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Kal_EquipSelC.Rainbow", "gCosmetics.Kaleido.EquipSelC.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Kal_EquipSelD.Rainbow", "gCosmetics.Kaleido.EquipSelD.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Kal_MapSelDunA.Rainbow", "gCosmetics.Kaleido.MapSelDunA.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Kal_MapSelDunB.Rainbow", "gCosmetics.Kaleido.MapSelDunB.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Kal_MapSelDunC.Rainbow", "gCosmetics.Kaleido.MapSelDunC.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Kal_MapSelDunD.Rainbow", "gCosmetics.Kaleido.MapSelDunD.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Kal_QuestStatusA.Rainbow", "gCosmetics.Kaleido.QuestStatusA.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Kal_QuestStatusB.Rainbow", "gCosmetics.Kaleido.QuestStatusB.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Kal_QuestStatusC.Rainbow", "gCosmetics.Kaleido.QuestStatusC.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Kal_QuestStatusD.Rainbow", "gCosmetics.Kaleido.QuestStatusD.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Kal_MapSelectA.Rainbow", "gCosmetics.Kaleido.MapSelectA.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Kal_MapSelectB.Rainbow", "gCosmetics.Kaleido.MapSelectB.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Kal_MapSelectC.Rainbow", "gCosmetics.Kaleido.MapSelectC.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Kal_MapSelectD.Rainbow", "gCosmetics.Kaleido.MapSelectD.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Kal_SaveA.Rainbow", "gCosmetics.Kaleido.SaveA.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Kal_SaveB.Rainbow", "gCosmetics.Kaleido.SaveB.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Kal_SaveC.Rainbow", "gCosmetics.Kaleido.SaveC.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Kal_SaveD.Rainbow", "gCosmetics.Kaleido.SaveD.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Kal_NamePanel.Rainbow", "gCosmetics.Kaleido.NamePanel.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Title_FileChoose.Rainbow", "gCosmetics.Title.FileChoose.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Title_NintendoLogo.Rainbow", "gCosmetics.Title.NintendoLogo.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Title_N64LogoRed.Rainbow", "gCosmetics.Title.N64LogoRed.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Title_N64LogoBlue.Rainbow", "gCosmetics.Title.N64LogoBlue.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Title_N64LogoGreen.Rainbow", "gCosmetics.Title.N64LogoGreen.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Title_N64LogoYellow.Rainbow", "gCosmetics.Title.N64LogoYellow.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Title_FirePrimary.Rainbow", "gCosmetics.Title.FirePrimary.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Title_FireSecondary.Rainbow", "gCosmetics.Title.FireSecondary.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Arrows_NormalPrimary.Rainbow", "gCosmetics.Arrows.NormalPrimary.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Arrows_NormalSecondary.Rainbow",
      "gCosmetics.Arrows.NormalSecondary.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Arrows_FirePrimary.Rainbow", "gCosmetics.Arrows.FirePrimary.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Arrows_FireSecondary.Rainbow", "gCosmetics.Arrows.FireSecondary.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Arrows_IcePrimary.Rainbow", "gCosmetics.Arrows.IcePrimary.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Arrows_IceSecondary.Rainbow", "gCosmetics.Arrows.IceSecondary.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Arrows_LightPrimary.Rainbow", "gCosmetics.Arrows.LightPrimary.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Arrows_LightSecondary.Rainbow", "gCosmetics.Arrows.LightSecondary.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Magic_DinsPrimary.Rainbow", "gCosmetics.Magic.DinsPrimary.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Magic_DinsSecondary.Rainbow", "gCosmetics.Magic.DinsSecondary.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Magic_FaroresPrimary.Rainbow", "gCosmetics.Magic.FaroresPrimary.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Magic_FaroresSecondary.Rainbow",
      "gCosmetics.Magic.FaroresSecondary.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Magic_NayrusPrimary.Rainbow", "gCosmetics.Magic.NayrusPrimary.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Magic_NayrusSecondary.Rainbow", "gCosmetics.Magic.NayrusSecondary.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.SpinAttack_Level1Primary.Rainbow",
      "gCosmetics.SpinAttack.Level1Primary.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.SpinAttack_Level1Secondary.Rainbow",
      "gCosmetics.SpinAttack.Level1Secondary.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.SpinAttack_Level2Primary.Rainbow",
      "gCosmetics.SpinAttack.Level2Primary.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.SpinAttack_Level2Secondary.Rainbow",
      "gCosmetics.SpinAttack.Level2Secondary.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Trails_Bombchu.Rainbow", "gCosmetics.Trails.Bombchu.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Trails_Boomerang.Rainbow", "gCosmetics.Trails.Boomerang.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Trails_KokiriSword.Rainbow", "gCosmetics.Trails.KokiriSword.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Trails_MasterSword.Rainbow", "gCosmetics.Trails.MasterSword.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Trails_BiggoronSword.Rainbow", "gCosmetics.Trails.BiggoronSword.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Trails_Stick.Rainbow", "gCosmetics.Trails.Stick.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Trails_Hammer.Rainbow", "gCosmetics.Trails.Hammer.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.World_BlockOfTime.Rainbow", "gCosmetics.World.BlockOfTime.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.World_Moon.Rainbow", "gCosmetics.World.Moon.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.World_GossipStone.Rainbow", "gCosmetics.World.GossipStone.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.World_RedIce.Rainbow", "gCosmetics.World.RedIce.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Navi_IdlePrimary.Rainbow", "gCosmetics.Navi.IdlePrimary.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Navi_IdleSecondary.Rainbow", "gCosmetics.Navi.IdleSecondary.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Navi_NPCPrimary.Rainbow", "gCosmetics.Navi.NPCPrimary.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Navi_NPCSecondary.Rainbow", "gCosmetics.Navi.NPCSecondary.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Navi_EnemyPrimary.Rainbow", "gCosmetics.Navi.EnemyPrimary.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Navi_EnemySecondary.Rainbow", "gCosmetics.Navi.EnemySecondary.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Navi_PropsPrimary.Rainbow", "gCosmetics.Navi.PropsPrimary.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Navi_PropsSecondary.Rainbow", "gCosmetics.Navi.PropsSecondary.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Ivan_IdlePrimary.Rainbow", "gCosmetics.Ivan.IdlePrimary.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Ivan_IdleSecondary.Rainbow", "gCosmetics.Ivan.IdleSecondary.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.NPC_FireKeesePrimary.Rainbow", "gCosmetics.NPC.FireKeesePrimary.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.NPC_FireKeeseSecondary.Rainbow",
      "gCosmetics.NPC.FireKeeseSecondary.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.NPC_IceKeesePrimary.Rainbow", "gCosmetics.NPC.IceKeesePrimary.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.NPC_IceKeeseSecondary.Rainbow", "gCosmetics.NPC.IceKeeseSecondary.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.NPC_Dog1.Rainbow", "gCosmetics.NPC.Dog1.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.NPC_Dog2.Rainbow", "gCosmetics.NPC.Dog2.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.NPC_GoldenSkulltula.Rainbow", "gCosmetics.NPC.GoldenSkulltula.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.NPC_Kokiri.Rainbow", "gCosmetics.NPC.Kokiri.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.NPC_Gerudo.Rainbow", "gCosmetics.NPC.Gerudo.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.NPC_MetalTrap.Rainbow", "gCosmetics.NPC.MetalTrap.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.NPC_IronKnuckles.Rainbow", "gCosmetics.NPC.IronKnuckles.Rainbow" },
    { MigrationAction::Rename, "gCosmetics.Link_KokiriTunic.Locked", "gCosmetics.Link.KokiriTunic.Locked" },
    { MigrationAction::Rename, "gCosmetics.Link_GoronTunic.Locked", "gCosmetics.Link.GoronTunic.Locked" },
    { MigrationAction::Rename, "gCosmetics.Link_ZoraTunic.Locked", "gCosmetics.Link.ZoraTunic.Locked" },
    { MigrationAction::Rename, "gCosmetics.Link_Hair.Locked", "gCosmetics.Link.Hair.Locked" },
    { MigrationAction::Rename, "gCosmetics.Link_Linen.Locked", "gCosmetics.Link.Linen.Locked" },
    { MigrationAction::Rename, "gCosmetics.Link_Boots.Locked", "gCosmetics.Link.Boots.Locked" },
    { MigrationAction::Rename, "gCosmetics.MirrorShield_Body.Locked", "gCosmetics.MirrorShield.Body.Locked" },
    { MigrationAction::Rename, "gCosmetics.MirrorShield_Mirror.Locked", "gCosmetics.MirrorShield.Mirror.Locked" },
    { MigrationAction::Rename, "gCosmetics.MirrorShield_Emblem.Locked", "gCosmetics.MirrorShield.Emblem.Locked" },
    { MigrationAction::Rename, "gCosmetics.Swords_KokiriBlade.Locked", "gCosmetics.Swords.KokiriBlade.Locked" },
    { MigrationAction::Rename, "gCosmetics.Swords_KokiriHilt.Locked", "gCosmetics.Swords.KokiriHilt.Locked" },
    { MigrationAction::Rename, "gCosmetics.Swords_MasterBlade.Locked", "gCosmetics.Swords.MasterBlade.Locked" },
    { MigrationAction::Rename, "gCosmetics.Swords_MasterHilt.Locked", "gCosmetics.Swords.MasterHilt.Locked" },
    { MigrationAction::Rename, "gCosmetics.Swords_BiggoronBlade.Locked", "gCosmetics.Swords.BiggoronBlade.Locked" },
    { MigrationAction::Rename, "gCosmetics.Swords_BiggoronHilt.Locked", "gCosmetics.Swords.BiggoronHilt.Locked" },
    { MigrationAction::Rename, "gCosmetics.Gloves_GoronBracelet.Locked", "gCosmetics.Gloves.GoronBracelet.Locked" },
    { MigrationAction::Rename, "gCosmetics.Gloves_SilverGauntlets.Locked", "gCosmetics.Gloves.SilverGauntlets.Locked" },
    { MigrationAction::Rename, "gCosmetics.Gloves_GoldenGauntlets.Locked", "gCosmetics.Gloves.GoldenGauntlets.Locked" },
    { MigrationAction::Rename, "gCosmetics.Gloves_GauntletsGem.Locked", "gCosmetics.Gloves.GauntletsGem.Locked" },
    { MigrationAction::Rename, "gCosmetics.Equipment_BoomerangBody.Locked",
      "gCosmetics.Equipment.BoomerangBody.Locked" },
    { MigrationAction::Rename, "gCosmetics.Equipment_BoomerangGem.Locked", "gCosmetics.Equipment.BoomerangGem.Locked" },
    { MigrationAction::Rename, "gCosmetics.Equipment_SlingshotBody.Locked",
      "gCosmetics.Equipment.SlingshotBody.Locked" },
    { MigrationAction::Rename, "gCosmetics.Equipment_SlingshotString.Locked",
      "gCosmetics.Equipment.SlingshotString.Locked" },
    { MigrationAction::Rename, "gCosmetics.Equipment_HammerHead.Locked", "gCosmetics.Equipment.HammerHead.Locked" },
    { MigrationAction::Rename, "gCosmetics.Equipment_HammerHandle.Locked", "gCosmetics.Equipment.HammerHandle.Locked" },
    { MigrationAction::Rename, "gCosmetics.Equipment_HookshotChain.Locked",
      "gCosmetics.Equipment.HookshotChain.Locked" },
    { MigrationAction::Rename, "gCosmetics.Equipment_HookshotTip.Locked", "gCosmetics.Equipment.HookshotTip.Locked" },
    { MigrationAction::Rename, "gCosmetics.HookshotReticle_Target.Locked", "gCosmetics.HookshotReticle.Target.Locked" },
    { MigrationAction::Rename, "gCosmetics.HookshotReticle_NonTarget.Locked",
      "gCosmetics.HookshotReticle.NonTarget.Locked" },
    { MigrationAction::Rename, "gCosmetics.Equipment_BowTips.Locked", "gCosmetics.Equipment.BowTips.Locked" },
    { MigrationAction::Rename, "gCosmetics.Equipment_BowString.Locked", "gCosmetics.Equipment.BowString.Locked" },
    { MigrationAction::Rename, "gCosmetics.Equipment_BowBody.Locked", "gCosmetics.Equipment.BowBody.Locked" },
    { MigrationAction::Rename, "gCosmetics.Equipment_BowHandle.Locked", "gCosmetics.Equipment.BowHandle.Locked" },
    { MigrationAction::Rename, "gCosmetics.Equipment_ChuFace.Locked", "gCosmetics.Equipment.ChuFace.Locked" },
    { MigrationAction::Rename, "gCosmetics.Equipment_ChuBody.Locked", "gCosmetics.Equipment.ChuBody.Locked" },
    { MigrationAction::Rename, "gCosmetics.Equipment_BunnyHood.Locked", "gCosmetics.Equipment.BunnyHood.Locked" },
    { MigrationAction::Rename, "gCosmetics.Consumable_Hearts.Locked", "gCosmetics.Consumable.Hearts.Locked" },
    { MigrationAction::Rename, "gCosmetics.Consumable_HeartBorder.Locked", "gCosmetics.Consumable.HeartBorder.Locked" },
    { MigrationAction::Rename, "gCosmetics.Consumable_DDHearts.Locked", "gCosmetics.Consumable.DDHearts.Locked" },
    { MigrationAction::Rename, "gCosmetics.Consumable_DDHeartBorder.Locked",
      "gCosmetics.Consumable.DDHeartBorder.Locked" },
    { MigrationAction::Rename, "gCosmetics.Consumable_Magic.Locked", "gCosmetics.Consumable.Magic.Locked" },
    { MigrationAction::Rename, "gCosmetics.Consumable_MagicActive.Locked", "gCosmetics.Consumable.MagicActive.Locked" },
    { MigrationAction::Rename, "gCosmetics.Consumable_MagicBorder.Locked", "gCosmetics.Consumable.MagicBorder.Locked" },
    { MigrationAction::Rename, "gCosmetics.Consumable_MagicBorderActive.Locked",
      "gCosmetics.Consumable.MagicBorderActive.Locked" },
    { MigrationAction::Rename, "gCosmetics.Consumable_GreenRupee.Locked", "gCosmetics.Consumable.GreenRupee.Locked" },
    { MigrationAction::Rename, "gCosmetics.Consumable_BlueRupee.Locked", "gCosmetics.Consumable.BlueRupee.Locked" },
    { MigrationAction::Rename, "gCosmetics.Consumable_RedRupee.Locked", "gCosmetics.Consumable.RedRupee.Locked" },
    { MigrationAction::Rename, "gCosmetics.Consumable_PurpleRupee.Locked", "gCosmetics.Consumable.PurpleRupee.Locked" },
    { MigrationAction::Rename, "gCosmetics.Consumable_GoldRupee.Locked", "gCosmetics.Consumable.GoldRupee.Locked" },
    { MigrationAction::Rename, "gCosmetics.Consumable_SilverRupee.Locked", "gCosmetics.Consumable.SilverRupee.Locked" },
    { MigrationAction::Rename, "gCosmetics.Hud_AButton.Locked", "gCosmetics.HUD.AButton.Locked" },
    { MigrationAction::Rename, "gCosmetics.Hud_BButton.Locked", "gCosmetics.HUD.BButton.Locked" },
    { MigrationAction::Rename, "gCosmetics.Hud_CButtons.Locked", "gCosmetics.HUD.CButtons.Locked" },
    { MigrationAction::Rename, "gCosmetics.Hud_CUpButton.Locked", "gCosmetics.HUD.CUpButton.Locked" },
    { MigrationAction::Rename, "gCosmetics.Hud_CDownButton.Locked", "gCosmetics.HUD.CDownButton.Locked" },
    { MigrationAction::Rename, "gCosmetics.Hud_CLeftButton.Locked", "gCosmetics.HUD.CLeftButton.Locked" },
    { MigrationAction::Rename, "gCosmetics.Hud_CRightButton.Locked", "gCosmetics.HUD.CRightButton.Locked" },
    { MigrationAction::Rename, "gCosmetics.Hud_StartButton.Locked", "gCosmetics.HUD.StartButton.Locked" },
    { MigrationAction::Rename, "gCosmetics.Hud_Dpad.Locked", "gCosmetics.HUD.Dpad.Locked" },
    { MigrationAction::Rename, "gCosmetics.Hud_KeyCount.Locked", "gCosmetics.HUD.KeyCount.Locked" },
    { MigrationAction::Rename, "gCosmetics.Hud_StoneOfAgony.Locked", "gCosmetics.HUD.StoneOfAgony.Locked" },
    { MigrationAction::Rename, "gCosmetics.Hud_Minimap.Locked", "gCosmetics.HUD.Minimap.Locked" },
    { MigrationAction::Rename, "gCosmetics.Hud_MinimapPosition.Locked", "gCosmetics.HUD.MinimapPosition.Locked" },
    { MigrationAction::Rename, "gCosmetics.Hud_MinimapEntrance.Locked", "gCosmetics.HUD.MinimapEntrance.Locked" },
    { MigrationAction::Rename, "gCosmetics.Hud_EnemyHealthBar.Locked", "gCosmetics.HUD.EnemyHealthBar.Locked" },
    { MigrationAction::Rename, "gCosmetics.Hud_EnemyHealthBorder.Locked", "gCosmetics.HUD.EnemyHealthBorder.Locked" },
    { MigrationAction::Rename, "gCosmetics.Hud_NameTagActorText.Locked", "gCosmetics.HUD.NameTagActorText.Locked" },
    { MigrationAction::Rename, "gCosmetics.Hud_NameTagActorBackground.Locked",
      "gCosmetics.HUD.NameTagActorBackground.Locked" },
    { MigrationAction::Rename, "gCosmetics.Kal_ItemSelA.Locked", "gCosmetics.Kaleido.ItemSelA.Locked" },
    { MigrationAction::Rename, "gCosmetics.Kal_ItemSelB.Locked", "gCosmetics.Kaleido.ItemSelB.Locked" },
    { MigrationAction::Rename, "gCosmetics.Kal_ItemSelC.Locked", "gCosmetics.Kaleido.ItemSelC.Locked" },
    { MigrationAction::Rename, "gCosmetics.Kal_ItemSelD.Locked", "gCosmetics.Kaleido.ItemSelD.Locked" },
    { MigrationAction::Rename, "gCosmetics.Kal_EquipSelA.Locked", "gCosmetics.Kaleido.EquipSelA.Locked" },
    { MigrationAction::Rename, "gCosmetics.Kal_EquipSelB.Locked", "gCosmetics.Kaleido.EquipSelB.Locked" },
    { MigrationAction::Rename, "gCosmetics.Kal_EquipSelC.Locked", "gCosmetics.Kaleido.EquipSelC.Locked" },
    { MigrationAction::Rename, "gCosmetics.Kal_EquipSelD.Locked", "gCosmetics.Kaleido.EquipSelD.Locked" },
    { MigrationAction::Rename, "gCosmetics.Kal_MapSelDunA.Locked", "gCosmetics.Kaleido.MapSelDunA.Locked" },
    { MigrationAction::Rename, "gCosmetics.Kal_MapSelDunB.Locked", "gCosmetics.Kaleido.MapSelDunB.Locked" },
    { MigrationAction::Rename, "gCosmetics.Kal_MapSelDunC.Locked", "gCosmetics.Kaleido.MapSelDunC.Locked" },
    { MigrationAction::Rename, "gCosmetics.Kal_MapSelDunD.Locked", "gCosmetics.Kaleido.MapSelDunD.Locked" },
    { MigrationAction::Rename, "gCosmetics.Kal_QuestStatusA.Locked", "gCosmetics.Kaleido.QuestStatusA.Locked" },
    { MigrationAction::Rename, "gCosmetics.Kal_QuestStatusB.Locked", "gCosmetics.Kaleido.QuestStatusB.Locked" },
    { MigrationAction::Rename, "gCosmetics.Kal_QuestStatusC.Locked", "gCosmetics.Kaleido.QuestStatusC.Locked" },
    { MigrationAction::Rename, "gCosmetics.Kal_QuestStatusD.Locked", "gCosmetics.Kaleido.QuestStatusD.Locked" },
    { MigrationAction::Rename, "gCosmetics.Kal_MapSelectA.Locked", "gCosmetics.Kaleido.MapSelectA.Locked" },
    { MigrationAction::Rename, "gCosmetics.Kal_MapSelectB.Locked", "gCosmetics.Kaleido.MapSelectB.Locked" },
    { MigrationAction::Rename, "gCosmetics.Kal_MapSelectC.Locked", "gCosmetics.Kaleido.MapSelectC.Locked" },
    { MigrationAction::Rename, "gCosmetics.Kal_MapSelectD.Locked", "gCosmetics.Kaleido.MapSelectD.Locked" },
    { MigrationAction::Rename, "gCosmetics.Kal_SaveA.Locked", "gCosmetics.Kaleido.SaveA.Locked" },
    { MigrationAction::Rename, "gCosmetics.Kal_SaveB.Locked", "gCosmetics.Kaleido.SaveB.Locked" },
    { MigrationAction::Rename, "gCosmetics.Kal_SaveC.Locked", "gCosmetics.Kaleido.SaveC.Locked" },
    { MigrationAction::Rename, "gCosmetics.Kal_SaveD.Locked", "gCosmetics.Kaleido.SaveD.Locked" },
    { MigrationAction::Rename, "gCosmetics.Kal_NamePanel.Locked", "gCosmetics.Kaleido.NamePanel.Locked" },
    { MigrationAction::Rename, "gCosmetics.Title_FileChoose.Locked", "gCosmetics.Title.FileChoose.Locked" },
    { MigrationAction::Rename, "gCosmetics.Title_NintendoLogo.Locked", "gCosmetics.Title.NintendoLogo.Locked" },
    { MigrationAction::Rename, "gCosmetics.Title_N64LogoRed.Locked", "gCosmetics.Title.N64LogoRed.Locked" },
    { MigrationAction::Rename, "gCosmetics.Title_N64LogoBlue.Locked", "gCosmetics.Title.N64LogoBlue.Locked" },
    { MigrationAction::Rename, "gCosmetics.Title_N64LogoGreen.Locked", "gCosmetics.Title.N64LogoGreen.Locked" },
    { MigrationAction::Rename, "gCosmetics.Title_N64LogoYellow.Locked", "gCosmetics.Title.N64LogoYellow.Locked" },
    { MigrationAction::Rename, "gCosmetics.Title_FirePrimary.Locked", "gCosmetics.Title.FirePrimary.Locked" },
    { MigrationAction::Rename, "gCosmetics.Title_FireSecondary.Locked", "gCosmetics.Title.FireSecondary.Locked" },
    { MigrationAction::Rename, "gCosmetics.Arrows_NormalPrimary.Locked", "gCosmetics.Arrows.NormalPrimary.Locked" },
    { MigrationAction::Rename, "gCosmetics.Arrows_NormalSecondary.Locked", "gCosmetics.Arrows.NormalSecondary.Locked" },
    { MigrationAction::Rename, "gCosmetics.Arrows_FirePrimary.Locked", "gCosmetics.Arrows.FirePrimary.Locked" },
    { MigrationAction::Rename, "gCosmetics.Arrows_FireSecondary.Locked", "gCosmetics.Arrows.FireSecondary.Locked" },
    { MigrationAction::Rename, "gCosmetics.Arrows_IcePrimary.Locked", "gCosmetics.Arrows.IcePrimary.Locked" },
    { MigrationAction::Rename, "gCosmetics.Arrows_IceSecondary.Locked", "gCosmetics.Arrows.IceSecondary.Locked" },
    { MigrationAction::Rename, "gCosmetics.Arrows_LightPrimary.Locked", "gCosmetics.Arrows.LightPrimary.Locked" },
    { MigrationAction::Rename, "gCosmetics.Arrows_LightSecondary.Locked", "gCosmetics.Arrows.LightSecondary.Locked" },
    { MigrationAction::Rename, "gCosmetics.Magic_DinsPrimary.Locked", "gCosmetics.Magic.DinsPrimary.Locked" },
    { MigrationAction::Rename, "gCosmetics.Magic_DinsSecondary.Locked", "gCosmetics.Magic.DinsSecondary.Locked" },
    { MigrationAction::Rename, "gCosmetics.Magic_FaroresPrimary.Locked", "gCosmetics.Magic.FaroresPrimary.Locked" },
    { MigrationAction::Rename, "gCosmetics.Magic_FaroresSecondary.Locked", "gCosmetics.Magic.FaroresSecondary.Locked" },
    { MigrationAction::Rename, "gCosmetics.Magic_NayrusPrimary.Locked", "gCosmetics.Magic.NayrusPrimary.Locked" },
    { MigrationAction::Rename, "gCosmetics.Magic_NayrusSecondary.Locked", "gCosmetics.Magic.NayrusSecondary.Locked" },
    { MigrationAction::Rename, "gCosmetics.SpinAttack_Level1Primary.Locked",
      "gCosmetics.SpinAttack.Level1Primary.Locked" },
    { MigrationAction::Rename, "gCosmetics.SpinAttack_Level1Secondary.Locked",
      "gCosmetics.SpinAttack.Level1Secondary.Locked" },
    { MigrationAction::Rename, "gCosmetics.SpinAttack_Level2Primary.Locked",
      "gCosmetics.SpinAttack.Level2Primary.Locked" },
    { MigrationAction::Rename, "gCosmetics.SpinAttack_Level2Secondary.Locked",
      "gCosmetics.SpinAttack.Level2Secondary.Locked" },
    { MigrationAction::Rename, "gCosmetics.Trails_Bombchu.Locked", "gCosmetics.Trails.Bombchu.Locked" },
    { MigrationAction::Rename, "gCosmetics.Trails_Boomerang.Locked", "gCosmetics.Trails.Boomerang.Locked" },
    { MigrationAction::Rename, "gCosmetics.Trails_KokiriSword.Locked", "gCosmetics.Trails.KokiriSword.Locked" },
    { MigrationAction::Rename, "gCosmetics.Trails_MasterSword.Locked", "gCosmetics.Trails.MasterSword.Locked" },
    { MigrationAction::Rename, "gCosmetics.Trails_BiggoronSword.Locked", "gCosmetics.Trails.BiggoronSword.Locked" },
    { MigrationAction::Rename, "gCosmetics.Trails_Stick.Locked", "gCosmetics.Trails.Stick.Locked" },
    { MigrationAction::Rename, "gCosmetics.Trails_Hammer.Locked", "gCosmetics.Trails.Hammer.Locked" },
    { MigrationAction::Rename, "gCosmetics.World_BlockOfTime.Locked", "gCosmetics.World.BlockOfTime.Locked" },
    { MigrationAction::Rename, "gCosmetics.World_Moon.Locked", "gCosmetics.World.Moon.Locked" },
    { MigrationAction::Rename, "gCosmetics.World_GossipStone.Locked", "gCosmetics.World.GossipStone.Locked" },
    { MigrationAction::Rename, "gCosmetics.World_RedIce.Locked", "gCosmetics.World.RedIce.Locked" },
    { MigrationAction::Rename, "gCosmetics.Navi_IdlePrimary.Locked", "gCosmetics.Navi.IdlePrimary.Locked" },
    { MigrationAction::Rename, "gCosmetics.Navi_IdleSecondary.Locked", "gCosmetics.Navi.IdleSecondary.Locked" },
    { MigrationAction::Rename, "gCosmetics.Navi_NPCPrimary.Locked", "gCosmetics.Navi.NPCPrimary.Locked" },
    { MigrationAction::Rename, "gCosmetics.Navi_NPCSecondary.Locked", "gCosmetics.Navi.NPCSecondary.Locked" },
    { MigrationAction::Rename, "gCosmetics.Navi_EnemyPrimary.Locked", "gCosmetics.Navi.EnemyPrimary.Locked" },
    { MigrationAction::Rename, "gCosmetics.Navi_EnemySecondary.Locked", "gCosmetics.Navi.EnemySecondary.Locked" },
    { MigrationAction::Rename, "gCosmetics.Navi_PropsPrimary.Locked", "gCosmetics.Navi.PropsPrimary.Locked" },
    { MigrationAction::Rename, "gCosmetics.Navi_PropsSecondary.Locked", "gCosmetics.Navi.PropsSecondary.Locked" },
    { MigrationAction::Rename, "gCosmetics.Ivan_IdlePrimary.Locked", "gCosmetics.Ivan.IdlePrimary.Locked" },
    { MigrationAction::Rename, "gCosmetics.Ivan_IdleSecondary.Locked", "gCosmetics.Ivan.IdleSecondary.Locked" },
    { MigrationAction::Rename, "gCosmetics.NPC_FireKeesePrimary.Locked", "gCosmetics.NPC.FireKeesePrimary.Locked" },
    { MigrationAction::Rename, "gCosmetics.NPC_FireKeeseSecondary.Locked", "gCosmetics.NPC.FireKeeseSecondary.Locked" },
    { MigrationAction::Rename, "gCosmetics.NPC_IceKeesePrimary.Locked", "gCosmetics.NPC.IceKeesePrimary.Locked" },
    { MigrationAction::Rename, "gCosmetics.NPC_IceKeeseSecondary.Locked", "gCosmetics.NPC.IceKeeseSecondary.Locked" },
    { MigrationAction::Rename, "gCosmetics.NPC_Dog1.Locked", "gCosmetics.NPC.Dog1.Locked" },
    { MigrationAction::Rename, "gCosmetics.NPC_Dog2.Locked", "gCosmetics.NPC.Dog2.Locked" },
    { MigrationAction::Rename, "gCosmetics.NPC_GoldenSkulltula.Locked", "gCosmetics.NPC.GoldenSkulltula.Locked" },
    { MigrationAction::Rename, "gCosmetics.NPC_Kokiri.Locked", "gCosmetics.NPC.Kokiri.Locked" },
    { MigrationAction::Rename, "gCosmetics.NPC_Gerudo.Locked", "gCosmetics.NPC.Gerudo.Locked" },
    { MigrationAction::Rename, "gCosmetics.NPC_MetalTrap.Locked", "gCosmetics.NPC.MetalTrap.Locked" },
    { MigrationAction::Rename, "gCosmetics.NPC_IronKnuckles.Locked", "gCosmetics.NPC.IronKnuckles.Locked" },
    { MigrationAction::Rename, "gCosmetics.Link_KokiriTunic.Changed", "gCosmetics.Link.KokiriTunic.Changed" },
    { MigrationAction::Rename, "gCosmetics.Link_GoronTunic.Changed", "gCosmetics.Link.GoronTunic.Changed" },
    { MigrationAction::Rename, "gCosmetics.Link_ZoraTunic.Changed", "gCosmetics.Link.ZoraTunic.Changed" },
    { MigrationAction::Rename, "gCosmetics.Link_Hair.Changed", "gCosmetics.Link.Hair.Changed" },
    { MigrationAction::Rename, "gCosmetics.Link_Linen.Changed", "gCosmetics.Link.Linen.Changed" },
    { MigrationAction::Rename, "gCosmetics.Link_Boots.Changed", "gCosmetics.Link.Boots.Changed" },
    { MigrationAction::Rename, "gCosmetics.MirrorShield_Body.Changed", "gCosmetics.MirrorShield.Body.Changed" },
    { MigrationAction::Rename, "gCosmetics.MirrorShield_Mirror.Changed", "gCosmetics.MirrorShield.Mirror.Changed" },
    { MigrationAction::Rename, "gCosmetics.MirrorShield_Emblem.Changed", "gCosmetics.MirrorShield.Emblem.Changed" },
    { MigrationAction::Rename, "gCosmetics.Swords_KokiriBlade.Changed", "gCosmetics.Swords.KokiriBlade.Changed" },
    { MigrationAction::Rename, "gCosmetics.Swords_KokiriHilt.Changed", "gCosmetics.Swords.KokiriHilt.Changed" },
    { MigrationAction::Rename, "gCosmetics.Swords_MasterBlade.Changed", "gCosmetics.Swords.MasterBlade.Changed" },
    { MigrationAction::Rename, "gCosmetics.Swords_MasterHilt.Changed", "gCosmetics.Swords.MasterHilt.Changed" },
    { MigrationAction::Rename, "gCosmetics.Swords_BiggoronBlade.Changed", "gCosmetics.Swords.BiggoronBlade.Changed" },
    { MigrationAction::Rename, "gCosmetics.Swords_BiggoronHilt.Changed", "gCosmetics.Swords.BiggoronHilt.Changed" },
    { MigrationAction::Rename, "gCosmetics.Gloves_GoronBracelet.Changed", "gCosmetics.Gloves.GoronBracelet.Changed" },
    { MigrationAction::Rename, "gCosmetics.Gloves_SilverGauntlets.Changed",
      "gCosmetics.Gloves.SilverGauntlets.Changed" },
    { MigrationAction::Rename, "gCosmetics.Gloves_GoldenGauntlets.Changed",
      "gCosmetics.Gloves.GoldenGauntlets.Changed" },
    { MigrationAction::Rename, "gCosmetics.Gloves_GauntletsGem.Changed", "gCosmetics.Gloves.GauntletsGem.Changed" },
    { MigrationAction::Rename, "gCosmetics.Equipment_BoomerangBody.Changed",
      "gCosmetics.Equipment.BoomerangBody.Changed" },
    { MigrationAction::Rename, "gCosmetics.Equipment_BoomerangGem.Changed",
      "gCosmetics.Equipment.BoomerangGem.Changed" },
    { MigrationAction::Rename, "gCosmetics.Equipment_SlingshotBody.Changed",
      "gCosmetics.Equipment.SlingshotBody.Changed" },
    { MigrationAction::Rename, "gCosmetics.Equipment_SlingshotString.Changed",
      "gCosmetics.Equipment.SlingshotString.Changed" },
    { MigrationAction::Rename, "gCosmetics.Equipment_HammerHead.Changed", "gCosmetics.Equipment.HammerHead.Changed" },
    { MigrationAction::Rename, "gCosmetics.Equipment_HammerHandle.Changed",
      "gCosmetics.Equipment.HammerHandle.Changed" },
    { MigrationAction::Rename, "gCosmetics.Equipment_HookshotChain.Changed",
      "gCosmetics.Equipment.HookshotChain.Changed" },
    { MigrationAction::Rename, "gCosmetics.Equipment_HookshotTip.Changed", "gCosmetics.Equipment.HookshotTip.Changed" },
    { MigrationAction::Rename, "gCosmetics.HookshotReticle_Target.Changed",
      "gCosmetics.HookshotReticle.Target.Changed" },
    { MigrationAction::Rename, "gCosmetics.HookshotReticle_NonTarget.Changed",
      "gCosmetics.HookshotReticle.NonTarget.Changed" },
    { MigrationAction::Rename, "gCosmetics.Equipment_BowTips.Changed", "gCosmetics.Equipment.BowTips.Changed" },
    { MigrationAction::Rename, "gCosmetics.Equipment_BowString.Changed", "gCosmetics.Equipment.BowString.Changed" },
    { MigrationAction::Rename, "gCosmetics.Equipment_BowBody.Changed", "gCosmetics.Equipment.BowBody.Changed" },
    { MigrationAction::Rename, "gCosmetics.Equipment_BowHandle.Changed", "gCosmetics.Equipment.BowHandle.Changed" },
    { MigrationAction::Rename, "gCosmetics.Equipment_ChuFace.Changed", "gCosmetics.Equipment.ChuFace.Changed" },
    { MigrationAction::Rename, "gCosmetics.Equipment_ChuBody.Changed", "gCosmetics.Equipment.ChuBody.Changed" },
    { MigrationAction::Rename, "gCosmetics.Equipment_BunnyHood.Changed", "gCosmetics.Equipment.BunnyHood.Changed" },
    { MigrationAction::Rename, "gCosmetics.Consumable_Hearts.Changed", "gCosmetics.Consumable.Hearts.Changed" },
    { MigrationAction::Rename, "gCosmetics.Consumable_HeartBorder.Changed",
      "gCosmetics.Consumable.HeartBorder.Changed" },
    { MigrationAction::Rename, "gCosmetics.Consumable_DDHearts.Changed", "gCosmetics.Consumable.DDHearts.Changed" },
    { MigrationAction::Rename, "gCosmetics.Consumable_DDHeartBorder.Changed",
      "gCosmetics.Consumable.DDHeartBorder.Changed" },
    { MigrationAction::Rename, "gCosmetics.Consumable_Magic.Changed", "gCosmetics.Consumable.Magic.Changed" },
    { MigrationAction::Rename, "gCosmetics.Consumable_MagicActive.Changed",
      "gCosmetics.Consumable.MagicActive.Changed" },
    { MigrationAction::Rename, "gCosmetics.Consumable_MagicBorder.Changed",
      "gCosmetics.Consumable.MagicBorder.Changed" },
    { MigrationAction::Rename, "gCosmetics.Consumable_MagicBorderActive.Changed",
      "gCosmetics.Consumable.MagicBorderActive.Changed" },
    { MigrationAction::Rename, "gCosmetics.Consumable_GreenRupee.Changed", "gCosmetics.Consumable.GreenRupee.Changed" },
    { MigrationAction::Rename, "gCosmetics.Consumable_BlueRupee.Changed", "gCosmetics.Consumable.BlueRupee.Changed" },
    { MigrationAction::Rename, "gCosmetics.Consumable_RedRupee.Changed", "gCosmetics.Consumable.RedRupee.Changed" },
    { MigrationAction::Rename, "gCosmetics.Consumable_PurpleRupee.Changed",
      "gCosmetics.Consumable.PurpleRupee.Changed" },
    { MigrationAction::Rename, "gCosmetics.Consumable_GoldRupee.Changed", "gCosmetics.Consumable.GoldRupee.Changed" },
    { MigrationAction::Rename, "gCosmetics.Consumable_SilverRupee.Changed",
      "gCosmetics.Consumable.SilverRupee.Changed" },
    { MigrationAction::Rename, "gCosmetics.Hud_AButton.Changed", "gCosmetics.HUD.AButton.Changed" },
    { MigrationAction::Rename, "gCosmetics.Hud_BButton.Changed", "gCosmetics.HUD.BButton.Changed" },
    { MigrationAction::Rename, "gCosmetics.Hud_CButtons.Changed", "gCosmetics.HUD.CButtons.Changed" },
    { MigrationAction::Rename, "gCosmetics.Hud_CUpButton.Changed", "gCosmetics.HUD.CUpButton.Changed" },
    { MigrationAction::Rename, "gCosmetics.Hud_CDownButton.Changed", "gCosmetics.HUD.CDownButton.Changed" },
    { MigrationAction::Rename, "gCosmetics.Hud_CLeftButton.Changed", "gCosmetics.HUD.CLeftButton.Changed" },
    { MigrationAction::Rename, "gCosmetics.Hud_CRightButton.Changed", "gCosmetics.HUD.CRightButton.Changed" },
    { MigrationAction::Rename, "gCosmetics.Hud_StartButton.Changed", "gCosmetics.HUD.StartButton.Changed" },
    { MigrationAction::Rename, "gCosmetics.Hud_Dpad.Changed", "gCosmetics.HUD.Dpad.Changed" },
    { MigrationAction::Rename, "gCosmetics.Hud_KeyCount.Changed", "gCosmetics.HUD.KeyCount.Changed" },
    { MigrationAction::Rename, "gCosmetics.Hud_StoneOfAgony.Changed", "gCosmetics.HUD.StoneOfAgony.Changed" },
    { MigrationAction::Rename, "gCosmetics.Hud_Minimap.Changed", "gCosmetics.HUD.Minimap.Changed" },
    { MigrationAction::Rename, "gCosmetics.Hud_MinimapPosition.Changed", "gCosmetics.HUD.MinimapPosition.Changed" },
    { MigrationAction::Rename, "gCosmetics.Hud_MinimapEntrance.Changed", "gCosmetics.HUD.MinimapEntrance.Changed" },
    { MigrationAction::Rename, "gCosmetics.Hud_EnemyHealthBar.Changed", "gCosmetics.HUD.EnemyHealthBar.Changed" },
    { MigrationAction::Rename, "gCosmetics.Hud_EnemyHealthBorder.Changed", "gCosmetics.HUD.EnemyHealthBorder.Changed" },
    { MigrationAction::Rename, "gCosmetics.Hud_NameTagActorText.Changed", "gCosmetics.HUD.NameTagActorText.Changed" },
    { MigrationAction::Rename, "gCosmetics.Hud_NameTagActorBackground.Changed",
      "gCosmetics.HUD.NameTagActorBackground.Changed" },
    { MigrationAction::Rename, "gCosmetics.Kal_ItemSelA.Changed", "gCosmetics.Kaleido.ItemSelA.Changed" },
    { MigrationAction::Rename, "gCosmetics.Kal_ItemSelB.Changed", "gCosmetics.Kaleido.ItemSelB.Changed" },
    { MigrationAction::Rename, "gCosmetics.Kal_ItemSelC.Changed", "gCosmetics.Kaleido.ItemSelC.Changed" },
    { MigrationAction::Rename, "gCosmetics.Kal_ItemSelD.Changed", "gCosmetics.Kaleido.ItemSelD.Changed" },
    { MigrationAction::Rename, "gCosmetics.Kal_EquipSelA.Changed", "gCosmetics.Kaleido.EquipSelA.Changed" },
    { MigrationAction::Rename, "gCosmetics.Kal_EquipSelB.Changed", "gCosmetics.Kaleido.EquipSelB.Changed" },
    { MigrationAction::Rename, "gCosmetics.Kal_EquipSelC.Changed", "gCosmetics.Kaleido.EquipSelC.Changed" },
    { MigrationAction::Rename, "gCosmetics.Kal_EquipSelD.Changed", "gCosmetics.Kaleido.EquipSelD.Changed" },
    { MigrationAction::Rename, "gCosmetics.Kal_MapSelDunA.Changed", "gCosmetics.Kaleido.MapSelDunA.Changed" },
    { MigrationAction::Rename, "gCosmetics.Kal_MapSelDunB.Changed", "gCosmetics.Kaleido.MapSelDunB.Changed" },
    { MigrationAction::Rename, "gCosmetics.Kal_MapSelDunC.Changed", "gCosmetics.Kaleido.MapSelDunC.Changed" },
    { MigrationAction::Rename, "gCosmetics.Kal_MapSelDunD.Changed", "gCosmetics.Kaleido.MapSelDunD.Changed" },
    { MigrationAction::Rename, "gCosmetics.Kal_QuestStatusA.Changed", "gCosmetics.Kaleido.QuestStatusA.Changed" },
    { MigrationAction::Rename, "gCosmetics.Kal_QuestStatusB.Changed", "gCosmetics.Kaleido.QuestStatusB.Changed" },
    { MigrationAction::Rename, "gCosmetics.Kal_QuestStatusC.Changed", "gCosmetics.Kaleido.QuestStatusC.Changed" },
    { MigrationAction::Rename, "gCosmetics.Kal_QuestStatusD.Changed", "gCosmetics.Kaleido.QuestStatusD.Changed" },
    { MigrationAction::Rename, "gCosmetics.Kal_MapSelectA.Changed", "gCosmetics.Kaleido.MapSelectA.Changed" },
    { MigrationAction::Rename, "gCosmetics.Kal_MapSelectB.Changed", "gCosmetics.Kaleido.MapSelectB.Changed" },
    { MigrationAction::Rename, "gCosmetics.Kal_MapSelectC.Changed", "gCosmetics.Kaleido.MapSelectC.Changed" },
    { MigrationAction::Rename, "gCosmetics.Kal_MapSelectD.Changed", "gCosmetics.Kaleido.MapSelectD.Changed" },
    { MigrationAction::Rename, "gCosmetics.Kal_SaveA.Changed", "gCosmetics.Kaleido.SaveA.Changed" },
    { MigrationAction::Rename, "gCosmetics.Kal_SaveB.Changed", "gCosmetics.Kaleido.SaveB.Changed" },
    { MigrationAction::Rename, "gCosmetics.Kal_SaveC.Changed", "gCosmetics.Kaleido.SaveC.Changed" },
    { MigrationAction::Rename, "gCosmetics.Kal_SaveD.Changed", "gCosmetics.Kaleido.SaveD.Changed" },
    { MigrationAction::Rename, "gCosmetics.Kal_NamePanel.Changed", "gCosmetics.Kaleido.NamePanel.Changed" },
    { MigrationAction::Rename, "gCosmetics.Title_FileChoose.Changed", "gCosmetics.Title.FileChoose.Changed" },
    { MigrationAction::Rename, "gCosmetics.Title_NintendoLogo.Changed", "gCosmetics.Title.NintendoLogo.Changed" },
    { MigrationAction::Rename, "gCosmetics.Title_N64LogoRed.Changed", "gCosmetics.Title.N64LogoRed.Changed" },
    { MigrationAction::Rename, "gCosmetics.Title_N64LogoBlue.Changed", "gCosmetics.Title.N64LogoBlue.Changed" },
    { MigrationAction::Rename, "gCosmetics.Title_N64LogoGreen.Changed", "gCosmetics.Title.N64LogoGreen.Changed" },
    { MigrationAction::Rename, "gCosmetics.Title_N64LogoYellow.Changed", "gCosmetics.Title.N64LogoYellow.Changed" },
    { MigrationAction::Rename, "gCosmetics.Title_FirePrimary.Changed", "gCosmetics.Title.FirePrimary.Changed" },
    { MigrationAction::Rename, "gCosmetics.Title_FireSecondary.Changed", "gCosmetics.Title.FireSecondary.Changed" },
    { MigrationAction::Rename, "gCosmetics.Arrows_NormalPrimary.Changed", "gCosmetics.Arrows.NormalPrimary.Changed" },
    { MigrationAction::Rename, "gCosmetics.Arrows_NormalSecondary.Changed",
      "gCosmetics.Arrows.NormalSecondary.Changed" },
    { MigrationAction::Rename, "gCosmetics.Arrows_FirePrimary.Changed", "gCosmetics.Arrows.FirePrimary.Changed" },
    { MigrationAction::Rename, "gCosmetics.Arrows_FireSecondary.Changed", "gCosmetics.Arrows.FireSecondary.Changed" },
    { MigrationAction::Rename, "gCosmetics.Arrows_IcePrimary.Changed", "gCosmetics.Arrows.IcePrimary.Changed" },
    { MigrationAction::Rename, "gCosmetics.Arrows_IceSecondary.Changed", "gCosmetics.Arrows.IceSecondary.Changed" },
    { MigrationAction::Rename, "gCosmetics.Arrows_LightPrimary.Changed", "gCosmetics.Arrows.LightPrimary.Changed" },
    { MigrationAction::Rename, "gCosmetics.Arrows_LightSecondary.Changed", "gCosmetics.Arrows.LightSecondary.Changed" },
    { MigrationAction::Rename, "gCosmetics.Magic_DinsPrimary.Changed", "gCosmetics.Magic.DinsPrimary.Changed" },
    { MigrationAction::Rename, "gCosmetics.Magic_DinsSecondary.Changed", "gCosmetics.Magic.DinsSecondary.Changed" },
    { MigrationAction::Rename, "gCosmetics.Magic_FaroresPrimary.Changed", "gCosmetics.Magic.FaroresPrimary.Changed" },
    { MigrationAction::Rename, "gCosmetics.Magic_FaroresSecondary.Changed",
      "gCosmetics.Magic.FaroresSecondary.Changed" },
    { MigrationAction::Rename, "gCosmetics.Magic_NayrusPrimary.Changed", "gCosmetics.Magic.NayrusPrimary.Changed" },
    { MigrationAction::Rename, "gCosmetics.Magic_NayrusSecondary.Changed", "gCosmetics.Magic.NayrusSecondary.Changed" },
    { MigrationAction::Rename, "gCosmetics.SpinAttack_Level1Primary.Changed",
      "gCosmetics.SpinAttack.Level1Primary.Changed" },
    { MigrationAction::Rename, "gCosmetics.SpinAttack_Level1Secondary.Changed",
      "gCosmetics.SpinAttack.Level1Secondary.Changed" },
    { MigrationAction::Rename, "gCosmetics.SpinAttack_Level2Primary.Changed",
      "gCosmetics.SpinAttack.Level2Primary.Changed" },
    { MigrationAction::Rename, "gCosmetics.SpinAttack_Level2Secondary.Changed",
      "gCosmetics.SpinAttack.Level2Secondary.Changed" },
    { MigrationAction::Rename, "gCosmetics.Trails_Bombchu.Changed", "gCosmetics.Trails.Bombchu.Changed" },
    { MigrationAction::Rename, "gCosmetics.Trails_Boomerang.Changed", "gCosmetics.Trails.Boomerang.Changed" },
    { MigrationAction::Rename, "gCosmetics.Trails_KokiriSword.Changed", "gCosmetics.Trails.KokiriSword.Changed" },
    { MigrationAction::Rename, "gCosmetics.Trails_MasterSword.Changed", "gCosmetics.Trails.MasterSword.Changed" },
    { MigrationAction::Rename, "gCosmetics.Trails_BiggoronSword.Changed", "gCosmetics.Trails.BiggoronSword.Changed" },
    { MigrationAction::Rename, "gCosmetics.Trails_Stick.Changed", "gCosmetics.Trails.Stick.Changed" },
    { MigrationAction::Rename, "gCosmetics.Trails_Hammer.Changed", "gCosmetics.Trails.Hammer.Changed" },
    { MigrationAction::Rename, "gCosmetics.World_BlockOfTime.Changed", "gCosmetics.World.BlockOfTime.Changed" },
    { MigrationAction::Rename, "gCosmetics.World_Moon.Changed", "gCosmetics.World.Moon.Changed" },
    { MigrationAction::Rename, "gCosmetics.World_GossipStone.Changed", "gCosmetics.World.GossipStone.Changed" },
    { MigrationAction::Rename, "gCosmetics.World_RedIce.Changed", "gCosmetics.World.RedIce.Changed" },
    { MigrationAction::Rename, "gCosmetics.Navi_IdlePrimary.Changed", "gCosmetics.Navi.IdlePrimary.Changed" },
    { MigrationAction::Rename, "gCosmetics.Navi_IdleSecondary.Changed", "gCosmetics.Navi.IdleSecondary.Changed" },
    { MigrationAction::Rename, "gCosmetics.Navi_NPCPrimary.Changed", "gCosmetics.Navi.NPCPrimary.Changed" },
    { MigrationAction::Rename, "gCosmetics.Navi_NPCSecondary.Changed", "gCosmetics.Navi.NPCSecondary.Changed" },
    { MigrationAction::Rename, "gCosmetics.Navi_EnemyPrimary.Changed", "gCosmetics.Navi.EnemyPrimary.Changed" },
    { MigrationAction::Rename, "gCosmetics.Navi_EnemySecondary.Changed", "gCosmetics.Navi.EnemySecondary.Changed" },
    { MigrationAction::Rename, "gCosmetics.Navi_PropsPrimary.Changed", "gCosmetics.Navi.PropsPrimary.Changed" },
    { MigrationAction::Rename, "gCosmetics.Navi_PropsSecondary.Changed", "gCosmetics.Navi.PropsSecondary.Changed" },
    { MigrationAction::Rename, "gCosmetics.Ivan_IdlePrimary.Changed", "gCosmetics.Ivan.IdlePrimary.Changed" },
    { MigrationAction::Rename, "gCosmetics.Ivan_IdleSecondary.Changed", "gCosmetics.Ivan.IdleSecondary.Changed" },
    { MigrationAction::Rename, "gCosmetics.NPC_FireKeesePrimary.Changed", "gCosmetics.NPC.FireKeesePrimary.Changed" },
    { MigrationAction::Rename, "gCosmetics.NPC_FireKeeseSecondary.Changed",
      "gCosmetics.NPC.FireKeeseSecondary.Changed" },
    { MigrationAction::Rename, "gCosmetics.NPC_IceKeesePrimary.Changed", "gCosmetics.NPC.IceKeesePrimary.Changed" },
    { MigrationAction::Rename, "gCosmetics.NPC_IceKeeseSecondary.Changed", "gCosmetics.NPC.IceKeeseSecondary.Changed" },
    { MigrationAction::Rename, "gCosmetics.NPC_Dog1.Changed", "gCosmetics.NPC.Dog1.Changed" },
    { MigrationAction::Rename, "gCosmetics.NPC_Dog2.Changed", "gCosmetics.NPC.Dog2.Changed" },
    { MigrationAction::Rename, "gCosmetics.NPC_GoldenSkulltula.Changed", "gCosmetics.NPC.GoldenSkulltula.Changed" },
    { MigrationAction::Rename, "gCosmetics.NPC_Kokiri.Changed", "gCosmetics.NPC.Kokiri.Changed" },
    { MigrationAction::Rename, "gCosmetics.NPC_Gerudo.Changed", "gCosmetics.NPC.Gerudo.Changed" },
    { MigrationAction::Rename, "gCosmetics.NPC_MetalTrap.Changed", "gCosmetics.NPC.MetalTrap.Changed" },
    { MigrationAction::Rename, "gCosmetics.NPC_IronKnuckles.Changed", "gCosmetics.NPC.IronKnuckles.Changed" },
    { MigrationAction::Rename, "gHUDMargin_T", "gCosmetics.HUD.Margin.T" },
    { MigrationAction::Rename, "gHUDMargin_L", "gCosmetics.HUD.Margin.L" },
    { MigrationAction::Rename, "gHUDMargin_R", "gCosmetics.HUD.Margin.R" },
    { MigrationAction::Rename, "gHUDMargin_B", "gCosmetics.HUD.Margin.B" },
    { MigrationAction::Rename, "gHearts", "gCosmetics.HUD.Hearts" },
    { MigrationAction::Rename, "gHeartsCount", "gCosmetics.HUD.HeartsCount" },
    { MigrationAction::Rename, "gMagicBar", "gCosmetics.HUD.MagicBar" },
    { MigrationAction::Rename, "gVSOA", "gCosmetics.HUD.VisualSoA" },
    { MigrationAction::Rename, "gBBtn", "gCosmetics.HUD.BButton" },
    { MigrationAction::Rename, "gABtn", "gCosmetics.HUD.AButton" },
    { MigrationAction::Rename, "gStartBtn", "gCosmetics.HUD.StartButton" },
    { MigrationAction::Rename, "gCBtnU", "gCosmetics.HUD.CUpButton" },
    { MigrationAction::Rename, "gCBtnD", "gCosmetics.HUD.CDownButton" },
    { MigrationAction::Rename, "gCBtnL", "gCosmetics.HUD.CLeftButton" },
    { MigrationAction::Rename, "gCBtnR", "gCosmetics.HUD.CRightButton" },
    { MigrationAction::Rename, "gDPad", "gCosmetics.HUD.Dpad" },
    { MigrationAction::Rename, "gMinimap", "gCosmetics.HUD.Minimap" },
    { MigrationAction::Rename, "gSKC", "gCosmetics.HUD.SmallKey" },
    { MigrationAction::Rename, "gRC", "gCosmetics.HUD.Rupees" },
    { MigrationAction::Rename, "gCarrots", "gCosmetics.HUD.Carrots" },
    { MigrationAction::Rename, "gTimers", "gCosmetics.HUD.Timers" },
    { MigrationAction::Rename, "gAS", "gCosmetics.HUD.ArcheryScore" },
    { MigrationAction::Rename, "gTCM", "gCosmetics.HUD.TitleCard.Map" },
    { MigrationAction::Rename, "gTCB", "gCosmetics.HUD.TitleCard.Boss" },
    { MigrationAction::Rename, "gIGT", "gCosmetics.HUD.IGT" },
    { MigrationAction::Rename, "gHeartsPosType", "gCosmetics.HUD.Hearts.PosType" },
    { MigrationAction::Rename, "gHeartsCountPosType", "gCosmetics.HUD.HeartsCount.PosType" },
    { MigrationAction::Rename, "gMagicBarPosType", "gCosmetics.HUD.MagicBar.PosType" },
    { MigrationAction::Rename, "gVSOAPosType", "gCosmetics.HUD.VisualSoA.PosType" },
    { MigrationAction::Rename, "gBBtnPosType", "gCosmetics.HUD.BButton.PosType" },
    { MigrationAction::Rename, "gABtnPosType", "gCosmetics.HUD.AButton.PosType" },
    { MigrationAction::Rename, "gStartBtnPosType", "gCosmetics.HUD.StartButton.PosType" },
    { MigrationAction::Rename, "gCBtnUPosType", "gCosmetics.HUD.CUpButton.PosType" },
    { MigrationAction::Rename, "gCBtnDPosType", "gCosmetics.HUD.CDownButton.PosType" },
    { MigrationAction::Rename, "gCBtnLPosType", "gCosmetics.HUD.CLeftButton.PosType" },
    { MigrationAction::Rename, "gCBtnRPosType", "gCosmetics.HUD.CRightButton.PosType" },
    { MigrationAction::Rename, "gDPadPosType", "gCosmetics.HUD.Dpad.PosType" },
    { MigrationAction::Rename, "gMinimapPosType", "gCosmetics.HUD.Minimap.PosType" },
    { MigrationAction::Rename, "gSKCPosType", "gCosmetics.HUD.SmallKey.PosType" },
    { MigrationAction::Rename, "gRCPosType", "gCosmetics.HUD.Rupees.PosType" },
    { MigrationAction::Rename, "gCarrotsPosType", "gCosmetics.HUD.Carrots.PosType" },
    { MigrationAction::Rename, "gTimersPosType", "gCosmetics.HUD.Timers.PosType" },
    { MigrationAction::Rename, "gASPosType", "gCosmetics.HUD.ArcheryScore.PosType" },
    { MigrationAction::Rename, "gTCMPosType", "gCosmetics.HUD.TitleCard.Map.PosType" },
    { MigrationAction::Rename, "gTCBPosType", "gCosmetics.HUD.TitleCard.Boss.PosType" },
    { MigrationAction::Rename, "gIGTPosType", "gCosmetics.HUD.IGT.PosType" },
    { MigrationAction::Rename, "gHeartsPosX", "gCosmetics.HUD.Hearts.PosX" },
    { MigrationAction::Rename, "gHeartsCountPosX", "gCosmetics.HUD.HeartsCount.PosX" },
    { MigrationAction::Rename, "gMagicBarPosX", "gCosmetics.HUD.MagicBar.PosX" },
    { MigrationAction::Rename, "gVSOAPosX", "gCosmetics.HUD.VisualSoA.PosX" },
    { MigrationAction::Rename, "gBBtnPosX", "gCosmetics.HUD.BButton.PosX" },
    { MigrationAction::Rename, "gABtnPosX", "gCosmetics.HUD.AButton.PosX" },
    { MigrationAction::Rename, "gStartBtnPosX", "gCosmetics.HUD.StartButton.PosX" },
    { MigrationAction::Rename, "gCBtnUPosX", "gCosmetics.HUD.CUpButton.PosX" },
    { MigrationAction::Rename, "gCBtnDPosX", "gCosmetics.HUD.CDownButton.PosX" },
    { MigrationAction::Rename, "gCBtnLPosX", "gCosmetics.HUD.CLeftButton.PosX" },
    { MigrationAction::Rename, "gCBtnRPosX", "gCosmetics.HUD.CRightButton.PosX" },
    { MigrationAction::Rename, "gDPadPosX", "gCosmetics.HUD.Dpad.PosX" },
    { MigrationAction::Rename, "gMinimapPosX", "gCosmetics.HUD.Minimap.PosX" },
    { MigrationAction::Rename, "gSKCPosX", "gCosmetics.HUD.SmallKey.PosX" },
    { MigrationAction::Rename, "gRCPosX", "gCosmetics.HUD.Rupees.PosX" },
    { MigrationAction::Rename, "gCarrotsPosX", "gCosmetics.HUD.Carrots.PosX" },
    { MigrationAction::Rename, "gTimersPosX", "gCosmetics.HUD.Timers.PosX" },
    { MigrationAction::Rename, "gASPosX", "gCosmetics.HUD.ArcheryScore.PosX" },
    { MigrationAction::Rename, "gTCMPosX", "gCosmetics.HUD.TitleCard.Map.PosX" },
    { MigrationAction::Rename, "gTCBPosX", "gCosmetics.HUD.TitleCard.Boss.PosX" },
    { MigrationAction::Rename, "gIGTPosX", "gCosmetics.HUD.IGT.PosX" },
    { MigrationAction::Rename, "gHeartsPosY", "gCosmetics.HUD.Hearts.PosY" },
    { MigrationAction::Rename, "gHeartsCountPosY", "gCosmetics.HUD.HeartsCount.PosY" },
    { MigrationAction::Rename, "gMagicBarPosY", "gCosmetics.HUD.MagicBar.PosY" },
    { MigrationAction::Rename, "gVSOAPosY", "gCosmetics.HUD.VisualSoA.PosY" },
    { MigrationAction::Rename, "gBBtnPosY", "gCosmetics.HUD.BButton.PosY" },
    { MigrationAction::Rename, "gABtnPosY", "gCosmetics.HUD.AButton.PosY" },
    { MigrationAction::Rename, "gStartBtnPosY", "gCosmetics.HUD.StartButton.PosY" },
    { MigrationAction::Rename, "gCBtnUPosY", "gCosmetics.HUD.CUpButton.PosY" },
    { MigrationAction::Rename, "gCBtnDPosY", "gCosmetics.HUD.CDownButton.PosY" },
    { MigrationAction::Rename, "gCBtnLPosY", "gCosmetics.HUD.CLeftButton.PosY" },
    { MigrationAction::Rename, "gCBtnRPosY", "gCosmetics.HUD.CRightButton.PosY" },
    { MigrationAction::Rename, "gDPadPosY", "gCosmetics.HUD.Dpad.PosY" },
    { MigrationAction::Rename, "gMinimapPosY", "gCosmetics.HUD.Minimap.PosY" },
    { MigrationAction::Rename, "gSKCPosY", "gCosmetics.HUD.SmallKey.PosY" },
    { MigrationAction::Rename, "gRCPosY", "gCosmetics.HUD.Rupees.PosY" },
    { MigrationAction::Rename, "gCarrotsPosY", "gCosmetics.HUD.Carrots.PosY" },
    { MigrationAction::Rename, "gTimersPosY", "gCosmetics.HUD.Timers.PosY" },
    { MigrationAction::Rename, "gASPosY", "gCosmetics.HUD.ArcheryScore.PosY" },
    { MigrationAction::Rename, "gTCMPosY", "gCosmetics.HUD.TitleCard.Map.PosY" },
    { MigrationAction::Rename, "gTCBPosY", "gCosmetics.HUD.TitleCard.Boss.PosY" },
    { MigrationAction::Rename, "gIGTPosY", "gCosmetics.HUD.IGT.PosY" },
    { MigrationAction::Rename, "gHeartsUseMargins", "gCosmetics.HUD.HeartsUseMargins" },
    { MigrationAction::Rename, "gHeartsCountUseMargins", "gCosmetics.HUD.HeartsCountUseMargins" },
    { MigrationAction::Rename, "gMagicBarUseMargins", "gCosmetics.HUD.MagicBarUseMargins" },
    { MigrationAction::Rename, "gVSOAUseMargins", "gCosmetics.HUD.VisualSoAUseMargins" },
    { MigrationAction::Rename, "gBBtnUseMargins", "gCosmetics.HUD.BButtonUseMargins" },
    { MigrationAction::Rename, "gABtnUseMargins", "gCosmetics.HUD.AButtonUseMargins" },
    { MigrationAction::Rename, "gStartBtnUseMargins", "gCosmetics.HUD.StartButtonUseMargins" },
    { MigrationAction::Rename, "gCBtnUUseMargins", "gCosmetics.HUD.CUpButtonUseMargins" },
    { MigrationAction::Rename, "gCBtnDUseMargins", "gCosmetics.HUD.CDownButtonUseMargins" },
    { MigrationAction::Rename, "gCBtnLUseMargins", "gCosmetics.HUD.CLeftButtonUseMargins" },
    { MigrationAction::Rename, "gCBtnRUseMargins", "gCosmetics.HUD.CRightButtonUseMargins" },
    { MigrationAction::Rename, "gDPadUseMargins", "gCosmetics.HUD.DpadUseMargins" },
    { MigrationAction::Rename, "gMinimapUseMargins", "gCosmetics.HUD.MinimapUseMargins" },
    { MigrationAction::Rename, "gSKCUseMargins", "gCosmetics.HUD.AButtonUseMargins" },
    { MigrationAction::Rename, "gRCUseMargins", "gCosmetics.HUD.StartButtonUseMargins" },
    { MigrationAction::Rename, "gCarrotsUseMargins", "gCosmetics.HUD.CUpButtonUseMargins" },
    { MigrationAction::Rename, "gTimersUseMargins", "gCosmetics.HUD.CDownButtonUseMargins" },
    { MigrationAction::Rename, "gASUseMargins", "gCosmetics.HUD.CLeftButtonUseMargins" },
    { MigrationAction::Rename, "gTCMUseMargins", "gCosmetics.HUD.CRightButtonUseMargins" },
    { MigrationAction::Rename, "gTCBUseMargins", "gCosmetics.HUD.DpadUseMargins" },
    { MigrationAction::Rename, "gIGTUseMargins", "gCosmetics.HUD.MinimapUseMargins" },
    { MigrationAction::Rename, "gUnfixGoronSpin", "gCosmetics.UnfixGoronSpin" },
    { MigrationAction::Rename, "gNL_Diamond_Env", "gCosmetics.Magic.NayrusSecondary" },
    { MigrationAction::Rename, "gUseSpellsCol", "gCosmetics.UseSpellsColors" },
    { MigrationAction::Rename, "gItemTrackerBgColorA", "gTrackers.ItemTracker.BgColor.Value.A" },
    { MigrationAction::Rename, "gItemTrackerBgColorB", "gTrackers.ItemTracker.BgColor.Value.B" },
    { MigrationAction::Rename, "gItemTrackerBgColorG", "gTrackers.ItemTracker.BgColor.Value.G" },
    { MigrationAction::Rename, "gItemTrackerBgColorR", "gTrackers.ItemTracker.BgColor.Value.R" },
    { MigrationAction::Rename, "gItemTrackerCapacityTrack", "gTrackers.ItemTracker.ItemCountType" },
    { MigrationAction::Rename, "gItemTrackerComboButton1", "gTrackers.ItemTracker.ComboButton1" },
    { MigrationAction::Rename, "gItemTrackerComboButton2", "gTrackers.ItemTracker.ComboButton2" },
    { MigrationAction::Rename, "gItemTrackerCurrentOnLeft", "gTrackers.ItemTracker.ItemCountAlignLeft" },
    { MigrationAction::Rename, "gItemTrackerDisplayDungeonItemsHorizontal",
      "gTrackers.ItemTracker.DungeonItems.Layout" },
    { MigrationAction::Rename, "gItemTrackerDisplayDungeonItemsMaps",
      "gTrackers.ItemTracker.DungeonItems.DisplayMaps" },
    { MigrationAction::Rename, "gItemTrackerDisplayType", "gTrackers.ItemTracker.DisplayType.Main" },
    { MigrationAction::Rename, "gItemTrackerDungeonItemsDisplayType",
      "gTrackers.ItemTracker.DisplayType.DungeonItems" },
    { MigrationAction::Rename, "gItemTrackerDungeonRewardsCircle", "gTrackers.ItemTracker.DungeonRewardsLayout" },
    { MigrationAction::Rename, "gItemTrackerDungeonRewardsDisplayType",
      "gTrackers.ItemTracker.DisplayType.DungeonRewards" },
    { MigrationAction::Rename, "gItemTrackerEquipmentItemsDisplayType", "gTrackers.ItemTracker.DisplayType.Equipment" },
    { MigrationAction::Rename, "gItemTrackerGregDisplayType", "gTrackers.ItemTracker.DisplayType.Greg" },
    { MigrationAction::Rename, "gItemTrackerHudEditMode", "gTrackers.ItemTracker.HudEditMode" },
    { MigrationAction::Rename, "gItemTrackerIconSize", "gTrackers.ItemTracker.IconSize" },
    { MigrationAction::Rename, "gItemTrackerIconSpacing", "gTrackers.ItemTracker.IconSpacing" },
    { MigrationAction::Rename, "gItemTrackerInventoryItemsDisplayType", "gTrackers.ItemTracker.DisplayType.Inventory" },
    { MigrationAction::Rename, "gItemTrackerKeyTrack", "gTrackers.ItemTracker.KeyCounts" },
    { MigrationAction::Rename, "gItemTrackerMiscItemsDisplayType", "gTrackers.ItemTracker.DisplayType.Misc" },
    { MigrationAction::Rename, "gItemTrackerNotesDisplayType", "gTrackers.ItemTracker.DisplayType.Notes" },
    { MigrationAction::Rename, "gItemTrackerShowOnlyPaused", "gTrackers.ItemTracker.ShowOnlyPaused" },
    { MigrationAction::Rename, "gItemTrackerSongsDisplayType", "gTrackers.ItemTracker.DisplayType.Songs" },
    { MigrationAction::Rename, "gItemTrackerTriforcePieceTrack", "gTrackers.ItemTracker.TriforcePieceCounts" },
    { MigrationAction::Rename, "gItemTrackerTriforcePiecesDisplayType",
      "gTrackers.ItemTracker.DisplayType.TriforcePieces" },
    { MigrationAction::Rename, "gItemTrackerWindowType", "gTrackers.ItemTracker.WindowType" },
    { MigrationAction::Rename, "gTrackers.ItemTracker.ItemTrackerTextSize", "gTrackers.ItemTracker.TextSize" },
    { MigrationAction::Rename, "gEntranceTrackerAutoScroll", "gTrackers.EntranceTracker.AutoScroll" },
    { MigrationAction::Rename, "gEntranceTrackerCollapseUndiscovered",
      "gTrackers.EntranceTracker.CollapseUndiscovered" },
    { MigrationAction::Rename, "gEntranceTrackerGroupBy", "gTrackers.EntranceTracker.GroupBy" },
    { MigrationAction::Rename, "gEntranceTrackerHideReverseEntrances",
      "gTrackers.EntranceTracker.HideReverseEntrances" },
    { MigrationAction::Rename, "gEntranceTrackerHighlightAvailable", "gTrackers.EntranceTracker.HighlightAvailable" },
    { MigrationAction::Rename, "gEntranceTrackerHighlightPrevious", "gTrackers.EntranceTracker.HighlightPrevious" },
    { MigrationAction::Rename, "gEntranceTrackerShowFrom", "gTrackers.EntranceTracker.ShowFrom" },
    { MigrationAction::Rename, "gEntranceTrackerShowTo", "gTrackers.EntranceTracker.ShowTo" },
    { MigrationAction::Rename, "gEntranceTrackerSortBy", "gTrackers.EntranceTracker.SortBy" },
    { MigrationAction::Rename, "gCheckTrackerAreaCompleteHide", "gTrackers.CheckTracker.AreaComplete.Hide" },
    { MigrationAction::Rename, "gCheckTrackerAreaExtraCompleteColor",
      "gTrackers.CheckTracker.AreaComplete.ExtraColor.Value" },
    { MigrationAction::Rename, "gCheckTrackerAreaExtraIncompleteColor.Value",
      "gTrackers.CheckTracker.AreaIncomplete.ExtraColor.Value" },
    { MigrationAction::Rename, "gCheckTrackerAreaIncompleteHide", "gTrackers.CheckTracker.AreaIncomplete.Hide" },
    { MigrationAction::Rename, "gCheckTrackerAreaMainCompleteColor",
      "gTrackers.CheckTracker.AreaComplete.MainColor.Value" },
    { MigrationAction::Rename, "gCheckTrackerAreaMainIncompleteColor.Value",
      "gTrackers.CheckTracker.AreaIncomplete.MainColor.Value" },
    { MigrationAction::Rename, "gCheckTrackerBgColor", "gTrackers.CheckTracker.BgColor.Value" },
    { MigrationAction::Rename, "gCheckTrackerCollectedExtraColor",
      "gTrackers.CheckTracker.Collected.ExtraColor.Value" },
    { MigrationAction::Rename, "gCheckTrackerCollectedHide", "gTrackers.CheckTracker.Collected.Hide" },
    { MigrationAction::Rename, "gCheckTrackerCollectedMainColor", "gTrackers.CheckTracker.Collected.MainColor.Value" },
    { MigrationAction::Rename, "gCheckTrackerComboButton1", "gTrackers.CheckTracker.ComboButton1" },
    { MigrationAction::Rename, "gCheckTrackerComboButton2", "gTrackers.CheckTracker.ComboButton2" },
    { MigrationAction::Rename, "gCheckTrackerDisplayType", "gTrackers.CheckTracker.DisplayType" },
    { MigrationAction::Rename, "gCheckTrackerHintedExtraColor", "gTrackers.CheckTracker.Hinted.ExtraColor.Value" },
    { MigrationAction::Rename, "gCheckTrackerHintedMainColor", "gTrackers.CheckTracker.Hinted.MainColor.Value" },
    { MigrationAction::Rename, "gCheckTrackerHintedHide", "gTrackers.CheckTracker.Hinted.Hide" },
    { MigrationAction::Rename, "gCheckTrackerHudEditMode", "gTrackers.CheckTracker.Draggable" },
    { MigrationAction::Rename, "gCheckTrackerKnownHide", "gTrackers.CheckTracker.Scummed.Hide" },
    { MigrationAction::Rename, "gCheckTrackerOptionAlwaysShowGSLocs", "gTrackers.CheckTracker.AlwaysShowGSLocs" },
    { MigrationAction::Rename, "gCheckTrackerOptionHideRightShopChecks",
      "gTrackers.CheckTracker.HideUnshuffledShopChecks" },
    { MigrationAction::Rename, "gCheckTrackerOptionMQSpoilers", "gTrackers.CheckTracker.MQSpoilers" },
    { MigrationAction::Rename, "gCheckTrackerOptionShowHidden", "gTrackers.CheckTracker.ShowHidden" },
    { MigrationAction::Rename, "gCheckTrackerSavedExtraColor", "gTrackers.CheckTracker.Saved.ExtraColor.Value" },
    { MigrationAction::Rename, "gCheckTrackerSavedHide", "gTrackers.CheckTracker.Saved.Hide" },
    { MigrationAction::Rename, "gCheckTrackerSavedMainColor", "gTrackers.CheckTracker.Saved.MainColor" },
    { MigrationAction::Rename, "gCheckTrackerScummedExtraColor", "gTrackers.CheckTracker.Scummed.ExtraColor" },
    { MigrationAction::Rename, "gCheckTrackerScummedMainColor", "gTrackers.CheckTracker.Scummed.MainColor" },
    { MigrationAction::Rename, "gCheckTrackerSeenExtraColor", "gTrackers.CheckTracker.Seen.ExtraColor" },
    { MigrationAction::Rename, "gCheckTrackerSeenHide", "gTrackers.CheckTracker.Seen.Hide" },
    { MigrationAction::Rename, "gCheckTrackerSeenMainColor", "gTrackers.CheckTracker.Seen.MainColor" },
    { MigrationAction::Rename, "gCheckTrackerShowOnlyPaused", "gTrackers.CheckTracker.ShowOnlyPaused" },
    { MigrationAction::Rename, "gCheckTrackerSkippedExtraColor", "gTrackers.CheckTracker.Skipped.ExtraColor" },
    { MigrationAction::Rename, "gCheckTrackerSkippedHide", "gTrackers.CheckTracker.Skipped.Hide" },
    { MigrationAction::Rename, "gCheckTrackerSkippedMainColor", "gTrackers.CheckTracker.Skipped.MainColor.Value" },
    { MigrationAction::Rename, "gCheckTrackerUncheckedExtraColor",
      "gTrackers.CheckTracker.Unchecked.ExtraColor.Value" },
    { MigrationAction::Rename, "gCheckTrackerUncheckedHide", "gTrackers.CheckTracker.Unchecked.Hide" },
    { MigrationAction::Rename, "gCheckTrackerUncheckedMainColor", "gTrackers.CheckTracker.Unchecked.MainColor.Value" },
    { MigrationAction::Rename, "gCheckTrackerWindowType", "gTrackers.CheckTracker.WindowType" },
    { MigrationAction::Rename, "gRandomize10GSHint", "gRandoSettings.10GSHint" },
    { MigrationAction::Rename, "gRandomize20GSHint", "gRandoSettings.20GSHint" },
    { MigrationAction::Rename, "gRandomize30GSHint", "gRandoSettings.30GSHint" },
    { MigrationAction::Rename, "gRandomize40GSHint", "gRandoSettings.40GSHint" },
    { MigrationAction::Rename, "gRandomize50GSHint", "gRandoSettings.50GSHint" },
    { MigrationAction::Rename, "gRandomizeAllLocationsReachable", "gRandoSettings.AllLocationsReachable" },
    { MigrationAction::Rename, "gRandomizeAltarHint", "gRandoSettings.AltarHint" },
    { MigrationAction::Rename, "gRandomizeBigPoeTargetCount", "gRandoSettings.BigPoeTargetCount" },
    { MigrationAction::Rename, "gRandomizeBlueFireArrows", "gRandoSettings.BlueFireArrows" },
    { MigrationAction::Rename, "gRandomizeBombchusInLogic", "gRandoSettings.BombchusInLogic" },
    { MigrationAction::Rename, "gRandomizeBossKeysanity", "gRandoSettings.BossKeysanity" },
    { MigrationAction::Rename, "gRandomizeBridgeRewardOptions", "gRandoSettings.BridgeRewardOptions" },
    { MigrationAction::Rename, "gRandomizeCompleteMaskQuest", "gRandoSettings.CompleteMaskQuest" },
    { MigrationAction::Rename, "gRandomizeCuccosToReturn", "gRandoSettings.CuccosToReturn" },
    { MigrationAction::Rename, "gRandomizeDampeHint", "gRandoSettings.DampeHint" },
    { MigrationAction::Rename, "gRandomizeDecoupleEntrances", "gRandoSettings.DecoupleEntrances" },
    { MigrationAction::Rename, "gRandomizeDoorOfTime", "gRandoSettings.DoorOfTime" },
    { MigrationAction::Rename, "gRandomizeDungeonCount", "gRandoSettings.DungeonCount" },
    { MigrationAction::Rename, "gRandomizeEnableBombchuDrops", "gRandoSettings.EnableBombchuDrops" },
    { MigrationAction::Rename, "gRandomizeEnabledTricks", "gRandoSettings.EnabledTricks" },
    { MigrationAction::Rename, "gRandomizeForest", "gRandoSettings.ClosedForest" },
    { MigrationAction::Rename, "gRandomizeFrogsHint", "gRandoSettings.FrogsHint" },
    { MigrationAction::Rename, "gRandomizeFullWallets", "gRandoSettings.FullWallets" },
    { MigrationAction::Rename, "gRandomizeGanonTrial", "gRandoSettings.GanonTrial" },
    { MigrationAction::Rename, "gRandomizeGanonTrialCount", "gRandoSettings.GanonTrialCount" },
    { MigrationAction::Rename, "gRandomizeGerudoFortress", "gRandoSettings.FortressCarpenters" },
    { MigrationAction::Rename, "gRandomizeGerudoKeys", "gRandoSettings.GerudoKeys" },
    { MigrationAction::Rename, "gRandomizeGossipStoneHints", "gRandoSettings.GossipStoneHints" },
    { MigrationAction::Rename, "gRandomizeGregHint", "gRandoSettings.GregHint" },
    { MigrationAction::Rename, "gRandomizeGsExpectSunsSong", "gRandoSettings.GsExpectSunsSong" },
    { MigrationAction::Rename, "gRandomizeHintClarity", "gRandoSettings.HintClarity" },
    { MigrationAction::Rename, "gRandomizeHintDistribution", "gRandoSettings.HintDistribution" },
    { MigrationAction::Rename, "gRandomizeIceTraps", "gRandoSettings.IceTraps" },
    { MigrationAction::Rename, "gRandomizeItemPool", "gRandoSettings.ItemPool" },
    { MigrationAction::Rename, "gRandomizeKakarikoGate", "gRandoSettings.KakarikoGate" },
    { MigrationAction::Rename, "gRandomizeKeysanity", "gRandoSettings.Keysanity" },
    { MigrationAction::Rename, "gRandomizeLAHint", "gRandoSettings.LAHint" },
    { MigrationAction::Rename, "gRandomizeLacsDungeonCount", "gRandoSettings.LacsDungeonCount" },
    { MigrationAction::Rename, "gRandomizeLacsMedallionCount", "gRandoSettings.LacsMedallionCount" },
    { MigrationAction::Rename, "gRandomizeLacsRewardCount", "gRandoSettings.LacsRewardCount" },
    { MigrationAction::Rename, "gRandomizeLacsRewardOptions", "gRandoSettings.LacsRewardOptions" },
    { MigrationAction::Rename, "gRandomizeLacsStoneCount", "gRandoSettings.LacsStoneCount" },
    { MigrationAction::Rename, "gRandomizeLacsTokenCount", "gRandoSettings.LacsTokenCount" },
    { MigrationAction::Rename, "gRandomizeLinksPocket", "gRandoSettings.LinksPocket" },
    { MigrationAction::Rename, "gRandomizeLogicRules", "gRandoSettings.LogicRules" },
    { MigrationAction::Rename, "gRandomizeMqDungeonCount", "gRandoSettings.MQDungeonCount" },
    { MigrationAction::Rename, "gRandomizeMqDungeons", "gRandoSettings.MQDungeons" },
    { MigrationAction::Rename, "gRandomizeMqDungeonsBottomOfTheWell", "gRandoSettings.MQDungeonsBottomOfTheWell" },
    { MigrationAction::Rename, "gRandomizeMqDungeonsDekuTree", "gRandoSettings.MQDungeonsDekuTree" },
    { MigrationAction::Rename, "gRandomizeMqDungeonsDodongosCavern", "gRandoSettings.MQDungeonsDodongosCavern" },
    { MigrationAction::Rename, "gRandomizeMqDungeonsFireTemple", "gRandoSettings.MQDungeonsFireTemple" },
    { MigrationAction::Rename, "gRandomizeMqDungeonsForestTemple", "gRandoSettings.MQDungeonsForestTemple" },
    { MigrationAction::Rename, "gRandomizeMqDungeonsGTG", "gRandoSettings.MQDungeonsGTG" },
    { MigrationAction::Rename, "gRandomizeMqDungeonsGanonsCastle", "gRandoSettings.MQDungeonsGanonsCastle" },
    { MigrationAction::Rename, "gRandomizeMqDungeonsIceCavern", "gRandoSettings.MQDungeonsIceCavern" },
    { MigrationAction::Rename, "gRandomizeMqDungeonsJabuJabu", "gRandoSettings.MQDungeonsJabuJabu" },
    { MigrationAction::Rename, "gRandomizeMqDungeonsShadowTemple", "gRandoSettings.MQDungeonsShadowTemple" },
    { MigrationAction::Rename, "gRandomizeMqDungeonsSpiritTemple", "gRandoSettings.MQDungeonsSpiritTemple" },
    { MigrationAction::Rename, "gRandomizeMqDungeonsWaterTemple", "gRandoSettings.MQDungeonsWaterTemple" },
    { MigrationAction::Rename, "gRandomizeMedallionCount", "gRandoSettings.MedallionCount" },
    { MigrationAction::Rename, "gRandomizeMixBosses", "gRandoSettings.MixBosses" },
    { MigrationAction::Rename, "gRandomizeMixDungeons", "gRandoSettings.MixDungeons" },
    { MigrationAction::Rename, "gRandomizeMixGrottos", "gRandoSettings.MixGrottos" },
    { MigrationAction::Rename, "gRandomizeMixInteriors", "gRandoSettings.MixInteriors" },
    { MigrationAction::Rename, "gRandomizeMixOverworld", "gRandoSettings.MixOverworld" },
    { MigrationAction::Rename, "gRandomizeMixedEntrances", "gRandoSettings.MixedEntrances" },
    { MigrationAction::Rename, "gRandomizeRainbowBridge", "gRandoSettings.RainbowBridge" },
    { MigrationAction::Rename, "gRandomizeRewardCount", "gRandoSettings.RewardCount" },
    { MigrationAction::Rename, "gRandomizeSariaHint", "gRandoSettings.SariaHint" },
    { MigrationAction::Rename, "gRandomizeScrubText", "gRandoSettings.ScrubText" },
    { MigrationAction::Remove, "gRandomizeShopsanity" },
    { MigrationAction::Rename, "gRandomizeShopsanityPrices", "gRandoSettings.ShopsanityPrices" },
    { MigrationAction::Rename, "gRandomizeShopsanityPricesAffordable", "gRandoSettings.ShopsanityPricesAffordable" },
    { MigrationAction::Rename, "gRandomizeShuffle100GSReward", "gRandoSettings.Shuffle100GSReward" },
    { MigrationAction::Rename, "gRandomizeShuffleAdultTrade", "gRandoSettings.ShuffleAdultTrade" },
    { MigrationAction::Rename, "gRandomizeShuffleBeans", "gRandoSettings.ShuffleBeans" },
    { MigrationAction::Rename, "gRandomizeShuffleBossEntrances", "gRandoSettings.ShuffleBossEntrances" },
    { MigrationAction::Rename, "gRandomizeShuffleCows", "gRandoSettings.ShuffleCows" },
    { MigrationAction::Rename, "gRandomizeShuffleDungeonReward", "gRandoSettings.ShuffleDungeonReward" },
    { MigrationAction::Rename, "gRandomizeShuffleDungeonsEntrances", "gRandoSettings.ShuffleDungeonsEntrances" },
    { MigrationAction::Rename, "gRandomizeShuffleFrogSongRupees", "gRandoSettings.ShuffleFrogSongRupees" },
    { MigrationAction::Rename, "gRandomizeShuffleGanonBossKey", "gRandoSettings.ShuffleGanonBossKey" },
    { MigrationAction::Rename, "gRandomizeShuffleGerudoToken", "gRandoSettings.ShuffleGerudoToken" },
    { MigrationAction::Rename, "gRandomizeShuffleGrottosEntrances", "gRandoSettings.ShuffleGrottosEntrances" },
    { MigrationAction::Rename, "gRandomizeShuffleInteriorsEntrances", "gRandoSettings.ShuffleInteriorsEntrances" },
    { MigrationAction::Rename, "gRandomizeShuffleKeyRings", "gRandoSettings.ShuffleKeyRings" },
    { MigrationAction::Rename, "gRandomizeShuffleKeyRingsBottomOfTheWell",
      "gRandoSettings.ShuffleKeyRingsBottomOfTheWell" },
    { MigrationAction::Rename, "gRandomizeShuffleKeyRingsFireTemple", "gRandoSettings.ShuffleKeyRingsFireTemple" },
    { MigrationAction::Rename, "gRandomizeShuffleKeyRingsForestTemple", "gRandoSettings.ShuffleKeyRingsForestTemple" },
    { MigrationAction::Rename, "gRandomizeShuffleKeyRingsGTG", "gRandoSettings.ShuffleKeyRingsGTG" },
    { MigrationAction::Rename, "gRandomizeShuffleKeyRingsGanonsCastle", "gRandoSettings.ShuffleKeyRingsGanonsCastle" },
    { MigrationAction::Rename, "gRandomizeShuffleKeyRingsGerudoFortress",
      "gRandoSettings.ShuffleKeyRingsGerudoFortress" },
    { MigrationAction::Rename, "gRandomizeShuffleKeyRingsRandomCount", "gRandoSettings.ShuffleKeyRingsRandomCount" },
    { MigrationAction::Rename, "gRandomizeShuffleKeyRingsShadowTemple", "gRandoSettings.ShuffleKeyRingsShadowTemple" },
    { MigrationAction::Rename, "gRandomizeShuffleKeyRingsSpiritTemple", "gRandoSettings.ShuffleKeyRingsSpiritTemple" },
    { MigrationAction::Rename, "gRandomizeShuffleKeyRingsWaterTemple", "gRandoSettings.ShuffleKeyRingsWaterTemple" },
    { MigrationAction::Rename, "gRandomizeShuffleKokiriSword", "gRandoSettings.ShuffleKokiriSword" },
    { MigrationAction::Rename, "gRandomizeShuffleMasterSword", "gRandoSettings.ShuffleMasterSword" },
    { MigrationAction::Rename, "gRandomizeShuffleMerchants", "gRandoSettings.ShuffleMerchants" },
    { MigrationAction::Rename, "gRandomizeShuffleOcarinas", "gRandoSettings.ShuffleOcarinas" },
    { MigrationAction::Rename, "gRandomizeShuffleOverworldEntrances", "gRandoSettings.ShuffleOverworldEntrances" },
    { MigrationAction::Rename, "gRandomizeShuffleOverworldSpawns", "gRandoSettings.ShuffleOverworldSpawns" },
    { MigrationAction::Rename, "gRandomizeShuffleOwlDrops", "gRandoSettings.ShuffleOwlDrops" },
    { MigrationAction::Remove, "gRandomizeShuffleScrubs" },
    { MigrationAction::Rename, "gRandomizeShuffleSongs", "gRandoSettings.ShuffleSongs" },
    { MigrationAction::Rename, "gRandomizeShuffleTokens", "gRandoSettings.ShuffleTokens" },
    { MigrationAction::Rename, "gRandomizeShuffleWarpSongs", "gRandoSettings.ShuffleWarpSongs" },
    { MigrationAction::Rename, "gRandomizeShuffleWeirdEgg", "gRandoSettings.ShuffleWeirdEgg" },
    { MigrationAction::Rename, "gRandomizeSkipChildZelda", "gRandoSettings.SkipChildZelda" },
    { MigrationAction::Rename, "gRandomizeSkipEponaRace", "gRandoSettings.SkipEponaRace" },
    { MigrationAction::Rename, "gRandomizeSkipScarecrowsSong", "gRandoSettings.SkipScarecrowsSong" },
    { MigrationAction::Rename, "gRandomizeSkipTowerEscape", "gRandoSettings.SkipTowerEscape" },
    { MigrationAction::Rename, "gRandomizeStartingAge", "gRandoSettings.StartingAge" },
    { MigrationAction::Rename, "gRandomizeStartingBoleroOfFire", "gRandoSettings.StartingBoleroOfFire" },
    { MigrationAction::Rename, "gRandomizeStartingConsumables", "gRandoSettings.StartingConsumables" },
    { MigrationAction::Rename, "gRandomizeStartingDekuShield", "gRandoSettings.StartingDekuShield" },
    { MigrationAction::Rename, "gRandomizeStartingEponasSong", "gRandoSettings.StartingEponasSong" },
    { MigrationAction::Rename, "gRandomizeStartingKokiriSword", "gRandoSettings.StartingKokiriSword" },
    { MigrationAction::Rename, "gRandomizeStartingMapsCompasses", "gRandoSettings.StartingMapsCompasses" },
    { MigrationAction::Rename, "gRandomizeStartingMinuetOfForest", "gRandoSettings.StartingMinuetOfForest" },
    { MigrationAction::Rename, "gRandomizeStartingNocturneOfShadow", "gRandoSettings.StartingNocturneOfShadow" },
    { MigrationAction::Rename, "gRandomizeStartingOcarina", "gRandoSettings.StartingOcarina" },
    { MigrationAction::Rename, "gRandomizeStartingPreludeOfLight", "gRandoSettings.StartingPreludeOfLight" },
    { MigrationAction::Rename, "gRandomizeStartingRequiemOfSpirit", "gRandoSettings.StartingRequiemOfSpirit" },
    { MigrationAction::Rename, "gRandomizeStartingSariasSong", "gRandoSettings.StartingSariasSong" },
    { MigrationAction::Rename, "gRandomizeStartingSerenadeOfWater", "gRandoSettings.StartingSerenadeOfWater" },
    { MigrationAction::Rename, "gRandomizeStartingSkulltulaToken", "gRandoSettings.StartingSkulltulaToken" },
    { MigrationAction::Rename, "gRandomizeStartingSongOfStorms", "gRandoSettings.StartingSongOfStorms" },
    { MigrationAction::Rename, "gRandomizeStartingSongOfTime", "gRandoSettings.StartingSongOfTime" },
    { MigrationAction::Rename, "gRandomizeStartingSunsSong", "gRandoSettings.StartingSunsSong" },
    { MigrationAction::Rename, "gRandomizeStartingZeldasLullaby", "gRandoSettings.StartingZeldasLullaby" },
    { MigrationAction::Rename, "gRandomizeStoneCount", "gRandoSettings.StoneCount" },
    { MigrationAction::Rename, "gRandomizeSunlightArrows", "gRandoSettings.SunlightArrows" },
    { MigrationAction::Rename, "gRandomizeTokenCount", "gRandoSettings.TokenCount" },
    { MigrationAction::Rename, "gRandomizeTriforceHunt", "gRandoSettings.TriforceHunt" },
    { MigrationAction::Rename, "gRandomizeTriforceHuntRequiredPieces", "gRandoSettings.TriforceHuntRequiredPieces" },
    { MigrationAction::Rename, "gRandomizeTriforceHuntTotalPieces", "gRandoSettings.TriforceHuntTotalPieces" },
    { MigrationAction::Rename, "gRandomizeWarpSongText", "gRandoSettings.WarpSongText" },
    { MigrationAction::Rename, "gRandomizeZorasFountain", "gRandoSettings.ZorasFountain" },
    { MigrationAction::Rename, "gLoadedPreset", "gRandoSettings.LoadedPreset" },
    { MigrationAction::Rename, "gRandoRelevantNavi", "gRandoEnhancements.RandoRelevantNavi" },
    { MigrationAction::Rename, "gRandomizeRupeeNames", "gRandoEnhancements.RandomizeRupeeNames" },
    { MigrationAction::Rename, "gRandoMatchKeyColors", "gRandoEnhancements.MatchKeyColors" },
    { MigrationAction::Rename, "gRandoQuestItemFanfares", "gRandoEnhancements.QuestItemFanfares" },
    { MigrationAction::Rename, "gInputEnabled", "gOpenWindows.InputViewer" },
    { MigrationAction::Rename, "gOpenMenuBar", "gSettings.OpenMenuBar" },
    { MigrationAction::Rename, "gRandomizeSkipChildStealth", "gRandoSettings.SkipChildStealth" },
    { MigrationAction::Rename, "gRandomizeExcludedLocations", "gRandoSettings.ExcludedLocations" },
    { MigrationAction::Rename, "gAltAssets", "gEnhancements.AltAssets" },
    { MigrationAction::Rename, "gMSAAValue", "gSettings.MSAAValue" },
    { MigrationAction::Rename, "gInternalResolution", "gSettings.InternalResolution" },
    { MigrationAction::Rename, "gTextureFilter", "gSettings.TextureFilter" },
    { MigrationAction::Rename, "gVsyncEnabled", "gSettings.VsyncEnabled" },
    { MigrationAction::Rename, "gControlNav", "gSettings.ControlNav" },
    { MigrationAction::Rename, "gEnableMultiViewports", "gSettings.EnableMultiViewports" },
    { MigrationAction::Rename, "gOverlayFont", "gSettings.OverlayFont" },
    { MigrationAction::Rename, "gCrowdControl", "gRemote.CrowdControl" },
    { MigrationAction::Remove, "gItemTrackerNotes0" },
    { MigrationAction::Remove, "gItemTrackerNotes1" },
    { MigrationAction::Remove, "gItemTrackerNotes2" },
    { MigrationAction::Remove, "gHideBuildInfo" },
    { MigrationAction::Remove, "gGameControlEditorEnabled" },
    { MigrationAction::Remove, "gPreset0" },
    { MigrationAction::Remove, "gPreset1" },
};
} // namespace SOH
