#include "ShufflePots.h"
#include "soh_assets.h"
#include "static_data.h"

extern "C" {
#include "variables.h"
#include "overlays/actors/ovl_Obj_Tsubo/z_obj_tsubo.h"
#include "overlays/actors/ovl_Door_Shutter/z_door_shutter.h"
extern PlayState* gPlayState;
}

extern void EnItem00_DrawRandomizedItem(EnItem00* enItem00, PlayState* play);

extern "C" void ObjTsubo_RandomizerDraw(Actor* thisx, PlayState* play) {
    float potSize = 1.0f;

    OPEN_DISPS(play->state.gfxCtx);
    Gfx_SetupDL_25Opa(play->state.gfxCtx);
    Matrix_Scale(potSize, potSize, potSize, MTXMODE_APPLY);
    gSPMatrix(POLY_OPA_DISP++, Matrix_NewMtx(play->state.gfxCtx, (char*)__FILE__, __LINE__),
              G_MTX_MODELVIEW | G_MTX_LOAD);

    gSPDisplayList(POLY_OPA_DISP++, (Gfx*)gRandoPotDL);
    CLOSE_DISPS(play->state.gfxCtx);
}

uint8_t ObjTsubo_RandomizerHoldsItem(ObjTsubo* potActor, PlayState* play) {
    RandomizerCheck rc = potActor->potIdentity.randomizerCheck;
    uint8_t isDungeon = Rando::StaticData::GetLocation(rc)->IsDungeon();
    uint8_t potSetting = Rando::Context::GetInstance()->GetOption(RSK_SHUFFLE_POTS).Get();

    // Don't pull randomized item if pot isn't randomized or is already checked
    if (!IS_RANDO || (potSetting == RO_SHUFFLE_POTS_OVERWORLD && isDungeon) ||
        (potSetting == RO_SHUFFLE_POTS_DUNGEONS && !isDungeon) ||
        Flags_GetRandomizerInf(potActor->potIdentity.randomizerInf) ||
        potActor->potIdentity.randomizerCheck == RC_UNKNOWN_CHECK) {
        return false;
    } else {
        return true;
    }
}

void ObjTsubo_RandomizerSpawnCollectible(ObjTsubo* potActor, PlayState* play) {
    EnItem00* item00 = (EnItem00*)Item_DropCollectible2(play, &potActor->actor.world.pos, ITEM00_SOH_DUMMY);
    item00->randoInf = potActor->potIdentity.randomizerInf;
    item00->itemEntry =
        Rando::Context::GetInstance()->GetFinalGIEntry(potActor->potIdentity.randomizerCheck, true, GI_NONE);
    item00->actor.draw = (ActorFunc)EnItem00_DrawRandomizedItem;
    item00->actor.velocity.y = 8.0f;
    item00->actor.speedXZ = 2.0f;
    item00->actor.world.rot.y = Rand_CenteredFloat(65536.0f);
}

void ObjTsubo_RandomizerInit(void* actorRef) {
    Actor* actor = static_cast<Actor*>(actorRef);

    ObjTsubo* potActor = static_cast<ObjTsubo*>(actorRef);

    potActor->potIdentity = OTRGlobals::Instance->gRandomizer->IdentifyPot(
        gPlayState->sceneNum, (s16)actor->world.pos.x, (s16)actor->world.pos.z);
}

void ShufflePots_OnVanillaBehaviorHandler(GIVanillaBehavior id, bool* should, va_list originalArgs) {
    va_list args;
    va_copy(args, originalArgs);

    // Draw custom model for pot to indicate it holding a randomized item.
    if (id == VB_POT_SETUP_DRAW) {
        ObjTsubo* potActor = va_arg(args, ObjTsubo*);
        if (ObjTsubo_RandomizerHoldsItem(potActor, gPlayState)) {
            potActor->actor.draw = (ActorFunc)ObjTsubo_RandomizerDraw;
            *should = false;
        }
    }

    // Do not spawn vanilla item from pot, instead spawn the ranomized item.
    if (id == VB_POT_DROP_ITEM) {
        ObjTsubo* potActor = va_arg(args, ObjTsubo*);
        if (ObjTsubo_RandomizerHoldsItem(potActor, gPlayState)) {
            ObjTsubo_RandomizerSpawnCollectible(potActor, gPlayState);
            *should = false;
        }
    }

    // Unlock early Ganon's Boss Key doors to allow access to the pots there when pots are shuffled in dungeon
    if (id == VB_LOCK_BOSS_DOOR) {
        DoorShutter* doorActor = va_arg(args, DoorShutter*);
        uint8_t shufflePotSetting = Rando::Context::GetInstance()->GetOption(RSK_SHUFFLE_POTS).Get();
        if (gPlayState->sceneNum == SCENE_GANONS_TOWER && doorActor->dyna.actor.world.pos.y == 800 &&
            (shufflePotSetting == RO_SHUFFLE_POTS_DUNGEONS || shufflePotSetting == RO_SHUFFLE_POTS_ALL)) {
            *should = false;
        }
    }

    va_end(args);
}

void Rando::StaticData::RegisterPotLocations() {
    static bool registered = false;
    if (registered)
        return;
    registered = true;
    // clang-format off
    // Overworld Pots
    //            Randomizer Check                              Randomizer Check                                           Quest         Area                          Scene ID                           Params                          Short Name                          Hint Text Key                     Vanilla             Spoiler Collection Check
    locationTable[RC_KF_LINKS_HOUSE_POT]                   =    Location::Pot(RC_KF_LINKS_HOUSE_POT,                       RCQUEST_BOTH, RCAREA_KOKIRI_FOREST,         SCENE_LINKS_HOUSE,                 TWO_ACTOR_PARAMS(-118, 51),     "Links House Pot",                  RHT_POT_KOKIRI_FOREST,            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_KF_LINKS_HOUSE_POT));
    locationTable[RC_KF_TWINS_HOUSE_POT_2]                 =    Location::Pot(RC_KF_TWINS_HOUSE_POT_2,                     RCQUEST_BOTH, RCAREA_KOKIRI_FOREST,         SCENE_TWINS_HOUSE,                 TWO_ACTOR_PARAMS(35, 57),       "Twins House Pot 2",                RHT_POT_KOKIRI_FOREST,            RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_KF_TWINS_HOUSE_POT_2));
    locationTable[RC_KF_TWINS_HOUSE_POT_1]                 =    Location::Pot(RC_KF_TWINS_HOUSE_POT_1,                     RCQUEST_BOTH, RCAREA_KOKIRI_FOREST,         SCENE_TWINS_HOUSE,                 TWO_ACTOR_PARAMS(33, -55),      "Twins House Pot 1",                RHT_POT_KOKIRI_FOREST,            RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_KF_TWINS_HOUSE_POT_1));
    locationTable[RC_KF_BROTHERS_HOUSE_POT_1]              =    Location::Pot(RC_KF_BROTHERS_HOUSE_POT_1,                  RCQUEST_BOTH, RCAREA_KOKIRI_FOREST,         SCENE_KNOW_IT_ALL_BROS_HOUSE,      TWO_ACTOR_PARAMS(-134, -29),    "Brothers House Pot 1",             RHT_POT_KOKIRI_FOREST,            RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_KF_BROTHERS_HOUSE_POT_1));
    locationTable[RC_KF_BROTHERS_HOUSE_POT_2]              =    Location::Pot(RC_KF_BROTHERS_HOUSE_POT_2,                  RCQUEST_BOTH, RCAREA_KOKIRI_FOREST,         SCENE_KNOW_IT_ALL_BROS_HOUSE,      TWO_ACTOR_PARAMS(-68, 114),     "Brothers House Pot 2",             RHT_POT_KOKIRI_FOREST,            RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_KF_BROTHERS_HOUSE_POT_2));
    locationTable[RC_GF_BREAK_ROOM_POT_1]                  =    Location::Pot(RC_GF_BREAK_ROOM_POT_1,                      RCQUEST_BOTH, RCAREA_GERUDO_FORTRESS,       SCENE_THIEVES_HIDEOUT,             TWO_ACTOR_PARAMS(1438, -3629),  "Break Room Pot 1",                 RHT_POT_GERUDO_FORTRESS,          RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_GF_BREAK_ROOM_POT_1));
    locationTable[RC_GF_BREAK_ROOM_POT_2]                  =    Location::Pot(RC_GF_BREAK_ROOM_POT_2,                      RCQUEST_BOTH, RCAREA_GERUDO_FORTRESS,       SCENE_THIEVES_HIDEOUT,             TWO_ACTOR_PARAMS(1403, -3679),  "Break Room Pot 2",                 RHT_POT_GERUDO_FORTRESS,          RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_GF_BREAK_ROOM_POT_2));
    locationTable[RC_GF_KITCHEN_POT_1]                     =    Location::Pot(RC_GF_KITCHEN_POT_1,                         RCQUEST_BOTH, RCAREA_GERUDO_FORTRESS,       SCENE_THIEVES_HIDEOUT,             TWO_ACTOR_PARAMS(1908, -789),   "Kitchen Pot 1",                    RHT_POT_GERUDO_FORTRESS,          RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_GF_KITCHEN_POT_1));
    locationTable[RC_GF_KITCHEN_POT_2]                     =    Location::Pot(RC_GF_KITCHEN_POT_2,                         RCQUEST_BOTH, RCAREA_GERUDO_FORTRESS,       SCENE_THIEVES_HIDEOUT,             TWO_ACTOR_PARAMS(1951, -850),   "Kitchen Pot 2",                    RHT_POT_GERUDO_FORTRESS,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GF_KITCHEN_POT_2));
    locationTable[RC_GF_NORTH_F1_CARPENTER_POT_1]          =    Location::Pot(RC_GF_NORTH_F1_CARPENTER_POT_1,              RCQUEST_BOTH, RCAREA_GERUDO_FORTRESS,       SCENE_THIEVES_HIDEOUT,             TWO_ACTOR_PARAMS(-475, -2622),  "North F1 Carpenter Pot 1",         RHT_POT_GERUDO_FORTRESS,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GF_NORTH_F1_CARPENTER_POT_1));
    locationTable[RC_GF_NORTH_F1_CARPENTER_POT_2]          =    Location::Pot(RC_GF_NORTH_F1_CARPENTER_POT_2,              RCQUEST_BOTH, RCAREA_GERUDO_FORTRESS,       SCENE_THIEVES_HIDEOUT,             TWO_ACTOR_PARAMS(-512, -2621),  "North F1 Carpenter Pot 2",         RHT_POT_GERUDO_FORTRESS,          RG_RED_RUPEE,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_GF_NORTH_F1_CARPENTER_POT_2));
    locationTable[RC_GF_NORTH_F1_CARPENTER_POT_3]          =    Location::Pot(RC_GF_NORTH_F1_CARPENTER_POT_3,              RCQUEST_BOTH, RCAREA_GERUDO_FORTRESS,       SCENE_THIEVES_HIDEOUT,             TWO_ACTOR_PARAMS(-511, -2582),  "North F1 Carpenter Pot 3",         RHT_POT_GERUDO_FORTRESS,          RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_GF_NORTH_F1_CARPENTER_POT_3));
    locationTable[RC_GF_NORTH_F2_CARPENTER_POT_1]          =    Location::Pot(RC_GF_NORTH_F2_CARPENTER_POT_1,              RCQUEST_BOTH, RCAREA_GERUDO_FORTRESS,       SCENE_THIEVES_HIDEOUT,             TWO_ACTOR_PARAMS(67, -1975),    "North F2 Carpenter Pot 1",         RHT_POT_GERUDO_FORTRESS,          RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_GF_NORTH_F2_CARPENTER_POT_1));
    locationTable[RC_GF_NORTH_F2_CARPENTER_POT_2]          =    Location::Pot(RC_GF_NORTH_F2_CARPENTER_POT_2,              RCQUEST_BOTH, RCAREA_GERUDO_FORTRESS,       SCENE_THIEVES_HIDEOUT,             TWO_ACTOR_PARAMS(39, -1943),    "North F2 Carpenter Pot 2",         RHT_POT_GERUDO_FORTRESS,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GF_NORTH_F2_CARPENTER_POT_2));
    locationTable[RC_GF_SOUTH_F1_CARPENTER_POT_1]          =    Location::Pot(RC_GF_SOUTH_F1_CARPENTER_POT_1,              RCQUEST_BOTH, RCAREA_GERUDO_FORTRESS,       SCENE_THIEVES_HIDEOUT,             TWO_ACTOR_PARAMS(691, 48),      "South F1 Carpenter Pot 1",         RHT_POT_GERUDO_FORTRESS,          RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_GF_SOUTH_F1_CARPENTER_POT_1));
    locationTable[RC_GF_SOUTH_F1_CARPENTER_POT_2]          =    Location::Pot(RC_GF_SOUTH_F1_CARPENTER_POT_2,              RCQUEST_BOTH, RCAREA_GERUDO_FORTRESS,       SCENE_THIEVES_HIDEOUT,             TWO_ACTOR_PARAMS(691, 16),      "South F1 Carpenter Pot 2",         RHT_POT_GERUDO_FORTRESS,          RG_RED_RUPEE,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_GF_SOUTH_F1_CARPENTER_POT_2));
    locationTable[RC_GF_SOUTH_F1_CARPENTER_POT_3]          =    Location::Pot(RC_GF_SOUTH_F1_CARPENTER_POT_3,              RCQUEST_BOTH, RCAREA_GERUDO_FORTRESS,       SCENE_THIEVES_HIDEOUT,             TWO_ACTOR_PARAMS(661, 16),      "South F1 Carpenter Pot 3",         RHT_POT_GERUDO_FORTRESS,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GF_SOUTH_F1_CARPENTER_POT_3));
    locationTable[RC_GF_SOUTH_F1_CARPENTER_CELL_POT_1]     =    Location::Pot(RC_GF_SOUTH_F1_CARPENTER_CELL_POT_1,         RCQUEST_BOTH, RCAREA_GERUDO_FORTRESS,       SCENE_THIEVES_HIDEOUT,             TWO_ACTOR_PARAMS(26, 524),      "South F1 Carpenter Cell Pot 1",    RHT_POT_GERUDO_FORTRESS,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GF_SOUTH_F1_CARPENTER_CELL_POT_1));
    locationTable[RC_GF_SOUTH_F1_CARPENTER_CELL_POT_2]     =    Location::Pot(RC_GF_SOUTH_F1_CARPENTER_CELL_POT_2,         RCQUEST_BOTH, RCAREA_GERUDO_FORTRESS,       SCENE_THIEVES_HIDEOUT,             TWO_ACTOR_PARAMS(61, 549),      "South F1 Carpenter Cell Pot 2",    RHT_POT_GERUDO_FORTRESS,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GF_SOUTH_F1_CARPENTER_CELL_POT_2));
    locationTable[RC_GF_SOUTH_F1_CARPENTER_CELL_POT_3]     =    Location::Pot(RC_GF_SOUTH_F1_CARPENTER_CELL_POT_3,         RCQUEST_BOTH, RCAREA_GERUDO_FORTRESS,       SCENE_THIEVES_HIDEOUT,             TWO_ACTOR_PARAMS(215, 549),     "South F1 Carpenter Cell Pot 3",    RHT_POT_GERUDO_FORTRESS,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GF_SOUTH_F1_CARPENTER_CELL_POT_3));
    locationTable[RC_GF_SOUTH_F1_CARPENTER_CELL_POT_4]     =    Location::Pot(RC_GF_SOUTH_F1_CARPENTER_CELL_POT_4,         RCQUEST_BOTH, RCAREA_GERUDO_FORTRESS,       SCENE_THIEVES_HIDEOUT,             TWO_ACTOR_PARAMS(254, 529),     "South F1 Carpenter Cell Pot 4",    RHT_POT_GERUDO_FORTRESS,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GF_SOUTH_F1_CARPENTER_CELL_POT_4));
    locationTable[RC_WASTELAND_NEAR_GS_POT_1]              =    Location::Pot(RC_WASTELAND_NEAR_GS_POT_1,                  RCQUEST_BOTH, RCAREA_WASTELAND,             SCENE_HAUNTED_WASTELAND,           TWO_ACTOR_PARAMS(488, -2424),   "Near GS Pot 1",                    RHT_POT_GERUDO_FORTRESS,          RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_WASTELAND_NEAR_GS_POT_1));
    locationTable[RC_WASTELAND_NEAR_GS_POT_2]              =    Location::Pot(RC_WASTELAND_NEAR_GS_POT_2,                  RCQUEST_BOTH, RCAREA_WASTELAND,             SCENE_HAUNTED_WASTELAND,           TWO_ACTOR_PARAMS(485, -2463),   "Near GS Pot 2",                    RHT_POT_GERUDO_FORTRESS,          RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_WASTELAND_NEAR_GS_POT_2));
    locationTable[RC_WASTELAND_NEAR_GS_POT_3]              =    Location::Pot(RC_WASTELAND_NEAR_GS_POT_3,                  RCQUEST_BOTH, RCAREA_WASTELAND,             SCENE_HAUNTED_WASTELAND,           TWO_ACTOR_PARAMS(806, -2426),   "Near GS Pot 3",                    RHT_POT_GERUDO_FORTRESS,          RG_DEKU_NUTS_5,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_WASTELAND_NEAR_GS_POT_3));
    locationTable[RC_WASTELAND_NEAR_GS_POT_4]              =    Location::Pot(RC_WASTELAND_NEAR_GS_POT_4,                  RCQUEST_BOTH, RCAREA_WASTELAND,             SCENE_HAUNTED_WASTELAND,           TWO_ACTOR_PARAMS(801, -2460),   "Near GS Pot 4",                    RHT_POT_GERUDO_FORTRESS,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_WASTELAND_NEAR_GS_POT_4));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_1]           =    Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_1,               RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(-60, 27),      "Guard House Child Pot 1",          RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_1));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_2]           =    Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_2,               RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(-89, 28),      "Guard House Child Pot 2",          RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_2));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_3]           =    Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_3,               RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(-110, 6),      "Guard House Child Pot 3",          RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_3));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_4]           =    Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_4,               RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(-58, -7),      "Guard House Child Pot 4",          RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_4));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_5]           =    Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_5,               RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(-80, -7),      "Guard House Child Pot 5",          RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_5));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_6]           =    Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_6,               RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(-65, -45),     "Guard House Child Pot 6",          RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_6));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_7]           =    Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_7,               RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(-85, -41),     "Guard House Child Pot 7",          RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_7));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_8]           =    Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_8,               RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(-107, -45),    "Guard House Child Pot 8",          RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_8));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_9]           =    Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_9,               RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(-66, -79),     "Guard House Child Pot 9",          RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_9));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_10]          =    Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_10,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(-88, -84),     "Guard House Child Pot 10",         RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_10));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_11]          =    Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_11,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(70, 215),      "Guard House Child Pot 11",         RHT_POT_MARKET,                   RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_11));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_12]          =    Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_12,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(68, 148),      "Guard House Child Pot 12",         RHT_POT_MARKET,                   RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_12));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_13]          =    Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_13,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(73, 117),      "Guard House Child Pot 13",         RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_13));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_14]          =    Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_14,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(40, 123),      "Guard House Child Pot 14",         RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_14));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_15]          =    Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_15,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(43, 89),       "Guard House Child Pot 15",         RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_15));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_16]          =    Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_16,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(68, 81),       "Guard House Child Pot 16",         RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_16));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_17]          =    Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_17,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(21, 73),       "Guard House Child Pot 17",         RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_17));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_18]          =    Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_18,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(64, 45),       "Guard House Child Pot 18",         RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_18));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_19]          =    Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_19,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(84, 31),       "Guard House Child Pot 19",         RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_19));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_20]          =    Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_20,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(42, 26),       "Guard House Child Pot 20",         RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_20));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_21]          =    Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_21,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(20, 34),       "Guard House Child Pot 21",         RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_21));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_22]          =    Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_22,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(89, -2),       "Guard House Child Pot 22",         RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_22));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_23]          =    Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_23,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(70, -12),      "Guard House Child Pot 23",         RHT_POT_MARKET,                   RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_23));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_24]          =    Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_24,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(42, -5),       "Guard House Child Pot 24",         RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_24));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_25]          =    Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_25,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(16, -6),       "Guard House Child Pot 25",         RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_25));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_26]          =    Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_26,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(68, -44),      "Guard House Child Pot 26",         RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_26));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_27]          =    Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_27,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(37, -40),      "Guard House Child Pot 27",         RHT_POT_MARKET,                   RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_27));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_28]          =    Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_28,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(70, -80),      "Guard House Child Pot 28",         RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_28));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_29]          =    Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_29,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(50, -74),      "Guard House Child Pot 29",         RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_29));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_30]          =    Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_30,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(28, -79),      "Guard House Child Pot 30",         RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_30));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_31]          =    Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_31,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(39, -111),     "Guard House Child Pot 31",         RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_31));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_32]          =    Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_32,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(169, 216),     "Guard House Child Pot 32",         RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_32));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_33]          =    Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_33,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(168, 166),     "Guard House Child Pot 33",         RHT_POT_MARKET,                   RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_33));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_34]          =    Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_34,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(170, 120),     "Guard House Child Pot 34",         RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_34));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_35]          =    Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_35,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(177, 85),      "Guard House Child Pot 35",         RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_35));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_36]          =    Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_36,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(155, 39),      "Guard House Child Pot 36",         RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_36));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_37]          =    Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_37,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(184, 13),      "Guard House Child Pot 37",         RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_37));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_38]          =    Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_38,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(156, -1),      "Guard House Child Pot 38",         RHT_POT_MARKET,                   RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_38));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_39]          =    Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_39,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(181, -33),     "Guard House Child Pot 39",         RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_39));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_40]          =    Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_40,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(156, -45),     "Guard House Child Pot 40",         RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_40));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_41]          =    Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_41,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(172, -82),     "Guard House Child Pot 41",         RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_41));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_42]          =    Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_42,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(170, -120),    "Guard House Child Pot 42",         RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_42));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_43]          =    Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_43,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(170, -166),    "Guard House Child Pot 43",         RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_43));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_44]          =    Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_44,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(170, -216),    "Guard House Child Pot 44",         RHT_POT_MARKET,                   RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_44));
    locationTable[RC_MK_GUARD_HOUSE_ADULT_POT_1]           =    Location::Pot(RC_MK_GUARD_HOUSE_ADULT_POT_1,               RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(61, 204),      "Guard House Adult Pot 1",          RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_ADULT_POT_1));
    locationTable[RC_MK_GUARD_HOUSE_ADULT_POT_2]           =    Location::Pot(RC_MK_GUARD_HOUSE_ADULT_POT_2,               RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(71, 132),      "Guard House Adult Pot 2",          RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_ADULT_POT_2));
    locationTable[RC_MK_GUARD_HOUSE_ADULT_POT_3]           =    Location::Pot(RC_MK_GUARD_HOUSE_ADULT_POT_3,               RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(74, 23),       "Guard House Adult Pot 3",          RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_ADULT_POT_3));
    locationTable[RC_MK_GUARD_HOUSE_ADULT_POT_4]           =    Location::Pot(RC_MK_GUARD_HOUSE_ADULT_POT_4,               RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(40, 4),        "Guard House Adult Pot 4",          RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_ADULT_POT_4));
    locationTable[RC_MK_GUARD_HOUSE_ADULT_POT_5]           =    Location::Pot(RC_MK_GUARD_HOUSE_ADULT_POT_5,               RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(71, -22),      "Guard House Adult Pot 5",          RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_ADULT_POT_5));
    locationTable[RC_MK_GUARD_HOUSE_ADULT_POT_6]           =    Location::Pot(RC_MK_GUARD_HOUSE_ADULT_POT_6,               RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(44, -151),     "Guard House Adult Pot 6",          RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_ADULT_POT_6));
    locationTable[RC_MK_GUARD_HOUSE_ADULT_POT_7]           =    Location::Pot(RC_MK_GUARD_HOUSE_ADULT_POT_7,               RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(79, -182),     "Guard House Adult Pot 7",          RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_ADULT_POT_7));
    locationTable[RC_MK_GUARD_HOUSE_ADULT_POT_8]           =    Location::Pot(RC_MK_GUARD_HOUSE_ADULT_POT_8,               RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(44, -198),     "Guard House Adult Pot 8",          RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_ADULT_POT_8));
    locationTable[RC_MK_GUARD_HOUSE_ADULT_POT_9]           =    Location::Pot(RC_MK_GUARD_HOUSE_ADULT_POT_9,               RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(168, 210),     "Guard House Adult Pot 9",          RHT_POT_MARKET,                   RG_RED_RUPEE,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_ADULT_POT_9));
    locationTable[RC_MK_GUARD_HOUSE_ADULT_POT_10]          =    Location::Pot(RC_MK_GUARD_HOUSE_ADULT_POT_10,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(167, -122),    "Guard House Adult Pot 10",         RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_ADULT_POT_10));
    locationTable[RC_MK_GUARD_HOUSE_ADULT_POT_11]          =    Location::Pot(RC_MK_GUARD_HOUSE_ADULT_POT_11,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(167, -210),    "Guard House Adult Pot 11",         RHT_POT_MARKET,                   RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_ADULT_POT_11));
    locationTable[RC_MK_BACK_ALLEY_HOUSE_POT_1]            =    Location::Pot(RC_MK_BACK_ALLEY_HOUSE_POT_1,                RCQUEST_BOTH, RCAREA_MARKET,                SCENE_BACK_ALLEY_HOUSE,            TWO_ACTOR_PARAMS(100, 45),      "Back Alley House Pot 1",           RHT_POT_MARKET,                   RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_BACK_ALLEY_HOUSE_POT_1));
    locationTable[RC_MK_BACK_ALLEY_HOUSE_POT_2]            =    Location::Pot(RC_MK_BACK_ALLEY_HOUSE_POT_2,                RCQUEST_BOTH, RCAREA_MARKET,                SCENE_BACK_ALLEY_HOUSE,            TWO_ACTOR_PARAMS(12, -180),     "Back Alley House Pot 2",           RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_BACK_ALLEY_HOUSE_POT_2));
    locationTable[RC_MK_BACK_ALLEY_HOUSE_POT_3]            =    Location::Pot(RC_MK_BACK_ALLEY_HOUSE_POT_3,                RCQUEST_BOTH, RCAREA_MARKET,                SCENE_BACK_ALLEY_HOUSE,            TWO_ACTOR_PARAMS(-54, -180),    "Back Alley House Pot 3",           RHT_POT_MARKET,                   RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_BACK_ALLEY_HOUSE_POT_3));
    locationTable[RC_KAK_NEAR_POTION_SHOP_POT_1]           =    Location::Pot(RC_KAK_NEAR_POTION_SHOP_POT_1,               RCQUEST_BOTH, RCAREA_KAKARIKO_VILLAGE,      SCENE_KAKARIKO_VILLAGE,            TWO_ACTOR_PARAMS(222, -377),    "Near Potion Shop Pot 1",           RHT_POT_KAKARIKO,                 RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_KAK_NEAR_POTION_SHOP_POT_1));
    locationTable[RC_KAK_NEAR_POTION_SHOP_POT_2]           =    Location::Pot(RC_KAK_NEAR_POTION_SHOP_POT_2,               RCQUEST_BOTH, RCAREA_KAKARIKO_VILLAGE,      SCENE_KAKARIKO_VILLAGE,            TWO_ACTOR_PARAMS(255, -366),    "Near Potion Shop Pot 2",           RHT_POT_KAKARIKO,                 RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_KAK_NEAR_POTION_SHOP_POT_2));
    locationTable[RC_KAK_NEAR_POTION_SHOP_POT_3]           =    Location::Pot(RC_KAK_NEAR_POTION_SHOP_POT_3,               RCQUEST_BOTH, RCAREA_KAKARIKO_VILLAGE,      SCENE_KAKARIKO_VILLAGE,            TWO_ACTOR_PARAMS(284, -356),    "Near Potion Shop Pot 3",           RHT_POT_KAKARIKO,                 RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_KAK_NEAR_POTION_SHOP_POT_3));
    locationTable[RC_KAK_NEAR_IMPAS_HOUSE_POT_1]           =    Location::Pot(RC_KAK_NEAR_IMPAS_HOUSE_POT_1,               RCQUEST_BOTH, RCAREA_KAKARIKO_VILLAGE,      SCENE_KAKARIKO_VILLAGE,            TWO_ACTOR_PARAMS(-384, 1568),   "Near Impas House Pot 1",           RHT_POT_KAKARIKO,                 RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_KAK_NEAR_IMPAS_HOUSE_POT_1));
    locationTable[RC_KAK_NEAR_IMPAS_HOUSE_POT_2]           =    Location::Pot(RC_KAK_NEAR_IMPAS_HOUSE_POT_2,               RCQUEST_BOTH, RCAREA_KAKARIKO_VILLAGE,      SCENE_KAKARIKO_VILLAGE,            TWO_ACTOR_PARAMS(-384, 1539),   "Near Impas House Pot 2",           RHT_POT_KAKARIKO,                 RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_KAK_NEAR_IMPAS_HOUSE_POT_2));
    locationTable[RC_KAK_NEAR_IMPAS_HOUSE_POT_3]           =    Location::Pot(RC_KAK_NEAR_IMPAS_HOUSE_POT_3,               RCQUEST_BOTH, RCAREA_KAKARIKO_VILLAGE,      SCENE_KAKARIKO_VILLAGE,            TWO_ACTOR_PARAMS(-384, 1510),   "Near Impas House Pot 3",           RHT_POT_KAKARIKO,                 RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_KAK_NEAR_IMPAS_HOUSE_POT_3));
    locationTable[RC_KAK_NEAR_GUARDS_HOUSE_POT_1]          =    Location::Pot(RC_KAK_NEAR_GUARDS_HOUSE_POT_1,              RCQUEST_BOTH, RCAREA_KAKARIKO_VILLAGE,      SCENE_KAKARIKO_VILLAGE,            TWO_ACTOR_PARAMS(-392, -870),   "Near Guards House Pot 1",          RHT_POT_KAKARIKO,                 RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_KAK_NEAR_GUARDS_HOUSE_POT_1));
    locationTable[RC_KAK_NEAR_GUARDS_HOUSE_POT_2]          =    Location::Pot(RC_KAK_NEAR_GUARDS_HOUSE_POT_2,              RCQUEST_BOTH, RCAREA_KAKARIKO_VILLAGE,      SCENE_KAKARIKO_VILLAGE,            TWO_ACTOR_PARAMS(-422, -883),   "Near Guards House Pot 2",          RHT_POT_KAKARIKO,                 RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_KAK_NEAR_GUARDS_HOUSE_POT_2));
    locationTable[RC_KAK_NEAR_GUARDS_HOUSE_POT_3]          =    Location::Pot(RC_KAK_NEAR_GUARDS_HOUSE_POT_3,              RCQUEST_BOTH, RCAREA_KAKARIKO_VILLAGE,      SCENE_KAKARIKO_VILLAGE,            TWO_ACTOR_PARAMS(-450, -895),   "Near Guards House Pot 3",          RHT_POT_KAKARIKO,                 RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_KAK_NEAR_GUARDS_HOUSE_POT_3));
    locationTable[RC_KAK_NEAR_MEDICINE_SHOP_POT_1]         =    Location::Pot(RC_KAK_NEAR_MEDICINE_SHOP_POT_1,             RCQUEST_BOTH, RCAREA_KAKARIKO_VILLAGE,      SCENE_KAKARIKO_VILLAGE,            TWO_ACTOR_PARAMS(781, 89),      "Near Medicine Shop Pot 1",         RHT_POT_KAKARIKO,                 RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_KAK_NEAR_MEDICINE_SHOP_POT_1));
    locationTable[RC_KAK_NEAR_MEDICINE_SHOP_POT_2]         =    Location::Pot(RC_KAK_NEAR_MEDICINE_SHOP_POT_2,             RCQUEST_BOTH, RCAREA_KAKARIKO_VILLAGE,      SCENE_KAKARIKO_VILLAGE,            TWO_ACTOR_PARAMS(815, 89),      "Near Medicine Shop Pot 2",         RHT_POT_KAKARIKO,                 RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_KAK_NEAR_MEDICINE_SHOP_POT_2));
    locationTable[RC_GY_DAMPES_GRAVE_POT_1]                =    Location::Pot(RC_GY_DAMPES_GRAVE_POT_1,                    RCQUEST_BOTH, RCAREA_GRAVEYARD,             SCENE_WINDMILL_AND_DAMPES_GRAVE,   TWO_ACTOR_PARAMS(-319, -1542),  "Dampes Grave Pot 1",               RHT_POT_GRAVEYARD,                RG_ARROWS_5,        SpoilerCollectionCheck::RandomizerInf(RAND_INF_GY_DAMPES_GRAVE_POT_1));
    locationTable[RC_GY_DAMPES_GRAVE_POT_2]                =    Location::Pot(RC_GY_DAMPES_GRAVE_POT_2,                    RCQUEST_BOTH, RCAREA_GRAVEYARD,             SCENE_WINDMILL_AND_DAMPES_GRAVE,   TWO_ACTOR_PARAMS(-319, -1600),  "Dampes Grave Pot 2",               RHT_POT_GRAVEYARD,                RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_GY_DAMPES_GRAVE_POT_2));
    locationTable[RC_GY_DAMPES_GRAVE_POT_3]                =    Location::Pot(RC_GY_DAMPES_GRAVE_POT_3,                    RCQUEST_BOTH, RCAREA_GRAVEYARD,             SCENE_WINDMILL_AND_DAMPES_GRAVE,   TWO_ACTOR_PARAMS(-364, -1571),  "Dampes Grave Pot 3",               RHT_POT_GRAVEYARD,                RG_RED_RUPEE,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_GY_DAMPES_GRAVE_POT_3));
    locationTable[RC_GY_DAMPES_GRAVE_POT_4]                =    Location::Pot(RC_GY_DAMPES_GRAVE_POT_4,                    RCQUEST_BOTH, RCAREA_GRAVEYARD,             SCENE_WINDMILL_AND_DAMPES_GRAVE,   TWO_ACTOR_PARAMS(198, -1540),   "Dampes Grave Pot 4",               RHT_POT_GRAVEYARD,                RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_GY_DAMPES_GRAVE_POT_4));
    locationTable[RC_GY_DAMPES_GRAVE_POT_5]                =    Location::Pot(RC_GY_DAMPES_GRAVE_POT_5,                    RCQUEST_BOTH, RCAREA_GRAVEYARD,             SCENE_WINDMILL_AND_DAMPES_GRAVE,   TWO_ACTOR_PARAMS(198, -1608),   "Dampes Grave Pot 5",               RHT_POT_GRAVEYARD,                RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GY_DAMPES_GRAVE_POT_5));
    locationTable[RC_GY_DAMPES_GRAVE_POT_6]                =    Location::Pot(RC_GY_DAMPES_GRAVE_POT_6,                    RCQUEST_BOTH, RCAREA_GRAVEYARD,             SCENE_WINDMILL_AND_DAMPES_GRAVE,   TWO_ACTOR_PARAMS(239, -1577),   "Dampes Grave Pot 6",               RHT_POT_GRAVEYARD,                RG_DEKU_NUTS_5,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_GY_DAMPES_GRAVE_POT_6));
    locationTable[RC_GC_LOWER_STAIRCASE_POT_1]             =    Location::Pot(RC_GC_LOWER_STAIRCASE_POT_1,                 RCQUEST_BOTH, RCAREA_GORON_CITY,            SCENE_GORON_CITY,                  TWO_ACTOR_PARAMS(-189, 866),    "Lower Staircase Pot 1",            RHT_POT_GORON_CITY,               RG_DEKU_STICK_1,    SpoilerCollectionCheck::RandomizerInf(RAND_INF_GC_LOWER_STAIRCASE_POT_1));
    locationTable[RC_GC_LOWER_STAIRCASE_POT_2]             =    Location::Pot(RC_GC_LOWER_STAIRCASE_POT_2,                 RCQUEST_BOTH, RCAREA_GORON_CITY,            SCENE_GORON_CITY,                  TWO_ACTOR_PARAMS(-271, 825),    "Lower Staircase Pot 2",            RHT_POT_GORON_CITY,               RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GC_LOWER_STAIRCASE_POT_2));
    locationTable[RC_GC_UPPER_STAIRCASE_POT_1]             =    Location::Pot(RC_GC_UPPER_STAIRCASE_POT_1,                 RCQUEST_BOTH, RCAREA_GORON_CITY,            SCENE_GORON_CITY,                  TWO_ACTOR_PARAMS(-1170, 60),    "Upper Staircase Pot 1",            RHT_POT_GORON_CITY,               RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_GC_UPPER_STAIRCASE_POT_1));
    locationTable[RC_GC_UPPER_STAIRCASE_POT_2]             =    Location::Pot(RC_GC_UPPER_STAIRCASE_POT_2,                 RCQUEST_BOTH, RCAREA_GORON_CITY,            SCENE_GORON_CITY,                  TWO_ACTOR_PARAMS(-1185, 95),    "Upper Staircase Pot 2",            RHT_POT_GORON_CITY,               RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_GC_UPPER_STAIRCASE_POT_2));
    locationTable[RC_GC_UPPER_STAIRCASE_POT_3]             =    Location::Pot(RC_GC_UPPER_STAIRCASE_POT_3,                 RCQUEST_BOTH, RCAREA_GORON_CITY,            SCENE_GORON_CITY,                  TWO_ACTOR_PARAMS(-1200, 63),    "Upper Staircase Pot 3",            RHT_POT_GORON_CITY,               RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_GC_UPPER_STAIRCASE_POT_3));
    locationTable[RC_GC_MEDIGORON_POT_1]                   =    Location::Pot(RC_GC_MEDIGORON_POT_1,                       RCQUEST_BOTH, RCAREA_GORON_CITY,            SCENE_GORON_CITY,                  TWO_ACTOR_PARAMS(-694, 1196),   "Medigoron Pot 1",                  RHT_POT_GORON_CITY,               RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_GC_MEDIGORON_POT_1));
    locationTable[RC_GC_DARUNIA_POT_1]                     =    Location::Pot(RC_GC_DARUNIA_POT_1,                         RCQUEST_BOTH, RCAREA_GORON_CITY,            SCENE_GORON_CITY,                  TWO_ACTOR_PARAMS(262, -1210),   "Darunia Pot 1",                    RHT_POT_GORON_CITY,               RG_ARROWS_5,        SpoilerCollectionCheck::RandomizerInf(RAND_INF_GC_DARUNIA_POT_1));
    locationTable[RC_GC_DARUNIA_POT_2]                     =    Location::Pot(RC_GC_DARUNIA_POT_2,                         RCQUEST_BOTH, RCAREA_GORON_CITY,            SCENE_GORON_CITY,                  TWO_ACTOR_PARAMS(261, -1254),   "Darunia Pot 2",                    RHT_POT_GORON_CITY,               RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GC_DARUNIA_POT_2));
    locationTable[RC_GC_DARUNIA_POT_3]                     =    Location::Pot(RC_GC_DARUNIA_POT_3,                         RCQUEST_BOTH, RCAREA_GORON_CITY,            SCENE_GORON_CITY,                  TWO_ACTOR_PARAMS(262, -1386),   "Darunia Pot 3",                    RHT_POT_GORON_CITY,               RG_RED_RUPEE,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_GC_DARUNIA_POT_3));
    locationTable[RC_DMC_NEAR_GC_POT_2]                    =    Location::Pot(RC_DMC_NEAR_GC_POT_2,                        RCQUEST_BOTH, RCAREA_DEATH_MOUNTAIN_CRATER, SCENE_DEATH_MOUNTAIN_CRATER,       TWO_ACTOR_PARAMS(-1546, 141),   "Near GC Pot 2",                    RHT_POT_DEATH_MOUNTAIN_CRATER,    RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_DMC_NEAR_GC_POT_2));
    locationTable[RC_DMC_NEAR_GC_POT_3]                    =    Location::Pot(RC_DMC_NEAR_GC_POT_3,                        RCQUEST_BOTH, RCAREA_DEATH_MOUNTAIN_CRATER, SCENE_DEATH_MOUNTAIN_CRATER,       TWO_ACTOR_PARAMS(-1641, -127),  "Near GC Pot 3",                    RHT_POT_DEATH_MOUNTAIN_CRATER,    RG_ARROWS_5,        SpoilerCollectionCheck::RandomizerInf(RAND_INF_DMC_NEAR_GC_POT_3));
    locationTable[RC_DMC_NEAR_GC_POT_1]                    =    Location::Pot(RC_DMC_NEAR_GC_POT_1,                        RCQUEST_BOTH, RCAREA_DEATH_MOUNTAIN_CRATER, SCENE_DEATH_MOUNTAIN_CRATER,       TWO_ACTOR_PARAMS(-1590, 132),   "Near GC Pot 1",                    RHT_POT_DEATH_MOUNTAIN_CRATER,    RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_DMC_NEAR_GC_POT_1));
    locationTable[RC_DMC_NEAR_GC_POT_4]                    =    Location::Pot(RC_DMC_NEAR_GC_POT_4,                        RCQUEST_BOTH, RCAREA_DEATH_MOUNTAIN_CRATER, SCENE_DEATH_MOUNTAIN_CRATER,       TWO_ACTOR_PARAMS(-1606, -166),  "Near GC Pot 4",                    RHT_POT_DEATH_MOUNTAIN_CRATER,    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_DMC_NEAR_GC_POT_4));
    locationTable[RC_ZD_NEAR_SHOP_POT_1]                   =    Location::Pot(RC_ZD_NEAR_SHOP_POT_1,                       RCQUEST_BOTH, RCAREA_ZORAS_DOMAIN,          SCENE_ZORAS_DOMAIN,                TWO_ACTOR_PARAMS(676, 377),     "Near Shop Pot 1",                  RHT_POT_ZORAS_DOMAIN,             RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_ZD_NEAR_SHOP_POT_1));
    locationTable[RC_ZD_NEAR_SHOP_POT_2]                   =    Location::Pot(RC_ZD_NEAR_SHOP_POT_2,                       RCQUEST_BOTH, RCAREA_ZORAS_DOMAIN,          SCENE_ZORAS_DOMAIN,                TWO_ACTOR_PARAMS(614, 419),     "Near Shop Pot 2",                  RHT_POT_ZORAS_DOMAIN,             RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_ZD_NEAR_SHOP_POT_2));
    locationTable[RC_ZD_NEAR_SHOP_POT_3]                   =    Location::Pot(RC_ZD_NEAR_SHOP_POT_3,                       RCQUEST_BOTH, RCAREA_ZORAS_DOMAIN,          SCENE_ZORAS_DOMAIN,                TWO_ACTOR_PARAMS(289, 415),     "Near Shop Pot 3",                  RHT_POT_ZORAS_DOMAIN,             RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_ZD_NEAR_SHOP_POT_3));
    locationTable[RC_ZD_NEAR_SHOP_POT_4]                   =    Location::Pot(RC_ZD_NEAR_SHOP_POT_4,                       RCQUEST_BOTH, RCAREA_ZORAS_DOMAIN,          SCENE_ZORAS_DOMAIN,                TWO_ACTOR_PARAMS(289, 289),     "Near Shop Pot 4",                  RHT_POT_ZORAS_DOMAIN,             RG_DEKU_NUTS_5,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_ZD_NEAR_SHOP_POT_4));
    locationTable[RC_ZD_NEAR_SHOP_POT_5]                   =    Location::Pot(RC_ZD_NEAR_SHOP_POT_5,                       RCQUEST_BOTH, RCAREA_ZORAS_DOMAIN,          SCENE_ZORAS_DOMAIN,                TWO_ACTOR_PARAMS(220, 384),     "Near Shop Pot 5",                  RHT_POT_ZORAS_DOMAIN,             RG_DEKU_STICK_1,    SpoilerCollectionCheck::RandomizerInf(RAND_INF_ZD_NEAR_SHOP_POT_5));
    locationTable[RC_ZF_HIDDEN_CAVE_POT_1]                 =    Location::Pot(RC_ZF_HIDDEN_CAVE_POT_1,                     RCQUEST_BOTH, RCAREA_ZORAS_FOUNTAIN,        SCENE_ZORAS_FOUNTAIN,              TWO_ACTOR_PARAMS(385, 2587),    "Hidden Cave Pot 1",                RHT_POT_ZORAS_FOUNTAIN,           RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_ZF_HIDDEN_CAVE_POT_1));
    locationTable[RC_ZF_HIDDEN_CAVE_POT_2]                 =    Location::Pot(RC_ZF_HIDDEN_CAVE_POT_2,                     RCQUEST_BOTH, RCAREA_ZORAS_FOUNTAIN,        SCENE_ZORAS_FOUNTAIN,              TWO_ACTOR_PARAMS(418, 2609),    "Hidden Cave Pot 2",                RHT_POT_ZORAS_FOUNTAIN,           RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_ZF_HIDDEN_CAVE_POT_2));
    locationTable[RC_ZF_HIDDEN_CAVE_POT_3]                 =    Location::Pot(RC_ZF_HIDDEN_CAVE_POT_3,                     RCQUEST_BOTH, RCAREA_ZORAS_FOUNTAIN,        SCENE_ZORAS_FOUNTAIN,              TWO_ACTOR_PARAMS(430, 2568),    "Hidden Cave Pot 3",                RHT_POT_ZORAS_FOUNTAIN,           RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_ZF_HIDDEN_CAVE_POT_3));
    locationTable[RC_ZF_NEAR_JABU_POT_1]                   =    Location::Pot(RC_ZF_NEAR_JABU_POT_1,                       RCQUEST_BOTH, RCAREA_ZORAS_FOUNTAIN,        SCENE_ZORAS_FOUNTAIN,              TWO_ACTOR_PARAMS(-1630, 170),   "Near Jabu Pot 1",                  RHT_POT_ZORAS_FOUNTAIN,           RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_ZF_NEAR_JABU_POT_1));
    locationTable[RC_ZF_NEAR_JABU_POT_2]                   =    Location::Pot(RC_ZF_NEAR_JABU_POT_2,                       RCQUEST_BOTH, RCAREA_ZORAS_FOUNTAIN,        SCENE_ZORAS_FOUNTAIN,              TWO_ACTOR_PARAMS(-1550, 170),   "Near Jabu Pot 2",                  RHT_POT_ZORAS_FOUNTAIN,           RG_DEKU_SEEDS_30,   SpoilerCollectionCheck::RandomizerInf(RAND_INF_ZF_NEAR_JABU_POT_2));
    locationTable[RC_ZF_NEAR_JABU_POT_3]                   =    Location::Pot(RC_ZF_NEAR_JABU_POT_3,                       RCQUEST_BOTH, RCAREA_ZORAS_FOUNTAIN,        SCENE_ZORAS_FOUNTAIN,              TWO_ACTOR_PARAMS(-1630, -260),  "Near Jabu Pot 3",                  RHT_POT_ZORAS_FOUNTAIN,           RG_DEKU_SEEDS_30,   SpoilerCollectionCheck::RandomizerInf(RAND_INF_ZF_NEAR_JABU_POT_3));
    locationTable[RC_ZF_NEAR_JABU_POT_4]                   =    Location::Pot(RC_ZF_NEAR_JABU_POT_4,                       RCQUEST_BOTH, RCAREA_ZORAS_FOUNTAIN,        SCENE_ZORAS_FOUNTAIN,              TWO_ACTOR_PARAMS(-1550, -260),  "Near Jabu Pot 4",                  RHT_POT_ZORAS_FOUNTAIN,           RG_DEKU_SEEDS_30,   SpoilerCollectionCheck::RandomizerInf(RAND_INF_ZF_NEAR_JABU_POT_4));
    locationTable[RC_LLR_FRONT_POT_1]                      =    Location::Pot(RC_LLR_FRONT_POT_1,                          RCQUEST_BOTH, RCAREA_LON_LON_RANCH,         SCENE_LON_LON_RANCH,               TWO_ACTOR_PARAMS(667, -3218),   "Front Pot 1",                      RHT_POT_LON_LON_RANCH,            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_LLR_FRONT_POT_1));
    locationTable[RC_LLR_FRONT_POT_2]                      =    Location::Pot(RC_LLR_FRONT_POT_2,                          RCQUEST_BOTH, RCAREA_LON_LON_RANCH,         SCENE_LON_LON_RANCH,               TWO_ACTOR_PARAMS(667, -3186),   "Front Pot 2",                      RHT_POT_LON_LON_RANCH,            RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_LLR_FRONT_POT_2));
    locationTable[RC_LLR_FRONT_POT_3]                      =    Location::Pot(RC_LLR_FRONT_POT_3,                          RCQUEST_BOTH, RCAREA_LON_LON_RANCH,         SCENE_LON_LON_RANCH,               TWO_ACTOR_PARAMS(667, -3154),   "Front Pot 3",                      RHT_POT_LON_LON_RANCH,            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_LLR_FRONT_POT_3));
    locationTable[RC_LLR_FRONT_POT_4]                      =    Location::Pot(RC_LLR_FRONT_POT_4,                          RCQUEST_BOTH, RCAREA_LON_LON_RANCH,         SCENE_LON_LON_RANCH,               TWO_ACTOR_PARAMS(667, -3122),   "Front Pot 4",                      RHT_POT_LON_LON_RANCH,            RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_LLR_FRONT_POT_4));
    locationTable[RC_LLR_RAIN_SHED_POT_1]                  =    Location::Pot(RC_LLR_RAIN_SHED_POT_1,                      RCQUEST_BOTH, RCAREA_LON_LON_RANCH,         SCENE_LON_LON_RANCH,               TWO_ACTOR_PARAMS(852, 172),     "Rain Shed Pot 1",                  RHT_POT_LON_LON_RANCH,            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_LLR_RAIN_SHED_POT_1));
    locationTable[RC_LLR_RAIN_SHED_POT_2]                  =    Location::Pot(RC_LLR_RAIN_SHED_POT_2,                      RCQUEST_BOTH, RCAREA_LON_LON_RANCH,         SCENE_LON_LON_RANCH,               TWO_ACTOR_PARAMS(840, 212),     "Rain Shed Pot 2",                  RHT_POT_LON_LON_RANCH,            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_LLR_RAIN_SHED_POT_2));
    locationTable[RC_LLR_RAIN_SHED_POT_3]                  =    Location::Pot(RC_LLR_RAIN_SHED_POT_3,                      RCQUEST_BOTH, RCAREA_LON_LON_RANCH,         SCENE_LON_LON_RANCH,               TWO_ACTOR_PARAMS(872, 219),     "Rain Shed Pot 3",                  RHT_POT_LON_LON_RANCH,            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_LLR_RAIN_SHED_POT_3));
    locationTable[RC_LLR_TALONS_HOUSE_POT_1]               =    Location::Pot(RC_LLR_TALONS_HOUSE_POT_1,                   RCQUEST_BOTH, RCAREA_LON_LON_RANCH,         SCENE_LON_LON_BUILDINGS,           TWO_ACTOR_PARAMS(1255, 47),     "Talons House Pot 1",               RHT_POT_LON_LON_RANCH,            RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_LLR_TALONS_HOUSE_POT_1));
    locationTable[RC_LLR_TALONS_HOUSE_POT_2]               =    Location::Pot(RC_LLR_TALONS_HOUSE_POT_2,                   RCQUEST_BOTH, RCAREA_LON_LON_RANCH,         SCENE_LON_LON_BUILDINGS,           TWO_ACTOR_PARAMS(1256, -51),    "Talons House Pot 2",               RHT_POT_LON_LON_RANCH,            RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_LLR_TALONS_HOUSE_POT_2));
    locationTable[RC_LLR_TALONS_HOUSE_POT_3]               =    Location::Pot(RC_LLR_TALONS_HOUSE_POT_3,                   RCQUEST_BOTH, RCAREA_LON_LON_RANCH,         SCENE_LON_LON_BUILDINGS,           TWO_ACTOR_PARAMS(1256, -78),    "Talons House Pot 3",               RHT_POT_LON_LON_RANCH,            RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_LLR_TALONS_HOUSE_POT_3));
    locationTable[RC_HF_COW_GROTTO_POT_1]                  =    Location::Pot(RC_HF_COW_GROTTO_POT_1,                      RCQUEST_BOTH, RCAREA_HYRULE_FIELD,          SCENE_GROTTOS,                     TWO_ACTOR_PARAMS(3410, -223),   "Cow Grotto Pot 1",                 RHT_POT_HYRULE_FIELD,             RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_HF_COW_GROTTO_POT_1));
    locationTable[RC_HF_COW_GROTTO_POT_2]                  =    Location::Pot(RC_HF_COW_GROTTO_POT_2,                      RCQUEST_BOTH, RCAREA_HYRULE_FIELD,          SCENE_GROTTOS,                     TWO_ACTOR_PARAMS(3390, -258),   "Cow Grotto Pot 2",                 RHT_POT_HYRULE_FIELD,             RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_HF_COW_GROTTO_POT_2));
    locationTable[RC_HC_STORMS_GROTTO_POT_1]               =    Location::Pot(RC_HC_STORMS_GROTTO_POT_1,                   RCQUEST_BOTH, RCAREA_HYRULE_CASTLE,         SCENE_GROTTOS,                     TWO_ACTOR_PARAMS(1843, 1014),   "Storms Grotto Pot 1",              RHT_POT_HYRULE_CASTLE,            RG_RED_RUPEE,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_HC_STORMS_GROTTO_POT_1));
    locationTable[RC_HC_STORMS_GROTTO_POT_2]               =    Location::Pot(RC_HC_STORMS_GROTTO_POT_2,                   RCQUEST_BOTH, RCAREA_HYRULE_CASTLE,         SCENE_GROTTOS,                     TWO_ACTOR_PARAMS(1769, 954),    "Storms Grotto Pot 2",              RHT_POT_HYRULE_CASTLE,            RG_ARROWS_5,        SpoilerCollectionCheck::RandomizerInf(RAND_INF_HC_STORMS_GROTTO_POT_2));
    locationTable[RC_HC_STORMS_GROTTO_POT_3]               =    Location::Pot(RC_HC_STORMS_GROTTO_POT_3,                   RCQUEST_BOTH, RCAREA_HYRULE_CASTLE,         SCENE_GROTTOS,                     TWO_ACTOR_PARAMS(1857, 897),    "Storms Grotto Pot 3",              RHT_POT_HYRULE_CASTLE,            RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_HC_STORMS_GROTTO_POT_3));
    locationTable[RC_HC_STORMS_GROTTO_POT_4]               =    Location::Pot(RC_HC_STORMS_GROTTO_POT_4,                   RCQUEST_BOTH, RCAREA_HYRULE_CASTLE,         SCENE_GROTTOS,                     TWO_ACTOR_PARAMS(1764, 847),    "Storms Grotto Pot 4",              RHT_POT_HYRULE_CASTLE,            RG_DEKU_NUTS_5,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_HC_STORMS_GROTTO_POT_4));

    // Dungeon Pots
    //            Randomizer Check                                  Randomizer Check                                           Quest            Area                       Scene ID                           Params                          Short Name                          Hint Text Key                   Vanilla             Spoiler Collection Check
    locationTable[RC_DODONGOS_CAVERN_LIZALFOS_POT_3]           =    Location::Pot(RC_DODONGOS_CAVERN_LIZALFOS_POT_3,           RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(2673, -2060),  "Lizalfos Pot 3",                   RHT_POT_DODONGOS_CAVERN,        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_LIZALFOS_POT_3));
    locationTable[RC_DODONGOS_CAVERN_LIZALFOS_POT_2]           =    Location::Pot(RC_DODONGOS_CAVERN_LIZALFOS_POT_2,           RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(3526, -2574),  "Lizalfos Pot 2",                   RHT_POT_DODONGOS_CAVERN,        RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_LIZALFOS_POT_2));
    locationTable[RC_DODONGOS_CAVERN_LIZALFOS_POT_1]           =    Location::Pot(RC_DODONGOS_CAVERN_LIZALFOS_POT_1,           RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(3579, -2574),  "Lizalfos Pot 1",                   RHT_POT_DODONGOS_CAVERN,        RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_LIZALFOS_POT_1));
    locationTable[RC_DODONGOS_CAVERN_LIZALFOS_POT_4]           =    Location::Pot(RC_DODONGOS_CAVERN_LIZALFOS_POT_4,           RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(2653, -2031),  "Lizalfos Pot 4",                   RHT_POT_DODONGOS_CAVERN,        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_LIZALFOS_POT_4));
    locationTable[RC_DODONGOS_CAVERN_SIDE_ROOM_POT_1]          =    Location::Pot(RC_DODONGOS_CAVERN_SIDE_ROOM_POT_1,          RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(1708, -471),   "Side Room Pot 1",                  RHT_POT_DODONGOS_CAVERN,        RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_SIDE_ROOM_POT_1));
    locationTable[RC_DODONGOS_CAVERN_SIDE_ROOM_POT_2]          =    Location::Pot(RC_DODONGOS_CAVERN_SIDE_ROOM_POT_2,          RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(1746, -424),   "Side Room Pot 2",                  RHT_POT_DODONGOS_CAVERN,        RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_SIDE_ROOM_POT_2));
    locationTable[RC_DODONGOS_CAVERN_SIDE_ROOM_POT_3]          =    Location::Pot(RC_DODONGOS_CAVERN_SIDE_ROOM_POT_3,          RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(2573, -489),   "Side Room Pot 3",                  RHT_POT_DODONGOS_CAVERN,        RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_SIDE_ROOM_POT_3));
    locationTable[RC_DODONGOS_CAVERN_SIDE_ROOM_POT_4]          =    Location::Pot(RC_DODONGOS_CAVERN_SIDE_ROOM_POT_4,          RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(2622, -489),   "Side Room Pot 4",                  RHT_POT_DODONGOS_CAVERN,        RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_SIDE_ROOM_POT_4));
    locationTable[RC_DODONGOS_CAVERN_SIDE_ROOM_POT_5]          =    Location::Pot(RC_DODONGOS_CAVERN_SIDE_ROOM_POT_5,          RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(3507, -964),   "Side Room Pot 5",                  RHT_POT_DODONGOS_CAVERN,        RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_SIDE_ROOM_POT_5));
    locationTable[RC_DODONGOS_CAVERN_SIDE_ROOM_POT_6]          =    Location::Pot(RC_DODONGOS_CAVERN_SIDE_ROOM_POT_6,          RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(3617, -911),   "Side Room Pot 6",                  RHT_POT_DODONGOS_CAVERN,        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_SIDE_ROOM_POT_6));
    locationTable[RC_DODONGOS_CAVERN_TORCH_ROOM_POT_1]         =    Location::Pot(RC_DODONGOS_CAVERN_TORCH_ROOM_POT_1,         RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(1846, -1879),  "Torch Room Pot 1",                 RHT_POT_DODONGOS_CAVERN,        RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_TORCH_ROOM_POT_1));
    locationTable[RC_DODONGOS_CAVERN_TORCH_ROOM_POT_2]         =    Location::Pot(RC_DODONGOS_CAVERN_TORCH_ROOM_POT_2,         RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(2050, -1964),  "Torch Room Pot 2",                 RHT_POT_DODONGOS_CAVERN,        RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_TORCH_ROOM_POT_2));
    locationTable[RC_DODONGOS_CAVERN_TORCH_ROOM_POT_3]         =    Location::Pot(RC_DODONGOS_CAVERN_TORCH_ROOM_POT_3,         RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(2281, -1799),  "Torch Room Pot 3",                 RHT_POT_DODONGOS_CAVERN,        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_TORCH_ROOM_POT_3));
    locationTable[RC_DODONGOS_CAVERN_TORCH_ROOM_POT_4]         =    Location::Pot(RC_DODONGOS_CAVERN_TORCH_ROOM_POT_4,         RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(1836, -1239),  "Torch Room Pot 4",                 RHT_POT_DODONGOS_CAVERN,        RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_TORCH_ROOM_POT_4));
    locationTable[RC_DODONGOS_CAVERN_STAIRCASE_POT_1]          =    Location::Pot(RC_DODONGOS_CAVERN_STAIRCASE_POT_1,          RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(-2414, -1823), "Staircase Pot 1",                  RHT_POT_DODONGOS_CAVERN,        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_STAIRCASE_POT_1));
    locationTable[RC_DODONGOS_CAVERN_STAIRCASE_POT_2]          =    Location::Pot(RC_DODONGOS_CAVERN_STAIRCASE_POT_2,          RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(-2368, -1827), "Staircase Pot 2",                  RHT_POT_DODONGOS_CAVERN,        RG_RED_RUPEE,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_STAIRCASE_POT_2));
    locationTable[RC_DODONGOS_CAVERN_STAIRCASE_POT_3]          =    Location::Pot(RC_DODONGOS_CAVERN_STAIRCASE_POT_3,          RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(-1574, -1831), "Staircase Pot 3",                  RHT_POT_DODONGOS_CAVERN,        RG_RED_RUPEE,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_STAIRCASE_POT_3));
    locationTable[RC_DODONGOS_CAVERN_STAIRCASE_POT_4]          =    Location::Pot(RC_DODONGOS_CAVERN_STAIRCASE_POT_4,          RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(-1528, -1827), "Staircase Pot 4",                  RHT_POT_DODONGOS_CAVERN,        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_STAIRCASE_POT_4));
    locationTable[RC_DODONGOS_CAVERN_SINGLE_EYE_POT_1]         =    Location::Pot(RC_DODONGOS_CAVERN_SINGLE_EYE_POT_1,         RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(2704, -483),   "Single Eye Pot 1",                 RHT_POT_DODONGOS_CAVERN,        RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_SINGLE_EYE_POT_1));
    locationTable[RC_DODONGOS_CAVERN_SINGLE_EYE_POT_2]         =    Location::Pot(RC_DODONGOS_CAVERN_SINGLE_EYE_POT_2,         RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(3195, -155),   "Single Eye Pot 2",                 RHT_POT_DODONGOS_CAVERN,        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_SINGLE_EYE_POT_2));
    locationTable[RC_DODONGOS_CAVERN_BLADE_POT_1]              =    Location::Pot(RC_DODONGOS_CAVERN_BLADE_POT_1,              RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(1656, -531),   "Blade Pot 1",                      RHT_POT_DODONGOS_CAVERN,        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_BLADE_POT_1));
    locationTable[RC_DODONGOS_CAVERN_BLADE_POT_2]              =    Location::Pot(RC_DODONGOS_CAVERN_BLADE_POT_2,              RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(1831, -593),   "Blade Pot 2",                      RHT_POT_DODONGOS_CAVERN,        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_BLADE_POT_2));
    locationTable[RC_DODONGOS_CAVERN_DOUBLE_EYE_POT_1]         =    Location::Pot(RC_DODONGOS_CAVERN_DOUBLE_EYE_POT_1,         RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(3329, -753),   "Double Eye Pot 1",                 RHT_POT_DODONGOS_CAVERN,        RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_DOUBLE_EYE_POT_1));
    locationTable[RC_DODONGOS_CAVERN_DOUBLE_EYE_POT_2]         =    Location::Pot(RC_DODONGOS_CAVERN_DOUBLE_EYE_POT_2,         RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(2705, -1086),  "Double Eye Pot 2",                 RHT_POT_DODONGOS_CAVERN,        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_DOUBLE_EYE_POT_2));
    locationTable[RC_DODONGOS_CAVERN_BACK_ROOM_POT_1]          =    Location::Pot(RC_DODONGOS_CAVERN_BACK_ROOM_POT_1,          RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(1027, -3704),  "Back Room Pot 1",                  RHT_POT_DODONGOS_CAVERN,        RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_BACK_ROOM_POT_1));
    locationTable[RC_DODONGOS_CAVERN_BACK_ROOM_POT_2]          =    Location::Pot(RC_DODONGOS_CAVERN_BACK_ROOM_POT_2,          RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(966, -3704),   "Back Room Pot 2",                  RHT_POT_DODONGOS_CAVERN,        RG_DEKU_SEEDS_30,   SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_BACK_ROOM_POT_2));
    locationTable[RC_DODONGOS_CAVERN_BACK_ROOM_POT_3]          =    Location::Pot(RC_DODONGOS_CAVERN_BACK_ROOM_POT_3,          RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(125, -3854),   "Back Room Pot 3",                  RHT_POT_DODONGOS_CAVERN,        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_BACK_ROOM_POT_3));
    locationTable[RC_DODONGOS_CAVERN_BACK_ROOM_POT_4]          =    Location::Pot(RC_DODONGOS_CAVERN_BACK_ROOM_POT_4,          RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(78, -3835),    "Back Room Pot 4",                  RHT_POT_DODONGOS_CAVERN,        RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_BACK_ROOM_POT_4));
    locationTable[RC_JABU_JABUS_BELLY_ABOVE_BIG_OCTO_POT_1]    =    Location::Pot(RC_JABU_JABUS_BELLY_ABOVE_BIG_OCTO_POT_1,    RCQUEST_VANILLA, RCAREA_JABU_JABUS_BELLY,   SCENE_JABU_JABU,                   TWO_ACTOR_PARAMS(-1295, -3612), "Above Big Octo Pot 1",             RHT_POT_JABU_JABUS_BELLY,       RG_DEKU_NUTS_5,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_ABOVE_BIG_OCTO_POT_1));
    locationTable[RC_JABU_JABUS_BELLY_ABOVE_BIG_OCTO_POT_2]    =    Location::Pot(RC_JABU_JABUS_BELLY_ABOVE_BIG_OCTO_POT_2,    RCQUEST_VANILLA, RCAREA_JABU_JABUS_BELLY,   SCENE_JABU_JABU,                   TWO_ACTOR_PARAMS(-1355, -3612), "Above Big Octo Pot 2",             RHT_POT_JABU_JABUS_BELLY,       RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_ABOVE_BIG_OCTO_POT_2));
    locationTable[RC_JABU_JABUS_BELLY_ABOVE_BIG_OCTO_POT_3]    =    Location::Pot(RC_JABU_JABUS_BELLY_ABOVE_BIG_OCTO_POT_3,    RCQUEST_VANILLA, RCAREA_JABU_JABUS_BELLY,   SCENE_JABU_JABU,                   TWO_ACTOR_PARAMS(-1422, -3612), "Above Big Octo Pot 3",             RHT_POT_JABU_JABUS_BELLY,       RG_DEKU_NUTS_5,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_ABOVE_BIG_OCTO_POT_3));
    locationTable[RC_JABU_JABUS_BELLY_BARINADE_POT_1]          =    Location::Pot(RC_JABU_JABUS_BELLY_BARINADE_POT_1,          RCQUEST_BOTH, RCAREA_JABU_JABUS_BELLY,      SCENE_JABU_JABU_BOSS,              TWO_ACTOR_PARAMS(-551, 33),     "Barinade Pot 1",                   RHT_POT_JABU_JABUS_BELLY,       RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_BARINADE_POT_1));
    locationTable[RC_JABU_JABUS_BELLY_BARINADE_POT_2]          =    Location::Pot(RC_JABU_JABUS_BELLY_BARINADE_POT_2,          RCQUEST_BOTH, RCAREA_JABU_JABUS_BELLY,      SCENE_JABU_JABU_BOSS,              TWO_ACTOR_PARAMS(551, 36),      "Barinade Pot 2",                   RHT_POT_JABU_JABUS_BELLY,       RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_BARINADE_POT_2));
    locationTable[RC_JABU_JABUS_BELLY_BARINADE_POT_3]          =    Location::Pot(RC_JABU_JABUS_BELLY_BARINADE_POT_3,          RCQUEST_BOTH, RCAREA_JABU_JABUS_BELLY,      SCENE_JABU_JABU_BOSS,              TWO_ACTOR_PARAMS(554, -493),    "Barinade Pot 3",                   RHT_POT_JABU_JABUS_BELLY,       RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_BARINADE_POT_3));
    locationTable[RC_JABU_JABUS_BELLY_BARINADE_POT_4]          =    Location::Pot(RC_JABU_JABUS_BELLY_BARINADE_POT_4,          RCQUEST_BOTH, RCAREA_JABU_JABUS_BELLY,      SCENE_JABU_JABU_BOSS,              TWO_ACTOR_PARAMS(-543, -496),   "Barinade Pot 4",                   RHT_POT_JABU_JABUS_BELLY,       RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_BARINADE_POT_4));
    locationTable[RC_JABU_JABUS_BELLY_BARINADE_POT_5]          =    Location::Pot(RC_JABU_JABUS_BELLY_BARINADE_POT_5,          RCQUEST_BOTH, RCAREA_JABU_JABUS_BELLY,      SCENE_JABU_JABU_BOSS,              TWO_ACTOR_PARAMS(-268, -786),   "Barinade Pot 5",                   RHT_POT_JABU_JABUS_BELLY,       RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_BARINADE_POT_5));
    locationTable[RC_JABU_JABUS_BELLY_BARINADE_POT_6]          =    Location::Pot(RC_JABU_JABUS_BELLY_BARINADE_POT_6,          RCQUEST_BOTH, RCAREA_JABU_JABUS_BELLY,      SCENE_JABU_JABU_BOSS,              TWO_ACTOR_PARAMS(279, -761),    "Barinade Pot 6",                   RHT_POT_JABU_JABUS_BELLY,       RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_BARINADE_POT_6));
    locationTable[RC_JABU_JABUS_BELLY_BASEMENT_POT_1]          =    Location::Pot(RC_JABU_JABUS_BELLY_BASEMENT_POT_1,          RCQUEST_VANILLA, RCAREA_JABU_JABUS_BELLY,   SCENE_JABU_JABU,                   TWO_ACTOR_PARAMS(703, -2371),   "Basement Pot 1",                   RHT_POT_JABU_JABUS_BELLY,       RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_BASEMENT_POT_1));
    locationTable[RC_JABU_JABUS_BELLY_BASEMENT_POT_2]          =    Location::Pot(RC_JABU_JABUS_BELLY_BASEMENT_POT_2,          RCQUEST_VANILLA, RCAREA_JABU_JABUS_BELLY,   SCENE_JABU_JABU,                   TWO_ACTOR_PARAMS(645, -2408),   "Basement Pot 2",                   RHT_POT_JABU_JABUS_BELLY,       RG_DEKU_SEEDS_30,   SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_BASEMENT_POT_2));
    locationTable[RC_JABU_JABUS_BELLY_BASEMENT_POT_3]          =    Location::Pot(RC_JABU_JABUS_BELLY_BASEMENT_POT_3,          RCQUEST_VANILLA, RCAREA_JABU_JABUS_BELLY,   SCENE_JABU_JABU,                   TWO_ACTOR_PARAMS(650, -2344),   "Basement Pot 3",                   RHT_POT_JABU_JABUS_BELLY,       RG_DEKU_NUTS_5,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_BASEMENT_POT_3));
    locationTable[RC_JABU_JABUS_BELLY_TWO_OCTOROK_POT_1]       =    Location::Pot(RC_JABU_JABUS_BELLY_TWO_OCTOROK_POT_1,       RCQUEST_VANILLA, RCAREA_JABU_JABUS_BELLY,   SCENE_JABU_JABU,                   TWO_ACTOR_PARAMS(-1127, -2271), "Two Octorok Pot 1",                RHT_POT_JABU_JABUS_BELLY,       RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_TWO_OCTOROK_POT_1));
    locationTable[RC_JABU_JABUS_BELLY_TWO_OCTOROK_POT_2]       =    Location::Pot(RC_JABU_JABUS_BELLY_TWO_OCTOROK_POT_2,       RCQUEST_VANILLA, RCAREA_JABU_JABUS_BELLY,   SCENE_JABU_JABU,                   TWO_ACTOR_PARAMS(-1178, -2272), "Two Octorok Pot 2",                RHT_POT_JABU_JABUS_BELLY,       RG_RED_RUPEE,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_TWO_OCTOROK_POT_2));
    locationTable[RC_JABU_JABUS_BELLY_TWO_OCTOROK_POT_3]       =    Location::Pot(RC_JABU_JABUS_BELLY_TWO_OCTOROK_POT_3,       RCQUEST_VANILLA, RCAREA_JABU_JABUS_BELLY,   SCENE_JABU_JABU,                   TWO_ACTOR_PARAMS(-1150, -2248), "Two Octorok Pot 3",                RHT_POT_JABU_JABUS_BELLY,       RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_TWO_OCTOROK_POT_3));
    locationTable[RC_JABU_JABUS_BELLY_TWO_OCTOROK_POT_4]       =    Location::Pot(RC_JABU_JABUS_BELLY_TWO_OCTOROK_POT_4,       RCQUEST_VANILLA, RCAREA_JABU_JABUS_BELLY,   SCENE_JABU_JABU,                   TWO_ACTOR_PARAMS(-1131, -2221), "Two Octorok Pot 4",                RHT_POT_JABU_JABUS_BELLY,       RG_RED_RUPEE,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_TWO_OCTOROK_POT_4));
    locationTable[RC_JABU_JABUS_BELLY_TWO_OCTOROK_POT_5]       =    Location::Pot(RC_JABU_JABUS_BELLY_TWO_OCTOROK_POT_5,       RCQUEST_VANILLA, RCAREA_JABU_JABUS_BELLY,   SCENE_JABU_JABU,                   TWO_ACTOR_PARAMS(-1173, -2227), "Two Octorok Pot 5",                RHT_POT_JABU_JABUS_BELLY,       RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_TWO_OCTOROK_POT_5));
    locationTable[RC_FOREST_TEMPLE_LOBBY_POT_1]                =    Location::Pot(RC_FOREST_TEMPLE_LOBBY_POT_1,                RCQUEST_VANILLA, RCAREA_FOREST_TEMPLE,      SCENE_FOREST_TEMPLE,               TWO_ACTOR_PARAMS(720, -1037),   "Lobby Pot 1",                      RHT_POT_FOREST_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_LOBBY_POT_1));
    locationTable[RC_FOREST_TEMPLE_LOBBY_POT_2]                =    Location::Pot(RC_FOREST_TEMPLE_LOBBY_POT_2,                RCQUEST_VANILLA, RCAREA_FOREST_TEMPLE,      SCENE_FOREST_TEMPLE,               TWO_ACTOR_PARAMS(675, -989),    "Lobby Pot 2",                      RHT_POT_FOREST_TEMPLE,          RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_LOBBY_POT_2));
    locationTable[RC_FOREST_TEMPLE_LOBBY_POT_3]                =    Location::Pot(RC_FOREST_TEMPLE_LOBBY_POT_3,                RCQUEST_VANILLA, RCAREA_FOREST_TEMPLE,      SCENE_FOREST_TEMPLE,               TWO_ACTOR_PARAMS(630, -943),    "Lobby Pot 3",                      RHT_POT_FOREST_TEMPLE,          RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_LOBBY_POT_3));
    locationTable[RC_FOREST_TEMPLE_LOBBY_POT_4]                =    Location::Pot(RC_FOREST_TEMPLE_LOBBY_POT_4,                RCQUEST_VANILLA, RCAREA_FOREST_TEMPLE,      SCENE_FOREST_TEMPLE,               TWO_ACTOR_PARAMS(-384, -937),   "Lobby Pot 4",                      RHT_POT_FOREST_TEMPLE,          RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_LOBBY_POT_4));
    locationTable[RC_FOREST_TEMPLE_LOBBY_POT_5]                =    Location::Pot(RC_FOREST_TEMPLE_LOBBY_POT_5,                RCQUEST_VANILLA, RCAREA_FOREST_TEMPLE,      SCENE_FOREST_TEMPLE,               TWO_ACTOR_PARAMS(-429, -981),   "Lobby Pot 5",                      RHT_POT_FOREST_TEMPLE,          RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_LOBBY_POT_5));
    locationTable[RC_FOREST_TEMPLE_LOBBY_POT_6]                =    Location::Pot(RC_FOREST_TEMPLE_LOBBY_POT_6,                RCQUEST_VANILLA, RCAREA_FOREST_TEMPLE,      SCENE_FOREST_TEMPLE,               TWO_ACTOR_PARAMS(-477, -1034),  "Lobby Pot 6",                      RHT_POT_FOREST_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_LOBBY_POT_6));
    locationTable[RC_FOREST_TEMPLE_LOWER_STALFOS_POT_1]        =    Location::Pot(RC_FOREST_TEMPLE_LOWER_STALFOS_POT_1,        RCQUEST_VANILLA, RCAREA_FOREST_TEMPLE,      SCENE_FOREST_TEMPLE,               TWO_ACTOR_PARAMS(90, -3583),    "Lower Stalfos Pot 1",              RHT_POT_FOREST_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_LOWER_STALFOS_POT_1));
    locationTable[RC_FOREST_TEMPLE_LOWER_STALFOS_POT_2]        =    Location::Pot(RC_FOREST_TEMPLE_LOWER_STALFOS_POT_2,        RCQUEST_VANILLA, RCAREA_FOREST_TEMPLE,      SCENE_FOREST_TEMPLE,               TWO_ACTOR_PARAMS(142, -3583),   "Lower Stalfos Pot 2",              RHT_POT_FOREST_TEMPLE,          RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_LOWER_STALFOS_POT_2));
    locationTable[RC_FOREST_TEMPLE_GREEN_POE_POT_1]            =    Location::Pot(RC_FOREST_TEMPLE_GREEN_POE_POT_1,            RCQUEST_VANILLA, RCAREA_FOREST_TEMPLE,      SCENE_FOREST_TEMPLE,               TWO_ACTOR_PARAMS(1647, -1529),  "Green Poe Pot 1",                  RHT_POT_FOREST_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_GREEN_POE_POT_1));
    locationTable[RC_FOREST_TEMPLE_GREEN_POE_POT_2]            =    Location::Pot(RC_FOREST_TEMPLE_GREEN_POE_POT_2,            RCQUEST_VANILLA, RCAREA_FOREST_TEMPLE,      SCENE_FOREST_TEMPLE,               TWO_ACTOR_PARAMS(1645, -1588),  "Green Poe Pot 2",                  RHT_POT_FOREST_TEMPLE,          RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_GREEN_POE_POT_2));
    locationTable[RC_FOREST_TEMPLE_UPPER_STALFOS_POT_1]        =    Location::Pot(RC_FOREST_TEMPLE_UPPER_STALFOS_POT_1,        RCQUEST_VANILLA, RCAREA_FOREST_TEMPLE,      SCENE_FOREST_TEMPLE,               TWO_ACTOR_PARAMS(32, -3012),    "Upper Stalfos Pot 1",              RHT_POT_FOREST_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_UPPER_STALFOS_POT_1));
    locationTable[RC_FOREST_TEMPLE_UPPER_STALFOS_POT_2]        =    Location::Pot(RC_FOREST_TEMPLE_UPPER_STALFOS_POT_2,        RCQUEST_VANILLA, RCAREA_FOREST_TEMPLE,      SCENE_FOREST_TEMPLE,               TWO_ACTOR_PARAMS(202, -3012),   "Upper Stalfos Pot 2",              RHT_POT_FOREST_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_UPPER_STALFOS_POT_2));
    locationTable[RC_FOREST_TEMPLE_UPPER_STALFOS_POT_3]        =    Location::Pot(RC_FOREST_TEMPLE_UPPER_STALFOS_POT_3,        RCQUEST_VANILLA, RCAREA_FOREST_TEMPLE,      SCENE_FOREST_TEMPLE,               TWO_ACTOR_PARAMS(30, -3631),    "Upper Stalfos Pot 3",              RHT_POT_FOREST_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_UPPER_STALFOS_POT_3));
    locationTable[RC_FOREST_TEMPLE_UPPER_STALFOS_POT_4]        =    Location::Pot(RC_FOREST_TEMPLE_UPPER_STALFOS_POT_4,        RCQUEST_VANILLA, RCAREA_FOREST_TEMPLE,      SCENE_FOREST_TEMPLE,               TWO_ACTOR_PARAMS(206, -3631),   "Upper Stalfos Pot 4",              RHT_POT_FOREST_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_UPPER_STALFOS_POT_4));
    locationTable[RC_FOREST_TEMPLE_BLUE_POE_POT_1]             =    Location::Pot(RC_FOREST_TEMPLE_BLUE_POE_POT_1,             RCQUEST_VANILLA, RCAREA_FOREST_TEMPLE,      SCENE_FOREST_TEMPLE,               TWO_ACTOR_PARAMS(830, -3639),   "Blue Poe Pot 1",                   RHT_POT_FOREST_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_BLUE_POE_POT_1));
    locationTable[RC_FOREST_TEMPLE_BLUE_POE_POT_2]             =    Location::Pot(RC_FOREST_TEMPLE_BLUE_POE_POT_2,             RCQUEST_VANILLA, RCAREA_FOREST_TEMPLE,      SCENE_FOREST_TEMPLE,               TWO_ACTOR_PARAMS(883, -3640),   "Blue Poe Pot 2",                   RHT_POT_FOREST_TEMPLE,          RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_BLUE_POE_POT_2));
    locationTable[RC_FOREST_TEMPLE_BLUE_POE_POT_3]             =    Location::Pot(RC_FOREST_TEMPLE_BLUE_POE_POT_3,             RCQUEST_VANILLA, RCAREA_FOREST_TEMPLE,      SCENE_FOREST_TEMPLE,               TWO_ACTOR_PARAMS(941, -3640),   "Blue Poe Pot 3",                   RHT_POT_FOREST_TEMPLE,          RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_BLUE_POE_POT_3));
    locationTable[RC_FOREST_TEMPLE_FROZEN_EYE_POT_1]           =    Location::Pot(RC_FOREST_TEMPLE_FROZEN_EYE_POT_1,           RCQUEST_VANILLA, RCAREA_FOREST_TEMPLE,      SCENE_FOREST_TEMPLE,               TWO_ACTOR_PARAMS(2312, -874),   "Frozen Eye Pot 1",                 RHT_POT_FOREST_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_FROZEN_EYE_POT_1));
    locationTable[RC_FOREST_TEMPLE_FROZEN_EYE_POT_2]           =    Location::Pot(RC_FOREST_TEMPLE_FROZEN_EYE_POT_2,           RCQUEST_VANILLA, RCAREA_FOREST_TEMPLE,      SCENE_FOREST_TEMPLE,               TWO_ACTOR_PARAMS(2364, -873),   "Frozen Eye Pot 2",                 RHT_POT_FOREST_TEMPLE,          RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_FROZEN_EYE_POT_2));
    locationTable[RC_FIRE_TEMPLE_NEAR_BOSS_POT_1]              =    Location::Pot(RC_FIRE_TEMPLE_NEAR_BOSS_POT_1,              RCQUEST_VANILLA, RCAREA_FIRE_TEMPLE,        SCENE_FIRE_TEMPLE,                 TWO_ACTOR_PARAMS(-1086, -714),  "Near Boss Pot 1",                  RHT_POT_FIRE_TEMPLE,            RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_NEAR_BOSS_POT_1));
    locationTable[RC_FIRE_TEMPLE_NEAR_BOSS_POT_2]              =    Location::Pot(RC_FIRE_TEMPLE_NEAR_BOSS_POT_2,              RCQUEST_VANILLA, RCAREA_FIRE_TEMPLE,        SCENE_FIRE_TEMPLE,                 TWO_ACTOR_PARAMS(-1138, -713),  "Near Boss Pot 2",                  RHT_POT_FIRE_TEMPLE,            RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_NEAR_BOSS_POT_2));
    locationTable[RC_FIRE_TEMPLE_NEAR_BOSS_POT_3]              =    Location::Pot(RC_FIRE_TEMPLE_NEAR_BOSS_POT_3,              RCQUEST_VANILLA, RCAREA_FIRE_TEMPLE,        SCENE_FIRE_TEMPLE,                 TWO_ACTOR_PARAMS(-1087, -765),  "Near Boss Pot 3",                  RHT_POT_FIRE_TEMPLE,            RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_NEAR_BOSS_POT_3));
    locationTable[RC_FIRE_TEMPLE_NEAR_BOSS_POT_4]              =    Location::Pot(RC_FIRE_TEMPLE_NEAR_BOSS_POT_4,              RCQUEST_VANILLA, RCAREA_FIRE_TEMPLE,        SCENE_FIRE_TEMPLE,                 TWO_ACTOR_PARAMS(-1139, -766),  "Near Boss Pot 4",                  RHT_POT_FIRE_TEMPLE,            RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_NEAR_BOSS_POT_4));
    locationTable[RC_FIRE_TEMPLE_BIG_LAVA_POT_1]               =    Location::Pot(RC_FIRE_TEMPLE_BIG_LAVA_POT_1,               RCQUEST_VANILLA, RCAREA_FIRE_TEMPLE,        SCENE_FIRE_TEMPLE,                 TWO_ACTOR_PARAMS(2226, -754),   "Big Lava Pot 1",                   RHT_POT_FIRE_TEMPLE,            RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_BIG_LAVA_POT_1));
    locationTable[RC_FIRE_TEMPLE_BIG_LAVA_POT_2]               =    Location::Pot(RC_FIRE_TEMPLE_BIG_LAVA_POT_2,               RCQUEST_VANILLA, RCAREA_FIRE_TEMPLE,        SCENE_FIRE_TEMPLE,                 TWO_ACTOR_PARAMS(2226, -809),   "Big Lava Pot 2",                   RHT_POT_FIRE_TEMPLE,            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_BIG_LAVA_POT_2));
    locationTable[RC_FIRE_TEMPLE_BIG_LAVA_POT_3]               =    Location::Pot(RC_FIRE_TEMPLE_BIG_LAVA_POT_3,               RCQUEST_VANILLA, RCAREA_FIRE_TEMPLE,        SCENE_FIRE_TEMPLE,                 TWO_ACTOR_PARAMS(2227, -862),   "Big Lava Pot 3",                   RHT_POT_FIRE_TEMPLE,            RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_BIG_LAVA_POT_3));
    locationTable[RC_FIRE_TEMPLE_FLAME_MAZE_LEFT_POT_1]        =    Location::Pot(RC_FIRE_TEMPLE_FLAME_MAZE_LEFT_POT_1,        RCQUEST_VANILLA, RCAREA_FIRE_TEMPLE,        SCENE_FIRE_TEMPLE,                 TWO_ACTOR_PARAMS(-2388, 953),   "Flame Maze Left Pot 1",            RHT_POT_FIRE_TEMPLE,            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_FLAME_MAZE_LEFT_POT_1));
    locationTable[RC_FIRE_TEMPLE_FLAME_MAZE_LEFT_POT_2]        =    Location::Pot(RC_FIRE_TEMPLE_FLAME_MAZE_LEFT_POT_2,        RCQUEST_VANILLA, RCAREA_FIRE_TEMPLE,        SCENE_FIRE_TEMPLE,                 TWO_ACTOR_PARAMS(-2423, 920),   "Flame Maze Left Pot 2",            RHT_POT_FIRE_TEMPLE,            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_FLAME_MAZE_LEFT_POT_2));
    locationTable[RC_FIRE_TEMPLE_FLAME_MAZE_LEFT_POT_3]        =    Location::Pot(RC_FIRE_TEMPLE_FLAME_MAZE_LEFT_POT_3,        RCQUEST_VANILLA, RCAREA_FIRE_TEMPLE,        SCENE_FIRE_TEMPLE,                 TWO_ACTOR_PARAMS(-2442, 715),   "Flame Maze Left Pot 3",            RHT_POT_FIRE_TEMPLE,            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_FLAME_MAZE_LEFT_POT_3));
    locationTable[RC_FIRE_TEMPLE_FLAME_MAZE_LEFT_POT_4]        =    Location::Pot(RC_FIRE_TEMPLE_FLAME_MAZE_LEFT_POT_4,        RCQUEST_VANILLA, RCAREA_FIRE_TEMPLE,        SCENE_FIRE_TEMPLE,                 TWO_ACTOR_PARAMS(-2408, 680),   "Flame Maze Left Pot 4",            RHT_POT_FIRE_TEMPLE,            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_FLAME_MAZE_LEFT_POT_4));
    locationTable[RC_FIRE_TEMPLE_FLAME_MAZE_RIGHT_POT_1]       =    Location::Pot(RC_FIRE_TEMPLE_FLAME_MAZE_RIGHT_POT_1,       RCQUEST_VANILLA, RCAREA_FIRE_TEMPLE,        SCENE_FIRE_TEMPLE,                 TWO_ACTOR_PARAMS(-2208, -168),  "Flame Maze Right Pot 1",           RHT_POT_FIRE_TEMPLE,            RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_FLAME_MAZE_RIGHT_POT_1));
    locationTable[RC_FIRE_TEMPLE_FLAME_MAZE_RIGHT_POT_2]       =    Location::Pot(RC_FIRE_TEMPLE_FLAME_MAZE_RIGHT_POT_2,       RCQUEST_VANILLA, RCAREA_FIRE_TEMPLE,        SCENE_FIRE_TEMPLE,                 TWO_ACTOR_PARAMS(-2253, -167),  "Flame Maze Right Pot 2",           RHT_POT_FIRE_TEMPLE,            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_FLAME_MAZE_RIGHT_POT_2));
    locationTable[RC_FIRE_TEMPLE_FLAME_MAZE_RIGHT_POT_3]       =    Location::Pot(RC_FIRE_TEMPLE_FLAME_MAZE_RIGHT_POT_3,       RCQUEST_VANILLA, RCAREA_FIRE_TEMPLE,        SCENE_FIRE_TEMPLE,                 TWO_ACTOR_PARAMS(-2657, -591),  "Flame Maze Right Pot 3",           RHT_POT_FIRE_TEMPLE,            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_FLAME_MAZE_RIGHT_POT_3));
    locationTable[RC_FIRE_TEMPLE_FLAME_MAZE_RIGHT_POT_4]       =    Location::Pot(RC_FIRE_TEMPLE_FLAME_MAZE_RIGHT_POT_4,       RCQUEST_VANILLA, RCAREA_FIRE_TEMPLE,        SCENE_FIRE_TEMPLE,                 TWO_ACTOR_PARAMS(-2657, -635),  "Flame Maze Right Pot 4",           RHT_POT_FIRE_TEMPLE,            RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_FLAME_MAZE_RIGHT_POT_4));
    locationTable[RC_WATER_TEMPLE_MAIN_LEVEL_2_POT_1]          =    Location::Pot(RC_WATER_TEMPLE_MAIN_LEVEL_2_POT_1,          RCQUEST_VANILLA, RCAREA_WATER_TEMPLE,       SCENE_WATER_TEMPLE,                TWO_ACTOR_PARAMS(330, -198),    "Main Level 2 Pot 1",               RHT_POT_WATER_TEMPLE,           RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MAIN_LEVEL_2_POT_1));
    locationTable[RC_WATER_TEMPLE_MAIN_LEVEL_2_POT_2]          =    Location::Pot(RC_WATER_TEMPLE_MAIN_LEVEL_2_POT_2,          RCQUEST_VANILLA, RCAREA_WATER_TEMPLE,       SCENE_WATER_TEMPLE,                TWO_ACTOR_PARAMS(331, -168),    "Main Level 2 Pot 2",               RHT_POT_WATER_TEMPLE,           RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MAIN_LEVEL_2_POT_2));
    locationTable[RC_WATER_TEMPLE_MAIN_LEVEL_1_POT_1]          =    Location::Pot(RC_WATER_TEMPLE_MAIN_LEVEL_1_POT_1,          RCQUEST_VANILLA, RCAREA_WATER_TEMPLE,       SCENE_WATER_TEMPLE,                TWO_ACTOR_PARAMS(-241, -1193),  "Main Level 1 Pot 1",               RHT_POT_WATER_TEMPLE,           RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MAIN_LEVEL_1_POT_1));
    locationTable[RC_WATER_TEMPLE_MAIN_LEVEL_1_POT_2]          =    Location::Pot(RC_WATER_TEMPLE_MAIN_LEVEL_1_POT_2,          RCQUEST_VANILLA, RCAREA_WATER_TEMPLE,       SCENE_WATER_TEMPLE,                TWO_ACTOR_PARAMS(-117, -1192),  "Main Level 1 Pot 2",               RHT_POT_WATER_TEMPLE,           RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MAIN_LEVEL_1_POT_2));
    locationTable[RC_WATER_TEMPLE_TORCH_POT_1]                 =    Location::Pot(RC_WATER_TEMPLE_TORCH_POT_1,                 RCQUEST_VANILLA, RCAREA_WATER_TEMPLE,       SCENE_WATER_TEMPLE,                TWO_ACTOR_PARAMS(1123, 62),     "Torch Pot 1",                      RHT_POT_WATER_TEMPLE,           RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_TORCH_POT_1));
    locationTable[RC_WATER_TEMPLE_TORCH_POT_2]                 =    Location::Pot(RC_WATER_TEMPLE_TORCH_POT_2,                 RCQUEST_VANILLA, RCAREA_WATER_TEMPLE,       SCENE_WATER_TEMPLE,                TWO_ACTOR_PARAMS(1123, 293),    "Torch Pot 2",                      RHT_POT_WATER_TEMPLE,           RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_TORCH_POT_2));
    locationTable[RC_WATER_TEMPLE_NEAR_COMPASS_POT_1]          =    Location::Pot(RC_WATER_TEMPLE_NEAR_COMPASS_POT_1,          RCQUEST_VANILLA, RCAREA_WATER_TEMPLE,       SCENE_WATER_TEMPLE,                TWO_ACTOR_PARAMS(1222, -617),   "Near Compass Pot 1",               RHT_POT_WATER_TEMPLE,           RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_NEAR_COMPASS_POT_1));
    locationTable[RC_WATER_TEMPLE_NEAR_COMPASS_POT_2]          =    Location::Pot(RC_WATER_TEMPLE_NEAR_COMPASS_POT_2,          RCQUEST_VANILLA, RCAREA_WATER_TEMPLE,       SCENE_WATER_TEMPLE,                TWO_ACTOR_PARAMS(1172, -617),   "Near Compass Pot 2",               RHT_POT_WATER_TEMPLE,           RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_NEAR_COMPASS_POT_2));
    locationTable[RC_WATER_TEMPLE_NEAR_COMPASS_POT_3]          =    Location::Pot(RC_WATER_TEMPLE_NEAR_COMPASS_POT_3,          RCQUEST_VANILLA, RCAREA_WATER_TEMPLE,       SCENE_WATER_TEMPLE,                TWO_ACTOR_PARAMS(1122, -617),   "Near Compass Pot 3",               RHT_POT_WATER_TEMPLE,           RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_NEAR_COMPASS_POT_3));
    locationTable[RC_WATER_TEMPLE_CENTRAL_BOW_POT_1]           =    Location::Pot(RC_WATER_TEMPLE_CENTRAL_BOW_POT_1,           RCQUEST_VANILLA, RCAREA_WATER_TEMPLE,       SCENE_WATER_TEMPLE,                TWO_ACTOR_PARAMS(688, 1008),    "Central Bow Pot 1",                RHT_POT_WATER_TEMPLE,           RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_CENTRAL_BOW_POT_1));
    locationTable[RC_WATER_TEMPLE_CENTRAL_BOW_POT_2]           =    Location::Pot(RC_WATER_TEMPLE_CENTRAL_BOW_POT_2,           RCQUEST_VANILLA, RCAREA_WATER_TEMPLE,       SCENE_WATER_TEMPLE,                TWO_ACTOR_PARAMS(631, 1008),    "Central Bow Pot 2",                RHT_POT_WATER_TEMPLE,           RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_CENTRAL_BOW_POT_2));
    locationTable[RC_WATER_TEMPLE_BEHIND_GATE_POT_1]           =    Location::Pot(RC_WATER_TEMPLE_BEHIND_GATE_POT_1,           RCQUEST_VANILLA, RCAREA_WATER_TEMPLE,       SCENE_WATER_TEMPLE,                TWO_ACTOR_PARAMS(-2313, 944),   "Behind Gate Pot 1",                RHT_POT_WATER_TEMPLE,           RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_BEHIND_GATE_POT_1));
    locationTable[RC_WATER_TEMPLE_BEHIND_GATE_POT_2]           =    Location::Pot(RC_WATER_TEMPLE_BEHIND_GATE_POT_2,           RCQUEST_VANILLA, RCAREA_WATER_TEMPLE,       SCENE_WATER_TEMPLE,                TWO_ACTOR_PARAMS(-2313, 905),   "Behind Gate Pot 2",                RHT_POT_WATER_TEMPLE,           RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_BEHIND_GATE_POT_2));
    locationTable[RC_WATER_TEMPLE_BEHIND_GATE_POT_3]           =    Location::Pot(RC_WATER_TEMPLE_BEHIND_GATE_POT_3,           RCQUEST_VANILLA, RCAREA_WATER_TEMPLE,       SCENE_WATER_TEMPLE,                TWO_ACTOR_PARAMS(-2314, 808),   "Behind Gate Pot 3",                RHT_POT_WATER_TEMPLE,           RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_BEHIND_GATE_POT_3));
    locationTable[RC_WATER_TEMPLE_BEHIND_GATE_POT_4]           =    Location::Pot(RC_WATER_TEMPLE_BEHIND_GATE_POT_4,           RCQUEST_VANILLA, RCAREA_WATER_TEMPLE,       SCENE_WATER_TEMPLE,                TWO_ACTOR_PARAMS(-2314, 770),   "Behind Gate Pot 4",                RHT_POT_WATER_TEMPLE,           RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_BEHIND_GATE_POT_4));
    locationTable[RC_WATER_TEMPLE_BASEMENT_BLOCK_PUZZLE_POT_1] =    Location::Pot(RC_WATER_TEMPLE_BASEMENT_BLOCK_PUZZLE_POT_1, RCQUEST_VANILLA, RCAREA_WATER_TEMPLE,       SCENE_WATER_TEMPLE,                TWO_ACTOR_PARAMS(-28, -3817),   "Basement Block Puzzle Pot 1",      RHT_POT_WATER_TEMPLE,           RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_BASEMENT_BLOCK_PUZZLE_POT_1));
    locationTable[RC_WATER_TEMPLE_BASEMENT_BLOCK_PUZZLE_POT_2] =    Location::Pot(RC_WATER_TEMPLE_BASEMENT_BLOCK_PUZZLE_POT_2, RCQUEST_VANILLA, RCAREA_WATER_TEMPLE,       SCENE_WATER_TEMPLE,                TWO_ACTOR_PARAMS(28, -3817),    "Basement Block Puzzle Pot 2",      RHT_POT_WATER_TEMPLE,           RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_BASEMENT_BLOCK_PUZZLE_POT_2));
    locationTable[RC_WATER_TEMPLE_RIVER_POT_1]                 =    Location::Pot(RC_WATER_TEMPLE_RIVER_POT_1,                 RCQUEST_VANILLA, RCAREA_WATER_TEMPLE,       SCENE_WATER_TEMPLE,                TWO_ACTOR_PARAMS(-2226, -2487), "River Pot 1",                      RHT_POT_WATER_TEMPLE,           RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_RIVER_POT_1));
    locationTable[RC_WATER_TEMPLE_RIVER_POT_2]                 =    Location::Pot(RC_WATER_TEMPLE_RIVER_POT_2,                 RCQUEST_VANILLA, RCAREA_WATER_TEMPLE,       SCENE_WATER_TEMPLE,                TWO_ACTOR_PARAMS(-2184, -2456), "River Pot 2",                      RHT_POT_WATER_TEMPLE,           RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_RIVER_POT_2));
    locationTable[RC_WATER_TEMPLE_LIKE_LIKE_POT_1]             =    Location::Pot(RC_WATER_TEMPLE_LIKE_LIKE_POT_1,             RCQUEST_VANILLA, RCAREA_WATER_TEMPLE,       SCENE_WATER_TEMPLE,                TWO_ACTOR_PARAMS(-3188, -1628), "Like Like Pot 1",                  RHT_POT_WATER_TEMPLE,           RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_LIKE_LIKE_POT_1));
    locationTable[RC_WATER_TEMPLE_LIKE_LIKE_POT_2]             =    Location::Pot(RC_WATER_TEMPLE_LIKE_LIKE_POT_2,             RCQUEST_VANILLA, RCAREA_WATER_TEMPLE,       SCENE_WATER_TEMPLE,                TWO_ACTOR_PARAMS(-3056, -1626), "Like Like Pot 2",                  RHT_POT_WATER_TEMPLE,           RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_LIKE_LIKE_POT_2));
    locationTable[RC_WATER_TEMPLE_BOSS_KEY_POT_1]              =    Location::Pot(RC_WATER_TEMPLE_BOSS_KEY_POT_1,              RCQUEST_VANILLA, RCAREA_WATER_TEMPLE,       SCENE_WATER_TEMPLE,                TWO_ACTOR_PARAMS(-1150, -3031), "Boss Key Pot 1",                   RHT_POT_WATER_TEMPLE,           RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_BOSS_KEY_POT_1));
    locationTable[RC_WATER_TEMPLE_BOSS_KEY_POT_2]              =    Location::Pot(RC_WATER_TEMPLE_BOSS_KEY_POT_2,              RCQUEST_VANILLA, RCAREA_WATER_TEMPLE,       SCENE_WATER_TEMPLE,                TWO_ACTOR_PARAMS(-1417, -3025), "Boss Key Pot 2",                   RHT_POT_WATER_TEMPLE,           RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_BOSS_KEY_POT_2));
    locationTable[RC_SHADOW_TEMPLE_NEAR_DEAD_HAND_POT_1]       =    Location::Pot(RC_SHADOW_TEMPLE_NEAR_DEAD_HAND_POT_1,       RCQUEST_VANILLA, RCAREA_SHADOW_TEMPLE,      SCENE_SHADOW_TEMPLE,               TWO_ACTOR_PARAMS(-1806, -344),  "Near Dead Hand Pot 1",             RHT_POT_SHADOW_TEMPLE,          RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_NEAR_DEAD_HAND_POT_1));
    locationTable[RC_SHADOW_TEMPLE_WHISPERING_WALLS_POT_1]     =    Location::Pot(RC_SHADOW_TEMPLE_WHISPERING_WALLS_POT_1,     RCQUEST_VANILLA, RCAREA_SHADOW_TEMPLE,      SCENE_SHADOW_TEMPLE,               TWO_ACTOR_PARAMS(-1721, 61),    "Whispering Walls Pot 1",           RHT_POT_SHADOW_TEMPLE,          RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_WHISPERING_WALLS_POT_1));
    locationTable[RC_SHADOW_TEMPLE_WHISPERING_WALLS_POT_2]     =    Location::Pot(RC_SHADOW_TEMPLE_WHISPERING_WALLS_POT_2,     RCQUEST_VANILLA, RCAREA_SHADOW_TEMPLE,      SCENE_SHADOW_TEMPLE,               TWO_ACTOR_PARAMS(-1407, 61),    "Whispering Walls Pot 2",           RHT_POT_SHADOW_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_WHISPERING_WALLS_POT_2));
    locationTable[RC_SHADOW_TEMPLE_WHISPERING_WALLS_POT_3]     =    Location::Pot(RC_SHADOW_TEMPLE_WHISPERING_WALLS_POT_3,     RCQUEST_VANILLA, RCAREA_SHADOW_TEMPLE,      SCENE_SHADOW_TEMPLE,               TWO_ACTOR_PARAMS(-1614, 61),    "Whispering Walls Pot 3",           RHT_POT_SHADOW_TEMPLE,          RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_WHISPERING_WALLS_POT_3));
    locationTable[RC_SHADOW_TEMPLE_WHISPERING_WALLS_POT_4]     =    Location::Pot(RC_SHADOW_TEMPLE_WHISPERING_WALLS_POT_4,     RCQUEST_VANILLA, RCAREA_SHADOW_TEMPLE,      SCENE_SHADOW_TEMPLE,               TWO_ACTOR_PARAMS(-1055, -257),  "Whispering Walls Pot 4",           RHT_POT_SHADOW_TEMPLE,          RG_DEKU_NUTS_5,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_WHISPERING_WALLS_POT_4));
    locationTable[RC_SHADOW_TEMPLE_WHISPERING_WALLS_POT_5]     =    Location::Pot(RC_SHADOW_TEMPLE_WHISPERING_WALLS_POT_5,     RCQUEST_VANILLA, RCAREA_SHADOW_TEMPLE,      SCENE_SHADOW_TEMPLE,               TWO_ACTOR_PARAMS(-1050, -130),  "Whispering Walls Pot 5",           RHT_POT_SHADOW_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_WHISPERING_WALLS_POT_5));
    locationTable[RC_SHADOW_TEMPLE_MAP_CHEST_POT_1]            =    Location::Pot(RC_SHADOW_TEMPLE_MAP_CHEST_POT_1,            RCQUEST_VANILLA, RCAREA_SHADOW_TEMPLE,      SCENE_SHADOW_TEMPLE,               TWO_ACTOR_PARAMS(-665, -760),   "Map Chest Pot 1",                  RHT_POT_SHADOW_TEMPLE,          RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_MAP_CHEST_POT_1));
    locationTable[RC_SHADOW_TEMPLE_MAP_CHEST_POT_2]            =    Location::Pot(RC_SHADOW_TEMPLE_MAP_CHEST_POT_2,            RCQUEST_VANILLA, RCAREA_SHADOW_TEMPLE,      SCENE_SHADOW_TEMPLE,               TWO_ACTOR_PARAMS(-929, -748),   "Map Chest Pot 2",                  RHT_POT_SHADOW_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_MAP_CHEST_POT_2));
    locationTable[RC_SHADOW_TEMPLE_FALLING_SPIKES_POT_1]       =    Location::Pot(RC_SHADOW_TEMPLE_FALLING_SPIKES_POT_1,       RCQUEST_VANILLA, RCAREA_SHADOW_TEMPLE,      SCENE_SHADOW_TEMPLE,               TWO_ACTOR_PARAMS(614, 3579),    "Falling Spikes Pot 1",             RHT_POT_SHADOW_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_FALLING_SPIKES_POT_1));
    locationTable[RC_SHADOW_TEMPLE_FALLING_SPIKES_POT_2]       =    Location::Pot(RC_SHADOW_TEMPLE_FALLING_SPIKES_POT_2,       RCQUEST_VANILLA, RCAREA_SHADOW_TEMPLE,      SCENE_SHADOW_TEMPLE,               TWO_ACTOR_PARAMS(798, 3574),    "Falling Spikes Pot 2",             RHT_POT_SHADOW_TEMPLE,          RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_FALLING_SPIKES_POT_2));
    locationTable[RC_SHADOW_TEMPLE_FALLING_SPIKES_POT_3]       =    Location::Pot(RC_SHADOW_TEMPLE_FALLING_SPIKES_POT_3,       RCQUEST_VANILLA, RCAREA_SHADOW_TEMPLE,      SCENE_SHADOW_TEMPLE,               TWO_ACTOR_PARAMS(1173, 3707),   "Falling Spikes Pot 3",             RHT_POT_SHADOW_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_FALLING_SPIKES_POT_3));
    locationTable[RC_SHADOW_TEMPLE_FALLING_SPIKES_POT_4]       =    Location::Pot(RC_SHADOW_TEMPLE_FALLING_SPIKES_POT_4,       RCQUEST_VANILLA, RCAREA_SHADOW_TEMPLE,      SCENE_SHADOW_TEMPLE,               TWO_ACTOR_PARAMS(1173, 3605),   "Falling Spikes Pot 4",             RHT_POT_SHADOW_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_FALLING_SPIKES_POT_4));
    locationTable[RC_SHADOW_TEMPLE_AFTER_WIND_POT_1]           =    Location::Pot(RC_SHADOW_TEMPLE_AFTER_WIND_POT_1,           RCQUEST_VANILLA, RCAREA_SHADOW_TEMPLE,      SCENE_SHADOW_TEMPLE,               TWO_ACTOR_PARAMS(4222, -916),   "After Wind Pot 1",                 RHT_POT_SHADOW_TEMPLE,          RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_AFTER_WIND_POT_1));
    locationTable[RC_SHADOW_TEMPLE_AFTER_WIND_POT_2]           =    Location::Pot(RC_SHADOW_TEMPLE_AFTER_WIND_POT_2,           RCQUEST_VANILLA, RCAREA_SHADOW_TEMPLE,      SCENE_SHADOW_TEMPLE,               TWO_ACTOR_PARAMS(4549, -922),   "After Wind Pot 2",                 RHT_POT_SHADOW_TEMPLE,          RG_DEKU_NUTS_5,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_AFTER_WIND_POT_2));
    locationTable[RC_SHADOW_TEMPLE_SPIKE_WALLS_POT_1]          =    Location::Pot(RC_SHADOW_TEMPLE_SPIKE_WALLS_POT_1,          RCQUEST_VANILLA, RCAREA_SHADOW_TEMPLE,      SCENE_SHADOW_TEMPLE,               TWO_ACTOR_PARAMS(-4303, -2657), "Spike Walls Pot 1",                RHT_POT_SHADOW_TEMPLE,          RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_SPIKE_WALLS_POT_1));
    locationTable[RC_SHADOW_TEMPLE_FLOORMASTER_POT_1]          =    Location::Pot(RC_SHADOW_TEMPLE_FLOORMASTER_POT_1,          RCQUEST_VANILLA, RCAREA_SHADOW_TEMPLE,      SCENE_SHADOW_TEMPLE,               TWO_ACTOR_PARAMS(-4575, -811),  "Floormaster Pot 1",                RHT_POT_SHADOW_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_FLOORMASTER_POT_1));
    locationTable[RC_SHADOW_TEMPLE_FLOORMASTER_POT_2]          =    Location::Pot(RC_SHADOW_TEMPLE_FLOORMASTER_POT_2,          RCQUEST_VANILLA, RCAREA_SHADOW_TEMPLE,      SCENE_SHADOW_TEMPLE,               TWO_ACTOR_PARAMS(-4036, -811),  "Floormaster Pot 2",                RHT_POT_SHADOW_TEMPLE,          RG_ARROWS_30,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_FLOORMASTER_POT_2));
    locationTable[RC_SHADOW_TEMPLE_AFTER_BOAT_POT_1]           =    Location::Pot(RC_SHADOW_TEMPLE_AFTER_BOAT_POT_1,           RCQUEST_VANILLA, RCAREA_SHADOW_TEMPLE,      SCENE_SHADOW_TEMPLE,               TWO_ACTOR_PARAMS(-2400, -1470), "After Boat Pot 1",                 RHT_POT_SHADOW_TEMPLE,          RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_AFTER_BOAT_POT_1));
    locationTable[RC_SHADOW_TEMPLE_AFTER_BOAT_POT_2]           =    Location::Pot(RC_SHADOW_TEMPLE_AFTER_BOAT_POT_2,           RCQUEST_VANILLA, RCAREA_SHADOW_TEMPLE,      SCENE_SHADOW_TEMPLE,               TWO_ACTOR_PARAMS(-2230, -1470), "After Boat Pot 2",                 RHT_POT_SHADOW_TEMPLE,          RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_AFTER_BOAT_POT_2));
    locationTable[RC_SHADOW_TEMPLE_AFTER_BOAT_POT_3]           =    Location::Pot(RC_SHADOW_TEMPLE_AFTER_BOAT_POT_3,           RCQUEST_VANILLA, RCAREA_SHADOW_TEMPLE,      SCENE_SHADOW_TEMPLE,               TWO_ACTOR_PARAMS(-2859, -497),  "After Boat Pot 3",                 RHT_POT_SHADOW_TEMPLE,          RG_ARROWS_30,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_AFTER_BOAT_POT_3));
    locationTable[RC_SHADOW_TEMPLE_AFTER_BOAT_POT_4]           =    Location::Pot(RC_SHADOW_TEMPLE_AFTER_BOAT_POT_4,           RCQUEST_VANILLA, RCAREA_SHADOW_TEMPLE,      SCENE_SHADOW_TEMPLE,               TWO_ACTOR_PARAMS(-2624, -492),  "After Boat Pot 4",                 RHT_POT_SHADOW_TEMPLE,          RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_AFTER_BOAT_POT_4));
    locationTable[RC_SPIRIT_TEMPLE_LOBBY_POT_1]                =    Location::Pot(RC_SPIRIT_TEMPLE_LOBBY_POT_1,                RCQUEST_VANILLA, RCAREA_SPIRIT_TEMPLE,      SCENE_SPIRIT_TEMPLE,               TWO_ACTOR_PARAMS(-181, 233),    "Lobby Pot 1",                      RHT_POT_SPIRIT_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_LOBBY_POT_1));
    locationTable[RC_SPIRIT_TEMPLE_LOBBY_POT_2]                =    Location::Pot(RC_SPIRIT_TEMPLE_LOBBY_POT_2,                RCQUEST_VANILLA, RCAREA_SPIRIT_TEMPLE,      SCENE_SPIRIT_TEMPLE,               TWO_ACTOR_PARAMS(184, 231),     "Lobby Pot 2",                      RHT_POT_SPIRIT_TEMPLE,          RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_LOBBY_POT_2));
    locationTable[RC_SPIRIT_TEMPLE_ANUBIS_POT_1]               =    Location::Pot(RC_SPIRIT_TEMPLE_ANUBIS_POT_1,               RCQUEST_VANILLA, RCAREA_SPIRIT_TEMPLE,      SCENE_SPIRIT_TEMPLE,               TWO_ACTOR_PARAMS(-975, -1751),  "Anubis Pot 1",                     RHT_POT_SPIRIT_TEMPLE,          RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_ANUBIS_POT_1));
    locationTable[RC_SPIRIT_TEMPLE_ANUBIS_POT_2]               =    Location::Pot(RC_SPIRIT_TEMPLE_ANUBIS_POT_2,               RCQUEST_VANILLA, RCAREA_SPIRIT_TEMPLE,      SCENE_SPIRIT_TEMPLE,               TWO_ACTOR_PARAMS(-928, -1827),  "Anubis Pot 2",                     RHT_POT_SPIRIT_TEMPLE,          RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_ANUBIS_POT_2));
    locationTable[RC_SPIRIT_TEMPLE_ANUBIS_POT_3]               =    Location::Pot(RC_SPIRIT_TEMPLE_ANUBIS_POT_3,               RCQUEST_VANILLA, RCAREA_SPIRIT_TEMPLE,      SCENE_SPIRIT_TEMPLE,               TWO_ACTOR_PARAMS(-1211, -1530), "Anubis Pot 3",                     RHT_POT_SPIRIT_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_ANUBIS_POT_3));
    locationTable[RC_SPIRIT_TEMPLE_ANUBIS_POT_4]               =    Location::Pot(RC_SPIRIT_TEMPLE_ANUBIS_POT_4,               RCQUEST_VANILLA, RCAREA_SPIRIT_TEMPLE,      SCENE_SPIRIT_TEMPLE,               TWO_ACTOR_PARAMS(-1055, -1680), "Anubis Pot 4",                     RHT_POT_SPIRIT_TEMPLE,          RG_DEKU_SHIELD,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_ANUBIS_POT_4));
    locationTable[RC_SPIRIT_TEMPLE_CHILD_CLIMB_POT_1]          =    Location::Pot(RC_SPIRIT_TEMPLE_CHILD_CLIMB_POT_1,          RCQUEST_VANILLA, RCAREA_SPIRIT_TEMPLE,      SCENE_SPIRIT_TEMPLE,               TWO_ACTOR_PARAMS(-1101, -1305), "Child Climb Pot 1",                RHT_POT_SPIRIT_TEMPLE,          RG_DEKU_SEEDS_30,   SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_CHILD_CLIMB_POT_1));
    locationTable[RC_SPIRIT_TEMPLE_AFTER_SUN_BLOCK_POT_1]      =    Location::Pot(RC_SPIRIT_TEMPLE_AFTER_SUN_BLOCK_POT_1,      RCQUEST_VANILLA, RCAREA_SPIRIT_TEMPLE,      SCENE_SPIRIT_TEMPLE,               TWO_ACTOR_PARAMS(-1781, 830),   "After Sun Block Pot 1",            RHT_POT_SPIRIT_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_AFTER_SUN_BLOCK_POT_1));
    locationTable[RC_SPIRIT_TEMPLE_AFTER_SUN_BLOCK_POT_2]      =    Location::Pot(RC_SPIRIT_TEMPLE_AFTER_SUN_BLOCK_POT_2,      RCQUEST_VANILLA, RCAREA_SPIRIT_TEMPLE,      SCENE_SPIRIT_TEMPLE,               TWO_ACTOR_PARAMS(-1617, 830),   "After Sun Block Pot 2",            RHT_POT_SPIRIT_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_AFTER_SUN_BLOCK_POT_2));
    locationTable[RC_SPIRIT_TEMPLE_CENTRAL_CHAMBER_POT_1]      =    Location::Pot(RC_SPIRIT_TEMPLE_CENTRAL_CHAMBER_POT_1,      RCQUEST_VANILLA, RCAREA_SPIRIT_TEMPLE,      SCENE_SPIRIT_TEMPLE,               TWO_ACTOR_PARAMS(-600, -1213),  "Central Chamber Pot 1",            RHT_POT_SPIRIT_TEMPLE,          RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_CENTRAL_CHAMBER_POT_1));
    locationTable[RC_SPIRIT_TEMPLE_CENTRAL_CHAMBER_POT_2]      =    Location::Pot(RC_SPIRIT_TEMPLE_CENTRAL_CHAMBER_POT_2,      RCQUEST_VANILLA, RCAREA_SPIRIT_TEMPLE,      SCENE_SPIRIT_TEMPLE,               TWO_ACTOR_PARAMS(-564, -1180),  "Central Chamber Pot 2",            RHT_POT_SPIRIT_TEMPLE,          RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_CENTRAL_CHAMBER_POT_2));
    locationTable[RC_SPIRIT_TEMPLE_CENTRAL_CHAMBER_POT_3]      =    Location::Pot(RC_SPIRIT_TEMPLE_CENTRAL_CHAMBER_POT_3,      RCQUEST_VANILLA, RCAREA_SPIRIT_TEMPLE,      SCENE_SPIRIT_TEMPLE,               TWO_ACTOR_PARAMS(-526, -1213),  "Central Chamber Pot 3",            RHT_POT_SPIRIT_TEMPLE,          RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_CENTRAL_CHAMBER_POT_3));
    locationTable[RC_SPIRIT_TEMPLE_CENTRAL_CHAMBER_POT_4]      =    Location::Pot(RC_SPIRIT_TEMPLE_CENTRAL_CHAMBER_POT_4,      RCQUEST_VANILLA, RCAREA_SPIRIT_TEMPLE,      SCENE_SPIRIT_TEMPLE,               TWO_ACTOR_PARAMS(639, -1224),   "Central Chamber Pot 4",            RHT_POT_SPIRIT_TEMPLE,          RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_CENTRAL_CHAMBER_POT_4));
    locationTable[RC_SPIRIT_TEMPLE_CENTRAL_CHAMBER_POT_5]      =    Location::Pot(RC_SPIRIT_TEMPLE_CENTRAL_CHAMBER_POT_5,      RCQUEST_VANILLA, RCAREA_SPIRIT_TEMPLE,      SCENE_SPIRIT_TEMPLE,               TWO_ACTOR_PARAMS(671, -1179),   "Central Chamber Pot 5",            RHT_POT_SPIRIT_TEMPLE,          RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_CENTRAL_CHAMBER_POT_5));
    locationTable[RC_SPIRIT_TEMPLE_CENTRAL_CHAMBER_POT_6]      =    Location::Pot(RC_SPIRIT_TEMPLE_CENTRAL_CHAMBER_POT_6,      RCQUEST_VANILLA, RCAREA_SPIRIT_TEMPLE,      SCENE_SPIRIT_TEMPLE,               TWO_ACTOR_PARAMS(704, -1224),   "Central Chamber Pot 6",            RHT_POT_SPIRIT_TEMPLE,          RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_CENTRAL_CHAMBER_POT_6));
    locationTable[RC_SPIRIT_TEMPLE_BEAMOS_HALL_POT_1]          =    Location::Pot(RC_SPIRIT_TEMPLE_BEAMOS_HALL_POT_1,          RCQUEST_VANILLA, RCAREA_SPIRIT_TEMPLE,      SCENE_SPIRIT_TEMPLE,               TWO_ACTOR_PARAMS(819, -333),    "Beamos Hall Pot 1",                RHT_POT_SPIRIT_TEMPLE,          RG_BOMBS_10,        SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_BEAMOS_HALL_POT_1));
    locationTable[RC_GANONS_CASTLE_FOREST_TRIAL_POT_1]         =    Location::Pot(RC_GANONS_CASTLE_FOREST_TRIAL_POT_1,         RCQUEST_VANILLA, RCAREA_GANONS_CASTLE,      SCENE_INSIDE_GANONS_CASTLE,        TWO_ACTOR_PARAMS(1725, 2336),   "Forest Trial Pot 1",               RHT_POT_GANONS_CASTLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_FOREST_TRIAL_POT_1));
    locationTable[RC_GANONS_CASTLE_FOREST_TRIAL_POT_2]         =    Location::Pot(RC_GANONS_CASTLE_FOREST_TRIAL_POT_2,         RCQUEST_VANILLA, RCAREA_GANONS_CASTLE,      SCENE_INSIDE_GANONS_CASTLE,        TWO_ACTOR_PARAMS(1892, 2241),   "Forest Trial Pot 2",               RHT_POT_GANONS_CASTLE,          RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_FOREST_TRIAL_POT_2));
    locationTable[RC_GANONS_CASTLE_FIRE_TRIAL_POT_2]           =    Location::Pot(RC_GANONS_CASTLE_FIRE_TRIAL_POT_2,           RCQUEST_VANILLA, RCAREA_GANONS_CASTLE,      SCENE_INSIDE_GANONS_CASTLE,        TWO_ACTOR_PARAMS(-1898, -3942), "Fire Trial Pot 2",                 RHT_POT_GANONS_CASTLE,          RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_FIRE_TRIAL_POT_2));
    locationTable[RC_GANONS_CASTLE_FIRE_TRIAL_POT_1]           =    Location::Pot(RC_GANONS_CASTLE_FIRE_TRIAL_POT_1,           RCQUEST_VANILLA, RCAREA_GANONS_CASTLE,      SCENE_INSIDE_GANONS_CASTLE,        TWO_ACTOR_PARAMS(-2051, -3855), "Fire Trial Pot 1",                 RHT_POT_GANONS_CASTLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_FIRE_TRIAL_POT_1));
    locationTable[RC_GANONS_CASTLE_WATER_TRIAL_POT_1]          =    Location::Pot(RC_GANONS_CASTLE_WATER_TRIAL_POT_1,          RCQUEST_VANILLA, RCAREA_GANONS_CASTLE,      SCENE_INSIDE_GANONS_CASTLE,        TWO_ACTOR_PARAMS(3558, -920),   "Water Trial Pot 1",                RHT_POT_GANONS_CASTLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_WATER_TRIAL_POT_1));
    locationTable[RC_GANONS_CASTLE_WATER_TRIAL_POT_2]          =    Location::Pot(RC_GANONS_CASTLE_WATER_TRIAL_POT_2,          RCQUEST_VANILLA, RCAREA_GANONS_CASTLE,      SCENE_INSIDE_GANONS_CASTLE,        TWO_ACTOR_PARAMS(3558, -761),   "Water Trial Pot 2",                RHT_POT_GANONS_CASTLE,          RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_WATER_TRIAL_POT_2));
    locationTable[RC_GANONS_CASTLE_WATER_TRIAL_POT_3]          =    Location::Pot(RC_GANONS_CASTLE_WATER_TRIAL_POT_3,          RCQUEST_VANILLA, RCAREA_GANONS_CASTLE,      SCENE_INSIDE_GANONS_CASTLE,        TWO_ACTOR_PARAMS(2822, -507),   "Water Trial Pot 3",                RHT_POT_GANONS_CASTLE,          RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_WATER_TRIAL_POT_3));
    locationTable[RC_GANONS_CASTLE_SHADOW_TRIAL_POT_1]         =    Location::Pot(RC_GANONS_CASTLE_SHADOW_TRIAL_POT_1,         RCQUEST_VANILLA, RCAREA_GANONS_CASTLE,      SCENE_INSIDE_GANONS_CASTLE,        TWO_ACTOR_PARAMS(1234, -3418),  "Shadow Trial Pot 1",               RHT_POT_GANONS_CASTLE,          RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_SHADOW_TRIAL_POT_1));
    locationTable[RC_GANONS_CASTLE_SHADOW_TRIAL_POT_2]         =    Location::Pot(RC_GANONS_CASTLE_SHADOW_TRIAL_POT_2,         RCQUEST_VANILLA, RCAREA_GANONS_CASTLE,      SCENE_INSIDE_GANONS_CASTLE,        TWO_ACTOR_PARAMS(1621, -3203),  "Shadow Trial Pot 2",               RHT_POT_GANONS_CASTLE,          RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_SHADOW_TRIAL_POT_2));
    locationTable[RC_GANONS_CASTLE_SHADOW_TRIAL_POT_3]         =    Location::Pot(RC_GANONS_CASTLE_SHADOW_TRIAL_POT_3,         RCQUEST_VANILLA, RCAREA_GANONS_CASTLE,      SCENE_INSIDE_GANONS_CASTLE,        TWO_ACTOR_PARAMS(2067, -4592),  "Shadow Trial Pot 3",               RHT_POT_GANONS_CASTLE,          RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_SHADOW_TRIAL_POT_3));
    locationTable[RC_GANONS_CASTLE_SHADOW_TRIAL_POT_4]         =    Location::Pot(RC_GANONS_CASTLE_SHADOW_TRIAL_POT_4,         RCQUEST_VANILLA, RCAREA_GANONS_CASTLE,      SCENE_INSIDE_GANONS_CASTLE,        TWO_ACTOR_PARAMS(2224, -4506),  "Shadow Trial Pot 4",               RHT_POT_GANONS_CASTLE,          RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_SHADOW_TRIAL_POT_4));
    locationTable[RC_GANONS_CASTLE_SPIRIT_TRIAL_POT_1]         =    Location::Pot(RC_GANONS_CASTLE_SPIRIT_TRIAL_POT_1,         RCQUEST_VANILLA, RCAREA_GANONS_CASTLE,      SCENE_INSIDE_GANONS_CASTLE,        TWO_ACTOR_PARAMS(-1398, 1402),  "Spirit Trial Pot 1",               RHT_POT_GANONS_CASTLE,          RG_DEKU_NUTS_5,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_SPIRIT_TRIAL_POT_1));
    locationTable[RC_GANONS_CASTLE_SPIRIT_TRIAL_POT_2]         =    Location::Pot(RC_GANONS_CASTLE_SPIRIT_TRIAL_POT_2,         RCQUEST_VANILLA, RCAREA_GANONS_CASTLE,      SCENE_INSIDE_GANONS_CASTLE,        TWO_ACTOR_PARAMS(-1237, 1494),  "Spirit Trial Pot 2",               RHT_POT_GANONS_CASTLE,          RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_SPIRIT_TRIAL_POT_2));
    locationTable[RC_GANONS_CASTLE_LIGHT_TRIAL_BOULDER_POT_1]  =    Location::Pot(RC_GANONS_CASTLE_LIGHT_TRIAL_BOULDER_POT_1,  RCQUEST_VANILLA, RCAREA_GANONS_CASTLE,      SCENE_INSIDE_GANONS_CASTLE,        TWO_ACTOR_PARAMS(-2581, -839),  "Light Trial Boulder Pot 1",        RHT_POT_GANONS_CASTLE,          RG_ARROWS_30,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_LIGHT_TRIAL_BOULDER_POT_1));
    locationTable[RC_GANONS_CASTLE_LIGHT_TRIAL_POT_1]          =    Location::Pot(RC_GANONS_CASTLE_LIGHT_TRIAL_POT_1,          RCQUEST_VANILLA, RCAREA_GANONS_CASTLE,      SCENE_INSIDE_GANONS_CASTLE,        TWO_ACTOR_PARAMS(-3840, -923),  "Light Trial Pot 1",                RHT_POT_GANONS_CASTLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_LIGHT_TRIAL_POT_1));
    locationTable[RC_GANONS_CASTLE_LIGHT_TRIAL_POT_2]          =    Location::Pot(RC_GANONS_CASTLE_LIGHT_TRIAL_POT_2,          RCQUEST_VANILLA, RCAREA_GANONS_CASTLE,      SCENE_INSIDE_GANONS_CASTLE,        TWO_ACTOR_PARAMS(-3840, -755),  "Light Trial Pot 2",                RHT_POT_GANONS_CASTLE,          RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_LIGHT_TRIAL_POT_2));
    locationTable[RC_GANONS_CASTLE_GANONS_TOWER_POT_1]         =    Location::Pot(RC_GANONS_CASTLE_GANONS_TOWER_POT_1,         RCQUEST_BOTH, RCAREA_GANONS_CASTLE,         SCENE_GANONS_TOWER,                TWO_ACTOR_PARAMS(-422, -258),   "Ganons Tower Pot 1",               RHT_POT_GANONS_CASTLE,          RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_GANONS_TOWER_POT_1));
    locationTable[RC_GANONS_CASTLE_GANONS_TOWER_POT_2]         =    Location::Pot(RC_GANONS_CASTLE_GANONS_TOWER_POT_2,         RCQUEST_BOTH, RCAREA_GANONS_CASTLE,         SCENE_GANONS_TOWER,                TWO_ACTOR_PARAMS(-382, -300),   "Ganons Tower Pot 2",               RHT_POT_GANONS_CASTLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_GANONS_TOWER_POT_2));
    locationTable[RC_GANONS_CASTLE_GANONS_TOWER_POT_3]         =    Location::Pot(RC_GANONS_CASTLE_GANONS_TOWER_POT_3,         RCQUEST_BOTH, RCAREA_GANONS_CASTLE,         SCENE_GANONS_TOWER,                TWO_ACTOR_PARAMS(-342, -341),   "Ganons Tower Pot 3",               RHT_POT_GANONS_CASTLE,          RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_GANONS_TOWER_POT_3));
    locationTable[RC_GANONS_CASTLE_GANONS_TOWER_POT_4]         =    Location::Pot(RC_GANONS_CASTLE_GANONS_TOWER_POT_4,         RCQUEST_BOTH, RCAREA_GANONS_CASTLE,         SCENE_GANONS_TOWER,                TWO_ACTOR_PARAMS(-298, -383),   "Ganons Tower Pot 4",               RHT_POT_GANONS_CASTLE,          RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_GANONS_TOWER_POT_4));
    locationTable[RC_GANONS_CASTLE_GANONS_TOWER_POT_5]         =    Location::Pot(RC_GANONS_CASTLE_GANONS_TOWER_POT_5,         RCQUEST_BOTH, RCAREA_GANONS_CASTLE,         SCENE_GANONS_TOWER,                TWO_ACTOR_PARAMS(-257, -424),   "Ganons Tower Pot 5",               RHT_POT_GANONS_CASTLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_GANONS_TOWER_POT_5));
    locationTable[RC_GANONS_CASTLE_GANONS_TOWER_POT_6]         =    Location::Pot(RC_GANONS_CASTLE_GANONS_TOWER_POT_6,         RCQUEST_BOTH, RCAREA_GANONS_CASTLE,         SCENE_GANONS_TOWER,                TWO_ACTOR_PARAMS(-382, -259),   "Ganons Tower Pot 6",               RHT_POT_GANONS_CASTLE,          RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_GANONS_TOWER_POT_6));
    locationTable[RC_GANONS_CASTLE_GANONS_TOWER_POT_7]         =    Location::Pot(RC_GANONS_CASTLE_GANONS_TOWER_POT_7,         RCQUEST_BOTH, RCAREA_GANONS_CASTLE,         SCENE_GANONS_TOWER,                TWO_ACTOR_PARAMS(-342, -300),   "Ganons Tower Pot 7",               RHT_POT_GANONS_CASTLE,          RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_GANONS_TOWER_POT_7));
    locationTable[RC_GANONS_CASTLE_GANONS_TOWER_POT_8]         =    Location::Pot(RC_GANONS_CASTLE_GANONS_TOWER_POT_8,         RCQUEST_BOTH, RCAREA_GANONS_CASTLE,         SCENE_GANONS_TOWER,                TWO_ACTOR_PARAMS(-301, -341),   "Ganons Tower Pot 8",               RHT_POT_GANONS_CASTLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_GANONS_TOWER_POT_8));
    locationTable[RC_GANONS_CASTLE_GANONS_TOWER_POT_9]         =    Location::Pot(RC_GANONS_CASTLE_GANONS_TOWER_POT_9,         RCQUEST_BOTH, RCAREA_GANONS_CASTLE,         SCENE_GANONS_TOWER,                TWO_ACTOR_PARAMS(-258, -383),   "Ganons Tower Pot 9",               RHT_POT_GANONS_CASTLE,          RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_GANONS_TOWER_POT_9));
    locationTable[RC_GANONS_CASTLE_GANONS_TOWER_POT_10]        =    Location::Pot(RC_GANONS_CASTLE_GANONS_TOWER_POT_10,        RCQUEST_BOTH, RCAREA_GANONS_CASTLE,         SCENE_GANONS_TOWER,                TWO_ACTOR_PARAMS(418, -261),    "Ganons Tower Pot 10",              RHT_POT_GANONS_CASTLE,          RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_GANONS_TOWER_POT_10));
    locationTable[RC_GANONS_CASTLE_GANONS_TOWER_POT_11]        =    Location::Pot(RC_GANONS_CASTLE_GANONS_TOWER_POT_11,        RCQUEST_BOTH, RCAREA_GANONS_CASTLE,         SCENE_GANONS_TOWER,                TWO_ACTOR_PARAMS(377, -301),    "Ganons Tower Pot 11",              RHT_POT_GANONS_CASTLE,          RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_GANONS_TOWER_POT_11));
    locationTable[RC_GANONS_CASTLE_GANONS_TOWER_POT_12]        =    Location::Pot(RC_GANONS_CASTLE_GANONS_TOWER_POT_12,        RCQUEST_BOTH, RCAREA_GANONS_CASTLE,         SCENE_GANONS_TOWER,                TWO_ACTOR_PARAMS(338, -340),    "Ganons Tower Pot 12",              RHT_POT_GANONS_CASTLE,          RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_GANONS_TOWER_POT_12));
    locationTable[RC_GANONS_CASTLE_GANONS_TOWER_POT_13]        =    Location::Pot(RC_GANONS_CASTLE_GANONS_TOWER_POT_13,        RCQUEST_BOTH, RCAREA_GANONS_CASTLE,         SCENE_GANONS_TOWER,                TWO_ACTOR_PARAMS(300, -380),    "Ganons Tower Pot 13",              RHT_POT_GANONS_CASTLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_GANONS_TOWER_POT_13));
    locationTable[RC_GANONS_CASTLE_GANONS_TOWER_POT_14]        =    Location::Pot(RC_GANONS_CASTLE_GANONS_TOWER_POT_14,        RCQUEST_BOTH, RCAREA_GANONS_CASTLE,         SCENE_GANONS_TOWER,                TWO_ACTOR_PARAMS(260, -420),    "Ganons Tower Pot 14",              RHT_POT_GANONS_CASTLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_GANONS_TOWER_POT_14));
    locationTable[RC_GANONS_CASTLE_GANONS_TOWER_POT_15]        =    Location::Pot(RC_GANONS_CASTLE_GANONS_TOWER_POT_15,        RCQUEST_BOTH, RCAREA_GANONS_CASTLE,         SCENE_GANONS_TOWER,                TWO_ACTOR_PARAMS(378, -261),    "Ganons Tower Pot 15",              RHT_POT_GANONS_CASTLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_GANONS_TOWER_POT_15));
    locationTable[RC_GANONS_CASTLE_GANONS_TOWER_POT_16]        =    Location::Pot(RC_GANONS_CASTLE_GANONS_TOWER_POT_16,        RCQUEST_BOTH, RCAREA_GANONS_CASTLE,         SCENE_GANONS_TOWER,                TWO_ACTOR_PARAMS(337, -300),    "Ganons Tower Pot 16",              RHT_POT_GANONS_CASTLE,          RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_GANONS_TOWER_POT_16));
    locationTable[RC_GANONS_CASTLE_GANONS_TOWER_POT_17]        =    Location::Pot(RC_GANONS_CASTLE_GANONS_TOWER_POT_17,        RCQUEST_BOTH, RCAREA_GANONS_CASTLE,         SCENE_GANONS_TOWER,                TWO_ACTOR_PARAMS(299, -340),    "Ganons Tower Pot 17",              RHT_POT_GANONS_CASTLE,          RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_GANONS_TOWER_POT_17));
    locationTable[RC_GANONS_CASTLE_GANONS_TOWER_POT_18]        =    Location::Pot(RC_GANONS_CASTLE_GANONS_TOWER_POT_18,        RCQUEST_BOTH, RCAREA_GANONS_CASTLE,         SCENE_GANONS_TOWER,                TWO_ACTOR_PARAMS(260, -380),    "Ganons Tower Pot 18",              RHT_POT_GANONS_CASTLE,          RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_GANONS_TOWER_POT_18));
    locationTable[RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_1]        =    Location::Pot(RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_1,        RCQUEST_VANILLA, RCAREA_BOTTOM_OF_THE_WELL, SCENE_BOTTOM_OF_THE_WELL,          TWO_ACTOR_PARAMS(-95, -673),    "Basement Pot 1",                   RHT_POT_BOTTOM_OF_THE_WELL,     RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_BASEMENT_POT_1));
    locationTable[RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_2]        =    Location::Pot(RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_2,        RCQUEST_VANILLA, RCAREA_BOTTOM_OF_THE_WELL, SCENE_BOTTOM_OF_THE_WELL,          TWO_ACTOR_PARAMS(75, -598),     "Basement Pot 2",                   RHT_POT_BOTTOM_OF_THE_WELL,     RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_BASEMENT_POT_2));
    locationTable[RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_3]        =    Location::Pot(RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_3,        RCQUEST_VANILLA, RCAREA_BOTTOM_OF_THE_WELL, SCENE_BOTTOM_OF_THE_WELL,          TWO_ACTOR_PARAMS(270, -1151),   "Basement Pot 3",                   RHT_POT_BOTTOM_OF_THE_WELL,     RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_BASEMENT_POT_3));
    locationTable[RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_4]        =    Location::Pot(RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_4,        RCQUEST_VANILLA, RCAREA_BOTTOM_OF_THE_WELL, SCENE_BOTTOM_OF_THE_WELL,          TWO_ACTOR_PARAMS(566, -1254),   "Basement Pot 4",                   RHT_POT_BOTTOM_OF_THE_WELL,     RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_BASEMENT_POT_4));
    locationTable[RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_5]        =    Location::Pot(RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_5,        RCQUEST_VANILLA, RCAREA_BOTTOM_OF_THE_WELL, SCENE_BOTTOM_OF_THE_WELL,          TWO_ACTOR_PARAMS(378, -1089),   "Basement Pot 5",                   RHT_POT_BOTTOM_OF_THE_WELL,     RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_BASEMENT_POT_5));
    locationTable[RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_6]        =    Location::Pot(RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_6,        RCQUEST_VANILLA, RCAREA_BOTTOM_OF_THE_WELL, SCENE_BOTTOM_OF_THE_WELL,          TWO_ACTOR_PARAMS(665, -1252),   "Basement Pot 6",                   RHT_POT_BOTTOM_OF_THE_WELL,     RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_BASEMENT_POT_6));
    locationTable[RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_7]        =    Location::Pot(RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_7,        RCQUEST_VANILLA, RCAREA_BOTTOM_OF_THE_WELL, SCENE_BOTTOM_OF_THE_WELL,          TWO_ACTOR_PARAMS(566, -1345),   "Basement Pot 7",                   RHT_POT_BOTTOM_OF_THE_WELL,     RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_BASEMENT_POT_7));
    locationTable[RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_8]        =    Location::Pot(RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_8,        RCQUEST_VANILLA, RCAREA_BOTTOM_OF_THE_WELL, SCENE_BOTTOM_OF_THE_WELL,          TWO_ACTOR_PARAMS(562, -1304),   "Basement Pot 8",                   RHT_POT_BOTTOM_OF_THE_WELL,     RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_BASEMENT_POT_8));
    locationTable[RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_9]        =    Location::Pot(RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_9,        RCQUEST_VANILLA, RCAREA_BOTTOM_OF_THE_WELL, SCENE_BOTTOM_OF_THE_WELL,          TWO_ACTOR_PARAMS(313, -1086),   "Basement Pot 9",                   RHT_POT_BOTTOM_OF_THE_WELL,     RG_DEKU_NUTS_5,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_BASEMENT_POT_9));
    locationTable[RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_10]       =    Location::Pot(RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_10,       RCQUEST_VANILLA, RCAREA_BOTTOM_OF_THE_WELL, SCENE_BOTTOM_OF_THE_WELL,          TWO_ACTOR_PARAMS(618, -1254),   "Basement Pot 10",                  RHT_POT_BOTTOM_OF_THE_WELL,     RG_DEKU_STICK_1,    SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_BASEMENT_POT_10));
    locationTable[RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_11]       =    Location::Pot(RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_11,       RCQUEST_VANILLA, RCAREA_BOTTOM_OF_THE_WELL, SCENE_BOTTOM_OF_THE_WELL,          TWO_ACTOR_PARAMS(257, -1099),   "Basement Pot 11",                  RHT_POT_BOTTOM_OF_THE_WELL,     RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_BASEMENT_POT_11));
    locationTable[RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_12]       =    Location::Pot(RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_12,       RCQUEST_VANILLA, RCAREA_BOTTOM_OF_THE_WELL, SCENE_BOTTOM_OF_THE_WELL,          TWO_ACTOR_PARAMS(296, -1206),   "Basement Pot 12",                  RHT_POT_BOTTOM_OF_THE_WELL,     RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_BASEMENT_POT_12));
    locationTable[RC_BOTTOM_OF_THE_WELL_LEFT_SIDE_POT_1]       =    Location::Pot(RC_BOTTOM_OF_THE_WELL_LEFT_SIDE_POT_1,       RCQUEST_VANILLA, RCAREA_BOTTOM_OF_THE_WELL, SCENE_BOTTOM_OF_THE_WELL,          TWO_ACTOR_PARAMS(-969, 55),     "Left Side Pot 1",                  RHT_POT_BOTTOM_OF_THE_WELL,     RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_LEFT_SIDE_POT_1));
    locationTable[RC_BOTTOM_OF_THE_WELL_LEFT_SIDE_POT_2]       =    Location::Pot(RC_BOTTOM_OF_THE_WELL_LEFT_SIDE_POT_2,       RCQUEST_VANILLA, RCAREA_BOTTOM_OF_THE_WELL, SCENE_BOTTOM_OF_THE_WELL,          TWO_ACTOR_PARAMS(-966, 9),      "Left Side Pot 2",                  RHT_POT_BOTTOM_OF_THE_WELL,     RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_LEFT_SIDE_POT_2));
    locationTable[RC_BOTTOM_OF_THE_WELL_LEFT_SIDE_POT_3]       =    Location::Pot(RC_BOTTOM_OF_THE_WELL_LEFT_SIDE_POT_3,       RCQUEST_VANILLA, RCAREA_BOTTOM_OF_THE_WELL, SCENE_BOTTOM_OF_THE_WELL,          TWO_ACTOR_PARAMS(-965, -32),    "Left Side Pot 3",                  RHT_POT_BOTTOM_OF_THE_WELL,     RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_LEFT_SIDE_POT_3));
    locationTable[RC_BOTTOM_OF_THE_WELL_NEAR_ENTRANCE_POT_1]   =    Location::Pot(RC_BOTTOM_OF_THE_WELL_NEAR_ENTRANCE_POT_1,   RCQUEST_VANILLA, RCAREA_BOTTOM_OF_THE_WELL, SCENE_BOTTOM_OF_THE_WELL,          TWO_ACTOR_PARAMS(-41, 389),     "Near Entrance Pot 1",              RHT_POT_BOTTOM_OF_THE_WELL,     RG_DEKU_STICK_1,    SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_NEAR_ENTRANCE_POT_1));
    locationTable[RC_BOTTOM_OF_THE_WELL_NEAR_ENTRANCE_POT_2]   =    Location::Pot(RC_BOTTOM_OF_THE_WELL_NEAR_ENTRANCE_POT_2,   RCQUEST_VANILLA, RCAREA_BOTTOM_OF_THE_WELL, SCENE_BOTTOM_OF_THE_WELL,          TWO_ACTOR_PARAMS(36, 389),      "Near Entrance Pot 2",              RHT_POT_BOTTOM_OF_THE_WELL,     RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_NEAR_ENTRANCE_POT_2));
    locationTable[RC_BOTTOM_OF_THE_WELL_FIRE_KEESE_POT_1]      =    Location::Pot(RC_BOTTOM_OF_THE_WELL_FIRE_KEESE_POT_1,      RCQUEST_VANILLA, RCAREA_BOTTOM_OF_THE_WELL, SCENE_BOTTOM_OF_THE_WELL,          TWO_ACTOR_PARAMS(874, -1294),   "Fire Keese Pot 1",                 RHT_POT_BOTTOM_OF_THE_WELL,     RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_FIRE_KEESE_POT_1));
    locationTable[RC_BOTTOM_OF_THE_WELL_UNDERWATER_POT]        =    Location::Pot(RC_BOTTOM_OF_THE_WELL_UNDERWATER_POT,        RCQUEST_VANILLA, RCAREA_BOTTOM_OF_THE_WELL, SCENE_BOTTOM_OF_THE_WELL,          TWO_ACTOR_PARAMS(3, -1641),     "Underwater Pot",                   RHT_POT_BOTTOM_OF_THE_WELL,     RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_UNDERWATER_POT));
    locationTable[RC_ICE_CAVERN_HALL_POT_1]                    =    Location::Pot(RC_ICE_CAVERN_HALL_POT_1,                    RCQUEST_VANILLA, RCAREA_ICE_CAVERN,         SCENE_ICE_CAVERN,                  TWO_ACTOR_PARAMS(-206, 449),    "Hall Pot 1",                       RHT_POT_ICE_CAVERN,             RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_ICE_CAVERN_HALL_POT_1));
    locationTable[RC_ICE_CAVERN_HALL_POT_2]                    =    Location::Pot(RC_ICE_CAVERN_HALL_POT_2,                    RCQUEST_VANILLA, RCAREA_ICE_CAVERN,         SCENE_ICE_CAVERN,                  TWO_ACTOR_PARAMS(-203, 492),    "Hall Pot 2",                       RHT_POT_ICE_CAVERN,             RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_ICE_CAVERN_HALL_POT_2));
    locationTable[RC_ICE_CAVERN_SPINNING_BLADE_POT_1]          =    Location::Pot(RC_ICE_CAVERN_SPINNING_BLADE_POT_1,          RCQUEST_VANILLA, RCAREA_ICE_CAVERN,         SCENE_ICE_CAVERN,                  TWO_ACTOR_PARAMS(433, -732),    "Spinning Blade Pot 1",             RHT_POT_ICE_CAVERN,             RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_ICE_CAVERN_SPINNING_BLADE_POT_1));
    locationTable[RC_ICE_CAVERN_SPINNING_BLADE_POT_2]          =    Location::Pot(RC_ICE_CAVERN_SPINNING_BLADE_POT_2,          RCQUEST_VANILLA, RCAREA_ICE_CAVERN,         SCENE_ICE_CAVERN,                  TWO_ACTOR_PARAMS(569, -175),    "Spinning Blade Pot 2",             RHT_POT_ICE_CAVERN,             RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_ICE_CAVERN_SPINNING_BLADE_POT_2));
    locationTable[RC_ICE_CAVERN_SPINNING_BLADE_POT_3]          =    Location::Pot(RC_ICE_CAVERN_SPINNING_BLADE_POT_3,          RCQUEST_VANILLA, RCAREA_ICE_CAVERN,         SCENE_ICE_CAVERN,                  TWO_ACTOR_PARAMS(521, -131),    "Spinning Blade Pot 3",             RHT_POT_ICE_CAVERN,             RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_ICE_CAVERN_SPINNING_BLADE_POT_3));
    locationTable[RC_ICE_CAVERN_NEAR_END_POT_1]                =    Location::Pot(RC_ICE_CAVERN_NEAR_END_POT_1,                RCQUEST_VANILLA, RCAREA_ICE_CAVERN,         SCENE_ICE_CAVERN,                  TWO_ACTOR_PARAMS(-1422, 586),   "Near End Pot 1",                   RHT_POT_ICE_CAVERN,             RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_ICE_CAVERN_NEAR_END_POT_1));
    locationTable[RC_ICE_CAVERN_NEAR_END_POT_2]                =    Location::Pot(RC_ICE_CAVERN_NEAR_END_POT_2,                RCQUEST_VANILLA, RCAREA_ICE_CAVERN,         SCENE_ICE_CAVERN,                  TWO_ACTOR_PARAMS(-1488, 676),   "Near End Pot 2",                   RHT_POT_ICE_CAVERN,             RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_ICE_CAVERN_NEAR_END_POT_2));
    locationTable[RC_ICE_CAVERN_FROZEN_POT_1]                  =    Location::Pot(RC_ICE_CAVERN_FROZEN_POT_1,                  RCQUEST_VANILLA, RCAREA_ICE_CAVERN,         SCENE_ICE_CAVERN,                  TWO_ACTOR_PARAMS(366, -2036),   "Frozen Pot 1",                     RHT_POT_ICE_CAVERN,             RG_PURPLE_RUPEE,    SpoilerCollectionCheck::RandomizerInf(RAND_INF_ICE_CAVERN_FROZEN_POT_1));

    // MQ Dungeon Pots
    //            Randomizer Check                                    Randomizer Check                                                  Quest       Area                            Scene ID                        Params                          Short Name                      Hint Text Key                   Vanilla             Spoiler Collection Check
    locationTable[RC_DODONGOS_CAVERN_MQ_STAIRCASE_POT_1]            = Location::Pot(RC_DODONGOS_CAVERN_MQ_STAIRCASE_POT_1,              RCQUEST_MQ, RCAREA_DODONGOS_CAVERN,         SCENE_DODONGOS_CAVERN,          TWO_ACTOR_PARAMS(-2391, -1804), "MQ Staircase Pot 1",           RHT_POT_DODONGOS_CAVERN,        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_STAIRCASE_POT_1));
    locationTable[RC_DODONGOS_CAVERN_MQ_STAIRCASE_POT_2]            = Location::Pot(RC_DODONGOS_CAVERN_MQ_STAIRCASE_POT_2,              RCQUEST_MQ, RCAREA_DODONGOS_CAVERN,         SCENE_DODONGOS_CAVERN,          TWO_ACTOR_PARAMS(-2361, -1161), "MQ Staircase Pot 2",           RHT_POT_DODONGOS_CAVERN,        RG_RED_RUPEE,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_STAIRCASE_POT_2));
    locationTable[RC_DODONGOS_CAVERN_MQ_STAIRCASE_POT_3]            = Location::Pot(RC_DODONGOS_CAVERN_MQ_STAIRCASE_POT_3,              RCQUEST_MQ, RCAREA_DODONGOS_CAVERN,         SCENE_DODONGOS_CAVERN,          TWO_ACTOR_PARAMS(-1525, -1800), "MQ Staircase Pot 3",           RHT_POT_DODONGOS_CAVERN,        RG_RED_RUPEE,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_STAIRCASE_POT_3));
    locationTable[RC_DODONGOS_CAVERN_MQ_STAIRCASE_POT_4]            = Location::Pot(RC_DODONGOS_CAVERN_MQ_STAIRCASE_POT_4,              RCQUEST_MQ, RCAREA_DODONGOS_CAVERN,         SCENE_DODONGOS_CAVERN,          TWO_ACTOR_PARAMS(-1558, -1169), "MQ Staircase Pot 4",           RHT_POT_DODONGOS_CAVERN,        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_STAIRCASE_POT_4));
    locationTable[RC_DODONGOS_CAVERN_MQ_TORCH_PUZZLE_MIDDLE_POT]    = Location::Pot(RC_DODONGOS_CAVERN_MQ_TORCH_PUZZLE_MIDDLE_POT,      RCQUEST_MQ, RCAREA_DODONGOS_CAVERN,         SCENE_DODONGOS_CAVERN,          TWO_ACTOR_PARAMS(1532, -1225),  "MQ Torch Puzzle Middle Pot",   RHT_POT_DODONGOS_CAVERN,        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_BLOCK_ROOM_POT_2));
    locationTable[RC_DODONGOS_CAVERN_MQ_BIG_BLOCK_POT_1]            = Location::Pot(RC_DODONGOS_CAVERN_MQ_BIG_BLOCK_POT_1,              RCQUEST_MQ, RCAREA_DODONGOS_CAVERN,         SCENE_DODONGOS_CAVERN,          TWO_ACTOR_PARAMS(3195, -155),   "MQ Big Block Pot 1",           RHT_POT_DODONGOS_CAVERN,        RG_DEKU_SEEDS_30,   SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_SILVER_BLOCK_POT_1));
    locationTable[RC_DODONGOS_CAVERN_MQ_BIG_BLOCK_POT_2]            = Location::Pot(RC_DODONGOS_CAVERN_MQ_BIG_BLOCK_POT_2,              RCQUEST_MQ, RCAREA_DODONGOS_CAVERN,         SCENE_DODONGOS_CAVERN,          TWO_ACTOR_PARAMS(2704, -483),   "MQ Big Block Pot 2",           RHT_POT_DODONGOS_CAVERN,        RG_DEKU_SEEDS_30,   SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_SILVER_BLOCK_POT_2));
    locationTable[RC_DODONGOS_CAVERN_MQ_UPPER_LIZALFOS_POT_1]       = Location::Pot(RC_DODONGOS_CAVERN_MQ_UPPER_LIZALFOS_POT_1,         RCQUEST_MQ, RCAREA_DODONGOS_CAVERN,         SCENE_DODONGOS_CAVERN,          TWO_ACTOR_PARAMS(3869, -811),   "MQ Upper Lizalfos Pot 1",      RHT_POT_DODONGOS_CAVERN,        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_UPPER_LIZALFOS_POT_1));
    locationTable[RC_DODONGOS_CAVERN_MQ_UPPER_LIZALFOS_POT_2]       = Location::Pot(RC_DODONGOS_CAVERN_MQ_UPPER_LIZALFOS_POT_2,         RCQUEST_MQ, RCAREA_DODONGOS_CAVERN,         SCENE_DODONGOS_CAVERN,          TWO_ACTOR_PARAMS(3867, -969),   "MQ Upper Lizalfos Pot 2",      RHT_POT_DODONGOS_CAVERN,        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_UPPER_LIZALFOS_POT_2));
    locationTable[RC_DODONGOS_CAVERN_MQ_UPPER_LIZALFOS_POT_3]       = Location::Pot(RC_DODONGOS_CAVERN_MQ_UPPER_LIZALFOS_POT_3,         RCQUEST_MQ, RCAREA_DODONGOS_CAVERN,         SCENE_DODONGOS_CAVERN,          TWO_ACTOR_PARAMS(4687, -1071),  "MQ Upper Lizalfos Pot 3",      RHT_POT_DODONGOS_CAVERN,        RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_UPPER_LIZALFOS_POT_3));
    locationTable[RC_DODONGOS_CAVERN_MQ_UPPER_LIZALFOS_POT_4]       = Location::Pot(RC_DODONGOS_CAVERN_MQ_UPPER_LIZALFOS_POT_4,         RCQUEST_MQ, RCAREA_DODONGOS_CAVERN,         SCENE_DODONGOS_CAVERN,          TWO_ACTOR_PARAMS(4693, -1194),  "MQ Upper Lizalfos Pot 4",      RHT_POT_DODONGOS_CAVERN,        RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_UPPER_LIZALFOS_POT_4));
    locationTable[RC_DODONGOS_CAVERN_MQ_TWO_FLAMES_POT_1]           = Location::Pot(RC_DODONGOS_CAVERN_MQ_TWO_FLAMES_POT_1,             RCQUEST_MQ, RCAREA_DODONGOS_CAVERN,         SCENE_DODONGOS_CAVERN,          TWO_ACTOR_PARAMS(3327, -805),   "MQ Two Flames Pot 1",          RHT_POT_DODONGOS_CAVERN,        RG_DEKU_SEEDS_30,   SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_TWO_FLAMES_POT_1));
    locationTable[RC_DODONGOS_CAVERN_MQ_TWO_FLAMES_POT_2]           = Location::Pot(RC_DODONGOS_CAVERN_MQ_TWO_FLAMES_POT_2,             RCQUEST_MQ, RCAREA_DODONGOS_CAVERN,         SCENE_DODONGOS_CAVERN,          TWO_ACTOR_PARAMS(3326, -753),   "MQ Two Flames Pot 2",          RHT_POT_DODONGOS_CAVERN,        RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_TWO_FLAMES_POT_2));
    locationTable[RC_DODONGOS_CAVERN_MQ_TORCH_PUZZLE_CORNER_POT]    = Location::Pot(RC_DODONGOS_CAVERN_MQ_TORCH_PUZZLE_CORNER_POT,      RCQUEST_MQ, RCAREA_DODONGOS_CAVERN,         SCENE_DODONGOS_CAVERN,          TWO_ACTOR_PARAMS(1827, -587),   "MQ Torch Puzzle Corner Pot",   RHT_POT_DODONGOS_CAVERN,        RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_BLOCK_ROOM_POT_1));
    locationTable[RC_DODONGOS_CAVERN_MQ_RIGHT_SIDE_POT_1]           = Location::Pot(RC_DODONGOS_CAVERN_MQ_RIGHT_SIDE_POT_1,             RCQUEST_MQ, RCAREA_DODONGOS_CAVERN,         SCENE_DODONGOS_CAVERN,          TWO_ACTOR_PARAMS(1727, -151),   "MQ Right Side Pot 1",          RHT_POT_DODONGOS_CAVERN,        RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_RIGHT_SIDE_POT_1));
    locationTable[RC_DODONGOS_CAVERN_MQ_RIGHT_SIDE_POT_2]           = Location::Pot(RC_DODONGOS_CAVERN_MQ_RIGHT_SIDE_POT_2,             RCQUEST_MQ, RCAREA_DODONGOS_CAVERN,         SCENE_DODONGOS_CAVERN,          TWO_ACTOR_PARAMS(1815, -154),   "MQ Right Side Pot 2",          RHT_POT_DODONGOS_CAVERN,        RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_RIGHT_SIDE_POT_2));
    locationTable[RC_DODONGOS_CAVERN_MQ_RIGHT_SIDE_POT_3]           = Location::Pot(RC_DODONGOS_CAVERN_MQ_RIGHT_SIDE_POT_3,             RCQUEST_MQ, RCAREA_DODONGOS_CAVERN,         SCENE_DODONGOS_CAVERN,          TWO_ACTOR_PARAMS(3442, -469),   "MQ Right Side Pot 3",          RHT_POT_DODONGOS_CAVERN,        RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_RIGHT_SIDE_POT_3));
    locationTable[RC_DODONGOS_CAVERN_MQ_RIGHT_SIDE_POT_4]           = Location::Pot(RC_DODONGOS_CAVERN_MQ_RIGHT_SIDE_POT_4,             RCQUEST_MQ, RCAREA_DODONGOS_CAVERN,         SCENE_DODONGOS_CAVERN,          TWO_ACTOR_PARAMS(3682, -466),   "MQ Right Side Pot 4",          RHT_POT_DODONGOS_CAVERN,        RG_DEKU_SEEDS_30,   SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_RIGHT_SIDE_POT_4));
    locationTable[RC_DODONGOS_CAVERN_MQ_POE_ROOM_POT_1]             = Location::Pot(RC_DODONGOS_CAVERN_MQ_POE_ROOM_POT_1,               RCQUEST_MQ, RCAREA_DODONGOS_CAVERN,         SCENE_DODONGOS_CAVERN,          TWO_ACTOR_PARAMS(1844, -1867),  "MQ Poe Room Pot 1",            RHT_POT_DODONGOS_CAVERN,        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_POE_ROOM_POT_1));
    locationTable[RC_DODONGOS_CAVERN_MQ_POE_ROOM_POT_2]             = Location::Pot(RC_DODONGOS_CAVERN_MQ_POE_ROOM_POT_2,               RCQUEST_MQ, RCAREA_DODONGOS_CAVERN,         SCENE_DODONGOS_CAVERN,          TWO_ACTOR_PARAMS(2272, -2213),  "MQ Poe Room Pot 2",            RHT_POT_DODONGOS_CAVERN,        RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_POE_ROOM_POT_2));
    locationTable[RC_DODONGOS_CAVERN_MQ_POE_ROOM_POT_3]             = Location::Pot(RC_DODONGOS_CAVERN_MQ_POE_ROOM_POT_3,               RCQUEST_MQ, RCAREA_DODONGOS_CAVERN,         SCENE_DODONGOS_CAVERN,          TWO_ACTOR_PARAMS(2240, -1369),  "MQ Poe Room Pot 3",            RHT_POT_DODONGOS_CAVERN,        RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_POE_ROOM_POT_3));
    locationTable[RC_DODONGOS_CAVERN_MQ_POE_ROOM_POT_4]             = Location::Pot(RC_DODONGOS_CAVERN_MQ_POE_ROOM_POT_4,               RCQUEST_MQ, RCAREA_DODONGOS_CAVERN,         SCENE_DODONGOS_CAVERN,          TWO_ACTOR_PARAMS(1535, -1299),  "MQ Poe Room Pot 4",            RHT_POT_DODONGOS_CAVERN,        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_POE_ROOM_POT_4));
    locationTable[RC_DODONGOS_CAVERN_MQ_BEFORE_BOSS_SW_POT]         = Location::Pot(RC_DODONGOS_CAVERN_MQ_BEFORE_BOSS_SW_POT,           RCQUEST_MQ, RCAREA_DODONGOS_CAVERN,         SCENE_DODONGOS_CAVERN,          TWO_ACTOR_PARAMS(258, -3055),   "MQ Before Boss SW Pot",        RHT_POT_DODONGOS_CAVERN,        RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_BEFORE_BOSS_POT_1));
    locationTable[RC_DODONGOS_CAVERN_MQ_BEFORE_BOSS_NE_POT]         = Location::Pot(RC_DODONGOS_CAVERN_MQ_BEFORE_BOSS_NE_POT,           RCQUEST_MQ, RCAREA_DODONGOS_CAVERN,         SCENE_DODONGOS_CAVERN,          TWO_ACTOR_PARAMS(-265, -2499),  "MQ Before Boss NE Pot",        RHT_POT_DODONGOS_CAVERN,        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_BEFORE_BOSS_POT_2));
    locationTable[RC_DODONGOS_CAVERN_MQ_ARMOS_ROOM_SE_POT]          = Location::Pot(RC_DODONGOS_CAVERN_MQ_ARMOS_ROOM_SE_POT,            RCQUEST_MQ, RCAREA_DODONGOS_CAVERN,         SCENE_DODONGOS_CAVERN,          TWO_ACTOR_PARAMS(1758, -3194),  "MQ Armos Room SE Pot",         RHT_POT_DODONGOS_CAVERN,        RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_ARMOS_POT_3));
    locationTable[RC_DODONGOS_CAVERN_MQ_ARMOS_ROOM_SW_POT]          = Location::Pot(RC_DODONGOS_CAVERN_MQ_ARMOS_ROOM_SW_POT,            RCQUEST_MQ, RCAREA_DODONGOS_CAVERN,         SCENE_DODONGOS_CAVERN,          TWO_ACTOR_PARAMS(1223, -3197),  "MQ Armos Room SW Pot",         RHT_POT_DODONGOS_CAVERN,        RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_ARMOS_POT_4));
    locationTable[RC_DODONGOS_CAVERN_MQ_BACKROOM_POT_1]             = Location::Pot(RC_DODONGOS_CAVERN_MQ_BACKROOM_POT_1,               RCQUEST_MQ, RCAREA_DODONGOS_CAVERN,         SCENE_DODONGOS_CAVERN,          TWO_ACTOR_PARAMS(1663, -4311),  "MQ Backroom Pot 1",            RHT_POT_DODONGOS_CAVERN,        RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_BACKROOM_POT_1));
    locationTable[RC_DODONGOS_CAVERN_MQ_BACKROOM_POT_2]             = Location::Pot(RC_DODONGOS_CAVERN_MQ_BACKROOM_POT_2,               RCQUEST_MQ, RCAREA_DODONGOS_CAVERN,         SCENE_DODONGOS_CAVERN,          TWO_ACTOR_PARAMS(1648, -4694),  "MQ Backroom Pot 2",            RHT_POT_DODONGOS_CAVERN,        RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_BACKROOM_POT_2));
    locationTable[RC_DODONGOS_CAVERN_MQ_ARMOS_ROOM_NW_POT]          = Location::Pot(RC_DODONGOS_CAVERN_MQ_ARMOS_ROOM_NW_POT,            RCQUEST_MQ, RCAREA_DODONGOS_CAVERN,         SCENE_DODONGOS_CAVERN,          TWO_ACTOR_PARAMS(1005, -3711),  "MQ Armos Room NW Pot",         RHT_POT_DODONGOS_CAVERN,        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_ARMOS_POT_1));
    locationTable[RC_DODONGOS_CAVERN_MQ_ARMOS_ROOM_NE_POT]          = Location::Pot(RC_DODONGOS_CAVERN_MQ_ARMOS_ROOM_NE_POT,            RCQUEST_MQ, RCAREA_DODONGOS_CAVERN,         SCENE_DODONGOS_CAVERN,          TWO_ACTOR_PARAMS(1066, -3711),  "MQ Armos Room NE Pot",         RHT_POT_DODONGOS_CAVERN,        RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_ARMOS_POT_2));
    locationTable[RC_JABU_JABUS_BELLY_MQ_ENTRANCE_POT_1]            = Location::Pot(RC_JABU_JABUS_BELLY_MQ_ENTRANCE_POT_1,              RCQUEST_MQ, RCAREA_JABU_JABUS_BELLY,        SCENE_JABU_JABU,                TWO_ACTOR_PARAMS(-183, -449),   "MQ Entrance Pot 1",            RHT_POT_JABU_JABUS_BELLY,       RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_MQ_ENTRANCE_POT_1));
    locationTable[RC_JABU_JABUS_BELLY_MQ_ENTRANCE_POT_2]            = Location::Pot(RC_JABU_JABUS_BELLY_MQ_ENTRANCE_POT_2,              RCQUEST_MQ, RCAREA_JABU_JABUS_BELLY,        SCENE_JABU_JABU,                TWO_ACTOR_PARAMS(202, -62),     "MQ Entrance Pot 2",            RHT_POT_JABU_JABUS_BELLY,       RG_DEKU_NUTS_5,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_MQ_ENTRANCE_POT_2));
    locationTable[RC_JABU_JABUS_BELLY_MQ_GEYSER_POT_1]              = Location::Pot(RC_JABU_JABUS_BELLY_MQ_GEYSER_POT_1,                RCQUEST_MQ, RCAREA_JABU_JABUS_BELLY,        SCENE_JABU_JABU,                TWO_ACTOR_PARAMS(-689, -1800),  "MQ Geyser Pot 1",              RHT_POT_JABU_JABUS_BELLY,       RG_ARROWS_5,        SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_MQ_GEYSER_POT_1));
    locationTable[RC_JABU_JABUS_BELLY_MQ_GEYSER_POT_2]              = Location::Pot(RC_JABU_JABUS_BELLY_MQ_GEYSER_POT_2,                RCQUEST_MQ, RCAREA_JABU_JABUS_BELLY,        SCENE_JABU_JABU,                TWO_ACTOR_PARAMS(-693, -1614),  "MQ Geyser Pot 2",              RHT_POT_JABU_JABUS_BELLY,       RG_DEKU_NUTS_5,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_MQ_GEYSER_POT_2));
    locationTable[RC_JABU_JABUS_BELLY_MQ_TIME_BLOCK_POT_1]          = Location::Pot(RC_JABU_JABUS_BELLY_MQ_TIME_BLOCK_POT_1,            RCQUEST_MQ, RCAREA_JABU_JABUS_BELLY,        SCENE_JABU_JABU,                TWO_ACTOR_PARAMS(697, -2318),   "MQ Time Block Pot 1",          RHT_POT_JABU_JABUS_BELLY,       RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_MQ_TIME_BLOCK_POT_1));
    locationTable[RC_JABU_JABUS_BELLY_MQ_TIME_BLOCK_POT_2]          = Location::Pot(RC_JABU_JABUS_BELLY_MQ_TIME_BLOCK_POT_2,            RCQUEST_MQ, RCAREA_JABU_JABUS_BELLY,        SCENE_JABU_JABU,                TWO_ACTOR_PARAMS(699, -2445),   "MQ Time Block Pot 2",          RHT_POT_JABU_JABUS_BELLY,       RG_ARROWS_5,        SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_MQ_TIME_BLOCK_POT_2));
    locationTable[RC_JABU_JABUS_BELLY_MQ_LIKE_LIKES_POT_1]          = Location::Pot(RC_JABU_JABUS_BELLY_MQ_LIKE_LIKES_POT_1,            RCQUEST_MQ, RCAREA_JABU_JABUS_BELLY,        SCENE_JABU_JABU,                TWO_ACTOR_PARAMS(612, -5932),   "MQ Like Likes Pot 1",          RHT_POT_JABU_JABUS_BELLY,       RG_ARROWS_5,        SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_MQ_LIKE_LIKES_POT_1));
    locationTable[RC_JABU_JABUS_BELLY_MQ_LIKE_LIKES_POT_2]          = Location::Pot(RC_JABU_JABUS_BELLY_MQ_LIKE_LIKES_POT_2,            RCQUEST_MQ, RCAREA_JABU_JABUS_BELLY,        SCENE_JABU_JABU,                TWO_ACTOR_PARAMS(718, -5929),   "MQ Like Likes Pot 2",          RHT_POT_JABU_JABUS_BELLY,       RG_BOMBS_10,        SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_MQ_LIKE_LIKES_POT_2));
    locationTable[RC_JABU_JABUS_BELLY_MQ_BEFORE_BOSS_POT_1]         = Location::Pot(RC_JABU_JABUS_BELLY_MQ_BEFORE_BOSS_POT_1,           RCQUEST_MQ, RCAREA_JABU_JABUS_BELLY,        SCENE_JABU_JABU,                TWO_ACTOR_PARAMS(1305, -2005),  "MQ Before Boss Pot 1",         RHT_POT_JABU_JABUS_BELLY,       RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_MQ_BEFORE_BOSS_POT_1));
    locationTable[RC_FOREST_TEMPLE_MQ_LOBBY_POT_1]                  = Location::Pot(RC_FOREST_TEMPLE_MQ_LOBBY_POT_1,                    RCQUEST_MQ, RCAREA_FOREST_TEMPLE,           SCENE_FOREST_TEMPLE,            TWO_ACTOR_PARAMS(675, -989),    "MQ Lobby Pot 1",               RHT_POT_FOREST_TEMPLE,          RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_MQ_LOBBY_POT_1));
    locationTable[RC_FOREST_TEMPLE_MQ_LOBBY_POT_2]                  = Location::Pot(RC_FOREST_TEMPLE_MQ_LOBBY_POT_2,                    RCQUEST_MQ, RCAREA_FOREST_TEMPLE,           SCENE_FOREST_TEMPLE,            TWO_ACTOR_PARAMS(-429, -981),   "MQ Lobby Pot 2",               RHT_POT_FOREST_TEMPLE,          RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_MQ_LOBBY_POT_2));
    locationTable[RC_FOREST_TEMPLE_MQ_LOBBY_POT_3]                  = Location::Pot(RC_FOREST_TEMPLE_MQ_LOBBY_POT_3,                    RCQUEST_MQ, RCAREA_FOREST_TEMPLE,           SCENE_FOREST_TEMPLE,            TWO_ACTOR_PARAMS(720, -1037),   "MQ Lobby Pot 3",               RHT_POT_FOREST_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_MQ_LOBBY_POT_3));
    locationTable[RC_FOREST_TEMPLE_MQ_LOBBY_POT_4]                  = Location::Pot(RC_FOREST_TEMPLE_MQ_LOBBY_POT_4,                    RCQUEST_MQ, RCAREA_FOREST_TEMPLE,           SCENE_FOREST_TEMPLE,            TWO_ACTOR_PARAMS(-477, -1034),  "MQ Lobby Pot 4",               RHT_POT_FOREST_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_MQ_LOBBY_POT_4));
    locationTable[RC_FOREST_TEMPLE_MQ_LOBBY_POT_5]                  = Location::Pot(RC_FOREST_TEMPLE_MQ_LOBBY_POT_5,                    RCQUEST_MQ, RCAREA_FOREST_TEMPLE,           SCENE_FOREST_TEMPLE,            TWO_ACTOR_PARAMS(630, -943),    "MQ Lobby Pot 5",               RHT_POT_FOREST_TEMPLE,          RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_MQ_LOBBY_POT_5));
    locationTable[RC_FOREST_TEMPLE_MQ_LOBBY_POT_6]                  = Location::Pot(RC_FOREST_TEMPLE_MQ_LOBBY_POT_6,                    RCQUEST_MQ, RCAREA_FOREST_TEMPLE,           SCENE_FOREST_TEMPLE,            TWO_ACTOR_PARAMS(-384, -937),   "MQ Lobby Pot 6",               RHT_POT_FOREST_TEMPLE,          RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_MQ_LOBBY_POT_6));
    locationTable[RC_FOREST_TEMPLE_MQ_WOLFOS_POT_1]                 = Location::Pot(RC_FOREST_TEMPLE_MQ_WOLFOS_POT_1,                   RCQUEST_MQ, RCAREA_FOREST_TEMPLE,           SCENE_FOREST_TEMPLE,            TWO_ACTOR_PARAMS(90, -3583),    "MQ Wolfos Pot 1",              RHT_POT_FOREST_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_MQ_LOWER_STALFOS_POT_1));
    locationTable[RC_FOREST_TEMPLE_MQ_WOLFOS_POT_2]                 = Location::Pot(RC_FOREST_TEMPLE_MQ_WOLFOS_POT_2,                   RCQUEST_MQ, RCAREA_FOREST_TEMPLE,           SCENE_FOREST_TEMPLE,            TWO_ACTOR_PARAMS(142, -3583),   "MQ Wolfos Pot 2",              RHT_POT_FOREST_TEMPLE,          RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_MQ_LOWER_STALFOS_POT_2));
    locationTable[RC_FOREST_TEMPLE_MQ_UPPER_STALFOS_POT_1]          = Location::Pot(RC_FOREST_TEMPLE_MQ_UPPER_STALFOS_POT_1,            RCQUEST_MQ, RCAREA_FOREST_TEMPLE,           SCENE_FOREST_TEMPLE,            TWO_ACTOR_PARAMS(206, -3631),   "MQ Upper Stalfos Pot 1",       RHT_POT_FOREST_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_MQ_UPPER_STALFOS_POT_1));
    locationTable[RC_FOREST_TEMPLE_MQ_UPPER_STALFOS_POT_2]          = Location::Pot(RC_FOREST_TEMPLE_MQ_UPPER_STALFOS_POT_2,            RCQUEST_MQ, RCAREA_FOREST_TEMPLE,           SCENE_FOREST_TEMPLE,            TWO_ACTOR_PARAMS(32, -3012),    "MQ Upper Stalfos Pot 2",       RHT_POT_FOREST_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_MQ_UPPER_STALFOS_POT_2));
    locationTable[RC_FOREST_TEMPLE_MQ_UPPER_STALFOS_POT_3]          = Location::Pot(RC_FOREST_TEMPLE_MQ_UPPER_STALFOS_POT_3,            RCQUEST_MQ, RCAREA_FOREST_TEMPLE,           SCENE_FOREST_TEMPLE,            TWO_ACTOR_PARAMS(30, -3631),    "MQ Upper Stalfos Pot 3",       RHT_POT_FOREST_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_MQ_UPPER_STALFOS_POT_3));
    locationTable[RC_FOREST_TEMPLE_MQ_UPPER_STALFOS_POT_4]          = Location::Pot(RC_FOREST_TEMPLE_MQ_UPPER_STALFOS_POT_4,            RCQUEST_MQ, RCAREA_FOREST_TEMPLE,           SCENE_FOREST_TEMPLE,            TWO_ACTOR_PARAMS(202, -3012),   "MQ Upper Stalfos Pot 4",       RHT_POT_FOREST_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_MQ_UPPER_STALFOS_POT_4));
    locationTable[RC_FOREST_TEMPLE_MQ_BLUE_POE_POT_1]               = Location::Pot(RC_FOREST_TEMPLE_MQ_BLUE_POE_POT_1,                 RCQUEST_MQ, RCAREA_FOREST_TEMPLE,           SCENE_FOREST_TEMPLE,            TWO_ACTOR_PARAMS(830, -3639),   "MQ Blue Poe Pot 1",            RHT_POT_FOREST_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_MQ_BLUE_POE_POT_1));
    locationTable[RC_FOREST_TEMPLE_MQ_BLUE_POE_POT_2]               = Location::Pot(RC_FOREST_TEMPLE_MQ_BLUE_POE_POT_2,                 RCQUEST_MQ, RCAREA_FOREST_TEMPLE,           SCENE_FOREST_TEMPLE,            TWO_ACTOR_PARAMS(941, -3640),   "MQ Blue Poe Pot 2",            RHT_POT_FOREST_TEMPLE,          RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_MQ_BLUE_POE_POT_2));
    locationTable[RC_FOREST_TEMPLE_MQ_BLUE_POE_POT_3]               = Location::Pot(RC_FOREST_TEMPLE_MQ_BLUE_POE_POT_3,                 RCQUEST_MQ, RCAREA_FOREST_TEMPLE,           SCENE_FOREST_TEMPLE,            TWO_ACTOR_PARAMS(883, -3640),   "MQ Blue Poe Pot 3",            RHT_POT_FOREST_TEMPLE,          RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_MQ_BLUE_POE_POT_3));
    locationTable[RC_FOREST_TEMPLE_MQ_GREEN_POE_POT_1]              = Location::Pot(RC_FOREST_TEMPLE_MQ_GREEN_POE_POT_1,                RCQUEST_MQ, RCAREA_FOREST_TEMPLE,           SCENE_FOREST_TEMPLE,            TWO_ACTOR_PARAMS(1647, -1529),  "MQ Green Poe Pot 1",           RHT_POT_FOREST_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_MQ_GREEN_POE_POT_1));
    locationTable[RC_FOREST_TEMPLE_MQ_GREEN_POE_POT_2]              = Location::Pot(RC_FOREST_TEMPLE_MQ_GREEN_POE_POT_2,                RCQUEST_MQ, RCAREA_FOREST_TEMPLE,           SCENE_FOREST_TEMPLE,            TWO_ACTOR_PARAMS(1645, -1588),  "MQ Green Poe Pot 2",           RHT_POT_FOREST_TEMPLE,          RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_MQ_GREEN_POE_POT_2));
    locationTable[RC_FOREST_TEMPLE_MQ_BASEMENT_POT_1]               = Location::Pot(RC_FOREST_TEMPLE_MQ_BASEMENT_POT_1,                 RCQUEST_MQ, RCAREA_FOREST_TEMPLE,           SCENE_FOREST_TEMPLE,            TWO_ACTOR_PARAMS(-461, -1095),  "MQ Basement Pot 1",            RHT_POT_FOREST_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_MQ_BASEMENT_POT_1));
    locationTable[RC_FOREST_TEMPLE_MQ_BASEMENT_POT_2]               = Location::Pot(RC_FOREST_TEMPLE_MQ_BASEMENT_POT_2,                 RCQUEST_MQ, RCAREA_FOREST_TEMPLE,           SCENE_FOREST_TEMPLE,            TWO_ACTOR_PARAMS(-351, -979),   "MQ Basement Pot 2",            RHT_POT_FOREST_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_MQ_BASEMENT_POT_2));
    locationTable[RC_FOREST_TEMPLE_MQ_BASEMENT_POT_3]               = Location::Pot(RC_FOREST_TEMPLE_MQ_BASEMENT_POT_3,                 RCQUEST_MQ, RCAREA_FOREST_TEMPLE,           SCENE_FOREST_TEMPLE,            TWO_ACTOR_PARAMS(-387, -1018),  "MQ Basement Pot 3",            RHT_POT_FOREST_TEMPLE,          RG_BOMBS_10,        SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_MQ_BASEMENT_POT_3));
    locationTable[RC_FOREST_TEMPLE_MQ_BASEMENT_POT_4]               = Location::Pot(RC_FOREST_TEMPLE_MQ_BASEMENT_POT_4,                 RCQUEST_MQ, RCAREA_FOREST_TEMPLE,           SCENE_FOREST_TEMPLE,            TWO_ACTOR_PARAMS(-424, -1057),  "MQ Basement Pot 4",            RHT_POT_FOREST_TEMPLE,          RG_ARROWS_5,        SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_MQ_BASEMENT_POT_4));
    locationTable[RC_GANONS_CASTLE_MQ_FOREST_TRIAL_POT_1]           = Location::Pot(RC_GANONS_CASTLE_MQ_FOREST_TRIAL_POT_1,             RCQUEST_MQ, RCAREA_GANONS_CASTLE,           SCENE_INSIDE_GANONS_CASTLE,     TWO_ACTOR_PARAMS(1873, 2252),   "MQ Forest Trial Pot 1",        RHT_POT_GANONS_CASTLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_MQ_FOREST_TRIAL_POT_1));
    locationTable[RC_GANONS_CASTLE_MQ_FOREST_TRIAL_POT_2]           = Location::Pot(RC_GANONS_CASTLE_MQ_FOREST_TRIAL_POT_2,             RCQUEST_MQ, RCAREA_GANONS_CASTLE,           SCENE_INSIDE_GANONS_CASTLE,     TWO_ACTOR_PARAMS(1726, 2326),   "MQ Forest Trial Pot 2",        RHT_POT_GANONS_CASTLE,          RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_MQ_FOREST_TRIAL_POT_2));
    locationTable[RC_GANONS_CASTLE_MQ_WATER_TRIAL_POT_1]            = Location::Pot(RC_GANONS_CASTLE_MQ_WATER_TRIAL_POT_1,              RCQUEST_MQ, RCAREA_GANONS_CASTLE,           SCENE_INSIDE_GANONS_CASTLE,     TWO_ACTOR_PARAMS(3544, -931),   "MQ Water Trial Pot 1",         RHT_POT_GANONS_CASTLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_MQ_WATER_TRIAL_POT_1));
    locationTable[RC_GANONS_CASTLE_MQ_WATER_TRIAL_POT_2]            = Location::Pot(RC_GANONS_CASTLE_MQ_WATER_TRIAL_POT_2,              RCQUEST_MQ, RCAREA_GANONS_CASTLE,           SCENE_INSIDE_GANONS_CASTLE,     TWO_ACTOR_PARAMS(3541, -755),   "MQ Water Trial Pot 2",         RHT_POT_GANONS_CASTLE,          RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_MQ_WATER_TRIAL_POT_2));
    locationTable[RC_GANONS_CASTLE_MQ_SHADOW_TRIAL_POT_1]           = Location::Pot(RC_GANONS_CASTLE_MQ_SHADOW_TRIAL_POT_1,             RCQUEST_MQ, RCAREA_GANONS_CASTLE,           SCENE_INSIDE_GANONS_CASTLE,     TWO_ACTOR_PARAMS(2073, -4592),  "MQ Shadow Trial Pot 1",        RHT_POT_GANONS_CASTLE,          RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_MQ_SHADOW_TRIAL_POT_1));
    locationTable[RC_GANONS_CASTLE_MQ_SHADOW_TRIAL_POT_2]           = Location::Pot(RC_GANONS_CASTLE_MQ_SHADOW_TRIAL_POT_2,             RCQUEST_MQ, RCAREA_GANONS_CASTLE,           SCENE_INSIDE_GANONS_CASTLE,     TWO_ACTOR_PARAMS(2217, -4516),  "MQ Shadow Trial Pot 2",        RHT_POT_GANONS_CASTLE,          RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_MQ_SHADOW_TRIAL_POT_2));
    locationTable[RC_GANONS_CASTLE_MQ_FIRE_TRIAL_POT_1]             = Location::Pot(RC_GANONS_CASTLE_MQ_FIRE_TRIAL_POT_1,               RCQUEST_MQ, RCAREA_GANONS_CASTLE,           SCENE_INSIDE_GANONS_CASTLE,     TWO_ACTOR_PARAMS(-1918, -3957), "MQ Fire Trial Pot 1",          RHT_POT_GANONS_CASTLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_MQ_FIRE_TRIAL_POT_1));
    locationTable[RC_GANONS_CASTLE_MQ_FIRE_TRIAL_POT_2]             = Location::Pot(RC_GANONS_CASTLE_MQ_FIRE_TRIAL_POT_2,               RCQUEST_MQ, RCAREA_GANONS_CASTLE,           SCENE_INSIDE_GANONS_CASTLE,     TWO_ACTOR_PARAMS(-2052, -3888), "MQ Fire Trial Pot 2",          RHT_POT_GANONS_CASTLE,          RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_MQ_FIRE_TRIAL_POT_2));
    locationTable[RC_GANONS_CASTLE_MQ_LIGHT_TRIAL_POT_1]            = Location::Pot(RC_GANONS_CASTLE_MQ_LIGHT_TRIAL_POT_1,              RCQUEST_MQ, RCAREA_GANONS_CASTLE,           SCENE_INSIDE_GANONS_CASTLE,     TWO_ACTOR_PARAMS(-3842, -759),  "MQ Light Trial Pot 1",         RHT_POT_GANONS_CASTLE,          RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_MQ_LIGHT_TRIAL_POT_1));
    locationTable[RC_GANONS_CASTLE_MQ_LIGHT_TRIAL_POT_2]            = Location::Pot(RC_GANONS_CASTLE_MQ_LIGHT_TRIAL_POT_2,              RCQUEST_MQ, RCAREA_GANONS_CASTLE,           SCENE_INSIDE_GANONS_CASTLE,     TWO_ACTOR_PARAMS(-3845, -933),  "MQ Light Trial Pot 2",         RHT_POT_GANONS_CASTLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_MQ_LIGHT_TRIAL_POT_2));
    locationTable[RC_GANONS_CASTLE_MQ_SPIRIT_TRIAL_POT_1]           = Location::Pot(RC_GANONS_CASTLE_MQ_SPIRIT_TRIAL_POT_1,             RCQUEST_MQ, RCAREA_GANONS_CASTLE,           SCENE_INSIDE_GANONS_CASTLE,     TWO_ACTOR_PARAMS(-1402, 1414),  "MQ Spirit Trial Pot 1",        RHT_POT_GANONS_CASTLE,          RG_DEKU_NUTS_5,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_MQ_SPIRIT_TRIAL_POT_1));
    locationTable[RC_GANONS_CASTLE_MQ_SPIRIT_TRIAL_POT_2]           = Location::Pot(RC_GANONS_CASTLE_MQ_SPIRIT_TRIAL_POT_2,             RCQUEST_MQ, RCAREA_GANONS_CASTLE,           SCENE_INSIDE_GANONS_CASTLE,     TWO_ACTOR_PARAMS(-1256, 1500),  "MQ Spirit Trial Pot 2",        RHT_POT_GANONS_CASTLE,          RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_MQ_SPIRIT_TRIAL_POT_2));
    locationTable[RC_SHADOW_TEMPLE_MQ_WHISPERING_WALLS_POT_1]       = Location::Pot(RC_SHADOW_TEMPLE_MQ_WHISPERING_WALLS_POT_1,         RCQUEST_MQ, RCAREA_SHADOW_TEMPLE,           SCENE_SHADOW_TEMPLE,            TWO_ACTOR_PARAMS(-1045, -130),  "MQ Whispering Walls Pot 1",    RHT_POT_SHADOW_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_MQ_WHISPERING_WALLS_POT_1));
    locationTable[RC_SHADOW_TEMPLE_MQ_WHISPERING_WALLS_POT_2]       = Location::Pot(RC_SHADOW_TEMPLE_MQ_WHISPERING_WALLS_POT_2,         RCQUEST_MQ, RCAREA_SHADOW_TEMPLE,           SCENE_SHADOW_TEMPLE,            TWO_ACTOR_PARAMS(-1045, -240),  "MQ Whispering Walls Pot 2",    RHT_POT_SHADOW_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_MQ_WHISPERING_WALLS_POT_2));
    locationTable[RC_SHADOW_TEMPLE_MQ_ENTRANCE_REDEAD_POT_1]        = Location::Pot(RC_SHADOW_TEMPLE_MQ_ENTRANCE_REDEAD_POT_1,          RCQUEST_MQ, RCAREA_SHADOW_TEMPLE,           SCENE_SHADOW_TEMPLE,            TWO_ACTOR_PARAMS(-657, -949),   "MQ Entrance Redead Pot 1",     RHT_POT_SHADOW_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_MQ_ENTRANCE_REDEAD_POT_1));
    locationTable[RC_SHADOW_TEMPLE_MQ_ENTRANCE_REDEAD_POT_2]        = Location::Pot(RC_SHADOW_TEMPLE_MQ_ENTRANCE_REDEAD_POT_2,          RCQUEST_MQ, RCAREA_SHADOW_TEMPLE,           SCENE_SHADOW_TEMPLE,            TWO_ACTOR_PARAMS(-713, -1016),  "MQ Entrance Redead Pot 2",     RHT_POT_SHADOW_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_MQ_ENTRANCE_REDEAD_POT_2));
    locationTable[RC_SHADOW_TEMPLE_MQ_LOWER_UMBRELLA_WEST_POT]      = Location::Pot(RC_SHADOW_TEMPLE_MQ_LOWER_UMBRELLA_WEST_POT,        RCQUEST_MQ, RCAREA_SHADOW_TEMPLE,           SCENE_SHADOW_TEMPLE,            TWO_ACTOR_PARAMS(614, 3579),    "MQ Lower Umbrella W Pot",      RHT_POT_SHADOW_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_MQ_FALLING_SPIKES_POT_1));
    locationTable[RC_SHADOW_TEMPLE_MQ_LOWER_UMBRELLA_EAST_POT]      = Location::Pot(RC_SHADOW_TEMPLE_MQ_LOWER_UMBRELLA_EAST_POT,        RCQUEST_MQ, RCAREA_SHADOW_TEMPLE,           SCENE_SHADOW_TEMPLE,            TWO_ACTOR_PARAMS(798, 3574),    "MQ Lower Umbrella E Pot",      RHT_POT_SHADOW_TEMPLE,          RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_MQ_FALLING_SPIKES_POT_2));
    locationTable[RC_SHADOW_TEMPLE_MQ_UPPER_UMBRELLA_SOUTH_POT]     = Location::Pot(RC_SHADOW_TEMPLE_MQ_UPPER_UMBRELLA_SOUTH_POT,       RCQUEST_MQ, RCAREA_SHADOW_TEMPLE,           SCENE_SHADOW_TEMPLE,            TWO_ACTOR_PARAMS(1173, 3707),   "MQ Upper Umbrella S Pot",      RHT_POT_SHADOW_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_MQ_FALLING_SPIKES_POT_3));
    locationTable[RC_SHADOW_TEMPLE_MQ_UPPER_UMBRELLA_NORTH_POT]     = Location::Pot(RC_SHADOW_TEMPLE_MQ_UPPER_UMBRELLA_NORTH_POT,       RCQUEST_MQ, RCAREA_SHADOW_TEMPLE,           SCENE_SHADOW_TEMPLE,            TWO_ACTOR_PARAMS(1173, 3605),   "MQ Upper Umbrella N Pot",      RHT_POT_SHADOW_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_MQ_FALLING_SPIKES_POT_4));
    locationTable[RC_SHADOW_TEMPLE_MQ_BEFORE_BOAT_POT_1]            = Location::Pot(RC_SHADOW_TEMPLE_MQ_BEFORE_BOAT_POT_1,              RCQUEST_MQ, RCAREA_SHADOW_TEMPLE,           SCENE_SHADOW_TEMPLE,            TWO_ACTOR_PARAMS(4222, -916),   "MQ Before Boat Pot 1",         RHT_POT_SHADOW_TEMPLE,          RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_MQ_BEFORE_BOAT_POT_1));
    locationTable[RC_SHADOW_TEMPLE_MQ_BEFORE_BOAT_POT_2]            = Location::Pot(RC_SHADOW_TEMPLE_MQ_BEFORE_BOAT_POT_2,              RCQUEST_MQ, RCAREA_SHADOW_TEMPLE,           SCENE_SHADOW_TEMPLE,            TWO_ACTOR_PARAMS(4549, -922),   "MQ Before Boat Pot 2",         RHT_POT_SHADOW_TEMPLE,          RG_DEKU_NUTS_5,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_MQ_BEFORE_BOAT_POT_2));
    locationTable[RC_SHADOW_TEMPLE_MQ_BEFORE_CHASM_WEST_POT]        = Location::Pot(RC_SHADOW_TEMPLE_MQ_BEFORE_CHASM_WEST_POT,          RCQUEST_MQ, RCAREA_SHADOW_TEMPLE,           SCENE_SHADOW_TEMPLE,            TWO_ACTOR_PARAMS(-2400, -1470), "MQ Before Chasm W Pot",        RHT_POT_SHADOW_TEMPLE,          RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_MQ_AFTER_BOAT_POT_1));
    locationTable[RC_SHADOW_TEMPLE_MQ_BEFORE_CHASM_EAST_POT]        = Location::Pot(RC_SHADOW_TEMPLE_MQ_BEFORE_CHASM_EAST_POT,          RCQUEST_MQ, RCAREA_SHADOW_TEMPLE,           SCENE_SHADOW_TEMPLE,            TWO_ACTOR_PARAMS(-2230, -1470), "MQ Before Chasm E Pot",        RHT_POT_SHADOW_TEMPLE,          RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_MQ_AFTER_BOAT_POT_2));
    locationTable[RC_SHADOW_TEMPLE_MQ_AFTER_CHASM_WEST_POT]         = Location::Pot(RC_SHADOW_TEMPLE_MQ_AFTER_CHASM_WEST_POT,           RCQUEST_MQ, RCAREA_SHADOW_TEMPLE,           SCENE_SHADOW_TEMPLE,            TWO_ACTOR_PARAMS(-2859, -497),  "MQ After Chasm W Pot",         RHT_POT_SHADOW_TEMPLE,          RG_ARROWS_30,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_MQ_AFTER_BOAT_POT_3));
    locationTable[RC_SHADOW_TEMPLE_MQ_AFTER_CHASM_EAST_POT]         = Location::Pot(RC_SHADOW_TEMPLE_MQ_AFTER_CHASM_EAST_POT,           RCQUEST_MQ, RCAREA_SHADOW_TEMPLE,           SCENE_SHADOW_TEMPLE,            TWO_ACTOR_PARAMS(-2624, -492),  "MQ After Chasm E Pot",         RHT_POT_SHADOW_TEMPLE,          RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_MQ_AFTER_BOAT_POT_4));
    locationTable[RC_SHADOW_TEMPLE_MQ_SPIKE_BARICADE_POT]           = Location::Pot(RC_SHADOW_TEMPLE_MQ_SPIKE_BARICADE_POT,             RCQUEST_MQ, RCAREA_SHADOW_TEMPLE,           SCENE_SHADOW_TEMPLE,            TWO_ACTOR_PARAMS(-4303, -2657), "MQ Spike Baricade Pot",        RHT_POT_SHADOW_TEMPLE,          RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_MQ_SPIKE_BARICADE_POT));
    locationTable[RC_SHADOW_TEMPLE_MQ_DEAD_HAND_POT_1]              = Location::Pot(RC_SHADOW_TEMPLE_MQ_DEAD_HAND_POT_1,                RCQUEST_MQ, RCAREA_SHADOW_TEMPLE,           SCENE_SHADOW_TEMPLE,            TWO_ACTOR_PARAMS(-4575, -811),  "MQ Dead Hand Pot 1",           RHT_POT_SHADOW_TEMPLE,          RG_BOMBS_10,        SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_MQ_DEAD_HAND_POT_1));
    locationTable[RC_SHADOW_TEMPLE_MQ_DEAD_HAND_POT_2]              = Location::Pot(RC_SHADOW_TEMPLE_MQ_DEAD_HAND_POT_2,                RCQUEST_MQ, RCAREA_SHADOW_TEMPLE,           SCENE_SHADOW_TEMPLE,            TWO_ACTOR_PARAMS(-4036, -811),  "MQ Dead Hand Pot 2",           RHT_POT_SHADOW_TEMPLE,          RG_ARROWS_30,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_MQ_DEAD_HAND_POT_2));
    locationTable[RC_BOTTOM_OF_THE_WELL_MQ_INNER_LOBBY_POT_1]       = Location::Pot(RC_BOTTOM_OF_THE_WELL_MQ_INNER_LOBBY_POT_1,         RCQUEST_MQ, RCAREA_BOTTOM_OF_THE_WELL,      SCENE_BOTTOM_OF_THE_WELL,       TWO_ACTOR_PARAMS(339, -377),    "MQ Inner Lobby Pot 1",         RHT_POT_BOTTOM_OF_THE_WELL,     RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_MQ_INNER_LOBBY_POT_1));
    locationTable[RC_BOTTOM_OF_THE_WELL_MQ_INNER_LOBBY_POT_2]       = Location::Pot(RC_BOTTOM_OF_THE_WELL_MQ_INNER_LOBBY_POT_2,         RCQUEST_MQ, RCAREA_BOTTOM_OF_THE_WELL,      SCENE_BOTTOM_OF_THE_WELL,       TWO_ACTOR_PARAMS(416, -377),    "MQ Inner Lobby Pot 2",         RHT_POT_BOTTOM_OF_THE_WELL,     RG_BOMBS_10,        SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_MQ_INNER_LOBBY_POT_2));
    locationTable[RC_BOTTOM_OF_THE_WELL_MQ_INNER_LOBBY_POT_3]       = Location::Pot(RC_BOTTOM_OF_THE_WELL_MQ_INNER_LOBBY_POT_3,         RCQUEST_MQ, RCAREA_BOTTOM_OF_THE_WELL,      SCENE_BOTTOM_OF_THE_WELL,       TWO_ACTOR_PARAMS(269, -376),    "MQ Inner Lobby Pot 3",         RHT_POT_BOTTOM_OF_THE_WELL,     RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_MQ_INNER_LOBBY_POT_3));
    locationTable[RC_BOTTOM_OF_THE_WELL_MQ_OUTER_LOBBY_POT]         = Location::Pot(RC_BOTTOM_OF_THE_WELL_MQ_OUTER_LOBBY_POT,           RCQUEST_MQ, RCAREA_BOTTOM_OF_THE_WELL,      SCENE_BOTTOM_OF_THE_WELL,       TWO_ACTOR_PARAMS(421, -174),    "MQ Outer Lobby Pot",           RHT_POT_BOTTOM_OF_THE_WELL,     RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_MQ_OUTER_LOBBY_POT));
    locationTable[RC_BOTTOM_OF_THE_WELL_MQ_EAST_INNER_ROOM_POT_1]   = Location::Pot(RC_BOTTOM_OF_THE_WELL_MQ_EAST_INNER_ROOM_POT_1,     RCQUEST_MQ, RCAREA_BOTTOM_OF_THE_WELL,      SCENE_BOTTOM_OF_THE_WELL,       TWO_ACTOR_PARAMS(288, -1240),   "MQ East Inner Pot 1",          RHT_POT_BOTTOM_OF_THE_WELL,     RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_MQ_SOUTH_KEY_POT_1));
    locationTable[RC_BOTTOM_OF_THE_WELL_MQ_EAST_INNER_ROOM_POT_2]   = Location::Pot(RC_BOTTOM_OF_THE_WELL_MQ_EAST_INNER_ROOM_POT_2,     RCQUEST_MQ, RCAREA_BOTTOM_OF_THE_WELL,      SCENE_BOTTOM_OF_THE_WELL,       TWO_ACTOR_PARAMS(438, -1234),   "MQ East Inner Pot 2",          RHT_POT_BOTTOM_OF_THE_WELL,     RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_MQ_SOUTH_KEY_POT_2));
    locationTable[RC_BOTTOM_OF_THE_WELL_MQ_EAST_INNER_ROOM_POT_3]   = Location::Pot(RC_BOTTOM_OF_THE_WELL_MQ_EAST_INNER_ROOM_POT_3,     RCQUEST_MQ, RCAREA_BOTTOM_OF_THE_WELL,      SCENE_BOTTOM_OF_THE_WELL,       TWO_ACTOR_PARAMS(443, -1114),   "MQ East Inner Pot 3",          RHT_POT_BOTTOM_OF_THE_WELL,     RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_MQ_SOUTH_KEY_POT_3));
    locationTable[RC_FIRE_TEMPLE_MQ_ENTRANCE_POT_1]                 = Location::Pot(RC_FIRE_TEMPLE_MQ_ENTRANCE_POT_1,                   RCQUEST_MQ, RCAREA_FIRE_TEMPLE,             SCENE_FIRE_TEMPLE,              TWO_ACTOR_PARAMS(-357, 957),    "MQ Entrance Pot 1",            RHT_POT_FIRE_TEMPLE,            RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_ENTRANCE_POT_1));
    locationTable[RC_FIRE_TEMPLE_MQ_ENTRANCE_POT_2]                 = Location::Pot(RC_FIRE_TEMPLE_MQ_ENTRANCE_POT_2,                   RCQUEST_MQ, RCAREA_FIRE_TEMPLE,             SCENE_FIRE_TEMPLE,              TWO_ACTOR_PARAMS(356, 959),     "MQ Entrance Pot 2",            RHT_POT_FIRE_TEMPLE,            RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_ENTRANCE_POT_2));
    locationTable[RC_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_1]         = Location::Pot(RC_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_1,           RCQUEST_MQ, RCAREA_FIRE_TEMPLE,             SCENE_FIRE_TEMPLE,              TWO_ACTOR_PARAMS(187, -1449),   "MQ Before Mini Boss Pot 1",    RHT_POT_FIRE_TEMPLE,            RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_1));
    locationTable[RC_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_2]         = Location::Pot(RC_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_2,           RCQUEST_MQ, RCAREA_FIRE_TEMPLE,             SCENE_FIRE_TEMPLE,              TWO_ACTOR_PARAMS(188, -1514),   "MQ Before Mini Boss Pot 2",    RHT_POT_FIRE_TEMPLE,            RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_2));
    locationTable[RC_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_3]         = Location::Pot(RC_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_3,           RCQUEST_MQ, RCAREA_FIRE_TEMPLE,             SCENE_FIRE_TEMPLE,              TWO_ACTOR_PARAMS(487, -1450),   "MQ Before Mini Boss Pot 3",    RHT_POT_FIRE_TEMPLE,            RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_3));
    locationTable[RC_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_4]         = Location::Pot(RC_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_4,           RCQUEST_MQ, RCAREA_FIRE_TEMPLE,             SCENE_FIRE_TEMPLE,              TWO_ACTOR_PARAMS(486, -1515),   "MQ Before Mini Boss Pot 4",    RHT_POT_FIRE_TEMPLE,            RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_4));
    locationTable[RC_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_5]         = Location::Pot(RC_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_5,           RCQUEST_MQ, RCAREA_FIRE_TEMPLE,             SCENE_FIRE_TEMPLE,              TWO_ACTOR_PARAMS(38, -1179),    "MQ Before Mini Boss Pot 5",    RHT_POT_FIRE_TEMPLE,            RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_5));
    locationTable[RC_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_6]         = Location::Pot(RC_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_6,           RCQUEST_MQ, RCAREA_FIRE_TEMPLE,             SCENE_FIRE_TEMPLE,              TWO_ACTOR_PARAMS(42, -1298),    "MQ Before Mini Boss Pot 6",    RHT_POT_FIRE_TEMPLE,            RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_6));
    locationTable[RC_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_7]         = Location::Pot(RC_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_7,           RCQUEST_MQ, RCAREA_FIRE_TEMPLE,             SCENE_FIRE_TEMPLE,              TWO_ACTOR_PARAMS(520, -943),    "MQ Before Mini Boss Pot 7",    RHT_POT_FIRE_TEMPLE,            RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_7));
    locationTable[RC_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_8]         = Location::Pot(RC_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_8,           RCQUEST_MQ, RCAREA_FIRE_TEMPLE,             SCENE_FIRE_TEMPLE,              TWO_ACTOR_PARAMS(35, -946),     "MQ Before Mini Boss Pot 8",    RHT_POT_FIRE_TEMPLE,            RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_8));
    locationTable[RC_FIRE_TEMPLE_MQ_OUTSIDE_BOSS_POT_1]             = Location::Pot(RC_FIRE_TEMPLE_MQ_OUTSIDE_BOSS_POT_1,               RCQUEST_MQ, RCAREA_FIRE_TEMPLE,             SCENE_FIRE_TEMPLE,              TWO_ACTOR_PARAMS(-1426, -711),  "MQ Outside Boss Pot 1",        RHT_POT_FIRE_TEMPLE,            RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_OUTSIDE_BOSS_POT_1));
    locationTable[RC_FIRE_TEMPLE_MQ_OUTSIDE_BOSS_POT_2]             = Location::Pot(RC_FIRE_TEMPLE_MQ_OUTSIDE_BOSS_POT_2,               RCQUEST_MQ, RCAREA_FIRE_TEMPLE,             SCENE_FIRE_TEMPLE,              TWO_ACTOR_PARAMS(-1425, -660),  "MQ Outside Boss Pot 2",        RHT_POT_FIRE_TEMPLE,            RG_ARROWS_30,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_OUTSIDE_BOSS_POT_2));
    locationTable[RC_FIRE_TEMPLE_MQ_LAVA_ROOM_NORTH_POT]            = Location::Pot(RC_FIRE_TEMPLE_MQ_LAVA_ROOM_NORTH_POT,              RCQUEST_MQ, RCAREA_FIRE_TEMPLE,             SCENE_FIRE_TEMPLE,              TWO_ACTOR_PARAMS(1646, -1260),  "MQ Lava Room North Pot",       RHT_POT_FIRE_TEMPLE,            RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_LAVA_POT_1));
    locationTable[RC_FIRE_TEMPLE_MQ_LAVA_ROOM_HIGH_POT]             = Location::Pot(RC_FIRE_TEMPLE_MQ_LAVA_ROOM_HIGH_POT,               RCQUEST_MQ, RCAREA_FIRE_TEMPLE,             SCENE_FIRE_TEMPLE,              TWO_ACTOR_PARAMS(2226, -809),   "MQ Lava Room High Pot",        RHT_POT_FIRE_TEMPLE,            RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_LAVA_POT_2));
    locationTable[RC_FIRE_TEMPLE_MQ_LAVA_ROOM_SOUTH_POT]            = Location::Pot(RC_FIRE_TEMPLE_MQ_LAVA_ROOM_SOUTH_POT,              RCQUEST_MQ, RCAREA_FIRE_TEMPLE,             SCENE_FIRE_TEMPLE,              TWO_ACTOR_PARAMS(1634, 1475),   "MQ Lava Room South Pot",       RHT_POT_FIRE_TEMPLE,            RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_LAVA_POT_3));
    locationTable[RC_FIRE_TEMPLE_MQ_LAVA_TORCH_POT_1]               = Location::Pot(RC_FIRE_TEMPLE_MQ_LAVA_TORCH_POT_1,                 RCQUEST_MQ, RCAREA_FIRE_TEMPLE,             SCENE_FIRE_TEMPLE,              TWO_ACTOR_PARAMS(845, -1591),   "MQ Lava Torch Pot 1",          RHT_POT_FIRE_TEMPLE,            RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_LAVA_TORCH_POT_1));
    locationTable[RC_FIRE_TEMPLE_MQ_LAVA_TORCH_POT_2]               = Location::Pot(RC_FIRE_TEMPLE_MQ_LAVA_TORCH_POT_2,                 RCQUEST_MQ, RCAREA_FIRE_TEMPLE,             SCENE_FIRE_TEMPLE,              TWO_ACTOR_PARAMS(901, -1593),   "MQ Lava Torch Pot 2",          RHT_POT_FIRE_TEMPLE,            RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_LAVA_TORCH_POT_2));
    locationTable[RC_FIRE_TEMPLE_MQ_ABOVE_LAVA_POT_1]               = Location::Pot(RC_FIRE_TEMPLE_MQ_ABOVE_LAVA_POT_1,                 RCQUEST_MQ, RCAREA_FIRE_TEMPLE,             SCENE_FIRE_TEMPLE,              TWO_ACTOR_PARAMS(1756, -155),   "MQ Above Lava Pot 1",          RHT_POT_FIRE_TEMPLE,            RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_ABOVE_LAVA_POT_1));
    locationTable[RC_FIRE_TEMPLE_MQ_ABOVE_LAVA_POT_2]               = Location::Pot(RC_FIRE_TEMPLE_MQ_ABOVE_LAVA_POT_2,                 RCQUEST_MQ, RCAREA_FIRE_TEMPLE,             SCENE_FIRE_TEMPLE,              TWO_ACTOR_PARAMS(1819, -82),    "MQ Above Lava Pot 2",          RHT_POT_FIRE_TEMPLE,            RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_ABOVE_LAVA_POT_2));
    locationTable[RC_FIRE_TEMPLE_MQ_ABOVE_LAVA_POT_3]               = Location::Pot(RC_FIRE_TEMPLE_MQ_ABOVE_LAVA_POT_3,                 RCQUEST_MQ, RCAREA_FIRE_TEMPLE,             SCENE_FIRE_TEMPLE,              TWO_ACTOR_PARAMS(1789, -122),   "MQ Above Lava Pot 3",          RHT_POT_FIRE_TEMPLE,            RG_BOMBS_10,        SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_ABOVE_LAVA_POT_3));
    locationTable[RC_FIRE_TEMPLE_MQ_FLAME_WALL_POT_1]               = Location::Pot(RC_FIRE_TEMPLE_MQ_FLAME_WALL_POT_1,                 RCQUEST_MQ, RCAREA_FIRE_TEMPLE,             SCENE_FIRE_TEMPLE,              TWO_ACTOR_PARAMS(1493, 386),    "MQ Flame Wall Pot 1",          RHT_POT_FIRE_TEMPLE,            RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_FLAME_WALL_POT_1));
    locationTable[RC_FIRE_TEMPLE_MQ_FLAME_WALL_POT_2]               = Location::Pot(RC_FIRE_TEMPLE_MQ_FLAME_WALL_POT_2,                 RCQUEST_MQ, RCAREA_FIRE_TEMPLE,             SCENE_FIRE_TEMPLE,              TWO_ACTOR_PARAMS(1445, 445),    "MQ Flame Wall Pot 2",          RHT_POT_FIRE_TEMPLE,            RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_FLAME_WALL_POT_2));
    locationTable[RC_FIRE_TEMPLE_MQ_PAST_FIRE_MAZE_SOUTH_POT]       = Location::Pot(RC_FIRE_TEMPLE_MQ_PAST_FIRE_MAZE_SOUTH_POT,         RCQUEST_MQ, RCAREA_FIRE_TEMPLE,             SCENE_FIRE_TEMPLE,              TWO_ACTOR_PARAMS(-2217, -163),  "MQ Past Fire Maze North Pot",  RHT_POT_FIRE_TEMPLE,            RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_FIRE_MAZE_POT_1));
    locationTable[RC_FIRE_TEMPLE_MQ_PAST_FIRE_MAZE_NORTH_POT]       = Location::Pot(RC_FIRE_TEMPLE_MQ_PAST_FIRE_MAZE_NORTH_POT,         RCQUEST_MQ, RCAREA_FIRE_TEMPLE,             SCENE_FIRE_TEMPLE,              TWO_ACTOR_PARAMS(-2657, -591),  "MQ Past Fire Maze South Pot",  RHT_POT_FIRE_TEMPLE,            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_FIRE_MAZE_POT_2));
    locationTable[RC_FIRE_TEMPLE_MQ_FIRE_MAZE_NORTHMOST_POT]        = Location::Pot(RC_FIRE_TEMPLE_MQ_FIRE_MAZE_NORTHMOST_POT,          RCQUEST_MQ, RCAREA_FIRE_TEMPLE,             SCENE_FIRE_TEMPLE,              TWO_ACTOR_PARAMS(-1849, -1114), "MQ Fire Maze Northmost Pot",   RHT_POT_FIRE_TEMPLE,            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_FIRE_MAZE_POT_3));
    locationTable[RC_FIRE_TEMPLE_MQ_FIRE_MAZE_NORTHWEST_POT]        = Location::Pot(RC_FIRE_TEMPLE_MQ_FIRE_MAZE_NORTHWEST_POT,          RCQUEST_MQ, RCAREA_FIRE_TEMPLE,             SCENE_FIRE_TEMPLE,              TWO_ACTOR_PARAMS(-2110, -646),  "MQ Fire Maze North West Pot",  RHT_POT_FIRE_TEMPLE,            RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_FIRE_MAZE_POT_4));
    locationTable[RC_FIRE_TEMPLE_MQ_SOUTH_FIRE_MAZE_WEST_POT]       = Location::Pot(RC_FIRE_TEMPLE_MQ_SOUTH_FIRE_MAZE_WEST_POT,         RCQUEST_MQ, RCAREA_FIRE_TEMPLE,             SCENE_FIRE_TEMPLE,              TWO_ACTOR_PARAMS(-2319, 838),   "MQ South Fire Maze W Pot",     RHT_POT_FIRE_TEMPLE,            RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_FIRE_MAZE_POT_5));
    locationTable[RC_FIRE_TEMPLE_MQ_SOUTH_FIRE_MAZE_EAST_POT]       = Location::Pot(RC_FIRE_TEMPLE_MQ_SOUTH_FIRE_MAZE_EAST_POT,         RCQUEST_MQ, RCAREA_FIRE_TEMPLE,             SCENE_FIRE_TEMPLE,              TWO_ACTOR_PARAMS(-2285, 803),   "MQ South Fire Maze E Pot",     RHT_POT_FIRE_TEMPLE,            RG_BOMBS_10,        SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_FIRE_MAZE_POT_6));
    locationTable[RC_ICE_CAVERN_MQ_ENTRANCE_POT]                    = Location::Pot(RC_ICE_CAVERN_MQ_ENTRANCE_POT,                      RCQUEST_MQ, RCAREA_ICE_CAVERN,              SCENE_ICE_CAVERN,               TWO_ACTOR_PARAMS(248, 2287),    "MQ Entrance Pot",              RHT_POT_ICE_CAVERN,             RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_ICE_CAVERN_MQ_ENTRANCE_POT));
    locationTable[RC_ICE_CAVERN_MQ_FIRST_CRYSTAL_POT_1]             = Location::Pot(RC_ICE_CAVERN_MQ_FIRST_CRYSTAL_POT_1,               RCQUEST_MQ, RCAREA_ICE_CAVERN,              SCENE_ICE_CAVERN,               TWO_ACTOR_PARAMS(51, 718),      "MQ First Crystal Pot 1",       RHT_POT_ICE_CAVERN,             RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_ICE_CAVERN_MQ_FIRST_CRYSTAL_POT_1));
    locationTable[RC_ICE_CAVERN_MQ_FIRST_CRYSTAL_POT_2]             = Location::Pot(RC_ICE_CAVERN_MQ_FIRST_CRYSTAL_POT_2,               RCQUEST_MQ, RCAREA_ICE_CAVERN,              SCENE_ICE_CAVERN,               TWO_ACTOR_PARAMS(52, 768),      "MQ First Crystal Pot 2",       RHT_POT_ICE_CAVERN,             RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_ICE_CAVERN_MQ_FIRST_CRYSTAL_POT_2));
    locationTable[RC_ICE_CAVERN_MQ_EARLY_WOLFOS_POT_1]              = Location::Pot(RC_ICE_CAVERN_MQ_EARLY_WOLFOS_POT_1,                RCQUEST_MQ, RCAREA_ICE_CAVERN,              SCENE_ICE_CAVERN,               TWO_ACTOR_PARAMS(433, -732),    "MQ Early Wolfos Pot 1",        RHT_POT_ICE_CAVERN,             RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_ICE_CAVERN_MQ_EARLY_WOLFOS_POT_1));
    locationTable[RC_ICE_CAVERN_MQ_EARLY_WOLFOS_POT_2]              = Location::Pot(RC_ICE_CAVERN_MQ_EARLY_WOLFOS_POT_2,                RCQUEST_MQ, RCAREA_ICE_CAVERN,              SCENE_ICE_CAVERN,               TWO_ACTOR_PARAMS(569, -175),    "MQ Early Wolfos Pot 2",        RHT_POT_ICE_CAVERN,             RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_ICE_CAVERN_MQ_EARLY_WOLFOS_POT_2));
    locationTable[RC_ICE_CAVERN_MQ_EARLY_WOLFOS_POT_3]              = Location::Pot(RC_ICE_CAVERN_MQ_EARLY_WOLFOS_POT_3,                RCQUEST_MQ, RCAREA_ICE_CAVERN,              SCENE_ICE_CAVERN,               TWO_ACTOR_PARAMS(521, -131),    "MQ Early Wolfos Pot 3",        RHT_POT_ICE_CAVERN,             RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_ICE_CAVERN_MQ_EARLY_WOLFOS_POT_3));
    locationTable[RC_ICE_CAVERN_MQ_EARLY_WOLFOS_POT_4]              = Location::Pot(RC_ICE_CAVERN_MQ_EARLY_WOLFOS_POT_4,                RCQUEST_MQ, RCAREA_ICE_CAVERN,              SCENE_ICE_CAVERN,               TWO_ACTOR_PARAMS(138, -672),    "MQ Early Wolfos Pot 4",        RHT_POT_ICE_CAVERN,             RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_ICE_CAVERN_MQ_EARLY_WOLFOS_POT_4));
    locationTable[RC_ICE_CAVERN_MQ_PUSH_BLOCK_POT_1]                = Location::Pot(RC_ICE_CAVERN_MQ_PUSH_BLOCK_POT_1,                  RCQUEST_MQ, RCAREA_ICE_CAVERN,              SCENE_ICE_CAVERN,               TWO_ACTOR_PARAMS(-1352, 639),   "MQ Push Block Pot 1",          RHT_POT_ICE_CAVERN,             RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_ICE_CAVERN_MQ_PUSH_BLOCK_POT_1));
    locationTable[RC_ICE_CAVERN_MQ_PUSH_BLOCK_POT_2]                = Location::Pot(RC_ICE_CAVERN_MQ_PUSH_BLOCK_POT_2,                  RCQUEST_MQ, RCAREA_ICE_CAVERN,              SCENE_ICE_CAVERN,               TWO_ACTOR_PARAMS(-1396, 596),   "MQ Push Block Pot 2",          RHT_POT_ICE_CAVERN,             RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_ICE_CAVERN_MQ_PUSH_BLOCK_POT_2));
    locationTable[RC_ICE_CAVERN_MQ_COMPASS_POT_1]                   = Location::Pot(RC_ICE_CAVERN_MQ_COMPASS_POT_1,                     RCQUEST_MQ, RCAREA_ICE_CAVERN,              SCENE_ICE_CAVERN,               TWO_ACTOR_PARAMS(902, -2720),   "MQ Compass Pot 1",             RHT_POT_ICE_CAVERN,             RG_BOMBS_10,        SpoilerCollectionCheck::RandomizerInf(RAND_INF_ICE_CAVERN_MQ_COMPASS_POT_1));
    locationTable[RC_ICE_CAVERN_MQ_COMPASS_POT_2]                   = Location::Pot(RC_ICE_CAVERN_MQ_COMPASS_POT_2,                     RCQUEST_MQ, RCAREA_ICE_CAVERN,              SCENE_ICE_CAVERN,               TWO_ACTOR_PARAMS(451, -2726),   "MQ Compass Pot 2",             RHT_POT_ICE_CAVERN,             RG_BOMBS_10,        SpoilerCollectionCheck::RandomizerInf(RAND_INF_ICE_CAVERN_MQ_COMPASS_POT_2));
    locationTable[RC_SPIRIT_TEMPLE_MQ_ENTRANCE_POT_1]               = Location::Pot(RC_SPIRIT_TEMPLE_MQ_ENTRANCE_POT_1,                 RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE,           SCENE_SPIRIT_TEMPLE,            TWO_ACTOR_PARAMS(-255, 742),    "MQ Entrance Pot 1",            RHT_POT_SPIRIT_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_ENTRANCE_POT_1));
    locationTable[RC_SPIRIT_TEMPLE_MQ_ENTRANCE_POT_2]               = Location::Pot(RC_SPIRIT_TEMPLE_MQ_ENTRANCE_POT_2,                 RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE,           SCENE_SPIRIT_TEMPLE,            TWO_ACTOR_PARAMS(306, 682),     "MQ Entrance Pot 2",            RHT_POT_SPIRIT_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_ENTRANCE_POT_2));
    locationTable[RC_SPIRIT_TEMPLE_MQ_ENTRANCE_POT_3]               = Location::Pot(RC_SPIRIT_TEMPLE_MQ_ENTRANCE_POT_3,                 RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE,           SCENE_SPIRIT_TEMPLE,            TWO_ACTOR_PARAMS(-297, 687),    "MQ Entrance Pot 3",            RHT_POT_SPIRIT_TEMPLE,          RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_ENTRANCE_POT_3));
    locationTable[RC_SPIRIT_TEMPLE_MQ_ENTRANCE_POT_4]               = Location::Pot(RC_SPIRIT_TEMPLE_MQ_ENTRANCE_POT_4,                 RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE,           SCENE_SPIRIT_TEMPLE,            TWO_ACTOR_PARAMS(260, 735),     "MQ Entrance Pot 4",            RHT_POT_SPIRIT_TEMPLE,          RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_ENTRANCE_POT_4));
    locationTable[RC_SPIRIT_TEMPLE_MQ_CHILD_SLUGMA_POT]             = Location::Pot(RC_SPIRIT_TEMPLE_MQ_CHILD_SLUGMA_POT,               RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE,           SCENE_SPIRIT_TEMPLE,            TWO_ACTOR_PARAMS(-894, -50),    "MQ Child Slugma Pot",          RHT_POT_SPIRIT_TEMPLE,          RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_CHILD_SLUGMA_POT));
    locationTable[RC_SPIRIT_TEMPLE_MQ_CHILD_GIBDO_POT_1]            = Location::Pot(RC_SPIRIT_TEMPLE_MQ_CHILD_GIBDO_POT_1,              RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE,           SCENE_SPIRIT_TEMPLE,            TWO_ACTOR_PARAMS(-382, -1256),  "MQ Child Gibdo Pot 1",         RHT_POT_SPIRIT_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_CHILD_GIBDO_POT_1));
    locationTable[RC_SPIRIT_TEMPLE_MQ_CHILD_GIBDO_POT_2]            = Location::Pot(RC_SPIRIT_TEMPLE_MQ_CHILD_GIBDO_POT_2,              RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE,           SCENE_SPIRIT_TEMPLE,            TWO_ACTOR_PARAMS(-381, -1207),  "MQ Child Gibdo Pot 2",         RHT_POT_SPIRIT_TEMPLE,          RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_CHILD_GIBDO_POT_2));
    locationTable[RC_SPIRIT_TEMPLE_MQ_CHILD_LIKE_LIKE_POT]          = Location::Pot(RC_SPIRIT_TEMPLE_MQ_CHILD_LIKE_LIKE_POT,            RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE,           SCENE_SPIRIT_TEMPLE,            TWO_ACTOR_PARAMS(-1035, -1240), "MQ Child Like Like Pot",       RHT_POT_SPIRIT_TEMPLE,          RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_CHILD_LIKE_LIKE_POT));
    locationTable[RC_SPIRIT_TEMPLE_MQ_CHILD_STALFOS_POT_1]          = Location::Pot(RC_SPIRIT_TEMPLE_MQ_CHILD_STALFOS_POT_1,            RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE,           SCENE_SPIRIT_TEMPLE,            TWO_ACTOR_PARAMS(-1166, -1931), "MQ Child Stalfos Pot 1",       RHT_POT_SPIRIT_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_CHILD_STALFOS_POT_1));
    locationTable[RC_SPIRIT_TEMPLE_MQ_CHILD_STALFOS_POT_2]          = Location::Pot(RC_SPIRIT_TEMPLE_MQ_CHILD_STALFOS_POT_2,            RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE,           SCENE_SPIRIT_TEMPLE,            TWO_ACTOR_PARAMS(-959, -1930),  "MQ Child Stalfos Pot 2",       RHT_POT_SPIRIT_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_CHILD_STALFOS_POT_2));
    locationTable[RC_SPIRIT_TEMPLE_MQ_CHILD_STALFOS_POT_3]          = Location::Pot(RC_SPIRIT_TEMPLE_MQ_CHILD_STALFOS_POT_3,            RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE,           SCENE_SPIRIT_TEMPLE,            TWO_ACTOR_PARAMS(-853, -1837),  "MQ Child Stalfos Pot 3",       RHT_POT_SPIRIT_TEMPLE,          RG_DEKU_SEEDS_30,   SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_CHILD_STALFOS_POT_3));
    locationTable[RC_SPIRIT_TEMPLE_MQ_CHILD_STALFOS_POT_4]          = Location::Pot(RC_SPIRIT_TEMPLE_MQ_CHILD_STALFOS_POT_4,            RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE,           SCENE_SPIRIT_TEMPLE,            TWO_ACTOR_PARAMS(-1250, -1852), "MQ Child Stalfos Pot 4",       RHT_POT_SPIRIT_TEMPLE,          RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_CHILD_STALFOS_POT_4));
    locationTable[RC_SPIRIT_TEMPLE_MQ_STATUE_2F_CENTER_EAST_POT]    = Location::Pot(RC_SPIRIT_TEMPLE_MQ_STATUE_2F_CENTER_EAST_POT,      RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE,           SCENE_SPIRIT_TEMPLE,            TWO_ACTOR_PARAMS(639, -1224),   "MQ Statue Room 2F CE Pot",     RHT_POT_SPIRIT_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_CENTRAL_CHAMBER_POT_1));
    locationTable[RC_SPIRIT_TEMPLE_MQ_STATUE_3F_EAST_POT]           = Location::Pot(RC_SPIRIT_TEMPLE_MQ_STATUE_3F_EAST_POT,             RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE,           SCENE_SPIRIT_TEMPLE,            TWO_ACTOR_PARAMS(-521, -1576),  "MQ Statue Room 3F E Pot",      RHT_POT_SPIRIT_TEMPLE,          RG_ARROWS_5,        SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_CENTRAL_CHAMBER_POT_2));
    locationTable[RC_SPIRIT_TEMPLE_MQ_STATUE_3F_WEST_POT]           = Location::Pot(RC_SPIRIT_TEMPLE_MQ_STATUE_3F_WEST_POT,             RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE,           SCENE_SPIRIT_TEMPLE,            TWO_ACTOR_PARAMS(-595, -1576),  "MQ Statue Room 3F W Pot",      RHT_POT_SPIRIT_TEMPLE,          RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_CENTRAL_CHAMBER_POT_3));
    locationTable[RC_SPIRIT_TEMPLE_MQ_STATUE_2F_WEST_POT]           = Location::Pot(RC_SPIRIT_TEMPLE_MQ_STATUE_2F_WEST_POT,             RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE,           SCENE_SPIRIT_TEMPLE,            TWO_ACTOR_PARAMS(-559, -1209),  "MQ Statue Room 2F W Pot",      RHT_POT_SPIRIT_TEMPLE,          RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_CENTRAL_CHAMBER_POT_4));
    locationTable[RC_SPIRIT_TEMPLE_MQ_STATUE_2F_EASTMOST_POT]       = Location::Pot(RC_SPIRIT_TEMPLE_MQ_STATUE_2F_EASTMOST_POT,         RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE,           SCENE_SPIRIT_TEMPLE,            TWO_ACTOR_PARAMS(704, -1224),   "MQ Statue Room 2F E Pot",      RHT_POT_SPIRIT_TEMPLE,          RG_DEKU_SEEDS_30,   SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_CENTRAL_CHAMBER_POT_5));
    locationTable[RC_SPIRIT_TEMPLE_MQ_SUN_BLOCKS_POT_1]             = Location::Pot(RC_SPIRIT_TEMPLE_MQ_SUN_BLOCKS_POT_1,               RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE,           SCENE_SPIRIT_TEMPLE,            TWO_ACTOR_PARAMS(-1276, 432),   "MQ Sun Blocks Pot 1",          RHT_POT_SPIRIT_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_SUN_BLOCKS_POT_1));
    locationTable[RC_SPIRIT_TEMPLE_MQ_SUN_BLOCKS_POT_2]             = Location::Pot(RC_SPIRIT_TEMPLE_MQ_SUN_BLOCKS_POT_2,               RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE,           SCENE_SPIRIT_TEMPLE,            TWO_ACTOR_PARAMS(-1635, -446),  "MQ Sun Blocks Pot 2",          RHT_POT_SPIRIT_TEMPLE,          RG_DEKU_SEEDS_30,   SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_SUN_BLOCKS_POT_2));
    locationTable[RC_SPIRIT_TEMPLE_MQ_LONG_CLIMB_POT_1]             = Location::Pot(RC_SPIRIT_TEMPLE_MQ_LONG_CLIMB_POT_1,               RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE,           SCENE_SPIRIT_TEMPLE,            TWO_ACTOR_PARAMS(829, 250),     "MQ Long Climb Pot 1",          RHT_POT_SPIRIT_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_LONG_CLIMB_POT_1));
    locationTable[RC_SPIRIT_TEMPLE_MQ_LONG_CLIMB_POT_2]             = Location::Pot(RC_SPIRIT_TEMPLE_MQ_LONG_CLIMB_POT_2,               RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE,           SCENE_SPIRIT_TEMPLE,            TWO_ACTOR_PARAMS(531, 249),     "MQ Long Climb Pot 2",          RHT_POT_SPIRIT_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_LONG_CLIMB_POT_2));
    locationTable[RC_SPIRIT_TEMPLE_MQ_BIG_MIRROR_POT_1]             = Location::Pot(RC_SPIRIT_TEMPLE_MQ_BIG_MIRROR_POT_1,               RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE,           SCENE_SPIRIT_TEMPLE,            TWO_ACTOR_PARAMS(244, -893),    "MQ Big Mirror Pot 1",          RHT_POT_SPIRIT_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_BIG_MIRROR_POT_1));
    locationTable[RC_SPIRIT_TEMPLE_MQ_BIG_MIRROR_POT_2]             = Location::Pot(RC_SPIRIT_TEMPLE_MQ_BIG_MIRROR_POT_2,               RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE,           SCENE_SPIRIT_TEMPLE,            TWO_ACTOR_PARAMS(-122, -741),   "MQ Big Mirror Pot 2",          RHT_POT_SPIRIT_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_BIG_MIRROR_POT_2));
    locationTable[RC_SPIRIT_TEMPLE_MQ_BIG_MIRROR_POT_3]             = Location::Pot(RC_SPIRIT_TEMPLE_MQ_BIG_MIRROR_POT_3,               RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE,           SCENE_SPIRIT_TEMPLE,            TWO_ACTOR_PARAMS(250, -741),    "MQ Big Mirror Pot 3",          RHT_POT_SPIRIT_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_BIG_MIRROR_POT_3));
    locationTable[RC_SPIRIT_TEMPLE_MQ_BIG_MIRROR_POT_4]             = Location::Pot(RC_SPIRIT_TEMPLE_MQ_BIG_MIRROR_POT_4,               RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE,           SCENE_SPIRIT_TEMPLE,            TWO_ACTOR_PARAMS(-123, -892),   "MQ Big Mirror Pot 4",          RHT_POT_SPIRIT_TEMPLE,          RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_BIG_MIRROR_POT_4));
    locationTable[RC_SPIRIT_TEMPLE_MQ_BEFORE_MIRROR_POT_1]          = Location::Pot(RC_SPIRIT_TEMPLE_MQ_BEFORE_MIRROR_POT_1,            RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE,           SCENE_SPIRIT_TEMPLE,            TWO_ACTOR_PARAMS(781, -841),    "MQ Before Mirror Pot 1",       RHT_POT_SPIRIT_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_BEFORE_MIRROR_POT_1));
    locationTable[RC_SPIRIT_TEMPLE_MQ_BEFORE_MIRROR_POT_2]          = Location::Pot(RC_SPIRIT_TEMPLE_MQ_BEFORE_MIRROR_POT_2,            RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE,           SCENE_SPIRIT_TEMPLE,            TWO_ACTOR_PARAMS(587, -844),    "MQ Before Mirror Pot 2",       RHT_POT_SPIRIT_TEMPLE,          RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_BEFORE_MIRROR_POT_2));
    locationTable[RC_SPIRIT_TEMPLE_MQ_EARLY_ADULT_POT_1]            = Location::Pot(RC_SPIRIT_TEMPLE_MQ_EARLY_ADULT_POT_1,              RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE,           SCENE_SPIRIT_TEMPLE,            TWO_ACTOR_PARAMS(1204, -910),   "MQ Early Adult Pot 1",         RHT_POT_SPIRIT_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_EARLY_ADULT_POT_1));
    locationTable[RC_SPIRIT_TEMPLE_MQ_EARLY_ADULT_POT_2]            = Location::Pot(RC_SPIRIT_TEMPLE_MQ_EARLY_ADULT_POT_2,              RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE,           SCENE_SPIRIT_TEMPLE,            TWO_ACTOR_PARAMS(1204, -1028),  "MQ Early Adult Pot 2",         RHT_POT_SPIRIT_TEMPLE,          RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_EARLY_ADULT_POT_2));
    locationTable[RC_WATER_TEMPLE_MQ_LOWER_TORCHES_POT_1]           = Location::Pot(RC_WATER_TEMPLE_MQ_LOWER_TORCHES_POT_1,             RCQUEST_MQ, RCAREA_WATER_TEMPLE,            SCENE_WATER_TEMPLE,             TWO_ACTOR_PARAMS(1123, 62),     "MQ Lower Torches Pot 1",       RHT_POT_WATER_TEMPLE,           RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_LOWER_TORCHES_POT_1));
    locationTable[RC_WATER_TEMPLE_MQ_LOWER_TORCHES_POT_2]           = Location::Pot(RC_WATER_TEMPLE_MQ_LOWER_TORCHES_POT_2,             RCQUEST_MQ, RCAREA_WATER_TEMPLE,            SCENE_WATER_TEMPLE,             TWO_ACTOR_PARAMS(1123, 293),    "MQ Lower Torches Pot 2",       RHT_POT_WATER_TEMPLE,           RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_LOWER_TORCHES_POT_2));
    locationTable[RC_WATER_TEMPLE_MQ_STORAGE_ROOM_A_POT_1]          = Location::Pot(RC_WATER_TEMPLE_MQ_STORAGE_ROOM_A_POT_1,            RCQUEST_MQ, RCAREA_WATER_TEMPLE,            SCENE_WATER_TEMPLE,             TWO_ACTOR_PARAMS(1222, -617),   "MQ Storage Room A Pot 1",      RHT_POT_WATER_TEMPLE,           RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_STORAGE_ROOM_A_POT_1));
    locationTable[RC_WATER_TEMPLE_MQ_STORAGE_ROOM_A_POT_2]          = Location::Pot(RC_WATER_TEMPLE_MQ_STORAGE_ROOM_A_POT_2,            RCQUEST_MQ, RCAREA_WATER_TEMPLE,            SCENE_WATER_TEMPLE,             TWO_ACTOR_PARAMS(1122, -617),   "MQ Storage Room A Pot 2",      RHT_POT_WATER_TEMPLE,           RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_STORAGE_ROOM_A_POT_2));
    locationTable[RC_WATER_TEMPLE_MQ_STORAGE_ROOM_A_POT_3]          = Location::Pot(RC_WATER_TEMPLE_MQ_STORAGE_ROOM_A_POT_3,            RCQUEST_MQ, RCAREA_WATER_TEMPLE,            SCENE_WATER_TEMPLE,             TWO_ACTOR_PARAMS(1172, -617),   "MQ Storage Room A Pot 3",      RHT_POT_WATER_TEMPLE,           RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_STORAGE_ROOM_A_POT_3));
    locationTable[RC_WATER_TEMPLE_MQ_GS_STORAGE_ROOM_POT_1]         = Location::Pot(RC_WATER_TEMPLE_MQ_GS_STORAGE_ROOM_POT_1,           RCQUEST_MQ, RCAREA_WATER_TEMPLE,            SCENE_WATER_TEMPLE,             TWO_ACTOR_PARAMS(-989, 255),    "MQ GS Storage Room Pot 1",     RHT_POT_WATER_TEMPLE,           RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_GS_STORAGE_ROOM_POT_1));
    locationTable[RC_WATER_TEMPLE_MQ_GS_STORAGE_ROOM_POT_2]         = Location::Pot(RC_WATER_TEMPLE_MQ_GS_STORAGE_ROOM_POT_2,           RCQUEST_MQ, RCAREA_WATER_TEMPLE,            SCENE_WATER_TEMPLE,             TWO_ACTOR_PARAMS(-1026, 257),   "MQ GS Storage Room Pot 2",     RHT_POT_WATER_TEMPLE,           RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_GS_STORAGE_ROOM_POT_2));
    locationTable[RC_WATER_TEMPLE_MQ_GS_STORAGE_ROOM_POT_3]         = Location::Pot(RC_WATER_TEMPLE_MQ_GS_STORAGE_ROOM_POT_3,           RCQUEST_MQ, RCAREA_WATER_TEMPLE,            SCENE_WATER_TEMPLE,             TWO_ACTOR_PARAMS(-1062, 256),   "MQ GS Storage Room Pot 3",     RHT_POT_WATER_TEMPLE,           RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_GS_STORAGE_ROOM_POT_3));
    locationTable[RC_WATER_TEMPLE_MQ_LIZALFOS_HALLWAY_WEST_POT]     = Location::Pot(RC_WATER_TEMPLE_MQ_LIZALFOS_HALLWAY_WEST_POT,       RCQUEST_MQ, RCAREA_WATER_TEMPLE,            SCENE_WATER_TEMPLE,             TWO_ACTOR_PARAMS(512, 579),     "MQ Lizalfos Hall W Pot",       RHT_POT_WATER_TEMPLE,           RG_RED_RUPEE,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_CENTRAL_GATE_POT_1));
    locationTable[RC_WATER_TEMPLE_MQ_LIZALFOS_HALLWAY_SOUTH_POT]    = Location::Pot(RC_WATER_TEMPLE_MQ_LIZALFOS_HALLWAY_SOUTH_POT,      RCQUEST_MQ, RCAREA_WATER_TEMPLE,            SCENE_WATER_TEMPLE,             TWO_ACTOR_PARAMS(602, 1022),    "MQ Lizalfos Hall S Pot",       RHT_POT_WATER_TEMPLE,           RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_CENTRAL_GATE_POT_2));
    locationTable[RC_WATER_TEMPLE_MQ_LIZALFOS_HALLWAY_SE_POT]       = Location::Pot(RC_WATER_TEMPLE_MQ_LIZALFOS_HALLWAY_SE_POT,         RCQUEST_MQ, RCAREA_WATER_TEMPLE,            SCENE_WATER_TEMPLE,             TWO_ACTOR_PARAMS(789, 947),     "MQ Lizalfos Hall SE Pot",      RHT_POT_WATER_TEMPLE,           RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_CENTRAL_GATE_POT_3));
    locationTable[RC_WATER_TEMPLE_MQ_LIZALFOS_CAGE_SOUTH_POT]       = Location::Pot(RC_WATER_TEMPLE_MQ_LIZALFOS_CAGE_SOUTH_POT,         RCQUEST_MQ, RCAREA_WATER_TEMPLE,            SCENE_WATER_TEMPLE,             TWO_ACTOR_PARAMS(768, 565),     "MQ Lizalfos Cage N Pot",       RHT_POT_WATER_TEMPLE,           RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_CENTRAL_GATE_POT_4));
    locationTable[RC_WATER_TEMPLE_MQ_LIZALFOS_CAGE_NORTH_POT]       = Location::Pot(RC_WATER_TEMPLE_MQ_LIZALFOS_CAGE_NORTH_POT,         RCQUEST_MQ, RCAREA_WATER_TEMPLE,            SCENE_WATER_TEMPLE,             TWO_ACTOR_PARAMS(767, 517),     "MQ Lizalfos Cage S Pot",       RHT_POT_WATER_TEMPLE,           RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_CENTRAL_GATE_POT_5));
    locationTable[RC_WATER_TEMPLE_MQ_STALFOS_PIT_MIDDLE_POT]        = Location::Pot(RC_WATER_TEMPLE_MQ_STALFOS_PIT_MIDDLE_POT,          RCQUEST_MQ, RCAREA_WATER_TEMPLE,            SCENE_WATER_TEMPLE,             TWO_ACTOR_PARAMS(-3500, -751),  "MQ Stalfos Pit Middle Pot",    RHT_POT_WATER_TEMPLE,           RG_DEKU_NUTS_5,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_BEFORE_DARK_LINK_POT_3));
    locationTable[RC_WATER_TEMPLE_MQ_STALFOS_PIT_SOUTH_POT]         = Location::Pot(RC_WATER_TEMPLE_MQ_STALFOS_PIT_SOUTH_POT,           RCQUEST_MQ, RCAREA_WATER_TEMPLE,            SCENE_WATER_TEMPLE,             TWO_ACTOR_PARAMS(-3500, -686),  "MQ Stalfos Pit South Pot",     RHT_POT_WATER_TEMPLE,           RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_BEFORE_DARK_LINK_POT_4));
    locationTable[RC_WATER_TEMPLE_MQ_STALFOS_PIT_NORTH_POT]         = Location::Pot(RC_WATER_TEMPLE_MQ_STALFOS_PIT_NORTH_POT,           RCQUEST_MQ, RCAREA_WATER_TEMPLE,            SCENE_WATER_TEMPLE,             TWO_ACTOR_PARAMS(-3500, -810),  "MQ Stalfos Pit North Pot",     RHT_POT_WATER_TEMPLE,           RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_BEFORE_DARK_LINK_POT_5));
    locationTable[RC_WATER_TEMPLE_MQ_BEFORE_DARK_LINK_POT_1]        = Location::Pot(RC_WATER_TEMPLE_MQ_BEFORE_DARK_LINK_POT_1,          RCQUEST_MQ, RCAREA_WATER_TEMPLE,            SCENE_WATER_TEMPLE,             TWO_ACTOR_PARAMS(-3188, -1628), "MQ Before Dark Link Pot 1",    RHT_POT_WATER_TEMPLE,           RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_BEFORE_DARK_LINK_POT_1));
    locationTable[RC_WATER_TEMPLE_MQ_BEFORE_DARK_LINK_POT_2]        = Location::Pot(RC_WATER_TEMPLE_MQ_BEFORE_DARK_LINK_POT_2,          RCQUEST_MQ, RCAREA_WATER_TEMPLE,            SCENE_WATER_TEMPLE,             TWO_ACTOR_PARAMS(-3056, -1626), "MQ Before Dark Link Pot 2",    RHT_POT_WATER_TEMPLE,           RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_BEFORE_DARK_LINK_POT_2));
    locationTable[RC_WATER_TEMPLE_MQ_AFTER_DARK_LINK_POT_1]         = Location::Pot(RC_WATER_TEMPLE_MQ_AFTER_DARK_LINK_POT_1,           RCQUEST_MQ, RCAREA_WATER_TEMPLE,            SCENE_WATER_TEMPLE,             TWO_ACTOR_PARAMS(-3072, -4024), "MQ After Dark Link Pot 1",     RHT_POT_WATER_TEMPLE,           RG_ARROWS_30,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_AFTER_DARK_LINK_POT_1));
    locationTable[RC_WATER_TEMPLE_MQ_AFTER_DARK_LINK_POT_2]         = Location::Pot(RC_WATER_TEMPLE_MQ_AFTER_DARK_LINK_POT_2,           RCQUEST_MQ, RCAREA_WATER_TEMPLE,            SCENE_WATER_TEMPLE,             TWO_ACTOR_PARAMS(-3169, -4023), "MQ After Dark Link Pot 2",     RHT_POT_WATER_TEMPLE,           RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_AFTER_DARK_LINK_POT_2));
    locationTable[RC_WATER_TEMPLE_MQ_RIVER_POT_1]                   = Location::Pot(RC_WATER_TEMPLE_MQ_RIVER_POT_1,                     RCQUEST_MQ, RCAREA_WATER_TEMPLE,            SCENE_WATER_TEMPLE,             TWO_ACTOR_PARAMS(-2226, -2487), "MQ River Pot 1",               RHT_POT_WATER_TEMPLE,           RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_RIVER_POT_1));
    locationTable[RC_WATER_TEMPLE_MQ_RIVER_POT_2]                   = Location::Pot(RC_WATER_TEMPLE_MQ_RIVER_POT_2,                     RCQUEST_MQ, RCAREA_WATER_TEMPLE,            SCENE_WATER_TEMPLE,             TWO_ACTOR_PARAMS(-2184, -2456), "MQ River Pot 2",               RHT_POT_WATER_TEMPLE,           RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_RIVER_POT_2));
    locationTable[RC_WATER_TEMPLE_MQ_BOSS_KEY_POT]                  = Location::Pot(RC_WATER_TEMPLE_MQ_BOSS_KEY_POT,                    RCQUEST_MQ, RCAREA_WATER_TEMPLE,            SCENE_WATER_TEMPLE,             TWO_ACTOR_PARAMS(-1033, -1569), "MQ Boss Key Pot",              RHT_POT_WATER_TEMPLE,           RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_BOSS_KEY_POT));
    locationTable[RC_WATER_TEMPLE_MQ_LOWEST_GS_POT_1]               = Location::Pot(RC_WATER_TEMPLE_MQ_LOWEST_GS_POT_1,                 RCQUEST_MQ, RCAREA_WATER_TEMPLE,            SCENE_WATER_TEMPLE,             TWO_ACTOR_PARAMS(-2314, 770),   "MQ Lowest GS Pot 1",           RHT_POT_WATER_TEMPLE,           RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_LOWEST_GS_POT_1));
    locationTable[RC_WATER_TEMPLE_MQ_LOWEST_GS_POT_2]               = Location::Pot(RC_WATER_TEMPLE_MQ_LOWEST_GS_POT_2,                 RCQUEST_MQ, RCAREA_WATER_TEMPLE,            SCENE_WATER_TEMPLE,             TWO_ACTOR_PARAMS(-2314, 808),   "MQ Lowest GS Pot 2",           RHT_POT_WATER_TEMPLE,           RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_LOWEST_GS_POT_2));
    locationTable[RC_WATER_TEMPLE_MQ_LOWEST_GS_POT_3]               = Location::Pot(RC_WATER_TEMPLE_MQ_LOWEST_GS_POT_3,                 RCQUEST_MQ, RCAREA_WATER_TEMPLE,            SCENE_WATER_TEMPLE,             TWO_ACTOR_PARAMS(-2313, 944),   "MQ Lowest GS Pot 3",           RHT_POT_WATER_TEMPLE,           RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_LOWEST_GS_POT_3));
    locationTable[RC_WATER_TEMPLE_MQ_LOWEST_GS_POT_4]               = Location::Pot(RC_WATER_TEMPLE_MQ_LOWEST_GS_POT_4,                 RCQUEST_MQ, RCAREA_WATER_TEMPLE,            SCENE_WATER_TEMPLE,             TWO_ACTOR_PARAMS(-2313, 905),   "MQ Lowest GS Pot 4",           RHT_POT_WATER_TEMPLE,           RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_LOWEST_GS_POT_4));
    locationTable[RC_WATER_TEMPLE_MQ_STORAGE_ROOM_B_POT_1]          = Location::Pot(RC_WATER_TEMPLE_MQ_STORAGE_ROOM_B_POT_1,            RCQUEST_MQ, RCAREA_WATER_TEMPLE,            SCENE_WATER_TEMPLE,             TWO_ACTOR_PARAMS(-1417, -3025), "MQ Storage Room B Pot 1",      RHT_POT_WATER_TEMPLE,           RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_STORAGE_ROOM_B_POT_1));
    locationTable[RC_WATER_TEMPLE_MQ_STORAGE_ROOM_B_POT_2]          = Location::Pot(RC_WATER_TEMPLE_MQ_STORAGE_ROOM_B_POT_2,            RCQUEST_MQ, RCAREA_WATER_TEMPLE,            SCENE_WATER_TEMPLE,             TWO_ACTOR_PARAMS(-1150, -3031), "MQ Storage Room B Pot 2",      RHT_POT_WATER_TEMPLE,           RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_STORAGE_ROOM_B_POT_2));
    locationTable[RC_WATER_TEMPLE_MQ_MINI_DODONGO_POT_1]            = Location::Pot(RC_WATER_TEMPLE_MQ_MINI_DODONGO_POT_1,              RCQUEST_MQ, RCAREA_WATER_TEMPLE,            SCENE_WATER_TEMPLE,             TWO_ACTOR_PARAMS(28, -3817),    "MQ Mini Dodongo Pot 1",        RHT_POT_WATER_TEMPLE,           RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_MINI_DODONGO_POT_1));
    locationTable[RC_WATER_TEMPLE_MQ_MINI_DODONGO_POT_2]            = Location::Pot(RC_WATER_TEMPLE_MQ_MINI_DODONGO_POT_2,              RCQUEST_MQ, RCAREA_WATER_TEMPLE,            SCENE_WATER_TEMPLE,             TWO_ACTOR_PARAMS(-28, -3817),   "MQ Mini Dodongo Pot 2",        RHT_POT_WATER_TEMPLE,           RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_MINI_DODONGO_POT_2));
    locationTable[RC_GERUDO_TRAINING_GROUND_MQ_LOBBY_LEFT_POT_1]    = Location::Pot(RC_GERUDO_TRAINING_GROUND_MQ_LOBBY_LEFT_POT_1,      RCQUEST_MQ, RCAREA_GERUDO_TRAINING_GROUND,  SCENE_GERUDO_TRAINING_GROUND,   TWO_ACTOR_PARAMS(-324, -72),    "MQ Lobby Left Pot 1",          RHT_POT_GERUDO_TRAINING_GROUND, RG_BLUE_RUPEE,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_GERUDO_TRAINING_GROUND_MQ_LOBBY_LEFT_POT_1));
    locationTable[RC_GERUDO_TRAINING_GROUND_MQ_LOBBY_LEFT_POT_2]    = Location::Pot(RC_GERUDO_TRAINING_GROUND_MQ_LOBBY_LEFT_POT_2,      RCQUEST_MQ, RCAREA_GERUDO_TRAINING_GROUND,  SCENE_GERUDO_TRAINING_GROUND,   TWO_ACTOR_PARAMS(-324, -177),   "MQ Lobby Left Pot 2",          RHT_POT_GERUDO_TRAINING_GROUND, RG_RECOVERY_HEART,   SpoilerCollectionCheck::RandomizerInf(RAND_INF_GERUDO_TRAINING_GROUND_MQ_LOBBY_LEFT_POT_2));
    locationTable[RC_GERUDO_TRAINING_GROUND_MQ_LOBBY_RIGHT_POT_1]   = Location::Pot(RC_GERUDO_TRAINING_GROUND_MQ_LOBBY_RIGHT_POT_1,     RCQUEST_MQ, RCAREA_GERUDO_TRAINING_GROUND,  SCENE_GERUDO_TRAINING_GROUND,   TWO_ACTOR_PARAMS(199, -79),     "MQ Lobby Right Pot 1",         RHT_POT_GERUDO_TRAINING_GROUND, RG_RECOVERY_HEART,   SpoilerCollectionCheck::RandomizerInf(RAND_INF_GERUDO_TRAINING_GROUND_MQ_LOBBY_RIGHT_POT_1));
    locationTable[RC_GERUDO_TRAINING_GROUND_MQ_LOBBY_RIGHT_POT_2]   = Location::Pot(RC_GERUDO_TRAINING_GROUND_MQ_LOBBY_RIGHT_POT_2,     RCQUEST_MQ, RCAREA_GERUDO_TRAINING_GROUND,  SCENE_GERUDO_TRAINING_GROUND,   TWO_ACTOR_PARAMS(197, -179),    "MQ Lobby Right Pot 2",         RHT_POT_GERUDO_TRAINING_GROUND, RG_BLUE_RUPEE,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_GERUDO_TRAINING_GROUND_MQ_LOBBY_RIGHT_POT_2));

    // clang-format on
}

static RegisterShipInitFunc initFunc(Rando::StaticData::RegisterPotLocations);
