#pragma once

#include <stdint.h>
#include "z64item.h"
#include "randomizer_inf.h"

#define MAX_TRICK_NAME_SIZE 50

#define TWO_ACTOR_PARAMS(a, b) ((((a)&0xFFFF) << 16) | ((b)&0xFFFF))

// This should probably go in a less rando-specific location
// but the best location will probably be in the modding engine
// which doesn't exist yet.
typedef enum {
    MOD_NONE,
    MOD_RANDOMIZER,
} ModIndex;
typedef enum {
    TABLE_VANILLA = MOD_NONE,
    TABLE_RANDOMIZER = MOD_RANDOMIZER,
} TableIndex;

typedef struct {
    char tex[512];
    uint16_t width;
    uint16_t height;
    uint8_t im_fmt;
    uint8_t im_siz;
    uint8_t id;
} Sprite;

typedef enum {
    HINT_TYPE_HINT_KEY,
    HINT_TYPE_AREA,
    HINT_TYPE_ITEM,
    HINT_TYPE_TRIAL,
    HINT_TYPE_ENTRANCE,
    HINT_TYPE_ITEM_AREA,
    HINT_TYPE_ALTAR_CHILD,
    HINT_TYPE_ALTAR_ADULT,
    HINT_TYPE_WOTH, // Way of the Hero
    HINT_TYPE_FOOLISH,
    HINT_TYPE_MESSAGE,
    HINT_TYPE_MAX
} HintType;

typedef enum {
    LOGIC_NONE,
    LOGIC_KOKIRI_SWORD,
    LOGIC_MASTER_SWORD,
    LOGIC_BIGGORON_SWORD,
    LOGIC_DEKU_SHIELD,
    LOGIC_HYLIAN_SHIELD,
    LOGIC_MIRROR_SHIELD,
    LOGIC_GORON_TUNIC,
    LOGIC_ZORA_TUNIC,
    LOGIC_IRON_BOOTS,
    LOGIC_HOVER_BOOTS,
    LOGIC_BOOMERANG,
    LOGIC_LENS_OF_TRUTH,
    LOGIC_HAMMER,
    LOGIC_STONE_OF_AGONY,
    LOGIC_DINS_FIRE,
    LOGIC_FARORES_WIND,
    LOGIC_NAYRUS_LOVE,
    LOGIC_FIRE_ARROWS,
    LOGIC_ICE_ARROWS,
    LOGIC_LIGHT_ARROWS,
    LOGIC_GERUDO_CARD,
    LOGIC_MAGIC_BEAN,
    LOGIC_MAGIC_BEANPACK,
    LOGIC_DOUBLE_DEFENSE,
    LOGIC_WEIRD_EGG,
    LOGIC_ZELDAS_LETTER,
    LOGIC_POCKET_EGG,
    LOGIC_COJIRO,
    LOGIC_ODD_MUSHROOM,
    LOGIC_ODD_POULTICE,
    LOGIC_POACHERS_SAW,
    LOGIC_BROKEN_SWORD,
    LOGIC_PRESCRIPTION,
    LOGIC_EYEBALL_FROG,
    LOGIC_EYEDROPS,
    LOGIC_CLAIM_CHECK,
    LOGIC_GOLD_SKULLTULA_TOKENS,
    LOGIC_PROGRESSIVE_HOOKSHOT,
    LOGIC_PROGRESSIVE_STRENGTH,
    LOGIC_PROGRESSIVE_BOMB_BAG,
    LOGIC_PROGRESSIVE_BOW,
    LOGIC_PROGRESSIVE_BULLET_BAG,
    LOGIC_PROGRESSIVE_STICK_BAG,
    LOGIC_PROGRESSIVE_NUT_BAG,
    LOGIC_PROGRESSIVE_WALLET,
    LOGIC_PROGRESSIVE_SCALE,
    LOGIC_BOMBCHUS,
    LOGIC_PROGRESSIVE_MAGIC,
    LOGIC_PROGRESSIVE_OCARINA,
    LOGIC_PROGRESSIVE_GIANT_KNIFE,
    LOGIC_BOTTLES,
    LOGIC_RUTOS_LETTER,
    LOGIC_BOTTLE_WITH_BIG_POE,
    LOGIC_ZELDAS_LULLABY,
    LOGIC_EPONAS_SONG,
    LOGIC_SARIAS_SONG,
    LOGIC_SUNS_SONG,
    LOGIC_SONG_OF_TIME,
    LOGIC_SONG_OF_STORMS,
    LOGIC_MINUET_OF_FOREST,
    LOGIC_BOLERO_OF_FIRE,
    LOGIC_SERENADE_OF_WATER,
    LOGIC_NOCTURNE_OF_SHADOW,
    LOGIC_REQUIEM_OF_SPIRIT,
    LOGIC_PRELUDE_OF_LIGHT,
    LOGIC_MAP_DEKU_TREE,
    LOGIC_MAP_DODONGOS_CAVERN,
    LOGIC_MAP_JABU_JABUS_BELLY,
    LOGIC_MAP_FOREST_TEMPLE,
    LOGIC_MAP_FIRE_TEMPLE,
    LOGIC_MAP_WATER_TEMPLE,
    LOGIC_MAP_SPIRIT_TEMPLE,
    LOGIC_MAP_SHADOW_TEMPLE,
    LOGIC_MAP_BOTTOM_OF_THE_WELL,
    LOGIC_MAP_ICE_CAVERN,
    LOGIC_COMPASS_DEKU_TREE,
    LOGIC_COMPASS_DODONGOS_CAVERN,
    LOGIC_COMPASS_JABU_JABUS_BELLY,
    LOGIC_COMPASS_FOREST_TEMPLE,
    LOGIC_COMPASS_FIRE_TEMPLE,
    LOGIC_COMPASS_WATER_TEMPLE,
    LOGIC_COMPASS_SPIRIT_TEMPLE,
    LOGIC_COMPASS_SHADOW_TEMPLE,
    LOGIC_COMPASS_BOTTOM_OF_THE_WELL,
    LOGIC_COMPASS_ICE_CAVERN,
    LOGIC_BOSS_KEY_FOREST_TEMPLE,
    LOGIC_BOSS_KEY_FIRE_TEMPLE,
    LOGIC_BOSS_KEY_WATER_TEMPLE,
    LOGIC_BOSS_KEY_SPIRIT_TEMPLE,
    LOGIC_BOSS_KEY_SHADOW_TEMPLE,
    LOGIC_BOSS_KEY_GANONS_CASTLE,
    LOGIC_FOREST_TEMPLE_KEYS,
    LOGIC_FIRE_TEMPLE_KEYS,
    LOGIC_WATER_TEMPLE_KEYS,
    LOGIC_SPIRIT_TEMPLE_KEYS,
    LOGIC_SHADOW_TEMPLE_KEYS,
    LOGIC_BOTTOM_OF_THE_WELL_KEYS,
    LOGIC_GERUDO_TRAINING_GROUND_KEYS,
    LOGIC_GERUDO_FORTRESS_KEYS,
    LOGIC_GANONS_CASTLE_KEYS,
    LOGIC_TREASURE_GAME_KEYS,
    LOGIC_SKELETON_KEY,
    LOGIC_GUARD_HOUSE_KEY,
    LOGIC_MARKET_BAZAAR_KEY,
    LOGIC_MARKET_POTION_SHOP_KEY,
    LOGIC_MASK_SHOP_KEY,
    LOGIC_MARKET_SHOOTING_GALLERY_KEY,
    LOGIC_BOMBCHU_BOWLING_KEY,
    LOGIC_TREASURE_CHEST_GAME_BUILDING_KEY,
    LOGIC_BOMBCHU_SHOP_KEY,
    LOGIC_RICHARDS_HOUSE_KEY,
    LOGIC_ALLEY_HOUSE_KEY,
    LOGIC_KAK_BAZAAR_KEY,
    LOGIC_KAK_POTION_SHOP_KEY,
    LOGIC_BOSS_HOUSE_KEY,
    LOGIC_GRANNYS_POTION_SHOP_KEY,
    LOGIC_SKULLTULA_HOUSE_KEY,
    LOGIC_IMPAS_HOUSE_KEY,
    LOGIC_WINDMILL_KEY,
    LOGIC_KAK_SHOOTING_GALLERY_KEY,
    LOGIC_DAMPES_HUT_KEY,
    LOGIC_TALONS_HOUSE_KEY,
    LOGIC_STABLES_KEY,
    LOGIC_BACK_TOWER_KEY,
    LOGIC_HYLIA_LAB_KEY,
    LOGIC_FISHING_HOLE_KEY,
    LOGIC_KOKIRI_EMERALD,
    LOGIC_GORON_RUBY,
    LOGIC_ZORA_SAPPHIRE,
    LOGIC_FOREST_MEDALLION,
    LOGIC_FIRE_MEDALLION,
    LOGIC_WATER_MEDALLION,
    LOGIC_SPIRIT_MEDALLION,
    LOGIC_SHADOW_MEDALLION,
    LOGIC_LIGHT_MEDALLION,
    LOGIC_GREG,
    LOGIC_PIECE_OF_HEART,
    LOGIC_HEART_CONTAINER,
    LOGIC_NUTS,
    LOGIC_BUY_ARROW,
    LOGIC_BUY_BOMB,
    LOGIC_STICKS,
    LOGIC_FISH_ACCESS,
    LOGIC_BUY_MAGIC_POTION,
    LOGIC_BUY_BOMBCHUS,
    LOGIC_BUY_SEED,
    LOGIC_BLUE_FIRE_ACCESS,
    LOGIC_BUGS_ACCESS,
    LOGIC_FAIRY_ACCESS,
    LOGIC_CAN_SUMMON_GOHMA,
    LOGIC_CAN_SUMMON_KINGDODONGO,
    LOGIC_CAN_SUMMON_BARINADE,
    LOGIC_CAN_SUMMON_PHANTOMGANON,
    LOGIC_CAN_SUMMON_VOLVAGIA,
    LOGIC_CAN_SUMMON_MORPHA,
    LOGIC_CAN_SUMMON_BONGOBONGO,
    LOGIC_CAN_SUMMON_TWINROVA,
    LOGIC_CAN_SUMMON_GANON,
    LOGIC_FISHING_POLE,
    LOGIC_OCARINA_A_BUTTON,
    LOGIC_OCARINA_C_UP_BUTTON,
    LOGIC_OCARINA_C_DOWN_BUTTON,
    LOGIC_OCARINA_C_LEFT_BUTTON,
    LOGIC_OCARINA_C_RIGHT_BUTTON,
    LOGIC_TRIFORCE_PIECES,
    LOGIC_MAX
} LogicVal;

typedef enum {
    RA_NONE,
    RA_LINKS_POCKET,
    RA_KOKIRI_FOREST,
    RA_THE_LOST_WOODS,
    RA_SACRED_FOREST_MEADOW,
    RA_HYRULE_FIELD,
    RA_LAKE_HYLIA,
    RA_GERUDO_VALLEY,
    RA_GERUDO_FORTRESS,
    RA_HAUNTED_WASTELAND,
    RA_DESERT_COLOSSUS,
    RA_THE_MARKET,
    RA_TEMPLE_OF_TIME,
    RA_HYRULE_CASTLE,
    RA_OUTSIDE_GANONS_CASTLE,
    RA_CASTLE_GROUNDS,
    RA_KAKARIKO_VILLAGE,
    RA_THE_GRAVEYARD,
    RA_DEATH_MOUNTAIN_TRAIL,
    RA_GORON_CITY,
    RA_DEATH_MOUNTAIN_CRATER,
    RA_ZORAS_RIVER,
    RA_ZORAS_DOMAIN,
    RA_ZORAS_FOUNTAIN,
    RA_LON_LON_RANCH,
    RA_DEKU_TREE,
    RA_DODONGOS_CAVERN,
    RA_JABU_JABUS_BELLY,
    RA_FOREST_TEMPLE,
    RA_FIRE_TEMPLE,
    RA_WATER_TEMPLE,
    RA_SPIRIT_TEMPLE,
    RA_SHADOW_TEMPLE,
    RA_BOTTOM_OF_THE_WELL,
    RA_ICE_CAVERN,
    RA_GERUDO_TRAINING_GROUND,
    RA_GANONS_CASTLE,
    RA_MAX
} RandomizerArea;

typedef enum {
    TK_LIGHT_TRIAL,
    TK_FOREST_TRIAL,
    TK_FIRE_TRIAL,
    TK_WATER_TRIAL,
    TK_SPIRIT_TRIAL,
    TK_SHADOW_TRIAL,
    TK_MAX
} TrialKey;

// Check types based on main settings
typedef enum {
    RCTYPE_STANDARD,                   // Base set of rando checks
    RCTYPE_SKULL_TOKEN,                // Gold Skulltulas
    RCTYPE_COW,                        // Cows
    RCTYPE_ADULT_TRADE,                // Adult trade quest checks
    RCTYPE_FROG_SONG,                  // Frog song purple rupee checks
    RCTYPE_MAP,                        // Maps
    RCTYPE_COMPASS,                    // Compasses
    RCTYPE_SMALL_KEY,                  // Small Keys
    RCTYPE_GF_KEY,                     // Gerudo Fortress Keys
    RCTYPE_BOSS_KEY,                   // Boss Keys
    RCTYPE_GANON_BOSS_KEY,             // Ganon's boss key
    RCTYPE_SHOP,                       // Shops
    RCTYPE_SCRUB,                      // Scrubs
    RCTYPE_MERCHANT,                   // Merchants
    RCTYPE_CHEST_GAME,                 // RANDOTODO replace this once we implement it, just using it to exclude for now
    RCTYPE_LINKS_POCKET,               // RANDOTODO this feels hacky, replace with better starting items
    RCTYPE_GOSSIP_STONE,               // RANDOTODO make these into event access
    RCTYPE_STATIC_HINT,                // RANDOTODO make these into event access
    RCTYPE_SONG_LOCATION,              // Song locations
    RCTYPE_BOSS_HEART_OR_OTHER_REWARD, // Boss heart container or lesser dungeon rewards (lens, ice arrow)
    RCTYPE_POT,                        // Pots
    RCTYPE_CRATE,                      // Crates
    RCTYPE_NLCRATE,                    // NL Crates
    RCTYPE_SMALL_CRATE,                // Small crates
    RCTYPE_DUNGEON_REWARD,             // Dungeon rewards (blue warps)
    RCTYPE_OCARINA,                    // Ocarina locations
    RCTYPE_BEEHIVE,                    // Beehives
    RCTYPE_FISH,                       // Fishes
    RCTYPE_FREESTANDING,               // Freestanding rupees and hearts
    RCTYPE_FAIRY,                      // Fairies
    RCTYPE_GRASS,                      // Grass
} RandomizerCheckType;

typedef enum { RCQUEST_VANILLA, RCQUEST_MQ, RCQUEST_BOTH } RandomizerCheckQuest;

typedef enum {
    RCAREA_KOKIRI_FOREST,
    RCAREA_LOST_WOODS,
    RCAREA_SACRED_FOREST_MEADOW,
    RCAREA_HYRULE_FIELD,
    RCAREA_LAKE_HYLIA,
    RCAREA_GERUDO_VALLEY,
    RCAREA_GERUDO_FORTRESS,
    RCAREA_WASTELAND,
    RCAREA_DESERT_COLOSSUS,
    RCAREA_MARKET,
    RCAREA_HYRULE_CASTLE,
    RCAREA_KAKARIKO_VILLAGE,
    RCAREA_GRAVEYARD,
    RCAREA_DEATH_MOUNTAIN_TRAIL,
    RCAREA_GORON_CITY,
    RCAREA_DEATH_MOUNTAIN_CRATER,
    RCAREA_ZORAS_RIVER,
    RCAREA_ZORAS_DOMAIN,
    RCAREA_ZORAS_FOUNTAIN,
    RCAREA_LON_LON_RANCH,
    RCAREA_DEKU_TREE,
    RCAREA_DODONGOS_CAVERN,
    RCAREA_JABU_JABUS_BELLY,
    RCAREA_FOREST_TEMPLE,
    RCAREA_FIRE_TEMPLE,
    RCAREA_WATER_TEMPLE,
    RCAREA_SPIRIT_TEMPLE,
    RCAREA_SHADOW_TEMPLE,
    RCAREA_BOTTOM_OF_THE_WELL,
    RCAREA_ICE_CAVERN,
    RCAREA_GERUDO_TRAINING_GROUND,
    RCAREA_GANONS_CASTLE,
    // If adding any more areas, Check Tracker will need a refactor
    RCAREA_INVALID
} RandomizerCheckArea;

// Check tracker check visibility categories
typedef enum {
    RCSHOW_UNCHECKED,
    RCSHOW_SEEN,
    RCSHOW_IDENTIFIED,
    RCSHOW_SCUMMED,
    RCSHOW_COLLECTED,
    RCSHOW_SAVED,
} RandomizerCheckStatus;

typedef enum {
    RR_NONE,
    RR_ROOT,
    RR_ROOT_EXITS,
    RR_CHILD_SPAWN,
    RR_ADULT_SPAWN,
    RR_MINUET_OF_FOREST_WARP,
    RR_BOLERO_OF_FIRE_WARP,
    RR_SERENADE_OF_WATER_WARP,
    RR_REQUIEM_OF_SPIRIT_WARP,
    RR_NOCTURNE_OF_SHADOW_WARP,
    RR_PRELUDE_OF_LIGHT_WARP,
    RR_KOKIRI_FOREST,
    RR_KF_LINKS_HOUSE,
    RR_KF_MIDOS_HOUSE,
    RR_KF_SARIAS_HOUSE,
    RR_KF_HOUSE_OF_TWINS,
    RR_KF_KNOW_IT_ALL_HOUSE,
    RR_KF_KOKIRI_SHOP,
    RR_KF_OUTSIDE_DEKU_TREE,
    RR_KF_STORMS_GROTTO,
    RR_THE_LOST_WOODS,
    RR_LW_BRIDGE_FROM_FOREST,
    RR_LW_BRIDGE,
    RR_LW_FOREST_EXIT,
    RR_LW_BEYOND_MIDO,
    RR_LW_NEAR_SHORTCUTS_GROTTO,
    RR_DEKU_THEATER,
    RR_LW_SCRUBS_GROTTO,
    RR_SFM_ENTRYWAY,
    RR_SACRED_FOREST_MEADOW,
    RR_SFM_WOLFOS_GROTTO,
    RR_SFM_FAIRY_GROTTO,
    RR_SFM_STORMS_GROTTO,
    RR_HYRULE_FIELD,
    RR_HF_SOUTHEAST_GROTTO,
    RR_HF_OPEN_GROTTO,
    RR_HF_INSIDE_FENCE_GROTTO,
    RR_HF_COW_GROTTO,
    RR_HF_COW_GROTTO_BEHIND_WEBS,
    RR_HF_NEAR_MARKET_GROTTO,
    RR_HF_FAIRY_GROTTO,
    RR_HF_NEAR_KAK_GROTTO,
    RR_HF_TEKTITE_GROTTO,
    RR_LAKE_HYLIA,
    RR_LH_FISHING_ISLAND,
    RR_LH_OWL_FLIGHT,
    RR_LH_LAB,
    RR_LH_FISHING_POND,
    RR_LH_GROTTO,
    RR_GERUDO_VALLEY,
    RR_GV_UPPER_STREAM,
    RR_GV_LOWER_STREAM,
    RR_GV_GROTTO_LEDGE,
    RR_GV_CRATE_LEDGE,
    RR_GV_OCTOROK_GROTTO,
    RR_GV_FORTRESS_SIDE,
    RR_GV_CARPENTER_TENT,
    RR_GV_STORMS_GROTTO,
    RR_GERUDO_FORTRESS,
    RR_GF_OUTSIDE_GATE,
    RR_GF_STORMS_GROTTO,
    RR_WASTELAND_NEAR_FORTRESS,
    RR_HAUNTED_WASTELAND,
    RR_WASTELAND_NEAR_COLOSSUS,
    RR_DESERT_COLOSSUS,
    RR_DESERT_COLOSSUS_OASIS,
    RR_DESERT_COLOSSUS_OUTSIDE_TEMPLE,
    RR_COLOSSUS_GREAT_FAIRY_FOUNTAIN,
    RR_COLOSSUS_GROTTO,
    RR_MARKET_ENTRANCE,
    RR_THE_MARKET,
    RR_MARKET_GUARD_HOUSE,
    RR_MARKET_BAZAAR,
    RR_MARKET_MASK_SHOP,
    RR_MARKET_SHOOTING_GALLERY,
    RR_MARKET_BOMBCHU_BOWLING,
    RR_MARKET_TREASURE_CHEST_GAME,
    RR_MARKET_POTION_SHOP,
    RR_MARKET_BACK_ALLEY,
    RR_MARKET_BOMBCHU_SHOP,
    RR_MARKET_DOG_LADY_HOUSE,
    RR_MARKET_MAN_IN_GREEN_HOUSE,
    RR_TOT_ENTRANCE,
    RR_TEMPLE_OF_TIME,
    RR_TOT_BEYOND_DOOR_OF_TIME,
    RR_CASTLE_GROUNDS,
    RR_CASTLE_GROUNDS_FROM_GANONS_CASTLE,
    RR_HYRULE_CASTLE_GROUNDS,
    RR_HC_GARDEN,
    RR_HC_GREAT_FAIRY_FOUNTAIN,
    RR_HC_STORMS_GROTTO,
    RR_HC_STORMS_GROTTO_BEHIND_WALLS,
    RR_GANONS_CASTLE_GROUNDS,
    RR_OGC_GREAT_FAIRY_FOUNTAIN,
    RR_GANONS_CASTLE_LEDGE,
    RR_KAKARIKO_VILLAGE,
    RR_KAK_CARPENTER_BOSS_HOUSE,
    RR_KAK_HOUSE_OF_SKULLTULA,
    RR_KAK_IMPAS_HOUSE,
    RR_KAK_IMPAS_LEDGE,
    RR_KAK_IMPAS_HOUSE_BACK,
    RR_KAK_WINDMILL,
    RR_KAK_BAZAAR,
    RR_KAK_SHOOTING_GALLERY,
    RR_KAK_POTION_SHOP_FRONT,
    RR_KAK_POTION_SHOP_BACK,
    RR_KAK_WATCHTOWER,
    RR_KAK_ROOFTOP,
    RR_KAK_IMPAS_ROOFTOP,
    RR_KAK_BEHIND_GATE,
    RR_KAK_BACKYARD,
    RR_KAK_WELL,
    RR_KAK_ODD_POTION_BUILDING,
    RR_KAK_REDEAD_GROTTO,
    RR_KAK_OPEN_GROTTO,
    RR_THE_GRAVEYARD,
    RR_GRAVEYARD_DAMPES_GRAVE,
    RR_GRAVEYARD_DAMPES_HOUSE,
    RR_GRAVEYARD_SHIELD_GRAVE,
    RR_GRAVEYARD_SHIELD_GRAVE_BACK,
    RR_GRAVEYARD_COMPOSERS_GRAVE,
    RR_GRAVEYARD_HEART_PIECE_GRAVE,
    RR_GRAVEYARD_WARP_PAD_REGION,
    RR_DEATH_MOUNTAIN_TRAIL,
    RR_DEATH_MOUNTAIN_SUMMIT,
    RR_DMT_OWL_FLIGHT,
    RR_DMT_GREAT_FAIRY_FOUNTAIN,
    RR_DMT_COW_GROTTO,
    RR_DMT_STORMS_GROTTO,
    RR_GORON_CITY,
    RR_GC_MEDIGORON,
    RR_GC_WOODS_WARP,
    RR_GC_DARUNIAS_CHAMBER,
    RR_GC_GROTTO_PLATFORM,
    RR_GC_SHOP,
    RR_GC_GROTTO,
    RR_DMC_UPPER_LOCAL,
    RR_DMC_CENTRAL_LOCAL,
    RR_DMC_LOWER_LOCAL,
    RR_DMC_LOWER_NEARBY,
    RR_DMC_UPPER_NEARBY,
    RR_DMC_CENTRAL_NEARBY,
    RR_DMC_LADDER_AREA_NEARBY,
    RR_DMC_UPPER_GROTTO,
    RR_DMC_HAMMER_GROTTO,
    RR_DMC_GREAT_FAIRY_FOUNTAIN,
    RR_DMC_DISTANT_PLATFORM,
    RR_ZR_FRONT,
    RR_ZORAS_RIVER,
    RR_ZR_BEHIND_WATERFALL,
    RR_ZR_OPEN_GROTTO,
    RR_ZR_FAIRY_GROTTO,
    RR_ZR_STORMS_GROTTO,
    RR_ZORAS_DOMAIN,
    RR_ZORAS_DOMAIN_ISLAND,
    RR_ZD_BEHIND_KING_ZORA,
    RR_ZD_SHOP,
    RR_ZD_STORMS_GROTTO,
    RR_ZORAS_FOUNTAIN,
    RR_ZF_ICEBERGS,
    RR_ZF_LAKEBED,
    RR_ZF_LEDGE,
    RR_ZF_HIDDEN_CAVE,
    RR_ZF_HIDDEN_LEDGE,
    RR_ZF_ROCK,
    RR_ZF_GREAT_FAIRY_FOUNTAIN,
    RR_LON_LON_RANCH,
    RR_LLR_TALONS_HOUSE,
    RR_LLR_STABLES,
    RR_LLR_TOWER,
    RR_LLR_GROTTO,

    RR_DEKU_TREE_ENTRYWAY,
    RR_DODONGOS_CAVERN_ENTRYWAY,
    RR_JABU_JABUS_BELLY_ENTRYWAY,
    RR_FOREST_TEMPLE_ENTRYWAY,
    RR_FIRE_TEMPLE_ENTRYWAY,
    RR_WATER_TEMPLE_ENTRYWAY,
    RR_SPIRIT_TEMPLE_ENTRYWAY,
    RR_SHADOW_TEMPLE_ENTRYWAY,
    RR_BOTTOM_OF_THE_WELL_ENTRYWAY,
    RR_ICE_CAVERN_ENTRYWAY,
    RR_GERUDO_TRAINING_GROUND_ENTRYWAY,
    RR_GANONS_CASTLE_ENTRYWAY,

    RR_DEKU_TREE_LOBBY,
    RR_DEKU_TREE_2F_MIDDLE_ROOM,
    RR_DEKU_TREE_SLINGSHOT_ROOM,
    RR_DEKU_TREE_COMPASS_ROOM,
    RR_DEKU_TREE_BASEMENT_LOWER,
    RR_DEKU_TREE_BASEMENT_SCRUB_ROOM,
    RR_DEKU_TREE_BASEMENT_WATER_ROOM_FRONT,
    RR_DEKU_TREE_BASEMENT_WATER_ROOM_BACK,
    RR_DEKU_TREE_BASEMENT_TORCH_ROOM,
    RR_DEKU_TREE_BASEMENT_BACK_LOBBY,
    RR_DEKU_TREE_BASEMENT_BACK_ROOM,
    RR_DEKU_TREE_BASEMENT_UPPER,
    RR_DEKU_TREE_OUTSIDE_BOSS_ROOM,

    RR_DEKU_TREE_MQ_1F,
    RR_DEKU_TREE_MQ_2F,
    RR_DEKU_TREE_MQ_3F,
    RR_DEKU_TREE_MQ_EYE_TARGET_ROOM,
    RR_DEKU_TREE_MQ_COMPASS_ROOM,
    RR_DEKU_TREE_MQ_PAST_BOULDER_VINES,
    RR_DEKU_TREE_MQ_BASEMENT,
    RR_DEKU_TREE_MQ_BASEMENT_SOUTHEAST_ROOM,
    RR_DEKU_TREE_MQ_BASEMENT_WATER_ROOM_FRONT,
    RR_DEKU_TREE_MQ_BASEMENT_WATER_ROOM_BACK,
    RR_DEKU_TREE_MQ_BASEMENT_SOUTHWEST_ROOM,
    RR_DEKU_TREE_MQ_BASEMENT_GRAVE_ROOM,
    RR_DEKU_TREE_MQ_BASEMENT_BACK_ROOM,
    RR_DEKU_TREE_MQ_BASEMENT_LEDGE,
    RR_DEKU_TREE_MQ_OUTSIDE_BOSS_ROOM,

    RR_DEKU_TREE_BOSS_ENTRYWAY,
    RR_DEKU_TREE_BOSS_ROOM,

    RR_DODONGOS_CAVERN_BEGINNING,
    RR_DODONGOS_CAVERN_LOBBY,
    RR_DODONGOS_CAVERN_LOBBY_SWITCH,
    RR_DODONGOS_CAVERN_SE_CORRIDOR,
    RR_DODONGOS_CAVERN_SE_ROOM,
    RR_DODONGOS_CAVERN_NEAR_LOWER_LIZALFOS,
    RR_DODONGOS_CAVERN_LOWER_LIZALFOS,
    RR_DODONGOS_CAVERN_DODONGO_ROOM,
    RR_DODONGOS_CAVERN_NEAR_DODONGO_ROOM,
    RR_DODONGOS_CAVERN_STAIRS_LOWER,
    RR_DODONGOS_CAVERN_STAIRS_UPPER,
    RR_DODONGOS_CAVERN_COMPASS_ROOM,
    RR_DODONGOS_CAVERN_ARMOS_ROOM,
    RR_DODONGOS_CAVERN_BOMB_ROOM_LOWER,
    RR_DODONGOS_CAVERN_2F_SIDE_ROOM,
    RR_DODONGOS_CAVERN_FIRST_SLINGSHOT_ROOM,
    RR_DODONGOS_CAVERN_UPPER_LIZALFOS,
    RR_DODONGOS_CAVERN_SECOND_SLINGSHOT_ROOM,
    RR_DODONGOS_CAVERN_BOMB_ROOM_UPPER,
    RR_DODONGOS_CAVERN_FAR_BRIDGE,
    RR_DODONGOS_CAVERN_BOSS_AREA,
    RR_DODONGOS_CAVERN_BACK_ROOM,

    RR_DODONGOS_CAVERN_MQ_BEGINNING,
    RR_DODONGOS_CAVERN_MQ_LOBBY,
    RR_DODONGOS_CAVERN_MQ_GOSSIP_STONE,
    RR_DODONGOS_CAVERN_MQ_MOUTH_SIDE_BRIDGE,
    RR_DODONGOS_CAVERN_MQ_STAIRS_PAST_MUD_WALL,
    RR_DODONGOS_CAVERN_MQ_STAIRS_LOWER,
    RR_DODONGOS_CAVERN_MQ_STAIRS_UPPER,
    RR_DODONGOS_CAVERN_MQ_STAIRS_PAST_BIG_SKULLTULAS,
    RR_DODONGOS_CAVERN_MQ_DODONGO_ROOM,
    RR_DODONGOS_CAVERN_MQ_TORCH_PUZZLE_LOWER,
    RR_DODONGOS_CAVERN_MQ_BIG_BLOCK_ROOM,
    RR_DODONGOS_CAVERN_MQ_LARVAE_ROOM,
    RR_DODONGOS_CAVERN_MQ_UPPER_LIZALFOS,
    RR_DODONGOS_CAVERN_MQ_TWO_FIRES_ROOM,
    RR_DODONGOS_CAVERN_MQ_TORCH_PUZZLE_UPPER,
    RR_DODONGOS_CAVERN_MQ_LOWER_RIGHT_SIDE,
    RR_DODONGOS_CAVERN_MQ_LOWER_LIZALFOS,
    RR_DODONGOS_CAVERN_MQ_POES_ROOM,
    RR_DODONGOS_CAVERN_MQ_MAD_SCRUB_ROOM,
    RR_DODONGOS_CAVERN_MQ_BEHIND_MOUTH,
    RR_DODONGOS_CAVERN_MQ_BACK_BEHIND_FIRE,
    RR_DODONGOS_CAVERN_MQ_BACK_SWITCH_GRAVE,
    RR_DODONGOS_CAVERN_BOSS_ENTRYWAY,
    RR_DODONGOS_CAVERN_BOSS_ROOM,

    RR_JABU_JABUS_BELLY_BEGINNING,
    RR_JABU_JABUS_BELLY_MAIN,
    RR_JABU_JABUS_BELLY_B1_NORTH,
    RR_JABU_JABUS_BELLY_WATER_SWITCH_ROOM_SOUTH,
    RR_JABU_JABUS_BELLY_WATER_SWITCH_ROOM_LEDGE,
    RR_JABU_JABUS_BELLY_COMPASS_ROOM,
    RR_JABU_JABUS_BELLY_BLUE_TENTACLE,
    RR_JABU_JABUS_BELLY_GREEN_TENTACLE,
    RR_JABU_JABUS_BELLY_BIGOCTO_LEDGE,
    RR_JABU_JABUS_BELLY_ABOVE_BIGOCTO,
    RR_JABU_JABUS_BELLY_LIFT_UPPER,
    RR_JABU_JABUS_BELLY_NEAR_BOSS_ROOM,

    RR_JABU_JABUS_BELLY_MQ_BEGINNING,
    RR_JABU_JABUS_BELLY_MQ_LIFT_ROOM,
    RR_JABU_JABUS_BELLY_MQ_LIFT_ROOM_EAST_LEDGE,
    RR_JABU_JABUS_BELLY_MQ_UNDERWATER_ALCOVE,
    RR_JABU_JABUS_BELLY_MQ_HOLES_ROOM,
    RR_JABU_JABUS_BELLY_MQ_WATER_SWITCH_ROOM,
    RR_JABU_JABUS_BELLY_MQ_FORKED_CORRIDOR,
    RR_JABU_JABUS_BELLY_MQ_WEST_FORKED_ROOMS,
    RR_JABU_JABUS_BELLY_MQ_INVISIBLE_KEESE_ROOM,
    RR_JABU_JABUS_BELLY_MQ_PAST_OCTO,
    RR_JABU_JABUS_BELLY_MQ_EAST_ROOM,

    RR_JABU_JABUS_BELLY_BOSS_ENTRYWAY,
    RR_JABU_JABUS_BELLY_BOSS_ROOM,

    RR_FOREST_TEMPLE_FIRST_ROOM,
    RR_FOREST_TEMPLE_SOUTH_CORRIDOR,
    RR_FOREST_TEMPLE_LOBBY,
    RR_FOREST_TEMPLE_NORTH_CORRIDOR,
    RR_FOREST_TEMPLE_LOWER_STALFOS,
    RR_FOREST_TEMPLE_NW_OUTDOORS_LOWER,
    RR_FOREST_TEMPLE_NW_OUTDOORS_UPPER,
    RR_FOREST_TEMPLE_NE_OUTDOORS_LOWER,
    RR_FOREST_TEMPLE_NE_OUTDOORS_UPPER,
    RR_FOREST_TEMPLE_MAP_ROOM,
    RR_FOREST_TEMPLE_SEWER,
    RR_FOREST_TEMPLE_BELOW_BOSS_KEY_CHEST,
    RR_FOREST_TEMPLE_FLOORMASTER_ROOM,
    RR_FOREST_TEMPLE_WEST_CORRIDOR,
    RR_FOREST_TEMPLE_BLOCK_PUSH_ROOM,
    RR_FOREST_TEMPLE_NW_CORRIDOR_TWISTED,
    RR_FOREST_TEMPLE_NW_CORRIDOR_STRAIGHTENED,
    RR_FOREST_TEMPLE_RED_POE_ROOM,
    RR_FOREST_TEMPLE_UPPER_STALFOS,
    RR_FOREST_TEMPLE_BLUE_POE_ROOM,
    RR_FOREST_TEMPLE_NE_CORRIDOR_STRAIGHTENED,
    RR_FOREST_TEMPLE_NE_CORRIDOR_TWISTED,
    RR_FOREST_TEMPLE_FROZEN_EYE_ROOM,
    RR_FOREST_TEMPLE_FALLING_ROOM,
    RR_FOREST_TEMPLE_GREEN_POE_ROOM,
    RR_FOREST_TEMPLE_EAST_CORRIDOR,
    RR_FOREST_TEMPLE_BOSS_REGION,

    RR_FOREST_TEMPLE_MQ_LOBBY,
    RR_FOREST_TEMPLE_MQ_CENTRAL_AREA,
    RR_FOREST_TEMPLE_MQ_WOLFOS_ROOM,
    RR_FOREST_TEMPLE_MQ_LOWER_BLOCK_PUZZLE,
    RR_FOREST_TEMPLE_MQ_MIDDLE_BLOCK_PUZZLE,
    RR_FOREST_TEMPLE_MQ_UPPER_BLOCK_PUZZLE,
    RR_FOREST_TEMPLE_MQ_STRAIGHT_HALLWAY,
    RR_FOREST_TEMPLE_MQ_FLOORMASTER_ROOM,
    RR_FOREST_TEMPLE_MQ_OUTDOOR_LEDGE,
    RR_FOREST_TEMPLE_MQ_NW_OUTDOORS,
    RR_FOREST_TEMPLE_MQ_NE_OUTDOORS,
    RR_FOREST_TEMPLE_MQ_OUTDOORS_TOP_LEDGES,
    RR_FOREST_TEMPLE_MQ_NE_OUTDOORS_LEDGE,
    RR_FOREST_TEMPLE_MQ_JOELLE_ROOM,
    RR_FOREST_TEMPLE_MQ_3_STALFOS_ROOM,
    RR_FOREST_TEMPLE_MQ_BETH_ROOM,
    RR_FOREST_TEMPLE_MQ_TORCH_SHOT_ROOM,
    RR_FOREST_TEMPLE_MQ_FALLING_ROOM,
    RR_FOREST_TEMPLE_MQ_AMY_ROOM,
    RR_FOREST_TEMPLE_MQ_BASEMENT,
    RR_FOREST_TEMPLE_MQ_BASEMENT_POT_ROOM,
    RR_FOREST_TEMPLE_MQ_BOSS_REGION,

    RR_FOREST_TEMPLE_BOSS_ENTRYWAY,
    RR_FOREST_TEMPLE_BOSS_ROOM,

    RR_FIRE_TEMPLE_FIRST_ROOM,
    RR_FIRE_TEMPLE_NEAR_BOSS_ROOM,
    RR_FIRE_TEMPLE_LOOP_ENEMIES,
    RR_FIRE_TEMPLE_LOOP_TILES,
    RR_FIRE_TEMPLE_LOOP_FLARE_DANCER,
    RR_FIRE_TEMPLE_LOOP_HAMMER_SWITCH,
    RR_FIRE_TEMPLE_LOOP_GORON_ROOM,
    RR_FIRE_TEMPLE_LOOP_EXIT,
    RR_FIRE_TEMPLE_BIG_LAVA_ROOM,
    RR_FIRE_TEMPLE_BIG_LAVA_ROOM_NORTH_GORON,
    RR_FIRE_TEMPLE_BIG_LAVA_ROOM_NORTH_TILES,
    RR_FIRE_TEMPLE_BIG_LAVA_ROOM_SOUTH_GORON,
    RR_FIRE_TEMPLE_FIRE_PILLAR_ROOM,
    RR_FIRE_TEMPLE_SHORTCUT_ROOM,
    RR_FIRE_TEMPLE_SHORTCUT_CLIMB,
    RR_FIRE_TEMPLE_BOULDER_MAZE_LOWER,
    RR_FIRE_TEMPLE_BOULDER_MAZE_LOWER_SIDE_ROOM,
    RR_FIRE_TEMPLE_EAST_CENTRAL_ROOM,
    RR_FIRE_TEMPLE_FIRE_WALL_CHASE,
    RR_FIRE_TEMPLE_MAP_AREA,
    RR_FIRE_TEMPLE_BOULDER_MAZE_UPPER,
    RR_FIRE_TEMPLE_SCARECROW_ROOM,
    RR_FIRE_TEMPLE_EAST_PEAK,
    RR_FIRE_TEMPLE_CORRIDOR,
    RR_FIRE_TEMPLE_FIRE_MAZE_ROOM,
    RR_FIRE_TEMPLE_FIRE_MAZE_UPPER,
    RR_FIRE_TEMPLE_FIRE_MAZE_SIDE_ROOM,
    RR_FIRE_TEMPLE_WEST_CENTRAL_LOWER,
    RR_FIRE_TEMPLE_WEST_CENTRAL_UPPER,
    RR_FIRE_TEMPLE_LATE_FIRE_MAZE,
    RR_FIRE_TEMPLE_UPPER_FLARE_DANCER,
    RR_FIRE_TEMPLE_WEST_CLIMB,
    RR_FIRE_TEMPLE_WEST_PEAK,
    RR_FIRE_TEMPLE_HAMMER_RETURN_PATH,
    RR_FIRE_TEMPLE_ABOVE_FIRE_MAZE,

    RR_FIRE_TEMPLE_MQ_FIRST_ROOM_LOWER,
    RR_FIRE_TEMPLE_MQ_FIRST_ROOM_UPPER,
    RR_FIRE_TEMPLE_MQ_MAP_ROOM_SOUTH,
    RR_FIRE_TEMPLE_MQ_STALFOS_ROOM,
    RR_FIRE_TEMPLE_MQ_IRON_KNUCKLE_ROOM,
    RR_FIRE_TEMPLE_MQ_LOWER_FLARE_DANCER,
    RR_FIRE_TEMPLE_MQ_MAP_ROOM_NORTH,
    RR_FIRE_TEMPLE_MQ_MAP_ROOM_CAGE,
    RR_FIRE_TEMPLE_MQ_NEAR_BOSS_ROOM,
    RR_FIRE_TEMPLE_MQ_NEAR_BOSS_ROOM_NORTH,
    RR_FIRE_TEMPLE_MQ_BIG_LAVA_ROOM,
    RR_FIRE_TEMPLE_MQ_TORCH_FIREWALL_ROOM,
    RR_FIRE_TEMPLE_MQ_ELEVATOR_ROOM,
    RR_FIRE_TEMPLE_MQ_BIG_TORCH_ROOM,
    RR_FIRE_TEMPLE_MQ_LOWER_MAZE,
    RR_FIRE_TEMPLE_MQ_LOWER_MAZE_CRATE_CAGE,
    RR_FIRE_TEMPLE_MQ_UPPER_MAZE,
    RR_FIRE_TEMPLE_MQ_UPPER_MAZE_BOX_CAGE,
    RR_FIRE_TEMPLE_MQ_MAZE_SHORTCUT,
    RR_FIRE_TEMPLE_MQ_MAZE_SHORTCUT_CAGE,
    RR_FIRE_TEMPLE_MQ_BURNING_BLOCK_CLIMB,
    RR_FIRE_TEMPLE_MQ_NARROW_PATH_ROOM,
    RR_FIRE_TEMPLE_MQ_HIGH_TORCH_ROOM,
    RR_FIRE_TEMPLE_MQ_SOUTH_FIRE_MAZE,
    RR_FIRE_TEMPLE_MQ_FIRE_MAZE_PLATFORMS,
    RR_FIRE_TEMPLE_MQ_NORTH_FIRE_MAZE,
    RR_FIRE_TEMPLE_MQ_WEST_FIRE_MAZE,
    RR_FIRE_TEMPLE_MQ_FIRE_MAZE_PAST_WALL,
    RR_FIRE_TEMPLE_MQ_UPPER_FLARE_DANCER,
    RR_FIRE_TEMPLE_MQ_SCARECROW_ROOM,
    RR_FIRE_TEMPLE_MQ_COLLAPSED_STAIRS,

    RR_FIRE_TEMPLE_BOSS_ENTRYWAY,
    RR_FIRE_TEMPLE_BOSS_ROOM,

    RR_WATER_TEMPLE_LOBBY,
    RR_WATER_TEMPLE_EAST_LOWER,
    RR_WATER_TEMPLE_MAP_ROOM,
    RR_WATER_TEMPLE_CRACKED_WALL,
    RR_WATER_TEMPLE_TORCH_ROOM,
    RR_WATER_TEMPLE_NORTH_LOWER,
    RR_WATER_TEMPLE_BOULDERS_LOWER,
    RR_WATER_TEMPLE_BLOCK_ROOM,
    RR_WATER_TEMPLE_JETS_ROOM,
    RR_WATER_TEMPLE_BOULDERS_UPPER,
    RR_WATER_TEMPLE_BOSS_KEY_ROOM,
    RR_WATER_TEMPLE_SOUTH_LOWER,
    RR_WATER_TEMPLE_WEST_LOWER,
    RR_WATER_TEMPLE_DRAGON_ROOM,
    RR_WATER_TEMPLE_CENTRAL_PILLAR_LOWER,
    RR_WATER_TEMPLE_CENTRAL_PILLAR_UPPER,
    RR_WATER_TEMPLE_CENTRAL_PILLAR_BASEMENT,
    RR_WATER_TEMPLE_EAST_MIDDLE,
    RR_WATER_TEMPLE_WEST_MIDDLE,
    RR_WATER_TEMPLE_HIGH_WATER,
    RR_WATER_TEMPLE_BLOCK_CORRIDOR,
    RR_WATER_TEMPLE_FALLING_PLATFORM_ROOM,
    RR_WATER_TEMPLE_DRAGON_PILLARS_ROOM,
    RR_WATER_TEMPLE_DARK_LINK_ROOM,
    RR_WATER_TEMPLE_LONGSHOT_ROOM,
    RR_WATER_TEMPLE_RIVER,
    RR_WATER_TEMPLE_PRE_BOSS_ROOM,

    RR_WATER_TEMPLE_MQ_3F_SOUTH_LEDGE,
    RR_WATER_TEMPLE_MQ_MAIN,
    RR_WATER_TEMPLE_MQ_3F_CENTRAL,
    RR_WATER_TEMPLE_MQ_2F_CENTRAL,
    RR_WATER_TEMPLE_MQ_2F_CENTRAL_HIGH,
    RR_WATER_TEMPLE_MQ_HIGH_EMBLEM,
    RR_WATER_TEMPLE_MQ_3F_NORTH_LEDGE,
    RR_WATER_TEMPLE_MQ_BOSS_DOOR,
    RR_WATER_TEMPLE_MQ_EAST_TOWER,
    RR_WATER_TEMPLE_MQ_EAST_TOWER_1F_ROOM,
    RR_WATER_TEMPLE_MQ_CENTRAL_PILLAR_1F,
    RR_WATER_TEMPLE_MQ_CENTRAL_PILLAR_2F,
    RR_WATER_TEMPLE_MQ_CENTRAL_PILLAR_HIGH,
    RR_WATER_TEMPLE_MQ_CENTRAL_PILLAR_B1,
    RR_WATER_TEMPLE_MQ_CENTRAL_PILLAR_B1_FINAL,
    RR_WATER_TEMPLE_MQ_BEHIND_BLUE_SWITCH_2F,
    RR_WATER_TEMPLE_MQ_BEHIND_BLUE_SWITCH_3F,
    RR_WATER_TEMPLE_MQ_STORAGE_ROOM,
    RR_WATER_TEMPLE_MQ_LIZALFOS_HALLWAY,
    RR_WATER_TEMPLE_MQ_LIZALFOS_CAGE,
    RR_WATER_TEMPLE_MQ_3F_EAST_LEDGE,
    RR_WATER_TEMPLE_MQ_WATERFALL,
    RR_WATER_TEMPLE_MQ_STALFOS_PIT,
    RR_WATER_TEMPLE_MQ_STALFOS_PIT_POTS,
    RR_WATER_TEMPLE_MQ_STALFOS_PIT_UPPER,
    RR_WATER_TEMPLE_MQ_AFTER_DARK_LINK,
    RR_WATER_TEMPLE_MQ_RIVER_SKULL,
    RR_WATER_TEMPLE_MQ_RIVER_POTS,
    RR_WATER_TEMPLE_MQ_DRAGON_ROOM_DOOR,
    RR_WATER_TEMPLE_MQ_DRAGON_ROOM_TUNNEL,
    RR_WATER_TEMPLE_MQ_DRAGON_ROOM_ALCOVE,
    RR_WATER_TEMPLE_MQ_BOSS_KEY_ROOM_SWITCH,
    RR_WATER_TEMPLE_MQ_BOSS_KEY_ROOM_PIT,
    RR_WATER_TEMPLE_MQ_BOSS_KEY_ROOM_CHEST,
    RR_WATER_TEMPLE_MQ_B1_GATE_SWITCH,
    RR_WATER_TEMPLE_MQ_TRIANGLE_TORCH_ROOM,
    RR_WATER_TEMPLE_MQ_TRIANGLE_TORCH_CAGE,
    RR_WATER_TEMPLE_MQ_CRATES_WHIRLPOOLS_ROOM,
    RR_WATER_TEMPLE_MQ_SINGLE_STALFOS_ROOM,
    RR_WATER_TEMPLE_MQ_4_TORCH_ROOM,
    RR_WATER_TEMPLE_MQ_DODONGO_ROOM,
    RR_WATER_TEMPLE_MQ_CRATES_WHIRLPOOLS_CAGE,

    RR_WATER_TEMPLE_BOSS_ENTRYWAY,
    RR_WATER_TEMPLE_BOSS_ROOM,

    RR_SPIRIT_TEMPLE_LOBBY,
    RR_SPIRIT_TEMPLE_CHILD,
    RR_SPIRIT_TEMPLE_CHILD_CLIMB,
    RR_SPIRIT_TEMPLE_EARLY_ADULT,
    RR_SPIRIT_TEMPLE_CENTRAL_CHAMBER,
    RR_SPIRIT_TEMPLE_OUTDOOR_HANDS,
    RR_SPIRIT_TEMPLE_BEYOND_CENTRAL_LOCKED_DOOR,
    RR_SPIRIT_TEMPLE_BEYOND_FINAL_LOCKED_DOOR,
    RR_SPIRIT_TEMPLE_INSIDE_STATUE_HEAD,

    RR_SPIRIT_TEMPLE_MQ_LOBBY,
    RR_SPIRIT_TEMPLE_MQ_1F_WEST,
    RR_SPIRIT_TEMPLE_MQ_1F_GIBDO_ROOM_NORTH,
    RR_SPIRIT_TEMPLE_MQ_1F_GIBDO_ROOM_SOUTH,
    RR_SPIRIT_TEMPLE_MQ_TURNTABLE_ROOM,
    RR_SPIRIT_TEMPLE_MQ_MAP_ROOM_NORTH,
    RR_SPIRIT_TEMPLE_MQ_MAP_ROOM_SOUTH,
    RR_SPIRIT_TEMPLE_MQ_WEST_1F_RUSTED_SWITCH,
    RR_SPIRIT_TEMPLE_MQ_UNDER_LIKE_LIKE,
    RR_SPIRIT_TEMPLE_MQ_BROKEN_WALL_ROOM,
    RR_SPIRIT_TEMPLE_MQ_STATUE_ROOM,
    RR_SPIRIT_TEMPLE_MQ_SUN_BLOCK_ROOM,
    RR_SPIRIT_TEMPLE_MQ_WEST_IRON_KNUCKLE,
    RR_SPIRIT_TEMPLE_MQ_SILVER_GAUNTLETS_HAND,
    RR_SPIRIT_TEMPLE_MQ_BIG_BLOCK_ROOM_SOUTH,
    RR_SPIRIT_TEMPLE_MQ_BIG_BLOCK_ROOM_NORTH,
    RR_SPIRIT_TEMPLE_MQ_STATUE_ROOM_EAST,
    RR_SPIRIT_TEMPLE_MQ_THREE_SUNS_ROOM_2F,
    RR_SPIRIT_TEMPLE_MQ_THREE_SUNS_ROOM_1F,
    RR_SPIRIT_TEMPLE_MQ_1F_EAST,
    RR_SPIRIT_TEMPLE_MQ_LEEVER_ROOM,
    RR_SPIRIT_TEMPLE_MQ_SYMPHONY_ROOM,
    RR_SPIRIT_TEMPLE_MQ_AFTER_SYMPHONY_ROOM,
    RR_SPIRIT_TEMPLE_MQ_FOUR_BEAMOS_ROOM,
    RR_SPIRIT_TEMPLE_MQ_SOT_SUN_ROOM,
    RR_SPIRIT_TEMPLE_MQ_EAST_STAIRS_TO_HAND,
    RR_SPIRIT_TEMPLE_MQ_EAST_IRON_KNUCKLE,
    RR_SPIRIT_TEMPLE_MQ_MIRROR_SHIELD_HAND,
    RR_SPIRIT_TEMPLE_MQ_3F_GIBDO_ROOM,
    RR_SPIRIT_TEMPLE_MQ_BIG_WALL,
    RR_SPIRIT_TEMPLE_MQ_4F_CENTRAL,
    RR_SPIRIT_TEMPLE_MQ_NINE_CHAIRS_ROOM,
    RR_SPIRIT_TEMPLE_MQ_BIG_MIRROR_ROOM,
    RR_SPIRIT_TEMPLE_MQ_BIG_MIRROR_CAVE,
    RR_SPIRIT_TEMPLE_MQ_INSIDE_STATUE_HEAD,

    RR_SPIRIT_TEMPLE_BOSS_ENTRYWAY,
    RR_SPIRIT_TEMPLE_BOSS_ROOM,

    RR_SHADOW_TEMPLE_BEGINNING,
    RR_SHADOW_TEMPLE_FIRST_BEAMOS,
    RR_SHADOW_TEMPLE_HUGE_PIT,
    RR_SHADOW_TEMPLE_WIND_TUNNEL,
    RR_SHADOW_TEMPLE_BEYOND_BOAT,

    RR_SHADOW_TEMPLE_MQ_ENTRYWAY,
    RR_SHADOW_TEMPLE_MQ_BEGINNING,
    RR_SHADOW_TEMPLE_MQ_SPINNER_ROOM,
    RR_SHADOW_TEMPLE_MQ_DEAD_HAND_AREA,
    RR_SHADOW_TEMPLE_MQ_FIRST_BEAMOS,
    RR_SHADOW_TEMPLE_MQ_B2_SPINNING_BLADE_ROOM,
    RR_SHADOW_TEMPLE_MQ_SHORTCUT_PATH,
    RR_SHADOW_TEMPLE_MQ_B2_TO_B3_CORRIDOR,
    RR_SHADOW_TEMPLE_MQ_UPPER_HUGE_PIT,
    RR_SHADOW_TEMPLE_MQ_INVISIBLE_BLADES_ROOM,
    RR_SHADOW_TEMPLE_MQ_LOWER_HUGE_PIT,
    RR_SHADOW_TEMPLE_MQ_STONE_UMBRELLA_ROOM,
    RR_SHADOW_TEMPLE_MQ_UPPER_STONE_UMBRELLA,
    RR_SHADOW_TEMPLE_MQ_FLOOR_SPIKES_ROOM,
    RR_SHADOW_TEMPLE_MQ_STALFOS_ROOM,
    RR_SHADOW_TEMPLE_MQ_WIND_TUNNEL,
    RR_SHADOW_TEMPLE_MQ_WIND_HINT_ROOM,
    RR_SHADOW_TEMPLE_MQ_B4_GIBDO_ROOM,
    RR_SHADOW_TEMPLE_MQ_DOCK,
    RR_SHADOW_TEMPLE_MQ_BEYOND_BOAT,
    RR_SHADOW_TEMPLE_MQ_ACROSS_CHASM,
    RR_SHADOW_TEMPLE_MQ_BOSS_DOOR,
    RR_SHADOW_TEMPLE_MQ_INVISIBLE_MAZE,
    RR_SHADOW_TEMPLE_MQ_SPIKE_WALLS_ROOM,

    RR_SHADOW_TEMPLE_BOSS_ENTRYWAY,
    RR_SHADOW_TEMPLE_BOSS_ROOM,

    RR_BOTTOM_OF_THE_WELL_PERIMETER,
    RR_BOTTOM_OF_THE_WELL_BEHIND_FAKE_WALLS,
    RR_BOTTOM_OF_THE_WELL_SOUTHWEST_ROOM,
    RR_BOTTOM_OF_THE_WELL_KEESE_BEAMOS_ROOM,
    RR_BOTTOM_OF_THE_WELL_LIKE_LIKE_CAGE,
    RR_BOTTOM_OF_THE_WELL_INNER_ROOMS,
    RR_BOTTOM_OF_THE_WELL_COFFIN_ROOM,
    RR_BOTTOM_OF_THE_WELL_DEAD_HAND_ROOM,
    RR_BOTTOM_OF_THE_WELL_BASEMENT,
    RR_BOTTOM_OF_THE_WELL_BASEMENT_USEFUL_BOMB_FLOWERS,
    RR_BOTTOM_OF_THE_WELL_BASEMENT_PLATFORM,

    RR_BOTTOM_OF_THE_WELL_MQ_PERIMETER,
    RR_BOTTOM_OF_THE_WELL_MQ_WEST_ROOM_SWITCH,
    RR_BOTTOM_OF_THE_WELL_MQ_COFFIN_ROOM,
    RR_BOTTOM_OF_THE_WELL_MQ_LOCKED_CAGE,
    RR_BOTTOM_OF_THE_WELL_MQ_DEAD_HAND_ROOM,
    RR_BOTTOM_OF_THE_WELL_MQ_MIDDLE,
    RR_BOTTOM_OF_THE_WELL_MQ_BASEMENT,
    RR_BOTTOM_OF_THE_WELL_MQ_BASEMENT_SWITCH_PLATFORM,

    RR_ICE_CAVERN_BEGINNING,
    RR_ICE_CAVERN_MAIN,

    RR_ICE_CAVERN_MQ_BEGINNING,
    RR_ICE_CAVERN_MQ_HUB,
    RR_ICE_CAVERN_MQ_MAP_ROOM,
    RR_ICE_CAVERN_MQ_SCARECROW_ROOM,
    RR_ICE_CAVERN_MQ_STALFOS_ROOM,
    RR_ICE_CAVERN_MQ_WEST_CORRIDOR,
    RR_ICE_CAVERN_MQ_COMPASS_ROOM,

    RR_GERUDO_TRAINING_GROUND_LOBBY,
    RR_GERUDO_TRAINING_GROUND_CENTRAL_MAZE,
    RR_GERUDO_TRAINING_GROUND_CENTRAL_MAZE_RIGHT,
    RR_GERUDO_TRAINING_GROUND_LAVA_ROOM,
    RR_GERUDO_TRAINING_GROUND_HAMMER_ROOM,
    RR_GERUDO_TRAINING_GROUND_EYE_STATUE_LOWER,
    RR_GERUDO_TRAINING_GROUND_EYE_STATUE_UPPER,
    RR_GERUDO_TRAINING_GROUND_HEAVY_BLOCK_ROOM,
    RR_GERUDO_TRAINING_GROUND_LIKE_LIKE_ROOM,

    RR_GERUDO_TRAINING_GROUND_MQ_LOBBY,
    RR_GERUDO_TRAINING_GROUND_MQ_MAZE_HIDDEN_ROOM,
    RR_GERUDO_TRAINING_GROUND_MQ_MAZE_FIRST_LOCK,
    RR_GERUDO_TRAINING_GROUND_MQ_MAZE_CENTER,
    RR_GERUDO_TRAINING_GROUND_MQ_SAND_ROOM,
    RR_GERUDO_TRAINING_GROUND_MQ_LEFT_SIDE,
    RR_GERUDO_TRAINING_GROUND_MQ_STALFOS_ROOM,
    RR_GERUDO_TRAINING_GROUND_MQ_BEHIND_BLOCK,
    RR_GERUDO_TRAINING_GROUND_MQ_STATUE_ROOM_LEDGE,
    RR_GERUDO_TRAINING_GROUND_MQ_MAGENTA_FIRE_ROOM,
    RR_GERUDO_TRAINING_GROUND_MQ_STATUE_ROOM,
    RR_GERUDO_TRAINING_GROUND_MQ_TORCH_SLUG_ROOM,
    RR_GERUDO_TRAINING_GROUND_MQ_SWITCH_LEDGE,
    RR_GERUDO_TRAINING_GROUND_MQ_LEDGE_SIDE_PLATFORMS,
    RR_GERUDO_TRAINING_GROUND_MQ_FURTHEST_PLATFORM,
    RR_GERUDO_TRAINING_GROUND_MQ_TORCH_SIDE_PLATFORMS,
    RR_GERUDO_TRAINING_GROUND_MQ_PLATFORMS_UNLIT_TORCH,
    RR_GERUDO_TRAINING_GROUND_MQ_DINOLFOS_ROOM,
    RR_GERUDO_TRAINING_GROUND_MQ_UNDERWATER,
    RR_GERUDO_TRAINING_GROUND_MQ_MAZE_RIGHT,

    RR_GANONS_CASTLE_LOBBY,
    RR_GANONS_CASTLE_MAIN,
    RR_GANONS_CASTLE_DEKU_SCRUBS,
    RR_GANONS_CASTLE_FOREST_TRIAL,
    RR_GANONS_CASTLE_FIRE_TRIAL,
    RR_GANONS_CASTLE_WATER_TRIAL,
    RR_GANONS_CASTLE_SHADOW_TRIAL,
    RR_GANONS_CASTLE_SPIRIT_TRIAL,
    RR_GANONS_CASTLE_LIGHT_TRIAL,

    RR_GANONS_CASTLE_MQ_LOBBY,
    RR_GANONS_CASTLE_MQ_MAIN,
    RR_GANONS_CASTLE_MQ_DEKU_SCRUBS,
    RR_GANONS_CASTLE_MQ_FOREST_TRIAL_STALFOS_ROOM,
    RR_GANONS_CASTLE_MQ_FOREST_TRIAL_BEAMOS_ROOM,
    RR_GANONS_CASTLE_MQ_FOREST_TRIAL_FINAL_ROOM,
    RR_GANONS_CASTLE_MQ_FIRE_TRIAL_MAIN_ROOM,
    RR_GANONS_CASTLE_MQ_FIRE_TRIAL_FINAL_ROOM,
    RR_GANONS_CASTLE_MQ_WATER_TRIAL_GEYSER_ROOM,
    RR_GANONS_CASTLE_MQ_WATER_TRIAL_BLOCK_ROOM,
    RR_GANONS_CASTLE_MQ_WATER_TRIAL_FINAL_ROOM,
    RR_GANONS_CASTLE_MQ_SHADOW_TRIAL_STARTING_LEDGE,
    RR_GANONS_CASTLE_MQ_SHADOW_TRIAL_CHEST_PLATFORM,
    RR_GANONS_CASTLE_MQ_SHADOW_TRIAL_MOVING_PLATFORM,
    RR_GANONS_CASTLE_MQ_SHADOW_TRIAL_BEAMOS_TORCH,
    RR_GANONS_CASTLE_MQ_SHADOW_TRIAL_FAR_SIDE,
    RR_GANONS_CASTLE_MQ_SHADOW_TRIAL_FINAL_ROOM,
    RR_GANONS_CASTLE_MQ_SPIRIT_TRIAL_CHAIRS_ROOM,
    RR_GANONS_CASTLE_MQ_SPIRIT_TRIAL_BEFORE_SWITCH,
    RR_GANONS_CASTLE_MQ_SPIRIT_TRIAL_AFTER_SWITCH,
    RR_GANONS_CASTLE_MQ_SPIRIT_TRIAL_FINAL_ROOM,
    RR_GANONS_CASTLE_MQ_LIGHT_TRIAL_DINOLFOS_ROOM,
    RR_GANONS_CASTLE_MQ_LIGHT_TRIAL_TRIFORCE_ROOM,
    RR_GANONS_CASTLE_MQ_LIGHT_TRIAL_BOULDER_ROOM_FRONT,
    RR_GANONS_CASTLE_MQ_LIGHT_TRIAL_BOULDER_ROOM_BACK,
    RR_GANONS_CASTLE_MQ_LIGHT_TRIAL_FINAL_ROOM,

    RR_GANONS_TOWER_ENTRYWAY,
    RR_GANONS_TOWER_FLOOR_1,
    RR_GANONS_TOWER_FLOOR_2,
    RR_GANONS_TOWER_FLOOR_3,
    RR_GANONS_TOWER_BEFORE_GANONDORF_LAIR,
    RR_GANONS_TOWER_GANONDORF_LAIR,
    RR_GANONS_CASTLE_ESCAPE,
    RR_GANONS_CASTLE_GANON_ARENA,

    RR_MARKER_AREAS_END, // Used for area key count

    // DUNGEONS
    RR_DEKU_TREE,
    RR_DODONGOS_CAVERN,
    RR_JABU_JABUS_BELLY,
    RR_FOREST_TEMPLE,
    RR_FIRE_TEMPLE,
    RR_WATER_TEMPLE,
    RR_SPIRIT_TEMPLE,
    RR_SHADOW_TEMPLE,
    RR_BOTTOM_OF_THE_WELL,
    RR_ICE_CAVERN,
    RR_GERUDO_TRAINING_GROUND,
    RR_GANONS_CASTLE,
    RR_MAX,
} RandomizerRegion;

typedef enum {
    RC_UNKNOWN_CHECK,
    RC_LINKS_POCKET,
    RC_QUEEN_GOHMA,
    RC_KING_DODONGO,
    RC_BARINADE,
    RC_PHANTOM_GANON,
    RC_VOLVAGIA,
    RC_MORPHA,
    RC_BONGO_BONGO,
    RC_TWINROVA,
    RC_GANON,
    RC_GIFT_FROM_RAURU,
    RC_SONG_FROM_IMPA,
    RC_SONG_FROM_MALON,
    RC_SONG_FROM_SARIA,
    RC_SONG_FROM_ROYAL_FAMILYS_TOMB,
    RC_SONG_FROM_OCARINA_OF_TIME,
    RC_SONG_FROM_WINDMILL,
    RC_SHEIK_IN_FOREST,
    RC_SHEIK_IN_CRATER,
    RC_SHEIK_IN_ICE_CAVERN,
    RC_SHEIK_AT_COLOSSUS,
    RC_SHEIK_IN_KAKARIKO,
    RC_SHEIK_AT_TEMPLE,
    RC_KF_MIDOS_TOP_LEFT_CHEST,
    RC_KF_MIDOS_TOP_RIGHT_CHEST,
    RC_KF_MIDOS_BOTTOM_LEFT_CHEST,
    RC_KF_MIDOS_BOTTOM_RIGHT_CHEST,
    RC_KF_KOKIRI_SWORD_CHEST,
    RC_KF_STORMS_GROTTO_CHEST,
    RC_KF_LINKS_HOUSE_COW,
    RC_KF_GS_KNOW_IT_ALL_HOUSE,
    RC_KF_GS_BEAN_PATCH,
    RC_KF_GS_HOUSE_OF_TWINS,
    RC_KF_SHOP_ITEM_1,
    RC_KF_SHOP_ITEM_2,
    RC_KF_SHOP_ITEM_3,
    RC_KF_SHOP_ITEM_4,
    RC_KF_SHOP_ITEM_5,
    RC_KF_SHOP_ITEM_6,
    RC_KF_SHOP_ITEM_7,
    RC_KF_SHOP_ITEM_8,
    RC_LW_GIFT_FROM_SARIA,
    RC_LW_OCARINA_MEMORY_GAME,
    RC_LW_TARGET_IN_WOODS,
    RC_LW_NEAR_SHORTCUTS_GROTTO_CHEST,
    RC_LW_TRADE_COJIRO,
    RC_DEKU_THEATER_SKULL_MASK,
    RC_DEKU_THEATER_MASK_OF_TRUTH,
    RC_LW_SKULL_KID,
    RC_LW_TRADE_ODD_POTION,
    RC_LW_DEKU_SCRUB_NEAR_BRIDGE,
    RC_LW_DEKU_SCRUB_NEAR_DEKU_THEATER_LEFT,
    RC_LW_DEKU_SCRUB_NEAR_DEKU_THEATER_RIGHT,
    RC_LW_DEKU_SCRUB_GROTTO_FRONT,
    RC_LW_DEKU_SCRUB_GROTTO_REAR,
    RC_LW_GS_BEAN_PATCH_NEAR_BRIDGE,
    RC_LW_GS_BEAN_PATCH_NEAR_THEATER,
    RC_LW_GS_ABOVE_THEATER,
    RC_SFM_WOLFOS_GROTTO_CHEST,
    RC_SFM_DEKU_SCRUB_GROTTO_FRONT,
    RC_SFM_DEKU_SCRUB_GROTTO_REAR,
    RC_SFM_GS,
    RC_HF_OCARINA_OF_TIME_ITEM,
    RC_HF_NEAR_MARKET_GROTTO_CHEST,
    RC_HF_TEKTITE_GROTTO_FREESTANDING_POH,
    RC_HF_SOUTHEAST_GROTTO_CHEST,
    RC_HF_OPEN_GROTTO_CHEST,
    RC_HF_DEKU_SCRUB_GROTTO,
    RC_HF_COW_GROTTO_COW,
    RC_HF_GS_COW_GROTTO,
    RC_HF_GS_NEAR_KAK_GROTTO,
    RC_MARKET_SHOOTING_GALLERY_REWARD,
    RC_MARKET_BOMBCHU_BOWLING_FIRST_PRIZE,
    RC_MARKET_BOMBCHU_BOWLING_SECOND_PRIZE,
    RC_MARKET_LOST_DOG,
    RC_MARKET_TREASURE_CHEST_GAME_REWARD,
    RC_MARKET_10_BIG_POES,
    RC_MARKET_TREASURE_CHEST_GAME_ITEM_1,
    RC_MARKET_TREASURE_CHEST_GAME_ITEM_2,
    RC_MARKET_TREASURE_CHEST_GAME_ITEM_3,
    RC_MARKET_TREASURE_CHEST_GAME_ITEM_4,
    RC_MARKET_TREASURE_CHEST_GAME_ITEM_5,
    RC_MARKET_TREASURE_CHEST_GAME_KEY_1,
    RC_MARKET_TREASURE_CHEST_GAME_KEY_2,
    RC_MARKET_TREASURE_CHEST_GAME_KEY_3,
    RC_MARKET_TREASURE_CHEST_GAME_KEY_4,
    RC_MARKET_TREASURE_CHEST_GAME_KEY_5,
    RC_MARKET_GS_GUARD_HOUSE,
    RC_MARKET_BAZAAR_ITEM_1,
    RC_MARKET_BAZAAR_ITEM_2,
    RC_MARKET_BAZAAR_ITEM_3,
    RC_MARKET_BAZAAR_ITEM_4,
    RC_MARKET_BAZAAR_ITEM_5,
    RC_MARKET_BAZAAR_ITEM_6,
    RC_MARKET_BAZAAR_ITEM_7,
    RC_MARKET_BAZAAR_ITEM_8,
    RC_MARKET_POTION_SHOP_ITEM_1,
    RC_MARKET_POTION_SHOP_ITEM_2,
    RC_MARKET_POTION_SHOP_ITEM_3,
    RC_MARKET_POTION_SHOP_ITEM_4,
    RC_MARKET_POTION_SHOP_ITEM_5,
    RC_MARKET_POTION_SHOP_ITEM_6,
    RC_MARKET_POTION_SHOP_ITEM_7,
    RC_MARKET_POTION_SHOP_ITEM_8,
    RC_MARKET_BOMBCHU_SHOP_ITEM_1,
    RC_MARKET_BOMBCHU_SHOP_ITEM_2,
    RC_MARKET_BOMBCHU_SHOP_ITEM_3,
    RC_MARKET_BOMBCHU_SHOP_ITEM_4,
    RC_MARKET_BOMBCHU_SHOP_ITEM_5,
    RC_MARKET_BOMBCHU_SHOP_ITEM_6,
    RC_MARKET_BOMBCHU_SHOP_ITEM_7,
    RC_MARKET_BOMBCHU_SHOP_ITEM_8,
    RC_TOT_LIGHT_ARROWS_CUTSCENE,
    RC_TOT_MASTER_SWORD,
    RC_HC_MALON_EGG,
    RC_HC_ZELDAS_LETTER,
    RC_HC_GREAT_FAIRY_REWARD,
    RC_HC_GS_TREE,
    RC_HC_GS_STORMS_GROTTO,
    RC_LLR_TALONS_CHICKENS,
    RC_LLR_FREESTANDING_POH,
    RC_LLR_DEKU_SCRUB_GROTTO_LEFT,
    RC_LLR_DEKU_SCRUB_GROTTO_CENTER,
    RC_LLR_DEKU_SCRUB_GROTTO_RIGHT,
    RC_LLR_STABLES_LEFT_COW,
    RC_LLR_STABLES_RIGHT_COW,
    RC_LLR_TOWER_LEFT_COW,
    RC_LLR_TOWER_RIGHT_COW,
    RC_LLR_GS_HOUSE_WINDOW,
    RC_LLR_GS_TREE,
    RC_LLR_GS_RAIN_SHED,
    RC_LLR_GS_BACK_WALL,
    RC_KAK_ANJU_AS_CHILD,
    RC_KAK_ANJU_AS_ADULT,
    RC_KAK_IMPAS_HOUSE_FREESTANDING_POH,
    RC_KAK_WINDMILL_FREESTANDING_POH,
    RC_KAK_MAN_ON_ROOF,
    RC_KAK_OPEN_GROTTO_CHEST,
    RC_KAK_REDEAD_GROTTO_CHEST,
    RC_KAK_SHOOTING_GALLERY_REWARD,
    RC_KAK_TRADE_ODD_MUSHROOM,
    RC_KAK_GRANNYS_SHOP,
    RC_KAK_TRADE_POCKET_CUCCO,
    RC_KAK_10_GOLD_SKULLTULA_REWARD,
    RC_KAK_20_GOLD_SKULLTULA_REWARD,
    RC_KAK_30_GOLD_SKULLTULA_REWARD,
    RC_KAK_40_GOLD_SKULLTULA_REWARD,
    RC_KAK_50_GOLD_SKULLTULA_REWARD,
    RC_KAK_100_GOLD_SKULLTULA_REWARD,
    RC_KAK_IMPAS_HOUSE_COW,
    RC_KAK_GS_TREE,
    RC_KAK_GS_GUARDS_HOUSE,
    RC_KAK_GS_WATCHTOWER,
    RC_KAK_GS_SKULLTULA_HOUSE,
    RC_KAK_GS_HOUSE_UNDER_CONSTRUCTION,
    RC_KAK_GS_ABOVE_IMPAS_HOUSE,
    RC_KAK_BAZAAR_ITEM_1,
    RC_KAK_BAZAAR_ITEM_2,
    RC_KAK_BAZAAR_ITEM_3,
    RC_KAK_BAZAAR_ITEM_4,
    RC_KAK_BAZAAR_ITEM_5,
    RC_KAK_BAZAAR_ITEM_6,
    RC_KAK_BAZAAR_ITEM_7,
    RC_KAK_BAZAAR_ITEM_8,
    RC_KAK_POTION_SHOP_ITEM_1,
    RC_KAK_POTION_SHOP_ITEM_2,
    RC_KAK_POTION_SHOP_ITEM_3,
    RC_KAK_POTION_SHOP_ITEM_4,
    RC_KAK_POTION_SHOP_ITEM_5,
    RC_KAK_POTION_SHOP_ITEM_6,
    RC_KAK_POTION_SHOP_ITEM_7,
    RC_KAK_POTION_SHOP_ITEM_8,
    RC_GRAVEYARD_SHIELD_GRAVE_CHEST,
    RC_GRAVEYARD_HEART_PIECE_GRAVE_CHEST,
    RC_GRAVEYARD_ROYAL_FAMILYS_TOMB_CHEST,
    RC_GRAVEYARD_FREESTANDING_POH,
    RC_GRAVEYARD_DAMPE_GRAVEDIGGING_TOUR,
    RC_GRAVEYARD_HOOKSHOT_CHEST,
    RC_GRAVEYARD_DAMPE_RACE_FREESTANDING_POH,
    RC_GRAVEYARD_GS_BEAN_PATCH,
    RC_GRAVEYARD_GS_WALL,
    RC_DMT_FREESTANDING_POH,
    RC_DMT_CHEST,
    RC_DMT_STORMS_GROTTO_CHEST,
    RC_DMT_TRADE_BROKEN_SWORD,
    RC_DMT_TRADE_EYEDROPS,
    RC_DMT_TRADE_CLAIM_CHECK,
    RC_DMT_GREAT_FAIRY_REWARD,
    RC_DMT_COW_GROTTO_COW,
    RC_DMT_GS_NEAR_KAK,
    RC_DMT_GS_BEAN_PATCH,
    RC_DMT_GS_ABOVE_DODONGOS_CAVERN,
    RC_DMT_GS_FALLING_ROCKS_PATH,
    RC_GC_DARUNIAS_JOY,
    RC_GC_POT_FREESTANDING_POH,
    RC_GC_ROLLING_GORON_AS_CHILD,
    RC_GC_ROLLING_GORON_AS_ADULT,
    RC_GC_MEDIGORON,
    RC_GC_MAZE_LEFT_CHEST,
    RC_GC_MAZE_RIGHT_CHEST,
    RC_GC_MAZE_CENTER_CHEST,
    RC_GC_DEKU_SCRUB_GROTTO_LEFT,
    RC_GC_DEKU_SCRUB_GROTTO_CENTER,
    RC_GC_DEKU_SCRUB_GROTTO_RIGHT,
    RC_GC_GS_CENTER_PLATFORM,
    RC_GC_GS_BOULDER_MAZE,
    RC_GC_SHOP_ITEM_1,
    RC_GC_SHOP_ITEM_2,
    RC_GC_SHOP_ITEM_3,
    RC_GC_SHOP_ITEM_4,
    RC_GC_SHOP_ITEM_5,
    RC_GC_SHOP_ITEM_6,
    RC_GC_SHOP_ITEM_7,
    RC_GC_SHOP_ITEM_8,
    RC_DMC_VOLCANO_FREESTANDING_POH,
    RC_DMC_WALL_FREESTANDING_POH,
    RC_DMC_UPPER_GROTTO_CHEST,
    RC_DMC_GREAT_FAIRY_REWARD,
    RC_DMC_DEKU_SCRUB,
    RC_DMC_DEKU_SCRUB_GROTTO_LEFT,
    RC_DMC_DEKU_SCRUB_GROTTO_CENTER,
    RC_DMC_DEKU_SCRUB_GROTTO_RIGHT,
    RC_DMC_GS_CRATE,
    RC_DMC_GS_BEAN_PATCH,
    RC_ZR_MAGIC_BEAN_SALESMAN,
    RC_ZR_OPEN_GROTTO_CHEST,
    RC_ZR_FROGS_ZELDAS_LULLABY,
    RC_ZR_FROGS_EPONAS_SONG,
    RC_ZR_FROGS_SARIAS_SONG,
    RC_ZR_FROGS_SUNS_SONG,
    RC_ZR_FROGS_SONG_OF_TIME,
    RC_ZR_FROGS_IN_THE_RAIN,
    RC_ZR_FROGS_OCARINA_GAME,
    RC_ZR_NEAR_OPEN_GROTTO_FREESTANDING_POH,
    RC_ZR_NEAR_DOMAIN_FREESTANDING_POH,
    RC_ZR_DEKU_SCRUB_GROTTO_FRONT,
    RC_ZR_DEKU_SCRUB_GROTTO_REAR,
    RC_ZR_GS_TREE,
    RC_ZR_GS_LADDER,
    RC_ZR_GS_NEAR_RAISED_GROTTOS,
    RC_ZR_GS_ABOVE_BRIDGE,
    RC_ZD_DIVING_MINIGAME,
    RC_ZD_CHEST,
    RC_ZD_KING_ZORA_THAWED,
    RC_ZD_TRADE_PRESCRIPTION,
    RC_ZD_GS_FROZEN_WATERFALL,
    RC_ZD_SHOP_ITEM_1,
    RC_ZD_SHOP_ITEM_2,
    RC_ZD_SHOP_ITEM_3,
    RC_ZD_SHOP_ITEM_4,
    RC_ZD_SHOP_ITEM_5,
    RC_ZD_SHOP_ITEM_6,
    RC_ZD_SHOP_ITEM_7,
    RC_ZD_SHOP_ITEM_8,
    RC_ZF_GREAT_FAIRY_REWARD,
    RC_ZF_ICEBERG_FREESTANDING_POH,
    RC_ZF_BOTTOM_FREESTANDING_POH,
    RC_ZF_GS_ABOVE_THE_LOG,
    RC_ZF_GS_TREE,
    RC_ZF_GS_HIDDEN_CAVE,
    RC_LH_UNDERWATER_ITEM,
    RC_LH_CHILD_FISHING,
    RC_LH_CHILD_FISH_1,
    RC_LH_CHILD_FISH_2,
    RC_LH_CHILD_FISH_3,
    RC_LH_CHILD_FISH_4,
    RC_LH_CHILD_FISH_5,
    RC_LH_CHILD_FISH_6,
    RC_LH_CHILD_FISH_7,
    RC_LH_CHILD_FISH_8,
    RC_LH_CHILD_FISH_9,
    RC_LH_CHILD_FISH_10,
    RC_LH_CHILD_FISH_11,
    RC_LH_CHILD_FISH_12,
    RC_LH_CHILD_FISH_13,
    RC_LH_CHILD_FISH_14,
    RC_LH_CHILD_FISH_15,
    RC_LH_CHILD_LOACH_1,
    RC_LH_CHILD_LOACH_2,
    RC_LH_ADULT_FISHING,
    RC_LH_HYRULE_LOACH,
    RC_LH_ADULT_FISH_1,
    RC_LH_ADULT_FISH_2,
    RC_LH_ADULT_FISH_3,
    RC_LH_ADULT_FISH_4,
    RC_LH_ADULT_FISH_5,
    RC_LH_ADULT_FISH_6,
    RC_LH_ADULT_FISH_7,
    RC_LH_ADULT_FISH_8,
    RC_LH_ADULT_FISH_9,
    RC_LH_ADULT_FISH_10,
    RC_LH_ADULT_FISH_11,
    RC_LH_ADULT_FISH_12,
    RC_LH_ADULT_FISH_13,
    RC_LH_ADULT_FISH_14,
    RC_LH_ADULT_FISH_15,
    RC_LH_ADULT_LOACH,
    RC_LH_LAB_DIVE,
    RC_LH_TRADE_FROG,
    RC_LH_FREESTANDING_POH,
    RC_LH_SUN,
    RC_LH_DEKU_SCRUB_GROTTO_LEFT,
    RC_LH_DEKU_SCRUB_GROTTO_CENTER,
    RC_LH_DEKU_SCRUB_GROTTO_RIGHT,
    RC_LH_GS_BEAN_PATCH,
    RC_LH_GS_LAB_WALL,
    RC_LH_GS_SMALL_ISLAND,
    RC_LH_GS_LAB_CRATE,
    RC_LH_GS_TREE,
    RC_GV_CRATE_FREESTANDING_POH,
    RC_GV_WATERFALL_FREESTANDING_POH,
    RC_GV_CHEST,
    RC_GV_TRADE_SAW,
    RC_GV_DEKU_SCRUB_GROTTO_FRONT,
    RC_GV_DEKU_SCRUB_GROTTO_REAR,
    RC_GV_COW,
    RC_GV_GS_SMALL_BRIDGE,
    RC_GV_GS_BEAN_PATCH,
    RC_GV_GS_BEHIND_TENT,
    RC_GV_GS_PILLAR,
    RC_GF_CHEST,
    RC_GF_HBA_1000_POINTS,
    RC_GF_HBA_1500_POINTS,
    RC_GF_GERUDO_MEMBERSHIP_CARD,
    RC_GF_NORTH_F1_CARPENTER,
    RC_GF_NORTH_F2_CARPENTER,
    RC_GF_SOUTH_F1_CARPENTER,
    RC_GF_SOUTH_F2_CARPENTER,
    RC_GF_GS_TOP_FLOOR,
    RC_GF_GS_ARCHERY_RANGE,
    RC_HIDEOUT_JAIL_GUARD_1_TORCH,
    RC_HIDEOUT_JAIL_GUARD_2_TORCHES,
    RC_HIDEOUT_JAIL_GUARD_3_TORCHES,
    RC_HIDEOUT_JAIL_GUARD_4_TORCHES,
    RC_HIDEOUT_GERUDO_MEMBERSHIP_CARD,
    RC_WASTELAND_BOMBCHU_SALESMAN,
    RC_WASTELAND_CHEST,
    RC_WASTELAND_GS,
    RC_COLOSSUS_GREAT_FAIRY_REWARD,
    RC_COLOSSUS_FREESTANDING_POH,
    RC_COLOSSUS_DEKU_SCRUB_GROTTO_FRONT,
    RC_COLOSSUS_DEKU_SCRUB_GROTTO_REAR,
    RC_COLOSSUS_GS_BEAN_PATCH,
    RC_COLOSSUS_GS_TREE,
    RC_COLOSSUS_GS_HILL,
    RC_OGC_GREAT_FAIRY_REWARD,
    RC_OGC_GS,
    RC_DEKU_TREE_MAP_CHEST,
    RC_DEKU_TREE_SLINGSHOT_ROOM_SIDE_CHEST,
    RC_DEKU_TREE_SLINGSHOT_CHEST,
    RC_DEKU_TREE_COMPASS_CHEST,
    RC_DEKU_TREE_COMPASS_ROOM_SIDE_CHEST,
    RC_DEKU_TREE_BASEMENT_CHEST,
    RC_DEKU_TREE_GS_COMPASS_ROOM,
    RC_DEKU_TREE_GS_BASEMENT_VINES,
    RC_DEKU_TREE_GS_BASEMENT_GATE,
    RC_DEKU_TREE_GS_BASEMENT_BACK_ROOM,
    RC_DEKU_TREE_MQ_MAP_CHEST,
    RC_DEKU_TREE_MQ_SLINGSHOT_CHEST,
    RC_DEKU_TREE_MQ_SLINGSHOT_ROOM_BACK_CHEST,
    RC_DEKU_TREE_MQ_COMPASS_CHEST,
    RC_DEKU_TREE_MQ_BASEMENT_CHEST,
    RC_DEKU_TREE_MQ_BEFORE_SPINNING_LOG_CHEST,
    RC_DEKU_TREE_MQ_AFTER_SPINNING_LOG_CHEST,
    RC_DEKU_TREE_MQ_DEKU_SCRUB,
    RC_DEKU_TREE_MQ_GS_LOBBY,
    RC_DEKU_TREE_MQ_GS_PAST_BOULDER_VINES,
    RC_DEKU_TREE_MQ_GS_BASEMENT_GRAVES_ROOM,
    RC_DEKU_TREE_MQ_GS_BASEMENT_BACK_ROOM,
    RC_DEKU_TREE_QUEEN_GOHMA_HEART,
    RC_DODONGOS_CAVERN_MAP_CHEST,
    RC_DODONGOS_CAVERN_COMPASS_CHEST,
    RC_DODONGOS_CAVERN_BOMB_FLOWER_PLATFORM_CHEST,
    RC_DODONGOS_CAVERN_BOMB_BAG_CHEST,
    RC_DODONGOS_CAVERN_END_OF_BRIDGE_CHEST,
    RC_DODONGOS_CAVERN_DEKU_SCRUB_SIDE_ROOM_NEAR_DODONGOS,
    RC_DODONGOS_CAVERN_DEKU_SCRUB_LOBBY,
    RC_DODONGOS_CAVERN_DEKU_SCRUB_NEAR_BOMB_BAG_LEFT,
    RC_DODONGOS_CAVERN_DEKU_SCRUB_NEAR_BOMB_BAG_RIGHT,
    RC_DODONGOS_CAVERN_GS_SIDE_ROOM_NEAR_LOWER_LIZALFOS,
    RC_DODONGOS_CAVERN_GS_SCARECROW,
    RC_DODONGOS_CAVERN_GS_ALCOVE_ABOVE_STAIRS,
    RC_DODONGOS_CAVERN_GS_VINES_ABOVE_STAIRS,
    RC_DODONGOS_CAVERN_GS_BACK_ROOM,
    RC_DODONGOS_CAVERN_MQ_MAP_CHEST,
    RC_DODONGOS_CAVERN_MQ_BOMB_BAG_CHEST,
    RC_DODONGOS_CAVERN_MQ_TORCH_PUZZLE_ROOM_CHEST,
    RC_DODONGOS_CAVERN_MQ_LARVAE_ROOM_CHEST,
    RC_DODONGOS_CAVERN_MQ_COMPASS_CHEST,
    RC_DODONGOS_CAVERN_MQ_UNDER_GRAVE_CHEST,
    RC_DODONGOS_CAVERN_MQ_DEKU_SCRUB_LOBBY_FRONT,
    RC_DODONGOS_CAVERN_MQ_DEKU_SCRUB_LOBBY_REAR,
    RC_DODONGOS_CAVERN_MQ_DEKU_SCRUB_SIDE_ROOM_NEAR_LOWER_LIZALFOS,
    RC_DODONGOS_CAVERN_MQ_DEKU_SCRUB_STAIRCASE,
    RC_DODONGOS_CAVERN_MQ_GS_SCRUB_ROOM,
    RC_DODONGOS_CAVERN_MQ_GS_LARVAE_ROOM,
    RC_DODONGOS_CAVERN_MQ_GS_LIZALFOS_ROOM,
    RC_DODONGOS_CAVERN_MQ_GS_SONG_OF_TIME_BLOCK_ROOM,
    RC_DODONGOS_CAVERN_MQ_GS_BACK_AREA,
    RC_DODONGOS_CAVERN_BOSS_ROOM_CHEST,
    RC_DODONGOS_CAVERN_KING_DODONGO_HEART,
    RC_JABU_JABUS_BELLY_BOOMERANG_CHEST,
    RC_JABU_JABUS_BELLY_MAP_CHEST,
    RC_JABU_JABUS_BELLY_COMPASS_CHEST,
    RC_JABU_JABUS_BELLY_DEKU_SCRUB,
    RC_JABU_JABUS_BELLY_GS_WATER_SWITCH_ROOM,
    RC_JABU_JABUS_BELLY_GS_LOBBY_BASEMENT_LOWER,
    RC_JABU_JABUS_BELLY_GS_LOBBY_BASEMENT_UPPER,
    RC_JABU_JABUS_BELLY_GS_NEAR_BOSS,
    RC_JABU_JABUS_BELLY_MQ_MAP_CHEST,
    RC_JABU_JABUS_BELLY_MQ_FIRST_ROOM_SIDE_CHEST,
    RC_JABU_JABUS_BELLY_MQ_SECOND_ROOM_LOWER_CHEST,
    RC_JABU_JABUS_BELLY_MQ_COMPASS_CHEST,
    RC_JABU_JABUS_BELLY_MQ_BASEMENT_NEAR_SWITCHES_CHEST,
    RC_JABU_JABUS_BELLY_MQ_BASEMENT_NEAR_VINES_CHEST,
    RC_JABU_JABUS_BELLY_MQ_BOOMERANG_ROOM_SMALL_CHEST,
    RC_JABU_JABUS_BELLY_MQ_BOOMERANG_CHEST,
    RC_JABU_JABUS_BELLY_MQ_FALLING_LIKE_LIKE_ROOM_CHEST,
    RC_JABU_JABUS_BELLY_MQ_SECOND_ROOM_UPPER_CHEST,
    RC_JABU_JABUS_BELLY_MQ_NEAR_BOSS_CHEST,
    RC_JABU_JABUS_BELLY_MQ_COW,
    RC_JABU_JABUS_BELLY_MQ_GS_BOOMERANG_CHEST_ROOM,
    RC_JABU_JABUS_BELLY_MQ_GS_TAILPASARAN_ROOM,
    RC_JABU_JABUS_BELLY_MQ_GS_INVISIBLE_ENEMIES_ROOM,
    RC_JABU_JABUS_BELLY_MQ_GS_NEAR_BOSS,
    RC_JABU_JABUS_BELLY_BARINADE_HEART,
    RC_BOTTOM_OF_THE_WELL_FRONT_LEFT_FAKE_WALL_CHEST,
    RC_BOTTOM_OF_THE_WELL_FRONT_CENTER_BOMBABLE_CHEST,
    RC_BOTTOM_OF_THE_WELL_BACK_LEFT_BOMBABLE_CHEST,
    RC_BOTTOM_OF_THE_WELL_UNDERWATER_LEFT_CHEST,
    RC_BOTTOM_OF_THE_WELL_FREESTANDING_KEY,
    RC_BOTTOM_OF_THE_WELL_COMPASS_CHEST,
    RC_BOTTOM_OF_THE_WELL_CENTER_SKULLTULA_CHEST,
    RC_BOTTOM_OF_THE_WELL_RIGHT_BOTTOM_FAKE_WALL_CHEST,
    RC_BOTTOM_OF_THE_WELL_FIRE_KEESE_CHEST,
    RC_BOTTOM_OF_THE_WELL_LIKE_LIKE_CHEST,
    RC_BOTTOM_OF_THE_WELL_MAP_CHEST,
    RC_BOTTOM_OF_THE_WELL_UNDERWATER_FRONT_CHEST,
    RC_BOTTOM_OF_THE_WELL_INVISIBLE_CHEST,
    RC_BOTTOM_OF_THE_WELL_LENS_OF_TRUTH_CHEST,
    RC_BOTTOM_OF_THE_WELL_GS_WEST_INNER_ROOM,
    RC_BOTTOM_OF_THE_WELL_GS_EAST_INNER_ROOM,
    RC_BOTTOM_OF_THE_WELL_GS_LIKE_LIKE_CAGE,
    RC_BOTTOM_OF_THE_WELL_MQ_MAP_CHEST,
    RC_BOTTOM_OF_THE_WELL_MQ_EAST_INNER_ROOM_FREESTANDING_KEY,
    RC_BOTTOM_OF_THE_WELL_MQ_COMPASS_CHEST,
    RC_BOTTOM_OF_THE_WELL_MQ_DEAD_HAND_FREESTANDING_KEY,
    RC_BOTTOM_OF_THE_WELL_MQ_LENS_OF_TRUTH_CHEST,
    RC_BOTTOM_OF_THE_WELL_MQ_GS_COFFIN_ROOM,
    RC_BOTTOM_OF_THE_WELL_MQ_GS_WEST_INNER_ROOM,
    RC_BOTTOM_OF_THE_WELL_MQ_GS_BASEMENT,
    RC_FOREST_TEMPLE_FIRST_ROOM_CHEST,
    RC_FOREST_TEMPLE_FIRST_STALFOS_CHEST,
    RC_FOREST_TEMPLE_RAISED_ISLAND_COURTYARD_CHEST,
    RC_FOREST_TEMPLE_MAP_CHEST,
    RC_FOREST_TEMPLE_WELL_CHEST,
    RC_FOREST_TEMPLE_EYE_SWITCH_CHEST,
    RC_FOREST_TEMPLE_BOSS_KEY_CHEST,
    RC_FOREST_TEMPLE_FLOORMASTER_CHEST,
    RC_FOREST_TEMPLE_RED_POE_CHEST,
    RC_FOREST_TEMPLE_BOW_CHEST,
    RC_FOREST_TEMPLE_BLUE_POE_CHEST,
    RC_FOREST_TEMPLE_FALLING_CEILING_ROOM_CHEST,
    RC_FOREST_TEMPLE_BASEMENT_CHEST,
    RC_FOREST_TEMPLE_GS_FIRST_ROOM,
    RC_FOREST_TEMPLE_GS_LOBBY,
    RC_FOREST_TEMPLE_GS_RAISED_ISLAND_COURTYARD,
    RC_FOREST_TEMPLE_GS_LEVEL_ISLAND_COURTYARD,
    RC_FOREST_TEMPLE_GS_BASEMENT,
    RC_FOREST_TEMPLE_MQ_FIRST_ROOM_CHEST,
    RC_FOREST_TEMPLE_MQ_WOLFOS_CHEST,
    RC_FOREST_TEMPLE_MQ_WELL_CHEST,
    RC_FOREST_TEMPLE_MQ_RAISED_ISLAND_COURTYARD_LOWER_CHEST,
    RC_FOREST_TEMPLE_MQ_RAISED_ISLAND_COURTYARD_UPPER_CHEST,
    RC_FOREST_TEMPLE_MQ_BOSS_KEY_CHEST,
    RC_FOREST_TEMPLE_MQ_REDEAD_CHEST,
    RC_FOREST_TEMPLE_MQ_MAP_CHEST,
    RC_FOREST_TEMPLE_MQ_BOW_CHEST,
    RC_FOREST_TEMPLE_MQ_COMPASS_CHEST,
    RC_FOREST_TEMPLE_MQ_FALLING_CEILING_ROOM_CHEST,
    RC_FOREST_TEMPLE_MQ_BASEMENT_CHEST,
    RC_FOREST_TEMPLE_MQ_GS_FIRST_HALLWAY,
    RC_FOREST_TEMPLE_MQ_GS_RAISED_ISLAND_COURTYARD,
    RC_FOREST_TEMPLE_MQ_GS_LEVEL_ISLAND_COURTYARD,
    RC_FOREST_TEMPLE_MQ_GS_WELL,
    RC_FOREST_TEMPLE_MQ_GS_BLOCK_PUSH_ROOM,
    RC_FOREST_TEMPLE_PHANTOM_GANON_HEART,
    RC_FIRE_TEMPLE_NEAR_BOSS_CHEST,
    RC_FIRE_TEMPLE_FLARE_DANCER_CHEST,
    RC_FIRE_TEMPLE_BOSS_KEY_CHEST,
    RC_FIRE_TEMPLE_BIG_LAVA_ROOM_LOWER_OPEN_DOOR_CHEST,
    RC_FIRE_TEMPLE_BIG_LAVA_ROOM_BLOCKED_DOOR_CHEST,
    RC_FIRE_TEMPLE_BOULDER_MAZE_LOWER_CHEST,
    RC_FIRE_TEMPLE_BOULDER_MAZE_SIDE_ROOM_CHEST,
    RC_FIRE_TEMPLE_MAP_CHEST,
    RC_FIRE_TEMPLE_BOULDER_MAZE_SHORTCUT_CHEST,
    RC_FIRE_TEMPLE_BOULDER_MAZE_UPPER_CHEST,
    RC_FIRE_TEMPLE_SCARECROW_CHEST,
    RC_FIRE_TEMPLE_COMPASS_CHEST,
    RC_FIRE_TEMPLE_MEGATON_HAMMER_CHEST,
    RC_FIRE_TEMPLE_HIGHEST_GORON_CHEST,
    RC_FIRE_TEMPLE_GS_BOSS_KEY_LOOP,
    RC_FIRE_TEMPLE_GS_SONG_OF_TIME_ROOM,
    RC_FIRE_TEMPLE_GS_BOULDER_MAZE,
    RC_FIRE_TEMPLE_GS_SCARECROW_CLIMB,
    RC_FIRE_TEMPLE_GS_SCARECROW_TOP,
    RC_FIRE_TEMPLE_MQ_MAP_ROOM_SIDE_CHEST,
    RC_FIRE_TEMPLE_MQ_MEGATON_HAMMER_CHEST,
    RC_FIRE_TEMPLE_MQ_MAP_CHEST,
    RC_FIRE_TEMPLE_MQ_NEAR_BOSS_CHEST,
    RC_FIRE_TEMPLE_MQ_BIG_LAVA_ROOM_BLOCKED_DOOR_CHEST,
    RC_FIRE_TEMPLE_MQ_BOSS_KEY_CHEST,
    RC_FIRE_TEMPLE_MQ_LIZALFOS_MAZE_SIDE_ROOM_CHEST,
    RC_FIRE_TEMPLE_MQ_COMPASS_CHEST,
    RC_FIRE_TEMPLE_MQ_LIZALFOS_MAZE_UPPER_CHEST,
    RC_FIRE_TEMPLE_MQ_LIZALFOS_MAZE_LOWER_CHEST,
    RC_FIRE_TEMPLE_MQ_FREESTANDING_KEY,
    RC_FIRE_TEMPLE_MQ_CHEST_ON_FIRE,
    RC_FIRE_TEMPLE_MQ_GS_BIG_LAVA_ROOM_OPEN_DOOR,
    RC_FIRE_TEMPLE_MQ_GS_SKULL_ON_FIRE,
    RC_FIRE_TEMPLE_MQ_GS_FIRE_WALL_MAZE_CENTER,
    RC_FIRE_TEMPLE_MQ_GS_FIRE_WALL_MAZE_SIDE_ROOM,
    RC_FIRE_TEMPLE_MQ_GS_ABOVE_FIRE_MAZE,
    RC_FIRE_TEMPLE_VOLVAGIA_HEART,
    RC_WATER_TEMPLE_COMPASS_CHEST,
    RC_WATER_TEMPLE_MAP_CHEST,
    RC_WATER_TEMPLE_CRACKED_WALL_CHEST,
    RC_WATER_TEMPLE_TORCHES_CHEST,
    RC_WATER_TEMPLE_BOSS_KEY_CHEST,
    RC_WATER_TEMPLE_CENTRAL_PILLAR_CHEST,
    RC_WATER_TEMPLE_CENTRAL_BOW_TARGET_CHEST,
    RC_WATER_TEMPLE_LONGSHOT_CHEST,
    RC_WATER_TEMPLE_RIVER_CHEST,
    RC_WATER_TEMPLE_DRAGON_CHEST,
    RC_WATER_TEMPLE_GS_BEHIND_GATE,
    RC_WATER_TEMPLE_GS_NEAR_BOSS_KEY_CHEST,
    RC_WATER_TEMPLE_GS_CENTRAL_PILLAR,
    RC_WATER_TEMPLE_GS_FALLING_PLATFORM_ROOM,
    RC_WATER_TEMPLE_GS_RIVER,
    RC_WATER_TEMPLE_MQ_LONGSHOT_CHEST,
    RC_WATER_TEMPLE_MQ_MAP_CHEST,
    RC_WATER_TEMPLE_MQ_COMPASS_CHEST,
    RC_WATER_TEMPLE_MQ_CENTRAL_PILLAR_CHEST,
    RC_WATER_TEMPLE_MQ_BOSS_KEY_CHEST,
    RC_WATER_TEMPLE_MQ_FREESTANDING_KEY,
    RC_WATER_TEMPLE_MQ_GS_LIZALFOS_HALLWAY,
    RC_WATER_TEMPLE_MQ_GS_BEFORE_UPPER_WATER_SWITCH,
    RC_WATER_TEMPLE_MQ_GS_RIVER,
    RC_WATER_TEMPLE_MQ_GS_FREESTANDING_KEY_AREA,
    RC_WATER_TEMPLE_MQ_GS_TRIPLE_WALL_TORCH,
    RC_WATER_TEMPLE_MORPHA_HEART,
    RC_SHADOW_TEMPLE_MAP_CHEST,
    RC_SHADOW_TEMPLE_HOVER_BOOTS_CHEST,
    RC_SHADOW_TEMPLE_COMPASS_CHEST,
    RC_SHADOW_TEMPLE_EARLY_SILVER_RUPEE_CHEST,
    RC_SHADOW_TEMPLE_INVISIBLE_BLADES_VISIBLE_CHEST,
    RC_SHADOW_TEMPLE_INVISIBLE_BLADES_INVISIBLE_CHEST,
    RC_SHADOW_TEMPLE_FALLING_SPIKES_LOWER_CHEST,
    RC_SHADOW_TEMPLE_FALLING_SPIKES_UPPER_CHEST,
    RC_SHADOW_TEMPLE_FALLING_SPIKES_SWITCH_CHEST,
    RC_SHADOW_TEMPLE_INVISIBLE_SPIKES_CHEST,
    RC_SHADOW_TEMPLE_FREESTANDING_KEY,
    RC_SHADOW_TEMPLE_WIND_HINT_CHEST,
    RC_SHADOW_TEMPLE_AFTER_WIND_ENEMY_CHEST,
    RC_SHADOW_TEMPLE_AFTER_WIND_HIDDEN_CHEST,
    RC_SHADOW_TEMPLE_SPIKE_WALLS_LEFT_CHEST,
    RC_SHADOW_TEMPLE_BOSS_KEY_CHEST,
    RC_SHADOW_TEMPLE_INVISIBLE_FLOORMASTER_CHEST,
    RC_SHADOW_TEMPLE_GS_LIKE_LIKE_ROOM,
    RC_SHADOW_TEMPLE_GS_FALLING_SPIKES_ROOM,
    RC_SHADOW_TEMPLE_GS_SINGLE_GIANT_POT,
    RC_SHADOW_TEMPLE_GS_NEAR_SHIP,
    RC_SHADOW_TEMPLE_GS_TRIPLE_GIANT_POT,
    RC_SHADOW_TEMPLE_MQ_EARLY_GIBDOS_CHEST,
    RC_SHADOW_TEMPLE_MQ_MAP_CHEST,
    RC_SHADOW_TEMPLE_MQ_NEAR_SHIP_INVISIBLE_CHEST,
    RC_SHADOW_TEMPLE_MQ_COMPASS_CHEST,
    RC_SHADOW_TEMPLE_MQ_HOVER_BOOTS_CHEST,
    RC_SHADOW_TEMPLE_MQ_INVISIBLE_BLADES_INVISIBLE_CHEST,
    RC_SHADOW_TEMPLE_MQ_INVISIBLE_BLADES_VISIBLE_CHEST,
    RC_SHADOW_TEMPLE_MQ_BEAMOS_SILVER_RUPEES_CHEST,
    RC_SHADOW_TEMPLE_MQ_FALLING_SPIKES_LOWER_CHEST,
    RC_SHADOW_TEMPLE_MQ_FALLING_SPIKES_UPPER_CHEST,
    RC_SHADOW_TEMPLE_MQ_FALLING_SPIKES_SWITCH_CHEST,
    RC_SHADOW_TEMPLE_MQ_INVISIBLE_SPIKES_CHEST,
    RC_SHADOW_TEMPLE_MQ_STALFOS_ROOM_CHEST,
    RC_SHADOW_TEMPLE_MQ_WIND_HINT_CHEST,
    RC_SHADOW_TEMPLE_MQ_AFTER_WIND_HIDDEN_CHEST,
    RC_SHADOW_TEMPLE_MQ_AFTER_WIND_ENEMY_CHEST,
    RC_SHADOW_TEMPLE_MQ_BOSS_KEY_CHEST,
    RC_SHADOW_TEMPLE_MQ_SPIKE_WALLS_LEFT_CHEST,
    RC_SHADOW_TEMPLE_MQ_FREESTANDING_KEY,
    RC_SHADOW_TEMPLE_MQ_BOMB_FLOWER_CHEST,
    RC_SHADOW_TEMPLE_MQ_GS_FALLING_SPIKES_ROOM,
    RC_SHADOW_TEMPLE_MQ_GS_WIND_HINT_ROOM,
    RC_SHADOW_TEMPLE_MQ_GS_AFTER_WIND,
    RC_SHADOW_TEMPLE_MQ_GS_AFTER_SHIP,
    RC_SHADOW_TEMPLE_MQ_GS_NEAR_BOSS,
    RC_SHADOW_TEMPLE_BONGO_BONGO_HEART,
    RC_SPIRIT_TEMPLE_CHILD_BRIDGE_CHEST,
    RC_SPIRIT_TEMPLE_CHILD_EARLY_TORCHES_CHEST,
    RC_SPIRIT_TEMPLE_CHILD_CLIMB_NORTH_CHEST,
    RC_SPIRIT_TEMPLE_CHILD_CLIMB_EAST_CHEST,
    RC_SPIRIT_TEMPLE_MAP_CHEST,
    RC_SPIRIT_TEMPLE_SUN_BLOCK_ROOM_CHEST,
    RC_SPIRIT_TEMPLE_MQ_ENTRANCE_FRONT_LEFT_CHEST,
    RC_SPIRIT_TEMPLE_MQ_ENTRANCE_BACK_RIGHT_CHEST,
    RC_SPIRIT_TEMPLE_MQ_ENTRANCE_FRONT_RIGHT_CHEST,
    RC_SPIRIT_TEMPLE_MQ_ENTRANCE_BACK_LEFT_CHEST,
    RC_SPIRIT_TEMPLE_MQ_MAP_CHEST,
    RC_SPIRIT_TEMPLE_MQ_MAP_ROOM_ENEMY_CHEST,
    RC_SPIRIT_TEMPLE_MQ_CHILD_CLIMB_NORTH_CHEST,
    RC_SPIRIT_TEMPLE_MQ_CHILD_CLIMB_SOUTH_CHEST,
    RC_SPIRIT_TEMPLE_MQ_COMPASS_CHEST,
    RC_SPIRIT_TEMPLE_MQ_SILVER_BLOCK_HALLWAY_CHEST,
    RC_SPIRIT_TEMPLE_MQ_SUN_BLOCK_ROOM_CHEST,
    RC_SPIRIT_TEMPLE_SILVER_GAUNTLETS_CHEST,
    RC_SPIRIT_TEMPLE_COMPASS_CHEST,
    RC_SPIRIT_TEMPLE_EARLY_ADULT_RIGHT_CHEST,
    RC_SPIRIT_TEMPLE_FIRST_MIRROR_LEFT_CHEST,
    RC_SPIRIT_TEMPLE_FIRST_MIRROR_RIGHT_CHEST,
    RC_SPIRIT_TEMPLE_STATUE_ROOM_NORTHEAST_CHEST,
    RC_SPIRIT_TEMPLE_STATUE_ROOM_HAND_CHEST,
    RC_SPIRIT_TEMPLE_NEAR_FOUR_ARMOS_CHEST,
    RC_SPIRIT_TEMPLE_HALLWAY_RIGHT_INVISIBLE_CHEST,
    RC_SPIRIT_TEMPLE_HALLWAY_LEFT_INVISIBLE_CHEST,
    RC_SPIRIT_TEMPLE_MQ_CHILD_HAMMER_SWITCH_CHEST,
    RC_SPIRIT_TEMPLE_MQ_STATUE_ROOM_LULLABY_CHEST,
    RC_SPIRIT_TEMPLE_MQ_STATUE_ROOM_INVISIBLE_CHEST,
    RC_SPIRIT_TEMPLE_MQ_LEEVER_ROOM_CHEST,
    RC_SPIRIT_TEMPLE_MQ_SYMPHONY_ROOM_CHEST,
    RC_SPIRIT_TEMPLE_MQ_BEAMOS_ROOM_CHEST,
    RC_SPIRIT_TEMPLE_MQ_CHEST_SWITCH_CHEST,
    RC_SPIRIT_TEMPLE_MQ_BOSS_KEY_CHEST,
    RC_SPIRIT_TEMPLE_MIRROR_SHIELD_CHEST,
    RC_SPIRIT_TEMPLE_BOSS_KEY_CHEST,
    RC_SPIRIT_TEMPLE_TOPMOST_CHEST,
    RC_SPIRIT_TEMPLE_MQ_MIRROR_PUZZLE_INVISIBLE_CHEST,
    RC_SPIRIT_TEMPLE_GS_METAL_FENCE,
    RC_SPIRIT_TEMPLE_GS_SUN_ON_FLOOR_ROOM,
    RC_SPIRIT_TEMPLE_GS_HALL_AFTER_SUN_BLOCK_ROOM,
    RC_SPIRIT_TEMPLE_GS_LOBBY,
    RC_SPIRIT_TEMPLE_GS_BOULDER_ROOM,
    RC_SPIRIT_TEMPLE_MQ_GS_SUN_BLOCK_ROOM,
    RC_SPIRIT_TEMPLE_MQ_GS_LEEVER_ROOM,
    RC_SPIRIT_TEMPLE_MQ_GS_SYMPHONY_ROOM,
    RC_SPIRIT_TEMPLE_MQ_GS_NINE_THRONES_ROOM_WEST,
    RC_SPIRIT_TEMPLE_MQ_GS_NINE_THRONES_ROOM_NORTH,
    RC_SPIRIT_TEMPLE_TWINROVA_HEART,
    RC_ICE_CAVERN_MAP_CHEST,
    RC_ICE_CAVERN_COMPASS_CHEST,
    RC_ICE_CAVERN_FREESTANDING_POH,
    RC_ICE_CAVERN_IRON_BOOTS_CHEST,
    RC_ICE_CAVERN_GS_SPINNING_SCYTHE_ROOM,
    RC_ICE_CAVERN_GS_HEART_PIECE_ROOM,
    RC_ICE_CAVERN_GS_PUSH_BLOCK_ROOM,
    RC_ICE_CAVERN_MQ_MAP_CHEST,
    RC_ICE_CAVERN_MQ_COMPASS_CHEST,
    RC_ICE_CAVERN_MQ_FREESTANDING_POH,
    RC_ICE_CAVERN_MQ_IRON_BOOTS_CHEST,
    RC_ICE_CAVERN_MQ_GS_RED_ICE,
    RC_ICE_CAVERN_MQ_GS_ICE_BLOCK,
    RC_ICE_CAVERN_MQ_GS_SCARECROW,
    RC_GERUDO_TRAINING_GROUND_LOBBY_LEFT_CHEST,
    RC_GERUDO_TRAINING_GROUND_LOBBY_RIGHT_CHEST,
    RC_GERUDO_TRAINING_GROUND_STALFOS_CHEST,
    RC_GERUDO_TRAINING_GROUND_BEFORE_HEAVY_BLOCK_CHEST,
    RC_GERUDO_TRAINING_GROUND_HEAVY_BLOCK_FIRST_CHEST,
    RC_GERUDO_TRAINING_GROUND_HEAVY_BLOCK_SECOND_CHEST,
    RC_GERUDO_TRAINING_GROUND_HEAVY_BLOCK_THIRD_CHEST,
    RC_GERUDO_TRAINING_GROUND_HEAVY_BLOCK_FOURTH_CHEST,
    RC_GERUDO_TRAINING_GROUND_EYE_STATUE_CHEST,
    RC_GERUDO_TRAINING_GROUND_NEAR_SCARECROW_CHEST,
    RC_GERUDO_TRAINING_GROUND_HAMMER_ROOM_CLEAR_CHEST,
    RC_GERUDO_TRAINING_GROUND_HAMMER_ROOM_SWITCH_CHEST,
    RC_GERUDO_TRAINING_GROUND_FREESTANDING_KEY,
    RC_GERUDO_TRAINING_GROUND_MAZE_RIGHT_CENTRAL_CHEST,
    RC_GERUDO_TRAINING_GROUND_MAZE_RIGHT_SIDE_CHEST,
    RC_GERUDO_TRAINING_GROUND_UNDERWATER_SILVER_RUPEE_CHEST,
    RC_GERUDO_TRAINING_GROUND_BEAMOS_CHEST,
    RC_GERUDO_TRAINING_GROUND_HIDDEN_CEILING_CHEST,
    RC_GERUDO_TRAINING_GROUND_MAZE_PATH_FIRST_CHEST,
    RC_GERUDO_TRAINING_GROUND_MAZE_PATH_SECOND_CHEST,
    RC_GERUDO_TRAINING_GROUND_MAZE_PATH_THIRD_CHEST,
    RC_GERUDO_TRAINING_GROUND_MAZE_PATH_FINAL_CHEST,
    RC_GERUDO_TRAINING_GROUND_MQ_LOBBY_LEFT_CHEST,
    RC_GERUDO_TRAINING_GROUND_MQ_LOBBY_RIGHT_CHEST,
    RC_GERUDO_TRAINING_GROUND_MQ_FIRST_IRON_KNUCKLE_CHEST,
    RC_GERUDO_TRAINING_GROUND_MQ_BEFORE_HEAVY_BLOCK_CHEST,
    RC_GERUDO_TRAINING_GROUND_MQ_HEAVY_BLOCK_CHEST,
    RC_GERUDO_TRAINING_GROUND_MQ_EYE_STATUE_CHEST,
    RC_GERUDO_TRAINING_GROUND_MQ_ICE_ARROWS_CHEST,
    RC_GERUDO_TRAINING_GROUND_MQ_SECOND_IRON_KNUCKLE_CHEST,
    RC_GERUDO_TRAINING_GROUND_MQ_FLAME_CIRCLE_CHEST,
    RC_GERUDO_TRAINING_GROUND_MQ_MAZE_RIGHT_CENTRAL_CHEST,
    RC_GERUDO_TRAINING_GROUND_MQ_MAZE_RIGHT_SIDE_CHEST,
    RC_GERUDO_TRAINING_GROUND_MQ_UNDERWATER_SILVER_RUPEE_CHEST,
    RC_GERUDO_TRAINING_GROUND_MQ_DINOLFOS_CHEST,
    RC_GERUDO_TRAINING_GROUND_MQ_HIDDEN_CEILING_CHEST,
    RC_GERUDO_TRAINING_GROUND_MQ_MAZE_PATH_FIRST_CHEST,
    RC_GERUDO_TRAINING_GROUND_MQ_MAZE_PATH_THIRD_CHEST,
    RC_GERUDO_TRAINING_GROUND_MQ_MAZE_PATH_SECOND_CHEST,
    RC_GANONS_CASTLE_FOREST_TRIAL_CHEST,
    RC_GANONS_CASTLE_WATER_TRIAL_LEFT_CHEST,
    RC_GANONS_CASTLE_WATER_TRIAL_RIGHT_CHEST,
    RC_GANONS_CASTLE_SHADOW_TRIAL_FRONT_CHEST,
    RC_GANONS_CASTLE_SHADOW_TRIAL_GOLDEN_GAUNTLETS_CHEST,
    RC_GANONS_CASTLE_LIGHT_TRIAL_FIRST_LEFT_CHEST,
    RC_GANONS_CASTLE_LIGHT_TRIAL_SECOND_LEFT_CHEST,
    RC_GANONS_CASTLE_LIGHT_TRIAL_THIRD_LEFT_CHEST,
    RC_GANONS_CASTLE_LIGHT_TRIAL_FIRST_RIGHT_CHEST,
    RC_GANONS_CASTLE_LIGHT_TRIAL_SECOND_RIGHT_CHEST,
    RC_GANONS_CASTLE_LIGHT_TRIAL_THIRD_RIGHT_CHEST,
    RC_GANONS_CASTLE_LIGHT_TRIAL_INVISIBLE_ENEMIES_CHEST,
    RC_GANONS_CASTLE_LIGHT_TRIAL_LULLABY_CHEST,
    RC_GANONS_CASTLE_SPIRIT_TRIAL_CRYSTAL_SWITCH_CHEST,
    RC_GANONS_CASTLE_SPIRIT_TRIAL_INVISIBLE_CHEST,
    RC_GANONS_CASTLE_DEKU_SCRUB_LEFT,
    RC_GANONS_CASTLE_DEKU_SCRUB_CENTER_LEFT,
    RC_GANONS_CASTLE_DEKU_SCRUB_CENTER_RIGHT,
    RC_GANONS_CASTLE_DEKU_SCRUB_RIGHT,
    RC_GANONS_CASTLE_MQ_FOREST_TRIAL_FREESTANDING_KEY,
    RC_GANONS_CASTLE_MQ_FOREST_TRIAL_EYE_SWITCH_CHEST,
    RC_GANONS_CASTLE_MQ_FOREST_TRIAL_FROZEN_EYE_SWITCH_CHEST,
    RC_GANONS_CASTLE_MQ_WATER_TRIAL_CHEST,
    RC_GANONS_CASTLE_MQ_SHADOW_TRIAL_BOMB_FLOWER_CHEST,
    RC_GANONS_CASTLE_MQ_SHADOW_TRIAL_EYE_SWITCH_CHEST,
    RC_GANONS_CASTLE_MQ_LIGHT_TRIAL_LULLABY_CHEST,
    RC_GANONS_CASTLE_MQ_SPIRIT_TRIAL_FIRST_CHEST,
    RC_GANONS_CASTLE_MQ_SPIRIT_TRIAL_INVISIBLE_CHEST,
    RC_GANONS_CASTLE_MQ_SPIRIT_TRIAL_SUN_FRONT_LEFT_CHEST,
    RC_GANONS_CASTLE_MQ_SPIRIT_TRIAL_SUN_BACK_LEFT_CHEST,
    RC_GANONS_CASTLE_MQ_SPIRIT_TRIAL_SUN_BACK_RIGHT_CHEST,
    RC_GANONS_CASTLE_MQ_SPIRIT_TRIAL_GOLDEN_GAUNTLETS_CHEST,
    RC_GANONS_CASTLE_MQ_DEKU_SCRUB_LEFT,
    RC_GANONS_CASTLE_MQ_DEKU_SCRUB_CENTER_LEFT,
    RC_GANONS_CASTLE_MQ_DEKU_SCRUB_CENTER,
    RC_GANONS_CASTLE_MQ_DEKU_SCRUB_CENTER_RIGHT,
    RC_GANONS_CASTLE_MQ_DEKU_SCRUB_RIGHT,
    RC_GANONS_TOWER_BOSS_KEY_CHEST,

    // Overworld Pots
    RC_KF_LINKS_HOUSE_POT,
    RC_KF_TWINS_HOUSE_POT_1,
    RC_KF_TWINS_HOUSE_POT_2,
    RC_KF_BROTHERS_HOUSE_POT_1,
    RC_KF_BROTHERS_HOUSE_POT_2,
    RC_GF_BREAK_ROOM_POT_1,
    RC_GF_BREAK_ROOM_POT_2,
    RC_GF_KITCHEN_POT_1,
    RC_GF_KITCHEN_POT_2,
    RC_GF_NORTH_F1_CARPENTER_POT_1,
    RC_GF_NORTH_F1_CARPENTER_POT_2,
    RC_GF_NORTH_F1_CARPENTER_POT_3,
    RC_GF_NORTH_F2_CARPENTER_POT_1,
    RC_GF_NORTH_F2_CARPENTER_POT_2,
    RC_GF_SOUTH_F1_CARPENTER_POT_1,
    RC_GF_SOUTH_F1_CARPENTER_POT_2,
    RC_GF_SOUTH_F1_CARPENTER_POT_3,
    RC_GF_SOUTH_F1_CARPENTER_CELL_POT_1,
    RC_GF_SOUTH_F1_CARPENTER_CELL_POT_2,
    RC_GF_SOUTH_F1_CARPENTER_CELL_POT_3,
    RC_GF_SOUTH_F1_CARPENTER_CELL_POT_4,
    RC_WASTELAND_NEAR_GS_POT_1,
    RC_WASTELAND_NEAR_GS_POT_2,
    RC_WASTELAND_NEAR_GS_POT_3,
    RC_WASTELAND_NEAR_GS_POT_4,
    RC_HF_COW_GROTTO_POT_1,
    RC_HF_COW_GROTTO_POT_2,
    RC_MK_GUARD_HOUSE_CHILD_POT_1,
    RC_MK_GUARD_HOUSE_CHILD_POT_2,
    RC_MK_GUARD_HOUSE_CHILD_POT_3,
    RC_MK_GUARD_HOUSE_CHILD_POT_4,
    RC_MK_GUARD_HOUSE_CHILD_POT_5,
    RC_MK_GUARD_HOUSE_CHILD_POT_6,
    RC_MK_GUARD_HOUSE_CHILD_POT_7,
    RC_MK_GUARD_HOUSE_CHILD_POT_8,
    RC_MK_GUARD_HOUSE_CHILD_POT_9,
    RC_MK_GUARD_HOUSE_CHILD_POT_10,
    RC_MK_GUARD_HOUSE_CHILD_POT_11,
    RC_MK_GUARD_HOUSE_CHILD_POT_12,
    RC_MK_GUARD_HOUSE_CHILD_POT_13,
    RC_MK_GUARD_HOUSE_CHILD_POT_14,
    RC_MK_GUARD_HOUSE_CHILD_POT_15,
    RC_MK_GUARD_HOUSE_CHILD_POT_16,
    RC_MK_GUARD_HOUSE_CHILD_POT_17,
    RC_MK_GUARD_HOUSE_CHILD_POT_18,
    RC_MK_GUARD_HOUSE_CHILD_POT_19,
    RC_MK_GUARD_HOUSE_CHILD_POT_20,
    RC_MK_GUARD_HOUSE_CHILD_POT_21,
    RC_MK_GUARD_HOUSE_CHILD_POT_22,
    RC_MK_GUARD_HOUSE_CHILD_POT_23,
    RC_MK_GUARD_HOUSE_CHILD_POT_24,
    RC_MK_GUARD_HOUSE_CHILD_POT_25,
    RC_MK_GUARD_HOUSE_CHILD_POT_26,
    RC_MK_GUARD_HOUSE_CHILD_POT_27,
    RC_MK_GUARD_HOUSE_CHILD_POT_28,
    RC_MK_GUARD_HOUSE_CHILD_POT_29,
    RC_MK_GUARD_HOUSE_CHILD_POT_30,
    RC_MK_GUARD_HOUSE_CHILD_POT_31,
    RC_MK_GUARD_HOUSE_CHILD_POT_32,
    RC_MK_GUARD_HOUSE_CHILD_POT_33,
    RC_MK_GUARD_HOUSE_CHILD_POT_34,
    RC_MK_GUARD_HOUSE_CHILD_POT_35,
    RC_MK_GUARD_HOUSE_CHILD_POT_36,
    RC_MK_GUARD_HOUSE_CHILD_POT_37,
    RC_MK_GUARD_HOUSE_CHILD_POT_38,
    RC_MK_GUARD_HOUSE_CHILD_POT_39,
    RC_MK_GUARD_HOUSE_CHILD_POT_40,
    RC_MK_GUARD_HOUSE_CHILD_POT_41,
    RC_MK_GUARD_HOUSE_CHILD_POT_42,
    RC_MK_GUARD_HOUSE_CHILD_POT_43,
    RC_MK_GUARD_HOUSE_CHILD_POT_44,
    RC_MK_GUARD_HOUSE_ADULT_POT_1,
    RC_MK_GUARD_HOUSE_ADULT_POT_2,
    RC_MK_GUARD_HOUSE_ADULT_POT_3,
    RC_MK_GUARD_HOUSE_ADULT_POT_4,
    RC_MK_GUARD_HOUSE_ADULT_POT_5,
    RC_MK_GUARD_HOUSE_ADULT_POT_6,
    RC_MK_GUARD_HOUSE_ADULT_POT_7,
    RC_MK_GUARD_HOUSE_ADULT_POT_8,
    RC_MK_GUARD_HOUSE_ADULT_POT_9,
    RC_MK_GUARD_HOUSE_ADULT_POT_10,
    RC_MK_GUARD_HOUSE_ADULT_POT_11,
    RC_MK_BACK_ALLEY_HOUSE_POT_1,
    RC_MK_BACK_ALLEY_HOUSE_POT_2,
    RC_MK_BACK_ALLEY_HOUSE_POT_3,
    RC_HC_STORMS_GROTTO_POT_1,
    RC_HC_STORMS_GROTTO_POT_2,
    RC_HC_STORMS_GROTTO_POT_3,
    RC_HC_STORMS_GROTTO_POT_4,
    RC_LLR_FRONT_POT_1,
    RC_LLR_FRONT_POT_2,
    RC_LLR_FRONT_POT_3,
    RC_LLR_FRONT_POT_4,
    RC_LLR_RAIN_SHED_POT_1,
    RC_LLR_RAIN_SHED_POT_2,
    RC_LLR_RAIN_SHED_POT_3,
    RC_LLR_TALONS_HOUSE_POT_1,
    RC_LLR_TALONS_HOUSE_POT_2,
    RC_LLR_TALONS_HOUSE_POT_3,
    RC_KAK_NEAR_POTION_SHOP_POT_1,
    RC_KAK_NEAR_POTION_SHOP_POT_2,
    RC_KAK_NEAR_POTION_SHOP_POT_3,
    RC_KAK_NEAR_IMPAS_HOUSE_POT_1,
    RC_KAK_NEAR_IMPAS_HOUSE_POT_2,
    RC_KAK_NEAR_IMPAS_HOUSE_POT_3,
    RC_KAK_NEAR_GUARDS_HOUSE_POT_1,
    RC_KAK_NEAR_GUARDS_HOUSE_POT_2,
    RC_KAK_NEAR_GUARDS_HOUSE_POT_3,
    RC_KAK_NEAR_MEDICINE_SHOP_POT_1,
    RC_KAK_NEAR_MEDICINE_SHOP_POT_2,
    RC_GY_DAMPES_GRAVE_POT_1,
    RC_GY_DAMPES_GRAVE_POT_2,
    RC_GY_DAMPES_GRAVE_POT_3,
    RC_GY_DAMPES_GRAVE_POT_4,
    RC_GY_DAMPES_GRAVE_POT_5,
    RC_GY_DAMPES_GRAVE_POT_6,
    RC_GC_LOWER_STAIRCASE_POT_1,
    RC_GC_LOWER_STAIRCASE_POT_2,
    RC_GC_UPPER_STAIRCASE_POT_1,
    RC_GC_UPPER_STAIRCASE_POT_2,
    RC_GC_UPPER_STAIRCASE_POT_3,
    RC_GC_MEDIGORON_POT_1,
    RC_GC_DARUNIA_POT_1,
    RC_GC_DARUNIA_POT_2,
    RC_GC_DARUNIA_POT_3,
    RC_DMC_NEAR_GC_POT_1,
    RC_DMC_NEAR_GC_POT_2,
    RC_DMC_NEAR_GC_POT_3,
    RC_DMC_NEAR_GC_POT_4,
    RC_ZD_NEAR_SHOP_POT_1,
    RC_ZD_NEAR_SHOP_POT_2,
    RC_ZD_NEAR_SHOP_POT_3,
    RC_ZD_NEAR_SHOP_POT_4,
    RC_ZD_NEAR_SHOP_POT_5,
    RC_ZF_HIDDEN_CAVE_POT_1,
    RC_ZF_HIDDEN_CAVE_POT_2,
    RC_ZF_HIDDEN_CAVE_POT_3,
    RC_ZF_NEAR_JABU_POT_1,
    RC_ZF_NEAR_JABU_POT_2,
    RC_ZF_NEAR_JABU_POT_3,
    RC_ZF_NEAR_JABU_POT_4,

    // Dungeon Pots
    RC_DODONGOS_CAVERN_LIZALFOS_POT_1,
    RC_DODONGOS_CAVERN_LIZALFOS_POT_2,
    RC_DODONGOS_CAVERN_LIZALFOS_POT_3,
    RC_DODONGOS_CAVERN_LIZALFOS_POT_4,
    RC_DODONGOS_CAVERN_SIDE_ROOM_POT_1,
    RC_DODONGOS_CAVERN_SIDE_ROOM_POT_2,
    RC_DODONGOS_CAVERN_SIDE_ROOM_POT_3,
    RC_DODONGOS_CAVERN_SIDE_ROOM_POT_4,
    RC_DODONGOS_CAVERN_SIDE_ROOM_POT_5,
    RC_DODONGOS_CAVERN_SIDE_ROOM_POT_6,
    RC_DODONGOS_CAVERN_TORCH_ROOM_POT_1,
    RC_DODONGOS_CAVERN_TORCH_ROOM_POT_2,
    RC_DODONGOS_CAVERN_TORCH_ROOM_POT_3,
    RC_DODONGOS_CAVERN_TORCH_ROOM_POT_4,
    RC_DODONGOS_CAVERN_STAIRCASE_POT_1,
    RC_DODONGOS_CAVERN_STAIRCASE_POT_2,
    RC_DODONGOS_CAVERN_STAIRCASE_POT_3,
    RC_DODONGOS_CAVERN_STAIRCASE_POT_4,
    RC_DODONGOS_CAVERN_SINGLE_EYE_POT_1,
    RC_DODONGOS_CAVERN_SINGLE_EYE_POT_2,
    RC_DODONGOS_CAVERN_BLADE_POT_1,
    RC_DODONGOS_CAVERN_BLADE_POT_2,
    RC_DODONGOS_CAVERN_DOUBLE_EYE_POT_1,
    RC_DODONGOS_CAVERN_DOUBLE_EYE_POT_2,
    RC_DODONGOS_CAVERN_BACK_ROOM_POT_1,
    RC_DODONGOS_CAVERN_BACK_ROOM_POT_2,
    RC_DODONGOS_CAVERN_BACK_ROOM_POT_3,
    RC_DODONGOS_CAVERN_BACK_ROOM_POT_4,
    RC_JABU_JABUS_BELLY_ABOVE_BIG_OCTO_POT_1,
    RC_JABU_JABUS_BELLY_ABOVE_BIG_OCTO_POT_2,
    RC_JABU_JABUS_BELLY_ABOVE_BIG_OCTO_POT_3,
    RC_JABU_JABUS_BELLY_BARINADE_POT_1,
    RC_JABU_JABUS_BELLY_BARINADE_POT_2,
    RC_JABU_JABUS_BELLY_BARINADE_POT_3,
    RC_JABU_JABUS_BELLY_BARINADE_POT_4,
    RC_JABU_JABUS_BELLY_BARINADE_POT_5,
    RC_JABU_JABUS_BELLY_BARINADE_POT_6,
    RC_JABU_JABUS_BELLY_BASEMENT_POT_1,
    RC_JABU_JABUS_BELLY_BASEMENT_POT_2,
    RC_JABU_JABUS_BELLY_BASEMENT_POT_3,
    RC_JABU_JABUS_BELLY_TWO_OCTOROK_POT_1,
    RC_JABU_JABUS_BELLY_TWO_OCTOROK_POT_2,
    RC_JABU_JABUS_BELLY_TWO_OCTOROK_POT_3,
    RC_JABU_JABUS_BELLY_TWO_OCTOROK_POT_4,
    RC_JABU_JABUS_BELLY_TWO_OCTOROK_POT_5,
    RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_1,
    RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_2,
    RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_3,
    RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_4,
    RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_5,
    RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_6,
    RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_7,
    RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_8,
    RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_9,
    RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_10,
    RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_11,
    RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_12,
    RC_BOTTOM_OF_THE_WELL_LEFT_SIDE_POT_1,
    RC_BOTTOM_OF_THE_WELL_LEFT_SIDE_POT_2,
    RC_BOTTOM_OF_THE_WELL_LEFT_SIDE_POT_3,
    RC_BOTTOM_OF_THE_WELL_NEAR_ENTRANCE_POT_1,
    RC_BOTTOM_OF_THE_WELL_NEAR_ENTRANCE_POT_2,
    RC_BOTTOM_OF_THE_WELL_FIRE_KEESE_POT_1,
    RC_BOTTOM_OF_THE_WELL_UNDERWATER_POT,
    RC_FOREST_TEMPLE_LOBBY_POT_1,
    RC_FOREST_TEMPLE_LOBBY_POT_2,
    RC_FOREST_TEMPLE_LOBBY_POT_3,
    RC_FOREST_TEMPLE_LOBBY_POT_4,
    RC_FOREST_TEMPLE_LOBBY_POT_5,
    RC_FOREST_TEMPLE_LOBBY_POT_6,
    RC_FOREST_TEMPLE_LOWER_STALFOS_POT_1,
    RC_FOREST_TEMPLE_LOWER_STALFOS_POT_2,
    RC_FOREST_TEMPLE_GREEN_POE_POT_1,
    RC_FOREST_TEMPLE_GREEN_POE_POT_2,
    RC_FOREST_TEMPLE_UPPER_STALFOS_POT_1,
    RC_FOREST_TEMPLE_UPPER_STALFOS_POT_2,
    RC_FOREST_TEMPLE_UPPER_STALFOS_POT_3,
    RC_FOREST_TEMPLE_UPPER_STALFOS_POT_4,
    RC_FOREST_TEMPLE_BLUE_POE_POT_1,
    RC_FOREST_TEMPLE_BLUE_POE_POT_2,
    RC_FOREST_TEMPLE_BLUE_POE_POT_3,
    RC_FOREST_TEMPLE_FROZEN_EYE_POT_1,
    RC_FOREST_TEMPLE_FROZEN_EYE_POT_2,
    RC_FIRE_TEMPLE_NEAR_BOSS_POT_1,
    RC_FIRE_TEMPLE_NEAR_BOSS_POT_2,
    RC_FIRE_TEMPLE_NEAR_BOSS_POT_3,
    RC_FIRE_TEMPLE_NEAR_BOSS_POT_4,
    RC_FIRE_TEMPLE_BIG_LAVA_POT_1,
    RC_FIRE_TEMPLE_BIG_LAVA_POT_2,
    RC_FIRE_TEMPLE_BIG_LAVA_POT_3,
    RC_FIRE_TEMPLE_FLAME_MAZE_LEFT_POT_1,
    RC_FIRE_TEMPLE_FLAME_MAZE_LEFT_POT_2,
    RC_FIRE_TEMPLE_FLAME_MAZE_LEFT_POT_3,
    RC_FIRE_TEMPLE_FLAME_MAZE_LEFT_POT_4,
    RC_FIRE_TEMPLE_FLAME_MAZE_RIGHT_POT_1,
    RC_FIRE_TEMPLE_FLAME_MAZE_RIGHT_POT_2,
    RC_FIRE_TEMPLE_FLAME_MAZE_RIGHT_POT_3,
    RC_FIRE_TEMPLE_FLAME_MAZE_RIGHT_POT_4,
    RC_WATER_TEMPLE_MAIN_LEVEL_2_POT_1,
    RC_WATER_TEMPLE_MAIN_LEVEL_2_POT_2,
    RC_WATER_TEMPLE_MAIN_LEVEL_1_POT_1,
    RC_WATER_TEMPLE_MAIN_LEVEL_1_POT_2,
    RC_WATER_TEMPLE_TORCH_POT_1,
    RC_WATER_TEMPLE_TORCH_POT_2,
    RC_WATER_TEMPLE_NEAR_COMPASS_POT_1,
    RC_WATER_TEMPLE_NEAR_COMPASS_POT_2,
    RC_WATER_TEMPLE_NEAR_COMPASS_POT_3,
    RC_WATER_TEMPLE_CENTRAL_BOW_POT_1,
    RC_WATER_TEMPLE_CENTRAL_BOW_POT_2,
    RC_WATER_TEMPLE_BEHIND_GATE_POT_1,
    RC_WATER_TEMPLE_BEHIND_GATE_POT_2,
    RC_WATER_TEMPLE_BEHIND_GATE_POT_3,
    RC_WATER_TEMPLE_BEHIND_GATE_POT_4,
    RC_WATER_TEMPLE_BASEMENT_BLOCK_PUZZLE_POT_1,
    RC_WATER_TEMPLE_BASEMENT_BLOCK_PUZZLE_POT_2,
    RC_WATER_TEMPLE_RIVER_POT_1,
    RC_WATER_TEMPLE_RIVER_POT_2,
    RC_WATER_TEMPLE_LIKE_LIKE_POT_1,
    RC_WATER_TEMPLE_LIKE_LIKE_POT_2,
    RC_WATER_TEMPLE_BOSS_KEY_POT_1,
    RC_WATER_TEMPLE_BOSS_KEY_POT_2,
    RC_SHADOW_TEMPLE_NEAR_DEAD_HAND_POT_1,
    RC_SHADOW_TEMPLE_WHISPERING_WALLS_POT_1,
    RC_SHADOW_TEMPLE_WHISPERING_WALLS_POT_2,
    RC_SHADOW_TEMPLE_WHISPERING_WALLS_POT_3,
    RC_SHADOW_TEMPLE_WHISPERING_WALLS_POT_4,
    RC_SHADOW_TEMPLE_WHISPERING_WALLS_POT_5,
    RC_SHADOW_TEMPLE_MAP_CHEST_POT_1,
    RC_SHADOW_TEMPLE_MAP_CHEST_POT_2,
    RC_SHADOW_TEMPLE_FALLING_SPIKES_POT_1,
    RC_SHADOW_TEMPLE_FALLING_SPIKES_POT_2,
    RC_SHADOW_TEMPLE_FALLING_SPIKES_POT_3,
    RC_SHADOW_TEMPLE_FALLING_SPIKES_POT_4,
    RC_SHADOW_TEMPLE_AFTER_WIND_POT_1,
    RC_SHADOW_TEMPLE_AFTER_WIND_POT_2,
    RC_SHADOW_TEMPLE_SPIKE_WALLS_POT_1,
    RC_SHADOW_TEMPLE_FLOORMASTER_POT_1,
    RC_SHADOW_TEMPLE_FLOORMASTER_POT_2,
    RC_SHADOW_TEMPLE_AFTER_BOAT_POT_1,
    RC_SHADOW_TEMPLE_AFTER_BOAT_POT_2,
    RC_SHADOW_TEMPLE_AFTER_BOAT_POT_3,
    RC_SHADOW_TEMPLE_AFTER_BOAT_POT_4,
    RC_SPIRIT_TEMPLE_LOBBY_POT_1,
    RC_SPIRIT_TEMPLE_LOBBY_POT_2,
    RC_SPIRIT_TEMPLE_ANUBIS_POT_1,
    RC_SPIRIT_TEMPLE_ANUBIS_POT_2,
    RC_SPIRIT_TEMPLE_ANUBIS_POT_3,
    RC_SPIRIT_TEMPLE_ANUBIS_POT_4,
    RC_SPIRIT_TEMPLE_CHILD_CLIMB_POT_1,
    RC_SPIRIT_TEMPLE_AFTER_SUN_BLOCK_POT_1,
    RC_SPIRIT_TEMPLE_AFTER_SUN_BLOCK_POT_2,
    RC_SPIRIT_TEMPLE_CENTRAL_CHAMBER_POT_1,
    RC_SPIRIT_TEMPLE_CENTRAL_CHAMBER_POT_2,
    RC_SPIRIT_TEMPLE_CENTRAL_CHAMBER_POT_3,
    RC_SPIRIT_TEMPLE_CENTRAL_CHAMBER_POT_4,
    RC_SPIRIT_TEMPLE_CENTRAL_CHAMBER_POT_5,
    RC_SPIRIT_TEMPLE_CENTRAL_CHAMBER_POT_6,
    RC_SPIRIT_TEMPLE_BEAMOS_HALL_POT_1,
    RC_ICE_CAVERN_HALL_POT_1,
    RC_ICE_CAVERN_HALL_POT_2,
    RC_ICE_CAVERN_SPINNING_BLADE_POT_1,
    RC_ICE_CAVERN_SPINNING_BLADE_POT_2,
    RC_ICE_CAVERN_SPINNING_BLADE_POT_3,
    RC_ICE_CAVERN_NEAR_END_POT_1,
    RC_ICE_CAVERN_NEAR_END_POT_2,
    RC_ICE_CAVERN_FROZEN_POT_1,
    RC_GANONS_CASTLE_FOREST_TRIAL_POT_1,
    RC_GANONS_CASTLE_FOREST_TRIAL_POT_2,
    RC_GANONS_CASTLE_FIRE_TRIAL_POT_1,
    RC_GANONS_CASTLE_FIRE_TRIAL_POT_2,
    RC_GANONS_CASTLE_WATER_TRIAL_POT_1,
    RC_GANONS_CASTLE_WATER_TRIAL_POT_2,
    RC_GANONS_CASTLE_WATER_TRIAL_POT_3,
    RC_GANONS_CASTLE_SHADOW_TRIAL_POT_1,
    RC_GANONS_CASTLE_SHADOW_TRIAL_POT_2,
    RC_GANONS_CASTLE_SHADOW_TRIAL_POT_3,
    RC_GANONS_CASTLE_SHADOW_TRIAL_POT_4,
    RC_GANONS_CASTLE_SPIRIT_TRIAL_POT_1,
    RC_GANONS_CASTLE_SPIRIT_TRIAL_POT_2,
    RC_GANONS_CASTLE_LIGHT_TRIAL_BOULDER_POT_1,
    RC_GANONS_CASTLE_LIGHT_TRIAL_POT_1,
    RC_GANONS_CASTLE_LIGHT_TRIAL_POT_2,
    RC_GANONS_CASTLE_GANONS_TOWER_POT_1,
    RC_GANONS_CASTLE_GANONS_TOWER_POT_2,
    RC_GANONS_CASTLE_GANONS_TOWER_POT_3,
    RC_GANONS_CASTLE_GANONS_TOWER_POT_4,
    RC_GANONS_CASTLE_GANONS_TOWER_POT_5,
    RC_GANONS_CASTLE_GANONS_TOWER_POT_6,
    RC_GANONS_CASTLE_GANONS_TOWER_POT_7,
    RC_GANONS_CASTLE_GANONS_TOWER_POT_8,
    RC_GANONS_CASTLE_GANONS_TOWER_POT_9,
    RC_GANONS_CASTLE_GANONS_TOWER_POT_10,
    RC_GANONS_CASTLE_GANONS_TOWER_POT_11,
    RC_GANONS_CASTLE_GANONS_TOWER_POT_12,
    RC_GANONS_CASTLE_GANONS_TOWER_POT_13,
    RC_GANONS_CASTLE_GANONS_TOWER_POT_14,
    RC_GANONS_CASTLE_GANONS_TOWER_POT_15,
    RC_GANONS_CASTLE_GANONS_TOWER_POT_16,
    RC_GANONS_CASTLE_GANONS_TOWER_POT_17,
    RC_GANONS_CASTLE_GANONS_TOWER_POT_18,

    // MQ Dungeon Pots
    RC_DODONGOS_CAVERN_MQ_STAIRCASE_POT_1,
    RC_DODONGOS_CAVERN_MQ_STAIRCASE_POT_2,
    RC_DODONGOS_CAVERN_MQ_STAIRCASE_POT_3,
    RC_DODONGOS_CAVERN_MQ_STAIRCASE_POT_4,
    RC_DODONGOS_CAVERN_MQ_TORCH_PUZZLE_MIDDLE_POT,
    RC_DODONGOS_CAVERN_MQ_BIG_BLOCK_POT_1,
    RC_DODONGOS_CAVERN_MQ_BIG_BLOCK_POT_2,
    RC_DODONGOS_CAVERN_MQ_UPPER_LIZALFOS_POT_1,
    RC_DODONGOS_CAVERN_MQ_UPPER_LIZALFOS_POT_2,
    RC_DODONGOS_CAVERN_MQ_UPPER_LIZALFOS_POT_3,
    RC_DODONGOS_CAVERN_MQ_UPPER_LIZALFOS_POT_4,
    RC_DODONGOS_CAVERN_MQ_TWO_FLAMES_POT_1,
    RC_DODONGOS_CAVERN_MQ_TWO_FLAMES_POT_2,
    RC_DODONGOS_CAVERN_MQ_TORCH_PUZZLE_CORNER_POT,
    RC_DODONGOS_CAVERN_MQ_RIGHT_SIDE_POT_1,
    RC_DODONGOS_CAVERN_MQ_RIGHT_SIDE_POT_2,
    RC_DODONGOS_CAVERN_MQ_RIGHT_SIDE_POT_3,
    RC_DODONGOS_CAVERN_MQ_RIGHT_SIDE_POT_4,
    RC_DODONGOS_CAVERN_MQ_POE_ROOM_POT_1,
    RC_DODONGOS_CAVERN_MQ_POE_ROOM_POT_2,
    RC_DODONGOS_CAVERN_MQ_POE_ROOM_POT_3,
    RC_DODONGOS_CAVERN_MQ_POE_ROOM_POT_4,
    RC_DODONGOS_CAVERN_MQ_BEFORE_BOSS_SW_POT,
    RC_DODONGOS_CAVERN_MQ_BEFORE_BOSS_NE_POT,
    RC_DODONGOS_CAVERN_MQ_ARMOS_ROOM_SE_POT,
    RC_DODONGOS_CAVERN_MQ_ARMOS_ROOM_SW_POT,
    RC_DODONGOS_CAVERN_MQ_BACKROOM_POT_1,
    RC_DODONGOS_CAVERN_MQ_BACKROOM_POT_2,
    RC_DODONGOS_CAVERN_MQ_ARMOS_ROOM_NW_POT,
    RC_DODONGOS_CAVERN_MQ_ARMOS_ROOM_NE_POT,
    RC_JABU_JABUS_BELLY_MQ_ENTRANCE_POT_1,
    RC_JABU_JABUS_BELLY_MQ_ENTRANCE_POT_2,
    RC_JABU_JABUS_BELLY_MQ_GEYSER_POT_1,
    RC_JABU_JABUS_BELLY_MQ_GEYSER_POT_2,
    RC_JABU_JABUS_BELLY_MQ_TIME_BLOCK_POT_1,
    RC_JABU_JABUS_BELLY_MQ_TIME_BLOCK_POT_2,
    RC_JABU_JABUS_BELLY_MQ_LIKE_LIKES_POT_1,
    RC_JABU_JABUS_BELLY_MQ_LIKE_LIKES_POT_2,
    RC_JABU_JABUS_BELLY_MQ_BEFORE_BOSS_POT_1,
    RC_BOTTOM_OF_THE_WELL_MQ_INNER_LOBBY_POT_1,
    RC_BOTTOM_OF_THE_WELL_MQ_INNER_LOBBY_POT_2,
    RC_BOTTOM_OF_THE_WELL_MQ_INNER_LOBBY_POT_3,
    RC_BOTTOM_OF_THE_WELL_MQ_OUTER_LOBBY_POT,
    RC_BOTTOM_OF_THE_WELL_MQ_EAST_INNER_ROOM_POT_1,
    RC_BOTTOM_OF_THE_WELL_MQ_EAST_INNER_ROOM_POT_2,
    RC_BOTTOM_OF_THE_WELL_MQ_EAST_INNER_ROOM_POT_3,
    RC_FOREST_TEMPLE_MQ_LOBBY_POT_1,
    RC_FOREST_TEMPLE_MQ_LOBBY_POT_2,
    RC_FOREST_TEMPLE_MQ_LOBBY_POT_3,
    RC_FOREST_TEMPLE_MQ_LOBBY_POT_4,
    RC_FOREST_TEMPLE_MQ_LOBBY_POT_5,
    RC_FOREST_TEMPLE_MQ_LOBBY_POT_6,
    RC_FOREST_TEMPLE_MQ_WOLFOS_POT_1,
    RC_FOREST_TEMPLE_MQ_WOLFOS_POT_2,
    RC_FOREST_TEMPLE_MQ_UPPER_STALFOS_POT_1,
    RC_FOREST_TEMPLE_MQ_UPPER_STALFOS_POT_2,
    RC_FOREST_TEMPLE_MQ_UPPER_STALFOS_POT_3,
    RC_FOREST_TEMPLE_MQ_UPPER_STALFOS_POT_4,
    RC_FOREST_TEMPLE_MQ_BLUE_POE_POT_1,
    RC_FOREST_TEMPLE_MQ_BLUE_POE_POT_2,
    RC_FOREST_TEMPLE_MQ_BLUE_POE_POT_3,
    RC_FOREST_TEMPLE_MQ_GREEN_POE_POT_1,
    RC_FOREST_TEMPLE_MQ_GREEN_POE_POT_2,
    RC_FOREST_TEMPLE_MQ_BASEMENT_POT_1,
    RC_FOREST_TEMPLE_MQ_BASEMENT_POT_2,
    RC_FOREST_TEMPLE_MQ_BASEMENT_POT_3,
    RC_FOREST_TEMPLE_MQ_BASEMENT_POT_4,
    RC_FIRE_TEMPLE_MQ_ENTRANCE_POT_1,
    RC_FIRE_TEMPLE_MQ_ENTRANCE_POT_2,
    RC_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_1,
    RC_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_2,
    RC_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_3,
    RC_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_4,
    RC_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_5,
    RC_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_6,
    RC_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_7,
    RC_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_8,
    RC_FIRE_TEMPLE_MQ_OUTSIDE_BOSS_POT_1,
    RC_FIRE_TEMPLE_MQ_OUTSIDE_BOSS_POT_2,
    RC_FIRE_TEMPLE_MQ_LAVA_ROOM_NORTH_POT,
    RC_FIRE_TEMPLE_MQ_LAVA_ROOM_HIGH_POT,
    RC_FIRE_TEMPLE_MQ_LAVA_ROOM_SOUTH_POT,
    RC_FIRE_TEMPLE_MQ_LAVA_TORCH_POT_1,
    RC_FIRE_TEMPLE_MQ_LAVA_TORCH_POT_2,
    RC_FIRE_TEMPLE_MQ_ABOVE_LAVA_POT_1,
    RC_FIRE_TEMPLE_MQ_ABOVE_LAVA_POT_2,
    RC_FIRE_TEMPLE_MQ_ABOVE_LAVA_POT_3,
    RC_FIRE_TEMPLE_MQ_FLAME_WALL_POT_1,
    RC_FIRE_TEMPLE_MQ_FLAME_WALL_POT_2,
    RC_FIRE_TEMPLE_MQ_PAST_FIRE_MAZE_SOUTH_POT,
    RC_FIRE_TEMPLE_MQ_PAST_FIRE_MAZE_NORTH_POT,
    RC_FIRE_TEMPLE_MQ_FIRE_MAZE_NORTHMOST_POT,
    RC_FIRE_TEMPLE_MQ_FIRE_MAZE_NORTHWEST_POT,
    RC_FIRE_TEMPLE_MQ_SOUTH_FIRE_MAZE_WEST_POT,
    RC_FIRE_TEMPLE_MQ_SOUTH_FIRE_MAZE_EAST_POT,
    RC_WATER_TEMPLE_MQ_LOWER_TORCHES_POT_1,
    RC_WATER_TEMPLE_MQ_LOWER_TORCHES_POT_2,
    RC_WATER_TEMPLE_MQ_STORAGE_ROOM_A_POT_1,
    RC_WATER_TEMPLE_MQ_STORAGE_ROOM_A_POT_2,
    RC_WATER_TEMPLE_MQ_STORAGE_ROOM_A_POT_3,
    RC_WATER_TEMPLE_MQ_GS_STORAGE_ROOM_POT_1,
    RC_WATER_TEMPLE_MQ_GS_STORAGE_ROOM_POT_2,
    RC_WATER_TEMPLE_MQ_GS_STORAGE_ROOM_POT_3,
    RC_WATER_TEMPLE_MQ_LIZALFOS_HALLWAY_WEST_POT,
    RC_WATER_TEMPLE_MQ_LIZALFOS_HALLWAY_SOUTH_POT,
    RC_WATER_TEMPLE_MQ_LIZALFOS_HALLWAY_SE_POT,
    RC_WATER_TEMPLE_MQ_LIZALFOS_CAGE_SOUTH_POT,
    RC_WATER_TEMPLE_MQ_LIZALFOS_CAGE_NORTH_POT,
    RC_WATER_TEMPLE_MQ_STALFOS_PIT_MIDDLE_POT,
    RC_WATER_TEMPLE_MQ_STALFOS_PIT_SOUTH_POT,
    RC_WATER_TEMPLE_MQ_STALFOS_PIT_NORTH_POT,
    RC_WATER_TEMPLE_MQ_BEFORE_DARK_LINK_POT_1,
    RC_WATER_TEMPLE_MQ_BEFORE_DARK_LINK_POT_2,
    RC_WATER_TEMPLE_MQ_AFTER_DARK_LINK_POT_1,
    RC_WATER_TEMPLE_MQ_AFTER_DARK_LINK_POT_2,
    RC_WATER_TEMPLE_MQ_RIVER_POT_1,
    RC_WATER_TEMPLE_MQ_RIVER_POT_2,
    RC_WATER_TEMPLE_MQ_BOSS_KEY_POT,
    RC_WATER_TEMPLE_MQ_LOWEST_GS_POT_1,
    RC_WATER_TEMPLE_MQ_LOWEST_GS_POT_2,
    RC_WATER_TEMPLE_MQ_LOWEST_GS_POT_3,
    RC_WATER_TEMPLE_MQ_LOWEST_GS_POT_4,
    RC_GERUDO_TRAINING_GROUND_MQ_LOBBY_LEFT_POT_1,
    RC_GERUDO_TRAINING_GROUND_MQ_LOBBY_LEFT_POT_2,
    RC_GERUDO_TRAINING_GROUND_MQ_LOBBY_RIGHT_POT_1,
    RC_GERUDO_TRAINING_GROUND_MQ_LOBBY_RIGHT_POT_2,
    RC_WATER_TEMPLE_MQ_STORAGE_ROOM_B_POT_1,
    RC_WATER_TEMPLE_MQ_STORAGE_ROOM_B_POT_2,
    RC_WATER_TEMPLE_MQ_MINI_DODONGO_POT_1,
    RC_WATER_TEMPLE_MQ_MINI_DODONGO_POT_2,
    RC_SHADOW_TEMPLE_MQ_WHISPERING_WALLS_POT_1,
    RC_SHADOW_TEMPLE_MQ_WHISPERING_WALLS_POT_2,
    RC_SHADOW_TEMPLE_MQ_ENTRANCE_REDEAD_POT_1,
    RC_SHADOW_TEMPLE_MQ_ENTRANCE_REDEAD_POT_2,
    RC_SHADOW_TEMPLE_MQ_LOWER_UMBRELLA_WEST_POT,
    RC_SHADOW_TEMPLE_MQ_LOWER_UMBRELLA_EAST_POT,
    RC_SHADOW_TEMPLE_MQ_UPPER_UMBRELLA_SOUTH_POT,
    RC_SHADOW_TEMPLE_MQ_UPPER_UMBRELLA_NORTH_POT,
    RC_SHADOW_TEMPLE_MQ_BEFORE_BOAT_POT_1,
    RC_SHADOW_TEMPLE_MQ_BEFORE_BOAT_POT_2,
    RC_SHADOW_TEMPLE_MQ_BEFORE_CHASM_WEST_POT,
    RC_SHADOW_TEMPLE_MQ_BEFORE_CHASM_EAST_POT,
    RC_SHADOW_TEMPLE_MQ_AFTER_CHASM_WEST_POT,
    RC_SHADOW_TEMPLE_MQ_AFTER_CHASM_EAST_POT,
    RC_SHADOW_TEMPLE_MQ_SPIKE_BARICADE_POT,
    RC_SHADOW_TEMPLE_MQ_DEAD_HAND_POT_1,
    RC_SHADOW_TEMPLE_MQ_DEAD_HAND_POT_2,
    RC_SPIRIT_TEMPLE_MQ_ENTRANCE_POT_1,
    RC_SPIRIT_TEMPLE_MQ_ENTRANCE_POT_2,
    RC_SPIRIT_TEMPLE_MQ_ENTRANCE_POT_3,
    RC_SPIRIT_TEMPLE_MQ_ENTRANCE_POT_4,
    RC_SPIRIT_TEMPLE_MQ_CHILD_SLUGMA_POT,
    RC_SPIRIT_TEMPLE_MQ_CHILD_GIBDO_POT_1,
    RC_SPIRIT_TEMPLE_MQ_CHILD_GIBDO_POT_2,
    RC_SPIRIT_TEMPLE_MQ_CHILD_STALFOS_POT_1,
    RC_SPIRIT_TEMPLE_MQ_CHILD_STALFOS_POT_2,
    RC_SPIRIT_TEMPLE_MQ_CHILD_STALFOS_POT_3,
    RC_SPIRIT_TEMPLE_MQ_CHILD_STALFOS_POT_4,
    RC_SPIRIT_TEMPLE_MQ_CHILD_LIKE_LIKE_POT,
    RC_SPIRIT_TEMPLE_MQ_STATUE_2F_CENTER_EAST_POT,
    RC_SPIRIT_TEMPLE_MQ_STATUE_3F_EAST_POT,
    RC_SPIRIT_TEMPLE_MQ_STATUE_3F_WEST_POT,
    RC_SPIRIT_TEMPLE_MQ_STATUE_2F_WEST_POT,
    RC_SPIRIT_TEMPLE_MQ_STATUE_2F_EASTMOST_POT,
    RC_SPIRIT_TEMPLE_MQ_SUN_BLOCKS_POT_1,
    RC_SPIRIT_TEMPLE_MQ_SUN_BLOCKS_POT_2,
    RC_SPIRIT_TEMPLE_MQ_LONG_CLIMB_POT_1,
    RC_SPIRIT_TEMPLE_MQ_LONG_CLIMB_POT_2,
    RC_SPIRIT_TEMPLE_MQ_BIG_MIRROR_POT_1,
    RC_SPIRIT_TEMPLE_MQ_BIG_MIRROR_POT_2,
    RC_SPIRIT_TEMPLE_MQ_BIG_MIRROR_POT_3,
    RC_SPIRIT_TEMPLE_MQ_BIG_MIRROR_POT_4,
    RC_SPIRIT_TEMPLE_MQ_BEFORE_MIRROR_POT_1,
    RC_SPIRIT_TEMPLE_MQ_BEFORE_MIRROR_POT_2,
    RC_SPIRIT_TEMPLE_MQ_EARLY_ADULT_POT_1,
    RC_SPIRIT_TEMPLE_MQ_EARLY_ADULT_POT_2,
    RC_ICE_CAVERN_MQ_ENTRANCE_POT,
    RC_ICE_CAVERN_MQ_FIRST_CRYSTAL_POT_1,
    RC_ICE_CAVERN_MQ_FIRST_CRYSTAL_POT_2,
    RC_ICE_CAVERN_MQ_EARLY_WOLFOS_POT_1,
    RC_ICE_CAVERN_MQ_EARLY_WOLFOS_POT_2,
    RC_ICE_CAVERN_MQ_EARLY_WOLFOS_POT_3,
    RC_ICE_CAVERN_MQ_EARLY_WOLFOS_POT_4,
    RC_ICE_CAVERN_MQ_PUSH_BLOCK_POT_1,
    RC_ICE_CAVERN_MQ_PUSH_BLOCK_POT_2,
    RC_ICE_CAVERN_MQ_COMPASS_POT_1,
    RC_ICE_CAVERN_MQ_COMPASS_POT_2,
    RC_GANONS_CASTLE_MQ_FOREST_TRIAL_POT_1,
    RC_GANONS_CASTLE_MQ_FOREST_TRIAL_POT_2,
    RC_GANONS_CASTLE_MQ_WATER_TRIAL_POT_1,
    RC_GANONS_CASTLE_MQ_WATER_TRIAL_POT_2,
    RC_GANONS_CASTLE_MQ_SHADOW_TRIAL_POT_1,
    RC_GANONS_CASTLE_MQ_SHADOW_TRIAL_POT_2,
    RC_GANONS_CASTLE_MQ_FIRE_TRIAL_POT_1,
    RC_GANONS_CASTLE_MQ_FIRE_TRIAL_POT_2,
    RC_GANONS_CASTLE_MQ_LIGHT_TRIAL_POT_1,
    RC_GANONS_CASTLE_MQ_LIGHT_TRIAL_POT_2,
    RC_GANONS_CASTLE_MQ_SPIRIT_TRIAL_POT_1,
    RC_GANONS_CASTLE_MQ_SPIRIT_TRIAL_POT_2,
    // End Pots

    // Overworld Crates
    RC_GV_FREESTANDING_POH_CRATE,
    RC_GV_NEAR_COW_CRATE,
    RC_GV_CRATE_BRIDGE_1,
    RC_GV_CRATE_BRIDGE_2,
    RC_GV_CRATE_BRIDGE_3,
    RC_GV_CRATE_BRIDGE_4,
    RC_GF_ABOVE_JAIL_CRATE,
    RC_GF_OUTSIDE_CENTER_CRATE_1,
    RC_GF_OUTSIDE_CENTER_CRATE_2,
    RC_GF_OUTSIDE_CENTER_CRATE_3,
    RC_GF_OUTSIDE_CENTER_CRATE_4,
    RC_GF_OUTSIDE_LEFT_CRATE_1,
    RC_GF_OUTSIDE_LEFT_CRATE_2,
    RC_GF_ARCHERY_RANGE_CRATE_1,
    RC_GF_ARCHERY_RANGE_CRATE_2,
    RC_GF_ARCHERY_RANGE_CRATE_3,
    RC_GF_ARCHERY_RANGE_CRATE_4,
    RC_GF_ARCHERY_RANGE_CRATE_5,
    RC_GF_ARCHERY_RANGE_CRATE_6,
    RC_GF_ARCHERY_RANGE_CRATE_7,
    RC_GF_ARCHERY_START_CRATE_1,
    RC_GF_ARCHERY_START_CRATE_2,
    RC_GF_ARCHERY_LEFT_END_CRATE_1,
    RC_GF_ARCHERY_LEFT_END_CRATE_2,
    RC_GF_ARCHERY_LEFT_END_CHILD_CRATE,
    RC_GF_ARCHERY_RIGHT_END_CRATE_1,
    RC_GF_ARCHERY_RIGHT_END_CRATE_2,
    RC_GF_KITCHEN_CRATE_1,
    RC_GF_KITCHEN_CRATE_2,
    RC_GF_KITCHEN_CRATE_3,
    RC_GF_KITCHEN_CRATE_4,
    RC_GF_KITCHEN_CRATE_5,
    RC_GF_BREAK_ROOM_CRATE_1,
    RC_GF_BREAK_ROOM_CRATE_2,
    RC_GF_BREAK_ROOM_CRATE_3,
    RC_GF_BREAK_ROOM_CRATE_4,
    RC_GF_NORTH_F1_CARPENTER_CRATE,
    RC_GF_NORTH_F3_CARPENTER_CRATE,
    RC_GF_SOUTH_F2_CARPENTER_CRATE_1,
    RC_GF_SOUTH_F2_CARPENTER_CRATE_2,
    RC_HW_BEFORE_QUICKSAND_CRATE,
    RC_HW_AFTER_QUICKSAND_CRATE_1,
    RC_HW_AFTER_QUICKSAND_CRATE_2,
    RC_HW_AFTER_QUICKSAND_CRATE_3,
    RC_HW_NEAR_COLOSSUS_CRATE,
    RC_MK_NEAR_BAZAAR_CRATE_1,
    RC_MK_NEAR_BAZAAR_CRATE_2,
    RC_MK_SHOOTING_GALLERY_CRATE_1,
    RC_MK_SHOOTING_GALLERY_CRATE_2,
    RC_MK_LOST_DOG_HOUSE_CRATE,
    RC_MK_GUARD_HOUSE_CRATE_1,
    RC_MK_GUARD_HOUSE_CRATE_2,
    RC_MK_GUARD_HOUSE_CRATE_3,
    RC_MK_GUARD_HOUSE_CRATE_4,
    RC_MK_GUARD_HOUSE_CRATE_5,
    RC_KAK_NEAR_OPEN_GROTTO_ADULT_CRATE_1,
    RC_KAK_NEAR_OPEN_GROTTO_ADULT_CRATE_2,
    RC_KAK_NEAR_OPEN_GROTTO_ADULT_CRATE_3,
    RC_KAK_NEAR_OPEN_GROTTO_ADULT_CRATE_4,
    RC_KAK_NEAR_POTION_SHOP_ADULT_CRATE,
    RC_KAK_NEAR_SHOOTING_GALLERY_ADULT_CRATE,
    RC_KAK_NEAR_BOARDING_HOUSE_ADULT_CRATE_1,
    RC_KAK_NEAR_BOARDING_HOUSE_ADULT_CRATE_2,
    RC_KAK_NEAR_IMPAS_HOUSE_ADULT_CRATE_1,
    RC_KAK_NEAR_IMPAS_HOUSE_ADULT_CRATE_2,
    RC_KAK_NEAR_BAZAAR_ADULT_CRATE_1,
    RC_KAK_NEAR_BAZAAR_ADULT_CRATE_2,
    RC_KAK_BEHIND_GS_HOUSE_ADULT_CRATE,
    RC_KAK_NEAR_GY_CHILD_CRATE,
    RC_KAK_NEAR_WINDMILL_CHILD_CRATE,
    RC_KAK_NEAR_FENCE_CHILD_CRATE,
    RC_KAK_NEAR_BOARDING_HOUSE_CHILD_CRATE,
    RC_KAK_NEAR_BAZAAR_CHILD_CRATE,
    RC_GRAVEYARD_CRATE,
    RC_GC_MAZE_CRATE,
    RC_DMC_CRATE,
    RC_LLR_NEAR_TREE_CRATE,
    RC_LH_LAB_CRATE,

    // MQ Dungeon Crates
    RC_DEKU_TREE_MQ_LOBBY_CRATE,
    RC_DEKU_TREE_MQ_SLINGSHOT_ROOM_CRATE_1,
    RC_DEKU_TREE_MQ_SLINGSHOT_ROOM_CRATE_2,
    RC_DODONGOS_CAVERN_MQ_POE_ROOM_CRATE_1,
    RC_DODONGOS_CAVERN_MQ_POE_ROOM_CRATE_2,
    RC_DODONGOS_CAVERN_MQ_POE_ROOM_CRATE_3,
    RC_DODONGOS_CAVERN_MQ_POE_ROOM_CRATE_4,
    RC_DODONGOS_CAVERN_MQ_POE_ROOM_CRATE_5,
    RC_DODONGOS_CAVERN_MQ_POE_ROOM_CRATE_6,
    RC_DODONGOS_CAVERN_MQ_POE_ROOM_CRATE_7,
    RC_DODONGOS_CAVERN_MQ_POE_ROOM_CRATE_8,
    RC_DODONGOS_CAVERN_MQ_STAIRCASE_LOWER_CRATE_1,
    RC_DODONGOS_CAVERN_MQ_STAIRCASE_LOWER_CRATE_2,
    RC_DODONGOS_CAVERN_MQ_STAIRCASE_UPPER_CRATE_1,
    RC_DODONGOS_CAVERN_MQ_STAIRCASE_UPPER_CRATE_2,
    RC_DODONGOS_CAVERN_MQ_STAIRCASE_UPPER_CRATE_3,
    RC_DODONGOS_CAVERN_MQ_STAIRCASE_UPPER_CRATE_4,
    RC_DODONGOS_CAVERN_MQ_TWO_FLAMES_CRATE_1,
    RC_DODONGOS_CAVERN_MQ_TWO_FLAMES_CRATE_2,
    RC_DODONGOS_CAVERN_MQ_LARVAE_ROOM_CRATE_1,
    RC_DODONGOS_CAVERN_MQ_LARVAE_ROOM_CRATE_2,
    RC_DODONGOS_CAVERN_MQ_LARVAE_ROOM_CRATE_3,
    RC_DODONGOS_CAVERN_MQ_LARVAE_ROOM_CRATE_4,
    RC_DODONGOS_CAVERN_MQ_LARVAE_ROOM_CRATE_5,
    RC_DODONGOS_CAVERN_MQ_LARVAE_ROOM_CRATE_6,
    RC_FIRE_TEMPLE_MQ_OUTSIDE_BOSS_CRATE_1,
    RC_FIRE_TEMPLE_MQ_OUTSIDE_BOSS_CRATE_2,
    RC_FIRE_TEMPLE_MQ_OUTSIDE_BOSS_CRATE_3,
    RC_FIRE_TEMPLE_MQ_OUTSIDE_BOSS_CRATE_4,
    RC_FIRE_TEMPLE_MQ_OUTSIDE_BOSS_CRATE_5,
    RC_FIRE_TEMPLE_MQ_OUTSIDE_BOSS_CRATE_6,
    RC_FIRE_TEMPLE_MQ_SHORTCUT_CRATE_1,
    RC_FIRE_TEMPLE_MQ_SHORTCUT_CRATE_2,
    RC_FIRE_TEMPLE_MQ_SHORTCUT_CRATE_3,
    RC_FIRE_TEMPLE_MQ_SHORTCUT_CRATE_4,
    RC_FIRE_TEMPLE_MQ_SHORTCUT_CRATE_5,
    RC_FIRE_TEMPLE_MQ_SHORTCUT_CRATE_6,
    RC_FIRE_TEMPLE_MQ_LIZALFOS_MAZE_LOWER_CRATE_1,
    RC_FIRE_TEMPLE_MQ_LIZALFOS_MAZE_LOWER_CRATE_2,
    RC_FIRE_TEMPLE_MQ_LIZALFOS_MAZE_LOWER_CRATE_3,
    RC_FIRE_TEMPLE_MQ_LIZALFOS_MAZE_UPPER_CRATE_1,
    RC_FIRE_TEMPLE_MQ_LIZALFOS_MAZE_UPPER_CRATE_2,
    RC_FIRE_TEMPLE_MQ_LIZALFOS_MAZE_UPPER_CRATE_3,
    RC_FIRE_TEMPLE_MQ_LAVA_TORCH_CRATE_1,
    RC_FIRE_TEMPLE_MQ_LAVA_TORCH_CRATE_2,
    RC_FIRE_TEMPLE_MQ_LAVA_TORCH_CRATE_3,
    RC_FIRE_TEMPLE_MQ_LAVA_TORCH_CRATE_4,
    RC_FIRE_TEMPLE_MQ_LAVA_TORCH_CRATE_5,
    RC_WATER_TEMPLE_MQ_CENTRAL_PILLAR_UPPER_CRATE_1,
    RC_WATER_TEMPLE_MQ_CENTRAL_PILLAR_UPPER_CRATE_2,
    RC_WATER_TEMPLE_MQ_CENTRAL_PILLAR_LOWER_CRATE_1,
    RC_WATER_TEMPLE_MQ_CENTRAL_PILLAR_LOWER_CRATE_2,
    RC_WATER_TEMPLE_MQ_CENTRAL_PILLAR_LOWER_CRATE_3,
    RC_WATER_TEMPLE_MQ_CENTRAL_PILLAR_LOWER_CRATE_4,
    RC_WATER_TEMPLE_MQ_CENTRAL_PILLAR_LOWER_CRATE_5,
    RC_WATER_TEMPLE_MQ_CENTRAL_PILLAR_LOWER_CRATE_6,
    RC_WATER_TEMPLE_MQ_CENTRAL_PILLAR_LOWER_CRATE_7,
    RC_WATER_TEMPLE_MQ_CENTRAL_PILLAR_LOWER_CRATE_8,
    RC_WATER_TEMPLE_MQ_CENTRAL_PILLAR_LOWER_CRATE_9,
    RC_WATER_TEMPLE_MQ_CENTRAL_PILLAR_LOWER_CRATE_10,
    RC_WATER_TEMPLE_MQ_CENTRAL_PILLAR_LOWER_CRATE_11,
    RC_WATER_TEMPLE_MQ_CENTRAL_PILLAR_LOWER_CRATE_12,
    RC_WATER_TEMPLE_MQ_CENTRAL_PILLAR_LOWER_CRATE_13,
    RC_WATER_TEMPLE_MQ_CENTRAL_PILLAR_LOWER_CRATE_14,
    RC_WATER_TEMPLE_MQ_LIZALFOS_HALLWAY_CRATE_1,
    RC_WATER_TEMPLE_MQ_LIZALFOS_HALLWAY_CRATE_2,
    RC_WATER_TEMPLE_MQ_LIZALFOS_HALLWAY_CRATE_3,
    RC_WATER_TEMPLE_MQ_LIZALFOS_HALLWAY_ROOM_CRATE_1,
    RC_WATER_TEMPLE_MQ_LIZALFOS_HALLWAY_ROOM_CRATE_2,
    RC_WATER_TEMPLE_MQ_LIZALFOS_HALLWAY_ROOM_CRATE_3,
    RC_WATER_TEMPLE_MQ_LIZALFOS_HALLWAY_ROOM_CRATE_4,
    RC_WATER_TEMPLE_MQ_LIZALFOS_HALLWAY_ROOM_CRATE_5,
    RC_WATER_TEMPLE_MQ_LIZALFOS_HALLWAY_GATE_CRATE_1,
    RC_WATER_TEMPLE_MQ_LIZALFOS_HALLWAY_GATE_CRATE_2,
    RC_WATER_TEMPLE_MQ_STORAGE_ROOM_A_CRATE_1,
    RC_WATER_TEMPLE_MQ_STORAGE_ROOM_A_CRATE_2,
    RC_WATER_TEMPLE_MQ_STORAGE_ROOM_A_CRATE_3,
    RC_WATER_TEMPLE_MQ_STORAGE_ROOM_A_CRATE_4,
    RC_WATER_TEMPLE_MQ_STORAGE_ROOM_A_CRATE_5,
    RC_WATER_TEMPLE_MQ_STORAGE_ROOM_A_CRATE_6,
    RC_WATER_TEMPLE_MQ_STORAGE_ROOM_A_CRATE_7,
    RC_WATER_TEMPLE_MQ_GS_STORAGE_ROOM_LOWER_CRATE_1,
    RC_WATER_TEMPLE_MQ_GS_STORAGE_ROOM_LOWER_CRATE_2,
    RC_WATER_TEMPLE_MQ_GS_STORAGE_ROOM_LOWER_CRATE_3,
    RC_WATER_TEMPLE_MQ_GS_STORAGE_ROOM_LOWER_CRATE_4,
    RC_WATER_TEMPLE_MQ_GS_STORAGE_ROOM_LOWER_CRATE_5,
    RC_WATER_TEMPLE_MQ_GS_STORAGE_ROOM_LOWER_CRATE_6,
    RC_WATER_TEMPLE_MQ_GS_STORAGE_ROOM_UPPER_CRATE_1,
    RC_WATER_TEMPLE_MQ_GS_STORAGE_ROOM_UPPER_CRATE_2,
    RC_WATER_TEMPLE_MQ_DRAGON_ROOM_TORCHES_CRATE_1,
    RC_WATER_TEMPLE_MQ_DRAGON_ROOM_TORCHES_CRATE_2,
    RC_WATER_TEMPLE_MQ_DRAGON_ROOM_SUBMERGED_CRATE_1,
    RC_WATER_TEMPLE_MQ_DRAGON_ROOM_SUBMERGED_CRATE_2,
    RC_WATER_TEMPLE_MQ_DRAGON_ROOM_SUBMERGED_CRATE_3,
    RC_WATER_TEMPLE_MQ_DRAGON_ROOM_SUBMERGED_CRATE_4,
    RC_WATER_TEMPLE_MQ_DRAGON_ROOM_DOOR_CRATE_1,
    RC_WATER_TEMPLE_MQ_DRAGON_ROOM_DOOR_CRATE_2,
    RC_WATER_TEMPLE_MQ_BK_ROOM_UPPER_CRATE,
    RC_WATER_TEMPLE_MQ_BK_ROOM_LOWER_CRATE_1,
    RC_WATER_TEMPLE_MQ_BK_ROOM_LOWER_CRATE_2,
    RC_WATER_TEMPLE_MQ_BK_ROOM_LOWER_CRATE_3,
    RC_WATER_TEMPLE_MQ_BK_ROOM_LOWER_CRATE_4,
    RC_WATER_TEMPLE_MQ_WHIRLPOOL_FRONT_CRATE_1,
    RC_WATER_TEMPLE_MQ_WHIRLPOOL_FRONT_CRATE_2,
    RC_WATER_TEMPLE_MQ_WHIRLPOOL_SUBMERGED_CRATE_1,
    RC_WATER_TEMPLE_MQ_WHIRLPOOL_SUBMERGED_CRATE_2,
    RC_WATER_TEMPLE_MQ_WHIRLPOOL_SUBMERGED_CRATE_3,
    RC_WATER_TEMPLE_MQ_WHIRLPOOL_SUBMERGED_CRATE_4,
    RC_WATER_TEMPLE_MQ_WHIRLPOOL_SUBMERGED_CRATE_5,
    RC_WATER_TEMPLE_MQ_WHIRLPOOL_SUBMERGED_CRATE_6,
    RC_WATER_TEMPLE_MQ_WHIRLPOOL_BEHIND_GATE_CRATE_1,
    RC_WATER_TEMPLE_MQ_WHIRLPOOL_BEHIND_GATE_CRATE_2,
    RC_WATER_TEMPLE_MQ_WHIRLPOOL_BEHIND_GATE_CRATE_3,
    RC_WATER_TEMPLE_MQ_WHIRLPOOL_BEHIND_GATE_CRATE_4,
    RC_WATER_TEMPLE_MQ_DODONGO_ROOM_UPPER_CRATE,
    RC_WATER_TEMPLE_MQ_DODONGO_ROOM_HALL_CRATE,
    RC_WATER_TEMPLE_MQ_DODONGO_ROOM_LOWER_CRATE_1,
    RC_WATER_TEMPLE_MQ_DODONGO_ROOM_LOWER_CRATE_2,
    RC_WATER_TEMPLE_MQ_DODONGO_ROOM_LOWER_CRATE_3,
    RC_WATER_TEMPLE_MQ_STORAGE_ROOM_B_CRATE_1,
    RC_WATER_TEMPLE_MQ_STORAGE_ROOM_B_CRATE_2,
    RC_WATER_TEMPLE_MQ_STORAGE_ROOM_B_CRATE_3,
    RC_WATER_TEMPLE_MQ_STORAGE_ROOM_B_CRATE_4,
    RC_WATER_TEMPLE_MQ_STORAGE_ROOM_B_CRATE_5,
    RC_WATER_TEMPLE_MQ_TRIPLE_TORCH_ROOM_SUBMERGED_CRATE_1,
    RC_WATER_TEMPLE_MQ_TRIPLE_TORCH_ROOM_SUBMERGED_CRATE_2,
    RC_WATER_TEMPLE_MQ_TRIPLE_TORCH_ROOM_SUBMERGED_CRATE_3,
    RC_WATER_TEMPLE_MQ_TRIPLE_TORCH_ROOM_SUBMERGED_CRATE_4,
    RC_WATER_TEMPLE_MQ_TRIPLE_TORCH_ROOM_SUBMERGED_CRATE_5,
    RC_WATER_TEMPLE_MQ_TRIPLE_TORCH_ROOM_SUBMERGED_CRATE_6,
    RC_WATER_TEMPLE_MQ_TRIPLE_TORCH_ROOM_GATE_CRATE_1,
    RC_WATER_TEMPLE_MQ_TRIPLE_TORCH_ROOM_GATE_CRATE_2,
    RC_WATER_TEMPLE_MQ_TRIPLE_TORCH_ROOM_GATE_CRATE_3,
    RC_SPIRIT_TEMPLE_MQ_STATUE_CRATE_1,
    RC_SPIRIT_TEMPLE_MQ_STATUE_CRATE_2,
    RC_SPIRIT_TEMPLE_MQ_BIG_MIRROR_CRATE_1,
    RC_SPIRIT_TEMPLE_MQ_BIG_MIRROR_CRATE_2,
    RC_SPIRIT_TEMPLE_MQ_BIG_MIRROR_CRATE_3,
    RC_SPIRIT_TEMPLE_MQ_BIG_MIRROR_CRATE_4,
    RC_GERUDO_TRAINING_GROUND_MQ_MAZE_CRATE,

    // Dungeon Small Crates
    RC_JABU_JABUS_BELLY_PLATFORM_ROOM_SMALL_CRATE_1,
    RC_JABU_JABUS_BELLY_PLATFORM_ROOM_SMALL_CRATE_2,
    RC_FIRE_TEMPLE_AFTER_HAMMER_SMALL_CRATE_1,
    RC_FIRE_TEMPLE_AFTER_HAMMER_SMALL_CRATE_2,
    RC_SPIRIT_TEMPLE_BEFORE_CHILD_CLIMB_SMALL_CRATE_1,
    RC_SPIRIT_TEMPLE_BEFORE_CHILD_CLIMB_SMALL_CRATE_2,

    // MQ Dungeon Small Crates
    RC_JABU_JABUS_BELLY_MQ_TRIPLE_HALLWAY_SMALL_CRATE_1,
    RC_JABU_JABUS_BELLY_MQ_TRIPLE_HALLWAY_SMALL_CRATE_2,
    RC_FOREST_TEMPLE_MQ_FROZEN_EYE_SWITCH_SMALL_CRATE_1,
    RC_FOREST_TEMPLE_MQ_FROZEN_EYE_SWITCH_SMALL_CRATE_2,
    RC_FOREST_TEMPLE_MQ_FROZEN_EYE_SWITCH_SMALL_CRATE_3,
    RC_FIRE_TEMPLE_MQ_LIZALFOS_MAZE_UPPER_SMALL_CRATE_1,
    RC_FIRE_TEMPLE_MQ_LIZALFOS_MAZE_UPPER_SMALL_CRATE_2,
    RC_FIRE_TEMPLE_MQ_LAVA_TORCH_SMALL_CRATE_1,
    RC_FIRE_TEMPLE_MQ_LAVA_TORCH_SMALL_CRATE_2,
    RC_FIRE_TEMPLE_MQ_LAVA_TORCH_SMALL_CRATE_3,
    RC_FIRE_TEMPLE_MQ_LAVA_TORCH_SMALL_CRATE_4,
    RC_FIRE_TEMPLE_MQ_LAVA_TORCH_SMALL_CRATE_5,
    RC_WATER_TEMPLE_MQ_DRAGON_ROOM_TORCHES_SMALL_CRATE_1,
    RC_WATER_TEMPLE_MQ_DRAGON_ROOM_TORCHES_SMALL_CRATE_2,
    RC_WATER_TEMPLE_MQ_DRAGON_ROOM_TORCHES_SMALL_CRATE_3,
    RC_WATER_TEMPLE_MQ_STORAGE_ROOM_A_SMALL_CRATE_1,
    RC_WATER_TEMPLE_MQ_STORAGE_ROOM_A_SMALL_CRATE_2,
    RC_WATER_TEMPLE_MQ_STORAGE_ROOM_A_SMALL_CRATE_3,
    RC_WATER_TEMPLE_MQ_STORAGE_ROOM_A_SMALL_CRATE_4,
    RC_WATER_TEMPLE_MQ_GS_STORAGE_ROOM_LOWER_SMALL_CRATE,
    RC_WATER_TEMPLE_MQ_GS_STORAGE_ROOM_UPPER_SMALL_CRATE,
    RC_SHADOW_TEMPLE_MQ_TRUTH_SPINNER_SMALL_CRATE_1,
    RC_SHADOW_TEMPLE_MQ_TRUTH_SPINNER_SMALL_CRATE_2,
    RC_SHADOW_TEMPLE_MQ_TRUTH_SPINNER_SMALL_CRATE_3,
    RC_SHADOW_TEMPLE_MQ_TRUTH_SPINNER_SMALL_CRATE_4,
    RC_SPIRIT_TEMPLE_MQ_STATUE_SMALL_CRATE,
    RC_SPIRIT_TEMPLE_MQ_BEAMOS_SMALL_CRATE,
    // End Crates

    RC_PIERRE,
    RC_DELIVER_RUTOS_LETTER,
    RC_MASTER_SWORD_PEDESTAL,
    RC_KF_DEKU_TREE_LEFT_GOSSIP_STONE,
    RC_KF_DEKU_TREE_RIGHT_GOSSIP_STONE,
    RC_KF_GOSSIP_STONE,
    RC_KF_STORMS_GROTTO_GOSSIP_STONE,
    RC_LW_GOSSIP_STONE,
    RC_LW_NEAR_SHORTCUTS_GROTTO_GOSSIP_STONE,
    RC_SFM_MAZE_LOWER_GOSSIP_STONE,
    RC_SFM_MAZE_UPPER_GOSSIP_STONE,
    RC_SFM_SARIA_GOSSIP_STONE,
    RC_HF_COW_GROTTO_GOSSIP_STONE,
    RC_HF_NEAR_MARKET_GROTTO_GOSSIP_STONE,
    RC_HF_OPEN_GROTTO_GOSSIP_STONE,
    RC_HF_SOUTHEAST_GROTTO_GOSSIP_STONE,
    RC_TOT_LEFT_CENTER_GOSSIP_STONE,
    RC_TOT_LEFTMOST_GOSSIP_STONE,
    RC_TOT_RIGHT_CENTER_GOSSIP_STONE,
    RC_TOT_RIGHTMOST_GOSSIP_STONE,
    RC_HC_MALON_GOSSIP_STONE,
    RC_HC_ROCK_WALL_GOSSIP_STONE,
    RC_HC_STORMS_GROTTO_GOSSIP_STONE,
    RC_KAK_OPEN_GROTTO_GOSSIP_STONE,
    RC_GRAVEYARD_GOSSIP_STONE,
    RC_DMT_GOSSIP_STONE,
    RC_DMT_STORMS_GROTTO_GOSSIP_STONE,
    RC_GC_MAZE_GOSSIP_STONE,
    RC_GC_MEDIGORON_GOSSIP_STONE,
    RC_DMC_GOSSIP_STONE,
    RC_DMC_UPPER_GROTTO_GOSSIP_STONE,
    RC_ZR_NEAR_DOMAIN_GOSSIP_STONE,
    RC_ZR_NEAR_GROTTOS_GOSSIP_STONE,
    RC_ZR_OPEN_GROTTO_GOSSIP_STONE,
    RC_ZD_GOSSIP_STONE,
    RC_ZF_JABU_GOSSIP_STONE,
    RC_ZF_FAIRY_GOSSIP_STONE,
    RC_LH_LAB_GOSSIP_STONE,
    RC_LH_SOUTHEAST_GOSSIP_STONE,
    RC_LH_SOUTHWEST_GOSSIP_STONE,
    RC_GV_GOSSIP_STONE,
    RC_COLOSSUS_GOSSIP_STONE,
    RC_DODONGOS_CAVERN_GOSSIP_STONE,
    RC_KF_STORMS_GROTTO_BEEHIVE_LEFT,
    RC_KF_STORMS_GROTTO_BEEHIVE_RIGHT,
    RC_LW_NEAR_SHORTCUTS_GROTTO_BEEHIVE_LEFT,
    RC_LW_NEAR_SHORTCUTS_GROTTO_BEEHIVE_RIGHT,
    RC_LW_DEKU_SCRUB_GROTTO_BEEHIVE,
    RC_SFM_STORMS_GROTTO_BEEHIVE,
    RC_HF_NEAR_MARKET_GROTTO_BEEHIVE_LEFT,
    RC_HF_NEAR_MARKET_GROTTO_BEEHIVE_RIGHT,
    RC_HF_OPEN_GROTTO_BEEHIVE_LEFT,
    RC_HF_OPEN_GROTTO_BEEHIVE_RIGHT,
    RC_HF_SOUTHEAST_GROTTO_BEEHIVE_LEFT,
    RC_HF_SOUTHEAST_GROTTO_BEEHIVE_RIGHT,
    RC_HF_INSIDE_FENCE_GROTTO_BEEHIVE,
    RC_LLR_GROTTO_BEEHIVE,
    RC_KAK_OPEN_GROTTO_BEEHIVE_LEFT,
    RC_KAK_OPEN_GROTTO_BEEHIVE_RIGHT,
    RC_DMT_COW_GROTTO_BEEHIVE,
    RC_DMT_STORMS_GROTTO_BEEHIVE_LEFT,
    RC_DMT_STORMS_GROTTO_BEEHIVE_RIGHT,
    RC_GC_GROTTO_BEEHIVE,
    RC_DMC_UPPER_GROTTO_BEEHIVE_LEFT,
    RC_DMC_UPPER_GROTTO_BEEHIVE_RIGHT,
    RC_DMC_HAMMER_GROTTO_BEEHIVE,
    RC_ZR_OPEN_GROTTO_BEEHIVE_LEFT,
    RC_ZR_OPEN_GROTTO_BEEHIVE_RIGHT,
    RC_ZR_STORMS_GROTTO_BEEHIVE,
    RC_ZD_IN_FRONT_OF_KING_ZORA_BEEHIVE_LEFT,
    RC_ZD_IN_FRONT_OF_KING_ZORA_BEEHIVE_RIGHT,
    RC_ZD_BEHIND_KING_ZORA_BEEHIVE,
    RC_LH_GROTTO_BEEHIVE,
    RC_GV_DEKU_SCRUB_GROTTO_BEEHIVE,
    RC_COLOSSUS_GROTTO_BEEHIVE,
    RC_GANONDORF_HINT,
    RC_SHEIK_HINT_GC,
    RC_SHEIK_HINT_MQ_GC,
    RC_TRIFORCE_COMPLETED,
    RC_DAMPE_HINT,
    RC_GREG_HINT,
    RC_SARIA_SONG_HINT,
    RC_ALTAR_HINT_CHILD,
    RC_ALTAR_HINT_ADULT,
    RC_FISHING_POLE_HINT,
    RC_TOT_SHEIK_HINT,
    RC_MASK_SHOP_HINT,
    RC_DMC_UPPER_GROTTO_FISH,
    RC_DMT_STORMS_GROTTO_FISH,
    RC_HF_SOUTHEAST_GROTTO_FISH,
    RC_HF_NEAR_MARKET_GROTTO_FISH,
    RC_HF_OPEN_GROTTO_FISH,
    RC_KAK_OPEN_GROTTO_FISH,
    RC_KF_STORMS_GROTTO_FISH,
    RC_LW_NEAR_SHORTCUTS_GROTTO_FISH,
    RC_ZR_OPEN_GROTTO_FISH,
    RC_ZD_FISH_1,
    RC_ZD_FISH_2,
    RC_ZD_FISH_3,
    RC_ZD_FISH_4,
    RC_ZD_FISH_5,
    RC_KF_BOULDER_RUPEE_1,
    RC_KF_BOULDER_RUPEE_2,
    RC_KF_BRIDGE_RUPEE,
    RC_KF_BEHIND_MIDOS_RUPEE,
    RC_KF_SARIAS_ROOF_WEST_HEART,
    RC_KF_SARIAS_ROOF_EAST_HEART,
    RC_KF_SARIAS_ROOF_NORTH_HEART,
    RC_KF_SOUTH_GRASS_WEST_RUPEE,
    RC_KF_NORTH_GRASS_WEST_RUPEE,
    RC_KF_NORTH_GRASS_EAST_RUPEE,
    RC_KF_SOUTH_GRASS_EAST_RUPEE,
    RC_KF_SARIAS_TOP_LEFT_HEART,
    RC_KF_SARIAS_TOP_RIGHT_HEART,
    RC_KF_SARIAS_BOTTOM_LEFT_HEART,
    RC_KF_SARIAS_BOTTOM_RIGHT_HEART,
    RC_KF_BEAN_RUPEE_1,
    RC_KF_BEAN_RUPEE_2,
    RC_KF_BEAN_RUPEE_3,
    RC_KF_BEAN_RUPEE_4,
    RC_KF_BEAN_RUPEE_5,
    RC_KF_BEAN_RUPEE_6,
    RC_KF_BEAN_RED_RUPEE,
    RC_LW_BOULDER_RUPEE,
    RC_LW_SHORTCUT_RUPEE_1,
    RC_LW_SHORTCUT_RUPEE_2,
    RC_LW_SHORTCUT_RUPEE_3,
    RC_LW_SHORTCUT_RUPEE_4,
    RC_LW_SHORTCUT_RUPEE_5,
    RC_LW_SHORTCUT_RUPEE_6,
    RC_LW_SHORTCUT_RUPEE_7,
    RC_LW_SHORTCUT_RUPEE_8,
    RC_LH_FRONT_RUPEE,
    RC_LH_MIDDLE_RUPEE,
    RC_LH_BACK_RUPEE,
    RC_LH_LAB_FRONT_RUPEE,
    RC_LH_LAB_LEFT_RUPEE,
    RC_LH_LAB_RIGHT_RUPEE,
    RC_GRAVEYARD_DAMPE_RACE_RUPEE_1,
    RC_GRAVEYARD_DAMPE_RACE_RUPEE_2,
    RC_GRAVEYARD_DAMPE_RACE_RUPEE_3,
    RC_GRAVEYARD_DAMPE_RACE_RUPEE_4,
    RC_GRAVEYARD_DAMPE_RACE_RUPEE_5,
    RC_GRAVEYARD_DAMPE_RACE_RUPEE_6,
    RC_GRAVEYARD_DAMPE_RACE_RUPEE_7,
    RC_GRAVEYARD_DAMPE_RACE_RUPEE_8,
    RC_GV_OCTOROK_GROTTO_FRONT_LEFT_BLUE_RUPEE,
    RC_GV_OCTOROK_GROTTO_FRONT_RIGHT_BLUE_RUPEE,
    RC_GV_OCTOROK_GROTTO_BACK_BLUE_RUPEE,
    RC_GV_OCTOROK_GROTTO_FRONT_LEFT_GREEN_RUPEE,
    RC_GV_OCTOROK_GROTTO_FRONT_RIGHT_GREEN_RUPEE,
    RC_GV_OCTOROK_GROTTO_BACK_LEFT_GREEN_RUPEE,
    RC_GV_OCTOROK_GROTTO_BACK_RIGHT_GREEN_RUPEE,
    RC_GV_OCTOROK_GROTTO_RED_RUPEE,
    RC_DMT_BLUE_RUPEE,
    RC_DMT_RED_RUPEE,
    RC_DMT_COW_GROTTO_LEFT_HEART,
    RC_DMT_COW_GROTTO_MIDDLE_LEFT_HEART,
    RC_DMT_COW_GROTTO_MIDDLE_RIGHT_HEART,
    RC_DMT_COW_GROTTO_RIGHT_HEART,
    RC_DMT_COW_GROTTO_RUPEE_1,
    RC_DMT_COW_GROTTO_RUPEE_2,
    RC_DMT_COW_GROTTO_RUPEE_3,
    RC_DMT_COW_GROTTO_RUPEE_4,
    RC_DMT_COW_GROTTO_RUPEE_5,
    RC_DMT_COW_GROTTO_RUPEE_6,
    RC_DMT_COW_GROTTO_RED_RUPEE,
    RC_DMC_NEAR_PLATFORM_RED_RUPEE,
    RC_DMC_MIDDLE_PLATFORM_RED_RUPEE,
    RC_DMC_MIDDLE_PLATFORM_BLUE_RUPEE_1,
    RC_DMC_MIDDLE_PLATFORM_BLUE_RUPEE_2,
    RC_DMC_MIDDLE_PLATFORM_BLUE_RUPEE_3,
    RC_DMC_MIDDLE_PLATFORM_BLUE_RUPEE_4,
    RC_DMC_MIDDLE_PLATFORM_BLUE_RUPEE_5,
    RC_DMC_MIDDLE_PLATFORM_BLUE_RUPEE_6,
    RC_DMC_DISTANT_PLATFORM_RED_RUPEE,
    RC_DMC_DISTANT_PLATFORM_GREEN_RUPEE_1,
    RC_DMC_DISTANT_PLATFORM_GREEN_RUPEE_2,
    RC_DMC_DISTANT_PLATFORM_GREEN_RUPEE_3,
    RC_DMC_DISTANT_PLATFORM_GREEN_RUPEE_4,
    RC_DMC_DISTANT_PLATFORM_GREEN_RUPEE_5,
    RC_DMC_DISTANT_PLATFORM_GREEN_RUPEE_6,
    RC_ZR_BENEATH_WATERFALL_LEFT_RUPEE,
    RC_ZR_BENEATH_WATERFALL_MIDDLE_LEFT_RUPEE,
    RC_ZR_BENEATH_WATERFALL_MIDDLE_RIGHT_RUPEE,
    RC_ZR_BENEATH_WATERFALL_RIGHT_RUPEE,
    RC_ZF_BOTTOM_NORTH_INNER_RUPEE,
    RC_ZF_BOTTOM_NORTHEAST_INNER_RUPEE,
    RC_ZF_BOTTOM_SOUTHEAST_INNER_RUPEE,
    RC_ZF_BOTTOM_SOUTH_INNER_RUPEE,
    RC_ZF_BOTTOM_SOUTHWEST_INNER_RUPEE,
    RC_ZF_BOTTOM_NORTHWEST_INNER_RUPEE,
    RC_ZF_BOTTOM_NORTH_MIDDLE_RUPEE,
    RC_ZF_BOTTOM_NORTHEAST_MIDDLE_RUPEE,
    RC_ZF_BOTTOM_SOUTHEAST_MIDDLE_RUPEE,
    RC_ZF_BOTTOM_SOUTH_MIDDLE_RUPEE,
    RC_ZF_BOTTOM_SOUTHWEST_MIDDLE_RUPEE,
    RC_ZF_BOTTOM_NORTHWEST_MIDDLE_RUPEE,
    RC_ZF_BOTTOM_NORTH_OUTER_RUPEE,
    RC_ZF_BOTTOM_NORTHEAST_OUTER_RUPEE,
    RC_ZF_BOTTOM_SOUTHEAST_OUTER_RUPEE,
    RC_ZF_BOTTOM_SOUTH_OUTER_RUPEE,
    RC_ZF_BOTTOM_SOUTHWEST_OUTER_RUPEE,
    RC_ZF_BOTTOM_NORTHWEST_OUTER_RUPEE,
    RC_DEKU_TREE_LOBBY_LOWER_HEART,
    RC_DEKU_TREE_LOBBY_UPPER_HEART,
    RC_DEKU_TREE_BEFORE_BOSS_LEFT_HEART,
    RC_DEKU_TREE_BEFORE_BOSS_MIDDLE_HEART,
    RC_DEKU_TREE_BEFORE_BOSS_RIGHT_HEART,
    RC_DODONGOS_CAVERN_LOWER_LIZALFOS_HEART,
    RC_DODONGOS_CAVERN_BLADE_ROOM_HEART,
    RC_DODONGOS_CAVERN_UPPER_LIZALFOS_LEFT_HEART,
    RC_DODONGOS_CAVERN_UPPER_LIZALFOS_RIGHT_HEART,
    RC_FOREST_TEMPLE_COURTYARD_RIGHT_HEART,
    RC_FOREST_TEMPLE_COURTYARD_LEFT_HEART,
    RC_FOREST_TEMPLE_WELL_WEST_HEART,
    RC_FOREST_TEMPLE_WELL_EAST_HEART,
    RC_FIRE_TEMPLE_FIRE_PILLAR_LEFT_HEART,
    RC_FIRE_TEMPLE_FIRE_PILLAR_RIGHT_HEART,
    RC_FIRE_TEMPLE_FIRE_PILLAR_BACK_HEART,
    RC_FIRE_TEMPLE_EAST_CENTRAL_LEFT_HEART,
    RC_FIRE_TEMPLE_EAST_CENTRAL_RIGHT_HEART,
    RC_FIRE_TEMPLE_EAST_CENTRAL_MIDDLE_HEART,
    RC_FIRE_TEMPLE_FIRE_WALL_EAST_HEART,
    RC_FIRE_TEMPLE_FIRE_WALL_WEST_HEART,
    RC_FIRE_TEMPLE_FIRE_WALL_EXIT_HEART,
    RC_WATER_TEMPLE_RIVER_HEART_1,
    RC_WATER_TEMPLE_RIVER_HEART_2,
    RC_WATER_TEMPLE_RIVER_HEART_3,
    RC_WATER_TEMPLE_RIVER_HEART_4,
    RC_SHADOW_TEMPLE_INVISIBLE_BLADES_LEFT_HEART,
    RC_SHADOW_TEMPLE_INVISIBLE_BLADES_RIGHT_HEART,
    RC_SHADOW_TEMPLE_SCARECROW_NORTH_HEART,
    RC_SHADOW_TEMPLE_SCARECROW_SOUTH_HEART,
    RC_SHADOW_TEMPLE_AFTER_SHIP_UPPER_LEFT_HEART,
    RC_SHADOW_TEMPLE_AFTER_SHIP_UPPER_RIGHT_HEART,
    RC_SHADOW_TEMPLE_AFTER_SHIP_LOWER_HEART,
    RC_SPIRIT_TEMPLE_ADULT_CLIMB_LEFT_HEART,
    RC_SPIRIT_TEMPLE_ADULT_CLIMB_RIGHT_HEART,
    RC_BOTTOM_OF_THE_WELL_BASEMENT_PLATFORM_LEFT_RUPEE,
    RC_BOTTOM_OF_THE_WELL_BASEMENT_PLATFORM_BACK_LEFT_RUPEE,
    RC_BOTTOM_OF_THE_WELL_BASEMENT_PLATFORM_MIDDLE_RUPEE,
    RC_BOTTOM_OF_THE_WELL_BASEMENT_PLATFORM_BACK_RIGHT_RUPEE,
    RC_BOTTOM_OF_THE_WELL_BASEMENT_PLATFORM_RIGHT_RUPEE,
    RC_BOTTOM_OF_THE_WELL_COFFIN_ROOM_FRONT_LEFT_HEART,
    RC_BOTTOM_OF_THE_WELL_COFFIN_ROOM_MIDDLE_RIGHT_HEART,
    RC_ICE_CAVERN_LOBBY_RUPEE,
    RC_ICE_CAVERN_MAP_ROOM_LEFT_HEART,
    RC_ICE_CAVERN_MAP_ROOM_MIDDLE_HEART,
    RC_ICE_CAVERN_MAP_ROOM_RIGHT_HEART,
    RC_ICE_CAVERN_SLIDING_BLOCK_RUPEE_1,
    RC_ICE_CAVERN_SLIDING_BLOCK_RUPEE_2,
    RC_ICE_CAVERN_SLIDING_BLOCK_RUPEE_3,
    RC_GERUDO_TRAINING_GROUND_BEAMOS_SOUTH_HEART,
    RC_GERUDO_TRAINING_GROUND_BEAMOS_EAST_HEART,
    RC_GANONS_CASTLE_SHADOW_TRIAL_HEART_1,
    RC_GANONS_CASTLE_SHADOW_TRIAL_HEART_2,
    RC_GANONS_CASTLE_SHADOW_TRIAL_HEART_3,
    RC_GANONS_CASTLE_FIRE_TRIAL_HEART,
    RC_GANONS_CASTLE_SPIRIT_TRIAL_HEART,
    RC_DEKU_TREE_MQ_COMPASS_ROOM_HEART,
    RC_DEKU_TREE_MQ_DEKU_BABA_HEART,
    RC_DEKU_TREE_MQ_LOBBY_HEART,
    RC_DEKU_TREE_MQ_SLINGSHOT_ROOM_HEART,
    RC_DEKU_TREE_MQ_BEFORE_BOSS_LEFT_HEART,
    RC_DEKU_TREE_MQ_BEFORE_BOSS_MIDDLE_HEART,
    RC_DEKU_TREE_MQ_BEFORE_BOSS_RIGHT_HEART,
    RC_DODONGOS_CAVERN_MQ_LIZALFOS_ROOM_HEART,
    RC_DODONGOS_CAVERN_MQ_TORCH_PUZZLE_ROOM_HEART,
    RC_JABU_JABUS_BELLY_MQ_LIFT_RUPEE_1,
    RC_JABU_JABUS_BELLY_MQ_LIFT_RUPEE_2,
    RC_JABU_JABUS_BELLY_MQ_LIFT_RUPEE_3,
    RC_JABU_JABUS_BELLY_MQ_LIFT_HEART_1,
    RC_JABU_JABUS_BELLY_MQ_LIFT_HEART_2,
    RC_FOREST_TEMPLE_MQ_COURTYARD_RIGHT_HEART,
    RC_FOREST_TEMPLE_MQ_COURTYARD_MIDDLE_HEART,
    RC_FOREST_TEMPLE_MQ_COURTYARD_LEFT_HEART,
    RC_FOREST_TEMPLE_MQ_WELL_WEST_HEART,
    RC_FOREST_TEMPLE_MQ_WELL_MIDDLE_HEART,
    RC_FOREST_TEMPLE_MQ_WELL_EAST_HEART,
    RC_FIRE_TEMPLE_MQ_FIRE_PILLAR_LEFT_HEART,
    RC_FIRE_TEMPLE_MQ_FIRE_PILLAR_RIGHT_HEART,
    RC_FIRE_TEMPLE_MQ_FIRE_PILLAR_LOWER_HEART,
    RC_SHADOW_TEMPLE_MQ_INVISIBLE_BLADES_LEFT_HEART,
    RC_SHADOW_TEMPLE_MQ_INVISIBLE_BLADES_RIGHT_HEART,
    RC_SHADOW_TEMPLE_MQ_SCARECROW_NORTH_HEART,
    RC_SHADOW_TEMPLE_MQ_SCARECROW_SOUTH_HEART,
    RC_SHADOW_TEMPLE_MQ_AFTER_SHIP_UPPER_LEFT_HEART,
    RC_SHADOW_TEMPLE_MQ_AFTER_SHIP_UPPER_RIGHT_HEART,
    RC_SHADOW_TEMPLE_MQ_AFTER_SHIP_LOWER_HEART,
    RC_SPIRIT_TEMPLE_MQ_CHILD_LEFT_HEART,
    RC_SPIRIT_TEMPLE_MQ_CHILD_RIGHT_HEART,
    RC_BOTTOM_OF_THE_WELL_MQ_COFFIN_ROOM_FRONT_RIGHT_HEART,
    RC_BOTTOM_OF_THE_WELL_MQ_BOMB_LEFT_HEART,
    RC_BOTTOM_OF_THE_WELL_MQ_BOMB_RIGHT_HEART,
    RC_BOTTOM_OF_THE_WELL_MQ_COFFIN_ROOM_MIDDLE_LEFT_HEART,
    RC_BOTTOM_OF_THE_WELL_MQ_BASEMENT_HALLWAY_FRONT_HEART,
    RC_BOTTOM_OF_THE_WELL_MQ_BASEMENT_HALLWAY_LEFT_HEART,
    RC_BOTTOM_OF_THE_WELL_MQ_BASEMENT_HALLWAY_RIGHT_HEART,
    RC_GANONS_CASTLE_MQ_WATER_TRIAL_HEART,
    RC_GANONS_CASTLE_MQ_LIGHT_TRIAL_RIGHT_HEART,
    RC_GANONS_CASTLE_MQ_LIGHT_TRIAL_LEFT_HEART,
    RC_SFM_FAIRY_GROTTO_FAIRY_1,
    RC_SFM_FAIRY_GROTTO_FAIRY_2,
    RC_SFM_FAIRY_GROTTO_FAIRY_3,
    RC_SFM_FAIRY_GROTTO_FAIRY_4,
    RC_SFM_FAIRY_GROTTO_FAIRY_5,
    RC_SFM_FAIRY_GROTTO_FAIRY_6,
    RC_SFM_FAIRY_GROTTO_FAIRY_7,
    RC_SFM_FAIRY_GROTTO_FAIRY_8,
    RC_ZR_FAIRY_GROTTO_FAIRY_1,
    RC_ZR_FAIRY_GROTTO_FAIRY_2,
    RC_ZR_FAIRY_GROTTO_FAIRY_3,
    RC_ZR_FAIRY_GROTTO_FAIRY_4,
    RC_ZR_FAIRY_GROTTO_FAIRY_5,
    RC_ZR_FAIRY_GROTTO_FAIRY_6,
    RC_ZR_FAIRY_GROTTO_FAIRY_7,
    RC_ZR_FAIRY_GROTTO_FAIRY_8,
    RC_HF_FAIRY_GROTTO_FAIRY_1,
    RC_HF_FAIRY_GROTTO_FAIRY_2,
    RC_HF_FAIRY_GROTTO_FAIRY_3,
    RC_HF_FAIRY_GROTTO_FAIRY_4,
    RC_HF_FAIRY_GROTTO_FAIRY_5,
    RC_HF_FAIRY_GROTTO_FAIRY_6,
    RC_HF_FAIRY_GROTTO_FAIRY_7,
    RC_HF_FAIRY_GROTTO_FAIRY_8,
    RC_ZD_FAIRY_GROTTO_FAIRY_1,
    RC_ZD_FAIRY_GROTTO_FAIRY_2,
    RC_ZD_FAIRY_GROTTO_FAIRY_3,
    RC_ZD_FAIRY_GROTTO_FAIRY_4,
    RC_ZD_FAIRY_GROTTO_FAIRY_5,
    RC_ZD_FAIRY_GROTTO_FAIRY_6,
    RC_ZD_FAIRY_GROTTO_FAIRY_7,
    RC_ZD_FAIRY_GROTTO_FAIRY_8,
    RC_GF_FAIRY_GROTTO_FAIRY_1,
    RC_GF_FAIRY_GROTTO_FAIRY_2,
    RC_GF_FAIRY_GROTTO_FAIRY_3,
    RC_GF_FAIRY_GROTTO_FAIRY_4,
    RC_GF_FAIRY_GROTTO_FAIRY_5,
    RC_GF_FAIRY_GROTTO_FAIRY_6,
    RC_GF_FAIRY_GROTTO_FAIRY_7,
    RC_GF_FAIRY_GROTTO_FAIRY_8,
    RC_GRAVEYARD_SHIELD_GRAVE_FAIRY_1,
    RC_GRAVEYARD_SHIELD_GRAVE_FAIRY_2,
    RC_GRAVEYARD_SHIELD_GRAVE_FAIRY_3,
    RC_GRAVEYARD_SHIELD_GRAVE_FAIRY_4,
    RC_GRAVEYARD_SHIELD_GRAVE_FAIRY_5,
    RC_GRAVEYARD_SHIELD_GRAVE_FAIRY_6,
    RC_GRAVEYARD_SHIELD_GRAVE_FAIRY_7,
    RC_GRAVEYARD_SHIELD_GRAVE_FAIRY_8,
    RC_GANONS_CASTLE_SCRUBS_FAIRY_1,
    RC_GANONS_CASTLE_SCRUBS_FAIRY_2,
    RC_GANONS_CASTLE_SCRUBS_FAIRY_3,
    RC_GANONS_CASTLE_SCRUBS_FAIRY_4,
    RC_GANONS_CASTLE_SCRUBS_FAIRY_5,
    RC_GANONS_CASTLE_SCRUBS_FAIRY_6,
    RC_GANONS_CASTLE_SCRUBS_FAIRY_7,
    RC_GANONS_CASTLE_SCRUBS_FAIRY_8,
    RC_GANONS_CASTLE_MQ_SCRUBS_FAIRY_1,
    RC_GANONS_CASTLE_MQ_SCRUBS_FAIRY_2,
    RC_GANONS_CASTLE_MQ_SCRUBS_FAIRY_3,
    RC_GANONS_CASTLE_MQ_SCRUBS_FAIRY_4,
    RC_GANONS_CASTLE_MQ_SCRUBS_FAIRY_5,
    RC_GANONS_CASTLE_MQ_SCRUBS_FAIRY_6,
    RC_GANONS_CASTLE_MQ_SCRUBS_FAIRY_7,
    RC_GANONS_CASTLE_MQ_SCRUBS_FAIRY_8,
    RC_COLOSSUS_OASIS_FAIRY_1,
    RC_COLOSSUS_OASIS_FAIRY_2,
    RC_COLOSSUS_OASIS_FAIRY_3,
    RC_COLOSSUS_OASIS_FAIRY_4,
    RC_COLOSSUS_OASIS_FAIRY_5,
    RC_COLOSSUS_OASIS_FAIRY_6,
    RC_COLOSSUS_OASIS_FAIRY_7,
    RC_COLOSSUS_OASIS_FAIRY_8,
    RC_ZR_BEAN_SPROUT_FAIRY_1,
    RC_ZR_BEAN_SPROUT_FAIRY_2,
    RC_ZR_BEAN_SPROUT_FAIRY_3,
    RC_KF_BEAN_SPROUT_FAIRY_1,
    RC_KF_BEAN_SPROUT_FAIRY_2,
    RC_KF_BEAN_SPROUT_FAIRY_3,
    RC_LW_BEAN_SPROUT_NEAR_BRIDGE_FAIRY_1,
    RC_LW_BEAN_SPROUT_NEAR_BRIDGE_FAIRY_2,
    RC_LW_BEAN_SPROUT_NEAR_BRIDGE_FAIRY_3,
    RC_LW_BEAN_SPROUT_NEAR_THEATER_FAIRY_1,
    RC_LW_BEAN_SPROUT_NEAR_THEATER_FAIRY_2,
    RC_LW_BEAN_SPROUT_NEAR_THEATER_FAIRY_3,
    RC_LH_BEAN_SPROUT_FAIRY_1,
    RC_LH_BEAN_SPROUT_FAIRY_2,
    RC_LH_BEAN_SPROUT_FAIRY_3,
    RC_GV_BEAN_SPROUT_FAIRY_1,
    RC_GV_BEAN_SPROUT_FAIRY_2,
    RC_GV_BEAN_SPROUT_FAIRY_3,
    RC_COLOSSUS_BEAN_SPROUT_FAIRY_1,
    RC_COLOSSUS_BEAN_SPROUT_FAIRY_2,
    RC_COLOSSUS_BEAN_SPROUT_FAIRY_3,
    RC_GRAVEYARD_BEAN_SPROUT_FAIRY_1,
    RC_GRAVEYARD_BEAN_SPROUT_FAIRY_2,
    RC_GRAVEYARD_BEAN_SPROUT_FAIRY_3,
    RC_DMC_BEAN_SPROUT_FAIRY_1,
    RC_DMC_BEAN_SPROUT_FAIRY_2,
    RC_DMC_BEAN_SPROUT_FAIRY_3,
    RC_DMT_BEAN_SPROUT_FAIRY_1,
    RC_DMT_BEAN_SPROUT_FAIRY_2,
    RC_DMT_BEAN_SPROUT_FAIRY_3,
    RC_TOT_LEFTMOST_GOSSIP_STONE_FAIRY,
    RC_TOT_LEFTMOST_GOSSIP_STONE_FAIRY_BIG,
    RC_TOT_LEFT_CENTER_GOSSIP_STONE_FAIRY,
    RC_TOT_LEFT_CENTER_GOSSIP_STONE_FAIRY_BIG,
    RC_TOT_RIGHT_CENTER_GOSSIP_STONE_FAIRY,
    RC_TOT_RIGHT_CENTER_GOSSIP_STONE_FAIRY_BIG,
    RC_TOT_RIGHTMOST_GOSSIP_STONE_FAIRY,
    RC_TOT_RIGHTMOST_GOSSIP_STONE_FAIRY_BIG,
    RC_DMC_GOSSIP_STONE_FAIRY,
    RC_DMC_GOSSIP_STONE_FAIRY_BIG,
    RC_DMT_GOSSIP_STONE_FAIRY,
    RC_DMT_GOSSIP_STONE_FAIRY_BIG,
    RC_COLOSSUS_GOSSIP_STONE_FAIRY,
    RC_COLOSSUS_GOSSIP_STONE_FAIRY_BIG,
    RC_DODONGOS_CAVERN_GOSSIP_STONE_FAIRY,
    RC_DODONGOS_CAVERN_GOSSIP_STONE_FAIRY_BIG,
    RC_DODONGOS_CAVERN_MQ_GOSSIP_STONE_FAIRY,
    RC_DODONGOS_CAVERN_MQ_GOSSIP_STONE_FAIRY_BIG,
    RC_GV_GOSSIP_STONE_FAIRY,
    RC_GV_GOSSIP_STONE_FAIRY_BIG,
    RC_GC_MAZE_GOSSIP_STONE_FAIRY,
    RC_GC_MAZE_GOSSIP_STONE_FAIRY_BIG,
    RC_GC_MEDIGORON_GOSSIP_STONE_FAIRY,
    RC_GC_MEDIGORON_GOSSIP_STONE_FAIRY_BIG,
    RC_GRAVEYARD_GOSSIP_STONE_FAIRY,
    RC_GRAVEYARD_GOSSIP_STONE_FAIRY_BIG,
    RC_HC_MALON_GOSSIP_STONE_FAIRY,
    RC_HC_MALON_GOSSIP_STONE_FAIRY_BIG,
    RC_HC_ROCK_WALL_GOSSIP_STONE_FAIRY,
    RC_HC_ROCK_WALL_GOSSIP_STONE_FAIRY_BIG,
    RC_HC_STORMS_GROTTO_GOSSIP_STONE_FAIRY,
    RC_HC_STORMS_GROTTO_GOSSIP_STONE_FAIRY_BIG,
    RC_KF_DEKU_TREE_LEFT_GOSSIP_STONE_FAIRY,
    RC_KF_DEKU_TREE_LEFT_GOSSIP_STONE_FAIRY_BIG,
    RC_KF_DEKU_TREE_RIGHT_GOSSIP_STONE_FAIRY,
    RC_KF_DEKU_TREE_RIGHT_GOSSIP_STONE_FAIRY_BIG,
    RC_KF_GOSSIP_STONE_FAIRY,
    RC_KF_GOSSIP_STONE_FAIRY_BIG,
    RC_KF_STORMS_GROTTO_GOSSIP_STONE_FAIRY,
    RC_KF_STORMS_GROTTO_GOSSIP_STONE_FAIRY_BIG,
    RC_LH_LAB_GOSSIP_STONE_FAIRY,
    RC_LH_LAB_GOSSIP_STONE_FAIRY_BIG,
    RC_LH_SOUTHEAST_GOSSIP_STONE_FAIRY,
    RC_LH_SOUTHEAST_GOSSIP_STONE_FAIRY_BIG,
    RC_LH_SOUTHWEST_GOSSIP_STONE_FAIRY,
    RC_LH_SOUTHWEST_GOSSIP_STONE_FAIRY_BIG,
    RC_LW_GOSSIP_STONE_FAIRY,
    RC_LW_GOSSIP_STONE_FAIRY_BIG,
    RC_SFM_MAZE_LOWER_GOSSIP_STONE_FAIRY,
    RC_SFM_MAZE_LOWER_GOSSIP_STONE_FAIRY_BIG,
    RC_SFM_MAZE_UPPER_GOSSIP_STONE_FAIRY,
    RC_SFM_MAZE_UPPER_GOSSIP_STONE_FAIRY_BIG,
    RC_SFM_SARIA_GOSSIP_STONE_FAIRY,
    RC_SFM_SARIA_GOSSIP_STONE_FAIRY_BIG,
    RC_ZD_GOSSIP_STONE_FAIRY,
    RC_ZD_GOSSIP_STONE_FAIRY_BIG,
    RC_ZF_FAIRY_GOSSIP_STONE_FAIRY,
    RC_ZF_FAIRY_GOSSIP_STONE_FAIRY_BIG,
    RC_ZF_JABU_GOSSIP_STONE_FAIRY,
    RC_ZF_JABU_GOSSIP_STONE_FAIRY_BIG,
    RC_ZR_NEAR_GROTTOS_GOSSIP_STONE_FAIRY,
    RC_ZR_NEAR_GROTTOS_GOSSIP_STONE_FAIRY_BIG,
    RC_ZR_NEAR_DOMAIN_GOSSIP_STONE_FAIRY,
    RC_ZR_NEAR_DOMAIN_GOSSIP_STONE_FAIRY_BIG,
    RC_HF_COW_GROTTO_GOSSIP_STONE_FAIRY,
    RC_HF_COW_GROTTO_GOSSIP_STONE_FAIRY_BIG,
    RC_HF_NEAR_MARKET_GROTTO_GOSSIP_STONE_FAIRY,
    RC_HF_NEAR_MARKET_GROTTO_GOSSIP_STONE_FAIRY_BIG,
    RC_HF_SOUTHEAST_GROTTO_GOSSIP_STONE_FAIRY,
    RC_HF_SOUTHEAST_GROTTO_GOSSIP_STONE_FAIRY_BIG,
    RC_HF_OPEN_GROTTO_GOSSIP_STONE_FAIRY,
    RC_HF_OPEN_GROTTO_GOSSIP_STONE_FAIRY_BIG,
    RC_KAK_OPEN_GROTTO_GOSSIP_STONE_FAIRY,
    RC_KAK_OPEN_GROTTO_GOSSIP_STONE_FAIRY_BIG,
    RC_ZR_OPEN_GROTTO_GOSSIP_STONE_FAIRY,
    RC_ZR_OPEN_GROTTO_GOSSIP_STONE_FAIRY_BIG,
    RC_LW_NEAR_SHORTCUTS_GROTTO_GOSSIP_STONE_FAIRY,
    RC_LW_NEAR_SHORTCUTS_GROTTO_GOSSIP_STONE_FAIRY_BIG,
    RC_DMT_STORMS_GROTTO_GOSSIP_STONE_FAIRY,
    RC_DMT_STORMS_GROTTO_GOSSIP_STONE_FAIRY_BIG,
    RC_DMC_UPPER_GROTTO_GOSSIP_STONE_FAIRY,
    RC_DMC_UPPER_GROTTO_GOSSIP_STONE_FAIRY_BIG,
    RC_LH_ISLAND_SUN_FAIRY,
    RC_HF_POND_STORMS_FAIRY,
    RC_HF_FENCE_GROTTO_STORMS_FAIRY,
    RC_DMT_FLAG_SUN_FAIRY,
    RC_DMT_COW_GROTTO_STORMS_FAIRY,
    RC_LW_SHORTCUT_STORMS_FAIRY,
    RC_GF_KITCHEN_SUN_FAIRY,
    RC_LW_DEKU_SCRUB_GROTTO_SUN_FAIRY,
    RC_GRAVEYARD_ROYAL_FAMILYS_TOMB_SUN_FAIRY,
    RC_SPIRIT_TEMPLE_BOULDER_ROOM_SUN_FAIRY,
    RC_SPIRIT_TEMPLE_ARMOS_ROOM_SUN_FAIRY,
    RC_SHADOW_TEMPLE_BEAMOS_STORM_FAIRY,
    RC_SHADOW_TEMPLE_PIT_STORM_FAIRY,
    RC_SHADOW_TEMPLE_WIND_HINT_SUN_FAIRY,
    RC_BOTTOM_OF_THE_WELL_BASEMENT_SUN_FAIRY,
    RC_ICE_CAVERN_ENTRANCE_STORMS_FAIRY,
    RC_GERUDO_TRAINING_GROUND_ENTRANCE_STORMS_FAIRY,
    RC_GANONS_CASTLE_SPIRIT_TRIAL_SUN_FAIRY,
    RC_FIRE_TEMPLE_MQ_LOOP_STALFOS_SUN_FAIRY,
    RC_FIRE_TEMPLE_MQ_LOOP_KNUCKLE_SUN_FAIRY,
    RC_WATER_TEMPLE_MQ_DARK_LINK_PILAR_SUN_FAIRY,
    RC_WATER_TEMPLE_MQ_DARK_LINK_LEFT_STORM_FAIRY,
    RC_WATER_TEMPLE_MQ_DARK_LINK_RIGHT_SUN_FAIRY,
    RC_SPIRIT_TEMPLE_MQ_DINALFOS_ROOM_SUN_FAIRY,
    RC_SHADOW_TEMPLE_MQ_BEAMOS_STORM_FAIRY,
    RC_SHADOW_TEMPLE_MQ_PIT_STORM_FAIRY,
    RC_SHADOW_TEMPLE_MQ_WIND_HINT_SUN_FAIRY,
    RC_BOTTOM_OF_THE_WELL_MQ_CELL_SUN_FAIRY,
    RC_BOTTOM_OF_THE_WELL_MQ_BASEMENT_SUN_FAIRY,

    // Overworld Grass
    RC_KF_CHILD_GRASS_1,
    RC_KF_CHILD_GRASS_2,
    RC_KF_CHILD_GRASS_3,
    RC_KF_CHILD_GRASS_4,
    RC_KF_CHILD_GRASS_5,
    RC_KF_CHILD_GRASS_6,
    RC_KF_CHILD_GRASS_7,
    RC_KF_CHILD_GRASS_8,
    RC_KF_CHILD_GRASS_9,
    RC_KF_CHILD_GRASS_10,
    RC_KF_CHILD_GRASS_11,
    RC_KF_CHILD_GRASS_12,
    RC_KF_CHILD_GRASS_MAZE_1,
    RC_KF_CHILD_GRASS_MAZE_2,
    RC_KF_CHILD_GRASS_MAZE_3,
    RC_KF_ADULT_GRASS_1,
    RC_KF_ADULT_GRASS_2,
    RC_KF_ADULT_GRASS_3,
    RC_KF_ADULT_GRASS_4,
    RC_KF_ADULT_GRASS_5,
    RC_KF_ADULT_GRASS_6,
    RC_KF_ADULT_GRASS_7,
    RC_KF_ADULT_GRASS_8,
    RC_KF_ADULT_GRASS_9,
    RC_KF_ADULT_GRASS_10,
    RC_KF_ADULT_GRASS_11,
    RC_KF_ADULT_GRASS_12,
    RC_KF_ADULT_GRASS_13,
    RC_KF_ADULT_GRASS_14,
    RC_KF_ADULT_GRASS_15,
    RC_KF_ADULT_GRASS_16,
    RC_KF_ADULT_GRASS_17,
    RC_KF_ADULT_GRASS_18,
    RC_KF_ADULT_GRASS_19,
    RC_KF_ADULT_GRASS_20,
    RC_LW_GRASS_1,
    RC_LW_GRASS_2,
    RC_LW_GRASS_3,
    RC_LW_GRASS_4,
    RC_LW_GRASS_5,
    RC_LW_GRASS_6,
    RC_LW_GRASS_7,
    RC_LW_GRASS_8,
    RC_LW_GRASS_9,
    RC_MARKET_GRASS_1,
    RC_MARKET_GRASS_2,
    RC_MARKET_GRASS_3,
    RC_MARKET_GRASS_4,
    RC_MARKET_GRASS_5,
    RC_MARKET_GRASS_6,
    RC_MARKET_GRASS_7,
    RC_MARKET_GRASS_8,
    RC_HC_GRASS_1,
    RC_HC_GRASS_2,
    RC_KAK_GRASS_1,
    RC_KAK_GRASS_2,
    RC_KAK_GRASS_3,
    RC_KAK_GRASS_4,
    RC_KAK_GRASS_5,
    RC_KAK_GRASS_6,
    RC_KAK_GRASS_7,
    RC_KAK_GRASS_8,
    RC_GY_GRASS_1,
    RC_GY_GRASS_2,
    RC_GY_GRASS_3,
    RC_GY_GRASS_4,
    RC_GY_GRASS_5,
    RC_GY_GRASS_6,
    RC_GY_GRASS_7,
    RC_GY_GRASS_8,
    RC_GY_GRASS_9,
    RC_GY_GRASS_10,
    RC_GY_GRASS_11,
    RC_GY_GRASS_12,
    RC_LH_GRASS_1,
    RC_LH_GRASS_2,
    RC_LH_GRASS_3,
    RC_LH_GRASS_4,
    RC_LH_GRASS_5,
    RC_LH_GRASS_6,
    RC_LH_GRASS_7,
    RC_LH_GRASS_8,
    RC_LH_GRASS_9,
    RC_LH_GRASS_10,
    RC_LH_GRASS_11,
    RC_LH_GRASS_12,
    RC_LH_GRASS_13,
    RC_LH_GRASS_14,
    RC_LH_GRASS_15,
    RC_LH_GRASS_16,
    RC_LH_GRASS_17,
    RC_LH_GRASS_18,
    RC_LH_GRASS_19,
    RC_LH_GRASS_20,
    RC_LH_GRASS_21,
    RC_LH_GRASS_22,
    RC_LH_GRASS_23,
    RC_LH_GRASS_24,
    RC_LH_GRASS_25,
    RC_LH_GRASS_26,
    RC_LH_GRASS_27,
    RC_LH_GRASS_28,
    RC_LH_GRASS_29,
    RC_LH_GRASS_30,
    RC_LH_GRASS_31,
    RC_LH_GRASS_32,
    RC_LH_GRASS_33,
    RC_LH_GRASS_34,
    RC_LH_GRASS_35,
    RC_LH_GRASS_36,
    RC_LH_CHILD_GRASS_1,
    RC_LH_CHILD_GRASS_2,
    RC_LH_CHILD_GRASS_3,
    RC_LH_CHILD_GRASS_4,
    RC_LH_WARP_PAD_GRASS_1,
    RC_LH_WARP_PAD_GRASS_2,
    RC_HF_NEAR_KF_GRASS_1,
    RC_HF_NEAR_KF_GRASS_2,
    RC_HF_NEAR_KF_GRASS_3,
    RC_HF_NEAR_KF_GRASS_4,
    RC_HF_NEAR_KF_GRASS_5,
    RC_HF_NEAR_KF_GRASS_6,
    RC_HF_NEAR_KF_GRASS_7,
    RC_HF_NEAR_KF_GRASS_8,
    RC_HF_NEAR_KF_GRASS_9,
    RC_HF_NEAR_KF_GRASS_10,
    RC_HF_NEAR_KF_GRASS_11,
    RC_HF_NEAR_KF_GRASS_12,
    RC_HF_NEAR_MARKET_GRASS_1,
    RC_HF_NEAR_MARKET_GRASS_2,
    RC_HF_NEAR_MARKET_GRASS_3,
    RC_HF_NEAR_MARKET_GRASS_4,
    RC_HF_NEAR_MARKET_GRASS_5,
    RC_HF_NEAR_MARKET_GRASS_6,
    RC_HF_NEAR_MARKET_GRASS_7,
    RC_HF_NEAR_MARKET_GRASS_8,
    RC_HF_NEAR_MARKET_GRASS_9,
    RC_HF_NEAR_MARKET_GRASS_10,
    RC_HF_NEAR_MARKET_GRASS_11,
    RC_HF_NEAR_MARKET_GRASS_12,
    RC_HF_SOUTH_GRASS_1,
    RC_HF_SOUTH_GRASS_2,
    RC_HF_SOUTH_GRASS_3,
    RC_HF_SOUTH_GRASS_4,
    RC_HF_SOUTH_GRASS_5,
    RC_HF_SOUTH_GRASS_6,
    RC_HF_SOUTH_GRASS_7,
    RC_HF_SOUTH_GRASS_8,
    RC_HF_SOUTH_GRASS_9,
    RC_HF_SOUTH_GRASS_10,
    RC_HF_SOUTH_GRASS_11,
    RC_HF_SOUTH_GRASS_12,
    RC_HF_CENTRAL_GRASS_1,
    RC_HF_CENTRAL_GRASS_2,
    RC_HF_CENTRAL_GRASS_3,
    RC_HF_CENTRAL_GRASS_4,
    RC_HF_CENTRAL_GRASS_5,
    RC_HF_CENTRAL_GRASS_6,
    RC_HF_CENTRAL_GRASS_7,
    RC_HF_CENTRAL_GRASS_8,
    RC_HF_CENTRAL_GRASS_9,
    RC_HF_CENTRAL_GRASS_10,
    RC_HF_CENTRAL_GRASS_11,
    RC_HF_CENTRAL_GRASS_12,
    RC_ZR_GRASS_1,
    RC_ZR_GRASS_2,
    RC_ZR_GRASS_3,
    RC_ZR_GRASS_4,
    RC_ZR_GRASS_5,
    RC_ZR_GRASS_6,
    RC_ZR_GRASS_7,
    RC_ZR_GRASS_8,
    RC_ZR_GRASS_9,
    RC_ZR_GRASS_10,
    RC_ZR_GRASS_11,
    RC_ZR_GRASS_12,
    RC_ZR_NEAR_FREESTANDING_POH_GRASS,
    // Grotto Grass
    RC_KF_STORMS_GROTTO_GRASS_1,
    RC_KF_STORMS_GROTTO_GRASS_2,
    RC_KF_STORMS_GROTTO_GRASS_3,
    RC_KF_STORMS_GROTTO_GRASS_4,
    RC_LW_NEAR_SHORTCUTS_GROTTO_GRASS_1,
    RC_LW_NEAR_SHORTCUTS_GROTTO_GRASS_2,
    RC_LW_NEAR_SHORTCUTS_GROTTO_GRASS_3,
    RC_LW_NEAR_SHORTCUTS_GROTTO_GRASS_4,
    RC_HF_NEAR_MARKET_GROTTO_GRASS_1,
    RC_HF_NEAR_MARKET_GROTTO_GRASS_2,
    RC_HF_NEAR_MARKET_GROTTO_GRASS_3,
    RC_HF_NEAR_MARKET_GROTTO_GRASS_4,
    RC_HF_OPEN_GROTTO_GRASS_1,
    RC_HF_OPEN_GROTTO_GRASS_2,
    RC_HF_OPEN_GROTTO_GRASS_3,
    RC_HF_OPEN_GROTTO_GRASS_4,
    RC_HF_SOUTHEAST_GROTTO_GRASS_1,
    RC_HF_SOUTHEAST_GROTTO_GRASS_2,
    RC_HF_SOUTHEAST_GROTTO_GRASS_3,
    RC_HF_SOUTHEAST_GROTTO_GRASS_4,
    RC_HF_COW_GROTTO_GRASS_1,
    RC_HF_COW_GROTTO_GRASS_2,
    RC_KAK_OPEN_GROTTO_GRASS_1,
    RC_KAK_OPEN_GROTTO_GRASS_2,
    RC_KAK_OPEN_GROTTO_GRASS_3,
    RC_KAK_OPEN_GROTTO_GRASS_4,
    RC_DMT_STORMS_GROTTO_GRASS_1,
    RC_DMT_STORMS_GROTTO_GRASS_2,
    RC_DMT_STORMS_GROTTO_GRASS_3,
    RC_DMT_STORMS_GROTTO_GRASS_4,
    RC_DMT_COW_GROTTO_GRASS_1,
    RC_DMT_COW_GROTTO_GRASS_2,
    RC_DMC_UPPER_GROTTO_GRASS_1,
    RC_DMC_UPPER_GROTTO_GRASS_2,
    RC_DMC_UPPER_GROTTO_GRASS_3,
    RC_DMC_UPPER_GROTTO_GRASS_4,
    RC_ZR_OPEN_GROTTO_GRASS_1,
    RC_ZR_OPEN_GROTTO_GRASS_2,
    RC_ZR_OPEN_GROTTO_GRASS_3,
    RC_ZR_OPEN_GROTTO_GRASS_4,
    // Dungeon Grass
    RC_DEKU_TREE_LOBBY_GRASS_1,
    RC_DEKU_TREE_LOBBY_GRASS_2,
    RC_DEKU_TREE_LOBBY_GRASS_3,
    RC_DEKU_TREE_LOBBY_GRASS_4,
    RC_DEKU_TREE_LOBBY_GRASS_5,
    RC_DEKU_TREE_SLINGSHOT_GRASS_1,
    RC_DEKU_TREE_SLINGSHOT_GRASS_2,
    RC_DEKU_TREE_SLINGSHOT_GRASS_3,
    RC_DEKU_TREE_SLINGSHOT_GRASS_4,
    RC_DEKU_TREE_COMPASS_GRASS_1,
    RC_DEKU_TREE_COMPASS_GRASS_2,
    RC_DEKU_TREE_BASEMENT_GRASS_1,
    RC_DEKU_TREE_BASEMENT_GRASS_2,
    RC_DEKU_TREE_BASEMENT_SCRUB_ROOM_GRASS_1,
    RC_DEKU_TREE_BASEMENT_SCRUB_ROOM_GRASS_2,
    RC_DEKU_TREE_BASEMENT_SCRUB_ROOM_GRASS_3,
    RC_DEKU_TREE_BASEMENT_SCRUB_ROOM_GRASS_4,
    RC_DEKU_TREE_BASEMENT_SPIKE_ROLLER_GRASS_1,
    RC_DEKU_TREE_BASEMENT_SPIKE_ROLLER_GRASS_2,
    RC_DEKU_TREE_BASEMENT_TORCHES_GRASS_1,
    RC_DEKU_TREE_BASEMENT_TORCHES_GRASS_2,
    RC_DEKU_TREE_BASEMENT_LARVAE_GRASS_1,
    RC_DEKU_TREE_BASEMENT_LARVAE_GRASS_2,
    RC_DEKU_TREE_BEFORE_BOSS_GRASS_1,
    RC_DEKU_TREE_BEFORE_BOSS_GRASS_2,
    RC_DEKU_TREE_BEFORE_BOSS_GRASS_3,
    RC_DODONGOS_CAVERN_FIRST_BRIDGE_GRASS,
    RC_DODONGOS_CAVERN_BLADE_GRASS,
    RC_DODONGOS_CAVERN_SINGLE_EYE_GRASS,
    RC_DODONGOS_CAVERN_BEFORE_BOSS_GRASS,
    RC_BOTTOM_OF_THE_WELL_BASEMENT_BEHIND_ROCKS_GRASS_1,
    RC_BOTTOM_OF_THE_WELL_BASEMENT_BEHIND_ROCKS_GRASS_2,
    RC_BOTTOM_OF_THE_WELL_BASEMENT_BEHIND_ROCKS_GRASS_3,
    RC_BOTTOM_OF_THE_WELL_BASEMENT_BEHIND_ROCKS_GRASS_4,
    RC_BOTTOM_OF_THE_WELL_BASEMENT_BEHIND_ROCKS_GRASS_5,
    RC_BOTTOM_OF_THE_WELL_BASEMENT_BEHIND_ROCKS_GRASS_6,
    RC_BOTTOM_OF_THE_WELL_BASEMENT_BEHIND_ROCKS_GRASS_7,
    RC_BOTTOM_OF_THE_WELL_BASEMENT_BEHIND_ROCKS_GRASS_8,
    RC_BOTTOM_OF_THE_WELL_BASEMENT_BEHIND_ROCKS_GRASS_9,
    RC_BOTTOM_OF_THE_WELL_BASEMENT_GRASS_1,
    RC_BOTTOM_OF_THE_WELL_BASEMENT_GRASS_2,
    RC_BOTTOM_OF_THE_WELL_BASEMENT_GRASS_3,
    // MQ Dungeon Grass
    RC_DEKU_TREE_MQ_LOBBY_GRASS_1,
    RC_DEKU_TREE_MQ_LOBBY_GRASS_2,
    RC_DEKU_TREE_MQ_LOBBY_GRASS_3,
    RC_DEKU_TREE_MQ_LOBBY_GRASS_4,
    RC_DEKU_TREE_MQ_LOBBY_GRASS_5,
    RC_DEKU_TREE_MQ_LOBBY_GRASS_6,
    RC_DEKU_TREE_MQ_LOBBY_GRASS_7,
    RC_DEKU_TREE_MQ_SLINGSHOT_GRASS_1,
    RC_DEKU_TREE_MQ_SLINGSHOT_GRASS_2,
    RC_DEKU_TREE_MQ_SLINGSHOT_GRASS_3,
    RC_DEKU_TREE_MQ_SLINGSHOT_GRASS_4,
    RC_DEKU_TREE_MQ_BEFORE_COMPASS_GRASS_1,
    RC_DEKU_TREE_MQ_BEFORE_COMPASS_GRASS_2,
    RC_DEKU_TREE_MQ_BEFORE_COMPASS_GRASS_3,
    RC_DEKU_TREE_MQ_BEFORE_COMPASS_GRASS_4,
    RC_DEKU_TREE_MQ_BEFORE_COMPASS_GRASS_5,
    RC_DEKU_TREE_MQ_BEFORE_COMPASS_GRASS_6,
    RC_DEKU_TREE_MQ_BEFORE_COMPASS_GRASS_7,
    RC_DEKU_TREE_MQ_COMPASS_GRASS_1,
    RC_DEKU_TREE_MQ_COMPASS_GRASS_2,
    RC_DEKU_TREE_MQ_COMPASS_GRASS_3,
    RC_DEKU_TREE_MQ_COMPASS_GRASS_4,
    RC_DEKU_TREE_MQ_BASEMENT_LOWER_GRASS_1,
    RC_DEKU_TREE_MQ_BASEMENT_LOWER_GRASS_2,
    RC_DEKU_TREE_MQ_BASEMENT_LOWER_GRASS_3,
    RC_DEKU_TREE_MQ_BASEMENT_LOWER_GRASS_4,
    RC_DEKU_TREE_MQ_BASEMENT_UPPER_GRASS_1,
    RC_DEKU_TREE_MQ_BASEMENT_UPPER_GRASS_2,
    RC_DEKU_TREE_MQ_BASEMENT_UPPER_GRASS_3,
    RC_DEKU_TREE_MQ_BASEMENT_SPIKE_ROLLER_FRONT_GRASS_1,
    RC_DEKU_TREE_MQ_BASEMENT_SPIKE_ROLLER_FRONT_GRASS_2,
    RC_DEKU_TREE_MQ_BASEMENT_SPIKE_ROLLER_FRONT_GRASS_3,
    RC_DEKU_TREE_MQ_BASEMENT_SPIKE_ROLLER_BACK_GRASS_1,
    RC_DEKU_TREE_MQ_BASEMENT_SPIKE_ROLLER_BACK_GRASS_2,
    RC_DEKU_TREE_MQ_BASEMENT_TORCHES_GRASS_1,
    RC_DEKU_TREE_MQ_BASEMENT_TORCHES_GRASS_2,
    RC_DEKU_TREE_MQ_BASEMENT_TORCHES_GRASS_3,
    RC_DEKU_TREE_MQ_BASEMENT_TORCHES_GRASS_4,
    RC_DEKU_TREE_MQ_BASEMENT_LARVAE_GRASS_1,
    RC_DEKU_TREE_MQ_BASEMENT_LARVAE_GRASS_2,
    RC_DEKU_TREE_MQ_BASEMENT_GRAVES_GRASS_1,
    RC_DEKU_TREE_MQ_BASEMENT_GRAVES_GRASS_2,
    RC_DEKU_TREE_MQ_BASEMENT_GRAVES_GRASS_3,
    RC_DEKU_TREE_MQ_BASEMENT_GRAVES_GRASS_4,
    RC_DEKU_TREE_MQ_BASEMENT_GRAVES_GRASS_5,
    RC_DEKU_TREE_MQ_BASEMENT_BACK_GRASS_1,
    RC_DEKU_TREE_MQ_BASEMENT_BACK_GRASS_2,
    RC_DEKU_TREE_MQ_BASEMENT_BACK_GRASS_3,
    RC_DEKU_TREE_MQ_BEFORE_BOSS_GRASS_1,
    RC_DEKU_TREE_MQ_BEFORE_BOSS_GRASS_2,
    RC_DEKU_TREE_MQ_BEFORE_BOSS_GRASS_3,
    RC_DODONGOS_CAVERN_MQ_COMPASS_GRASS_1,
    RC_DODONGOS_CAVERN_MQ_COMPASS_GRASS_2,
    RC_DODONGOS_CAVERN_MQ_COMPASS_GRASS_3,
    RC_DODONGOS_CAVERN_MQ_COMPASS_GRASS_4,
    RC_DODONGOS_CAVERN_MQ_ARMOS_GRASS,
    RC_DODONGOS_CAVERN_MQ_BACK_POE_GRASS,
    RC_DODONGOS_CAVERN_MQ_SCRUB_GRASS_1,
    RC_DODONGOS_CAVERN_MQ_SCRUB_GRASS_2,
    RC_JABU_JABUS_BELLY_MQ_FIRST_GRASS_1,
    RC_JABU_JABUS_BELLY_MQ_FIRST_GRASS_2,
    RC_JABU_JABUS_BELLY_MQ_PIT_GRASS_1,
    RC_JABU_JABUS_BELLY_MQ_PIT_GRASS_2,
    RC_JABU_JABUS_BELLY_MQ_BASEMENT_GRASS_1,
    RC_JABU_JABUS_BELLY_MQ_BASEMENT_GRASS_2,
    RC_JABU_JABUS_BELLY_MQ_BASEMENT_GRASS_3,
    RC_JABU_JABUS_BELLY_MQ_WIGGLERS_GRASS,
    RC_JABU_JABUS_BELLY_MQ_AFTER_BIG_OCTO_GRASS_1,
    RC_JABU_JABUS_BELLY_MQ_AFTER_BIG_OCTO_GRASS_2,
    RC_JABU_JABUS_BELLY_MQ_FALLING_LIKE_LIKE_GRASS,
    RC_JABU_JABUS_BELLY_MQ_BASEMENT_BOOMERANG_GRASS,
    RC_JABU_JABUS_BELLY_MQ_BEFORE_BOSS_GRASS_1,
    RC_JABU_JABUS_BELLY_MQ_BEFORE_BOSS_GRASS_2,
    RC_BOTTOM_OF_THE_WELL_MQ_DEAD_HAND_GRASS_1,
    RC_BOTTOM_OF_THE_WELL_MQ_DEAD_HAND_GRASS_2,
    RC_BOTTOM_OF_THE_WELL_MQ_DEAD_HAND_GRASS_3,
    RC_BOTTOM_OF_THE_WELL_MQ_DEAD_HAND_GRASS_4,
    // Shared Dungeon Grass
    RC_DEKU_TREE_QUEEN_GOHMA_GRASS_1,
    RC_DEKU_TREE_QUEEN_GOHMA_GRASS_2,
    RC_DEKU_TREE_QUEEN_GOHMA_GRASS_3,
    RC_DEKU_TREE_QUEEN_GOHMA_GRASS_4,
    RC_DEKU_TREE_QUEEN_GOHMA_GRASS_5,
    RC_DEKU_TREE_QUEEN_GOHMA_GRASS_6,
    RC_DEKU_TREE_QUEEN_GOHMA_GRASS_7,
    RC_DEKU_TREE_QUEEN_GOHMA_GRASS_8,
    // End Grass

    RC_MAX
} RandomizerCheck;

// Randomizer tricks and glitches (beta) based on knowledge of what is achievable in Ship of Harkinian randomizer,
// excludes Item manipulation (e.g. RBA/GIM/SRM) and Wrong Warping and Ganon BK Skip
// TODO Fill and alphabetize (area based tricks must have area in name and come alphetized after general tricks)
// TODO test commented out tricks to see if consistent with console on SoH
// TODO add area specific tricks
typedef enum {
    RT_VISIBLE_COLLISION, // -- general tricks
    RT_GROTTOS_WITHOUT_AGONY,
    RT_FEWER_TUNIC_REQUIREMENTS,
    RT_RUSTED_SWITCHES,
    RT_FLAMING_CHESTS,
    RT_BUNNY_HOOD_JUMPS,
    RT_DAMAGE_BOOST_SIMPLE,
    RT_HOVER_BOOST_SIMPLE,
    RT_BOMBCHU_BEEHIVES,
    RT_BLUE_FIRE_MUD_WALLS,
    RT_KF_ADULT_GS, // -- location tricks
    RT_LW_BRIDGE,
    RT_LW_MIDO_BACKFLIP,
    RT_LW_GS_BEAN,
    RT_HC_STORMS_GS,
    RT_KAK_MAN_ON_ROOF,
    RT_KAK_TOWER_GS,
    RT_KAK_ADULT_WINDMILL_POH,
    RT_KAK_CHILD_WINDMILL_POH,
    RT_KAK_ROOFTOP_GS,
    RT_GY_POH,
    RT_GY_CHILD_DAMPE_RACE_POH,
    RT_GY_SHADOW_FIRE_ARROWS,
    RT_DMT_SOIL_GS,
    RT_DMT_BOMBABLE,
    RT_DMT_HOOKSHOT_LOWER_GS,
    RT_DMT_HOVERS_LOWER_GS,
    RT_DMT_BEAN_LOWER_GS,
    RT_DMT_JS_LOWER_GS,
    RT_DMT_CLIMB_HOVERS,
    RT_DMT_UPPER_GS,
    RT_DMT_BOLERO_BIGGORON,
    RT_GC_POT,
    RT_GC_POT_STRENGTH,
    RT_GC_ROLLING_STRENGTH,
    RT_GC_LEFTMOST,
    RT_GC_GROTTO,
    RT_GC_LINK_GORON_DINS,
    RT_DMC_HOVER_BEAN_POH,
    RT_DMC_BOLERO_JUMP,
    RT_DMC_BOULDER_JS,
    RT_DMC_BOULDER_SKIP,
    RT_ZR_LOWER,
    RT_ZR_UPPER,
    RT_ZR_HOVERS,
    RT_ZR_CUCCO,
    RT_ZD_KING_ZORA_SKIP,
    RT_ZD_GS,
    RT_ZF_GREAT_FAIRY_WITHOUT_EXPLOSIVES,
    RT_LH_LAB_WALL_GS,
    RT_LH_LAB_DIVING,
    RT_LH_WATER_HOOKSHOT,
    RT_GV_CRATE_HOVERS,
    RT_GF_KITCHEN,
    RT_GF_JUMP,
    RT_GF_WARRIOR_WITH_DIFFICULT_WEAPON,
    RT_HW_BUNNY_CROSSING,
    RT_HW_CROSSING,
    RT_LENS_HW,
    RT_HW_REVERSE,
    RT_COLOSSUS_GS,
    RT_DEKU_BASEMENT_GS,
    RT_DEKU_B1_SKIP,
    RT_DEKU_B1_BOW_WEBS,
    RT_DEKU_B1_BACKFLIP_OVER_SPIKED_LOG,
    RT_DEKU_MQ_COMPASS_GS,
    RT_DEKU_MQ_LOG,
    RT_DC_SCARECROW_GS,
    RT_DC_VINES_GS,
    RT_DC_STAIRCASE,
    RT_DC_SLINGSHOT_SKIP,
    RT_DC_SCRUB_ROOM,
    RT_DC_JUMP,
    RT_DC_HAMMER_FLOOR,
    RT_DC_MQ_CHILD_BOMBS,
    RT_DC_MQ_CHILD_EYES,
    RT_DC_MQ_ADULT_EYES,
    RT_JABU_ALCOVE_JUMP_DIVE,
    RT_JABU_BOSS_HOVER,
    RT_JABU_NEAR_BOSS_RANGED,
    RT_JABU_NEAR_BOSS_EXPLOSIVES,
    RT_LENS_JABU_MQ,
    RT_JABU_MQ_RANG_JUMP,
    RT_JABU_MQ_SOT_GS,
    RT_LENS_BOTW,
    RT_BOTW_CHILD_DEADHAND,
    RT_BOTW_BASEMENT,
    RT_BOTW_MQ_PITS,
    RT_BOTW_MQ_DEADHAND_KEY,
    RT_FOREST_FIRST_GS,
    RT_FOREST_OUTDOORS_EAST_GS,
    RT_FOREST_VINES,
    RT_FOREST_OUTDOORS_LEDGE,
    RT_FOREST_DOORFRAME,
    RT_FOREST_OUTSIDE_BACKDOOR,
    RT_FOREST_OUTDOORS_HEARTS_BOOMERANG,
    RT_FOREST_MQ_WELL_SWIM,
    RT_FOREST_MQ_BLOCK_PUZZLE,
    RT_FOREST_MQ_JS_HALLWAY_SWITCH,
    RT_FOREST_MQ_HOOKSHOT_HALLWAY_SWITCH,
    RT_FOREST_MQ_RANG_HALLWAY_SWITCH,
    RT_FIRE_BOSS_DOOR_JUMP,
    RT_FIRE_SOT,
    RT_FIRE_STRENGTH,
    RT_FIRE_SCARECROW,
    RT_FIRE_FLAME_MAZE,
    RT_FIRE_MQ_NEAR_BOSS,
    RT_FIRE_MQ_BLOCKED_CHEST,
    RT_FIRE_MQ_BK_CHEST,
    RT_FIRE_MQ_CLIMB,
    RT_FIRE_MQ_MAZE_SIDE_ROOM,
    RT_FIRE_MQ_MAZE_HOVERS,
    RT_FIRE_MQ_MAZE_JUMP,
    RT_FIRE_MQ_ABOVE_MAZE_GS,
    RT_FIRE_MQ_FLAME_MAZE,
    RT_WATER_LONGSHOT_TORCH,
    RT_WATER_CRACKED_WALL_HOVERS,
    RT_WATER_CRACKED_WALL,
    RT_WATER_BK_REGION,
    RT_WATER_NORTH_BASEMENT_LEDGE_JUMP,
    RT_WATER_BK_JUMP_DIVE,
    RT_WATER_FW_CENTRAL_GS,
    RT_WATER_IRONS_CENTRAL_GS,
    RT_WATER_CENTRAL_BOW,
    RT_WATER_HOOKSHOT_FALLING_PLATFORM_GS,
    RT_WATER_RANG_FALLING_PLATFORM_GS,
    RT_WATER_RIVER_GS,
    RT_WATER_DRAGON_JUMP_DIVE,
    RT_WATER_ADULT_DRAGON,
    RT_WATER_CHILD_DRAGON,
    RT_WATER_MQ_CENTRAL_PILLAR,
    RT_WATER_MQ_LOCKED_GS,
    RT_LENS_SHADOW,
    RT_LENS_SHADOW_PLATFORM,
    RT_LENS_BONGO,
    RT_SHADOW_UMBRELLA,
    RT_SHADOW_UMBRELLA_GS,
    RT_SHADOW_FREESTANDING_KEY,
    RT_SHADOW_STATUE,
    RT_SHADOW_BONGO,
    RT_LENS_SHADOW_MQ,
    RT_LENS_SHADOW_MQ_INVISIBLE_BLADES,
    RT_LENS_SHADOW_MQ_PLATFORM,
    RT_LENS_SHADOW_MQ_DEADHAND,
    RT_SHADOW_MQ_GAP,
    RT_SHADOW_MQ_INVISIBLE_BLADES,
    RT_SHADOW_MQ_HUGE_PIT,
    RT_SHADOW_MQ_WINDY_WALKWAY,
    RT_LENS_SPIRIT,
    RT_SPIRIT_CHILD_CHU,
    RT_SPIRIT_LOBBY_GS,
    RT_SPIRIT_LOWER_ADULT_SWITCH,
    RT_SPIRIT_LOBBY_JUMP,
    RT_SPIRIT_PLATFORM_HOOKSHOT,
    RT_SPIRIT_MAP_CHEST,
    RT_SPIRIT_SUN_CHEST,
    RT_SPIRIT_WALL,
    RT_LENS_SPIRIT_MQ,
    RT_SPIRIT_MQ_SUN_BLOCK_SOT,
    RT_SPIRIT_MQ_SUN_BLOCK_GS,
    RT_SPIRIT_MQ_LOWER_ADULT,
    RT_SPIRIT_MQ_FROZEN_EYE,
    RT_ICE_BLOCK_GS,
    RT_ICE_MQ_RED_ICE_GS,
    RT_ICE_MQ_SCARECROW,
    RT_LENS_GTG,
    RT_GTG_WITHOUT_HOOKSHOT,
    RT_GTG_FAKE_WALL,
    RT_LENS_GTG_MQ,
    RT_GTG_MQ_WITH_HOOKSHOT,
    RT_GTG_MQ_WIHTOUT_HOOKSHOT,
    RT_LENS_GANON,
    RT_GANON_SPIRIT_TRIAL_HOOKSHOT,
    RT_LENS_GANON_MQ,
    RT_GANON_MQ_FIRE_TRIAL,
    RT_GANON_MQ_SHADOW_TRIAL,
    RT_GANON_MQ_LIGHT_TRIAL,
    // RT_ROCS_FEATHER                       --if implemented with main branch
    RT_FOREST_TEMPLE_BK_SKIP,
    RT_FIRE_TEMPLE_BK_SKIP,
    RT_WATER_TEMPLE_BK_SKIP,
    RT_SHADOW_TEMPLE_BK_SKIP,
    RT_SPIRIT_BK_SKIP,
    RT_ACUTE_ANGLE_CLIP,
    RT_ADVANCED_CLIPS,
    RT_BLANK_A,
    RT_CUTSCENE_DIVE,
    RT_DAMAGE_BOOST,
    RT_DOOM_JUMP,
    RT_EPG,
    RT_EQUIP_SWAP,
    RT_EQUIP_SWAP_EXPECTS_DINS,
    RT_FLAME_STORAGE,
    RT_FW_VOID_WARP,
    RT_GROUND_CLIP,
    RT_GROUND_JUMP,
    RT_HESS,
    RT_HOOKSHOT_CLIP,
    RT_HOOKSHOT_JUMP,
    RT_HOVERING,
    RT_HOVERING_BOW_SLINGSHOT,
    RT_HOVERING_CONTORTION,
    RT_HOVERING_ENEMY,
    RT_HOVERING_HOVERBOOTS,
    RT_HOVERING_WALL,
    RT_ISG,
    RT_LADDER_CLIP,
    RT_LADDER_CLIP_HOOKSHOT,
    RT_LEDGE_CANCEL,
    RT_LEDGE_CLIP,
    RT_LUNGE_STORAGE,
    RT_MEGAFLIP_BOMB,
    RT_MEGAFLIP_BOMBCHU,
    RT_MEGASIDEHOP_BOMB,
    RT_MEGASIDEHOP_BOMBCHU,
    RT_NAVI_DIVE,
    RT_OCARINA_ITEMS,
    RT_OCARINA_ITEMS_BOMB,
    RT_OCARINA_ITEMS_ESS,
    RT_OCARINA_ITEMS_PUTAWAY,
    RT_ONE_WAY_CLIPS,
    RT_QUICKDRAW,
    RT_QUICK_PUTAWAY,
    RT_SEEMWALK,
    RT_SUPERSLIDE,
    RT_SUPERSLIDE_TELEPORT,
    RT_TIME_STOP,
    RT_TRIPLE_SLASH_CLIP,
    RT_VINE_CLIP,
    RT_WEIRDSHOT,
    RT_WEIRDSHOT_CHU,
    RT_WEIRDSHOT_ENEMY,
    // RT_FW_VOID_WARP_FAST_TEXT,                      --untested tricks:
    // RT_ACTION_SWAP,
    // RT_ACTOR_GLITCH,
    // RT_ADULT_TRADE_TTG,
    RT_TOT_DOT_SKIP_SWORDLESS, // AREA SPECIFIC GLITCHES
    RT_TOT_DOT_SKIP_SWORD,
    RT_LLR_STEAL_EPONA,
    RT_KF_POKEY_SKIP,
    RT_KF_MIDO_SKIP,
    RT_KF_CLIMB_TWINS_HOUSE,
    RT_KF_DEKU_TREE_ADULT,
    RT_KF_WWT_ESCAPE,
    RT_LW_AQUAESCAPE,
    RT_LW_MIDO_SKIP_ADULT,
    RT_OGC_GOLD_GAUNTLET_SKIP,
    RT_GY_ROYAL_TOMB,
    RT_ZF_JABU_ADULT_BOMBCHU,
    RT_ZF_JABU_ADULT_HOVERBOOT_BOMB,
    RT_FOREST_TEMPLE_FOUR_POE_SKIP,
    RT_FOREST_TEMPLE_JUNGLE_JUMP,
    // RT_DARUNIA_CUTSCENE,
    RT_BOTTOM_OF_THE_WELL_SKULL_PUSH,
    RT_GANONS_CASTLE_BARRIER_SKIP_HOVER,
    RT_GANONS_CASTLE_GOLD_GAUNTLET_SKIP,
    RT_MAX
} RandomizerTrick;

// based on https://github.com/TestRunnerSRL/OoT-Randomizer/blob/e337d7f603b91a6bacb618fb32cc7fd70ed9ffca/ItemList.py
typedef enum {
    RG_NONE,
    RG_KOKIRI_SWORD,
    RG_MASTER_SWORD,
    RG_GIANTS_KNIFE,
    RG_BIGGORON_SWORD,
    RG_DEKU_SHIELD,
    RG_HYLIAN_SHIELD,
    RG_MIRROR_SHIELD,
    RG_GORON_TUNIC,
    RG_ZORA_TUNIC,
    RG_IRON_BOOTS,
    RG_HOVER_BOOTS,
    RG_BOOMERANG,
    RG_LENS_OF_TRUTH,
    RG_MEGATON_HAMMER,
    RG_STONE_OF_AGONY,
    RG_DINS_FIRE,
    RG_FARORES_WIND,
    RG_NAYRUS_LOVE,
    RG_FIRE_ARROWS,
    RG_ICE_ARROWS,
    RG_LIGHT_ARROWS,
    RG_GERUDO_MEMBERSHIP_CARD,
    RG_MAGIC_BEAN,
    RG_MAGIC_BEAN_PACK,
    RG_DOUBLE_DEFENSE,
    RG_WEIRD_EGG,
    RG_ZELDAS_LETTER,
    RG_POCKET_EGG,
    RG_COJIRO,
    RG_ODD_MUSHROOM,
    RG_ODD_POTION,
    RG_POACHERS_SAW,
    RG_BROKEN_SWORD,
    RG_PRESCRIPTION,
    RG_EYEBALL_FROG,
    RG_EYEDROPS,
    RG_CLAIM_CHECK,
    RG_GOLD_SKULLTULA_TOKEN,
    RG_PROGRESSIVE_HOOKSHOT,
    RG_PROGRESSIVE_STRENGTH,
    RG_PROGRESSIVE_BOMB_BAG,
    RG_PROGRESSIVE_BOW,
    RG_PROGRESSIVE_SLINGSHOT,
    RG_PROGRESSIVE_WALLET,
    RG_PROGRESSIVE_SCALE,
    RG_PROGRESSIVE_NUT_UPGRADE,
    RG_PROGRESSIVE_STICK_UPGRADE,
    RG_PROGRESSIVE_BOMBCHUS,
    RG_PROGRESSIVE_MAGIC_METER,
    RG_MAGIC_SINGLE, // Added for refactor of GetItemEntries
    RG_MAGIC_DOUBLE, // Added for refactor of GetItemEntries
    RG_PROGRESSIVE_OCARINA,
    RG_PROGRESSIVE_GORONSWORD,
    RG_EMPTY_BOTTLE,
    RG_BOTTLE_WITH_MILK,
    RG_BOTTLE_WITH_RED_POTION,
    RG_BOTTLE_WITH_GREEN_POTION,
    RG_BOTTLE_WITH_BLUE_POTION,
    RG_BOTTLE_WITH_FAIRY,
    RG_BOTTLE_WITH_FISH,
    RG_BOTTLE_WITH_BLUE_FIRE,
    RG_BOTTLE_WITH_BUGS,
    RG_BOTTLE_WITH_POE,
    RG_RUTOS_LETTER,
    RG_BOTTLE_WITH_BIG_POE,
    RG_ZELDAS_LULLABY,
    RG_EPONAS_SONG,
    RG_SARIAS_SONG,
    RG_SUNS_SONG,
    RG_SONG_OF_TIME,
    RG_SONG_OF_STORMS,
    RG_MINUET_OF_FOREST,
    RG_BOLERO_OF_FIRE,
    RG_SERENADE_OF_WATER,
    RG_REQUIEM_OF_SPIRIT,
    RG_NOCTURNE_OF_SHADOW,
    RG_PRELUDE_OF_LIGHT,
    RG_DEKU_TREE_MAP,
    RG_DODONGOS_CAVERN_MAP,
    RG_JABU_JABUS_BELLY_MAP,
    RG_FOREST_TEMPLE_MAP,
    RG_FIRE_TEMPLE_MAP,
    RG_WATER_TEMPLE_MAP,
    RG_SPIRIT_TEMPLE_MAP,
    RG_SHADOW_TEMPLE_MAP,
    RG_BOTTOM_OF_THE_WELL_MAP,
    RG_ICE_CAVERN_MAP,
    RG_DEKU_TREE_COMPASS,
    RG_DODONGOS_CAVERN_COMPASS,
    RG_JABU_JABUS_BELLY_COMPASS,
    RG_FOREST_TEMPLE_COMPASS,
    RG_FIRE_TEMPLE_COMPASS,
    RG_WATER_TEMPLE_COMPASS,
    RG_SPIRIT_TEMPLE_COMPASS,
    RG_SHADOW_TEMPLE_COMPASS,
    RG_BOTTOM_OF_THE_WELL_COMPASS,
    RG_ICE_CAVERN_COMPASS,
    RG_FOREST_TEMPLE_BOSS_KEY,
    RG_FIRE_TEMPLE_BOSS_KEY,
    RG_WATER_TEMPLE_BOSS_KEY,
    RG_SPIRIT_TEMPLE_BOSS_KEY,
    RG_SHADOW_TEMPLE_BOSS_KEY,
    RG_GANONS_CASTLE_BOSS_KEY,
    RG_FOREST_TEMPLE_SMALL_KEY,
    RG_FIRE_TEMPLE_SMALL_KEY,
    RG_WATER_TEMPLE_SMALL_KEY,
    RG_SPIRIT_TEMPLE_SMALL_KEY,
    RG_SHADOW_TEMPLE_SMALL_KEY,
    RG_BOTTOM_OF_THE_WELL_SMALL_KEY,
    RG_GERUDO_TRAINING_GROUND_SMALL_KEY,
    RG_GERUDO_FORTRESS_SMALL_KEY,
    RG_GANONS_CASTLE_SMALL_KEY,
    RG_TREASURE_GAME_SMALL_KEY,
    RG_FOREST_TEMPLE_KEY_RING,
    RG_FIRE_TEMPLE_KEY_RING,
    RG_WATER_TEMPLE_KEY_RING,
    RG_SPIRIT_TEMPLE_KEY_RING,
    RG_SHADOW_TEMPLE_KEY_RING,
    RG_BOTTOM_OF_THE_WELL_KEY_RING,
    RG_GERUDO_TRAINING_GROUND_KEY_RING,
    RG_GERUDO_FORTRESS_KEY_RING,
    RG_GANONS_CASTLE_KEY_RING,
    RG_TREASURE_GAME_KEY_RING,
    RG_KOKIRI_EMERALD,
    RG_GORON_RUBY,
    RG_ZORA_SAPPHIRE,
    RG_FOREST_MEDALLION,
    RG_FIRE_MEDALLION,
    RG_WATER_MEDALLION,
    RG_SPIRIT_MEDALLION,
    RG_SHADOW_MEDALLION,
    RG_LIGHT_MEDALLION,
    RG_RECOVERY_HEART,
    RG_GREEN_RUPEE,
    RG_GREG_RUPEE,
    RG_BLUE_RUPEE,
    RG_RED_RUPEE,
    RG_PURPLE_RUPEE,
    RG_HUGE_RUPEE,
    RG_PIECE_OF_HEART,
    RG_HEART_CONTAINER,
    RG_ICE_TRAP,
    RG_MILK,
    RG_FISH,
    RG_BOMBS_5,
    RG_BOMBS_10,
    RG_BOMBS_20,
    RG_BOMBCHU_5,
    RG_BOMBCHU_10,
    RG_BOMBCHU_20,
    RG_ARROWS_5,
    RG_ARROWS_10,
    RG_ARROWS_30,
    RG_DEKU_NUTS_5,
    RG_DEKU_NUTS_10,
    RG_DEKU_SEEDS_30,
    RG_DEKU_STICK_1,
    RG_RED_POTION_REFILL,
    RG_GREEN_POTION_REFILL,
    RG_BLUE_POTION_REFILL,
    RG_TREASURE_GAME_HEART,
    RG_TREASURE_GAME_GREEN_RUPEE,
    RG_BUY_DEKU_NUTS_5,
    RG_BUY_ARROWS_30,
    RG_BUY_ARROWS_50,
    RG_BUY_BOMBS_525,
    RG_BUY_DEKU_NUTS_10,
    RG_BUY_DEKU_STICK_1,
    RG_BUY_BOMBS_10,
    RG_BUY_FISH,
    RG_BUY_RED_POTION_30,
    RG_BUY_GREEN_POTION,
    RG_BUY_BLUE_POTION,
    RG_BUY_HYLIAN_SHIELD,
    RG_BUY_DEKU_SHIELD,
    RG_BUY_GORON_TUNIC,
    RG_BUY_ZORA_TUNIC,
    RG_BUY_HEART,
    RG_BUY_BOMBCHUS_10,
    RG_BUY_BOMBCHUS_20,
    RG_BUY_DEKU_SEEDS_30,
    RG_SOLD_OUT,
    RG_BUY_BLUE_FIRE,
    RG_BUY_BOTTLE_BUG,
    RG_BUY_POE,
    RG_BUY_FAIRYS_SPIRIT,
    RG_BUY_ARROWS_10,
    RG_BUY_BOMBS_20,
    RG_BUY_BOMBS_30,
    RG_BUY_BOMBS_535,
    RG_BUY_RED_POTION_40,
    RG_BUY_RED_POTION_50,
    RG_TRIFORCE,
    RG_TRIFORCE_PIECE,
    RG_GOHMA_SOUL,
    RG_KING_DODONGO_SOUL,
    RG_BARINADE_SOUL,
    RG_PHANTOM_GANON_SOUL,
    RG_VOLVAGIA_SOUL,
    RG_MORPHA_SOUL,
    RG_BONGO_BONGO_SOUL,
    RG_TWINROVA_SOUL,
    RG_GANON_SOUL,
    RG_OCARINA_A_BUTTON,
    RG_OCARINA_C_UP_BUTTON,
    RG_OCARINA_C_DOWN_BUTTON,
    RG_OCARINA_C_LEFT_BUTTON,
    RG_OCARINA_C_RIGHT_BUTTON,
    RG_SKELETON_KEY,
    RG_FISHING_POLE,
    RG_DEKU_STICK_BAG,
    RG_DEKU_NUT_BAG,
    RG_HINT,
    RG_TYCOON_WALLET,
    RG_BRONZE_SCALE,
    RG_CHILD_WALLET,
    RG_BOMBCHU_BAG,
    RG_QUIVER_INF,
    RG_BOMB_BAG_INF,
    RG_BULLET_BAG_INF,
    RG_STICK_UPGRADE_INF,
    RG_NUT_UPGRADE_INF,
    RG_MAGIC_INF,
    RG_BOMBCHU_INF,
    RG_WALLET_INF,
    RG_FAIRY_OCARINA,
    RG_OCARINA_OF_TIME,
    RG_BOMB_BAG,
    RG_BIG_BOMB_BAG,
    RG_BIGGEST_BOMB_BAG,
    RG_FAIRY_BOW,
    RG_BIG_QUIVER,
    RG_BIGGEST_QUIVER,
    RG_FAIRY_SLINGSHOT,
    RG_BIG_BULLET_BAG,
    RG_BIGGEST_BULLET_BAG,
    RG_GORONS_BRACELET,
    RG_SILVER_GAUNTLETS,
    RG_GOLDEN_GAUNTLETS,
    RG_SILVER_SCALE,
    RG_GOLDEN_SCALE,
    RG_ADULT_WALLET,
    RG_GIANT_WALLET,
    RG_DEKU_NUT_CAPACITY_30,
    RG_DEKU_NUT_CAPACITY_40,
    RG_DEKU_STICK_CAPACITY_20,
    RG_DEKU_STICK_CAPACITY_30,
    RG_HOOKSHOT,
    RG_LONGSHOT,
    RG_SCARECROW,

    // Overworld keys
    RG_GUARD_HOUSE_KEY,
    RG_MARKET_BAZAAR_KEY,
    RG_MARKET_POTION_SHOP_KEY,
    RG_MASK_SHOP_KEY,
    RG_MARKET_SHOOTING_GALLERY_KEY,
    RG_BOMBCHU_BOWLING_KEY,
    RG_TREASURE_CHEST_GAME_BUILDING_KEY,
    RG_BOMBCHU_SHOP_KEY,
    RG_RICHARDS_HOUSE_KEY,
    RG_ALLEY_HOUSE_KEY,
    RG_KAK_BAZAAR_KEY,
    RG_KAK_POTION_SHOP_KEY,
    RG_BOSS_HOUSE_KEY,
    RG_GRANNYS_POTION_SHOP_KEY,
    RG_SKULLTULA_HOUSE_KEY,
    RG_IMPAS_HOUSE_KEY,
    RG_WINDMILL_KEY,
    RG_KAK_SHOOTING_GALLERY_KEY,
    RG_DAMPES_HUT_KEY,
    RG_TALONS_HOUSE_KEY,
    RG_STABLES_KEY,
    RG_BACK_TOWER_KEY,
    RG_HYLIA_LAB_KEY,
    RG_FISHING_HOLE_KEY,
    // Logic Only
    RG_DISTANT_SCARECROW,
    RG_STICKS,
    RG_NUTS,
    RG_EPONA,
    RG_MAX
} RandomizerGet;

typedef enum {
    RH_NONE,
    RH_KF_DEKU_TREE_LEFT_GOSSIP_STONE,
    RH_KF_DEKU_TREE_RIGHT_GOSSIP_STONE,
    RH_KF_GOSSIP_STONE,
    RH_KF_STORMS_GROTTO_GOSSIP_STONE,
    RH_LW_GOSSIP_STONE,
    RH_LW_NEAR_SHORTCUTS_GROTTO_GOSSIP_STONE,
    RH_SFM_MAZE_NEAR_LW_GOSSIP_STONE,
    RH_SFM_MAZE_CENTER_GOSSIP_STONE,
    RH_SFM_SARIA_GOSSIP_STONE,
    RH_HF_COW_GROTTO_GOSSIP_STONE,
    RH_HF_NEAR_MARKET_GROTTO_GOSSIP_STONE,
    RH_HF_OPEN_GROTTO_GOSSIP_STONE,
    RH_HF_SOUTHEAST_GROTTO_GOSSIP_STONE,
    RH_TOT_LEFT_CENTER_GOSSIP_STONE,
    RH_TOT_LEFTMOST_GOSSIP_STONE,
    RH_TOT_RIGHT_CENTER_GOSSIP_STONE,
    RH_TOT_RIGHTMOST_GOSSIP_STONE,
    RH_HC_MALON_GOSSIP_STONE,
    RH_HC_ROCK_WALL_GOSSIP_STONE,
    RH_HC_STORMS_GROTTO_GOSSIP_STONE,
    RH_KAK_OPEN_GROTTO_GOSSIP_STONE,
    RH_GRAVEYARD_GOSSIP_STONE,
    RH_DMT_GOSSIP_STONE,
    RH_DMT_STORMS_GROTTO_GOSSIP_STONE,
    RH_GC_MAZE_GOSSIP_STONE,
    RH_GC_MEDIGORON_GOSSIP_STONE,
    RH_DMC_GOSSIP_STONE,
    RH_DMC_UPPER_GROTTO_GOSSIP_STONE,
    RH_ZR_NEAR_DOMAIN_GOSSIP_STONE,
    RH_ZR_NEAR_GROTTOS_GOSSIP_STONE,
    RH_ZR_OPEN_GROTTO_GOSSIP_STONE,
    RH_ZD_GOSSIP_STONE,
    RH_ZF_JABU_GOSSIP_STONE,
    RH_ZF_FAIRY_GOSSIP_STONE,
    RH_LH_LAB_GOSSIP_STONE,
    RH_LH_SOUTHEAST_GOSSIP_STONE,
    RH_LH_SOUTHWEST_GOSSIP_STONE,
    RH_GV_GOSSIP_STONE,
    RH_COLOSSUS_GOSSIP_STONE,
    RH_DODONGOS_CAVERN_GOSSIP_STONE,
    RH_GANONDORF_HINT,
    RH_GANONDORF_JOKE,
    RH_SHEIK_HINT,
    RH_DAMPES_DIARY,
    RH_GREG_RUPEE,
    RH_ALTAR_CHILD,
    RH_ALTAR_ADULT,
    RH_SARIA_HINT,
    RH_LOACH_HINT,
    RH_FISHING_POLE,
    RH_MINUET_WARP_LOC,
    RH_BOLERO_WARP_LOC,
    RH_SERENADE_WARP_LOC,
    RH_REQUIEM_WARP_LOC,
    RH_NOCTURNE_WARP_LOC,
    RH_PRELUDE_WARP_LOC,
    RH_HBA_HINT,
    RH_MALON_HINT,
    RH_CHICKENS_HINT,
    RH_BIG_POES_HINT,
    RH_BIGGORON_HINT,
    RH_FROGS_HINT,
    RH_OOT_HINT,
    RH_KAK_10_SKULLS_HINT,
    RH_KAK_20_SKULLS_HINT,
    RH_KAK_30_SKULLS_HINT,
    RH_KAK_40_SKULLS_HINT,
    RH_KAK_50_SKULLS_HINT,
    RH_KAK_100_SKULLS_HINT,
    RH_MASK_SHOP_HINT,
    RH_MAX,
} RandomizerHint;

typedef enum {
    RHT_NONE,
    RHT_WAY_OF_THE_HERO,
    RHT_FOOLISH,
    RHT_CAN_BE_FOUND_AT,
    RHT_HOARDS,
    // Junk
    RHT_JUNK01,
    RHT_JUNK02,
    RHT_JUNK03,
    RHT_JUNK04,
    RHT_JUNK05,
    RHT_JUNK06,
    RHT_JUNK07,
    RHT_JUNK08,
    RHT_JUNK09,
    RHT_JUNK10,
    RHT_JUNK11,
    RHT_JUNK12,
    RHT_JUNK13,
    RHT_JUNK14,
    RHT_JUNK15,
    RHT_JUNK16,
    RHT_JUNK17,
    RHT_JUNK18,
    RHT_JUNK19,
    RHT_JUNK20,
    RHT_JUNK21,
    RHT_JUNK22,
    RHT_JUNK23,
    RHT_JUNK24,
    RHT_JUNK25,
    RHT_JUNK26,
    RHT_JUNK27,
    RHT_JUNK28,
    RHT_JUNK29,
    RHT_JUNK30,
    RHT_JUNK31,
    RHT_JUNK32,
    RHT_JUNK33,
    RHT_JUNK34,
    RHT_JUNK35,
    RHT_JUNK36,
    RHT_JUNK37,
    RHT_JUNK38,
    RHT_JUNK39,
    RHT_JUNK40,
    RHT_JUNK41,
    RHT_JUNK42,
    RHT_JUNK43,
    RHT_JUNK44,
    RHT_JUNK45,
    RHT_JUNK46,
    RHT_JUNK47,
    RHT_JUNK48,
    RHT_JUNK49,
    RHT_JUNK50,
    RHT_JUNK51,
    RHT_JUNK52,
    RHT_JUNK53,
    RHT_JUNK54,
    RHT_JUNK55,
    RHT_JUNK56,
    RHT_JUNK57,
    RHT_JUNK58,
    RHT_JUNK59,
    RHT_JUNK60,
    RHT_JUNK61,
    RHT_JUNK62,
    RHT_JUNK63,
    RHT_JUNK64,
    RHT_JUNK65,
    RHT_JUNK66,
    RHT_JUNK67,
    RHT_JUNK68,
    RHT_JUNK69,
    RHT_JUNK70,
    RHT_JUNK71,
    // Locations
    RHT_LINKS_POCKET,
    RHT_QUEEN_GOHMA,
    RHT_KING_DODONGO,
    RHT_BARINADE,
    RHT_PHANTOM_GANON,
    RHT_VOLVAGIA,
    RHT_MORPHA,
    RHT_BONGO_BONGO,
    RHT_TWINROVA,
    RHT_GIFT_FROM_RAURU,
    RHT_SONG_FROM_IMPA,
    RHT_SONG_FROM_MALON,
    RHT_SONG_FROM_SARIA,
    RHT_SONG_FROM_ROYAL_FAMILYS_TOMB,
    RHT_SONG_FROM_OCARINA_OF_TIME,
    RHT_SONG_FROM_WINDMILL,
    RHT_SHEIK_IN_FOREST,
    RHT_SHEIK_IN_CRATER,
    RHT_SHEIK_IN_ICE_CAVERN,
    RHT_SHEIK_AT_COLOSSUS,
    RHT_SHEIK_IN_KAKARIKO,
    RHT_SHEIK_AT_TEMPLE,
    RHT_KF_MIDOS_TOP_LEFT_CHEST,
    RHT_KF_MIDOS_TOP_RIGHT_CHEST,
    RHT_KF_MIDOS_BOTTOM_LEFT_CHEST,
    RHT_KF_MIDOS_BOTTOM_RIGHT_CHEST,
    RHT_KF_KOKIRI_SWORD_CHEST,
    RHT_KF_STORMS_GROTTO_CHEST,
    RHT_KF_STORMS_GROTTO_FISH,
    RHT_KF_LINKS_HOUSE_COW,
    RHT_KF_GS_KNOW_IT_ALL_HOUSE,
    RHT_KF_GS_BEAN_PATCH,
    RHT_KF_GS_HOUSE_OF_TWINS,
    RHT_KF_SHOP_ITEM_1,
    RHT_KF_SHOP_ITEM_2,
    RHT_KF_SHOP_ITEM_3,
    RHT_KF_SHOP_ITEM_4,
    RHT_KF_SHOP_ITEM_5,
    RHT_KF_SHOP_ITEM_6,
    RHT_KF_SHOP_ITEM_7,
    RHT_KF_SHOP_ITEM_8,
    RHT_LW_GIFT_FROM_SARIA,
    RHT_LW_OCARINA_MEMORY_GAME,
    RHT_LW_TARGET_IN_WOODS,
    RHT_LW_NEAR_SHORTCUTS_GROTTO_CHEST,
    RHT_LW_NEAR_SHORTCUTS_GROTTO_FISH,
    RHT_LW_TRADE_COJIRO,
    RHT_DEKU_THEATER_SKULL_MASK,
    RHT_DEKU_THEATER_MASK_OF_TRUTH,
    RHT_LW_SKULL_KID,
    RHT_LW_TRADE_ODD_POTION,
    RHT_LW_DEKU_SCRUB_NEAR_BRIDGE,
    RHT_LW_DEKU_SCRUB_NEAR_DEKU_THEATER_LEFT,
    RHT_LW_DEKU_SCRUB_NEAR_DEKU_THEATER_RIGHT,
    RHT_LW_DEKU_SCRUB_GROTTO_FRONT,
    RHT_LW_DEKU_SCRUB_GROTTO_REAR,
    RHT_LW_GS_BEAN_PATCH_NEAR_BRIDGE,
    RHT_LW_GS_BEAN_PATCH_NEAR_THEATER,
    RHT_LW_GS_ABOVE_THEATER,
    RHT_SFM_WOLFOS_GROTTO_CHEST,
    RHT_SFM_DEKU_SCRUB_GROTTO_FRONT,
    RHT_SFM_DEKU_SCRUB_GROTTO_REAR,
    RHT_SFM_GS,
    RHT_HF_OCARINA_OF_TIME_ITEM,
    RHT_HF_NEAR_MARKET_GROTTO_CHEST,
    RHT_HF_NEAR_MARKET_GROTTO_FISH,
    RHT_HF_TEKTITE_GROTTO_FREESTANDING_POH,
    RHT_HF_SOUTHEAST_GROTTO_CHEST,
    RHT_HF_SOUTHEAST_GROTTO_FISH,
    RHT_HF_OPEN_GROTTO_CHEST,
    RHT_HF_OPEN_GROTTO_FISH,
    RHT_HF_DEKU_SCRUB_GROTTO,
    RHT_HF_COW_GROTTO_COW,
    RHT_HF_GS_COW_GROTTO,
    RHT_HF_GS_NEAR_KAK_GROTTO,
    RHT_MARKET_SHOOTING_GALLERY_REWARD,
    RHT_MARKET_BOMBCHU_BOWLING_FIRST_PRIZE,
    RHT_MARKET_BOMBCHU_BOWLING_SECOND_PRIZE,
    RHT_MARKET_BOMBCHU_BOWLING_BOMBCHUS,
    RHT_MARKET_LOST_DOG,
    RHT_MARKET_TREASURE_CHEST_GAME_REWARD,
    RHT_MARKET_10_BIG_POES,
    RHT_MARKET_TREASURE_CHEST_GAME_ITEM_1,
    RHT_MARKET_TREASURE_CHEST_GAME_ITEM_2,
    RHT_MARKET_TREASURE_CHEST_GAME_ITEM_3,
    RHT_MARKET_TREASURE_CHEST_GAME_ITEM_4,
    RHT_MARKET_TREASURE_CHEST_GAME_ITEM_5,
    RHT_MARKET_TREASURE_CHEST_GAME_KEY_1,
    RHT_MARKET_TREASURE_CHEST_GAME_KEY_2,
    RHT_MARKET_TREASURE_CHEST_GAME_KEY_3,
    RHT_MARKET_TREASURE_CHEST_GAME_KEY_4,
    RHT_MARKET_TREASURE_CHEST_GAME_KEY_5,
    RHT_MARKET_GS_GUARD_HOUSE,
    RHT_MARKET_BAZAAR_ITEM_1,
    RHT_MARKET_BAZAAR_ITEM_2,
    RHT_MARKET_BAZAAR_ITEM_3,
    RHT_MARKET_BAZAAR_ITEM_4,
    RHT_MARKET_BAZAAR_ITEM_5,
    RHT_MARKET_BAZAAR_ITEM_6,
    RHT_MARKET_BAZAAR_ITEM_7,
    RHT_MARKET_BAZAAR_ITEM_8,
    RHT_MARKET_POTION_SHOP_ITEM_1,
    RHT_MARKET_POTION_SHOP_ITEM_2,
    RHT_MARKET_POTION_SHOP_ITEM_3,
    RHT_MARKET_POTION_SHOP_ITEM_4,
    RHT_MARKET_POTION_SHOP_ITEM_5,
    RHT_MARKET_POTION_SHOP_ITEM_6,
    RHT_MARKET_POTION_SHOP_ITEM_7,
    RHT_MARKET_POTION_SHOP_ITEM_8,
    RHT_MARKET_BOMBCHU_SHOP_ITEM_1,
    RHT_MARKET_BOMBCHU_SHOP_ITEM_2,
    RHT_MARKET_BOMBCHU_SHOP_ITEM_3,
    RHT_MARKET_BOMBCHU_SHOP_ITEM_4,
    RHT_MARKET_BOMBCHU_SHOP_ITEM_5,
    RHT_MARKET_BOMBCHU_SHOP_ITEM_6,
    RHT_MARKET_BOMBCHU_SHOP_ITEM_7,
    RHT_MARKET_BOMBCHU_SHOP_ITEM_8,
    RHT_TOT_MASTER_SWORD,
    RHT_TOT_LIGHT_ARROWS_CUTSCENE,
    RHT_HC_MALON_EGG,
    RHT_HC_ZELDAS_LETTER,
    RHT_HC_GREAT_FAIRY_REWARD,
    RHT_HC_GS_TREE,
    RHT_HC_GS_STORMS_GROTTO,
    RHT_LLR_TALONS_CHICKENS,
    RHT_LLR_FREESTANDING_POH,
    RHT_LLR_DEKU_SCRUB_GROTTO_LEFT,
    RHT_LLR_DEKU_SCRUB_GROTTO_CENTER,
    RHT_LLR_DEKU_SCRUB_GROTTO_RIGHT,
    RHT_LLR_STABLES_LEFT_COW,
    RHT_LLR_STABLES_RIGHT_COW,
    RHT_LLR_TOWER_LEFT_COW,
    RHT_LLR_TOWER_RIGHT_COW,
    RHT_LLR_GS_HOUSE_WINDOW,
    RHT_LLR_GS_TREE,
    RHT_LLR_GS_RAIN_SHED,
    RHT_LLR_GS_BACK_WALL,
    RHT_KAK_ANJU_AS_CHILD,
    RHT_KAK_ANJU_AS_ADULT,
    RHT_KAK_IMPAS_HOUSE_FREESTANDING_POH,
    RHT_KAK_WINDMILL_FREESTANDING_POH,
    RHT_KAK_MAN_ON_ROOF,
    RHT_KAK_OPEN_GROTTO_CHEST,
    RHT_KAK_OPEN_GROTTO_FISH,
    RHT_KAK_REDEAD_GROTTO_CHEST,
    RHT_KAK_SHOOTING_GALLERY_REWARD,
    RHT_KAK_TRADE_ODD_MUSHROOM,
    RHT_KAK_GRANNYS_SHOP,
    RHT_KAK_TRADE_POCKET_CUCCO,
    RHT_KAK_10_GOLD_SKULLTULA_REWARD,
    RHT_KAK_20_GOLD_SKULLTULA_REWARD,
    RHT_KAK_30_GOLD_SKULLTULA_REWARD,
    RHT_KAK_40_GOLD_SKULLTULA_REWARD,
    RHT_KAK_50_GOLD_SKULLTULA_REWARD,
    RHT_KAK_100_GOLD_SKULLTULA_REWARD,
    RHT_KAK_IMPAS_HOUSE_COW,
    RHT_KAK_GS_TREE,
    RHT_KAK_GS_GUARDS_HOUSE,
    RHT_KAK_GS_WATCHTOWER,
    RHT_KAK_GS_SKULLTULA_HOUSE,
    RHT_KAK_GS_HOUSE_UNDER_CONSTRUCTION,
    RHT_KAK_GS_ABOVE_IMPAS_HOUSE,
    RHT_KAK_BAZAAR_ITEM_1,
    RHT_KAK_BAZAAR_ITEM_2,
    RHT_KAK_BAZAAR_ITEM_3,
    RHT_KAK_BAZAAR_ITEM_4,
    RHT_KAK_BAZAAR_ITEM_5,
    RHT_KAK_BAZAAR_ITEM_6,
    RHT_KAK_BAZAAR_ITEM_7,
    RHT_KAK_BAZAAR_ITEM_8,
    RHT_KAK_POTION_SHOP_ITEM_1,
    RHT_KAK_POTION_SHOP_ITEM_2,
    RHT_KAK_POTION_SHOP_ITEM_3,
    RHT_KAK_POTION_SHOP_ITEM_4,
    RHT_KAK_POTION_SHOP_ITEM_5,
    RHT_KAK_POTION_SHOP_ITEM_6,
    RHT_KAK_POTION_SHOP_ITEM_7,
    RHT_KAK_POTION_SHOP_ITEM_8,
    RHT_GRAVEYARD_SHIELD_GRAVE_CHEST,
    RHT_GRAVEYARD_HEART_PIECE_GRAVE_CHEST,
    RHT_GRAVEYARD_ROYAL_FAMILYS_TOMB_CHEST,
    RHT_GRAVEYARD_FREESTANDING_POH,
    RHT_GRAVEYARD_DAMPE_GRAVEDIGGING_TOUR,
    RHT_GRAVEYARD_HOOKSHOT_CHEST,
    RHT_GRAVEYARD_DAMPE_RACE_FREESTANDING_POH,
    RHT_GRAVEYARD_GS_BEAN_PATCH,
    RHT_GRAVEYARD_GS_WALL,
    RHT_DMT_FREESTANDING_POH,
    RHT_DMT_CHEST,
    RHT_DMT_STORMS_GROTTO_CHEST,
    RHT_DMT_STORMS_GROTTO_FISH,
    RHT_DMT_TRADE_BROKEN_SWORD,
    RHT_DMT_TRADE_EYEDROPS,
    RHT_DMT_TRADE_CLAIM_CHECK,
    RHT_DMT_GREAT_FAIRY_REWARD,
    RHT_DMT_COW_GROTTO_COW,
    RHT_DMT_GS_NEAR_KAK,
    RHT_DMT_GS_BEAN_PATCH,
    RHT_DMT_GS_ABOVE_DODONGOS_CAVERN,
    RHT_DMT_GS_FALLING_ROCKS_PATH,
    RHT_GC_DARUNIAS_JOY,
    RHT_GC_POT_FREESTANDING_POH,
    RHT_GC_ROLLING_GORON_AS_CHILD,
    RHT_GC_ROLLING_GORON_AS_ADULT,
    RHT_GC_MEDIGORON,
    RHT_GC_MAZE_LEFT_CHEST,
    RHT_GC_MAZE_RIGHT_CHEST,
    RHT_GC_MAZE_CENTER_CHEST,
    RHT_GC_DEKU_SCRUB_GROTTO_LEFT,
    RHT_GC_DEKU_SCRUB_GROTTO_CENTER,
    RHT_GC_DEKU_SCRUB_GROTTO_RIGHT,
    RHT_GC_GS_CENTER_PLATFORM,
    RHT_GC_GS_BOULDER_MAZE,
    RHT_GC_SHOP_ITEM_1,
    RHT_GC_SHOP_ITEM_2,
    RHT_GC_SHOP_ITEM_3,
    RHT_GC_SHOP_ITEM_4,
    RHT_GC_SHOP_ITEM_5,
    RHT_GC_SHOP_ITEM_6,
    RHT_GC_SHOP_ITEM_7,
    RHT_GC_SHOP_ITEM_8,
    RHT_DMC_VOLCANO_FREESTANDING_POH,
    RHT_DMC_WALL_FREESTANDING_POH,
    RHT_DMC_UPPER_GROTTO_CHEST,
    RHT_DMC_UPPER_GROTTO_FISH,
    RHT_DMC_GREAT_FAIRY_REWARD,
    RHT_DMC_DEKU_SCRUB,
    RHT_DMC_DEKU_SCRUB_GROTTO_LEFT,
    RHT_DMC_DEKU_SCRUB_GROTTO_CENTER,
    RHT_DMC_DEKU_SCRUB_GROTTO_RIGHT,
    RHT_DMC_GS_CRATE,
    RHT_DMC_GS_BEAN_PATCH,
    RHT_ZR_MAGIC_BEAN_SALESMAN,
    RHT_ZR_OPEN_GROTTO_CHEST,
    RHT_ZR_OPEN_GROTTO_FISH,
    RHT_ZR_FROGS_ZELDAS_LULLABY,
    RHT_ZR_FROGS_EPONAS_SONG,
    RHT_ZR_FROGS_SARIAS_SONG,
    RHT_ZR_FROGS_SUNS_SONG,
    RHT_ZR_FROGS_SONG_OF_TIME,
    RHT_ZR_FROGS_IN_THE_RAIN,
    RHT_ZR_FROGS_OCARINA_GAME,
    RHT_ZR_NEAR_OPEN_GROTTO_FREESTANDING_POH,
    RHT_ZR_NEAR_DOMAIN_FREESTANDING_POH,
    RHT_ZR_DEKU_SCRUB_GROTTO_FRONT,
    RHT_ZR_DEKU_SCRUB_GROTTO_REAR,
    RHT_ZR_GS_TREE,
    RHT_ZR_GS_LADDER,
    RHT_ZR_GS_NEAR_RAISED_GROTTOS,
    RHT_ZR_GS_ABOVE_BRIDGE,
    RHT_ZD_DIVING_MINIGAME,
    RHT_ZD_CHEST,
    RHT_ZD_KING_ZORA_THAWED,
    RHT_ZD_TRADE_PRESCRIPTION,
    RHT_ZD_GS_FROZEN_WATERFALL,
    RHT_ZD_SHOP_ITEM_1,
    RHT_ZD_SHOP_ITEM_2,
    RHT_ZD_SHOP_ITEM_3,
    RHT_ZD_SHOP_ITEM_4,
    RHT_ZD_SHOP_ITEM_5,
    RHT_ZD_SHOP_ITEM_6,
    RHT_ZD_SHOP_ITEM_7,
    RHT_ZD_SHOP_ITEM_8,
    RHT_ZD_FISH,
    RHT_ZF_GREAT_FAIRY_REWARD,
    RHT_ZF_ICEBERG_FREESTANDING_POH,
    RHT_ZF_BOTTOM_FREESTANDING_POH,
    RHT_ZF_GS_ABOVE_THE_LOG,
    RHT_ZF_GS_TREE,
    RHT_ZF_GS_HIDDEN_CAVE,
    RHT_LH_UNDERWATER_ITEM,
    RHT_LH_CHILD_FISHING,
    RHT_LH_ADULT_FISHING,
    RHT_LH_POND_FISH,
    RHT_LH_HYRULE_LOACH,
    RHT_LH_LAB_DIVE,
    RHT_LH_TRADE_FROG,
    RHT_LH_FREESTANDING_POH,
    RHT_LH_SUN,
    RHT_LH_DEKU_SCRUB_GROTTO_LEFT,
    RHT_LH_DEKU_SCRUB_GROTTO_CENTER,
    RHT_LH_DEKU_SCRUB_GROTTO_RIGHT,
    RHT_LH_GS_BEAN_PATCH,
    RHT_LH_GS_LAB_WALL,
    RHT_LH_GS_SMALL_ISLAND,
    RHT_LH_GS_LAB_CRATE,
    RHT_LH_GS_TREE,
    RHT_GV_CRATE_FREESTANDING_POH,
    RHT_GV_WATERFALL_FREESTANDING_POH,
    RHT_GV_CHEST,
    RHT_GV_TRADE_SAW,
    RHT_GV_DEKU_SCRUB_GROTTO_FRONT,
    RHT_GV_DEKU_SCRUB_GROTTO_REAR,
    RHT_GV_COW,
    RHT_GV_GS_SMALL_BRIDGE,
    RHT_GV_GS_BEAN_PATCH,
    RHT_GV_GS_BEHIND_TENT,
    RHT_GV_GS_PILLAR,
    RHT_GF_CHEST,
    RHT_GF_HBA_1000_POINTS,
    RHT_GF_HBA_1500_POINTS,
    RHT_GF_GERUDO_MEMBERSHIP_CARD,
    RHT_GF_NORTH_F1_CARPENTER,
    RHT_GF_NORTH_F2_CARPENTER,
    RHT_GF_SOUTH_F1_CARPENTER,
    RHT_GF_SOUTH_F2_CARPENTER,
    RHT_GF_GS_TOP_FLOOR,
    RHT_GF_GS_ARCHERY_RANGE,
    RHT_HIDEOUT_JAIL_GUARD_1_TORCH,
    RHT_HIDEOUT_JAIL_GUARD_2_TORCHES,
    RHT_HIDEOUT_JAIL_GUARD_3_TORCHES,
    RHT_HIDEOUT_JAIL_GUARD_4_TORCHES,
    RHT_HIDEOUT_GERUDO_MEMBERSHIP_CARD,
    RHT_WASTELAND_BOMBCHU_SALESMAN,
    RHT_WASTELAND_CHEST,
    RHT_WASTELAND_GS,
    RHT_COLOSSUS_GREAT_FAIRY_REWARD,
    RHT_COLOSSUS_FREESTANDING_POH,
    RHT_COLOSSUS_DEKU_SCRUB_GROTTO_FRONT,
    RHT_COLOSSUS_DEKU_SCRUB_GROTTO_REAR,
    RHT_COLOSSUS_GS_BEAN_PATCH,
    RHT_COLOSSUS_GS_TREE,
    RHT_COLOSSUS_GS_HILL,
    RHT_OGC_GREAT_FAIRY_REWARD,
    RHT_OGC_GS,
    RHT_DEKU_TREE_MAP_CHEST,
    RHT_DEKU_TREE_SLINGSHOT_ROOM_SIDE_CHEST,
    RHT_DEKU_TREE_SLINGSHOT_CHEST,
    RHT_DEKU_TREE_COMPASS_CHEST,
    RHT_DEKU_TREE_COMPASS_ROOM_SIDE_CHEST,
    RHT_DEKU_TREE_BASEMENT_CHEST,
    RHT_DEKU_TREE_GS_COMPASS_ROOM,
    RHT_DEKU_TREE_GS_BASEMENT_VINES,
    RHT_DEKU_TREE_GS_BASEMENT_GATE,
    RHT_DEKU_TREE_GS_BASEMENT_BACK_ROOM,
    RHT_DEKU_TREE_MQ_MAP_CHEST,
    RHT_DEKU_TREE_MQ_SLINGSHOT_CHEST,
    RHT_DEKU_TREE_MQ_SLINGSHOT_ROOM_BACK_CHEST,
    RHT_DEKU_TREE_MQ_COMPASS_CHEST,
    RHT_DEKU_TREE_MQ_BASEMENT_CHEST,
    RHT_DEKU_TREE_MQ_BEFORE_SPINNING_LOG_CHEST,
    RHT_DEKU_TREE_MQ_AFTER_SPINNING_LOG_CHEST,
    RHT_DEKU_TREE_MQ_DEKU_SCRUB,
    RHT_DEKU_TREE_MQ_GS_LOBBY,
    RHT_DEKU_TREE_MQ_GS_PAST_BOULDER_VINES,
    RHT_DEKU_TREE_MQ_GS_BASEMENT_GRAVES_ROOM,
    RHT_DEKU_TREE_MQ_GS_BASEMENT_BACK_ROOM,
    RHT_DEKU_TREE_QUEEN_GOHMA_HEART,
    RHT_DODONGOS_CAVERN_MAP_CHEST,
    RHT_DODONGOS_CAVERN_COMPASS_CHEST,
    RHT_DODONGOS_CAVERN_BOMB_FLOWER_PLATFORM_CHEST,
    RHT_DODONGOS_CAVERN_BOMB_BAG_CHEST,
    RHT_DODONGOS_CAVERN_END_OF_BRIDGE_CHEST,
    RHT_DODONGOS_CAVERN_DEKU_SCRUB_SIDE_ROOM_NEAR_DODONGOS,
    RHT_DODONGOS_CAVERN_DEKU_SCRUB_LOBBY,
    RHT_DODONGOS_CAVERN_DEKU_SCRUB_NEAR_BOMB_BAG_LEFT,
    RHT_DODONGOS_CAVERN_DEKU_SCRUB_NEAR_BOMB_BAG_RIGHT,
    RHT_DODONGOS_CAVERN_GS_SIDE_ROOM_NEAR_LOWER_LIZALFOS,
    RHT_DODONGOS_CAVERN_GS_SCARECROW,
    RHT_DODONGOS_CAVERN_GS_ALCOVE_ABOVE_STAIRS,
    RHT_DODONGOS_CAVERN_GS_VINES_ABOVE_STAIRS,
    RHT_DODONGOS_CAVERN_GS_BACK_ROOM,
    RHT_DODONGOS_CAVERN_MQ_MAP_CHEST,
    RHT_DODONGOS_CAVERN_MQ_BOMB_BAG_CHEST,
    RHT_DODONGOS_CAVERN_MQ_TORCH_PUZZLE_ROOM_CHEST,
    RHT_DODONGOS_CAVERN_MQ_LARVAE_ROOM_CHEST,
    RHT_DODONGOS_CAVERN_MQ_COMPASS_CHEST,
    RHT_DODONGOS_CAVERN_MQ_UNDER_GRAVE_CHEST,
    RHT_DODONGOS_CAVERN_MQ_DEKU_SCRUB_LOBBY_FRONT,
    RHT_DODONGOS_CAVERN_MQ_DEKU_SCRUB_LOBBY_REAR,
    RHT_DODONGOS_CAVERN_MQ_DEKU_SCRUB_SIDE_ROOM_NEAR_LOWER_LIZALFOS,
    RHT_DODONGOS_CAVERN_MQ_DEKU_SCRUB_STAIRCASE,
    RHT_DODONGOS_CAVERN_MQ_GS_SCRUB_ROOM,
    RHT_DODONGOS_CAVERN_MQ_GS_LARVAE_ROOM,
    RHT_DODONGOS_CAVERN_MQ_GS_LIZALFOS_ROOM,
    RHT_DODONGOS_CAVERN_MQ_GS_SONG_OF_TIME_BLOCK_ROOM,
    RHT_DODONGOS_CAVERN_MQ_GS_BACK_AREA,
    RHT_DODONGOS_CAVERN_BOSS_ROOM_CHEST,
    RHT_DODONGOS_CAVERN_KING_DODONGO_HEART,
    RHT_JABU_JABUS_BELLY_BOOMERANG_CHEST,
    RHT_JABU_JABUS_BELLY_MAP_CHEST,
    RHT_JABU_JABUS_BELLY_COMPASS_CHEST,
    RHT_JABU_JABUS_BELLY_DEKU_SCRUB,
    RHT_JABU_JABUS_BELLY_GS_WATER_SWITCH_ROOM,
    RHT_JABU_JABUS_BELLY_GS_LOBBY_BASEMENT_LOWER,
    RHT_JABU_JABUS_BELLY_GS_LOBBY_BASEMENT_UPPER,
    RHT_JABU_JABUS_BELLY_GS_NEAR_BOSS,
    RHT_JABU_JABUS_BELLY_MQ_MAP_CHEST,
    RHT_JABU_JABUS_BELLY_MQ_FIRST_ROOM_SIDE_CHEST,
    RHT_JABU_JABUS_BELLY_MQ_SECOND_ROOM_LOWER_CHEST,
    RHT_JABU_JABUS_BELLY_MQ_COMPASS_CHEST,
    RHT_JABU_JABUS_BELLY_MQ_BASEMENT_NEAR_SWITCHES_CHEST,
    RHT_JABU_JABUS_BELLY_MQ_BASEMENT_NEAR_VINES_CHEST,
    RHT_JABU_JABUS_BELLY_MQ_BOOMERANG_ROOM_SMALL_CHEST,
    RHT_JABU_JABUS_BELLY_MQ_BOOMERANG_CHEST,
    RHT_JABU_JABUS_BELLY_MQ_FALLING_LIKE_LIKE_ROOM_CHEST,
    RHT_JABU_JABUS_BELLY_MQ_SECOND_ROOM_UPPER_CHEST,
    RHT_JABU_JABUS_BELLY_MQ_NEAR_BOSS_CHEST,
    RHT_JABU_JABUS_BELLY_MQ_COW,
    RHT_JABU_JABUS_BELLY_MQ_GS_BOOMERANG_CHEST_ROOM,
    RHT_JABU_JABUS_BELLY_MQ_GS_TAILPASARAN_ROOM,
    RHT_JABU_JABUS_BELLY_MQ_GS_INVISIBLE_ENEMIES_ROOM,
    RHT_JABU_JABUS_BELLY_MQ_GS_NEAR_BOSS,
    RHT_JABU_JABUS_BELLY_BARINADE_HEART,
    RHT_BOTTOM_OF_THE_WELL_FRONT_LEFT_FAKE_WALL_CHEST,
    RHT_BOTTOM_OF_THE_WELL_FRONT_CENTER_BOMBABLE_CHEST,
    RHT_BOTTOM_OF_THE_WELL_BACK_LEFT_BOMBABLE_CHEST,
    RHT_BOTTOM_OF_THE_WELL_UNDERWATER_LEFT_CHEST,
    RHT_BOTTOM_OF_THE_WELL_FREESTANDING_KEY,
    RHT_BOTTOM_OF_THE_WELL_COMPASS_CHEST,
    RHT_BOTTOM_OF_THE_WELL_CENTER_SKULLTULA_CHEST,
    RHT_BOTTOM_OF_THE_WELL_RIGHT_BOTTOM_FAKE_WALL_CHEST,
    RHT_BOTTOM_OF_THE_WELL_FIRE_KEESE_CHEST,
    RHT_BOTTOM_OF_THE_WELL_LIKE_LIKE_CHEST,
    RHT_BOTTOM_OF_THE_WELL_MAP_CHEST,
    RHT_BOTTOM_OF_THE_WELL_UNDERWATER_FRONT_CHEST,
    RHT_BOTTOM_OF_THE_WELL_INVISIBLE_CHEST,
    RHT_BOTTOM_OF_THE_WELL_LENS_OF_TRUTH_CHEST,
    RHT_BOTTOM_OF_THE_WELL_GS_WEST_INNER_ROOM,
    RHT_BOTTOM_OF_THE_WELL_GS_EAST_INNER_ROOM,
    RHT_BOTTOM_OF_THE_WELL_GS_LIKE_LIKE_CAGE,
    RHT_BOTTOM_OF_THE_WELL_MQ_MAP_CHEST,
    RHT_BOTTOM_OF_THE_WELL_MQ_EAST_INNER_ROOM_FREESTANDING_KEY,
    RHT_BOTTOM_OF_THE_WELL_MQ_COMPASS_CHEST,
    RHT_BOTTOM_OF_THE_WELL_MQ_DEAD_HAND_FREESTANDING_KEY,
    RHT_BOTTOM_OF_THE_WELL_MQ_LENS_OF_TRUTH_CHEST,
    RHT_BOTTOM_OF_THE_WELL_MQ_GS_COFFIN_ROOM,
    RHT_BOTTOM_OF_THE_WELL_MQ_GS_WEST_INNER_ROOM,
    RHT_BOTTOM_OF_THE_WELL_MQ_GS_BASEMENT,
    RHT_FOREST_TEMPLE_FIRST_ROOM_CHEST,
    RHT_FOREST_TEMPLE_FIRST_STALFOS_CHEST,
    RHT_FOREST_TEMPLE_RAISED_ISLAND_COURTYARD_CHEST,
    RHT_FOREST_TEMPLE_MAP_CHEST,
    RHT_FOREST_TEMPLE_WELL_CHEST,
    RHT_FOREST_TEMPLE_EYE_SWITCH_CHEST,
    RHT_FOREST_TEMPLE_BOSS_KEY_CHEST,
    RHT_FOREST_TEMPLE_FLOORMASTER_CHEST,
    RHT_FOREST_TEMPLE_RED_POE_CHEST,
    RHT_FOREST_TEMPLE_BOW_CHEST,
    RHT_FOREST_TEMPLE_BLUE_POE_CHEST,
    RHT_FOREST_TEMPLE_FALLING_CEILING_ROOM_CHEST,
    RHT_FOREST_TEMPLE_BASEMENT_CHEST,
    RHT_FOREST_TEMPLE_GS_FIRST_ROOM,
    RHT_FOREST_TEMPLE_GS_LOBBY,
    RHT_FOREST_TEMPLE_GS_RAISED_ISLAND_COURTYARD,
    RHT_FOREST_TEMPLE_GS_LEVEL_ISLAND_COURTYARD,
    RHT_FOREST_TEMPLE_GS_BASEMENT,
    RHT_FOREST_TEMPLE_MQ_FIRST_ROOM_CHEST,
    RHT_FOREST_TEMPLE_MQ_WOLFOS_CHEST,
    RHT_FOREST_TEMPLE_MQ_WELL_CHEST,
    RHT_FOREST_TEMPLE_MQ_RAISED_ISLAND_COURTYARD_LOWER_CHEST,
    RHT_FOREST_TEMPLE_MQ_RAISED_ISLAND_COURTYARD_UPPER_CHEST,
    RHT_FOREST_TEMPLE_MQ_BOSS_KEY_CHEST,
    RHT_FOREST_TEMPLE_MQ_REDEAD_CHEST,
    RHT_FOREST_TEMPLE_MQ_MAP_CHEST,
    RHT_FOREST_TEMPLE_MQ_BOW_CHEST,
    RHT_FOREST_TEMPLE_MQ_COMPASS_CHEST,
    RHT_FOREST_TEMPLE_MQ_FALLING_CEILING_ROOM_CHEST,
    RHT_FOREST_TEMPLE_MQ_BASEMENT_CHEST,
    RHT_FOREST_TEMPLE_MQ_GS_FIRST_HALLWAY,
    RHT_FOREST_TEMPLE_MQ_GS_RAISED_ISLAND_COURTYARD,
    RHT_FOREST_TEMPLE_MQ_GS_LEVEL_ISLAND_COURTYARD,
    RHT_FOREST_TEMPLE_MQ_GS_WELL,
    RHT_FOREST_TEMPLE_MQ_GS_BLOCK_PUSH_ROOM,
    RHT_FOREST_TEMPLE_PHANTOM_GANON_HEART,
    RHT_FIRE_TEMPLE_NEAR_BOSS_CHEST,
    RHT_FIRE_TEMPLE_FLARE_DANCER_CHEST,
    RHT_FIRE_TEMPLE_BOSS_KEY_CHEST,
    RHT_FIRE_TEMPLE_BIG_LAVA_ROOM_LOWER_OPEN_DOOR_CHEST,
    RHT_FIRE_TEMPLE_BIG_LAVA_ROOM_BLOCKED_DOOR_CHEST,
    RHT_FIRE_TEMPLE_BOULDER_MAZE_LOWER_CHEST,
    RHT_FIRE_TEMPLE_BOULDER_MAZE_SIDE_ROOM_CHEST,
    RHT_FIRE_TEMPLE_MAP_CHEST,
    RHT_FIRE_TEMPLE_BOULDER_MAZE_SHORTCUT_CHEST,
    RHT_FIRE_TEMPLE_BOULDER_MAZE_UPPER_CHEST,
    RHT_FIRE_TEMPLE_SCARECROW_CHEST,
    RHT_FIRE_TEMPLE_COMPASS_CHEST,
    RHT_FIRE_TEMPLE_MEGATON_HAMMER_CHEST,
    RHT_FIRE_TEMPLE_HIGHEST_GORON_CHEST,
    RHT_FIRE_TEMPLE_GS_BOSS_KEY_LOOP,
    RHT_FIRE_TEMPLE_GS_SONG_OF_TIME_ROOM,
    RHT_FIRE_TEMPLE_GS_BOULDER_MAZE,
    RHT_FIRE_TEMPLE_GS_SCARECROW_CLIMB,
    RHT_FIRE_TEMPLE_GS_SCARECROW_TOP,
    RHT_FIRE_TEMPLE_MQ_MAP_ROOM_SIDE_CHEST,
    RHT_FIRE_TEMPLE_MQ_MEGATON_HAMMER_CHEST,
    RHT_FIRE_TEMPLE_MQ_MAP_CHEST,
    RHT_FIRE_TEMPLE_MQ_NEAR_BOSS_CHEST,
    RHT_FIRE_TEMPLE_MQ_BIG_LAVA_ROOM_BLOCKED_DOOR_CHEST,
    RHT_FIRE_TEMPLE_MQ_BOSS_KEY_CHEST,
    RHT_FIRE_TEMPLE_MQ_LIZALFOS_MAZE_SIDE_ROOM_CHEST,
    RHT_FIRE_TEMPLE_MQ_COMPASS_CHEST,
    RHT_FIRE_TEMPLE_MQ_LIZALFOS_MAZE_UPPER_CHEST,
    RHT_FIRE_TEMPLE_MQ_LIZALFOS_MAZE_LOWER_CHEST,
    RHT_FIRE_TEMPLE_MQ_FREESTANDING_KEY,
    RHT_FIRE_TEMPLE_MQ_CHEST_ON_FIRE,
    RHT_FIRE_TEMPLE_MQ_GS_BIG_LAVA_ROOM_OPEN_DOOR,
    RHT_FIRE_TEMPLE_MQ_GS_SKULL_ON_FIRE,
    RHT_FIRE_TEMPLE_MQ_GS_FIRE_WALL_MAZE_CENTER,
    RHT_FIRE_TEMPLE_MQ_GS_FIRE_WALL_MAZE_SIDE_ROOM,
    RHT_FIRE_TEMPLE_MQ_GS_ABOVE_FIRE_WALL_MAZE,
    RHT_FIRE_TEMPLE_VOLVAGIA_HEART,
    RHT_WATER_TEMPLE_COMPASS_CHEST,
    RHT_WATER_TEMPLE_MAP_CHEST,
    RHT_WATER_TEMPLE_CRACKED_WALL_CHEST,
    RHT_WATER_TEMPLE_TORCHES_CHEST,
    RHT_WATER_TEMPLE_BOSS_KEY_CHEST,
    RHT_WATER_TEMPLE_CENTRAL_PILLAR_CHEST,
    RHT_WATER_TEMPLE_CENTRAL_BOW_TARGET_CHEST,
    RHT_WATER_TEMPLE_LONGSHOT_CHEST,
    RHT_WATER_TEMPLE_RIVER_CHEST,
    RHT_WATER_TEMPLE_DRAGON_CHEST,
    RHT_WATER_TEMPLE_GS_BEHIND_GATE,
    RHT_WATER_TEMPLE_GS_NEAR_BOSS_KEY_CHEST,
    RHT_WATER_TEMPLE_GS_CENTRAL_PILLAR,
    RHT_WATER_TEMPLE_GS_FALLING_PLATFORM_ROOM,
    RHT_WATER_TEMPLE_GS_RIVER,
    RHT_WATER_TEMPLE_MQ_LONGSHOT_CHEST,
    RHT_WATER_TEMPLE_MQ_MAP_CHEST,
    RHT_WATER_TEMPLE_MQ_COMPASS_CHEST,
    RHT_WATER_TEMPLE_MQ_CENTRAL_PILLAR_CHEST,
    RHT_WATER_TEMPLE_MQ_BOSS_KEY_CHEST,
    RHT_WATER_TEMPLE_MQ_FREESTANDING_KEY,
    RHT_WATER_TEMPLE_MQ_GS_LIZALFOS_HALLWAY,
    RHT_WATER_TEMPLE_MQ_GS_BEFORE_UPPER_WATER_SWITCH,
    RHT_WATER_TEMPLE_MQ_GS_RIVER,
    RHT_WATER_TEMPLE_MQ_GS_FREESTANDING_KEY_AREA,
    RHT_WATER_TEMPLE_MQ_GS_TRIPLE_WALL_TORCH,
    RHT_WATER_TEMPLE_MORPHA_HEART,
    RHT_SHADOW_TEMPLE_MAP_CHEST,
    RHT_SHADOW_TEMPLE_HOVER_BOOTS_CHEST,
    RHT_SHADOW_TEMPLE_COMPASS_CHEST,
    RHT_SHADOW_TEMPLE_EARLY_SILVER_RUPEE_CHEST,
    RHT_SHADOW_TEMPLE_INVISIBLE_BLADES_VISIBLE_CHEST,
    RHT_SHADOW_TEMPLE_INVISIBLE_BLADES_INVISIBLE_CHEST,
    RHT_SHADOW_TEMPLE_FALLING_SPIKES_LOWER_CHEST,
    RHT_SHADOW_TEMPLE_FALLING_SPIKES_UPPER_CHEST,
    RHT_SHADOW_TEMPLE_FALLING_SPIKES_SWITCH_CHEST,
    RHT_SHADOW_TEMPLE_INVISIBLE_SPIKES_CHEST,
    RHT_SHADOW_TEMPLE_FREESTANDING_KEY,
    RHT_SHADOW_TEMPLE_WIND_HINT_CHEST,
    RHT_SHADOW_TEMPLE_AFTER_WIND_ENEMY_CHEST,
    RHT_SHADOW_TEMPLE_AFTER_WIND_HIDDEN_CHEST,
    RHT_SHADOW_TEMPLE_SPIKE_WALLS_LEFT_CHEST,
    RHT_SHADOW_TEMPLE_BOSS_KEY_CHEST,
    RHT_SHADOW_TEMPLE_INVISIBLE_FLOORMASTER_CHEST,
    RHT_SHADOW_TEMPLE_GS_LIKE_LIKE_ROOM,
    RHT_SHADOW_TEMPLE_GS_FALLING_SPIKES_ROOM,
    RHT_SHADOW_TEMPLE_GS_SINGLE_GIANT_POT,
    RHT_SHADOW_TEMPLE_GS_NEAR_SHIP,
    RHT_SHADOW_TEMPLE_GS_TRIPLE_GIANT_POT,
    RHT_SHADOW_TEMPLE_MQ_EARLY_GIBDOS_CHEST,
    RHT_SHADOW_TEMPLE_MQ_MAP_CHEST,
    RHT_SHADOW_TEMPLE_MQ_NEAR_SHIP_INVISIBLE_CHEST,
    RHT_SHADOW_TEMPLE_MQ_COMPASS_CHEST,
    RHT_SHADOW_TEMPLE_MQ_HOVER_BOOTS_CHEST,
    RHT_SHADOW_TEMPLE_MQ_INVISIBLE_BLADES_INVISIBLE_CHEST,
    RHT_SHADOW_TEMPLE_MQ_INVISIBLE_BLADES_VISIBLE_CHEST,
    RHT_SHADOW_TEMPLE_MQ_BEAMOS_SILVER_RUPEES_CHEST,
    RHT_SHADOW_TEMPLE_MQ_FALLING_SPIKES_LOWER_CHEST,
    RHT_SHADOW_TEMPLE_MQ_FALLING_SPIKES_UPPER_CHEST,
    RHT_SHADOW_TEMPLE_MQ_FALLING_SPIKES_SWITCH_CHEST,
    RHT_SHADOW_TEMPLE_MQ_INVISIBLE_SPIKES_CHEST,
    RHT_SHADOW_TEMPLE_MQ_STALFOS_ROOM_CHEST,
    RHT_SHADOW_TEMPLE_MQ_WIND_HINT_CHEST,
    RHT_SHADOW_TEMPLE_MQ_AFTER_WIND_HIDDEN_CHEST,
    RHT_SHADOW_TEMPLE_MQ_AFTER_WIND_ENEMY_CHEST,
    RHT_SHADOW_TEMPLE_MQ_BOSS_KEY_CHEST,
    RHT_SHADOW_TEMPLE_MQ_SPIKE_WALLS_LEFT_CHEST,
    RHT_SHADOW_TEMPLE_MQ_FREESTANDING_KEY,
    RHT_SHADOW_TEMPLE_MQ_BOMB_FLOWER_CHEST,
    RHT_SHADOW_TEMPLE_MQ_GS_FALLING_SPIKES_ROOM,
    RHT_SHADOW_TEMPLE_MQ_GS_WIND_HINT_ROOM,
    RHT_SHADOW_TEMPLE_MQ_GS_AFTER_WIND,
    RHT_SHADOW_TEMPLE_MQ_GS_AFTER_SHIP,
    RHT_SHADOW_TEMPLE_MQ_GS_NEAR_BOSS,
    RHT_SHADOW_TEMPLE_BONGO_BONGO_HEART,
    RHT_SPIRIT_TEMPLE_CHILD_BRIDGE_CHEST,
    RHT_SPIRIT_TEMPLE_CHILD_EARLY_TORCHES_CHEST,
    RHT_SPIRIT_TEMPLE_CHILD_CLIMB_NORTH_CHEST,
    RHT_SPIRIT_TEMPLE_CHILD_CLIMB_EAST_CHEST,
    RHT_SPIRIT_TEMPLE_MAP_CHEST,
    RHT_SPIRIT_TEMPLE_SUN_BLOCK_ROOM_CHEST,
    RHT_SPIRIT_TEMPLE_MQ_ENTRANCE_FRONT_LEFT_CHEST,
    RHT_SPIRIT_TEMPLE_MQ_ENTRANCE_BACK_RIGHT_CHEST,
    RHT_SPIRIT_TEMPLE_MQ_ENTRANCE_FRONT_RIGHT_CHEST,
    RHT_SPIRIT_TEMPLE_MQ_ENTRANCE_BACK_LEFT_CHEST,
    RHT_SPIRIT_TEMPLE_MQ_MAP_CHEST,
    RHT_SPIRIT_TEMPLE_MQ_MAP_ROOM_ENEMY_CHEST,
    RHT_SPIRIT_TEMPLE_MQ_CHILD_CLIMB_NORTH_CHEST,
    RHT_SPIRIT_TEMPLE_MQ_CHILD_CLIMB_SOUTH_CHEST,
    RHT_SPIRIT_TEMPLE_MQ_COMPASS_CHEST,
    RHT_SPIRIT_TEMPLE_MQ_SILVER_BLOCK_HALLWAY_CHEST,
    RHT_SPIRIT_TEMPLE_MQ_SUN_BLOCK_ROOM_CHEST,
    RHT_SPIRIT_TEMPLE_SILVER_GAUNTLETS_CHEST,
    RHT_SPIRIT_TEMPLE_COMPASS_CHEST,
    RHT_SPIRIT_TEMPLE_EARLY_ADULT_RIGHT_CHEST,
    RHT_SPIRIT_TEMPLE_FIRST_MIRROR_LEFT_CHEST,
    RHT_SPIRIT_TEMPLE_FIRST_MIRROR_RIGHT_CHEST,
    RHT_SPIRIT_TEMPLE_STATUE_ROOM_NORTHEAST_CHEST,
    RHT_SPIRIT_TEMPLE_STATUE_ROOM_HAND_CHEST,
    RHT_SPIRIT_TEMPLE_NEAR_FOUR_ARMOS_CHEST,
    RHT_SPIRIT_TEMPLE_HALLWAY_RIGHT_INVISIBLE_CHEST,
    RHT_SPIRIT_TEMPLE_HALLWAY_LEFT_INVISIBLE_CHEST,
    RHT_SPIRIT_TEMPLE_MQ_CHILD_HAMMER_SWITCH_CHEST,
    RHT_SPIRIT_TEMPLE_MQ_STATUE_ROOM_LULLABY_CHEST,
    RHT_SPIRIT_TEMPLE_MQ_STATUE_ROOM_INVISIBLE_CHEST,
    RHT_SPIRIT_TEMPLE_MQ_LEEVER_ROOM_CHEST,
    RHT_SPIRIT_TEMPLE_MQ_SYMPHONY_ROOM_CHEST,
    RHT_SPIRIT_TEMPLE_MQ_BEAMOS_ROOM_CHEST,
    RHT_SPIRIT_TEMPLE_MQ_CHEST_SWITCH_CHEST,
    RHT_SPIRIT_TEMPLE_MQ_BOSS_KEY_CHEST,
    RHT_SPIRIT_TEMPLE_MIRROR_SHIELD_CHEST,
    RHT_SPIRIT_TEMPLE_BOSS_KEY_CHEST,
    RHT_SPIRIT_TEMPLE_TOPMOST_CHEST,
    RHT_SPIRIT_TEMPLE_MQ_MIRROR_PUZZLE_INVISIBLE_CHEST,
    RHT_SPIRIT_TEMPLE_GS_METAL_FENCE,
    RHT_SPIRIT_TEMPLE_GS_SUN_ON_FLOOR_ROOM,
    RHT_SPIRIT_TEMPLE_GS_HALL_AFTER_SUN_BLOCK_ROOM,
    RHT_SPIRIT_TEMPLE_GS_LOBBY,
    RHT_SPIRIT_TEMPLE_GS_BOULDER_ROOM,
    RHT_SPIRIT_TEMPLE_MQ_GS_SUN_BLOCK_ROOM,
    RHT_SPIRIT_TEMPLE_MQ_GS_LEEVER_ROOM,
    RHT_SPIRIT_TEMPLE_MQ_GS_SYMPHONY_ROOM,
    RHT_SPIRIT_TEMPLE_MQ_GS_NINE_THRONES_ROOM_WEST,
    RHT_SPIRIT_TEMPLE_MQ_GS_NINE_THRONES_ROOM_NORTH,
    RHT_SPIRIT_TEMPLE_TWINROVA_HEART,
    RHT_ICE_CAVERN_MAP_CHEST,
    RHT_ICE_CAVERN_COMPASS_CHEST,
    RHT_ICE_CAVERN_FREESTANDING_POH,
    RHT_ICE_CAVERN_IRON_BOOTS_CHEST,
    RHT_ICE_CAVERN_GS_SPINNING_SCYTHE_ROOM,
    RHT_ICE_CAVERN_GS_HEART_PIECE_ROOM,
    RHT_ICE_CAVERN_GS_PUSH_BLOCK_ROOM,
    RHT_ICE_CAVERN_MQ_MAP_CHEST,
    RHT_ICE_CAVERN_MQ_COMPASS_CHEST,
    RHT_ICE_CAVERN_MQ_FREESTANDING_POH,
    RHT_ICE_CAVERN_MQ_IRON_BOOTS_CHEST,
    RHT_ICE_CAVERN_MQ_GS_RED_ICE,
    RHT_ICE_CAVERN_MQ_GS_ICE_BLOCK,
    RHT_ICE_CAVERN_MQ_GS_SCARECROW,
    RHT_GERUDO_TRAINING_GROUND_LOBBY_LEFT_CHEST,
    RHT_GERUDO_TRAINING_GROUND_LOBBY_RIGHT_CHEST,
    RHT_GERUDO_TRAINING_GROUND_STALFOS_CHEST,
    RHT_GERUDO_TRAINING_GROUND_BEFORE_HEAVY_BLOCK_CHEST,
    RHT_GERUDO_TRAINING_GROUND_HEAVY_BLOCK_FIRST_CHEST,
    RHT_GERUDO_TRAINING_GROUND_HEAVY_BLOCK_SECOND_CHEST,
    RHT_GERUDO_TRAINING_GROUND_HEAVY_BLOCK_THIRD_CHEST,
    RHT_GERUDO_TRAINING_GROUND_HEAVY_BLOCK_FOURTH_CHEST,
    RHT_GERUDO_TRAINING_GROUND_EYE_STATUE_CHEST,
    RHT_GERUDO_TRAINING_GROUND_NEAR_SCARECROW_CHEST,
    RHT_GERUDO_TRAINING_GROUND_HAMMER_ROOM_CLEAR_CHEST,
    RHT_GERUDO_TRAINING_GROUND_HAMMER_ROOM_SWITCH_CHEST,
    RHT_GERUDO_TRAINING_GROUND_FREESTANDING_KEY,
    RHT_GERUDO_TRAINING_GROUND_MAZE_RIGHT_CENTRAL_CHEST,
    RHT_GERUDO_TRAINING_GROUND_MAZE_RIGHT_SIDE_CHEST,
    RHT_GERUDO_TRAINING_GROUND_UNDERWATER_SILVER_RUPEE_CHEST,
    RHT_GERUDO_TRAINING_GROUND_BEAMOS_CHEST,
    RHT_GERUDO_TRAINING_GROUND_HIDDEN_CEILING_CHEST,
    RHT_GERUDO_TRAINING_GROUND_MAZE_PATH_FIRST_CHEST,
    RHT_GERUDO_TRAINING_GROUND_MAZE_PATH_SECOND_CHEST,
    RHT_GERUDO_TRAINING_GROUND_MAZE_PATH_THIRD_CHEST,
    RHT_GERUDO_TRAINING_GROUND_MAZE_PATH_FINAL_CHEST,
    RHT_GERUDO_TRAINING_GROUND_MQ_LOBBY_LEFT_CHEST,
    RHT_GERUDO_TRAINING_GROUND_MQ_LOBBY_RIGHT_CHEST,
    RHT_GERUDO_TRAINING_GROUND_MQ_FIRST_IRON_KNUCKLE_CHEST,
    RHT_GERUDO_TRAINING_GROUND_MQ_BEFORE_HEAVY_BLOCK_CHEST,
    RHT_GERUDO_TRAINING_GROUND_MQ_HEAVY_BLOCK_CHEST,
    RHT_GERUDO_TRAINING_GROUND_MQ_EYE_STATUE_CHEST,
    RHT_GERUDO_TRAINING_GROUND_MQ_ICE_ARROWS_CHEST,
    RHT_GERUDO_TRAINING_GROUND_MQ_SECOND_IRON_KNUCKLE_CHEST,
    RHT_GERUDO_TRAINING_GROUND_MQ_FLAME_CIRCLE_CHEST,
    RHT_GERUDO_TRAINING_GROUND_MQ_MAZE_RIGHT_CENTRAL_CHEST,
    RHT_GERUDO_TRAINING_GROUND_MQ_MAZE_RIGHT_SIDE_CHEST,
    RHT_GERUDO_TRAINING_GROUND_MQ_UNDERWATER_SILVER_RUPEE_CHEST,
    RHT_GERUDO_TRAINING_GROUND_MQ_DINOLFOS_CHEST,
    RHT_GERUDO_TRAINING_GROUND_MQ_HIDDEN_CEILING_CHEST,
    RHT_GERUDO_TRAINING_GROUND_MQ_MAZE_PATH_FIRST_CHEST,
    RHT_GERUDO_TRAINING_GROUND_MQ_MAZE_PATH_THIRD_CHEST,
    RHT_GERUDO_TRAINING_GROUND_MQ_MAZE_PATH_SECOND_CHEST,
    RHT_GANONS_CASTLE_FOREST_TRIAL_CHEST,
    RHT_GANONS_CASTLE_WATER_TRIAL_LEFT_CHEST,
    RHT_GANONS_CASTLE_WATER_TRIAL_RIGHT_CHEST,
    RHT_GANONS_CASTLE_SHADOW_TRIAL_FRONT_CHEST,
    RHT_GANONS_CASTLE_SHADOW_TRIAL_GOLDEN_GAUNTLETS_CHEST,
    RHT_GANONS_CASTLE_LIGHT_TRIAL_FIRST_LEFT_CHEST,
    RHT_GANONS_CASTLE_LIGHT_TRIAL_SECOND_LEFT_CHEST,
    RHT_GANONS_CASTLE_LIGHT_TRIAL_THIRD_LEFT_CHEST,
    RHT_GANONS_CASTLE_LIGHT_TRIAL_FIRST_RIGHT_CHEST,
    RHT_GANONS_CASTLE_LIGHT_TRIAL_SECOND_RIGHT_CHEST,
    RHT_GANONS_CASTLE_LIGHT_TRIAL_THIRD_RIGHT_CHEST,
    RHT_GANONS_CASTLE_LIGHT_TRIAL_INVISIBLE_ENEMIES_CHEST,
    RHT_GANONS_CASTLE_LIGHT_TRIAL_LULLABY_CHEST,
    RHT_GANONS_CASTLE_SPIRIT_TRIAL_CRYSTAL_SWITCH_CHEST,
    RHT_GANONS_CASTLE_SPIRIT_TRIAL_INVISIBLE_CHEST,
    RHT_GANONS_CASTLE_DEKU_SCRUB_LEFT,
    RHT_GANONS_CASTLE_DEKU_SCRUB_CENTER_LEFT,
    RHT_GANONS_CASTLE_DEKU_SCRUB_CENTER_RIGHT,
    RHT_GANONS_CASTLE_DEKU_SCRUB_RIGHT,
    RHT_GANONS_CASTLE_MQ_FOREST_TRIAL_FREESTANDING_KEY,
    RHT_GANONS_CASTLE_MQ_FOREST_TRIAL_EYE_SWITCH_CHEST,
    RHT_GANONS_CASTLE_MQ_FOREST_TRIAL_FROZEN_EYE_SWITCH_CHEST,
    RHT_GANONS_CASTLE_MQ_WATER_TRIAL_CHEST,
    RHT_GANONS_CASTLE_MQ_SHADOW_TRIAL_BOMB_FLOWER_CHEST,
    RHT_GANONS_CASTLE_MQ_SHADOW_TRIAL_EYE_SWITCH_CHEST,
    RHT_GANONS_CASTLE_MQ_LIGHT_TRIAL_LULLABY_CHEST,
    RHT_GANONS_CASTLE_MQ_SPIRIT_TRIAL_FIRST_CHEST,
    RHT_GANONS_CASTLE_MQ_SPIRIT_TRIAL_INVISIBLE_CHEST,
    RHT_GANONS_CASTLE_MQ_SPIRIT_TRIAL_SUN_FRONT_LEFT_CHEST,
    RHT_GANONS_CASTLE_MQ_SPIRIT_TRIAL_SUN_BACK_LEFT_CHEST,
    RHT_GANONS_CASTLE_MQ_SPIRIT_TRIAL_SUN_BACK_RIGHT_CHEST,
    RHT_GANONS_CASTLE_MQ_SPIRIT_TRIAL_GOLDEN_GAUNTLETS_CHEST,
    RHT_GANONS_CASTLE_MQ_DEKU_SCRUB_LEFT,
    RHT_GANONS_CASTLE_MQ_DEKU_SCRUB_CENTER_LEFT,
    RHT_GANONS_CASTLE_MQ_DEKU_SCRUB_CENTER,
    RHT_GANONS_CASTLE_MQ_DEKU_SCRUB_CENTER_RIGHT,
    RHT_GANONS_CASTLE_MQ_DEKU_SCRUB_RIGHT,
    RHT_GANONS_TOWER_BOSS_KEY_CHEST,
    RHT_DELIVER_RUTOS_LETTER,
    RHT_MASTER_SWORD_PEDESTAL,
    // Beehives
    RHT_BEEHIVE_CHEST_GROTTO,
    RHT_BEEHIVE_COW_GROTTO,
    RHT_BEEHIVE_LONELY_SCRUB_GROTTO,
    RHT_BEEHIVE_SCRUB_PAIR_GROTTO,
    RHT_BEEHIVE_SCRUB_TRIO_GROTTO,
    RHT_BEEHIVE_IN_FRONT_OF_KING_ZORA,
    RHT_BEEHIVE_BEHIND_KING_ZORA,
    // Items
    RHT_KOKIRI_SWORD,
    RHT_MASTER_SWORD,
    RHT_GIANTS_KNIFE,
    RHT_BIGGORON_SWORD,
    RHT_DEKU_SHIELD,
    RHT_HYLIAN_SHIELD,
    RHT_MIRROR_SHIELD,
    RHT_GORON_TUNIC,
    RHT_ZORA_TUNIC,
    RHT_IRON_BOOTS,
    RHT_HOVER_BOOTS,
    RHT_BOOMERANG,
    RHT_LENS_OF_TRUTH,
    RHT_MEGATON_HAMMER,
    RHT_STONE_OF_AGONY,
    RHT_DINS_FIRE,
    RHT_FARORES_WIND,
    RHT_NAYRUS_LOVE,
    RHT_FIRE_ARROWS,
    RHT_ICE_ARROWS,
    RHT_LIGHT_ARROWS,
    RHT_GERUDO_MEMBERSHIP_CARD,
    RHT_MAGIC_BEAN,
    RHT_MAGIC_BEAN_PACK,
    RHT_DOUBLE_DEFENSE,
    RHT_WEIRD_EGG,
    RHT_ZELDAS_LETTER,
    RHT_POCKET_EGG,
    RHT_POCKET_CUCCO,
    RHT_COJIRO,
    RHT_ODD_MUSHROOM,
    RHT_ODD_POTION,
    RHT_POACHERS_SAW,
    RHT_BROKEN_SWORD,
    RHT_PRESCRIPTION,
    RHT_EYEBALL_FROG,
    RHT_EYEDROPS,
    RHT_CLAIM_CHECK,
    RHT_GOLD_SKULLTULA_TOKEN,
    RHT_PROGRESSIVE_HOOKSHOT,
    RHT_PROGRESSIVE_STRENGTH,
    RHT_PROGRESSIVE_BOMB_BAG,
    RHT_PROGRESSIVE_BOW,
    RHT_PROGRESSIVE_SLINGSHOT,
    RHT_PROGRESSIVE_WALLET,
    RHT_PROGRESSIVE_SCALE,
    RHT_PROGRESSIVE_NUT_UPGRADE,
    RHT_PROGRESSIVE_STICK_UPGRADE,
    RHT_PROGRESSIVE_BOMBCHUS,
    RHT_PROGRESSIVE_MAGIC_METER,
    RHT_MAGIC_SINGLE, // Added for refactor of GetItemEntries
    RHT_MAGIC_DOUBLE, // Added for refactor of GetItemEntries
    RHT_PROGRESSIVE_OCARINA,
    RHT_PROGRESSIVE_GORONSWORD,
    RHT_EMPTY_BOTTLE,
    RHT_BOTTLE_WITH_MILK,
    RHT_BOTTLE_WITH_RED_POTION,
    RHT_BOTTLE_WITH_GREEN_POTION,
    RHT_BOTTLE_WITH_BLUE_POTION,
    RHT_BOTTLE_WITH_FAIRY,
    RHT_BOTTLE_WITH_FISH,
    RHT_BOTTLE_WITH_BLUE_FIRE,
    RHT_BOTTLE_WITH_BUGS,
    RHT_BOTTLE_WITH_POE,
    RHT_RUTOS_LETTER,
    RHT_BOTTLE_WITH_BIG_POE,
    RHT_ZELDAS_LULLABY,
    RHT_EPONAS_SONG,
    RHT_SARIAS_SONG,
    RHT_SUNS_SONG,
    RHT_SONG_OF_TIME,
    RHT_SONG_OF_STORMS,
    RHT_MINUET_OF_FOREST,
    RHT_BOLERO_OF_FIRE,
    RHT_SERENADE_OF_WATER,
    RHT_REQUIEM_OF_SPIRIT,
    RHT_NOCTURNE_OF_SHADOW,
    RHT_PRELUDE_OF_LIGHT,
    RHT_DEKU_TREE_MAP,
    RHT_DODONGOS_CAVERN_MAP,
    RHT_JABU_JABUS_BELLY_MAP,
    RHT_FOREST_TEMPLE_MAP,
    RHT_FIRE_TEMPLE_MAP,
    RHT_WATER_TEMPLE_MAP,
    RHT_SPIRIT_TEMPLE_MAP,
    RHT_SHADOW_TEMPLE_MAP,
    RHT_BOTTOM_OF_THE_WELL_MAP,
    RHT_ICE_CAVERN_MAP,
    RHT_DEKU_TREE_COMPASS,
    RHT_DODONGOS_CAVERN_COMPASS,
    RHT_JABU_JABUS_BELLY_COMPASS,
    RHT_FOREST_TEMPLE_COMPASS,
    RHT_FIRE_TEMPLE_COMPASS,
    RHT_WATER_TEMPLE_COMPASS,
    RHT_SPIRIT_TEMPLE_COMPASS,
    RHT_SHADOW_TEMPLE_COMPASS,
    RHT_BOTTOM_OF_THE_WELL_COMPASS,
    RHT_ICE_CAVERN_COMPASS,
    RHT_FOREST_TEMPLE_BOSS_KEY,
    RHT_FIRE_TEMPLE_BOSS_KEY,
    RHT_WATER_TEMPLE_BOSS_KEY,
    RHT_SPIRIT_TEMPLE_BOSS_KEY,
    RHT_SHADOW_TEMPLE_BOSS_KEY,
    RHT_GANONS_CASTLE_BOSS_KEY,
    RHT_FOREST_TEMPLE_SMALL_KEY,
    RHT_FIRE_TEMPLE_SMALL_KEY,
    RHT_WATER_TEMPLE_SMALL_KEY,
    RHT_SPIRIT_TEMPLE_SMALL_KEY,
    RHT_SHADOW_TEMPLE_SMALL_KEY,
    RHT_BOTTOM_OF_THE_WELL_SMALL_KEY,
    RHT_GERUDO_TRAINING_GROUND_SMALL_KEY,
    RHT_GERUDO_FORTRESS_SMALL_KEY,
    RHT_GANONS_CASTLE_SMALL_KEY,
    RHT_TREASURE_GAME_SMALL_KEY,
    RHT_FOREST_TEMPLE_KEY_RING,
    RHT_FIRE_TEMPLE_KEY_RING,
    RHT_WATER_TEMPLE_KEY_RING,
    RHT_SPIRIT_TEMPLE_KEY_RING,
    RHT_SHADOW_TEMPLE_KEY_RING,
    RHT_BOTTOM_OF_THE_WELL_KEY_RING,
    RHT_GERUDO_TRAINING_GROUND_KEY_RING,
    RHT_GERUDO_FORTRESS_KEY_RING,
    RHT_GANONS_CASTLE_KEY_RING,
    RHT_TREASURE_GAME_KEY_RING,
    RHT_KOKIRI_EMERALD,
    RHT_GORON_RUBY,
    RHT_ZORA_SAPPHIRE,
    RHT_FOREST_MEDALLION,
    RHT_FIRE_MEDALLION,
    RHT_WATER_MEDALLION,
    RHT_SPIRIT_MEDALLION,
    RHT_SHADOW_MEDALLION,
    RHT_LIGHT_MEDALLION,
    RHT_RECOVERY_HEART,
    RHT_GREEN_RUPEE,
    RHT_GREG_RUPEE,
    RHT_BLUE_RUPEE,
    RHT_RED_RUPEE,
    RHT_PURPLE_RUPEE,
    RHT_HUGE_RUPEE,
    RHT_PIECE_OF_HEART,
    RHT_HEART_CONTAINER,
    RHT_ICE_TRAP,
    RHT_MILK,
    RHT_BOMBS_5,
    RHT_BOMBS_10,
    RHT_BOMBS_20,
    RHT_BOMBCHUS_5,
    RHT_BOMBCHUS_10,
    RHT_BOMBCHUS_20,
    RHT_ARROWS_5,
    RHT_ARROWS_10,
    RHT_ARROWS_30,
    RHT_DEKU_NUTS_5,
    RHT_DEKU_NUTS_10,
    RHT_DEKU_SEEDS_30,
    RHT_DEKU_STICK_1,
    RHT_RED_POTION_REFILL,
    RHT_GREEN_POTION_REFILL,
    RHT_BLUE_POTION_REFILL,
    RHT_TREASURE_GAME_HEART,
    RHT_TREASURE_GAME_GREEN_RUPEE,
    RHT_BUY_DEKU_NUTS_5,
    RHT_BUY_ARROWS_30,
    RHT_BUY_ARROWS_50,
    RHT_BUY_BOMBS_525,
    RHT_BUY_DEKU_NUTS_10,
    RHT_BUY_DEKU_STICK_1,
    RHT_BUY_BOMBS_10,
    RHT_BUY_FISH,
    RHT_BUY_RED_POTION_30,
    RHT_BUY_GREEN_POTION,
    RHT_BUY_BLUE_POTION,
    RHT_BUY_HYLIAN_SHIELD,
    RHT_BUY_DEKU_SHIELD,
    RHT_BUY_GORON_TUNIC,
    RHT_BUY_ZORA_TUNIC,
    RHT_BUY_HEART,
    RHT_BUY_BOMBCHUS_10,
    RHT_BUY_BOMBCHUS_20,
    RHT_BUY_DEKU_SEEDS_30,
    RHT_SOLD_OUT,
    RHT_BUY_BLUE_FIRE,
    RHT_BUY_BOTTLE_BUG,
    RHT_BUY_POE,
    RHT_BUY_FAIRYS_SPIRIT,
    RHT_BUY_ARROWS_10,
    RHT_BUY_BOMBS_20,
    RHT_BUY_BOMBS_30,
    RHT_BUY_BOMBS_535,
    RHT_BUY_RED_POTION_40,
    RHT_BUY_RED_POTION_50,
    RHT_TRIFORCE,
    RHT_BOMBCHU_BAG,
    RHT_QUIVER_INF,
    RHT_BOMB_BAG_INF,
    RHT_BULLET_BAG_INF,
    RHT_STICK_UPGRADE_INF,
    RHT_NUT_UPGRADE_INF,
    RHT_MAGIC_INF,
    RHT_BOMBCHU_INF,
    RHT_WALLET_INF,
    RHT_HINT,
    RHT_TYCOON_WALLET,
    RHT_CHILD_WALLET,
    RHT_HOOKSHOT,
    RHT_LONGSHOT,
    RHT_FAIRY_OCARINA,
    RHT_OCARINA_OF_TIME,
    RHT_BOMB_BAG,
    RHT_BIG_BOMB_BAG,
    RHT_BIGGEST_BOMB_BAG,
    RHT_FAIRY_BOW,
    RHT_BIG_QUIVER,
    RHT_BIGGEST_QUIVER,
    RHT_FAIRY_SLINGSHOT,
    RHT_BIG_BULLET_BAG,
    RHT_BIGGEST_BULLET_BAG,
    RHT_GORONS_BRACELET,
    RHT_SILVER_GAUNTLETS,
    RHT_GOLDEN_GAUNTLETS,
    RHT_SILVER_SCALE,
    RHT_GOLDEN_SCALE,
    RHT_ADULT_WALLET,
    RHT_GIANT_WALLET,
    RHT_DEKU_NUT_CAPACITY_30,
    RHT_DEKU_NUT_CAPACITY_40,
    RHT_DEKU_STICK_CAPACITY_20,
    RHT_DEKU_STICK_CAPACITY_30,
    RHT_TRIFORCE_PIECE,
    RHT_GOHMA_SOUL,
    RHT_KING_DODONGO_SOUL,
    RHT_BARINADE_SOUL,
    RHT_PHANTOM_GANON_SOUL,
    RHT_VOLVAGIA_SOUL,
    RHT_MORPHA_SOUL,
    RHT_BONGO_BONGO_SOUL,
    RHT_TWINROVA_SOUL,
    RHT_GANON_SOUL,
    RHT_OCARINA_A_BUTTON,
    RHT_OCARINA_C_UP_BUTTON,
    RHT_OCARINA_C_DOWN_BUTTON,
    RHT_OCARINA_C_LEFT_BUTTON,
    RHT_OCARINA_C_RIGHT_BUTTON,
    RHT_BRONZE_SCALE,
    RHT_FISHING_POLE,
    RHT_SKELETON_KEY,
    RHT_EPONA,
    RHT_OVERWORLD_KEY,
    RHT_HINT_MYSTERIOUS,
    RHT_MYSTERIOUS_ITEM,
    RHT_MYSTERIOUS_ITEM_CAPITAL,
    // Entrances
    RHT_DESERT_COLOSSUS_TO_COLOSSUS_GROTTO,
    RHT_GV_GROTTO_LEDGE_TO_GV_OCTOROK_GROTTO,
    RHT_GC_GROTTO_PLATFORM_TO_GC_GROTTO,
    RHT_GERUDO_FORTRESS_TO_GF_STORMS_GROTTO,
    RHT_ZORAS_DOMAIN_TO_ZD_STORMS_GROTTO,
    RHT_HYRULE_CASTLE_GROUNDS_TO_HC_STORMS_GROTTO,
    RHT_GV_FORTRESS_SIDE_TO_GV_STORMS_GROTTO,
    RHT_DESERT_COLOSSUS_TO_COLOSSUS_GREAT_FAIRY_FOUNTAIN,
    RHT_GANONS_CASTLE_GROUNDS_TO_OGC_GREAT_FAIRY_FOUNTAIN,
    RHT_ZORAS_FOUNTAIN_TO_ZF_GREAT_FAIRY_FOUNTAIN,
    RHT_GV_FORTRESS_SIDE_TO_GV_CARPENTER_TENT,
    RHT_GRAVEYARD_WARP_PAD_REGION_TO_SHADOW_TEMPLE_ENTRYWAY,
    RHT_LAKE_HYLIA_TO_WATER_TEMPLE_LOBBY,
    RHT_GERUDO_FORTRESS_TO_GERUDO_TRAINING_GROUND_LOBBY,
    RHT_ZORAS_FOUNTAIN_TO_JABU_JABUS_BELLY_BEGINNING,
    RHT_KAKARIKO_VILLAGE_TO_BOTTOM_OF_THE_WELL,
    // Exits
    RHT_KOKIRI_FOREST,
    RHT_THE_LOST_WOODS,
    RHT_SACRED_FOREST_MEADOW,
    RHT_HYRULE_FIELD,
    RHT_LAKE_HYLIA,
    RHT_GERUDO_VALLEY,
    RHT_GERUDO_FORTRESS,
    RHT_HAUNTED_WASTELAND,
    RHT_DESERT_COLOSSUS,
    RHT_THE_MARKET,
    RHT_TEMPLE_OF_TIME,
    RHT_HYRULE_CASTLE,
    RHT_OUTSIDE_GANONS_CASTLE,
    RHT_CASTLE_GROUNDS,
    RHT_KAKARIKO_VILLAGE,
    RHT_THE_GRAVEYARD,
    RHT_DEATH_MOUNTAIN_TRAIL,
    RHT_GORON_CITY,
    RHT_DEATH_MOUNTAIN_CRATER,
    RHT_ZORAS_RIVER,
    RHT_ZORAS_DOMAIN,
    RHT_ZORAS_FOUNTAIN,
    RHT_LON_LON_RANCH,
    // Regions
    RHT_KF_LINKS_HOUSE,
    RHT_KF_MIDOS_HOUSE,
    RHT_KF_SARIAS_HOUSE,
    RHT_KF_HOUSE_OF_TWINS,
    RHT_KF_KNOW_IT_ALL_HOUSE,
    RHT_KF_KOKIRI_SHOP,
    RHT_LH_LAB,
    RHT_LH_FISHING_POND,
    RHT_GV_CARPENTER_TENT,
    RHT_MARKET_GUARD_HOUSE,
    RHT_MARKET_MASK_SHOP,
    RHT_MARKET_BOMBCHU_BOWLING,
    RHT_MARKET_POTION_SHOP,
    RHT_MARKET_TREASURE_CHEST_GAME,
    RHT_MARKET_BOMBCHU_SHOP,
    RHT_MARKET_MAN_IN_GREEN_HOUSE,
    RHT_KAK_WINDMILL,
    RHT_KAK_CARPENTER_BOSS_HOUSE,
    RHT_KAK_HOUSE_OF_SKULLTULA,
    RHT_KAK_IMPAS_HOUSE,
    RHT_KAK_IMPAS_HOUSE_BACK,
    RHT_KAK_ODD_POTION_BUILDING,
    RHT_GRAVEYARD_DAMPES_HOUSE,
    RHT_GC_SHOP,
    RHT_ZD_SHOP,
    RHT_LLR_TALONS_HOUSE,
    RHT_LLR_STABLES,
    RHT_LLR_TOWER,
    RHT_MARKET_BAZAAR,
    RHT_MARKET_SHOOTING_GALLERY,
    RHT_KAK_BAZAAR,
    RHT_KAK_POTION_SHOP_FRONT,
    RHT_KAK_POTION_SHOP_BACK,
    RHT_KAK_SHOOTING_GALLERY,
    RHT_COLOSSUS_GREAT_FAIRY_FOUNTAIN,
    RHT_HC_GREAT_FAIRY_FOUNTAIN,
    RHT_OGC_GREAT_FAIRY_FOUNTAIN,
    RHT_DMC_GREAT_FAIRY_FOUNTAIN,
    RHT_DMT_GREAT_FAIRY_FOUNTAIN,
    RHT_ZF_GREAT_FAIRY_FOUNTAIN,
    RHT_GRAVEYARD_SHIELD_GRAVE,
    RHT_GRAVEYARD_HEART_PIECE_GRAVE,
    RHT_GRAVEYARD_COMPOSERS_GRAVE,
    RHT_GRAVEYARD_DAMPES_GRAVE,
    RHT_DMT_COW_GROTTO,
    RHT_HC_STORMS_GROTTO,
    RHT_HF_TEKTITE_GROTTO,
    RHT_HF_NEAR_KAK_GROTTO,
    RHT_HF_COW_GROTTO,
    RHT_KAK_REDEAD_GROTTO,
    RHT_SFM_WOLFOS_GROTTO,
    RHT_GV_OCTOROK_GROTTO,
    RHT_DEKU_THEATER,
    RHT_ZR_OPEN_GROTTO,
    RHT_DMC_UPPER_GROTTO,
    RHT_DMT_STORMS_GROTTO,
    RHT_KAK_OPEN_GROTTO,
    RHT_HF_NEAR_MARKET_GROTTO,
    RHT_HF_OPEN_GROTTO,
    RHT_HF_SOUTHEAST_GROTTO,
    RHT_KF_STORMS_GROTTO,
    RHT_LW_NEAR_SHORTCUTS_GROTTO,
    RHT_HF_INSIDE_FENCE_GROTTO,
    RHT_LW_SCRUBS_GROTTO,
    RHT_COLOSSUS_GROTTO,
    RHT_ZR_STORMS_GROTTO,
    RHT_SFM_STORMS_GROTTO,
    RHT_GV_STORMS_GROTTO,
    RHT_LH_GROTTO,
    RHT_DMC_HAMMER_GROTTO,
    RHT_GC_GROTTO,
    RHT_LLR_GROTTO,
    RHT_ZR_FAIRY_GROTTO,
    RHT_HF_FAIRY_GROTTO,
    RHT_SFM_FAIRY_GROTTO,
    RHT_ZD_STORMS_GROTTO,
    RHT_GF_STORMS_GROTTO,
    RHT_DEKU_TREE,
    RHT_DODONGOS_CAVERN,
    RHT_JABU_JABUS_BELLY,
    RHT_FOREST_TEMPLE,
    RHT_FIRE_TEMPLE,
    RHT_WATER_TEMPLE,
    RHT_SPIRIT_TEMPLE,
    RHT_SHADOW_TEMPLE,
    RHT_ICE_CAVERN,
    RHT_BOTTOM_OF_THE_WELL,
    RHT_GERUDO_TRAINING_GROUND,
    RHT_GANONS_CASTLE,
    // Bridge Hints
    RHT_BRIDGE_OPEN_HINT,
    RHT_BRIDGE_VANILLA_HINT,
    RHT_BRIDGE_STONES_HINT,
    RHT_BRIDGE_MEDALLIONS_HINT,
    RHT_BRIDGE_REWARDS_HINT,
    RHT_BRIDGE_DUNGEONS_HINT,
    RHT_BRIDGE_TOKENS_HINT,
    RHT_BRIDGE_GREG_HINT,
    // Ganon Boss Key
    RHT_GANON_BK_START_WITH_HINT,
    RHT_GANON_BK_VANILLA_HINT,
    RHT_GANON_BK_OWN_DUNGEON_HINT,
    RHT_GANON_BK_OVERWORLD_HINT,
    RHT_GANON_BK_ANY_DUNGEON_HINT,
    RHT_GANON_BK_ANYWHERE_HINT,
    RHT_GANON_BK_TRIFORCE_HINT,
    RHT_GANON_BK_SKULLTULA_HINT,
    // LACS
    RHT_LACS_VANILLA_HINT,
    RHT_LACS_MEDALLIONS_HINT,
    RHT_LACS_STONES_HINT,
    RHT_LACS_REWARDS_HINT,
    RHT_LACS_DUNGEONS_HINT,
    RHT_LACS_TOKENS_HINT,
    // Trials
    RHT_SIX_TRIALS,
    RHT_ZERO_TRIALS,
    RHT_TRIAL_OFF,
    RHT_TRIAL_ON,
    RHT_LIGHT_TRIAL,
    RHT_FOREST_TRIAL,
    RHT_FIRE_TRIAL,
    RHT_WATER_TRIAL,
    RHT_SPIRIT_TRIAL,
    RHT_SHADOW_TRIAL,
    // Altar
    RHT_CHILD_ALTAR_STONES,
    RHT_CHILD_ALTAR_TEXT_END_DOTOPEN,
    RHT_CHILD_ALTAR_TEXT_END_DOTSONGONLY,
    RHT_CHILD_ALTAR_TEXT_END_DOTCLOSED,
    RHT_ADULT_ALTAR_MEDALLIONS,
    RHT_ADULT_ALTAR_TEXT_END,
    // Static Item Hints
    RHT_GANONDORF_HINT_LA_ONLY,
    RHT_GANONDORF_HINT_MS_ONLY,
    RHT_GANONDORF_HINT_LA_AND_MS,
    RHT_SHEIK_HINT_LA_ONLY,
    RHT_DAMPE_DIARY,
    RHT_GREG_HINT,
    RHT_SARIA_TALK_HINT,
    RHT_SARIA_SONG_HINT,
    RHT_LOACH_HINT,
    RHT_FISHING_POLE_HINT,
    // Static Entrance Hints
    RHT_WARP_SONG,
    // Static Location Hints
    RHT_CARPET_SALESMAN_DIALOG_FIRST,
    RHT_CARPET_SALESMAN_DIALOG_MYSTERIOUS,
    RHT_HBA_HINT_SIGN,
    RHT_HBA_HINT_NOT_ON_HORSE,
    RHT_HBA_HINT_INITIAL,
    RHT_HBA_HINT_HAVE_1000,
    RHT_MALON_HINT_HOW_IS_EPONA,
    RHT_MALON_HINT_OBSTICLE_COURSE,
    RHT_MALON_HINT_TURNING_EVIL,
    RHT_MALON_HINT_INGO_TEMPTED,
    RHT_CHICKENS_HINT,
    RHT_BIG_POES_HINT,
    RHT_BIGGORON_HINT,
    RHT_FROGS_HINT,
    RHT_OOT_HINT,
    RHT_SKULLS_HINT,
    RHT_MASK_SHOP_HINT,
    // Shuffle Pots
    RHT_POT_KOKIRI_FOREST,
    RHT_POT_GERUDO_FORTRESS,
    RHT_POT_WASTELAND,
    RHT_POT_MARKET,
    RHT_POT_KAKARIKO,
    RHT_POT_GRAVEYARD,
    RHT_POT_GORON_CITY,
    RHT_POT_DEATH_MOUNTAIN_CRATER,
    RHT_POT_ZORAS_DOMAIN,
    RHT_POT_ZORAS_FOUNTAIN,
    RHT_POT_LON_LON_RANCH,
    RHT_POT_HYRULE_FIELD,
    RHT_POT_HYRULE_CASTLE,
    RHT_POT_DODONGOS_CAVERN,
    RHT_POT_JABU_JABUS_BELLY,
    RHT_POT_FOREST_TEMPLE,
    RHT_POT_FIRE_TEMPLE,
    RHT_POT_WATER_TEMPLE,
    RHT_POT_GERUDO_TRAINING_GROUND,
    RHT_POT_SHADOW_TEMPLE,
    RHT_POT_SPIRIT_TEMPLE,
    RHT_POT_GANONS_CASTLE,
    RHT_POT_BOTTOM_OF_THE_WELL,
    RHT_POT_ICE_CAVERN,
    // Shuffle Crates
    RHT_CRATE_GERUDO_VALLEY,
    RHT_CRATE_GERUDOS_FORTRESS,
    RHT_CRATE_THIEVES_HIDEOUT,
    RHT_CRATE_WASTELAND,
    RHT_CRATE_MARKET,
    RHT_CRATE_DOG_LADY_HOUSE,
    RHT_CRATE_MARKET_GUARD_HOUSE,
    RHT_CRATE_KAKARIKO_VILLAGE,
    RHT_CRATE_GRAVEYARD,
    RHT_CRATE_GORON_CITY,
    RHT_CRATE_DEATH_MOUNTAIN_CRATER,
    RHT_CRATE_LON_LON_RANCH,
    RHT_CRATE_LAKESIDE_LABORATORY,
    RHT_CRATE_DEKU_TREE,
    RHT_CRATE_DODONGOS_CAVERN,
    RHT_CRATE_JABU_JABU,
    RHT_CRATE_FOREST_TEMPLE,
    RHT_CRATE_FIRE_TEMPLE,
    RHT_CRATE_WATER_TEMPLE,
    RHT_CRATE_SPIRIT_TEMPLE,
    RHT_CRATE_SHADOW_TEMPLE,
    RHT_CRATE_GERUDO_TRAINING_GROUND,
    // Ganon Line
    RHT_GANON_JOKE01,
    RHT_GANON_JOKE02,
    RHT_GANON_JOKE03,
    RHT_GANON_JOKE04,
    RHT_GANON_JOKE05,
    RHT_GANON_JOKE06,
    RHT_GANON_JOKE07,
    RHT_GANON_JOKE08,
    RHT_GANON_JOKE09,
    RHT_GANON_JOKE10,
    RHT_GANON_JOKE11,
    // Misc utilities
    RHT_YOUR_POCKET,
    RHT_ISOLATED_PLACE,
    RHT_DUNGEON_ORDINARY,
    RHT_DUNGEON_MASTERFUL,
    // Shuffle Rupees & Hearts
    RHT_KOKIRI_FOREST_RUPEE,
    RHT_KOKIRI_FOREST_HEART,
    RHT_SARIAS_HOUSE_HEART,
    RHT_LOST_WOODS_RUPEE,
    RHT_LOST_WOODS_SHORTCUT_RUPEE,
    RHT_LAKE_HYLIA_RUPEE,
    RHT_LABORATORY_RUPEE,
    RHT_DAMPES_GRAVE_RUPEE,
    RHT_GERUDO_VALLEY_GROTTO_RUPEE,
    RHT_DEATH_MOUNTAIN_TRAIL_RUPEE,
    RHT_DEATH_MOUNTAIN_TRAIL_GROTTO_HEART,
    RHT_DEATH_MOUNTAIN_TRAIL_GROTTO_RUPEE,
    RHT_DEATH_MOUNTAIN_CRATER_RUPEE,
    RHT_ZORAS_RIVER_WATERFALL_RUPEE,
    RHT_ZORAS_FOUNTAIN_RUPEE,
    RHT_DEKU_TREE_HEART,
    RHT_DODONGOS_CAVERN_HEART,
    RHT_JABU_JABU_RUPEE,
    RHT_JABU_JABU_HEART,
    RHT_FOREST_TEMPLE_HEART,
    RHT_FIRE_TEMPLE_HEART,
    RHT_WATER_TEMPLE_HEART,
    RHT_SHADOW_TEMPLE_HEART,
    RHT_SPIRIT_TEMPLE_HEART,
    RHT_SPIRIT_TEMPLE_MQ_HEART,
    RHT_BOTTOM_OF_THE_WELL_HEART,
    RHT_BOTTOM_OF_THE_WELL_RUPEE,
    RHT_ICE_CAVERN_HEART,
    RHT_ICE_CAVERN_RUPEE,
    RHT_GERUDO_TRAINING_GROUNDS_HEART,
    RHT_GANONS_CASTLE_HEART,
    // Fairy Shuffle
    RHT_SFM_FAIRY_GROTTO_FAIRY,
    RHT_ZR_FAIRY_GROTTO_FAIRY,
    RHT_HF_FAIRY_GROTTO_FAIRY,
    RHT_ZD_FAIRY_GROTTO_FAIRY,
    RHT_GF_FAIRY_GROTTO_FAIRY,
    RHT_GRAVEYARD_SHIELD_GRAVE_FAIRY,
    RHT_GANONS_CASTLE_SCRUBS_FAIRY,
    RHT_COLOSSUS_OASIS_FAIRY,
    RHT_ZR_BEAN_SPROUT_FAIRY,
    RHT_KF_BEAN_SPROUT_FAIRY,
    RHT_LW_BEAN_SPROUT_NEAR_BRIDGE_FAIRY,
    RHT_LW_BEAN_SPROUT_NEAR_THEATER_FAIRY,
    RHT_LH_BEAN_SPROUT_FAIRY,
    RHT_GV_BEAN_SPROUT_FAIRY,
    RHT_COLOSSUS_BEAN_SPROUT_FAIRY,
    RHT_GRAVEYARD_BEAN_SPROUT_FAIRY,
    RHT_DMC_BEAN_SPROUT_FAIRY,
    RHT_DMT_BEAN_SPROUT_FAIRY,
    RHT_TOT_GOSSIP_STONE_FAIRY,
    RHT_TOT_GOSSIP_STONE_FAIRY_BIG,
    RHT_DMC_GOSSIP_STONE_FAIRY,
    RHT_DMC_GOSSIP_STONE_FAIRY_BIG,
    RHT_DMT_GOSSIP_STONE_FAIRY,
    RHT_DMT_GOSSIP_STONE_FAIRY_BIG,
    RHT_COLOSSUS_GOSSIP_STONE_FAIRY,
    RHT_COLOSSUS_GOSSIP_STONE_FAIRY_BIG,
    RHT_DODONGOS_CAVERN_GOSSIP_STONE_FAIRY,
    RHT_DODONGOS_CAVERN_GOSSIP_STONE_FAIRY_BIG,
    RHT_GV_GOSSIP_STONE_FAIRY,
    RHT_GV_GOSSIP_STONE_FAIRY_BIG,
    RHT_GC_MAZE_GOSSIP_STONE_FAIRY,
    RHT_GC_MAZE_GOSSIP_STONE_FAIRY_BIG,
    RHT_GC_MEDIGORON_GOSSIP_STONE_FAIRY,
    RHT_GC_MEDIGORON_GOSSIP_STONE_FAIRY_BIG,
    RHT_GRAVEYARD_GOSSIP_STONE_FAIRY,
    RHT_GRAVEYARD_GOSSIP_STONE_FAIRY_BIG,
    RHT_HC_MALON_GOSSIP_STONE_FAIRY,
    RHT_HC_MALON_GOSSIP_STONE_FAIRY_BIG,
    RHT_HC_ROCK_WALL_GOSSIP_STONE_FAIRY,
    RHT_HC_ROCK_WALL_GOSSIP_STONE_FAIRY_BIG,
    RHT_HC_STORMS_GROTTO_GOSSIP_STONE_FAIRY,
    RHT_HC_STORMS_GROTTO_GOSSIP_STONE_FAIRY_BIG,
    RHT_KF_DEKU_TREE_GOSSIP_STONE_FAIRY,
    RHT_KF_DEKU_TREE_GOSSIP_STONE_FAIRY_BIG,
    RHT_KF_GOSSIP_STONE_FAIRY,
    RHT_KF_GOSSIP_STONE_FAIRY_BIG,
    RHT_KF_STORMS_GROTTO_GOSSIP_STONE_FAIRY,
    RHT_KF_STORMS_GROTTO_GOSSIP_STONE_FAIRY_BIG,
    RHT_LH_LAB_GOSSIP_STONE_FAIRY,
    RHT_LH_LAB_GOSSIP_STONE_FAIRY_BIG,
    RHT_LH_SOUTHEAST_GOSSIP_STONE_FAIRY,
    RHT_LH_SOUTHEAST_GOSSIP_STONE_FAIRY_BIG,
    RHT_LH_SOUTHWEST_GOSSIP_STONE_FAIRY,
    RHT_LH_SOUTHWEST_GOSSIP_STONE_FAIRY_BIG,
    RHT_LW_GOSSIP_STONE_FAIRY,
    RHT_LW_GOSSIP_STONE_FAIRY_BIG,
    RHT_SFM_MAZE_GOSSIP_STONE_FAIRY,
    RHT_SFM_MAZE_GOSSIP_STONE_FAIRY_BIG,
    RHT_SFM_SARIA_GOSSIP_STONE_FAIRY,
    RHT_SFM_SARIA_GOSSIP_STONE_FAIRY_BIG,
    RHT_ZD_GOSSIP_STONE_FAIRY,
    RHT_ZD_GOSSIP_STONE_FAIRY_BIG,
    RHT_ZF_FAIRY_GOSSIP_STONE_FAIRY,
    RHT_ZF_FAIRY_GOSSIP_STONE_FAIRY_BIG,
    RHT_ZF_JABU_GOSSIP_STONE_FAIRY,
    RHT_ZF_JABU_GOSSIP_STONE_FAIRY_BIG,
    RHT_ZR_NEAR_GROTTOS_GOSSIP_STONE_FAIRY,
    RHT_ZR_NEAR_GROTTOS_GOSSIP_STONE_FAIRY_BIG,
    RHT_ZR_NEAR_DOMAIN_GOSSIP_STONE_FAIRY,
    RHT_ZR_NEAR_DOMAIN_GOSSIP_STONE_FAIRY_BIG,
    RHT_HF_COW_GROTTO_GOSSIP_STONE_FAIRY,
    RHT_HF_COW_GROTTO_GOSSIP_STONE_FAIRY_BIG,
    RHT_HF_NEAR_MARKET_GROTTO_GOSSIP_STONE_FAIRY,
    RHT_HF_NEAR_MARKET_GROTTO_GOSSIP_STONE_FAIRY_BIG,
    RHT_HF_SOUTHEAST_GROTTO_GOSSIP_STONE_FAIRY,
    RHT_HF_SOUTHEAST_GROTTO_GOSSIP_STONE_FAIRY_BIG,
    RHT_HF_OPEN_GROTTO_GOSSIP_STONE_FAIRY,
    RHT_HF_OPEN_GROTTO_GOSSIP_STONE_FAIRY_BIG,
    RHT_KAK_OPEN_GROTTO_GOSSIP_STONE_FAIRY,
    RHT_KAK_OPEN_GROTTO_GOSSIP_STONE_FAIRY_BIG,
    RHT_ZR_OPEN_GROTTO_GOSSIP_STONE_FAIRY,
    RHT_ZR_OPEN_GROTTO_GOSSIP_STONE_FAIRY_BIG,
    RHT_LW_NEAR_SHORTCUTS_GROTTO_GOSSIP_STONE_FAIRY,
    RHT_LW_NEAR_SHORTCUTS_GROTTO_GOSSIP_STONE_FAIRY_BIG,
    RHT_DMT_STORMS_GROTTO_GOSSIP_STONE_FAIRY,
    RHT_DMT_STORMS_GROTTO_GOSSIP_STONE_FAIRY_BIG,
    RHT_DMC_UPPER_GROTTO_GOSSIP_STONE_FAIRY,
    RHT_DMC_UPPER_GROTTO_GOSSIP_STONE_FAIRY_BIG,
    RHT_LH_ISLAND_SUN_FAIRY,
    RHT_HF_POND_STORMS_FAIRY,
    RHT_HF_FENCE_GROTTO_STORMS_FAIRY,
    RHT_DMT_FLAG_SUN_FAIRY,
    RHT_DMT_COW_GROTTO_STORMS_FAIRY,
    RHT_LW_SHORTCUT_STORMS_FAIRY,
    RHT_GF_KITCHEN_SUN_FAIRY,
    RHT_LW_DEKU_SCRUB_GROTTO_SUN_FAIRY,
    RHT_GRAVEYARD_ROYAL_FAMILYS_TOMB_SUN_FAIRY,
    RHT_SPIRIT_TEMPLE_BOULDER_ROOM_SUN_FAIRY,
    RHT_SPIRIT_TEMPLE_ARMOS_ROOM_SUN_FAIRY,
    RHT_SHADOW_TEMPLE_BEAMOS_STORM_FAIRY,
    RHT_SHADOW_TEMPLE_PIT_STORM_FAIRY,
    RHT_SHADOW_TEMPLE_WIND_HINT_SUN_FAIRY,
    RHT_BOTTOM_OF_THE_WELL_BASEMENT_SUN_FAIRY,
    RHT_ICE_CAVERN_ENTRANCE_STORMS_FAIRY,
    RHT_GERUDO_TRAINING_GROUND_ENTRANCE_STORMS_FAIRY,
    RHT_GANONS_CASTLE_SPIRIT_TRIAL_SUN_FAIRY,
    RHT_FIRE_TEMPLE_MQ_LOOP_STALFOS_SUN_FAIRY,
    RHT_FIRE_TEMPLE_MQ_LOOP_KNUCKLE_SUN_FAIRY,
    RHT_WATER_TEMPLE_MQ_DARK_LINK_PILAR_SUN_FAIRY,
    RHT_WATER_TEMPLE_MQ_DARK_LINK_LEFT_STORM_FAIRY,
    RHT_WATER_TEMPLE_MQ_DARK_LINK_RIGHT_SUN_FAIRY,
    RHT_SPIRIT_TEMPLE_MQ_DINALFOS_ROOM_SUN_FAIRY,
    RHT_BOTTOM_OF_THE_WELL_MQ_CELL_SUN_FAIRY,
    RHT_BOTTOM_OF_THE_WELL_MQ_BASEMENT_SUN_FAIRY,
    // GRASS
    RHT_KF_GRASS,
    RHT_LW_GRASS,
    RHT_MARKET_GRASS,
    RHT_HC_GRASS,
    RHT_KAK_GRASS,
    RHT_GY_GRASS,
    RHT_LH_GRASS,
    RHT_HF_GRASS,
    RHT_ZR_GRASS,
    RHT_GROTTO_GRASS,
    RHT_DEKU_TREE_GRASS,
    RHT_DODONGOS_CAVERN_GRASS,
    RHT_BOTTOM_OF_THE_WELL_GRASS,
    RHT_JABU_JABUS_BELLY_GRASS,
    // MAX
    RHT_MAX,
} RandomizerHintTextKey;

typedef struct {
    RandomizerGet rgID;
    RandomizerGet fakeRgID;
    char trickName[MAX_TRICK_NAME_SIZE];
} RandomizerGetData;

typedef enum {
    RSG_NONE,
    RSG_LOGIC,
    RSG_EXCLUDES_KOKIRI_FOREST,
    RSG_EXCLUDES_LOST_WOODS,
    RSG_EXCLUDES_SACRED_FOREST_MEADOW,
    RSG_EXCLUDES_DEKU_TREE,
    RSG_EXCLUDES_FOREST_TEMPLE,
    RSG_EXCLUDES_KAKARIKO_VILLAGE,
    RSG_EXCLUDES_GRAVEYARD,
    RSG_EXCLUDES_BOTTOM_OF_THE_WELL,
    RSG_EXCLUDES_SHADOW_TEMPLE,
    RSG_EXCLUDES_DEATH_MOUNTAIN_TRAIL,
    RSG_EXCLUDES_DEATH_MOUNTAIN_CRATER,
    RSG_EXCLUDES_GORON_CITY,
    RSG_EXCLUDES_DODONGOS_CAVERN,
    RSG_EXCLUDES_FIRE_TEMPLE,
    RSG_EXCLUDES_ZORAS_RIVER,
    RSG_EXCLUDES_ZORAS_DOMAIN,
    RSG_EXCLUDES_ZORAS_FOUNTAIN,
    RSG_EXCLUDES_JABU_JABU,
    RSG_EXCLUDES_ICE_CAVERN,
    RSG_EXCLUDES_HYRULE_FIELD,
    RSG_EXCLUDES_LON_LON_RANCH,
    RSG_EXCLUDES_LAKE_HYLIA,
    RSG_EXCLUDES_WATER_TEMPLE,
    RSG_EXCLUDES_GERUDO_VALLEY,
    RSG_EXCLUDES_GERUDO_FORTRESS,
    RSG_EXCLUDES_HAUNTED_WASTELAND,
    RSG_EXCLUDES_DESERT_COLOSSUS,
    RSG_EXCLUDES_GERUDO_TRAINING_GROUND,
    RSG_EXCLUDES_SPIRIT_TEMPLE,
    RSG_EXCLUDES_HYRULE_CASTLE,
    RSG_EXCLUDES_MARKET,
    RSG_EXCLUDES_GANONS_CASTLE,
    RSG_EXCLUDES,
    RSG_TRICKS,
    RSG_GLITCHES,
    RSG_AREA_ACCESS_IMGUI,
    RSG_WORLD_IMGUI,
    RSG_SHUFFLE_ENTRANCES_IMGUI,
    RSG_WORLD_IMGUI_TABLE,
    RSG_SHUFFLE_ITEMS_IMGUI,
    RSG_SHUFFLE_NPCS_IMGUI,
    RSG_SHUFFLE_DUNGEON_ITEMS_IMGUI,
    RSG_ITEMS_IMGUI_TABLE,
    RSG_TIMESAVERS_IMGUI,
    RSG_ITEM_POOL_HINTS_IMGUI,
    RSG_EXTRA_HINTS_IMGUI,
    RSG_ITEM_POOL_HINTS_IMGUI_COLUMN,
    RSG_ADDITIONAL_FEATURES_IMGUI,
    RSG_GAMEPLAY_IMGUI_TABLE,
    RSG_STARTING_EQUIPMENT_IMGUI,
    RSG_STARTING_ITEMS_IMGUI,
    RSG_STARTING_NORMAL_SONGS_IMGUI,
    RSG_STARTING_WARP_SONGS_IMGUI,
    RSG_STARTING_SONGS_IMGUI,
    RSG_STARTING_INVENTORY_IMGUI_TABLE,
    RSG_OPEN,
    RSG_WORLD,
    RSG_SHUFFLE,
    RSG_SHUFFLE_DUNGEON_ITEMS,
    RSG_SHUFFLE_DUNGEON_QUESTS,
    RSG_DETAILED_LOGIC,
    RSG_STARTING_ITEMS,
    RSG_STARTING_SONGS,
    RSG_STARTING_OTHER,
    RSG_STARTING_INVENTORY,
    RSG_TIMESAVERS,
    RSG_MISC,
    RSG_ITEM_POOL,
    RSG_MAX,
} RandomizerSettingGroupKey;

typedef enum {
    RSK_NONE,
    RSK_LOGIC_RULES,
    RSK_FOREST,
    RSK_KAK_GATE,
    RSK_DOOR_OF_TIME,
    RSK_ZORAS_FOUNTAIN,
    RSK_SLEEPING_WATERFALL,
    RSK_STARTING_AGE,
    RSK_SELECTED_STARTING_AGE,
    RSK_GERUDO_FORTRESS,
    RSK_RAINBOW_BRIDGE,
    RSK_RAINBOW_BRIDGE_STONE_COUNT,
    RSK_RAINBOW_BRIDGE_MEDALLION_COUNT,
    RSK_RAINBOW_BRIDGE_REWARD_COUNT,
    RSK_RAINBOW_BRIDGE_DUNGEON_COUNT,
    RSK_RAINBOW_BRIDGE_TOKEN_COUNT,
    RSK_BRIDGE_OPTIONS,
    RSK_GANONS_TRIALS,
    RSK_TRIAL_COUNT,
    RSK_STARTING_OCARINA,
    RSK_SHUFFLE_OCARINA,
    RSK_SHUFFLE_OCARINA_BUTTONS,
    RSK_SHUFFLE_SWIM,
    RSK_STARTING_DEKU_SHIELD,
    RSK_STARTING_KOKIRI_SWORD,
    RSK_STARTING_MASTER_SWORD,
    RSK_STARTING_ZELDAS_LULLABY,
    RSK_STARTING_EPONAS_SONG,
    RSK_STARTING_SARIAS_SONG,
    RSK_STARTING_SUNS_SONG,
    RSK_STARTING_SONG_OF_TIME,
    RSK_STARTING_SONG_OF_STORMS,
    RSK_STARTING_MINUET_OF_FOREST,
    RSK_STARTING_BOLERO_OF_FIRE,
    RSK_STARTING_SERENADE_OF_WATER,
    RSK_STARTING_REQUIEM_OF_SPIRIT,
    RSK_STARTING_NOCTURNE_OF_SHADOW,
    RSK_STARTING_PRELUDE_OF_LIGHT,
    RSK_SHUFFLE_KOKIRI_SWORD,
    RSK_SHUFFLE_MASTER_SWORD,
    RSK_SHUFFLE_CHILD_WALLET,
    RSK_INCLUDE_TYCOON_WALLET,
    RSK_SHUFFLE_DUNGEON_REWARDS,
    RSK_SHUFFLE_SONGS,
    RSK_SHUFFLE_TOKENS,
    RSK_SHOPSANITY,
    RSK_SHOPSANITY_COUNT,
    RSK_SHOPSANITY_PRICES,
    RSK_SHOPSANITY_PRICES_FIXED_PRICE,
    RSK_SHOPSANITY_PRICES_RANGE_1,
    RSK_SHOPSANITY_PRICES_RANGE_2,
    RSK_SHOPSANITY_PRICES_NO_WALLET_WEIGHT,
    RSK_SHOPSANITY_PRICES_CHILD_WALLET_WEIGHT,
    RSK_SHOPSANITY_PRICES_ADULT_WALLET_WEIGHT,
    RSK_SHOPSANITY_PRICES_GIANT_WALLET_WEIGHT,
    RSK_SHOPSANITY_PRICES_TYCOON_WALLET_WEIGHT,
    RSK_SHOPSANITY_PRICES_AFFORDABLE,
    RSK_SHUFFLE_SCRUBS,
    RSK_SCRUBS_PRICES,
    RSK_SCRUBS_PRICES_FIXED_PRICE,
    RSK_SCRUBS_PRICES_RANGE_1,
    RSK_SCRUBS_PRICES_RANGE_2,
    RSK_SCRUBS_PRICES_NO_WALLET_WEIGHT,
    RSK_SCRUBS_PRICES_CHILD_WALLET_WEIGHT,
    RSK_SCRUBS_PRICES_ADULT_WALLET_WEIGHT,
    RSK_SCRUBS_PRICES_GIANT_WALLET_WEIGHT,
    RSK_SCRUBS_PRICES_TYCOON_WALLET_WEIGHT,
    RSK_SCRUBS_PRICES_AFFORDABLE,
    RSK_SHUFFLE_BEEHIVES,
    RSK_SHUFFLE_COWS,
    RSK_SHUFFLE_WEIRD_EGG,
    RSK_SHUFFLE_GERUDO_MEMBERSHIP_CARD,
    RSK_SHUFFLE_POTS,
    RSK_SHUFFLE_CRATES,
    RSK_SHUFFLE_FROG_SONG_RUPEES,
    RSK_ITEM_POOL,
    RSK_ICE_TRAPS,
    RSK_GOSSIP_STONE_HINTS,
    RSK_TOT_ALTAR_HINT,
    RSK_GANONDORF_HINT,
    RSK_SHEIK_LA_HINT,
    RSK_DAMPES_DIARY_HINT,
    RSK_GREG_HINT,
    RSK_LOACH_HINT,
    RSK_SARIA_HINT,
    RSK_FROGS_HINT,
    RSK_OOT_HINT,
    RSK_KAK_10_SKULLS_HINT,
    RSK_KAK_20_SKULLS_HINT,
    RSK_KAK_30_SKULLS_HINT,
    RSK_KAK_40_SKULLS_HINT,
    RSK_KAK_50_SKULLS_HINT,
    RSK_KAK_100_SKULLS_HINT,
    RSK_MASK_SHOP_HINT,
    RSK_BIGGORON_HINT,
    RSK_BIG_POES_HINT,
    RSK_CHICKENS_HINT,
    RSK_MALON_HINT,
    RSK_HBA_HINT,
    RSK_WARP_SONG_HINTS,
    RSK_SCRUB_TEXT_HINT,
    RSK_MERCHANT_TEXT_HINT,
    RSK_FISHING_POLE_HINT,
    RSK_HINT_CLARITY,
    RSK_HINT_DISTRIBUTION,
    RSK_SHUFFLE_MAPANDCOMPASS,
    RSK_KEYSANITY,
    RSK_GERUDO_KEYS,
    RSK_BOSS_KEYSANITY,
    RSK_GANONS_BOSS_KEY,
    RSK_SKIP_CHILD_STEALTH,
    RSK_SKIP_CHILD_ZELDA,
    RSK_STARTING_STICKS,
    RSK_STARTING_NUTS,
    RSK_FULL_WALLETS,
    RSK_SHUFFLE_CHEST_MINIGAME,
    RSK_CUCCO_COUNT,
    RSK_BIG_POE_COUNT,
    RSK_SKIP_EPONA_RACE,
    RSK_COMPLETE_MASK_QUEST,
    RSK_SKIP_SCARECROWS_SONG,
    RSK_SKULLS_SUNS_SONG,
    RSK_SHUFFLE_ADULT_TRADE,
    RSK_SHUFFLE_MERCHANTS,
    RSK_MERCHANT_PRICES,
    RSK_MERCHANT_PRICES_FIXED_PRICE,
    RSK_MERCHANT_PRICES_RANGE_1,
    RSK_MERCHANT_PRICES_RANGE_2,
    RSK_MERCHANT_PRICES_NO_WALLET_WEIGHT,
    RSK_MERCHANT_PRICES_CHILD_WALLET_WEIGHT,
    RSK_MERCHANT_PRICES_ADULT_WALLET_WEIGHT,
    RSK_MERCHANT_PRICES_GIANT_WALLET_WEIGHT,
    RSK_MERCHANT_PRICES_TYCOON_WALLET_WEIGHT,
    RSK_MERCHANT_PRICES_AFFORDABLE,
    RSK_BLUE_FIRE_ARROWS,
    RSK_SUNLIGHT_ARROWS,
    RSK_ENABLE_BOMBCHU_DROPS,
    RSK_BOMBCHU_BAG,
    RSK_LINKS_POCKET,
    RSK_MQ_DUNGEON_RANDOM,
    RSK_MQ_DUNGEON_COUNT,
    RSK_MQ_DUNGEON_SET,
    RSK_MQ_DEKU_TREE,
    RSK_MQ_DODONGOS_CAVERN,
    RSK_MQ_JABU_JABU,
    RSK_MQ_FOREST_TEMPLE,
    RSK_MQ_FIRE_TEMPLE,
    RSK_MQ_WATER_TEMPLE,
    RSK_MQ_SPIRIT_TEMPLE,
    RSK_MQ_SHADOW_TEMPLE,
    RSK_MQ_BOTTOM_OF_THE_WELL,
    RSK_MQ_ICE_CAVERN,
    RSK_MQ_GTG,
    RSK_MQ_GANONS_CASTLE,
    RSK_LACS_STONE_COUNT,
    RSK_LACS_MEDALLION_COUNT,
    RSK_LACS_REWARD_COUNT,
    RSK_LACS_DUNGEON_COUNT,
    RSK_LACS_TOKEN_COUNT,
    RSK_LACS_OPTIONS,
    RSK_KEYRINGS,
    RSK_KEYRINGS_RANDOM_COUNT,
    RSK_KEYRINGS_GERUDO_FORTRESS,
    RSK_KEYRINGS_FOREST_TEMPLE,
    RSK_KEYRINGS_FIRE_TEMPLE,
    RSK_KEYRINGS_WATER_TEMPLE,
    RSK_KEYRINGS_SPIRIT_TEMPLE,
    RSK_KEYRINGS_SHADOW_TEMPLE,
    RSK_KEYRINGS_BOTTOM_OF_THE_WELL,
    RSK_KEYRINGS_GTG,
    RSK_KEYRINGS_GANONS_CASTLE,
    RSK_SHUFFLE_ENTRANCES,
    RSK_SHUFFLE_DUNGEON_ENTRANCES,
    RSK_SHUFFLE_OVERWORLD_ENTRANCES,
    RSK_SHUFFLE_INTERIOR_ENTRANCES,
    RSK_SHUFFLE_GROTTO_ENTRANCES,
    RSK_SHUFFLE_OWL_DROPS,
    RSK_SHUFFLE_WARP_SONGS,
    RSK_SHUFFLE_OVERWORLD_SPAWNS,
    RSK_MIXED_ENTRANCE_POOLS,
    RSK_MIX_DUNGEON_ENTRANCES,
    RSK_MIX_BOSS_ENTRANCES,
    RSK_MIX_OVERWORLD_ENTRANCES,
    RSK_MIX_INTERIOR_ENTRANCES,
    RSK_MIX_GROTTO_ENTRANCES,
    RSK_DECOUPLED_ENTRANCES,
    RSK_STARTING_SKULLTULA_TOKEN,
    RSK_STARTING_HEARTS,
    RSK_DAMAGE_MULTIPLIER,
    RSK_ALL_LOCATIONS_REACHABLE,
    RSK_SHUFFLE_BOSS_ENTRANCES,
    RSK_SHUFFLE_100_GS_REWARD,
    RSK_TRIFORCE_HUNT,
    RSK_TRIFORCE_HUNT_PIECES_TOTAL,
    RSK_TRIFORCE_HUNT_PIECES_REQUIRED,
    RSK_SHUFFLE_BOSS_SOULS,
    RSK_FISHSANITY,
    RSK_FISHSANITY_POND_COUNT,
    RSK_FISHSANITY_AGE_SPLIT,
    RSK_SHUFFLE_FISHING_POLE,
    RSK_INFINITE_UPGRADES,
    RSK_SKELETON_KEY,
    RSK_SHUFFLE_DEKU_STICK_BAG,
    RSK_SHUFFLE_DEKU_NUT_BAG,
    RSK_SHUFFLE_FREESTANDING,
    RSK_SHUFFLE_FAIRIES,
    RSK_LOCK_OVERWORLD_DOORS,
    RSK_SHUFFLE_GRASS,
    RSK_MAX
} RandomizerSettingKey;

// Generic Settings (any binary option can use this)
//  off/on
typedef enum {
    RO_GENERIC_OFF,
    RO_GENERIC_ON,
} RandoOptionGenericOffOn;

// no/yes
typedef enum {
    RO_GENERIC_NO,
    RO_GENERIC_YES,
} RandoOptionGenericNoYes;

// don't skip/skip
typedef enum {
    RO_GENERIC_DONT_SKIP,
    RO_GENERIC_SKIP,
} RandoOptionGenericSkip;

// Closed Forest settings (On, Deku Only, Off)
typedef enum {
    RO_CLOSED_FOREST_ON,
    RO_CLOSED_FOREST_DEKU_ONLY,
    RO_CLOSED_FOREST_OFF,
} RandoOptionForest;

// Door of Time settings (closed, song only, open)
typedef enum {
    RO_DOOROFTIME_CLOSED,
    RO_DOOROFTIME_SONGONLY,
    RO_DOOROFTIME_OPEN,
} RandoOptionDoorOfTime;

// Zora's Fountain settings (closed, closed as child, open)
typedef enum {
    RO_ZF_CLOSED,
    RO_ZF_CLOSED_CHILD,
    RO_ZF_OPEN,
} RandoOptionZorasFountain;

// Sleeping Waterfall settings (closed, open)
typedef enum {
    RO_WATERFALL_CLOSED,
    RO_WATERFALL_OPEN,
} RandoOptionSleepingWaterfall;

// Starting Age settings (child, adult, random)
typedef enum {
    RO_AGE_CHILD,
    RO_AGE_ADULT,
    RO_AGE_RANDOM,
} RandoOptionStartingAge;

// Fortress Carpenters settings (normal, fast, free)
typedef enum {
    RO_GF_CARPENTERS_NORMAL,
    RO_GF_CARPENTERS_FAST,
    RO_GF_CARPENTERS_FREE,
} RandoOptionGerudoFortress;

// Kakariko Gate settings (closed/open)
typedef enum {
    RO_KAK_GATE_CLOSED,
    RO_KAK_GATE_OPEN,
} RandoOptionKakarikoGate;

// Rainbow Bridge settings (vanilla, always open, stones, medallions, dungeon rewards, dungeons, tokens)
typedef enum {
    RO_BRIDGE_VANILLA = 0,
    RO_BRIDGE_ALWAYS_OPEN,
    RO_BRIDGE_STONES,
    RO_BRIDGE_MEDALLIONS,
    RO_BRIDGE_DUNGEON_REWARDS,
    RO_BRIDGE_DUNGEONS,
    RO_BRIDGE_TOKENS,
    RO_BRIDGE_GREG,
} RandoOptionRainbowBridge;

// Bridge Reward Options settings (Standard rewards, Greg as reward, Greg as wildcard)
typedef enum {
    RO_BRIDGE_STANDARD_REWARD,
    RO_BRIDGE_GREG_REWARD,
    RO_BRIDGE_WILDCARD_REWARD,
} RandoOptionBridgeRewards;

// Shopsanity settings (off, specific count, random)
typedef enum {
    RO_SHOPSANITY_OFF,
    RO_SHOPSANITY_SPECIFIC_COUNT,
    RO_SHOPSANITY_RANDOM,
} RandoOptionShopsanity;

// Shopsanity count settings (0-7 items)
typedef enum {
    RO_SHOPSANITY_COUNT_ZERO_ITEMS,
    RO_SHOPSANITY_COUNT_ONE_ITEM,
    RO_SHOPSANITY_COUNT_TWO_ITEMS,
    RO_SHOPSANITY_COUNT_THREE_ITEMS,
    RO_SHOPSANITY_COUNT_FOUR_ITEMS,
    RO_SHOPSANITY_COUNT_FIVE_ITEMS,
    RO_SHOPSANITY_COUNT_SIX_ITEMS,
    RO_SHOPSANITY_COUNT_SEVEN_ITEMS,
    RO_SHOPSANITY_COUNT_EIGHT_ITEMS,
} RandoOptionShopsanityCount;

// Shopsanity price ranges
typedef enum {
    RO_PRICE_VANILLA,
    RO_PRICE_CHEAP_BALANCED, // Balanced random from 0-95, favoring lower numbers
    RO_PRICE_BALANCED,       // Random from 0-300, favoring lower numbers
    RO_PRICE_FIXED,
    RO_PRICE_RANGE,
    RO_PRICE_SET_BY_WALLET,
} RandoOptionPrices;

// Scrubsanity settings (off, affordable, expensive, random)
typedef enum {
    RO_SCRUBS_OFF,
    RO_SCRUBS_ONE_TIME_ONLY,
    RO_SCRUBS_ALL,
} RandoOptionScrubsanity;

// Ammo drop settings (on, "on+bombchu", off)
typedef enum {
    RO_AMMO_DROPS_OFF,
    // RO_AMMO_DROPS_ON_PLUS_BOMBCHU,
    RO_AMMO_DROPS_ON,
} RandoOptionAmmoDrops;

typedef enum {
    RO_BOSS_SOULS_OFF,
    RO_BOSS_SOULS_ON,
    RO_BOSS_SOULS_ON_PLUS_GANON,
} RandoOptionBossSouls;

// Fishsanity settings (off, loach only, pond only, grottos only, both)
typedef enum {
    RO_FISHSANITY_OFF,
    RO_FISHSANITY_HYRULE_LOACH,
    RO_FISHSANITY_POND,
    RO_FISHSANITY_OVERWORLD,
    RO_FISHSANITY_BOTH
} RandoOptionsFishsanity;

// Infinite Upgrades settings (off, progressive, condensed progressive)
typedef enum {
    RO_INF_UPGRADES_OFF,
    RO_INF_UPGRADES_PROGRESSIVE,
    RO_INF_UPGRADES_CONDENSED_PROGRESSIVE,
} RandoOptionInfiniteUpgrades;

// Any Dungeon Item (start with, vanilla, own dungeon, any dungeon,
// overworld, anywhere)
typedef enum {
    RO_DUNGEON_ITEM_LOC_STARTWITH,
    RO_DUNGEON_ITEM_LOC_VANILLA,
    RO_DUNGEON_ITEM_LOC_OWN_DUNGEON,
    RO_DUNGEON_ITEM_LOC_ANY_DUNGEON,
    RO_DUNGEON_ITEM_LOC_OVERWORLD,
    RO_DUNGEON_ITEM_LOC_ANYWHERE,
} RandoOptionDungeonItemLocation;

// Dungeon reward settings
typedef enum {
    RO_DUNGEON_REWARDS_VANILLA,
    RO_DUNGEON_REWARDS_END_OF_DUNGEON,
    RO_DUNGEON_REWARDS_ANY_DUNGEON,
    RO_DUNGEON_REWARDS_OVERWORLD,
    RO_DUNGEON_REWARDS_ANYWHERE,
} RandoOptionDungeonRewards;

// Keyring Settings
typedef enum {
    RO_KEYRINGS_OFF,
    RO_KEYRINGS_RANDOM,
    RO_KEYRINGS_COUNT,
    RO_KEYRINGS_SELECTION,
} RandoOptionKeyrings;

typedef enum {
    RO_KEYRING_FOR_DUNGEON_OFF,
    RO_KEYRING_FOR_DUNGEON_RANDOM,
    RO_KEYRING_FOR_DUNGEON_ON,
} RandoOptionKeyringForDungeon;

// Ganon's Boss Key Settings (vanilla, own dungeon, start with,
// overworld, anywhere, 100 GS reward)
typedef enum {
    RO_GANON_BOSS_KEY_VANILLA,
    RO_GANON_BOSS_KEY_OWN_DUNGEON,
    RO_GANON_BOSS_KEY_STARTWITH,
    RO_GANON_BOSS_KEY_ANY_DUNGEON,
    RO_GANON_BOSS_KEY_OVERWORLD,
    RO_GANON_BOSS_KEY_ANYWHERE,
    RO_GANON_BOSS_KEY_LACS_VANILLA,
    RO_GANON_BOSS_KEY_LACS_STONES,
    RO_GANON_BOSS_KEY_LACS_MEDALLIONS,
    RO_GANON_BOSS_KEY_LACS_REWARDS,
    RO_GANON_BOSS_KEY_LACS_DUNGEONS,
    RO_GANON_BOSS_KEY_LACS_TOKENS,
    RO_GANON_BOSS_KEY_KAK_TOKENS,
} RandoOptionGanonsBossKey;

typedef enum {
    RO_LACS_VANILLA,
    RO_LACS_STONES,
    RO_LACS_MEDALLIONS,
    RO_LACS_REWARDS,
    RO_LACS_DUNGEONS,
    RO_LACS_TOKENS,
} RandoOptionLACSCondition;

// LACS Reward Options settings (Standard rewards, Greg as reward, Greg as wildcard)
typedef enum {
    RO_LACS_STANDARD_REWARD,
    RO_LACS_GREG_REWARD,
    RO_LACS_WILDCARD_REWARD,
} RandoOptionLACSRewards;

// Ganon's Trials
typedef enum {
    RO_GANONS_TRIALS_SKIP,
    RO_GANONS_TRIALS_SET_NUMBER,
    RO_GANONS_TRIALS_RANDOM_NUMBER,
} RandoOptionGanonsTrials;

// Shuffle Dungeon Entrance Settings (Off, on, plus ganon)
typedef enum {
    RO_DUNGEON_ENTRANCE_SHUFFLE_OFF,
    RO_DUNGEON_ENTRANCE_SHUFFLE_ON,
    RO_DUNGEON_ENTRANCE_SHUFFLE_ON_PLUS_GANON,
} RandoOptionDungeonEntranceShuffle;

// Shuffle Boss Room Entrance Settings (Off, Age Restricted, Full)
typedef enum {
    RO_BOSS_ROOM_ENTRANCE_SHUFFLE_OFF,
    RO_BOSS_ROOM_ENTRANCE_SHUFFLE_AGE_RESTRICTED,
    RO_BOSS_ROOM_ENTRANCE_SHUFFLE_FULL,
} RandoOptionBossRoomEntranceShuffle;

// Shuffle Interior Entrance Settings (Off, simple, all)
typedef enum {
    RO_INTERIOR_ENTRANCE_SHUFFLE_OFF,
    RO_INTERIOR_ENTRANCE_SHUFFLE_SIMPLE,
    RO_INTERIOR_ENTRANCE_SHUFFLE_ALL,
} RandoOptionInteriorEntranceShuffle;

// Song shuffle Settings (Song locations, Dungeon rewards, anywhere)
typedef enum {
    RO_SONG_SHUFFLE_SONG_LOCATIONS,
    RO_SONG_SHUFFLE_DUNGEON_REWARDS,
    RO_SONG_SHUFFLE_ANYWHERE,
} RandoOptionSongShuffle;

// Shuffle Merchants Settings (Off, Beans Only, All but Beans, All)
typedef enum {
    RO_SHUFFLE_MERCHANTS_OFF,
    RO_SHUFFLE_MERCHANTS_BEANS_ONLY,
    RO_SHUFFLE_MERCHANTS_ALL_BUT_BEANS,
    RO_SHUFFLE_MERCHANTS_ALL
} RandoOptionShuffleMerchants;

// Starting Ocarina Settings (off, fairy)
typedef enum {
    RO_STARTING_OCARINA_OFF,
    RO_STARTING_OCARINA_FAIRY,
    RO_STARTING_OCARINA_TIME,
} RandoOptionStartingOcarina;

// Item Pool Settings
typedef enum {
    RO_ITEM_POOL_PLENTIFUL,
    RO_ITEM_POOL_BALANCED,
    RO_ITEM_POOL_SCARCE,
    RO_ITEM_POOL_MINIMAL,
} RandoOptionItemPool;

// Ice Trap Settings
typedef enum {
    RO_ICE_TRAPS_OFF,
    RO_ICE_TRAPS_NORMAL,
    RO_ICE_TRAPS_EXTRA,
    RO_ICE_TRAPS_MAYHEM,
    RO_ICE_TRAPS_ONSLAUGHT,
} RandoOptionIceTraps;

// Gossip Stone Hint Settings (no hints, needs nothing,
// needs mask of truth, needs stone of agony)
typedef enum {
    RO_GOSSIP_STONES_NONE,
    RO_GOSSIP_STONES_NEED_NOTHING,
    RO_GOSSIP_STONES_NEED_TRUTH,
    RO_GOSSIP_STONES_NEED_STONE,
} RandoOptionGossipStones;

// Hint Clarity Settings (obscure, ambiguous, clear)
typedef enum {
    RO_HINT_CLARITY_OBSCURE,
    RO_HINT_CLARITY_AMBIGUOUS,
    RO_HINT_CLARITY_CLEAR,
} RandoOptionHintClarity;

// Hint Distribution Settings (useless, balanced, strong, very strong)
typedef enum {
    RO_HINT_DIST_USELESS,
    RO_HINT_DIST_BALANCED,
    RO_HINT_DIST_STRONG,
    RO_HINT_DIST_VERY_STRONG,
} RandoOptionHintDistribution;

// Gerudo Fortress Key Settings (vanilla, any dungeon, overworld,
// anywhere)
typedef enum {
    RO_GERUDO_KEYS_VANILLA,
    RO_GERUDO_KEYS_ANY_DUNGEON,
    RO_GERUDO_KEYS_OVERWORLD,
    RO_GERUDO_KEYS_ANYWHERE,
} RandoOptionGerudoKeys;

// Tokensanity settings (off, dungeons, overworld, all)
typedef enum {
    RO_TOKENSANITY_OFF,
    RO_TOKENSANITY_DUNGEONS,
    RO_TOKENSANITY_OVERWORLD,
    RO_TOKENSANITY_ALL,
} RandoOptionTokensanity;

// Freestanding Hearts/Rupees settings (off, dungeons, overworld, all)
typedef enum {
    RO_SHUFFLE_FREESTANDING_OFF,
    RO_SHUFFLE_FREESTANDING_DUNGEONS,
    RO_SHUFFLE_FREESTANDING_OVERWORLD,
    RO_SHUFFLE_FREESTANDING_ALL,
} RandoOptionFreestanding;

// Shuffle Pots settings (off, dungeons, overworld, all)
typedef enum {
    RO_SHUFFLE_POTS_OFF,
    RO_SHUFFLE_POTS_DUNGEONS,
    RO_SHUFFLE_POTS_OVERWORLD,
    RO_SHUFFLE_POTS_ALL,
} RandoOptionShufflePots;

// Shuffle Grass settings (off, dungeons, overworld, all)
typedef enum {
    RO_SHUFFLE_GRASS_OFF,
    RO_SHUFFLE_GRASS_DUNGEONS,
    RO_SHUFFLE_GRASS_OVERWORLD,
    RO_SHUFFLE_GRASS_ALL,
} RandoOptionShuffleGrass;

// Shuffle Crates settings (off, dungeons, overworld, all)
typedef enum {
    RO_SHUFFLE_CRATES_OFF,
    RO_SHUFFLE_CRATES_DUNGEONS,
    RO_SHUFFLE_CRATES_OVERWORLD,
    RO_SHUFFLE_CRATES_ALL,
} RandoOptionShuffleCrates;

// Link's Pocket Settings (dungeon reward, advancement, anything, nothing)
typedef enum {
    RO_LINKS_POCKET_DUNGEON_REWARD,
    RO_LINKS_POCKET_ADVANCEMENT,
    RO_LINKS_POCKET_ANYTHING,
    RO_LINKS_POCKET_NOTHING,
} RandoOptionLinksPocket;

// Logic (glitchless/no logic)
typedef enum {
    RO_LOGIC_GLITCHLESS,
    RO_LOGIC_NO_LOGIC,
    RO_LOGIC_VANILLA,
} RandoOptionLogic;

// Damage Multiplier
typedef enum {
    RO_DAMAGE_MULTIPLIER_HALF,
    RO_DAMAGE_MULTIPLIER_DEFAULT,
    RO_DAMAGE_MULTIPLIER_DOUBLE,
    RO_DAMAGE_MULTIPLIER_QUADRUPLE,
    RO_DAMAGE_MULTIPLIER_OCTUPLE,
    RO_DAMAGE_MULTIPLIER_SEXDECUPLE,
    RO_DAMAGE_MULTIPLIER_OHKO,
} RandoOptionDamageMultiplier;

// MQ Dungeons
typedef enum {
    RO_MQ_DUNGEONS_NONE,
    RO_MQ_DUNGEONS_SET_NUMBER,
    RO_MQ_DUNGEONS_RANDOM_NUMBER,
    RO_MQ_DUNGEONS_SELECTION,
} RandoOptionMQDungeons;

typedef enum {
    RO_LOCATION_INCLUDE,
    RO_LOCATION_EXCLUDE,
} RandoOptionLocationInclusion;

typedef enum {
    RO_CHEST_GAME_OFF,
    RO_CHEST_GAME_SINGLE_KEYS,
    RO_CHEST_GAME_PACK,
} RandoOptionChestGame;

typedef enum {
    RO_MQ_SET_VANILLA,
    RO_MQ_SET_MQ,
    RO_MQ_SET_RANDOM,
} RandoOptionMQSet;

typedef enum {
    CAN_OBTAIN,
    CANT_OBTAIN_MISC,
    CANT_OBTAIN_ALREADY_HAVE,
    CANT_OBTAIN_NEED_UPGRADE,
    CANT_OBTAIN_NEED_EMPTY_BOTTLE,
} ItemObtainability;

typedef struct BeehiveIdentity {
    RandomizerInf randomizerInf;
    RandomizerCheck randomizerCheck;
} BeehiveIdentity;

typedef struct ScrubIdentity {
    RandomizerInf randomizerInf;
    RandomizerCheck randomizerCheck;
    GetItemID getItemId;
    int32_t itemPrice;
    uint8_t isShuffled;
} ScrubIdentity;

typedef struct ShopItemIdentity {
    RandomizerInf randomizerInf;
    RandomizerCheck randomizerCheck;
    GetItemID ogItemId;
    int32_t enGirlAShopItem;
    int32_t itemPrice;
} ShopItemIdentity;

typedef struct CowIdentity {
    RandomizerInf randomizerInf;
    RandomizerCheck randomizerCheck;
} CowIdentity;

typedef struct PotIdentity {
    RandomizerInf randomizerInf;
    RandomizerCheck randomizerCheck;
} PotIdentity;

typedef struct FishIdentity {
    RandomizerInf randomizerInf;
    RandomizerCheck randomizerCheck;
} FishIdentity;

typedef struct GrassIdentity {
    RandomizerInf randomizerInf;
    RandomizerCheck randomizerCheck;
} GrassIdentity;

typedef struct CrateIdentity {
    RandomizerInf randomizerInf;
    RandomizerCheck randomizerCheck;
} CrateIdentity;

typedef struct SmallCrateIdentity {
    RandomizerInf randomizerInf;
    RandomizerCheck randomizerCheck;
} SmallCrateIdentity;

typedef enum {
    TRACKER_WINDOW_FLOATING,
    TRACKER_WINDOW_WINDOW,
} TrackerWindowType;

typedef enum {
    TRACKER_DISPLAY_ALWAYS,
    TRACKER_DISPLAY_COMBO_BUTTON,
} TrackerDisplayType;

typedef enum {
    TRACKER_COMBO_BUTTON_A,
    TRACKER_COMBO_BUTTON_B,
    TRACKER_COMBO_BUTTON_C_UP,
    TRACKER_COMBO_BUTTON_C_DOWN,
    TRACKER_COMBO_BUTTON_C_LEFT,
    TRACKER_COMBO_BUTTON_C_RIGHT,
    TRACKER_COMBO_BUTTON_L,
    TRACKER_COMBO_BUTTON_Z,
    TRACKER_COMBO_BUTTON_R,
    TRACKER_COMBO_BUTTON_START,
    TRACKER_COMBO_BUTTON_D_UP,
    TRACKER_COMBO_BUTTON_D_DOWN,
    TRACKER_COMBO_BUTTON_D_LEFT,
    TRACKER_COMBO_BUTTON_D_RIGHT,
} TrackerComboButton;

typedef enum {
    TH_MESSAGE_START,
    TH_MESSAGE_PROGRESS,
    TH_MESSAGE_HALFWAY,
    TH_MESSAGE_ALMOSTDONE,
    TH_MESSAGE_FINISHED,
    TH_MESSAGE_SURPLUS,
} TriforceHuntMessages;

typedef enum {
    RE_GOLD_SKULLTULA,
    RE_BIG_SKULLTULA,
    RE_DODONGO,
    RE_LIZALFOS,
    RE_GOHMA_LARVA,
    RE_KEESE,
    RE_FIRE_KEESE,
    RE_MAD_SCRUB,
    RE_BLUE_BUBBLE,
    RE_DEAD_HAND,
    RE_DEKU_BABA,
    RE_WITHERED_DEKU_BABA,
    RE_LIKE_LIKE,
    RE_STALFOS,
    RE_IRON_KNUCKLE,
    RE_FLARE_DANCER,
    RE_WOLFOS,
    RE_FLOORMASTER,
    RE_REDEAD,
    RE_MEG,
    RE_ARMOS,
    RE_GREEN_BUBBLE,
    RE_DINOLFOS,
    RE_TORCH_SLUG,
    RE_FREEZARD,
    RE_SHELL_BLADE,
    RE_SPIKE,
    RE_WHITE_WOLFOS,
    RE_STINGER,
    RE_BIG_OCTO,
    RE_GERUDO_WARRIOR,
    RE_GIBDO,
    RE_GOHMA,
    RE_KING_DODONGO,
    RE_BARINADE,
    RE_PHANTOM_GANON,
    RE_VOLVAGIA,
    RE_MORPHA,
    RE_BONGO_BONGO,
    RE_TWINROVA,
    RE_GANONDORF,
    RE_GANON,
    RE_DARK_LINK,
    RE_ANUBIS,
    RE_BEAMOS,
    RE_WALLMASTER,
    RE_PURPLE_LEEVER,
    RE_TENTACLE,
    RE_BARI,
    RE_SHABOM,
    RE_OCTOROK,
} RandomizerEnemy;

// RANDOTODO compare child long jumpslash range with adult short
typedef enum {
    ED_CLOSE,
    // hammer or kokiri sword
    ED_SHORT_JUMPSLASH,
    ED_MASTER_SWORD_JUMPSLASH,
    // sticks or BGS
    ED_LONG_JUMPSLASH,
    ED_BOMB_THROW,
    ED_BOOMERANG,
    ED_HOOKSHOT,
    ED_LONGSHOT,
    ED_FAR,
} EnemyDistance;

typedef enum {
    WL_LOW,
    WL_MID,
    WL_HIGH,
    WL_LOW_OR_MID,
    WL_HIGH_OR_MID,
} RandoWaterLevel;

#define ENTRANCE_GROTTO_LOAD_START 0x0700
#define ENTRANCE_GROTTO_EXIT_START 0x0800

#define ENTRANCE_GROTTO_LOAD(index) ENTRANCE_GROTTO_LOAD_START + index
#define ENTRANCE_GROTTO_EXIT(index) ENTRANCE_GROTTO_EXIT_START + index

typedef enum {
    /* 0x00 */ GROTTO_COLOSSUS_OFFSET,
    /* 0x01 */ GROTTO_LH_OFFSET,
    /* 0x02 */ GROTTO_ZR_STORMS_OFFSET,
    /* 0x03 */ GROTTO_ZR_FAIRY_OFFSET,
    /* 0x04 */ GROTTO_ZR_OPEN_OFFSET,
    /* 0x05 */ GROTTO_DMC_HAMMER_OFFSET,
    /* 0x06 */ GROTTO_DMC_UPPER_OFFSET,
    /* 0x07 */ GROTTO_GORON_CITY_OFFSET,
    /* 0x08 */ GROTTO_DMT_STORMS_OFFSET,
    /* 0x09 */ GROTTO_DMT_COW_OFFSET,
    /* 0x0A */ GROTTO_KAK_OPEN_OFFSET,
    /* 0x0B */ GROTTO_KAK_REDEAD_OFFSET,
    /* 0x0C */ GROTTO_HC_STORMS_OFFSET,
    /* 0x0D */ GROTTO_HF_TEKTITE_OFFSET,
    /* 0x0E */ GROTTO_HF_NEAR_KAK_OFFSET,
    /* 0x0F */ GROTTO_HF_FAIRY_OFFSET,
    /* 0x10 */ GROTTO_HF_NEAR_MARKET_OFFSET,
    /* 0x11 */ GROTTO_HF_COW_OFFSET,
    /* 0x12 */ GROTTO_HF_INSIDE_FENCE_OFFSET,
    /* 0x13 */ GROTTO_HF_OPEN_OFFSET,
    /* 0x14 */ GROTTO_HF_SOUTHEAST_OFFSET,
    /* 0x15 */ GROTTO_LLR_OFFSET,
    /* 0x16 */ GROTTO_SFM_WOLFOS_OFFSET,
    /* 0x17 */ GROTTO_SFM_STORMS_OFFSET,
    /* 0x18 */ GROTTO_SFM_FAIRY_OFFSET,
    /* 0x19 */ GROTTO_LW_SCRUBS_OFFSET,
    /* 0x1A */ GROTTO_LW_NEAR_SHORTCUTS_OFFSET,
    /* 0x1B */ GROTTO_KF_STORMS_OFFSET,
    /* 0x1C */ GROTTO_ZD_STORMS_OFFSET,
    /* 0x1D */ GROTTO_GF_STORMS_OFFSET,
    /* 0x1E */ GROTTO_GV_STORMS_OFFSET,
    /* 0x1F */ GROTTO_GV_OCTOROK_OFFSET,
    /* 0x20 */ GROTTO_LW_DEKU_THEATRE_OFFSET,
    /* 0x21 */ GROTTO_OFFSET_MAX,
} GrottoEntranceOffsets;
